﻿/**
 * @fileOverview Plugin to style the Html TreeGrid elements
 * @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
 *  Use of this code is subject to the terms of our license.
 *  A copy of the current license can be obtained at any time by e-mailing
 *  licensing@syncfusion.com. Any infringement will be prosecuted under
 *  applicable laws. 
 * @version 12.1 
 * @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
 */

(function ($, ej, undefined) {
    'use strict';
    ej.widget("ejTreeGrid", "ej.TreeGrid", {
        // widget element will be automatically set in this
        _rootCSS: "e-treegrid",

        // widget element will be automatically set in this
        element: null,

        validTags: ["div"],

        // user defined model will be automatically set in this
        model: null,

        _requiresID: true,
        _tags: [
        {
            tag: "columns",
            attr: ["field", "headerText", "allowSorting", "editType", "allowFiltering", "filterEditType", "width", "visible", "editParams", 
			"format", "isFrozen", "allowFreezing", "headerTemplateID", "allowCellSelection", "showCheckbox","showInColumnChooser", "validationRules.minlength", "isTemplateColumn",
			"validationRules.maxlength", "validationRules.range", "validationRules.number", "validationRules.required", "displayAsCheckbox", "editTemplate", "priority","filterType", [{ tag: "commands", attr: ["type", "buttonOptions"] }]],
            content: "template"
        },
        {
            tag: "summaryRows",
            attr: ["title", "summaryColumns"],
            content: "template"
        },
        {
            tag: "stackedHeaderRows",
            attr: ["stackedHeaderColumns"],
            content: "template"
        },

         {
             tag: "summaryRows.summaryColumns",
             attr: ["summaryType", "dataMember", "displayColumn", "prefix", "suffix", "format", "customSummaryValue"],
             content: "template"
         },
        {
            tag: "sortSettings.sortedColumns",
            attr: ["field", "direction"]
        },
        {
            tag: "sizeSettings",
            attr:["height","width"]
        },
        {
            tag:"columnResizeSettings",
            attr: ["columnResizeMode"]
        }
        ],
        _columns: function (index, property, value, old) {
            var proxy = this, model = proxy.model,
                columns = model.columns, isPropertyUpdated = false,
                length = columns.length,
                runTimeProperties = ['allowCellSelection', 'allowEditing', 'allowFilteringBlankContent', 'allowFreezing',
                    'allowSorting', 'editParams', 'headerTooltip', 'tooltip', 'validationRules'];

            if (!ej.isNullOrUndefined(index)) {
                //Refresh column property changes by Angular
                if (typeof index === 'object') {
                    for (var colIndex = 0; colIndex < index.length; colIndex++) {
                        var updatedColumn = index[colIndex];
                        if (!ej.isNullOrUndefined(updatedColumn.index) && !ej.isNullOrUndefined(updatedColumn['change'])) {
                            for (var i = 0; i < length; i++) {
                                var column = columns[i];
                                if (column.index && column.index == updatedColumn.index) {
                                    for (var prop in updatedColumn['change']) {
                                        if (runTimeProperties.indexOf(prop) == -1)
                                            isPropertyUpdated = true;
                                    }
                                    break;
                                }
                            }
                        }
                    }
                }
                    // Refresh Column changes by Angularjs
                else if (property && value) {
                    if (runTimeProperties.indexOf(prop) == -1)
                        isPropertyUpdated = true;
                }
                else
                    isPropertyUpdated = true;

                if (isPropertyUpdated) {
                    proxy.updateColumns();
                }
            }
        },
        keyConfigs: {
            focus: "e",
            editRecord: "113", //F2
            saveRequest: "13", // enter
            cancelRequest: "27", //Esc         
            firstRowSelection: "36", //"Home",
            lastRowSelection: "35", //"End",
            leftArrow:"37",//Left arrow
            upArrow: "38", //Up arrow
            rightArrow:"39",//Right arrow
            downArrow: "40", //Down arrow
            moveCellRight: "9", //tab
            moveCellLeft: "shift+9", //shifttab
            shiftDownArrow: "shift+40", //shift + DownArrow
            shiftUpArrow: "shift+38", //shift + UpArrow
            shiftRightArrow: "shift+39", //shift + RightArrow
            shiftLeftArrow: "shift+37", //shift + LeftArrow
            shiftHomeButton: "shift+36",//shift + HomeButton
            shiftEndButton: "shift+35",//shift + EndButton
            selectedRowExpand: "alt+40", //"AltPlusDownArrow",
            totalRowExpand: "ctrl+40", //"CtrlPlusDownArrow",
            selectedRowCollapse: "alt+38", //"AltPlusUpArrow",
            totalRowCollapse: "ctrl+38", //"CtrlPlusUpArrow"
            deleteRecord: "46", // delete
            spaceBar: "32", // spacebar
            topRowSelection: "ctrl+36", //CtrlPlusHome
            bottomRowSelection: "ctrl+35",//CtrlPlusEnd
            nextPage: "33", //PageUp
            prevPage:"34", //PageDown            
        },
      
        defaults: {
            expandStateMapping:"",
            allowFiltering: false,            
            allowDragAndDrop: false,            
            dragTooltip: {                
                showTooltip: false,                
                tooltipItems:[],                
                tooltipTemplate: "",
            },  
            allowSorting: false,            
            allowColumnResize: false, 
            allowSelection: true,   
            allowPaging: false,
            allowSearching: false,
            dataSource: null,
            query: ej.Query(),            
            idMapping: "",            
            parentIdMapping: "",
            hierarchyRowIndexMapping: "",
            readOnly: false,
            showGridCellTooltip: false,
            cellTooltipTemplate:null,
            showGridExpandCellTooltip: false,
            showColumnChooser: false,
            showColumnOptions: false,
            enableAltRow: true,			
            enableLoadOnDemand: false,
            totalSummaryHeight: 90,
            collapsibleTotalSummary :false,
            hasChildMapping: "",
            selectedItem: null,            
            showStackedHeader: false,
            stackedHeaderRows:[],
            toolbarSettings: {                
                showToolbar: false,                
                toolbarItems: [],
                customToolbarItems:[]
            },         
            editSettings: {       
                allowEditing: false,  
                allowAdding: true,
                allowDeleting: true,                
                editMode: "cellEditing",                
                rowPosition: "top",
                beginEditAction: "dblclick",
                dialogEditorTemplateID: "",
                showDeleteConfirmDialog: false,
                showConfirmDialog: true,
                batchEditSettings: { editMode: "cell" }
            },                   
            enableVirtualization: false,   
            allowMultiSorting: false,
            sortSettings: {                
                sortedColumns: []
            },
            filterSettings: {
                filterBarMode: "immediate",
                filteredColumns: [],
                filterType: "filterbar",
                maxFilterChoices:1000,
                enableCaseSensitivity:false,
                enableComplexBlankFilter:true,
                blankValue: "",
                filterHierarchyMode:"parent"
            },
            printingEnabled:false,
            selectionType: "single",    
            selectionMode: "row",
            selectionSettings: {
                selectionType: "single",
                selectionMode: "row",
                enableHierarchySelection: true,
                enableSelectAll: true,
            },
            searchSettings: {
                fields: [],
                key: "",
                operator: "contains",
                ignoreCase: true,
                searchHierarchyMode: "parent"
            },
            selectedCellIndexes: [],
            selectedRowIndex: -1,
            showSummaryRow: false,
            summaryRows: [],            
            showTotalSummary: false,    
            allowKeyboardNavigation: true,
            cssClass: "",           
            locale: "en-US",
            isEdit: false,
            allowScrolling: false,
            scrollSettings: {
                frozenColumns: 0
            },
            groupSettings: {
                groupedColumns:[]
            },            
            sizeSettings: {                
                height: "",                
                width: "",
            },            
            enableResize: true,            
            allowTextWrap: false,
            isResponsive: true,
            showDetailsRow: false,            
            showDetailsRowInfoColumn: false,            
            detailsTemplate: "",            
            detailsRowHeight: 100,    
            rowDataBound: null,                        
            load: null,
            create:null,
            queryCellInfo: null,            
            rowSelecting: null,           
            rowSelected: null,           
            cellSelecting: null,
            cellSelected: null,
            beginEdit: null,            
            endEdit: null,            
            expanding: null,            
            expanded: null,            
            collapsing: null,            
            collapsed: null,            
            dataBound:null,
            beforePrint:null,
            actionComplete: null,            
            actionBegin: null,
            actionFailure: null,
            currentViewData: [],
            flatRecords: [],
            parentRecords: [],
            updatedRecords:[],
            summaryRowRecords: [],
            selectedItems: [],
            ids: [],      
            columns: [
            {
                field: "",
                headerText: "",
                editType: "",
                filterEditType: "",
                allowFiltering: false,
                allowFilteringBlankContent: true,
                allowSorting: false,
                visible: "",
                textAlign: ej.TextAlign.Left,
                headerTextAlign: ej.TextAlign.Left,
                allowCellSelection: true,
                clipMode: "",
                headerTooltip: "",
                tooltip:"",
                showInColumnChooser:true,
                commands: [{
                    type: "",
                    buttonOptions:{},
                }],
                filterType:null
                //isFrozen: false,
                //allowFreezing: false
                //displayAsCheckbox
            },
            ],
            columnDialogFields: [],
            commonWidth: 150,
            dateFormat: "MM/dd/yyyy",   
            rowHeight: 30,
            emptyRecordText: "No Records To Display",            
            treeColumnIndex: 0,
            //API for toggle between datepicker and datetimepicker
            workingTimeScale: 'TimeScale8Hours',            
            childMapping: "",             
            enableCollapseAll:false,
            rowTemplateID: "",
            altRowTemplateID: "",
            parseRowTemplate: true,
            contextMenuSettings: {                
                showContextMenu: false,                
                contextMenuItems: []
            },            
            contextMenuOpen: null,            
            rowDragStart: null,            
            rowDrag: null,            
            rowDragStop: null,       
            rowDropActionBegin:null,
            detailsDataBound: null,            
            detailsShown: null,            
            detailsHidden: null,
            toolbarClick: null,
            columnResizeStart: null,
            columnResizeEnd: null,
            columnResized:null,
            recordClick:null,
            recordDoubleClick:null,
            exportToPdfAction: "",
            exportToExcelAction: "",
            allowMultipleExporting: false,
            headerTextOverflow: "none",
            pageSettings: {
                pageSize: 12,
                pageCount: 8,
                currentPage: 1,
                totalPages: 0,
                totalRecordsCount: null,
                pageSizeMode: "all",
                template: null,                
                printMode:"allPages",
            },
            allowColumnReordering: false,
            columnResizeSettings: {
                columnResizeMode: "normal"
            }
        },

        dataTypes: {
            columns: "array",            
            totalSummaryHeight: "number",
            collapsibleTotalSummary :"boolean",
            summaryRows: "array",
            sortSettings: {
                sortedColumns: "array"
            },
            searchSettings:{
                fields: "array"
            },
            dragTooltip:{
                tooltipItems: "array"
            },
            columnDialogFields: "array",
            filterSettings: {
                filteredColumns: "array",
                filterType: "string",
            },
            dataSource: "data",
            query: "data",
            currentViewData: "array",
            flatRecords: "array",
            parentRecords: "array",
            updatedRecords:"array",
            summaryRowRecords: "array",
            ids: "array",
            contextMenuSettings: {
                contextMenuItems: "array"
            },
            toolbarSettings: {
                toolbarItems: "array",
                customToolbarItems: "array"
            },
            stackedHeaderRows:"array",
            selectedItem: "data",            
        },
        ignoreOnExport: [
            "isEdit", "toolbarClick", "query", "queryCellInfo", "selectionType", "currentViewData", "enableRTL", "rowDataBound", "rowTemplate","parseRowTemplate",
             "detailsTemplate", "editSettings", "localization", "cssClass", "dataSource", "allowKeyboardNavigation","pageSettings","columnResizeSettings"
        ],
        observables: ["selectedRowIndex", "dataSource", "selectedCellIndexes","pageSettings.currentPage"],
        selectedItem: ej.util.valueFunction("selectedItem"),
        selectedRowIndex: ej.util.valueFunction("selectedRowIndex"),
        selectedCellIndexes: ej.util.valueFunction("selectedCellIndexes"),
        dataSource: ej.util.valueFunction("dataSource"),
        _currentPage: ej.util.valueFunction("pageSettings.currentPage"),
        _dataSource: ej.util.valueFunction("dataSource"),

        //GET THE LOCALIZATION OBJECT
        _getLocalizedLabels: function () {
            return ej.getLocalizedConstants(this.sfType, this.model.locale);
        },

        //GET THE GRID HEADER TABLE FOR COLUMNRESIZE
        getHeaderTable: function () {
            return this._$gridHeaderTable;
        },

        //SET THE GRID HEADER TABLE
        setGridHeaderTable: function (value) {
            this._$gridHeaderTable = value;
        },

        //GET THE GRIDHEADERCONTENT
        getHeaderContent: function () {
            return this._$gridHeaderContent;
        },

        //SET THE GRIDHEADERCONTENT
        setGridHeaderContent: function (value) {
            this._$gridHeaderContent = value;
        },

        //GET THE GRID CONTENT TABLE
        getContentTable: function () {
            return this._$gridContentTable;
        },

        getContent: function () {
            return this._$gridContent;
        },
        
        //SET THE GRID CONTENT TABLE
        setGridContentTable: function (value) {
            this._$gridContentTable = value;
        },

        //GET TREEGRID ROWS
        getRows: function () {
            var $gridRows = this._excludeDetailRows();
            return $gridRows;
        },

        //GET SELECTED CELLS
        getSelectedCells: function () {
            var proxy = this, model = proxy.model;
            var selectedCellElement = [];
            this._selectedCellDetails.forEach(function (item) {
                //Replace the current view cell Element after scrolling.
                var record = item.data,
                    currentRowIndex = model.currentViewData.indexOf(record), $target = $(item.cellElement);
                if (currentRowIndex != -1) {
                    if (proxy._frozenColumnsLength > 0) {
                        if ($target.closest(".e-frozencontentdiv").length > 0)
                            item.cellElement = proxy.getRows()[0][currentRowIndex].childNodes[item.cellIndex];
                        else if ($target.closest(".e-movablecontentdiv").length > 0)
                            item.cellElement = proxy.getRows()[1][currentRowIndex].childNodes[(item.cellIndex - proxy._frozenColumnsLength)];
                    } else {
                        item.cellElement = proxy.getRows()[currentRowIndex].childNodes[item.cellIndex];
                    }
                }
                selectedCellElement.push(item.cellElement);
            });
            return selectedCellElement;
        },

        //SET TREEGRID ROWS
        setGridRows: function (value) {
            this._$gridRows = value;
        },

        //GET CURRENTVIEWPORT DATA
        getCurrentViewData: function () {
            return this.model.currentViewData;
        },

        //GET UPDATED CURRENT VIEW DATA
        getUpdatedCurrentViewData: function () {
            this._updateCurrentViewData();
            return this.model.currentViewData;
        },

        //GET FLAT RECORDS
        getFlatRecords: function () {
            return this.model.flatRecords;
        },

        //SET THE HEIGHT BASED ON THE RECORD COUNT
        setHeight: function (value) {
            this._totalHeight = value;
        },

        //GET THE UPDATEDRECORDS[SORTED,FILTERED,EXPANDCOLLAPSED]
        getUpdatedRecords: function () {
            var model = this.model,updatedRecords = [];
            if (!this._isFromGantt && model.allowPaging)
                updatedRecords = this._updatedPageData;
            else
                updatedRecords = (this._isHierarchyFiltering && model.enableVirtualization) ? this._updateFilterParent : model.updatedRecords;
            return updatedRecords;
        },

        //GET THE SUMMARY ROW RECORDS
        getsummaryRowRecords: function () {
            return this.model.summaryRowRecords;
        },
        //GET THE UPDATED DATASOURCE
        getDataSource: function () {
            return this.dataSource();
        },

        //get the ids collection after perform the addition/delete operation
        //GET THE IDS COLLECTION AFTER PERFORM THE ADDITION/DELETE OPERATION
        getUpdatedIds: function () {
            return this.model.ids;
        },

        //GET THE RECORDS COUNT
        getRecordsCount: function () {
            return this._recordsCount;
        },

        //SET THE RECORDS COUNT
        setRecordsCount: function (value) {
            this._recordsCount = value;
        },
        // SET FOCUS ON ELEMENT
        setFocusOnTreeGridElement: function(){
            this._focusTreeGridElement();
        },

        //update the collapseRecount by updatedRecord collection
        updateCollapsedRecordCount:function()
        {
            var proxy = this,
                model = this.model,
                resultRecord = [];
            resultRecord = model.updatedRecords.filter(function (record) {
                return proxy.getExpandStatus(record) == false;
            });
            proxy._totalCollapsedRecordCount = resultRecord.length;
        },

        //In virtualization get expaned records from total record collection only
        _getDetailsExpandedRecords: function (records) {            
            var resultRecord = records.filter(function (record) {
                return record.isDetailsExpanded == true;
            });
            return resultRecord;
        },

        /* Get expanded details row height */
        _getExpandedDetailsRowHeight:function()
        {
            var proxy = this,                
                expandedRecords, 
                detailExpandedRecords,
                updatedRecords = proxy.getUpdatedRecords();

            expandedRecords = this.getExpandedRecords(updatedRecords);
            detailExpandedRecords = this._getDetailsExpandedRecords(expandedRecords);
            detailExpandedRecords = proxy._spliceSummaryRows(detailExpandedRecords);
            return (proxy._detailsRowHeight * detailExpandedRecords.length);
        },
        //return collapsed recount length in updatedRecord collectio
        getCollapsedRecordCount: function (records) {
            return this._totalCollapsedRecordCount;
        },
        
        _colgroupRefresh: function () {
            var gridheaderCol, gridcontentCol;
            
            if (this._frozenColumnsLength > 0) {
                gridheaderCol = $(this.getHeaderTable()).find('colgroup');
                gridcontentCol = $(this.getContentTable()).find('colgroup');
            }
            else {
                gridheaderCol = $(this.getHeaderTable()).find('colgroup')[0];
                gridcontentCol = $(this.getContentTable()).find('colgroup')[0];
            }
            var headerColClone = $(gridheaderCol).clone();
            var contentColClone = $(gridcontentCol).clone();
            var footerColClone = $(gridcontentCol).clone();
            $(gridcontentCol).remove();
            $(gridheaderCol).remove();
            if (this._frozenColumnsLength > 0) {
                $(headerColClone[0]).prependTo(this.getHeaderTable()[0]);
                $(headerColClone[1]).prependTo(this.getHeaderTable()[1]);
                $(contentColClone[0]).prependTo(this.getContentTable()[0]);
                $(contentColClone[1]).prependTo(this.getContentTable()[1]);
                if (this._$footertableContent) {
                    $(this._$footertableContent).find('colgroup').remove();
                    $(footerColClone[0]).prependTo(this._$footertableContent[0]);
                    $(footerColClone[1]).prependTo(this._$footertableContent[1]);
                }
                if (this._$totalSummaryRowContainer) {
                    if (!this.mobileDevice() && this.getBrowserDetails().browser == "safari")
                        this._$totalSummaryRowContainer.find("#e-movablefooter" + this._id).css("margin-left", "auto");
                }
                if (!this.mobileDevice() && this.getBrowserDetails().browser == "safari")
                    this.getHeaderContent().find("#e-movableheader" + this._id).add(this.getContent().find("#e-movablecontainer" + this._id)).css("margin-left", "auto");
                $("#e-frozencontentdiv" + this._id).length && $("#e-frozencontentdiv" + this._id).css("width", "100%");
            }
            else {
                $(headerColClone).prependTo(this.getHeaderTable());
                $(contentColClone).prependTo(this.getContentTable());
                if (this._$footertableContent) {
                    $(this._$footertableContent).find('colgroup').remove();
                    $(footerColClone).prependTo(this._$footertableContent);
                }
            }
        },
        _detailColsRefresh: function () {
            this._$headerCols = this.getHeaderTable().children("colgroup").find("col");
            this._$contentCols = this.getContentTable().children("colgroup").find("col");
            if (this.model.detailsTemplate && this.model.showDetailsRow && this.model.showDetailsRowInfoColumn || this.model.isFromGantt) {
                var colCount = this.model.columns.length;
                if (this._$headerCols.length > colCount) this._$headerCols.splice(colCount, 1);
                if (this._$contentCols.length > colCount) this._$contentCols.splice(colCount, 1);
            }
        },

        _swipeAction: function (e) {
            var proxy = this,curPage = this._currentPage(), doPage = true;
            var hScroll = (this.getScrollObject() || {})._scrollXdata;
            if (hScroll)
                doPage = e.type == "swipeleft" ? hScroll.scrollable - hScroll.sTop == 0 : hScroll.sTop == 0;

            switch (e.type) {
                case "swipeleft":
                    if (this.model.allowPaging && curPage != this.model.pageSettings.totalPages && !this.model.isEdit && !this._isRowEdit) {
                        if (doPage || !proxy.getScrollElement().ejScroller("isHScroll")) {
                            this.element.ejTreeGrid("gotoPage", curPage + 1);
                        }
                    }
                    break;
                case "swiperight":
                    if (this.model.allowPaging && curPage > 1 && !this.model.isEdit && !this._isRowEdit) {
                        if (doPage || !proxy.getScrollElement().ejScroller("isHScroll")) {
                            this.element.ejTreeGrid("gotoPage", curPage - 1);
                        }
                    }
                    break;
            }
        },

        // Move to the given page number
        gotoPage: function (pageIndex) {
            var proxy = this, model = proxy.model,
               args = {}, returnValue;

            if (!this.model.allowPaging)
                return;
            if (this._isLoadonDemand && pageIndex && pageIndex != this._currentPage()) {
                this._currentPage(pageIndex);
                proxy._refreshDataSource();
                if (this._prevPageNo != pageIndex)
                    this._prevPageNo = pageIndex;
                return;
            }
            if (ej.isNullOrUndefined(pageIndex))
                pageIndex = proxy._currentPage();
            else {
                if (pageIndex <= 0)
                    pageIndex = 1;
                else if (pageIndex > model.pageSettings.totalPages)
                    pageIndex = model.pageSettings.totalPages;
            }
            pageIndex = ej.isNullOrUndefined(pageIndex) ? proxy._currentPage() : pageIndex;
            this._currentPage(pageIndex);           
            args.previousPage = this._prevPageNo;                      
            args.currentPage = pageIndex;
            if (model.allowPaging) {                
                args.requestType = ej.TreeGrid.Actions.Paging;
            }
            proxy._setScrollTop();
            proxy._updateCurrentViewData();
            if (model.enableVirtualization && model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root && $.inArray(pageIndex, proxy._expandCollapsePages) == -1) {
                proxy._isInExpandCollapseAll = true;
                if (proxy._expandCollapseSettings.state == "collapseAll")
                    proxy._collapseAllData();
                else if (proxy._expandCollapseSettings.state == "expandAll")
                    proxy._expandAllData();
                if (proxy._expandCollapseSettings.atLevel == "collapseAtLevel")
                    proxy._collapsePageLevelData();
                else if (proxy._expandCollapseSettings.atLevel == "expandAtLevel")
                    proxy._expandPageLevelData();
                if(model.enableAltRow)
                    proxy.updateAltRow();
            }
            proxy.renderRecords(args);
            proxy._isInExpandCollapseAll = false;
            proxy._updateWrapRowHeights("total,header");
            proxy.updateHeight();
            proxy._updateScrollCss();
            if (model.allowTextWrap && model.enableVirtualization)
                proxy._updateWrapRowHeights("header", "refreshAll");
            if (returnValue)
                this._currentPage(args.previousPage);
            if (args.previousPage != args.currentPage) {                
                proxy.clearSelection(-1);
                proxy.selectedRowIndex(-1);
                proxy.model.selectedItem = null;
                proxy._cancelSaveTools();
                proxy._clearContextMenu();
                if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                    proxy._clearMultiSelectPopup();
                }
                this._prevPageNo = args.currentPage;
            }
        },

        /*Handler for pager change event*/
        _pagerClickHandler: function (sender) {
            var proxy = this;
            if (proxy.model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root && proxy._batchEditStarts && !(proxy._confirmDialog).is(":visible")) {
                proxy._batchSaveLostWarning("Paging");
                proxy._batchTempDetails.currentPage = sender.currentPage;
                return true;
            }
            if (this._prevPageNo == sender.currentPage)
                return;
            proxy._isNextPage = true;
            if (proxy._isRowEdit)
                proxy.cancelRowEditCell();
            else if (proxy.model.isEdit)
                proxy.cancelEditCell();
            this.gotoPage(sender.currentPage);
            return false;
        },
        /*CONSTRUCTOR FUNCTION*/
        _init: function () {

            var proxy = this;

            /* default value for asp designer */
            if (proxy.model.isdesignMode) {
                proxy.model.columns = [
                    { "field": "Column1", "HeaderText": "Column1" },
                    { "field": "Column2", "HeaderText": "Column2" },
                    { "field": "Column3", "HeaderText": "Column3" },
                    { "field": "Column4", "HeaderText": "Column4" }];
                proxy.model.childMapping = "Children";
                var tempDataSource = [{ Column1: "Value 1", Column2: "Value 2", Column3: "Value 3", Column4: "Value 4", Children: [] }];

                for (var count = 1; count < 4; count++) {
                    var data = { Column1: "Value 1", Column2: "Value 2", Column3: "Value 3", Column4: "Value 4" };
                    tempDataSource[0].Children.push(data);
                }
                this.dataSource(tempDataSource);
                this.model.sizeSettings.height = "250px";
                this.model.sizeSettings.width = "600px";
            }            
            proxy._detailsColumnWidth = 35; //default width of details info column
            proxy._summaryColumnIndex = 0;
            if (proxy.model.summaryRows.length <= 1)
                proxy.model.collapsibleTotalSummary = false;
            proxy._trigger("load");
            proxy._validateColumnFields();
            if (!proxy.model.isFromGantt)
                proxy._validateColumnVisible(true);
            proxy._initPrivateProperties();
            proxy._setCultureInfo();

            if (this.dataSource() instanceof ej.DataManager && this.dataSource().ready != undefined) {
                var proxy = this;
                proxy._renderWaitingPopup();
                this.dataSource().ready.done(function (args) {                    
                    proxy.model.dataSource = ej.DataManager(args.result);
                    proxy._initDatasource();
                });
            }
            else
                proxy._initDatasource();
            proxy._processEditing();
            if (proxy.model.showColumnChooser && proxy.model.showColumnOptions) {
                proxy.columnAddDialogTemplate();
                proxy._renderUpdateConfirmDialog();
            }
            proxy._createDragTooltipTemplate();
            proxy._createTooltipTemplate();
        },

        // Method to render the outer div element for Add/edit dialog into the DOM
        _renderEditDialog: function () {
            var $dialog = ej.buildTag("div.e-editdialog-padding", "", { display: "none" }, { id: this._id + "_dialogEdit" });
            return $dialog;
        },

        // Method to validate the column fields
        _validateColumnFields: function ()
        {
            var proxy = this, column = proxy.model.columns;
            if (!ej.isNullOrUndefined(column)) {
                for (var index = 0; index < column.length; index++) {
                    if (ej.isNullOrUndefined(column[index].field)) {
                        column[index].field = "";
                        column[index].allowEditing = false;
                        column[index].allowSorting = false;
                        column[index].allowFiltering = false;
                    }
                    if (proxy._isCommandColumn(column[index])) {                       
                        column[index].allowEditing = false;
                        column[index].allowSorting = false;
                        column[index].allowFiltering = false;
                        column[index].showCheckbox = false;
                    }
                    if (ej.isNullOrUndefined(column[index].editType)) {
                        column[index].editType = "stringedit";
                    }
                    if (column[index].editType == "maskedit" && ej.isNullOrUndefined(column[index].filterEditType)) {
                        column[index].filterEditType = "stringedit";
                    }
                }
            }

        },

        //Set the column visibility based on column priority and document width
        _validateColumnVisible: function (isLoad) {
            var proxy = this,
                model = proxy.model;
            proxy._autoHideColumns = [];
            proxy._autoShowColumns = [];
            if (model.isResponsive && model.enableResize) {
                var width = document.body.clientWidth,
                    priority = null;
                if (width >= 960 && width <= 1120)
                    priority = 6;
                else if (width >= 800 && width <= 959)
                    priority = 5;
                else if (width >= 640 && width <= 799)
                    priority = 4;
                else if (width >= 480 && width <= 639)
                    priority = 3;
                else if (width >= 0 && width <= 479)
                    priority = 2;
                var columns = model.columns,
                    length = columns.length;
                if (isLoad) {
                    for (var index = 0; index < length; index++) {
                        if (priority != null) {
                            if (columns[index].priority)
                                if (columns[index].priority >= priority)
                                    columns[index].visible = false;
                                else
                                    columns[index].visible = true;
                        }
                        else {
                            if (!ej.isNullOrUndefined(columns[index].priority))
                                columns[index].visible = true;
                        }
                    }
                }
                else
                {
                    for (var index = 0; index < length; index++) {
                        if (priority != null) {
                            if (columns[index].priority)
                                if (columns[index].priority >= priority) {
                                    if (columns[index].visible != false)
                                        proxy._autoHideColumns.push(columns[index]);
                                }
                                else
                                {
                                    if (columns[index].visible == false)
                                        proxy._autoShowColumns.push(columns[index]);
                                }
                                    
                        }
                        else {
                            if (!ej.isNullOrUndefined(columns[index].priority))
                                if (columns[index].visible == false)
                                    proxy._autoShowColumns.push(columns[index]);
                        }
                    }
                }
            }
        },
        //Hidden column using column values
        _hideColumns: function (columns) {
            var proxy = this,
                model = proxy.model;
            columns.reverse();
            columns.sort(function (column1, column2) {
                return column2.priority - column1.priority;
            });
            if (columns && columns.length > 0) {
                var length = columns.length;
                //Table column width not properly set in firefox, so just remove and add the e-table class from both header and content table
                proxy.getHeaderTable().removeClass("e-table");
                proxy.getContentTable().removeClass("e-table");
                for (var columnIndex = 0; columnIndex < length; columnIndex++) {
                    var column = columns[columnIndex],
                        index = $.inArray(column, model.columns),
                        headerText = column.headerText,
                        hiddenColumnIndex = $.inArray(headerText, proxy._hiddenColumns);
                    /* check this is single column in frozen or movable column*/
                    if (column && !proxy._isColumnHidable(column))
                        continue;
                    if (index != -1 && index != model.treeColumnIndex && hiddenColumnIndex == -1)
                        proxy._autoHideShowColumn(headerText,index, true);
                    index = $.inArray(headerText, proxy._visibleColumns);
                    if (index != -1)
                        proxy._visibleColumns.splice(index, 1);
                }
                proxy.getHeaderTable().addClass("e-table");
                proxy.getContentTable().addClass("e-table");
                proxy._updateHideShowColumnAlignment();
            }
        },
        //Show the hidden column using column values
        _showColumns: function (columns) {
            var proxy = this,
                model = proxy.model;
            if (columns && columns.length > 0) {
                var length = columns.length;
                for (var columnIndex = 0; columnIndex < length; columnIndex++) {
                    var column = columns[columnIndex],
                        index = $.inArray(column, model.columns),
                        headerText = column.headerText,
                        visibleColumnIndex = $.inArray(headerText, proxy._visibleColumns);
                    if (index != -1 && visibleColumnIndex == -1)
                        proxy._autoHideShowColumn(headerText,index, false);
                    index = $.inArray(headerText, proxy._hiddenColumns);
                    if (index != -1)
                        proxy._hiddenColumns.splice(index, 1);
                }
                proxy._updateHideShowColumnAlignment();
            }
        },

        //Hide or show the tree grid column
        _autoHideShowColumn: function (headerText,index, isHide)
        {
            var proxy = this,
                model = proxy.model;
            /* Cancel edited cell before sort the column*/
            proxy._cancelEditState();
            if (isHide) {
                proxy._hiddenColumns.push(headerText);
                model.columns[index].visible = false;
            }
            else {
                proxy._visibleColumns.push(headerText);
                model.columns[index].visible = true;
            }
            var headerrows,
                headerColgroup = proxy.getHeaderTable().find('colgroup').find("col"),
                headerRows = proxy.getHeaderTable().find('thead').find("th.e-headercell");
            if (model.allowFiltering && model.filterSettings.filterType == "filterbar") {
                headerrows = proxy.getHeaderTable().find('thead').find("th.e-filterbarcell");
            }
            if (proxy.model.columns[index].isFrozen == true) {
                proxy._updateFrozenColumnOnResize(index);
                proxy.setWidthToColumns();
            }
            var gridColGroup = proxy.getContentTable().find("colgroup").find("col"),
                isFrozenTable = true, cellIndex = index, footerRows,gridrows;
            gridrows = proxy.getRows();
            footerRows = proxy._getFooterRows();
            if (isHide) {
                $(headerColgroup).eq(index).hide();
                $(headerRows).eq(index).addClass("e-hide");
                headerrows && $(headerrows).eq(index).addClass("e-hide");
                $(gridColGroup).eq(index).hide();
            }
            else {
                $(headerColgroup).eq(index).show();
                $(headerRows).eq(index).removeClass("e-hide");
                headerrows && $(headerrows).eq(index).removeClass("e-hide");
                $(gridColGroup).eq(index).show();
            }

            if (proxy._frozenColumnsLength > 0) {
                if (index < proxy._frozenColumnsLength) {
                    gridrows = $(proxy.getRows()[0]);
                    footerRows = $(footerRows[0]);
                    if (proxy._gridRows && $(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                        var tempColSpan = parseInt($(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                        tempColSpan = isHide ? (tempColSpan - 1) : (tempColSpan + 1);
                        $(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan);
                    }
                }
                else {
                    gridrows = $(proxy.getRows()[1]);
                    footerRows = $(footerRows[1]);
                    isFrozenTable = false;
                    cellIndex = index - this._frozenColumnsLength;
                    if (proxy._gridRows && $(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                        var tempColSpan = parseInt($(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                        tempColSpan = isHide ? (tempColSpan - 1) : (tempColSpan + 1);
                        $(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan);
                    }
                }
            } else {
                if (proxy._gridRows && $(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                    var tempColSpan = parseInt($(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                    tempColSpan = isHide ? (tempColSpan - 1) : (tempColSpan + 1);
                    $(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan);
                }
            }

            //Display or hide the specified column     
            gridrows.each(function () {
                var td = $(this).children("td").eq(cellIndex);
                isHide ? $(td).addClass("e-hide") : $(td).removeClass("e-hide");
                if (model.showSummaryRow && $(td).hasClass('e-summaryrowcell') && isFrozenTable)
                    proxy._updateSummaryRowTitleCell(headerColgroup, td[0].parentElement);
            });

            //Show or Hide the specific column from total summary row.
            if (model.showTotalSummary) {
                var footerColGroup = proxy._$footertableContent.find("colgroup").find("col");
                isHide ? $(footerColGroup).eq(index).hide() : $(footerColGroup).eq(index).show();
                footerRows.each(function () {
                    var footerSummarytd = $(this).children("td").eq(cellIndex);
                    isHide ? $(footerSummarytd).addClass("e-hide"):$(footerSummarytd).removeClass("e-hide");
                    if ($(footerSummarytd).hasClass('e-footersummaryrowcell'))
                        proxy._updateSummaryRowTitleCell(footerColGroup, footerSummarytd[0].parentElement);
                });
            }
        },

        //Update the tree grid alignment after hide and show the column
        _updateHideShowColumnAlignment:function()
        {
            var proxy = this,
                model = proxy.model;
            proxy._addInitTemplate();
            var gridContent = proxy.getScrollElement();
            if (model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns && !model.isFromGantt)
                proxy._updateTableWidth();
            gridContent.ejScroller("refresh");
            proxy._updateScrollCss();
            proxy._updateHeaderScrollLeft();
            if (model.showColumnChooser) {
                proxy._renderColumnChooserList(true);
                proxy._updateColumnMenuVisibility();
            }
        },
        // Method to create a template for add/edit dialog.
        _treeGridAddEditDialog: function () {
            if (this.model.columns.length == 0)
                return;
            var $formbody = ej.buildTag('div');
            var $form = ej.buildTag('form.e-treegridform', "", {}, { id: this._id + "EditFormDialog" });
            var $table = ej.buildTag('table', "", { "margin": "0 auto" }, {});
            var $innerTr, $labelTd, $valueTd, trElement, tdElement;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if ((this.model.columns[columnCount].visible || ej.isNullOrUndefined(this.model.columns[columnCount].visible)) && this.model.columns[columnCount].field != "checkboxState" && !this._isCommandColumn(this.model.columns[columnCount])) {
                    trElement = 'tr';
                    tdElement = 'td';
                    $innerTr = ej.buildTag(trElement);
                    $labelTd = ej.buildTag(tdElement + ".e-editdialoglabel", "", { "text-align": "right" });
                    $valueTd = ej.buildTag(tdElement + ".e-editdialogrowcell", "", { "text-align": "left" });
                    if (this.model.columns[columnCount].allowEditing != false) {
                        $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
                    }
                    if (this.model.columns[columnCount].headerText == undefined)
                        this.model.columns[columnCount].headerText = this.model.columns[columnCount].field;
                    $labelTd.append("<label>" + this.model.columns[columnCount].headerText + "</label>");
                    if (this.model.columns[columnCount]["allowEditing"] != false && this.model.columns[columnCount]["field"])
                        ej.TreeGrid._initCellEditType(this, $valueTd, this._id, columnCount);
                    this.model.columns[columnCount]["allowEditing"] == false && $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                    if (this.model.columns[columnCount]["visible"] === false)
                        $innerTr.addClass("e-hide");
                    $table.append($innerTr);
                }
            }
            $form.append($table);
            $form.appendTo($formbody);
            $formbody = this._renderDiaglogButton($form, $formbody, $table);
            $.templates(this._id + "_JSONDialogEditingTemplate", $formbody.html());
        },

        //  Method to create a template for add/edit template dialog.
        _treeGridAddEditDialogTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $tbody = ej.buildTag('div', "", { 'display': 'none' });
            var $form = ej.buildTag('form.e-treegridform', "", {}, { id: this._id + "EditFormDialog" });
            var cloneElement = this.model.editSettings.dialogEditorTemplateID;
            $form.html($(cloneElement).html());
            $tbody = this._renderDiaglogButton($form, $tbody);
            $.templates(this._id + "_JSONdialogTemplateMode", $tbody.html());
        },

        // To render the save/cancel buttons for treegrid add/edit dialog and add/edit template dialog
        _renderDiaglogButton: function (form, $formbody, $table) {
            var btnId, contextMenuLabel = this._contextMenuTexts, savebtn, cancelbtn;
            btnId = "EditFormDialog_";
            if (ej.isNullOrUndefined($table)) {
                savebtn = ej.buildTag('input.e-save e-button e-btnsub e-flat', "", { 'margin-left': '33%' }, { type: "button", id: btnId + this._id + "_Save" });
                savebtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: contextMenuLabel["saveText"], width: "30%" });
                cancelbtn = ej.buildTag('input.e-cancel e-button e-btncan e-flat', "", { 'margin-left': '7%' }, { type: "button", id: btnId + this._id + "_Cancel" });
                cancelbtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: contextMenuLabel["cancelText"], width: "30%" });
            } else {
                savebtn = ej.buildTag('input.e-save e-button e-btnsub e-flat', "", { 'margin-left': '0px' }, { type: "button", id: btnId + this._id + "_Save" });
                savebtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: contextMenuLabel["saveText"], width: "45%" });
                cancelbtn = ej.buildTag('input.e-cancel e-button e-btncan e-flat', "", { 'margin-left': '10%' }, { type: "button", id: btnId + this._id + "_Cancel" });
                cancelbtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: contextMenuLabel["cancelText"], width: "45%" });
            }
            var btnDiv = ej.buildTag('div', "", {}, { 'class': "e-editform-btn e-editdialogbutton" }),
                $innerTr, $innerTd1, $innerTd2;
            if (!ej.isNullOrUndefined($table)) {
                $innerTr = ej.buildTag('tr');
                $innerTd1 = ej.buildTag('td.e-editdialoglabel');
                $innerTd2 = ej.buildTag('td.e-editdialogrowcell');
            }
            btnDiv.append(savebtn);
            btnDiv.append(cancelbtn);
            if (!ej.isNullOrUndefined($table)) {
                $innerTd2.append(btnDiv);
                $innerTr.append($innerTd1);
                $innerTr.append($innerTd2);
                $table.append($innerTr);
            }
            form.appendTo($formbody);
            if (!ej.isNullOrUndefined($table))
                form.append($table);
            else
                form.append(btnDiv);
            return $formbody;
        },

        /* update allow keyboard navigation value*/
        updateAllowKeyboardNavigation: function (bool)
        {
            this.model.allowKeyboardNavigation = bool;
        },
        _updateSummaryTitleColumn: function () {
            var proxy = this,
                model = this.model;
            if (model.collapsibleTotalSummary && this._summaryColumnIndex == 0){
                if(model.columns[this._summaryColumnIndex].visible && !$("#" + proxy._id + "-footersummaryrow").find(".e-summarytitle").hasClass("e-summarycell")) {
                    $("#" + proxy._id + "-footersummaryrow").find(".e-summarytitle").addClass("e-summarycell");
                    $("#" + proxy._id + "-footersummaryrow").find(".e-summarycell").css({ "overflow": "hidden", "padding-left": "25px" , "float":"none" });
                    $("#" + proxy._id + "-footersummaryrow").find(".e-summarytitle").parent().addClass("e-summaryindend").css({ "padding-left": '0px' });
                }
                else if (!model.columns[this._summaryColumnIndex].visible && $("#" + proxy._id + "-footersummaryrow").find(".e-summarytitle").hasClass("e-summarycell")) {
                    $("#" + proxy._id + "-footersummaryrow").find(".e-summarytitle").removeClass("e-summarycell");
                    $("#" + proxy._id + "-footersummaryrow").find(".e-summarytitle").css({ "padding-left": "0px" , "float": "left"});
                    $("#" + proxy._id + "-footersummaryrow").find(".e-summarytitle").parent().removeClass("e-summaryindend").css({ "padding-left": '10px' });
                }
            }
        },
        /* Refresh footer summary records if total summary exist */
        _refreshFooterSummaryRecords: function () {
            var proxy = this,
                model = this.model;
            if (model.showTotalSummary && proxy._$footerContainer.length > 0) {
                if (this._frozenColumnsLength > 0) {
                    proxy._$footerContainer.html(this._renderSummaryByFrozenDesign());
                    this._renderFrozenSummary();
                } else {
                    var $table = ej.buildTag('table.e-table e-zerospace#' + proxy._id + "summarye-table", "",
                    { top: "0px" }),
                      $tbody = ej.buildTag('tbody');
                    $table.append(proxy.getContentTable().find('colgroup').clone()).append($tbody);
                    $tbody.html($.render[this._id + "_Template"](proxy._footerSummaryRecord));
                    proxy._$footerContainer.empty().append($table);
                }
                proxy._$footertableContent = proxy._$footerContainer.find(".e-table");
                proxy._updateSummaryTitleColumn();
            }
        },
        /* update scrollLeft of header container with content scrollLeft*/
        _updateHeaderScrollLeft: function (scrollLeft) {
            var proxy = this;
            if (ej.isNullOrUndefined(scrollLeft))
                scrollLeft = proxy.getScrollElement().ejScroller("option", "scrollLeft");
            if (this._frozenColumnsLength > 0) {
                proxy._$gridHeaderContainer.find("#e-movableheader" + this._id).scrollLeft(scrollLeft);
                if (proxy.model.showTotalSummary && proxy._$footerContainer)
                    proxy._$footerContainer.find("#e-movablefooter" + this._id).scrollLeft(scrollLeft);
            } else {
                proxy._$gridHeaderContainer.scrollLeft(scrollLeft);
                if (proxy.model.showTotalSummary && proxy._$footerContainer)
                    proxy._$footerContainer.scrollLeft(scrollLeft);
            }
        },
        //Update Columns Dynamically.
        updateColumns: function (columns) {
            var proxy = this,
               model = this.model;
            if (!ej.isNullOrUndefined(columns))
                model.columns = columns;
            if (model.isFromGantt && model.enableSerialNumber)
                proxy._updateTreeColumnIndex();
            proxy._refreshFrozenColumns();
            proxy._updateWrapRowHeights(null, "refreshAll");
            if (model.isFromGantt)
                proxy._updateAddEditDialogTemplate();
        },
        //Update the Gantt Columns from TreeGrid Columns.
        updateToGanttColumns: function () {
            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");
            ganttObject.updateGanttColumns(columns);            
        },

        //Update the add and edit dialog template.
        _updateAddEditDialogTemplate: function () {
            var proxy = this, ganttObject;
            ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");
            ganttObject.addDialogTemplate();
            ganttObject.editDialogTemplate();
        },
        /* Refresh columns collection with freeze and unfreeze*/
        _refreshFrozenColumns: function (isDataSourceUpdate) {
            var proxy = this,
               model = this.model,
               layoutUpdated = false;
            proxy._initFrozenColumns();
            proxy._processEditing();
            //Rerendering the column header with new columns.;
            proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
            proxy._addInitTemplate();
            if (model.isFromGantt)
                proxy._$gridContent.removeClass("e-borderbox");
            /* Render frozen and movable table if any column is frozen*/
            if (this._frozenColumnsLength > 0 && proxy._$frozenTableContent.length == 0) {
                layoutUpdated = true;
                var contentWidth = proxy._$gridContent.css("width"),
                    contentHeight = proxy._$gridContent.css("height");
                proxy._$gridContent.ejScroller("destroy");
                proxy._$gridContent.css("width", contentWidth);
                proxy._$gridContent.css("height", contentHeight);
                proxy._$gridContainer.html(proxy._renderByFrozenDesign());
                proxy.setGridContentTable(proxy._$gridContainer.find(".e-table"));
                if (model.allowTextWrap && !model.enableVirtualization)
                    proxy._$gridContainer.find("#e-movablecontent" + proxy._id + ",#e-movablecontentdiv" + proxy._id + ",#e-frozencontentdiv" + proxy._id).css("height", "auto");
                else
                    proxy._$gridContainer.find("#e-movablecontent" + proxy._id + ",#e-movablecontentdiv" + proxy._id + ",#e-frozencontentdiv" + proxy._id).css("height", proxy._$gridContainer.css("height"));
                proxy._renderScroller(true);
            } else if (proxy._$frozenTableContent.length > 0 && this._frozenColumnsLength == 0) {
                layoutUpdated = true;
                proxy._$gridContent.find("#e-movablecontainer" + proxy._id).ejScroller("destroy");
                var $table = ej.buildTag('table.e-table e-zerospace#' + proxy._id + "e-table", "",
                     { top: "0px", "position": "relative" }),
                     $tbody = ej.buildTag('tbody');
                $table.append(proxy.getHeaderTable().find('colgroup').clone()).append($tbody);
                proxy._$gridContainer.html($table);
                proxy.setGridContentTable($table);
                proxy._renderScroller(true);
            }
            /* Update table private  properties*/
            proxy._$frozenTableContent = $("#" + proxy._id + "frozene-table");
            proxy._$movableTableContent = $("#" + proxy._id + "movablee-table");
            proxy._$tableContent = $("#" + proxy._id + "e-table");

            if (model.showTotalSummary && proxy._$footerContainer.length > 0 && !isDataSourceUpdate) {
                proxy._refreshFooterSummaryRecords();
            }

            if (layoutUpdated) {
                proxy.onScrollHelper(0);
            }
            if (!isDataSourceUpdate) {
                var args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;
                /* for details rows detailsDataBound event can be called after the width update of column */
                proxy._isRendered = false;
                //Rerendering the content of the tree grid
                proxy.sendDataRenderingRequest(args);
                proxy._isRendered = true;
            }
            proxy.setWidthToColumns();
            proxy._trigger("headerRefresh");
            proxy._isRendered = true;
            if (!isDataSourceUpdate) {
                /* trigger queryCellInfo ,detailsDataBoundEvent after update the width of the columns */
                proxy._trigger("refresh");
                this._eventBindings();
            }
            this._hideCollapsedDetailsRows();

            /* Update the scrollbar*/
            if (!isDataSourceUpdate) {
                proxy.getScrollElement().ejScroller("refresh");
                proxy._updateScrollCss();
                if (model.isFromGantt) {
                    proxy._trigger("refreshScrollCss");
                }
            }
            if (model.showColumnChooser || model.contextMenuSettings.showContextMenu)
                proxy._renderColumnChooserList(true);
            /* update header and footer scrollLeft value*/
            proxy._updateHeaderScrollLeft();            
            
            if (model.isFromGantt) {
                /*Add additional td on column header to maintain column width*/
                proxy._addEmptyColumntoGrid();
                /*Updating the column properties to Gantt's model.columns*/
                proxy.updateToGanttColumns();
            }           
        },
        /* Freeze all Columns before argumented column*/
        freezePrecedingColumns: function (field) {
            var proxy = this, model = this.model,
                columns = model.columns,
                currentColumn = this.getColumnByField(field),
                columnIndex = columns.indexOf(currentColumn);
            /* Clear edit state , context menu and details template*/
            proxy._saveEditedElements();

            this._clearContextMenu();
            this._removeDetailsRow();
            this._clearColumnMenu();

            /* Check column is available or not*/
            if (ej.isNullOrUndefined(currentColumn) || columnIndex == -1)
                return false;
            /*Check all previous columns are in frozen*/
            if (columnIndex == 0 && columnIndex <= this._frozenColumnsLength)
                return false;
            /*Check frozen column width and container width */
            var frozenWidth = this._getFrozenColumnWidth(columnIndex);
            frozenWidth = frozenWidth + 18;
            if (frozenWidth > this._gridWidth)
                return false;
            /* update isFrozen API of columns and refresh the columns*/
            for (var count = 0; count < columnIndex; count++) {
                columns[count].isFrozen = true;
            }
            proxy._refreshFrozenColumns();
            proxy._updateWrapRowHeights(null,"virtualRefresh");
        },
        /* Public method for freeze or unfreeze the columns */
        freezeColumn: function (field, isFrozen) {
            var proxy = this, model = this.model,
                columns = model.columns,                
                currentColumn = this.getColumnByField(field);

            /* Clear edit state , context menu and details template*/
            proxy._saveEditedElements();

            this._clearContextMenu();
            this._removeDetailsRow();
            this._clearColumnMenu();

            if (model.selectionSettings.selectionMode == "cell")
                proxy.clearSelection();

            /* Check column is available or not*/
            if (ej.isNullOrUndefined(currentColumn))
                return false;
            /*Check all other columns are in frozen*/
            if (proxy._frozenColumnsLength == model.columns.length - 1 && isFrozen)
                return false;
            /* Check column is in same state*/
            if (currentColumn.isFrozen == isFrozen)
                return false;
            /*Check frozen column width and container width */
            var frozenWidth = this._getFrozenColumnWidth(),
                columnIndex = columns.indexOf(currentColumn);
            frozenWidth = frozenWidth + (isFrozen ? (this.columnsWidthCollection[columnIndex] + 18) : (-this.columnsWidthCollection[columnIndex]));
            if (frozenWidth > this._gridWidth)
                return false;

            currentColumn.isFrozen = isFrozen;
            /* refresh columns collection in UI*/
            proxy._refreshFrozenColumns();
            /*Updating the frozen columns when dynamically freeze the column */
            if (isFrozen)
                proxy._allowFreezingDefault = this._frozenColumnsLength > 0 ? true : false;
			proxy._off(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);
            if (this._frozenColumnsLength)
                proxy._on(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);
            proxy._updateWrapRowHeights(null, "virtualRefresh");
        },

        //Public method use to change checkbox Column
        updateCheckboxColumn: function(fieldName){
            var proxy = this, model = proxy.model,
                columns = model.columns,
                column = proxy.getColumnByField(fieldName);
            if (!proxy._checkboxSelection)
                return;
            $.each (columns, function (index, value) {
                value["showCheckbox"] = false;
            });
            if (!ej.isNullOrUndefined(column)) {                
                var index = columns.indexOf(column);
                if(index > -1)
                    columns[index]["showCheckbox"] = true;
            }            
            var selectionSettings = {}, args = {};
            selectionSettings.selectionType = ej.TreeGrid.SelectionType.Checkbox;
            args.selectionSettings = selectionSettings;
            proxy._refreshSelectionSettings(args, "selectionSettings");
        },

        //Refresh selectionSettings when update by setModel
        _refreshSelectionSettings: function (options, prop) {
            var proxy = this,
                model = proxy.model,
                isSelectionChanged = false;

            isSelectionChanged = !ej.isNullOrUndefined(options[prop].selectionType) || !ej.isNullOrUndefined(options[prop].selectionMode);

            if (prop == "selectionType") {
                model.selectionSettings.selectionType = options[prop];
                isSelectionChanged = true;
            }
            else if (prop == "selectionMode") {
                model.selectionSettings.selectionMode = options[prop];
                isSelectionChanged = true;
                if (model.showColumnChooser && model.showColumnOptions)
                    proxy.columnAddDialogTemplate();
            }
            else
                $.extend(model.selectionSettings, options[prop]);
                    

            var isCheckboxType = (model.selectionSettings.selectionType == ej.TreeGrid.SelectionType.Checkbox &&
              model.selectionSettings.selectionMode == ej.TreeGrid.SelectionMode.Row);

            // Unchecked all checked records
            if ((!ej.isNullOrUndefined(options[prop].enableHierarchySelection) && model.selectedItems.length > 0) ||
                    (!isCheckboxType && proxy._checkboxSelection)) {
                proxy._isFromSetmodel = true;
                proxy.selectAllRows(false);
                proxy._isFromSetmodel = false;
            }

            if (proxy._checkboxSelection || isCheckboxType) {
                proxy._checkboxSelection = isCheckboxType;
                proxy._processEditing();
                if ((!isCheckboxType && proxy._checkboxColumnName == "checkboxState") ||
                        (model.columns[0].nonColumn && proxy._checkboxColumnName != "checkboxState")) {
                    var treeGridIndexColumn = model.columns[model.treeColumnIndex];
                    if (model.columns[0]['isFrozen']) {
                        model.columns[0]['isFrozen'] = false;
                        proxy._frozenColumnsLength -= 1;
                        var index = proxy._frozenColumns.indexOf(model.columns[0]);
                        proxy._frozenColumns.splice(index, 1);
                        model.scrollSettings.frozenColumns = this._frozenColumnsLength;
                    }
                    model.columns.splice(0, 1);
                    if (treeGridIndexColumn)
                        model.treeColumnIndex = model.columns.indexOf(treeGridIndexColumn);
                }
                if (!proxy._checkboxSelection) proxy._checkboxColumnName = null;

                proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                proxy._setWidthToHeaders();
                proxy._trigger("headerRefresh");
                if (model.allowFiltering)
                    proxy._resizeFilteringElements();
                if (isSelectionChanged) {
                    proxy._addInitTemplate();
                    this.renderRecords();
                }
                proxy.setWidthToColumns();
                proxy.getScrollElement().ejScroller("refresh");
                proxy._updateScrollCss();
                proxy._updateHeaderScrollLeft();
                //Updating header checkbox state while render dynamically            
                if (model.selectedItems.length > 0)
                    proxy.selectAllRows();
            }
            if(model.isFromGantt)
                proxy._processEditing();

        },

        //#region RUNTIME UPDATE METHOD FOR TREEGRID
        //METHOD FOR UPDATE THE TREEGRID IN RUN TIME
        _setModel: function (options) {

            var proxy = this,
                model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords();

            for (var prop in options) {

                switch (prop) {

                    case "enableAltRow":
                        model.enableAltRow = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        proxy._updateWrapRowHeights("header", "refreshAll");
                        break;
                    case "allowFiltering":
                        if (!model.allowFiltering && proxy._filteredColumnValueID.length > 0) {
                            proxy._clearAllFilter(proxy._filteredColumnValue);
                        }
                        model.allowFiltering = options[prop];
                        proxy._initiateSetModel();
                        model.allowFiltering = options[prop];
                        var scroller = proxy.getScrollElement().ejScroller("instance"),
                          left = scroller.scrollLeft(), totalSummaryHeight = 0,
                          top = scroller.scrollTop();
                        //Re render the grid header
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        proxy._setWidthToHeaders();
                        proxy._trigger("headerRefresh");
                        if (model.showTotalSummary)
                            totalSummaryHeight = proxy.element.find('.e-footersummaryrowdiv').outerHeight();
                        $("#" + proxy._id).addClass('e-treegrid-overflow');
                        proxy._viewPortHeight = proxy.element.height() - (proxy._getNonContentHeight() + totalSummaryHeight);
                        $("#" + proxy._id).removeClass('e-treegrid-overflow');
                        if (proxy._frozenColumnsLength > 0) {
                            proxy._$gridContent.css("height", proxy._viewPortHeight);
                        }
                        scroller.option("height",model.sizeSettings.height == "auto"?0:proxy._viewPortHeight);
                        if (model.enableVirtualization) {
                            var length = model.currentViewData.length,
                                rowHeight = model.rowHeight, isTopChanged;
                            rowHeight += (model.showDetailsRow && !model.showDetailsRowInfoColumn) ? model.detailsRowHeight : 0;
                            if ((length * rowHeight) < proxy._viewPortHeight) {
                                if (length > 0) {
                                    top = top - (proxy._viewPortHeight - ((length - 1) * rowHeight));
                                    if (top < 0)
                                        top = 0;
                                } else {
                                    proxy.processBindings();
                                    proxy.renderRecords();
                                }
                            }
                        }
                        //Update columnOptions fields
                        if (model.showColumnChooser && model.showColumnOptions)
                            proxy.columnAddDialogTemplate();
                        if (model.allowFiltering && model.filterSettings.filterType == "menu" && proxy._renderedFilterMenuDialog.length == 0)
                            proxy._renderFilterMenu();
                        //update the column width
                        this.setWidthToColumns();
                        scroller.scrollY(top, true);
                        scroller.scrollX(left, true);
                        proxy._updateHeaderScrollLeft(left);
                        proxy._updateScrollCss();
                        proxy._updateWrapRowHeights("row,total", "refreshAll");
                        break;
                    case "enableResize":
                        proxy.model.enableResize = options[prop];
                        if (model.enableResize) {
                            proxy._on($(window), "resize", proxy._windowResize);
                        }
                        else {
                            proxy._off($(window), "resize", proxy._windowResize);
                        }
                        break;

                    case "isResponsive":
                        proxy.model.isResponsive = options[prop];
                        if (model.isResponsive) {
                            proxy._on($(window), "resize", proxy._windowResize);
                            $("#" + proxy._id + "_toolbarItems").addClass("e-treegridadaptivetoolbar");
                            proxy._mediaQueryUpdate();
                            proxy._off(this.element, "swipeleft swiperight", ".e-gridcontent div > .e-table", $.proxy(this._swipeAction, this));
                            proxy._on(this.element, "swipeleft swiperight", ".e-gridcontent div > .e-table", $.proxy(this._swipeAction, this));
                        }
                        else {
                            proxy._off($(window), "resize", proxy._windowResize);
                            proxy._off(this.element, "swipeleft swiperight", ".e-gridcontent div > .e-table", $.proxy(this._swipeAction, this));
                            $("#" + proxy._id + "_toolbarItems").removeClass("e-treegridadaptivetoolbar");
                            proxy._mediaQueryUpdate();
                        }
                        break;

                    case "enableVirtualization":
                        model.enableVirtualization = options[prop];
                        var previousRecord = proxy._prevSelectedItem,
                            selectedRecord = proxy.selectedRowIndex() && updatedRecords[proxy.selectedRowIndex()];
                        proxy._vScrollDist = 1; //To set table top value properly on virtualization enabled mode while updating current view data
                        proxy.processBindings();
                        // change the table top position is 0 and change the update records for non-virtulization mode.
                        if (!model.enableVirtualization) {
                            if (this._frozenColumnsLength > 0) {
                                proxy._$frozenTableContent.css({
                                    "top": 0
                                });
                                proxy._$movableTableContent.css({
                                    "top": 0
                                });
                            } else {
                                proxy._$tableContent.css({
                                    "top": 0
                                });
                            }
                        }
                        selectedRecord && proxy.selectedRowIndex(updatedRecords.indexOf(selectedRecord));
                        var args = {};
                        args.requestType = ej.TreeGrid.Actions.Refresh;
                        proxy.sendDataRenderingRequest(args);
                        proxy._updateWrapRowHeights(null,"refreshAll");
                        proxy._isRowEdit = model.isEdit = false;
                        proxy._cancelSaveTools();
                        proxy._clearColumnMenu();
                        proxy._clearContextMenu();
                        this._removeDetailsRow();
                        proxy._vScrollDist = 0;
                        break;

                    case "allowColumnResize":
                        model.allowColumnResize = options[prop];
                        if (model.allowColumnResize)
                            proxy._resizer = new ej.gridFeatures.gridResize(proxy);
                        proxy._enableColumnResizeEvents();
                        break;

                    case "allowSorting":
                        model.allowSorting = options[prop];
                        proxy.sortSetting(options[prop]);
                        if (model.showColumnChooser && model.showColumnOptions)
                            proxy.columnAddDialogTemplate();
                        break;
                    case "allowPaging":
                        var $pagerElement = proxy.element.find("#" + proxy._id + "Pager");
                        if (model.allowPaging) {
                            proxy._updateTotalRecordsCount();
                            proxy.element.append(proxy._renderGridPager());
                            proxy.element.append($pagerElement);
                            proxy.element.find(".e-pager").css({ "width": proxy._gridWidth - 2 });
                            var scroller = proxy.getScrollElement().ejScroller("instance"),
                                left = scroller.scrollLeft(),
                                top = scroller.scrollTop();
                            proxy._viewPortHeight = proxy._getViewPortHeight();
                            if (model.showTotalSummary)
                                proxy._viewPortHeight -= proxy._$totalSummaryRowContainer.outerHeight();
                            if (proxy._frozenColumnsLength > 0)
                                proxy._$gridContent.css("height", proxy._viewPortHeight);
                            scroller.option("height", model.sizeSettings.height == "auto" ? 0 : proxy._viewPortHeight);
                            scroller.scrollY(top, true);
                            scroller.scrollX(left, true);
                            proxy._pagerHeight = $($pagerElement).outerHeight();
                            proxy.gotoPage();
                        }
                        else {

                            var height = $($pagerElement).outerHeight();
                            $($pagerElement).remove();
                            proxy._viewPortHeight = proxy._viewPortHeight + height;
                            if (this._frozenColumnsLength > 0) {
                                proxy._$gridContent.css("height", proxy._viewPortHeight);
                            }

                            proxy.getScrollElement().ejScroller({
                                height: proxy._viewPortHeight,//for border-width of gridcontent
                            });
                            if (model.pageSettings.totalRecordsCount > 0) {
                                model.pageSettings.totalRecordsCount = null;
                                proxy._refreshDataSource();
                            }
                            else
                                proxy.refreshContent();
                            proxy.renderRecords();
                            proxy._isRefreshAddedRecord = false;
                        }
                        proxy._updateWrapRowHeights("header", "refreshAll");
                        break;
                   
                    case "dragTooltip":
                        var tooltipValue = options[prop].tooltipItems;
                        if (tooltipValue && !Array.isArray(tooltipValue)) {
                            tooltipValue = tooltipValue.length > 0 ? tooltipValue.split(",") : [];
                            options[prop].tooltipItems = tooltipValue;
                        }
                        $.extend(model.dragTooltip, options[prop]);
                        proxy._createDragTooltipTemplate();
                        proxy.refreshContent();
                        break;

                    case "allowDragAndDrop":
                        model.allowDragAndDrop = options[prop];
                        var matched = jQuery.uaMatch(navigator.userAgent);                                                
                        proxy.refreshContent();
                        break;

                    case "sortSettings":
                        $.extend(model.sortSettings, options[prop]);
                        proxy.refreshContent();
                        var refreshArgs = {};
                        refreshArgs.requestType = ej.TreeGrid.Actions.Sorting;
                        proxy.sendDataRenderingRequest(refreshArgs);                       
                        break;

                    case "allowSelection":                      
                        model.allowSelection = options[prop];
                        if (!model.allowSelection) {
                            if (proxy._checkboxSelection && model.selectedItems.length > 0) {
                                proxy._isFromSetmodel = true;
                                proxy.selectAllRows(false);
                                proxy._isFromSetmodel = false;
                            }
                            proxy.clearSelection(-1);
                            proxy.selectedRowIndex(-1);
                            proxy.model.selectedItem = null;
                            proxy._cancelSaveTools();
                        }
                        break;

                    case "allowTextWrap":
                    case "headerTextOverflow":
                        if (prop == "allowTextWrap")
                            model.allowTextWrap = options[prop];
                        if (prop == "headerTextOverflow") {
                            model.headerTextOverflow = options[prop];
                            proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                            proxy.setWidthToColumns();
                            proxy._trigger("headerRefresh");
                            proxy.getScrollElement().ejScroller("refresh");
                            proxy._updateScrollCss();
                        }
                        if (prop == "allowTextWrap") {
                            proxy._addInitTemplate();
                            proxy.renderRecords(args);
                            if (proxy.model.showTotalSummary) {
                                proxy._createTotalSummaryRow();
                            }
                        }
                        proxy._updateWrapRowHeights(null,"refreshAll");
                        break;

                    case "selectionType":
                    case "selectionMode":
                    case "selectionSettings":
                        this._initiateSetModel();
                        proxy.clearAllSelection();
                        proxy._refreshSelectionSettings(options, prop);
                        if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && $("#" + proxy._id + "_selectionpopup").length == 0 && !proxy.model.isFromGantt) {
                            proxy._renderMultiSelectionIndicator();
                        }
                        break;
                    case "showSummaryRow":
                        model.showSummaryRow = options[prop];
                        if (model.showSummaryRow) {
                            proxy._createSummaryRow();
                            var refreshArgs = {};
                            refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
                            proxy.sendDataRenderingRequest(refreshArgs);
                        }
                        else {
                            $(proxy.element).find(".e-summaryrow").hide();
                            model.updatedRecords = proxy._spliceSummaryRows(model.updatedRecords);
                            if (model.allowPaging) { 
                                proxy._updateCurrentViewData();
                                var refreshArgs = {};
                                refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
                                proxy.sendDataRenderingRequest(refreshArgs);
                            }
                        }                       
                        proxy.updateCollapsedRecordCount();
                        proxy._updateWrapRowHeights("header");
                        proxy.updateHeight();
                        if (model.allowTextWrap || model.headerTextOverflow == "wrap") {
                            proxy._updateGridContentHeight();
                        }
                        break;
                    case "totalSummaryHeight":
                        if (!model.showTotalSummary) return;                        
                        proxy._createTotalSummaryRow();
                        proxy._viewPortHeight = proxy._getViewPortHeight();                  
                        proxy._viewPortHeight -= proxy._$totalSummaryRowContainer.outerHeight();                      
                        proxy._updateSummaryHeight();                                
                        break;
                    case "collapsibleTotalSummary":
                        if (!model.showTotalSummary) return;
                        if (model.summaryRows.length == 1) {
                            model.collapsibleTotalSummary = false;
                            return;
                        }
                        proxy._addInitTemplate();
                        proxy._createTotalSummaryRow();
                        if (!model.collapsibleTotalSummary) {
                            proxy._viewPortHeight = proxy._getViewPortHeight();
                            proxy._viewPortHeight -= $(proxy._$totalSummaryRowContainer).outerHeight();
                            proxy._updateSummaryHeight();
                        }
                        break;
                    case "showTotalSummary":
                        model.showTotalSummary = options[prop];
                        if (model.showTotalSummary) {
                            proxy._createTotalSummaryRow();
                            proxy._viewPortHeight = proxy._getViewPortHeight();
                            proxy._viewPortHeight -= $(proxy._$totalSummaryRowContainer).outerHeight();
                        }
                        else {
                            $(proxy.element).find(".e-footersummaryrowdiv").remove();
                            proxy._$totalSummaryRowContainer = null,
                            proxy._$footertableContent = null,
                            proxy._$footerContainer = null,
                            proxy._footerSummaryRecord = [];
                            proxy._footerSummaryRows = [];
                            proxy._viewPortHeight = proxy._getViewPortHeight();
                        }
                        proxy._updateSummaryHeight();
                       
                        break;
                    case "selectedRowIndex":
                        if (!this.isEdit && !this._isRowEdit) {
                            if (this.selectedRowIndex() >= 0
                                && !ej.isNullOrUndefined(this.selectedRowIndex()) && model.allowSelection && updatedRecords.length > this.selectedRowIndex()
                                && this.getExpandStatus(updatedRecords[this.selectedRowIndex()])) {
                                var index = this.selectedRowIndex();
                                if (!proxy._rowSelectingEventTrigger(this._previousIndex, index)) {
                                    if (model.showSummaryRow) {
                                        var recordsWithOutSummary = proxy._spliceSummaryRows(model.updatedRecords),
                                            currentSelectedRecord = recordsWithOutSummary[index];
                                        index = updatedRecords.indexOf(currentSelectedRecord);
                                    }
                                    proxy.selectRows(index);
                                    proxy._rowSelectedEventTrigger(index);
                                }
                            } else {
                                proxy.clearSelection(-1);
                                proxy.selectedRowIndex(-1);
                                proxy.model.selectedItem = null;
                                proxy._previousIndex = -1;
                            }
                            proxy._cancelSaveTools();
                        } else {
                            this.selectedRowIndex(updatedRecords.indexOf(proxy._prevSelectedItem));
                        }
                        break;
                    case "selectedCellIndexes":
                        if (model.allowSelection && model.selectionSettings.selectionMode == "cell") {                            
                            proxy.selectCells(options[prop]);                            
                        }
                        break;
                    case "showStackedHeader":
                    case "stackedHeaderRows":
                        if (model.showStackedHeader && ((model["stackedHeaderRows"] && model.stackedHeaderRows.length > 0) ||
                            (options["stackedHeaderRows"] && options.stackedHeaderRows.length > 0))) {
                            if (ej.getObject("stackedHeaderRows.length", options))
                                model.stackedHeaderRows = options["stackedHeaderRows"];
                            (proxy.getHeaderTable().find(".e-stackedHeaderRow").length>0) && proxy.getHeaderTable().find(".e-stackedHeaderRow").remove();
                            proxy._refreshStackedHeader();
                        }
                        else
                            (proxy.getHeaderTable().find(".e-stackedHeaderRow").length > 0) && proxy.getHeaderTable().find(".e-stackedHeaderRow").remove();
                        break;
                    case "editSettings":
                        $.extend(model.editSettings, options[prop]);
                        proxy._processEditing();
                        proxy._enableEditingEvents();
                        proxy._updateToolbarItems();
                        if (model.editSettings.allowDeleting && model.editSettings.showDeleteConfirmDialog)
                        ej.isNullOrUndefined(proxy._confirmDialog) && proxy._renderConfirmDialog();
                        break;

                    case "allowKeyboardNavigation":
                        model.allowKeyboardNavigation = options[prop];
                        break;

                    case "dataSource":
                        var eventArgs = {};
                        eventArgs["requestType"] = ej.TreeGrid.Actions.RefreshDataSource;
                        eventArgs["isUpdateColumns"] = false;
                        if (!proxy._isRowEdit && !proxy.model.isEdit && !proxy._trigger("actionBegin", eventArgs)) {
                            proxy._fieldNamesToQuery = [];
                            proxy._assignedIdCollections = [];
                            proxy._assignedParendIDs = [];
                            proxy._refreshDataSource(eventArgs);
                        }
                        break;

                    case "columns":
                        model.columns = options[prop].slice(0);
                        proxy._validateColumnFields();
                        proxy._validateColumnVisible(true);
                        proxy._refreshFrozenColumns();
                        proxy._updateWrapRowHeights(null, "refreshAll");
                        break;
                    
                    case "rowHeight":
                        model.rowHeight = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        if (proxy.model.showTotalSummary) {
                            proxy._createTotalSummaryRow();
                            proxy._viewPortHeight = proxy._getViewPortHeight();
                            proxy._viewPortHeight -= proxy._$totalSummaryRowContainer.outerHeight();
                            proxy._updateSummaryHeight();                           
                        }
                        proxy.updateHeight();
                        break;

                    case "treeColumnIndex":
                        proxy.columnIndex(options[prop]);
                        break;

                    case "showColumnChooser":
                        model.showColumnChooser = options[prop];
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        proxy.setWidthToColumns();
                        proxy._trigger("headerRefresh");
                        proxy.getScrollElement().ejScroller("refresh");
                        proxy._updateScrollCss();
                        if (!model.showColumnChooser){
                            proxy._clearColumnMenu();
                            $("#" + proxy._id + "ccDiv_wrapper").remove();
                            $("#" + proxy._id + "filterMenu_wrapper").remove();
                        }
                        else {
                            proxy._renderColumnChooser();
                            if(model.allowFiltering && model.filterSettings.filterType == "menu" && proxy._renderedFilterMenuDialog.length == 0)
                            proxy._renderFilterMenu();
                        }
                        proxy._updateWrapRowHeights("row,total", "refreshAll");
                        /*Add empty column in gantt*/
                        if (model.isFromGantt) {
                            proxy._addEmptyColumntoGrid();
                            this._updateHeaderScrollLeft(this.getScrollElement().ejScroller("model.scrollLeft"));
                        }
                        break;
                    case "showColumnOptions":
                        model.showColumnOptions = options[prop];
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        proxy.setWidthToColumns();
                        proxy._trigger("headerRefresh");
                        proxy.getScrollElement().ejScroller("refresh");
                        proxy._updateScrollCss();
                        if (model.showColumnChooser && model.showColumnOptions) {
                            proxy.columnAddDialogTemplate();
                            proxy._renderUpdateConfirmDialog();
                        }
                        else {
                            proxy._clearColumnMenu();
                            $("#" + proxy._id + "ccDiv_wrapper").remove();
                            $("#" + proxy._id + "filterMenu_wrapper").remove();
                        }
                        proxy._updateWrapRowHeights("row,total", "refreshAll");
                        /*Add empty column in gantt*/
                        if (model.isFromGantt) {
                            proxy._addEmptyColumntoGrid();
                        }
                        break;
                    case "locale":
                        proxy._clearContextMenu();
                        proxy._clearColumnMenu();
                        proxy._removeDetailsRow();
                        model.locale = options[prop];
                        proxy._setCultureInfo();
                        proxy._processEditing();
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        //Re render the grid header(for updating filtering elements also) and content
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        proxy.renderRecords(args);
                        proxy.setWidthToColumns();
                        proxy._trigger("headerRefresh");
                        // Update the scrollbar
                        proxy.getScrollElement().ejScroller("refresh");
                        proxy._updateScrollCss();
                        // Update the column chooser
                        if (model.showColumnChooser)
                            proxy._renderColumnChooserList(true);
                        // Update the toolbar
                        if (model.toolbarSettings.showToolbar)
                            proxy._updateToolbar();
                        if (model.allowPaging)
                            proxy.getPager().ejPager("option", options[prop]).ejPager("refreshPager");
                        if (proxy.model.showColumnChooser && proxy.model.showColumnOptions) {
                            proxy.columnAddDialogTemplate();
                            proxy._renderUpdateConfirmDialog();
                        }
                        if (model.allowFiltering && model.filterSettings.filterType == "menu")
                            proxy._renderFilterMenu();
                        proxy._updateWrapRowHeights(null,"refreshAll");
                        break;
                    case "toolbarSettings": 
                        $.extend(proxy.model.toolbarSettings, options[prop]);
                        proxy._updateToolbar();
                        proxy._updateWrapRowHeights("header","refreshAll");
                        break;
                    case "contextMenuSettings":
                        $.extend(proxy.model.contextMenuSettings, options[prop]);
                        if (!model.contextMenuSettings.showContextMenu)
                            proxy._clearContextMenu();
                        this._removeDetailsRow();
                        break;
                    case "enableCollapseAll":
                        model.enableCollapseAll = options[prop];
                        if (model.enableCollapseAll)
                            proxy.collapseAll();
                        else
                            proxy.expandAll();
                        break;
                    case "showDetailsRow":
                        model.showDetailsRow = options[prop];
                        this._updateDetailsRowProperties("showDetailsRow");
                        break;
                    case "detailsRowHeight":
                        model.detailsRowHeight = options[prop];
                        this._updateDetailsRowProperties("detailsRowHeight");
                        break;
                    case "detailsTemplate":
                        model.detailsTemplate = options[prop];
                        this._updateDetailsRowProperties("detailsTemplate");
                        break;
                    case "showDetailsRowInfoColumn":
                        model.showDetailsRowInfoColumn = options[prop];
                        this._updateDetailsRowProperties("showDetailsRowInfoColumn");
                        break;
                    case "altRowTemplateID":
                        this._initiateSetModel();
                        model.altRowTemplateID = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        proxy._updateWrapRowHeights("header");
                        this.updateHeight();
                        if (model.allowTextWrap || model.headerTextOverflow == "wrap") {
                            proxy._updateGridContentHeight();
                        }
                        break;
                    case "rowTemplateID":
                        this._initiateSetModel();
                        model.rowTemplateID = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        proxy._updateWrapRowHeights("header");
                        this.updateHeight();
                        if (model.allowTextWrap || model.headerTextOverflow == "wrap") {
                            proxy._updateGridContentHeight();
                        }
                        break;
                    case "pageSettings":
                        this._initiateSetModel();
                        if (model.allowPaging) {
                            var pagerModel = {};
                            $.extend(model.pageSettings, options[prop]);
                            $.extend(pagerModel, model.pageSettings);
                            delete pagerModel.template;
                            this.getPager().ejPager("option", pagerModel).ejPager("refreshPager");
                            if (options[prop].totalRecordsCount > 0) {
                                proxy._refreshDataSource();
                            }
                                else
                                    proxy.gotoPage();
                            }
                        break;
                    case "sizeSettings":
                        this._initiateSetModel();
                        $.extend(model.sizeSettings, options[prop]);

                        this._calculateDimensions();
                        if (this._getViewPortHeight() <= 0) {
                            model.sizeSettings.height = "450px";
                            this._updateElementHeight();
                        }
                        this._windowResize();
                        break;
                    case "readOnly":
                        model.readOnly = options[prop];
                        break;
                    case "showGridCellTooltip":
                        model.showGridCellTooltip = options[prop];
                        break;

                    case "showGridExpandCellTooltip":
                        model.showGridExpandCellTooltip = options[prop];
                        break;

                    case "allowMultiSorting":
                        model.allowMultiSorting = options[prop];
                        break;

                    case "query":
                        if (!ej.isNullOrUndefined(options[prop]))
                            model.query = options[prop];
                        else
                            model.query = ej.Query();
                        proxy._refreshDataSource();
                        break;
                    case "filterSettings":
                        if (model.allowFiltering) {
                            if (!options.filterSettings.filterType && model.filterSettings.filterType == "filterbar") {
                            var filterSettings,
                                filterBarMode, args = {};
                            $.extend(proxy.model.filterSettings, options[prop]);
                            filterSettings = proxy.model.filterSettings;
                            filterBarMode = ej.isNullOrUndefined(filterSettings.filterBarMode) ? "immediate" : filterSettings.filterBarMode;
                            // Get the filteredColumn value before going to clear the filter columns
                            var newFilterColumn = $.extend([], filterSettings.filteredColumns);

                            for (var column = 0; column < model.columns.length; column++) {
                                if (model.columns[column].allowFiltering)
                                proxy._clearFilterElementValue(model.columns[column]);
                            }

                            filterSettings.filteredColumns = newFilterColumn;
                            filterSettings.filterBarMode = filterBarMode;
                            args.requestType = "filtering";

                            for (var i = 0; i < newFilterColumn.length; i++)
                                $("#" + proxy._id + "_" + newFilterColumn[i].field.split('.').join("") + "_filterbarcell").val(newFilterColumn[i].value);
                            //Render the record with sorting and filtering
                            proxy._ensureDataSource(args);
                            proxy._off(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                            proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
                            //Method binding for filtering data.
                            if (filterBarMode == "immediate") {
                                proxy._on(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                                proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox")._on(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
                            }
                            }
                            else if (options.filterSettings.filterType) {
                                proxy.clearFilter();
                                model.allowFiltering = true;
                                $.extend(proxy.model.filterSettings, options[prop]);
                                proxy._initiateSetModel();
                                if (proxy._isExcelFilter) {
                                    proxy._isExcelFilter = false;
                                    proxy._excelFilter.resetExcelFilter();
                                    proxy._excelFilter = null;
                                    proxy._excelColTypes = [];
                                }
                                var scroller = proxy.getScrollElement().ejScroller("instance"),
                                  left = scroller.scrollLeft(),
                                  top = scroller.scrollTop();
                                //Re render the grid header
                                proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                                proxy._setWidthToHeaders();
                                proxy._trigger("headerRefresh");
                                $("#" + proxy._id).addClass('e-treegrid-overflow');
                                proxy._viewPortHeight = proxy._getViewPortHeight();
                                $("#" + proxy._id).removeClass('e-treegrid-overflow');
                                if (model.filterSettings.filterType == "menu" && proxy._renderedFilterMenuDialog.length == 0)
                                    proxy._renderFilterMenu();
                                else if (model.filterSettings.filterType == "excel") {
                                    proxy._renderExcelFilter();
                                }
                                if (proxy._frozenColumnsLength > 0) {
                                    proxy._$gridContent.css("height", proxy._viewPortHeight);
                                }
                                scroller.option("height", model.sizeSettings.height == "auto" ? 0 : proxy._viewPortHeight);
                                if (model.enableVirtualization) {
                                    var length = model.currentViewData.length,
                                        rowHeight = model.rowHeight, isTopChanged;
                                    rowHeight += (model.showDetailsRow && !model.showDetailsRowInfoColumn) ? model.detailsRowHeight : 0;
                                    if ((length * rowHeight) < proxy._viewPortHeight) {
                                        if (length > 0) {
                                            top = top - (proxy._viewPortHeight - ((length - 1) * rowHeight));
                                            if (top < 0)
                                                top = 0;
                                        } else {
                                            proxy.processBindings();
                                            proxy.renderRecords();
                                        }
                                    }
                                }
                                //Update columnOptions fields
                                if (model.showColumnChooser && model.showColumnOptions)
                                    proxy.columnAddDialogTemplate();
                                //update the column width
                                this.setWidthToColumns();
                                scroller.scrollY(top, true);
                                scroller.scrollX(left, true);
                                proxy._updateHeaderScrollLeft(left);
                                proxy._updateScrollCss();
                            }
                            else
                            {
                                if (model.filterSettings.filterType == "excel") {
                                    proxy._excelColTypes = [];
                                    proxy._excelFilter.resetExcelFilter();
                                    proxy._excelFilter = null;
                                    proxy._renderExcelFilter();
                                }
                            }
                            if (options.filterSettings.filterHierarchyMode)
                                proxy.clearFilter();
                        }
                        break;
                    case "searchSettings":
                        var settings = options[prop];
                        for (var setting in settings) {
                            switch (setting) {
                                case "searchHierarchyMode":
                                    proxy.model.searchSettings.searchHierarchyMode = settings[setting];
                                    proxy.element.find("input.e-searchinput").val("");
                                    $("span.e-searchcancel").addClass("e-hide");
                                    proxy.search("");
                                    break;
                            }
                        }
                    case "columnResizeSettings":
                        var $headerTable, $contentTable, $totalSummaryTable;
                        this._initiateSetModel();
                        $.extend(model.columnResizeSettings, options[prop]);
                        this._updateGridResizeSettings();
                        var columnResizeMode = model.columnResizeSettings.columnResizeMode;
                        if (columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns) {
                            this._updateTableWidth();
                        } else {
                            if (this._frozenColumnsLength > 0) {
                                $headerTable = $(this.getHeaderTable()[1]);
                                $contentTable = $(this.getContentTable()[1]);
                                $totalSummaryTable = model.showTotalSummary ? $(this._$footertableContent[1]) : null;
                            } else {
                                $headerTable = this.getHeaderTable();
                                $contentTable = this.getContentTable();
                                $totalSummaryTable = model.showTotalSummary ? $(this._$footertableContent[0]) : null;
                            }
                            //set width to 100% for all tables in treegrid in normal resize mode
                            $headerTable.css("width", "").removeClass("e-tableLastCell");
                            $contentTable.css("width", "").removeClass("e-tableLastCell");
                            $totalSummaryTable && $totalSummaryTable.css("width", "").removeClass("e-tableLastCell");
                        }
                        this._updateScrollCss();
                        break;
                    case "cssClass":
                        this.element.removeClass(proxy._cssClass).addClass(options[prop]);
                        this.model.allowPaging && ($("#" + this._id + "Pager").ejPager("model.cssClass", options[prop]));
                        model.toolbarSettings.showToolbar && ($("#" + proxy._id + "_toolbarItems").ejToolbar("model.cssClass", options[prop]));
                        if (proxy._updateConfirmDialog) {
                            proxy._updateConfirmDialog.data("ejDialog") && proxy._updateConfirmDialog.ejDialog("model.cssClass", options[prop]);
                        }
                        for (var count = 0; count < proxy._renderedFilterMenuDialog.length; count++) {
                            if (proxy._renderedFilterMenuDialog[count] == "stringedit") {
                                $("#" + this._id + "filterStringMenuDiv_wrapper").find(".e-dropdownlist").ejDropDownList("model.cssClass", options[prop]);
                                $("#" + this._id + "filterStringMenuDiv_wrapper").find(".e-button").ejButton("model.cssClass", options[prop]);
                                $("#" + this._id + "filterStringMenuDiv_wrapper").find(".e-autocomplete").ejAutocomplete("model.cssClass", options[prop]);
                            }
                            else if (proxy._renderedFilterMenuDialog[count] == "numericedit") {
                                $("#" + this._id + "filterNumericMenuDiv_wrapper").find(".e-dropdownlist").ejDropDownList("model.cssClass", options[prop]);
                                $("#" + this._id + "filterNumericMenuDiv_wrapper").find(".e-button").ejButton("model.cssClass", options[prop]);
                                $("#" + this._id + "filterNumericMenuDiv_wrapper").find(".e-numerictextbox").ejNumericTextbox("model.cssClass", options[prop]);
                            }
                            else if (proxy._renderedFilterMenuDialog[count] == "datepicker") {
                                $("#" + this._id + "filterDatepickerMenuDiv_wrapper").find(".e-dropdownlist").ejDropDownList("model.cssClass", options[prop]);
                                $("#" + this._id + "filterDatepickerMenuDiv_wrapper").find(".e-button").ejButton("model.cssClass", options[prop]);
                                $("#" + this._id + "filterDatepickerMenuDiv_wrapper").find(".e-datepicker").ejDatePicker("model.cssClass", options[prop]);
                            }
                            else if (proxy._renderedFilterMenuDialog[count] == "datetimepicker") {
                                $("#" + this._id + "filterDatetimePickerMenuDiv_wrapper").find(".e-dropdownlist").ejDropDownList("model.cssClass", options[prop]);
                                $("#" + this._id + "filterDatetimePickerMenuDiv_wrapper").find(".e-button").ejButton("model.cssClass", options[prop]);
                                $("#" + this._id + "filterDatetimePickerMenuDiv_wrapper").find(".e-datetimepicker").ejDateTimePicker("model.cssClass", options[prop]);
                            }
                            else if ("#" + proxy._renderedFilterMenuDialog[count] == "dropdownedit") {
                                $("#" + this._id + "filterDropdownMenuDiv_wrapper").find(".e-dropdownlist").ejDropDownList("model.cssClass", options[prop]);
                                $("#" + this._id + "filterDropdownMenuDiv_wrapper").find(".e-button").ejButton("model.cssClass", options[prop]);
                            }
                            else if ("#" + proxy._renderedFilterMenuDialog[count] == "booleanedit") {
                                $("#" + this._id + "filterBooleanMenuDiv_wrapper").find(".e-dropdownlist").ejDropDownList("model.cssClass", options[prop]);
                                $("#" + this._id + "filterBooleanMenuDiv_wrapper").find(".e-button").ejButton("model.cssClass", options[prop]);
                            }
                        }
                        this.columnAddDialogTemplate();
                        $("#" + proxy._id + 'ConfirmDialog_wrapper').find(".e-button").ejButton("model.cssClass", options[prop]);
                        if (this.model.editSettings.allowEditing && this.model.editSettings.editMode == "dialogEditing" && (ej.isNullOrUndefined(proxy.model.editSettings.dialogEditorTemplateID) || proxy.model.editSettings.dialogEditorTemplateID == "")) {
                            proxy._treeGridAddEditDialog();
                        } else if (this.model.editSettings.allowEditing && this.model.editSettings.editMode == "dialogEditing" && (!ej.isNullOrUndefined(proxy.model.editSettings.dialogEditorTemplateID) && proxy.model.editSettings.dialogEditorTemplateID != "")) {
                            proxy._treeGridAddEditDialogTemplate();
                        }
                        if (model.showColumnChooser)
                            this._renderColumnChooserList(true);
                        proxy._cssClass = options[prop];
                        break;
                    case "enableLoadOnDemand":
                        model.enableLoadOnDemand = options[prop];
                        proxy._refreshDataSource();
                        break;
                }
            }
        },
        //update container height on dynamic update of total summary
        _updateSummaryHeight: function () {
            var proxy = this,
               model = this.model, prevSummaryTop;
            var scroller = proxy.getScrollElement().ejScroller("instance"),
                           left = scroller.scrollLeft(),
                           top = scroller.scrollTop();
            prevSummaryTop = top;
            if (proxy._frozenColumnsLength > 0) {
                proxy._$gridContent.css("height", proxy._viewPortHeight);
            }
            scroller.option("height", model.sizeSettings.height == "auto" ? 0 : proxy._viewPortHeight);          
            if (model.enableVirtualization) {
                var length = model.currentViewData.length,
                    rowHeight = model.rowHeight;
                rowHeight += (model.showDetailsRow && !model.showDetailsRowInfoColumn) ? model.detailsRowHeight : 0;
                if ((length * rowHeight) < proxy._viewPortHeight) {
                    if (length > 0) {
                        top = top - (proxy._viewPortHeight - ((length - 1) * rowHeight));
                        if (top < 0)
                            top = 0;
                    } else {
                        proxy.processBindings();
                        proxy.renderRecords();
                    }
                }
            }
            scroller.scrollY(top, true);
            scroller.scrollX(left, true);
            proxy._updateHeaderScrollLeft(left);
            proxy._updateScrollCss();
            proxy._updateWrapRowHeights("header", "refreshAll");
            if (prevSummaryTop == top) {
                proxy._updateCurrentViewData();
                var refreshArgs = {};
                refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.sendDataRenderingRequest(refreshArgs);
            }
        },
        /* clear contextMenu, columnChooser, detailsTemplate and save edited item before change
        property by setModel*/
        _initiateSetModel: function ()
        {
            var proxy = this;

            proxy._saveEditedElements();
            proxy._clearContextMenu();
            proxy._clearColumnMenu();
            proxy._removeDetailsRow();
        },
        /* update the tree grid control as per new detailsRow fetaures API values by setModel */
        _updateDetailsRowProperties: function (property) {

            var proxy = this,
                model = this.model;

            /* cance edited cell or row */
            if(model.isEdit || this._isRowEdit){
                this.cancelRowEditCell();
                this._cancelSaveTools();
            }
            /* clear columnchooser menu and contextmenu and detail wrapper */
            this.clearColumnMenu();
            this._clearContextMenu();
            this._removeDetailsRow();
             
            /* update the detailsRowHeight private property */
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn) {
                proxy._detailsRowHeight = model.detailsRowHeight;
            } else {
                proxy._detailsRowHeight = 0;
            }

            /* update the details expand status as per details feture API values */
            model.flatRecords.forEach(function (record) {
                if (model.showDetailsRow && model.detailsTemplate) {
                    record.isDetailsExpanded = model.showDetailsRowInfoColumn ? false : true;
                } else {
                    record.isDetailsExpanded = false;
                }
            });
            
            /* update the tree grid control as per new detailsRow API's */
            if ((property == "detailsTemplate" || property == "detailsRowHeight") && model.showDetailsRowInfoColumn) {
                proxy._addInitTemplate();
            } else {
                proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                proxy._addInitTemplate();
                /* while changing details row features API scroll top is assigned to beacause of rowHeight changes*/
                this.onScrollHelper(0);
                proxy.refreshContent();

                /* for details rows detailsDataBound event can be called after the width update of column */
                proxy._isRendered = false;
                //Rerendering the content of the tree grid
                var refreshArgs = {};
                refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.sendDataRenderingRequest(refreshArgs);
                proxy.setWidthToColumns();
                proxy._isRendered = true;
                proxy._trigger("headerRefresh");
                proxy._trigger("refresh");
                /* trigger queryCellInfo ,detailsDataBoundEvent after update the width of the columns */
                this._eventBindings();
                this._hideCollapsedDetailsRows();

                proxy.getScrollElement().ejScroller("refresh");
                proxy._updateScrollCss();
            }
            proxy._updateWrapRowHeights(null,"refreshAll");
        },
        /* hide the collapsed state detail rows for prevent alignement issue for rendering sf control in  display none state*/
        _hideCollapsedDetailsRows:function()
        {
            var proxy = this, model = this.model;
            /* Collapse hidden expanded details rows */
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn) {
                if (this._frozenColumnsLength > 0)
                    $(proxy.getTreeGridRows()[0]).add(proxy.getTreeGridRows()[1]).filter("tr.e-collapsedrowexpandeddetailsrow").css("display", "none");
                else
                    $(proxy.getTreeGridRows()).filter("tr.e-collapsedrowexpandeddetailsrow").css("display", "none");
            }
        },
        _updateToolbar: function () {
            var proxy = this,
                model = proxy.model,
                scroller = proxy.getScrollElement().ejScroller("instance"),
                           left = scroller.scrollLeft(),
                           top = scroller.scrollTop();
            //$.extend(proxy.model.toolbarSettings, options[prop]);
            $("#" + proxy._id + "_toolbarItems").remove();
            $("#" + proxy._id + "_toolbarItems_Main").remove();
            if (model.toolbarSettings.showToolbar)
                proxy._renderToolbarTemplate().insertBefore($("#" + proxy._id + "e-gridheader"));
            if (model.isEdit)
                proxy.cancelEditCell();
            if (proxy._isRowEdit) {
                proxy.cancelRowEditCell();
                proxy._cancelSaveTools();
            }
            proxy._viewPortHeight = proxy._getViewPortHeight();
            if (proxy._frozenColumnsLength > 0) {
                proxy._$gridContent.css("height", proxy._viewPortHeight);
            }
            scroller.option("height", model.sizeSettings.height == "auto" ? 0 : proxy._viewPortHeight);
            if (model.enableVirtualization) {
                proxy.processBindings();
                proxy.renderRecords();
            }
            scroller.option("scrollTop", top);
            scroller.option("scrollLeft", left);
            proxy._clearContextMenu();
            this._removeDetailsRow();

        },
        //refresh data source on set model     
        _refreshDataSource: function (refreshArgs)
        {
            var proxy = this, model = this.model, args;
            //Cancelling the editable state of the row
            if (proxy._isRowEdit || proxy.model.isEdit) {
                proxy.cancelRowEditCell();
            }
            proxy.resetModelCollections();
            proxy._createdAt = "load";
            if (refreshArgs && refreshArgs.isUpdateColumns) {
                proxy._validateColumnFields();
                proxy._validateColumnVisible(true);
                proxy._refreshFrozenColumns(true);
            }

            proxy._dataManager = proxy.dataSource() instanceof ej.DataManager ? proxy.dataSource() :
                proxy.dataSource() != null ? ej.DataManager(proxy.dataSource()) : null;

            if (this.dataSource() instanceof ej.DataManager || this.dataSource()) {
                proxy._isForDataSourceRefresh = true;
                proxy._initDatasource(); 
            }
            else if (this.dataSource() == null) {
                args = {};
                args["requestType"] = ej.TreeGrid.Actions.RefreshDataSource;
                proxy._ensureDataSource(args);
                proxy._createdAt = null;
                if (model.showSummaryRow || model.showTotalSummary) {
                    proxy._summaryRow();
                }
            }
            $(".e-tooltiptable").remove();
        },
        //REFRESH THE TREEGRIDCONTENT
        refreshContent: function () {

            var proxy = this,
                args = {};     
            args.requestType = ej.TreeGrid.Actions.Refresh;
            proxy.processBindings(args);
        },


        //RESET THE COLLECTIONS WHILE CHANGING THE DATASOURCE
       
        resetModelCollections: function () {

            var proxy = this,
            model = proxy.model;

            // model.sortSettings.sortedColumns = [];
            model.currentViewData = [];
            model.flatRecords = [];
            model.parentRecords = [];
            model.updatedRecords = [];
            model.summaryRowRecords = [];
            model.ids = [];
            proxy.secondaryDatasource = [];
            proxy._parentRecords = [];
            proxy._storedIndex = -1;
            proxy._idGenerator = -1; //unique id only use for treegrid
            proxy._taskIds = [];
            proxy._jsonData = null;
            proxy._lastInsertedId = -1;
        },        

        //#endregion

        //DECLARE THE PRIVATE VARIABLES
        _initPrivateProperties: function () {

            var proxy = this,
                model = this.model;
            proxy._isInManualExpandCollapse = false; // Property to diffrentiate footer expand and collapse action by user or by method
            proxy._id = proxy.element.attr("id"),
            proxy.columnsWidthCollection = [],
            proxy._cSortedColumn = null,
            proxy._taskIds=[],
            proxy._maxZIndex=0,
            proxy._cSortedDirection = null,
            proxy._dataManager = null,
            proxy._persistCollapseState = false,
            proxy._filteredRecords = [],
            proxy._disabledToolItems = [],
            proxy._currentFilterColumn = [],
            proxy._fieldColumnNames = [],
            proxy._multiSortRequest = null,
            proxy._$gridContainer = null,
            proxy._$gridContent = null,
            proxy._$gridContentTable = null,
            proxy._$gridHeaderTable = null,
            proxy._$gridHeaderContainer = null,
            proxy._$totalSummaryRowContainer = null,
            proxy._recordsCount = null,
            proxy._$gridRows = null,
            proxy._gridRows = null,
            proxy._commandColumnLength = 0,
            this._frozenColumnsLength = 0,
            proxy._initFrozenColumns(),
            proxy._allowFreezingDefault = this._frozenColumnsLength > 0 ? true : false;
            proxy._calculateDimensions(),
            proxy._headerColumnNames = [],
            proxy._hiddenColumns = [],
            proxy._multiSelectCtrlRequest = false,
            proxy._multiSelectShiftRequest = false,
            proxy._ganttTouchTrigger = false,
            proxy._moveRow = false,
            proxy._draggedRecords = [];
            proxy._offset = 0,
            proxy._previousIndex = -1,
            proxy._prevRBottom = 0,
            proxy._prevRTop = 0,
            proxy._prevScrollTop = 0,
            proxy._prevScrollLeft = 0,
            proxy._prevVBottom = 0,
            proxy._resizer = null,
            proxy._scrollTop = 0,
            proxy._scrollLeft = 0,
            proxy._searchString = "",
            proxy._fieldName = "",
            proxy._selectedCellsinARow = [],
            proxy._currentRecordIndex = -1,
            proxy._sortedRecords = [],
            proxy._storedIndex = -1,
            proxy._rowIndex = -1,
            proxy._$tableContent = null,
            proxy._$footertableContent = null,
             proxy._$footerContainer = null,
            proxy._summaryExpanded = true,
            proxy._tempsortedrecords = [],
            proxy._totalHeight = 0,
            proxy._viewPortHeight = 0,
            proxy._recordIndexCount = 0,
            proxy._totalCollapsedRecordCount = 0,
            proxy._visibleColumns = [],
            proxy._visibleRange = null,
            proxy._vScrollDir = 1,
            proxy._vScrollDist = 0,
            proxy._cellEditTemplate = $(),
            proxy._tooltipTimer,
            proxy._totalBorderWidth = 2;//left and right border width
            proxy._totalBorderHeight = 1;
            proxy._toolboxTooltipTexts = null;
            proxy._contextMenuTexts = null;
            proxy._columnMenuTexts = null;
            proxy._deleteColumnText = null;
            proxy._okButtonText = null;
            proxy._cancelButtonText = null;
            proxy._confirmDeleteText = null;
            proxy._dropDownListBlanksText = null;
            proxy._editTypeText = null;
            proxy._textAlignType = null;
            proxy._clipMode = null;
            proxy._dropDownListClearText = null;
            proxy._trueText = null;
            proxy._falseText = null;
            proxy._rowDragIndexes = [];
            proxy._dropCancel = false;
            proxy._cellEditingDetails = {
                cellValue: null,
                rowIndex: -1,
                columnIndex: -1,
                fieldName: null,
                _data: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                data: null
            },
           proxy._previousSelectedCellDetails = {
               rowElement: null,
               cellElement: null,
               cellIndex: -1,
               rowIndex: -1,
           },
           proxy._shiftKeyFirstElementDetails = {
               firstElementRowIndex: -1,
               firstElementCellIndex: -1
           },
            proxy._selectedCellDetails = [];
            proxy._focusingRowIndex = -1,
            proxy._rowIndexOfLastSelectedCell = -1,
            proxy._lastSelectedCellIndex = -1,
            proxy._isEnterKeyPressed = false,
            proxy._isShiftKeyNavigation = false;
            proxy._scrollBarHeight = 18,
            proxy._removedCount = 0,
            proxy._$gridHeaderContent = null,
            proxy.secondaryDatasource = [],
            //variables for context menu items
            proxy._contextMenuItems = null;
            proxy._subContextMenuItems = null;
           
            proxy._createContextMenuTemplate();
            proxy._createColumnMenuTemplate();
            proxy._retrivedData = this.dataSource();
            proxy._createdAt = "load";
            proxy._isInExpandCollapseAll = false;
            proxy._isLoadonDemand = false;
            proxy._assignedIdCollections = [];
            proxy._totalItemCounts = null;
            proxy._assignedParendIDs = [];
            proxy._fieldNamesToQuery = [];
            proxy._isForDataSourceRefresh = false;
            proxy._allParentItems = [];
            proxy._queryForLoadonDemand = [];
            proxy._pageRecordCountOnDemand = null;
            proxy._isFromDemandMethod = false;                        
            proxy._dataManager = proxy.dataSource() instanceof ej.DataManager ? proxy.dataSource() :
                proxy.dataSource() != null ? ej.DataManager(proxy.dataSource()) : null;
            proxy._dragMouseDown = false;
            proxy._dragTooltip = false;
            proxy._dragMouseLeave = false;
            proxy._timerDragDown = null;
            proxy._timerDragUp = null;
            /* Check data manger is updatable */ 
            proxy._isDataManagerUpdate = false;
            proxy._jsonData = null;

            /* flag for refreshing newly added record */
            proxy._isRefreshAddedRecord = false;
            /* Used as flag for refresh chart and grid rows on adding when parent item is in collpased state*/
            proxy._isInAdd = false;         
            /* get unique index for tree grid add whil add */
            proxy._maxRowIndex = 0;
           /* used to check treegrid rendering is completed on load time*/
            proxy._isRendered = false;
            this._updateGridResizeSettings();
            proxy.batchChanges = {
                added: [],
                deleted: [],
                changed: [],
                updatedHierarchyRowIndexData: []
            };
            proxy._batchEditCancel = false;
            proxy._isSummaryRow = false;
            proxy._flatChildRecords = [];
            proxy._footerSummaryRecord = [];
            proxy._filteredChildRecords = [];
            proxy._parentRecords = [];
            proxy._zerothLevelParentRecords = [];
            proxy._detailsRowHeight = 0;
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn)
            {
                proxy._detailsRowHeight = model.detailsRowHeight;
            }
            proxy._isEmptyRow = false;            
            proxy._prevSelectedItem = null;
            proxy._dataSourcefields = [];
            proxy._columnDialogTexts = null;
            proxy._columnDialogTitle = null;
            proxy._addColumnFields = [];
            proxy._addColumnFieldsTxt = [];
            proxy._targetColumnIndex = 0;
            proxy._insertPosition = null;
            proxy._updateconfirmDialog = null;
            proxy._columnRenameDialog = null;
            proxy._renderedSubMenuType = null;
            proxy.localizedLabels = proxy._getLocalizedLabels();
            proxy._gridRecordsCount = model.pageSettings.totalRecordsCount;                            
            proxy._gridPager = null;
            proxy._summaryRowsCount = 0;                                 
            proxy._updatedPageData = [];
            proxy._isNextPage = false;
            model.selectionSettings.selectionType = model.selectionSettings.selectionType != "single" ? model.selectionSettings.selectionType :
                    model.selectionType != "single" ? model.selectionType : model.selectionSettings.selectionType;
            model.selectionSettings.selectionMode = model.selectionSettings.selectionMode != "row" ? model.selectionSettings.selectionMode :
                    model.selectionMode != "row" ? model.selectionMode : model.selectionSettings.selectionMode;
            proxy._idGenerator = -1;
            proxy._checkboxColumnName = null;
            proxy._isFromSetmodel = false;
            proxy._flatFilteredRecords = [];
            proxy._checkboxSelection = (model.selectionSettings.selectionType == ej.TreeGrid.SelectionType.Checkbox && 
                model.selectionSettings.selectionMode == ej.TreeGrid.SelectionMode.Row),            
            proxy._addCheckboxColumn = this._getDefaultCheckboxColumn(),
            proxy._filteredColumnValue = [];
            proxy._filteredColumnValueID = [];
            proxy._filterIconTarget = null;
            proxy._renderedFilterMenuDialog = [];
            proxy._filtermenutext = null;
            proxy._headerDragStatus = "";
            proxy._draggedColumn = null;
            proxy._mouseDragged = false;
            if (!ej.isNullOrUndefined(model.indentLevelWidth) && model.isFromGantt)
                proxy.treeIndentLevelWidth = model.indentLevelWidth;
            else
                proxy.treeIndentLevelWidth =  proxy.treeIndentLevelWidth ? proxy.treeIndentLevelWidth : 10;
            proxy._draggedColumnTemplate = null;
            proxy._viewPortWidth = proxy._gridLeft = 0;            
            proxy._responsiveMinWidth = 768;
            proxy._dy = proxy._dx = 0;
            proxy._isTouchEvent = false;
            proxy._isTouchTrigger = false;
            proxy._currentEditRecordDialogIndex = -1;
            proxy._isTreeAddEditDialogSave = false;
            proxy._displayContextMenu = model.contextMenuSettings.showContextMenu ? true : false;
            proxy._isEmptyRowDeleted = false;
            proxy._cssClass = this.model.cssClass;
            proxy._batchEditStarts = false;
            proxy._batchSaveCancel = "";
            proxy._batchSaveConfirmText = null;
            proxy._batchSaveLostChangesText = null;
            proxy._cancelEditText = null;
            proxy._batchTempDetails = {};
            proxy._autoHideColumns = [];
            proxy._autoShowColumns = [];
            proxy._textWrapRowsDatas = [];
            proxy._textWrapNoCalculate = false;
            proxy._expandCollapsePages = [];
            proxy._expandCollapseSettings = {
                state: "",
                atLevel: "",
                level: 0,
            };
            proxy._excelFilter = null;
            proxy._menuColTypes = [];
            proxy._excelColTypes = [];
            proxy._isExcelFilter = false;
            proxy._customFilter = null;
            proxy._mediaStatus = false;
            proxy._filterString = "";
            proxy._isCloseFilterMenu = false;
            proxy._isRemoteData = false;
            proxy._lastInsertedId = -1;
            proxy._confirmDialog = null;
            proxy._shiftSelectedRecord = null;
            proxy._deleteRowIndex = -1;
            proxy._isFromContextmenu = false;
            proxy._emptyRowIndex = -1;
        },
        //Return default check box selection column obect
        _getDefaultCheckboxColumn: function () {
            return {
                field: "checkboxState",
                headerText: "",
                allowEditing: false,
                allowDelete: false,
                allowSorting: false,
                allowFiltering: false,
                allowCellSelection: false,
                isFrozen: false,
                nonColumn:true,
                editType: "booleanedit",
                width: 50,
                textAlign:"center"
            };
        },
        //update columnResizeSettings API to grid resizeSettings API
        _updateGridResizeSettings: function () {
            var resizeSettings = {
                resizeMode: (this.model.columnResizeSettings.columnResizeMode == "fixedcolumns" ? "control" : this.model.columnResizeSettings.columnResizeMode)
            };
            this.model.resizeSettings = resizeSettings;
        },
        //set size to treegrid core by user values
        _calculateDimensions: function ()
        {
            var proxy = this, model = this.model;

            if (!model.isFromGantt) {
                if (model.sizeSettings.width)
                    proxy.element.css("width", model.sizeSettings.width);

                if (model.sizeSettings.height)
                    proxy.element.css("height", model.sizeSettings.height);
                else
                {
                    if (!proxy.element[0].style.height) {
                        proxy.element.height(450);
                    }
                    else
                        model.sizeSettings.height = proxy.element[0].style.height;
                }
            }
            proxy._gridHeight = Math.round(proxy.element.height());
            proxy._gridWidth = Math.round(proxy.element.width());

            if (this._frozenColumnsLength > 0) {
                var frozenWidth = 0;
                for (var count = 0; count <= this._frozenColumnsLength; count++) {
                    if (model.columns[count])
                        frozenWidth += ej.isNullOrUndefined(model.columns[count].width) ? 150 : parseInt(model.columns[count].width);
                }
                if (proxy._gridWidth < frozenWidth) {
                    proxy.element.css("width", frozenWidth);
                    model.sizeSettings.width = proxy._gridWidth = frozenWidth;
                }
            }
        },

        /* Reordering and initialize frozen and unfrozen columns*/
        _initFrozenColumns: function ()
        {
            var frozen = [], unfrozen = [], visibleFrozenColumns = 0, visibleUnfrozenColumns = 0,
                model = this.model, columns = model.columns, treeColumn, summaryTreeColumn;
            treeColumn = columns[model.treeColumnIndex];
            if (model.showTotalSummary && model.collapsibleTotalSummary) summaryTreeColumn = columns[this._summaryColumnIndex];
            for (var columnCount = 0; columnCount < columns.length; columnCount++) {
                if (columns[columnCount]["isFrozen"] === true) {
                    frozen.push(columns[columnCount]);
                    if (ej.isNullOrUndefined(columns[columnCount].visible) || columns[columnCount].visible || columns[columnCount] == treeColumn)
                        visibleFrozenColumns += 1;
                }
                else {
                    unfrozen.push(columns[columnCount]);
                    if (ej.isNullOrUndefined(columns[columnCount].visible) || columns[columnCount].visible || columns[columnCount] == treeColumn)
                        visibleUnfrozenColumns += 1;
                }
            }
            if (frozen.length > 0) {
                /*one column must be in un frozen state*/
                if (model.columns.length && model.columns.length == frozen.length) {
                    unfrozen = frozen.splice(frozen.length - 1, 1);
                    unfrozen[0].isFrozen = false;
                }
                model.columns = $.merge($.merge([], frozen), unfrozen);
            }
            this._frozenColumns = frozen;
            this._unFrozenColumns = unfrozen;
            /*Update minimum number of column to visible*/
            if (visibleFrozenColumns == 0 && this._frozenColumns[0])
                this._frozenColumns[0].visible = true;
            if (visibleUnfrozenColumns == 0 && this._unFrozenColumns[0])
                this._unFrozenColumns[0].visible = true;
            if (treeColumn)
                model.treeColumnIndex = model.columns.indexOf(treeColumn);
            if (model.showTotalSummary && model.collapsibleTotalSummary)
                this._summaryColumnIndex = model.columns.indexOf(summaryTreeColumn);
            this._frozenColumnsLength = frozen.length;
            model.scrollSettings.frozenColumns = this._frozenColumnsLength;
        },
        //update altrow when enableCollapseAll enabled.
        updateAltRowOnCollapseAll: function () {
            var proxy=this, model=proxy.model;
            if (model.enableAltRow && model.enableCollapseAll) {
                var records = model.parentRecords,
                    record;
                for (var count = 0; count < records.length; count++) {
                    record = records[count];
                    if (record.isSummaryRow)
                        continue;
                    record.isAltRow = count % 2 == 0 ? false : true;
                }
            }
        },
        //update altrow when expandStateMapping is defined.
        updateAltRowOnRendering: function () {
            var proxy = this, model = proxy.model;           
            var records = model.updatedRecords.filter(function (record) {
                if (record.parentItem == null)
                    return record;
                else if (record.parentItem.expanded)
                    return record;
            });
            for (var count = 0; count < records.length; count++) {
                var record = records[count];
                if (record.isSummaryRow)
                    continue;
                record.isAltRow = count % 2 == 0 ? false : true;
            }            
        },
        //DATASOURCE CHECKING
        _checkDataBinding: function () {

            var proxy = this,
                model = proxy.model;
            $("#" + proxy._id).addClass('e-treegrid-overflow');
            proxy._createdAt = null;
            proxy.element.addClass('e-treegrid-core');
            proxy.element.attr("tabindex", "0");
            model.cssClass && this.element.addClass(model.cssClass);
            proxy._renderToolbar();
            if (proxy._checkboxSelection)
                proxy._updateCheckboxColumnName();
            if (model.isFromGantt && model.enableSerialNumber)
                proxy._updateTreeColumnIndex();
            proxy._renderAfterColumnInitialize();
            if (proxy.model.showColumnChooser || model.contextMenuSettings.showContextMenu) {
                proxy._renderColumnChooser();
            }
            if (proxy.model.allowFiltering && model.filterSettings.filterType == "menu")
                proxy._renderFilterMenu();
            else if (model.allowFiltering && model.filterSettings.filterType == "excel")
                proxy._renderExcelFilter();
            proxy._setWidthToHeaders();
            if (proxy.model.allowPaging) {
                proxy.element.append(proxy._renderGridPager());
            }
            proxy._viewPortHeight = proxy._getViewPortHeight();

            //rendering confirmation dialog initially
            if (proxy.model.editSettings.showConfirmDialog || proxy.model.editSettings.showDeleteConfirmDialog) {
                proxy._renderConfirmDialog();
            }

            /* check if viewport heigth value*/
            if (!model.isFromGantt && proxy._viewPortHeight < 0) {
                proxy._updateElementHeight();
            }
            proxy._searchString = model.searchSettings.key;
            proxy._ensureDataSource();
            if (model.showSummaryRow)
                proxy._createSummaryRow();
            if (!model.isFromGantt)
                proxy.updateAltRowOnCollapseAll();
            if (model.expandStateMapping && model.enableAltRow)
                proxy.updateAltRowOnRendering();
            proxy._initGridRender();          
            proxy._$gridContent = $("#" + proxy._id + "e-gridcontent");
            proxy._$gridContainer = $("#" + proxy._id + "e-gridcontainer");
            proxy._$tableContent = $("#" + proxy._id + "e-table");
            proxy._$frozenTableContent = $("#" + proxy._id + "frozene-table");
            proxy._$movableTableContent = $("#" + proxy._id + "movablee-table");
            proxy._viewPortHeight = proxy._getViewPortHeight();
            proxy._$gridContent.css({ "height": proxy._viewPortHeight + "px" });
            proxy.element.find(".e-pager").css({ "width": proxy._gridWidth - 2 });
            proxy._pagerHeight = proxy.element.find(".e-pager").outerHeight();
            if (model.showTotalSummary) {
                proxy._createTotalSummaryRow();
                proxy._viewPortHeight = proxy._getViewPortHeight();
                if (!model.allowPaging || model.sizeSettings.height) {
                    proxy._viewPortHeight -= proxy._$totalSummaryRowContainer.height() < model.totalSummaryHeight ? proxy._$totalSummaryRowContainer.height() : model.totalSummaryHeight;
                    proxy._viewPortHeight = proxy._viewPortHeight - 1 // border bottom
                }
            }
            proxy._updatePagerTextAlignment();
           // proxy.updateHeight();
            proxy._wireEvents();
            
            proxy.setWidthToColumns();
            proxy._trigger("headerRefresh");
            proxy._renderScroller();
            proxy._updateWrapRowHeights();
            if (model.allowTextWrap && model.enableVirtualization) {
                proxy._textWrapRowsDatas = proxy._getAvgTextWrapRowHeight("getAvgHeight");
            }
            proxy.updateHeight();
            proxy._updateScrollCss();
            proxy._resizeFilteringElements();
            proxy._updateFixedTableBorder();
            proxy._initialEndRendering();
            if (model.allowTextWrap || model.headerTextOverflow == "wrap")
                proxy._updateGridContentHeight();
            if (model.isFromGantt) {
                proxy._addEmptyColumntoGrid();
            }
            if (!model.isFromGantt) {
                proxy._checkDataManagerUpdate();
            } else {
                proxy._isDataManagerUpdate = model.dataManagerUpdate.isDataManagerUpdate;
                proxy._jsonData = model.dataManagerUpdate.jsonData;
            }
            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && !proxy.model.isFromGantt) {
                proxy._renderMultiSelectionIndicator();
            }
            $("#" + proxy._id).removeClass('e-treegrid-overflow');
        },

        //Render Multi selection indicator
        _renderMultiSelectionIndicator: function () {
            this._multiSelectPopup = ej.buildTag("div.e-treegridpopup#" + this._id + "_selectionpopup", "", { display: "none" });
            var $content = ej.buildTag("div.e-popupcontent"), $downTail = ej.buildTag("div.e-downtail e-tail");
                var $selElement = ej.buildTag("span.e-rowselect e-icon");
                $content.append($selElement);
            this._multiSelectPopup.append($content);
            this._multiSelectPopup.append($downTail);
            this.element.append(this._multiSelectPopup);
        },

        //Updated records of online data manager 
        getUpdatedDataManagerData: function () {
            var proxy = this, data;
            data = proxy._jsonData;
            return data;
        },
        /* To update tree column index of TaskId column during Serial number */
        _updateTreeColumnIndex: function () {
            var proxy = this, model = proxy.model,
                columns = model.columns,
                taskIdColumn, taskIdIndex,
                treeIndex = +(model.treeColumnIndex);
            $.map(columns, function (col) {
                if (col.field == "taskId") {
                    taskIdColumn = col;
                    return false;
                }
            });
            taskIdIndex = columns.indexOf(taskIdColumn);
            if (taskIdIndex == treeIndex) {
                if (taskIdIndex != columns.length - 1)
                    model.treeColumnIndex = treeIndex + 1;
                else
                    model.treeColumnIndex = treeIndex - 1;
            }
        },

        //Method to update the text wrap grids heights.
        //element atgument is used prevent the specific row heights.
        //refreshAction argument is used to make necessary view port height updation.
        //Index and type argument is used to update the specific row height.
        _updateWrapRowHeights: function (element, refreshAction, index, type) {
            var proxy = this, model = this.model;
            if ((model.allowTextWrap || model.headerTextOverflow == "wrap") && proxy._$frozenTableContent.length > 0) {
                proxy.getScrollElement().ejScroller("refresh");
                if (model.headerTextOverflow == "wrap" && (ej.isNullOrUndefined(element) || element.indexOf("header") == -1))
                    proxy._updateHeaderHeight();
                if (model.allowTextWrap && (ej.isNullOrUndefined(element) || element.indexOf("row") == -1)) {
                    if (ej.isNullOrUndefined(index) && ej.isNullOrUndefined(type))
                        proxy._updateRowHeight();
                    else
                        proxy._updateRowHeight(index, type);
                    if (model.showTotalSummary && (ej.isNullOrUndefined(element) || element.indexOf("total") == -1)) {
                        proxy._updateTotalSummaryHeight();
                    }
                    proxy.getScrollElement().ejScroller("refresh");
                    var movableScrollTop = proxy.getScrollElement().ejScroller("model.scrollTop");
                    proxy._getFrozenScrollElement().scrollTop(movableScrollTop);
                }
            }
            if (refreshAction == "virtualRefresh" && (model.allowTextWrap || model.headerTextOverflow == "wrap")) {
                if (model.allowTextWrap && model.enableVirtualization) {
                    proxy.updateHeight();
                }
                proxy._updateGridContentHeight();
            } else if (refreshAction == "refreshAll" && (model.allowTextWrap || model.headerTextOverflow == "wrap")) {
                proxy.updateHeight();
                proxy._updateGridContentHeight();
            }
            if (model.sizeSettings.height == "auto") {
                proxy._updateContentHeight();
            }
        },

        //Update Header height when and frozen column text wrap is in enable state.
        _updateHeaderHeight: function () {
            var proxy = this, maxheight = 0, frozen = proxy.getHeaderContent().find("#e-frozenheaderdiv" + this._id).find('tr')[0],
                movable = proxy.getHeaderContent().find("#e-movableheaderdiv" + this._id).find('tr')[0];
            $(movable).css('height', 'auto');
            $(frozen).css('height', 'auto');
            if (frozen.offsetHeight > movable.offsetHeight)
                $(movable).css('height', frozen.offsetHeight);
            else if (movable.offsetHeight > frozen.offsetHeight)
                $(frozen).css('height', movable.offsetHeight);
        },

        //Update total summary row height when text wrap and frozen column  is in enable state.
        _updateTotalSummaryHeight: function () {
            var proxy = this, model = this.model, maxheight = 0, frozen = $(proxy._$footertableContent[0]).find('tr'),
                movable = $(proxy._$footertableContent[1]).find('tr'), length = $(frozen).length, frozenHeight, movableHeight;
            for (var index = 0; index < length; index++) {
                $(movable[index]).css('height', model.rowHeight);
                $(frozen[index]).css('height', model.rowHeight);
                frozenHeight = $(frozen[index]).height();
                movableHeight = $(movable[index]).height();
                if (frozenHeight >= movableHeight) {
                    $(movable[index]).css('height', frozenHeight);
                    $(frozen[index]).css('height', frozenHeight);
                } else if (movableHeight > frozenHeight) {
                    $(frozen[index]).css('height', movableHeight);
                    $(movable[index]).css('height', movableHeight);
                }
            }
                if (proxy._$totalSummaryRowContainer.hasClass("e-scroller"))
                    proxy._$totalSummaryRowContainer.ejScroller("refresh");
        },

        //Update all rows height when text wrap and frozen column is in enable state.
        _updateRowHeight: function (index, type) {
            if (this.getRows().length > 0) {
            var proxy = this, model = this.model, frozenHeight = 0, start = 0, movableHeight = 0, length = 0, frozen = [], movable = [], currentData;
            if (!ej.isNullOrUndefined(index)) {
                if (model.enableVirtualization && type != "rowEdit") {
                    currentData = model.currentViewData[index];
                    index = proxy.getUpdatedRecords().indexOf(currentData);
                }
                start = index;
                length = index + 1;
                frozen[start] = $(ej.TreeGrid.getRowByIndex(proxy, index)[0]);
                movable[start] = $(ej.TreeGrid.getRowByIndex(proxy, index)[1]);
            } else {
                start = 0;
                length = proxy.getRows()[0] ? proxy.getRows()[0].length : 0;
                frozen = proxy.getRows()[0];
                movable = proxy.getRows()[1];
            }
            for (var rowIndex = start; rowIndex < length; rowIndex++) {
                if ((!ej.isNullOrUndefined(index) && (type == "rowEdit" || type == "savecell")) || type == "resize") {
                    $(frozen[rowIndex]).css('height', model.rowHeight);
                    $(movable[rowIndex]).css('height', model.rowHeight);
                }
                if (type != "rowEdit") {
                    frozenHeight = $(frozen[rowIndex]).height();
                    movableHeight = $(movable[rowIndex]).height();
                    if (frozenHeight >= movableHeight) {
                        $(movable[rowIndex]).css('height', frozenHeight);
                        $(frozen[rowIndex]).css('height', frozenHeight);
                    } else if (movableHeight > frozenHeight) {
                        $(frozen[rowIndex]).css('height', movableHeight);
                        $(movable[rowIndex]).css('height', movableHeight);
                    }
                }
            }
                }
        },

        //Check if data manager is updatable
        _checkDataManagerUpdate: function () {
            var proxy = this,
                dataSource = this.dataSource();
            proxy._isDataManagerUpdate = false;
            proxy._jsonData = null;

            if (dataSource instanceof ej.DataManager) {
                if (dataSource.dataSource.offline && dataSource.dataSource.json) {
                    proxy._isDataManagerUpdate = true;
                    proxy._jsonData = dataSource.dataSource.json;
                }
                else if (!dataSource.dataSource.offline) {
                    proxy._isDataManagerUpdate = true;
                    proxy._jsonData = proxy._retrivedData;
                }
            }
        },

        _initDatasource: function () {
            var proxy = this,
                model = proxy.model, retriveData;
            if (!model.isFromGantt) {
                if (this.dataSource() instanceof ej.DataManager) {

                    if (ej.isNullOrUndefined(proxy._waitingPopupObj))
                        proxy._renderWaitingPopup();
                    else
                        proxy._waitingPopupObj.show();
                    var isRemoteData = this.dataSource().adaptor instanceof ej.UrlAdaptor || this.dataSource().adaptor instanceof ej.WebMethodAdaptor ||
                                            this.dataSource().adaptor instanceof ej.WebApiAdaptor || this.dataSource().adaptor instanceof ej.ODataAdaptor,
                        isStaticData = (ej.pvt.filterQueries(model.query.queries, "onTake").length == 0 && ej.pvt.filterQueries(model.query.queries, "onPage").length == 0
                                        && ej.pvt.filterQueries(model.query.queries, "onWhere").length == 0) ? false : true;
                    proxy._isRemoteData = isRemoteData;
                    if (isRemoteData && !this.dataSource().dataSource.offline && !isStaticData && model.enableLoadOnDemand) {
                        proxy._isLoadonDemand = true;
                        proxy._getAndRenderRecordsonDemand();
                    }
                    else {
                        proxy._isLoadonDemand = false;
                        model.query.requiresCount();
                        var queryPromise = this.dataSource().executeQuery(model.query);
                        queryPromise.done(ej.proxy(function (e) { proxy._renderQueriedRecords(e, proxy); }))
                                    .fail(ej.proxy(function (e) { proxy._triggerFailureEvent(e, proxy); }));                           
                    }
                }
                else if (this.dataSource()) {
                    proxy._retrivedData = proxy.dataSource();
                    proxy._renderQueriedRecords(null, proxy, proxy._retrivedData);                    
                }
                else {
                    proxy._checkDataBinding();
                }
            } else {
                proxy._isLoadonDemand = model.isLoadonDemand;
                proxy._checkDataBinding();
            }            
        },
        _intersectionObjects: function (flatCollection, rootCollection) {
            var result = [];
            while (flatCollection.length > 0 && rootCollection.length > 0) {
                var index = rootCollection.indexOf(flatCollection[0]);
                if ( index == -1) {
                    flatCollection.shift();
                }
                else {
                    result.push(flatCollection.shift());
                    rootCollection.splice(index, 1);
                }
            }

            return result;
        },

        _getAndRenderRecordsonDemand:function(){
            var proxy = this, model = proxy.model;
            model.query.requiresCount();
            model.pageSettings.pageSizeMode = ej.TreeGrid.PageSizeMode.Root;
            if (model.parentIdMapping) {
                proxy._queryAndRenderParentRecords();
            }
            else {
                var newQuery = proxy._updateQueryForLoadonDemand();
                var queryPromise = proxy.dataSource().executeQuery(newQuery);
                queryPromise.done(ej.proxy(function (e) {
                    //proxy._fieldNamesToQuery = existSelectQuery[0].e.fieldNames;
                    proxy._pageRecordCountOnDemand = e.count;
                    proxy._renderQueriedRecords(e, proxy);
                    proxy._setScrollTop();
                    proxy.updateHeight();

                })).fail(ej.proxy(function (e) { proxy._triggerFailureEvent(e, proxy); }));                   
            }
            proxy._allParentItems = $.extend([], model.parentRecords);
        },

        /*Create waiting popup for load on demand request*/
        _renderWaitingPopup: function () {
            var proxy=this;
            $(this.element).ejWaitingPopup({ showOnInit: true, });
            this._waitingPopupObj = $(this.element).data("ejWaitingPopup")
        },

        _renderQueriedRecords: function (e, proxy, retrivedItems) {
            var model = proxy.model;
            if (retrivedItems)
                proxy._retrivedData = retrivedItems;
            else {
                proxy._retrivedData = e.result;
                if (proxy._fieldNamesToQuery.length == 0 && proxy._retrivedData.length > 0) {
                    for (var field in e.result[0])
                        proxy._fieldNamesToQuery.push(field);
                }
            }
            if (model.flatRecords.length == 0) {
                if (model.allowPaging && model.parentIdMapping) {
                    if (!proxy._isRemoteData) {
                        if (ej.isNullOrUndefined(model.pageSettings.totalRecordsCount))
                            model.pageSettings.totalRecordsCount = e && !ej.isNullOrUndefined(e.count) ? e.count : null;
                        else
                            proxy._retrivedData = proxy._retrivedData.splice(model.pageSettings.totalRecordsCount);
                    }
                    else
                        model.pageSettings.totalRecordsCount = e.count;
                }
                
                proxy._createQueriedRecords(proxy._retrivedData);
            }
            if (!proxy._isForDataSourceRefresh) {
                if (model.showSummaryRow && model.printingEnabled)
                    proxy._parentRecords = proxy._getParentRecords(model.updatedRecords, proxy)
                proxy._checkDataBinding();
            }
            else {
                var args = {};
                args["requestType"] = ej.TreeGrid.Actions.RefreshDataSource;
                proxy._ensureDataSource(args);
                if (!model.isFromGantt) {
                    proxy._checkDataManagerUpdate();
                }
                if (proxy.selectedRowIndex() !== -1 && proxy.selectedRowIndex() < proxy.model.currentViewData.length) {
                    proxy.selectRows(proxy.selectedRowIndex());
                }
                if (proxy.getScrollElement().hasClass("e-scroller"))
                    proxy.getScrollElement().ejScroller("refresh");
                if (model.showSummaryRow || model.showTotalSummary) {
                    proxy._summaryRow();
                }
                proxy._createdAt = null;
                proxy._isForDataSourceRefresh = false;
            }
            if (e)
                proxy._waitingPopupObj.hide();
        },

        _createQueriedRecords: function (retrivedItems) {
            var proxy = this, model = proxy.model;
            if (model.idMapping && model.parentIdMapping && retrivedItems.length > 0) {
                //cloning the datasource                    
                var data = []; proxy._taskIds = [], proxy._lastInsertedId = -1;
                for (var i = 0; i < retrivedItems.length; i++) {
                    var tempData = retrivedItems[i];
                    data.push($.extend(true, {}, tempData));
                    if (!ej.isNullOrUndefined(tempData[model.idMapping])) {
                        proxy._updateLastInsertedId(tempData[model.idMapping]);
                        proxy._taskIds.push(tempData[model.idMapping]);
                    }
                }
                proxy._reconstructDatasource(data);
                proxy._createRecords(proxy.secondaryDatasource);
            }
            else
                retrivedItems && proxy._createRecords(retrivedItems);
        },

        _queryAndRenderParentRecords: function () {
            var proxy = this, model = proxy.model;
            if (model.hasChildMapping)
                proxy._renderRootRecords(proxy);            
            else {
                if (proxy._assignedIdCollections.length == 0 && proxy._assignedParendIDs.length == 0) {
                var queryParentIds = ej.Query(), parentIdQueryPromise,
                    existSelectQuery = ej.pvt.filterQueries(model.query.queries, "onSelect");

                queryParentIds.queries = $.extend([], model.query.queries);
                queryParentIds.requiresCount();
                if (existSelectQuery.length > 0)
                    queryParentIds.queries.splice($.inArray(existSelectQuery[0], queryParentIds.queries), 1);
                queryParentIds.select(model.parentIdMapping, model.idMapping);

                parentIdQueryPromise = this.dataSource().executeQuery(queryParentIds);
                parentIdQueryPromise.done(ej.proxy(function (e) {
                    e.result.map(function (res) {
                        proxy._assignedIdCollections.push(res[model.idMapping]);
                        if (res[model.parentIdMapping] && proxy._assignedParendIDs.indexOf(res[model.parentIdMapping]) == -1)
                            proxy._assignedParendIDs.push(res[model.parentIdMapping]);
                    });

                    proxy._totalItemCounts = e.result.length;
                        proxy._renderRootRecords(proxy);

                    })).fail(ej.proxy(function (e) { proxy._triggerFailureEvent(e, proxy); }));
                }
                else
                    proxy._renderRootRecords(proxy);
            }
        },

        _renderRootRecords:function(proxy){
                    var updatedQuery = proxy._updateQueryForLoadonDemand();
                    var queryPromise = proxy.dataSource().executeQuery(updatedQuery);

                    queryPromise.done(ej.proxy(function (e) { proxy._renderZerothLevelParentItems(e); }))
                                .fail(ej.proxy(function (e) { proxy._triggerFailureEvent(e, proxy); }));
        },
        _renderExpandedChildRecords: function (expcolargs) {
            var proxy = this, model = proxy.model;          
            proxy._ensureDataSource(expcolargs);
            if (model.showSummaryRow) {
                proxy._parentRecords = proxy._allParentItems;
                proxy._createSummaryRow();
            }
            if (model.enableAltRow) {
                proxy.updateAltRow();
            }
            proxy.sendDataRenderingRequest(expcolargs);
            proxy._setScrollTop();
            proxy.updateHeight();
            proxy._waitingPopupObj.hide();            
        },
        _triggerFailureEvent: function (e, proxy) {
            proxy._waitingPopupObj.hide();
            var args = { error: e.error };
            proxy._trigger("actionFailure", args);
        },
        _renderZerothLevelParentItems: function (e) {
            var proxy = this, model = proxy.model;           
            proxy._pageRecordCountOnDemand = e.count;
            proxy._renderQueriedRecords(e, proxy);
            proxy._allParentItems = $.extend([], model.parentRecords);
            proxy._isForDataSourceRefresh = false;
            proxy._maxRowIndex = proxy._totalItemCounts;
        },
        _updateQueryForLoadonDemand: function () {
            var proxy = this, model = proxy.model,
                updateQuery = new ej.Query(),
                condition = ej.Predicate(model.parentIdMapping, ej.FilterOperators.equal, null);

            updateQuery.queries = $.extend([], model.query.queries);
            if (model.parentIdMapping)
                updateQuery.where(condition);
            if (model.allowPaging)
                updateQuery.page(proxy._currentPage(), proxy.model.pageSettings.pageSize);
            updateQuery.requiresCount();
            proxy._queryForLoadonDemand = updateQuery;
            //model.query = updateQuery;
            return updateQuery;
        },
                    
        _reconstructDatasource: function (datasource) {
            var proxy = this, model = proxy.model, data, filter;                      

            filter = ej.Predicate(model.parentIdMapping, ej.FilterOperators.notEqual, "null");
            data = ej.DataManager(datasource).executeLocal(ej.Query().where(filter).group(model.parentIdMapping));            
            var tempParent = [];

            if (!model.childMapping) model.childMapping = "Children";

            for (var i = 0; i < data.length; i++) {
                if (!ej.isNullOrUndefined(data[i].key)) {

                    var index = proxy._taskIds.indexOf(data[i].key);
                    if (index > -1) {
                        if (model.hierarchyRowIndexMapping && data[i].items.length > 1)
                            datasource[index][model.childMapping] = ej.DataManager(data[i].items).executeLocal(ej.Query().sortBy(model.hierarchyRowIndexMapping, ej.sortOrder.Ascending, false));
                        else
                            datasource[index][model.childMapping] = data[i].items;
                        continue;
                    }
                }
                tempParent.push.apply(tempParent, data[i].items);
            }
            proxy.secondaryDatasource = proxy._intersectionObjects(datasource, tempParent);
            if (model.hierarchyRowIndexMapping && proxy.secondaryDatasource.length > 1)
                proxy.secondaryDatasource = ej.DataManager(proxy.secondaryDatasource).executeLocal(ej.Query().sortBy(model.hierarchyRowIndexMapping, ej.sortOrder.Ascending, false));
        },        
               
        //Window resize event
        _windowResize: function (e)
        {
            if (!this.element.is(":visible")) {
                return;
            }
            var proxy = this,
               model = this.model;
            $("#" + proxy._id).addClass('e-treegrid-overflow');

            var treeGridHeader = proxy._$gridHeaderContent,
               totalSummaryRowContent = proxy._$totalSummaryRowContainer,
               treeGridContent,
               treegridToolbar = $("#" + proxy._id + "_toolbarItems"),
               sizeSettingsHeight = model.sizeSettings.height,
               sizeSettingsWidth = model.sizeSettings.width,
               elementStyleWidth = proxy.element[0].style.width,
               elementStyleHeight = proxy.element[0].style.height,
               width, height;

            proxy._clearContextMenu();
            if (!proxy.model.isResponsive && !ej.isTouchDevice()) {
                    proxy._clearColumnMenu();
                    proxy._clearFilterMenu();
            }
            else {
                if (!ej.isTouchDevice()) {
                    proxy._clearColumnMenu('resize');
                }
            }
            if (model.isResponsive && proxy._isExcelFilter)
            {
                if (proxy._checkBoxDiv)
                    proxy._checkBoxDiv.ejScroller({ width: "100%" }).ejScroller('refresh');
                if (proxy._customFilter) {
                    proxy._customFilter.ejDialog("width", "99.9%");
                   // proxy._customFilter.css("width", "95.5%");
                }
            }
            if (this._frozenColumnsLength > 0)
                treeGridContent = this.element.find("#e-movablecontainer" + this._id);
            else
                treeGridContent = $("#" + proxy._id + "e-gridcontent");


            $("#" + this._id + "detailscellwrapper").length > 0 && this._removeDetailsRow();

            //Width calculation
            if ((sizeSettingsWidth && (typeof(sizeSettingsWidth) != "number" && sizeSettingsWidth.indexOf("%") != -1)) || elementStyleWidth.indexOf("%") != -1) {
                var ganttWidth = sizeSettingsWidth ? sizeSettingsWidth : elementStyleWidth;
                var containerWidth = $(proxy.element).parent().width() ? $(proxy.element).parent().width() : $(proxy.element).width();
                var tempWidth = (containerWidth / 100) * parseInt(ganttWidth);
                width = tempWidth;
            }
            else
                width = $(proxy.element).width();
            proxy._gridWidth = Math.round(width);
            if (proxy._frozenColumnsLength > 0) {
                var totalWidth = 0;
                for (var i = 0; i < proxy._frozenColumnsLength; i++) {
                    if (model.columns[i].visible == true) {
                        totalWidth += proxy.columnsWidthCollection[i];
                    }
                }
                if (width < totalWidth)
                    proxy._gridWidth = width = totalWidth;
            }
            //height calculation
            if ((sizeSettingsHeight && (typeof (sizeSettingsHeight) != "number" && sizeSettingsHeight.indexOf("%") != -1)) || elementStyleHeight.indexOf("%") != -1) {
                var ganttHeight = sizeSettingsHeight ? sizeSettingsHeight : elementStyleHeight;
                var containerHeight = $(proxy.element).parent().height() ? $(proxy.element).parent().height() : $(proxy.element).height();
                var tempHeight;
                if ($(proxy.element).parent().height())
                    tempHeight = (containerHeight / 100) * parseInt(ganttHeight);
                else
                    tempHeight = $(proxy.element).height();
                height = tempHeight;
            }
            else
                height = $(proxy.element).height();

            //excludes border 
            height = height - proxy._totalBorderHeight; 
            
            if (proxy._$totalSummaryRowContainer)
                height -= proxy._$totalSummaryRowContainer.outerHeight();
            width = Math.round(width - proxy._totalBorderWidth);
            var treeGridWidth, headerHeight, headerWidth, top, left,
                toolbarHeight = treegridToolbar.length > 0 ? treegridToolbar.outerHeight() : 0;
            top = treeGridContent.ejScroller("option", "scrollTop");
            left = treeGridContent.ejScroller("option", "scrollLeft");
            var scrollerWidth = width,
                PagerHgDiff = 0;
            if (model.allowPaging) {
                var pagerHeight = proxy.element.find('.e-pager').outerHeight(),
                    isPagerTemplate = ej.isNullOrUndefined(model.pageSettings.template)? false : true;
                height -= pagerHeight;               
                if (!model.sizeSettings.height) {
                    var totalWidth = 0;
                    if (pagerHeight > proxy._pagerHeight && !isPagerTemplate)
                        PagerHgDiff = pagerHeight - proxy._pagerHeight;
                    for (var i = 0; i < proxy.columnsWidthCollection.length; i++)
                        totalWidth += proxy.columnsWidthCollection[i];
                    if (totalWidth > proxy._gridWidth) {
                        height = height + proxy._scrollBarHeight;
                    }                   
                }
            }
            if (this._frozenColumnsLength > 0) {
                proxy._$gridContent.css("width", width);
                proxy._$gridContent.css("height", height - $(treeGridHeader).outerHeight() - toolbarHeight);
                scrollerWidth = width - this._getFrozenColumnWidth() - 1;//frozen div border width
            }
            proxy.getScrollElement().ejScroller({
                width: scrollerWidth,
                height: height - $(treeGridHeader).outerHeight() - toolbarHeight + PagerHgDiff//for border-width of gridcontent
            });
            proxy._updateScrollCss();
           
            if (model.showTotalSummary && proxy._$totalSummaryRowContainer) {
                if (proxy._$totalSummaryRowContainer.height() >= model.totalSummaryHeight && proxy._$totalSummaryRowContainer.hasClass("e-scroller"))
                    proxy._$totalSummaryRowContainer.ejScroller("refresh");
                else if ($(totalSummaryRowContent).hasClass("e-scrollcss") && proxy.getScrollElement().children(".e-vscrollbar").length > 0 )
                    $(totalSummaryRowContent).width(width - 17);
                else 
                    $(totalSummaryRowContent).width(width);
            }
            if (model.toolbarSettings.showToolbar && treegridToolbar.length > 0) {
                treegridToolbar.width(width);
            }

            proxy.getScrollElement().ejScroller("refresh");

            if ($(treeGridHeader).hasClass("e-scrollcss") && proxy.getScrollElement().children(".e-vscrollbar").length > 0) {
                $(treeGridHeader).width(width - 17);
            }
            else {
                $(treeGridHeader).width(width);
            }

            proxy.updateViewPortHeight();

            if(model.showTotalSummary)
                proxy._viewPortHeight = proxy._viewPortHeight - proxy._$totalSummaryRowContainer.outerHeight();
            proxy.getScrollElement().ejScroller("option", "scrollTop",top);
            proxy.getScrollElement().ejScroller("option", "scrollLeft", left);
            proxy._updateHeaderScrollLeft();
            if (model.enableVirtualization) {
                proxy.cancelRowEditCell(); // To hide or destroy popup menu
                var tempArgs = {},
                    rowHeight = model.rowHeight;
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.processBindings(tempArgs);                
                proxy.sendDataRenderingRequest(tempArgs);
                /* check detail template and its row height */
                if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn)
                {
                    rowHeight += model.detailsRowHeight;
                }
                
                if ((model.currentViewData.length * rowHeight) < proxy._viewPortHeight) {

                    var currentPosition = top - (proxy._viewPortHeight - model.currentViewData.length * rowHeight);
                    if (currentPosition < 0)
                        currentPosition = 0;
                    proxy.getScrollElement().ejScroller("scrollY",currentPosition, true);
                }
            }
            if (model.allowPaging)
                proxy.element.find(".e-pager").css({ "width": proxy._gridWidth - 2 });
            proxy._resizeFilteringElements();
            if (model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns && !model.isFromGantt) {
                this._updateTableWidth(true);
            }
            if (proxy.model.isResponsive && !proxy.model.isFromGantt) {
                proxy._mediaQueryUpdate();
            }
            proxy._updatePagerTextAlignment();
            proxy._updateWrapRowHeights(null, "virtualRefresh", null, "resize");
            if (model.allowPaging && !model.sizeSettings.height) {
                proxy.updateHeight();
            }
            if (model.allowTextWrap || model.headerTextOverflow == "wrap") {
                if (proxy.model.allowTextWrap && proxy.model.enableVirtualization) {
                    proxy.updateHeight();
                }                
            }
            proxy._updateGridContentHeight();
            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                proxy._alignMultiSelectPopup();
            }
            $("#" + proxy._id).removeClass('e-treegrid-overflow');
            if (model.isResponsive && model.enableResize) {
                proxy._validateColumnVisible();
                proxy._hideColumns(proxy._autoHideColumns);
                proxy._showColumns(proxy._autoShowColumns);
            }
        },
		
        _updatePagerTextAlignment: function () {
            var proxy = this;
            if (proxy.model.allowPaging) {
                if (($('#' + proxy._id + 'Pager').find('.e-pagercontainer').outerWidth() + $('#' + proxy._id + 'Pager').find('.e-parentmsgbar').outerWidth() + 10) > $('#' + proxy._id).width()) {
                    var pagecontainer = ($('#' + proxy._id).width() - $('#' + proxy._id + 'Pager').find('.e-pagercontainer').outerWidth()) / 2;
                    var parentmsgbar = ($('#' + proxy._id).width() - $('#' + proxy._id + 'Pager').find('.e-parentmsgbar').width()) / 2;
                    if (pagecontainer > 0) {
                        $('#' + proxy._id + 'Pager').find('.e-pagercontainer').css("margin-left", pagecontainer + 'px');
                    }
                    if (parentmsgbar > 0) {
                        $('#' + proxy._id + 'Pager').find('.e-parentmsgbar').css("margin-right", (parentmsgbar - 6) + 'px');
                    }
                } else {
                    $('#' + proxy._id + 'Pager').find('.e-pagercontainer').css("margin-left", "");
                    $('#' + proxy._id + 'Pager').find('.e-parentmsgbar').css("margin-right", "");
                }
            }
        },

        _alignMultiSelectPopup: function() {
            var proxy = this,
                contentHeightLimit = proxy._$gridContent.offset().top + proxy._$gridContent.outerHeight(),
                contentWidthLimit = proxy._$gridContent.offset().left + proxy._$gridContent.outerWidth();
            if ((proxy._multiSelectPopup.offset().top + proxy._multiSelectPopup.outerHeight() + 10) > contentHeightLimit || proxy._multiSelectPopup.offset().top < proxy._$gridContent.offset().top || (proxy._multiSelectPopup.offset().left + proxy._multiSelectPopup.outerWidth()) > contentWidthLimit || proxy._multiSelectPopup.offset().left < proxy._$gridContent.offset().left) {
                var top = proxy._$gridContent.offset().top + (proxy._$gridContent.outerHeight() / 2),
               left = proxy._$gridContent.offset().left + (proxy._$gridContent.outerWidth() / 2);
                if (proxy._$gridContent.find(".e-hscrollbar").length > 0)
                    top = top - proxy._$gridContent.find(".e-hscrollbar").outerHeight();
                if (proxy._$gridContent.find(".e-vscrollbar").length > 0)
                    left = left - proxy._$gridContent.find(".e-vscrollbar").outerWidth();
            proxy._multiSelectPopup.offset({ top: (top - (this._multiSelectPopup.height() / 2) - 5), left: (left - (this._multiSelectPopup.width() / 2)) });
        }
        },

        _mediaQueryUpdate: function () {
            var proxy = this, length, sliceFilterMenuDivID, filterMenuDivID,
                temp = "",
                controlPosition  = proxy.getOffsetRect($("#" + proxy._id)[0]);

            // Insert dialog-box changes with respect to window resize.

            if ($("#" + proxy._id + "_dialogColumnAdd_wrapper").length > 0 && $("#" + proxy._id + "_dialogColumnAdd_wrapper").css('display') != 'none') {
                if ($(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive) {

                        $("#" + proxy._id + "_dialogColumnAdd").removeClass("e-treegridadaptivedialog");
                        $("#" + proxy._id + "_dialogColumnAdd_wrapper").removeClass("e-treegridadaptivedialog");
                        $("#" + proxy._id + "_dialogColumnAdd_wrapper").find("div.e-dialog-scroller").removeClass("e-treegridadaptivedialog");
                        $("#" + proxy._id + "_dialogColumnAdd").find(".e-dropdowntreegrid").css('width', ($("#" + proxy._id).width() - 80) + 'px');  // 80px ((28px of $("#" + proxy._id + "_dialogColumnAdd"))+(18px of Hscrollbar)+(30px of $("#" + proxy._id + "ColumnAddForm div"))) get minused in width to place the dropdown dialog treegrid div in absolute position.
                        $("#" + proxy._id + "_dialogColumnAdd").ejDialog({ width: $("#" + proxy._id).width() + "px", height: $("#" + proxy._id).height() + "px", minHeight: "150px", minWidth: "150px", showOnInit: false, enableResize: false, allowDraggable: false, enableModal: false, isResponsive: false });

                        $("#" + proxy._id + "_dialogColumnAdd_wrapper").css({ 'left': controlPosition.left + "px", 'top': controlPosition.top + "px" });
                        $("#" + proxy._id + "_dialogColumnAdd_wrapper").removeClass("e-shadow");
                        $("#" + proxy._id + "_dialogColumnAdd").removeClass("e-shadow");

                } else {
                    $("#" + proxy._id + "_dialogColumnAdd").ejDialog({ width: "auto", height: "auto", minHeight: 0, minWidth: 0, showOnInit: false, enableResize: false, allowDraggable: true, enableModal: true, isResponsive: true });
                    $("#" + proxy._id + "_dialogColumnAdd").addClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").addClass("e-treegridadaptivedialog e-treegrid-dialog");
                    $("#" + proxy._id + "_dialogColumnAdd").find(".e-dropdowntreegrid").css('width', '438px');  // fixed width of dialog dropdown treegrid div.
                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").find("div.e-dialog-scroller").removeClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").addClass("e-shadow");
                }
            }

            // Column rename dialog-box changes with respect to window resize.

            if ($("#" + proxy._id + "ColumnRenameDialog_wrapper").length > 0 && $("#" + proxy._id + "ColumnRenameDialog_wrapper").css('display') != 'none') {

                if ($(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive) {

                        $("#" + proxy._id + "ColumnRenameDialog").removeClass("e-treegridadaptivedialog");
                        proxy._columnRenameDialog.ejDialog({ width: $("#" + proxy._id).width() + "px", height: $("#" + proxy._id).height() + "px", position: { X: controlPosition.left, Y: controlPosition.top }, minHeight: "50px", minWidth: "50px", showOnInit: false, allowDraggable: false, enableResize: false, enableModal: false, isResponsive: false });
                        $("#" + proxy._id + "ColumnRenameDialog_wrapper").css({ 'left': controlPosition.left + "px", 'top': controlPosition.top + "px" });
                        $("#" + proxy._id + "ColumnRenameDialog_wrapper").removeClass("e-shadow");
                        $("#" + proxy._id + "ColumnRenameDialog").removeClass("e-shadow");
                        $("#" + proxy._id + "ColumnRenameDialog_wrapper").removeClass("e-treegridadaptivedialog");

                } else {
                    proxy._columnRenameDialog.ejDialog({ width: "auto", height: "auto", minHeight: 0, minWidth: 0, showOnInit: false, enableResize: false, allowDraggable: true, enableModal: true, isResponsive: true });
                    $("#" + proxy._id + "ColumnRenameDialog").addClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "ColumnRenameDialog_wrapper").addClass("e-treegridadaptivedialog e-treegrid-dialog");
                }
            }

            // Filter menu dialog-box changes with respect to window resize.

            if (proxy._renderedSubMenuType) { // proxy._renderedSubMenuType shows the rendered type of sub menu (filter menu / column chooser) dialog of column menu.
                temp = "";
                if (proxy._renderedSubMenuType.indexOf("Desktop") == -1) {
                    temp = proxy._renderedSubMenuType;
                } else {
                    var index = proxy._renderedSubMenuType.indexOf("Desktop");
                    temp = proxy._renderedSubMenuType.slice(0, index);
                }
                if ($(temp + "_wrapper").length > 0 && $(temp + "_wrapper").css('display') != 'none') {
                    if ($(window).width() <= proxy._responsiveMinWidth && proxy._renderedSubMenuType.indexOf("Desktop") == -1 && proxy.model.isResponsive) {

                        var dialogInnerDivHeight = ($("#" + proxy._id).height() - 50) + "px"; // (48px - Dialog box header height) + (2px - excess border of parent div).
                            $(proxy._renderedSubMenuType + "_wrapper").find("div.e-dialog-scroller").addClass("e-treegridadaptivewidth");
                            $(proxy._renderedSubMenuType).ejDialog({ width: $("#" + proxy._id).width() + "px", height: $("#" + proxy._id).height() + "px", position: { X: controlPosition.left + "px", Y: controlPosition.top + "px" }, minHeight: "50px", minWidth: "50px", showOnInit: false, enableResize: false, enableModal: false, isResponsive: false });
                            if ($(proxy._renderedSubMenuType).height() > $(proxy._renderedSubMenuType + "_wrapper").height() - 48) { // 48px - Dialog box header height
                                $(proxy._renderedSubMenuType + "_wrapper").find("div.e-dialog-scroller").css({ 'height': ($("#" + proxy._id).height() - 48) + "px", 'width': 'auto', 'padding': '0px' }); // 48px - Dialog box header height
                                $(proxy._renderedSubMenuType).ejDialog("refresh"); // 'refresh' event get called inorder to update dialog scroll bar
                            } else {
                                $(proxy._renderedSubMenuType+"_wrapper").find("div.e-dialog-scroller").css({ 'height': dialogInnerDivHeight, 'width': 'auto' });
                              //$(proxy._renderedSubMenuType).ejDialog("refresh"); // 'refresh' event get called inorder to update dialog scroll bar
                            }

                    } else {
                        if (ej.isTouchDevice()) {
                            if (((proxy._renderedSubMenuType.indexOf("Desktop") == -1) && ($(window).width() > proxy._responsiveMinWidth)) || ((proxy._renderedSubMenuType.indexOf("Desktop") != -1) && ($(window).width() <= proxy._responsiveMinWidth))) {
                                proxy._clearColumnMenu();
                                proxy._clearFilterMenu();
                            }
                        } else {
                            proxy._clearFilterMenu();
                        }
                    }
                } else if ((ej.isTouchDevice() && !proxy._renderedSubMenuType) || (ej.isTouchDevice() && $(temp + "_wrapper").length > 0 && $(temp + "_wrapper").css('display') == 'none')) {
                    proxy._clearColumnMenu();
                }
            } else if ((ej.isTouchDevice() && !proxy._renderedSubMenuType) || (ej.isTouchDevice() && $(temp + "_wrapper").length > 0 && $(temp + "_wrapper").css('display') == 'none')) {
                proxy._clearColumnMenu();
            }
            if (proxy._isExcelFilter) {
                var isResponsive = $(window).width() > proxy._responsiveMinWidth;
                if (!proxy._mediaStatus || (proxy._mediaStatus && isResponsive)) {
                    proxy._excelFilter.closeXFDialog();
                    proxy._mediaStatus = false;
                    //proxy._clearFilterMenu();
                    if (proxy._customFilter)
                    {
                        var customFilterId = $(proxy._customFilter).attr("id");
                        $("#"+customFilterId).css("display", "none");
                        $("#" + customFilterId + "_CustomFDlg_wrapper").css("display", "none");
                    }

                }
            }

            // Column chooser dialog-box changes with respect to window resize.

            if ($("#" + proxy._id + "ccDiv_wrapper").length > 0 && $("#" + proxy._id + "ccDiv_wrapper").css('display') != 'none') {
                if ($(window).width() < proxy._responsiveMinWidth && proxy._renderedSubMenuType.indexOf("AdaptiveColChooser") == 0 && proxy.model.isResponsive) {

                    var dialogInnerDivHeight = ($("#" + proxy._id).height() - 66) + "px"; //(48px - Dialog box header height) + (16px - $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller") of padding) + (2px - excess border of parent div) 
                        $("#" + proxy._id + "ccDiv").ejDialog({ width: $("#" + proxy._id).width() + "px", height: $("#" + proxy._id).height() + "px", minHeight: "150px", minWidth: "150px", showOnInit: false, enableResize: false, enableModal: false, isResponsive: false });
                        $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").removeClass("e-treegridadaptivedialog");
                        $("#" + proxy._id + "ccDiv").css({ 'width': '100%', 'height': 'auto' });
                        if ($("#" + proxy._id + "ccDiv").height() > $("#" + proxy._id + "ccDiv_wrapper").height() - 48) { //(48px - Dialog box header height)
                            $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").css({ 'height': ($("#" + proxy._id).height() - 48) + "px", 'width': 'auto', 'padding': '0px' }); //(48px - Dialog box header height)
                            $("#" + proxy._id + "ccDiv").removeClass("e-columnSelector");
                            $("#" + proxy._id + "ccDiv").ejDialog("refresh");
                        } else {
                            $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").css({ 'height': dialogInnerDivHeight, 'width': 'auto', 'padding': '8px 16px' });
                            $("#" + proxy._id + "ccDiv").css({ 'padding': '0px' });
                        }
                        $("#" + proxy._id + "ccDiv_wrapper").removeClass("e-shadow");
                        $("#" + proxy._id + "ccDiv").removeClass("e-shadow");

                } else {
                    proxy._clearColumnMenu();
                }
            }

            if ($("#" + proxy._id + "_dialogEdit").length > 0 && $("#" + proxy._id + "_dialogEdit").css('display') != 'none') {
                if ($(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive) {
                    $("#" + proxy._id + "_dialogEdit").ejDialog({ width: $("#" + proxy._id).width() + "px", height: $("#" + proxy._id).height() + "px", minHeight: "150px", minWidth: "150px", showOnInit: false, enableResize: false, allowDraggable: false, enableModal: false, isResponsive: false });
                    $("#" + proxy._id + "_dialogEdit_wrapper").css({ 'left': controlPosition.left + "px", 'top': controlPosition.top + "px" });
                    $("#" + proxy._id + "_dialogEdit_wrapper").removeClass("e-shadow").removeClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "_dialogEdit").removeClass("e-shadow");
                } else {
                    $("#" + proxy._id + "_dialogEdit").ejDialog({ width: "auto", height: "auto", minHeight: "150px", minWidth: "150px", showOnInit: false, enableResize: false, allowDraggable: true, enableModal: true, isResponsive: true });
                    $("#" + proxy._id + "_dialogEdit_wrapper").addClass("e-shadow").addClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "_dialogEdit").addClass("e-shadow");
                }
            }

            if (proxy._renderedSubMenuType && (proxy._renderedSubMenuType.indexOf("AdaptiveColChooser") == 0 || proxy._renderedSubMenuType.indexOf("ColumnDesktop") == 0)) {
                proxy._clearColumnMenu();
            }
        },

        updateResponsiveMinWidth: function (width) {
            var proxy = this;
            if (parseInt(width) < 150)
                width = 150;

            proxy._responsiveMinWidth = parseInt(width);

            proxy._mediaQueryUpdate();

        },

        //PROCESSBINDING
        processBindings: function (args) {

            var proxy = this,
                model = proxy.model;

            model.query = new ej.Query();

            if (!proxy._isEmptyRow && !proxy._moveRow && proxy._trigger("actionBegin", args)) {
                if (args.requestType == "filtering") 
                    $("#" + proxy._id + "_" + args.currentFilteringColumn + "_filterbarcell").val("");               
                return true;
            }

            proxy._ensureDataSource(args);

        },
        //updated the treegrid record collection after record update in Gantt
        setUpdatedRecords: function (flatRecords, updatedRecords, ids, parentRecords, dataSource) {
            var model = this.model;
            model.flatRecords = flatRecords;
            model.updatedRecords = updatedRecords;
            model.ids = ids;
            model.parentRecords = parentRecords;
            this.dataSource(dataSource);
        },
        //In virtualization get expaned records from total record collection only
        getExpandedRecords: function (records) {
            var proxy = this;
            var resultRecord = records.filter(function (record) {
                return proxy.getExpandStatus(record) == true;
            });
            return resultRecord;
        },

        //update the expandStatus of all child records in non-virtualization mode
        updateExpandStatus:function(record , expanded)
        {
            var proxy = this,
                childRecords = record.childRecords;
                for (var count = 0; count < childRecords.length; count++) {
                    childRecords[count].isExpanded = expanded;
                    if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                        proxy.updateExpandStatus(childRecords[count], expanded);
                    }
                }
        },
        //Expand collapse, sort, delete, filter search the record collections
        _ensureDataSource: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns,
                filteredColumns = model.filterSettings.filteredColumns,
                isupdatedHeight = false,
                index,
                dataSource = this.dataSource(),
                childIndex, ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");
            proxy._filteredRecords = [];
            proxy._flatFilteredRecords = [];

            model.query.requiresCount();
            proxy._queryManagar = new ej.Query();
            proxy._queryManagar.requiresCount();

            if ($.type(dataSource) !== "array" && !(dataSource instanceof ej.DataManager)) {
                this.dataSource([]);
                dataSource = this.dataSource();
            }

            //Expand Collapse Opearation
            if (args && args.requestType === ej.TreeGrid.Actions.ExpandCollapse) {

                isupdatedHeight = true;
                proxy._filteredRecords = [];
                args.data.expanded = args.expanded;               
                if (!ej.isNullOrUndefined(args.data.item[model.expandStateMapping])) {
                    args.data[model.expandStateMapping] = args.expanded;
                    args.data.item[model.expandStateMapping] = args.expanded;
                }
                model.updatedRecords = [];
                proxy._flatFilteredRecords = model.flatRecords;
                proxy._filteredRecords = proxy._flatFilteredRecords.filter(function (record) {
                    return proxy.getExpandStatus(record) == true;
                });
                model.updatedRecords = proxy._filteredRecords;

                //condition for refreshing last record only
                if (args.isFromExpandCollapse && !args.isLastRefresh)
                    return;
            }

            // Condition for filtering columns.
            if ((args && args.requestType == "filtering") || (model.allowFiltering || model.isFromGantt) && filteredColumns.length) {
                //Removed the summary rows before filtering the datasource.
                if (model.showSummaryRow) {
                    model.updatedRecords = proxy._spliceSummaryRows(model.updatedRecords);
                    model.flatRecords = proxy._spliceSummaryRows(model.flatRecords);
                }
                if (((model.allowFiltering || model.isFromGantt) && filteredColumns.length)) {
                    var predicate, firstFilterCondition = filteredColumns[0];
                    // Condition to collect the details of filtering column.
                    if (proxy._isExcelFilter && !(firstFilterCondition instanceof ej.Predicate)) {
                        predicate = proxy._getExcelPredicate(args);
                    }
                    else {
                        if (firstFilterCondition.value == proxy._dropDownListBlanksText) {
                            predicate = ej.Predicate(firstFilterCondition.field, "equal", null, true);
                            predicate = predicate["or"](firstFilterCondition.field, "equal", "", true);
                        }
                        else if (firstFilterCondition.value == proxy._dropDownListClearText)
                            predicate = ej.Predicate(firstFilterCondition.field, "equal", "", true);
                        else if (!firstFilterCondition.isComplex)
                            predicate = ej.Predicate(firstFilterCondition.field, firstFilterCondition.operator, firstFilterCondition.value, !firstFilterCondition.matchcase);
                        else
                            predicate = firstFilterCondition;
                    if (ej.isNullOrUndefined(args))
                        proxy._storePreviousFilteredValue(firstFilterCondition.field, firstFilterCondition.value, firstFilterCondition.operator);
                    for (var i = 1; i < filteredColumns.length; i++) {
                        //Loop to gather all the column condition as array.
                        if (!filteredColumns[i].isComplex)
                            predicate = predicate[filteredColumns[i].predicate](filteredColumns[i].field, filteredColumns[i].operator, filteredColumns[i].value, !filteredColumns[i].matchcase);
                        else
                            predicate = predicate["and"](filteredColumns[i]);
                        if (ej.isNullOrUndefined(args))
                            proxy._storePreviousFilteredValue(filteredColumns[i].field, filteredColumns[i].value, filteredColumns[i].operator);
                    }
                    }
                    proxy._queryManagar.where(predicate);
                    proxy._filteredRecords = model.flatRecords.slice();
                    var dataManager = new ej.DataManager(proxy._filteredRecords);
                    proxy._tempfilteredrecords = dataManager.executeLocal(proxy._queryManagar).result;
                    proxy._updateFilteredRecords(proxy._tempfilteredrecords,false);
                    model.updatedRecords = (proxy._isHierarchyFiltering && model.enableVirtualization) ? proxy._updateFilterParent : proxy._filteredRecords;
                    if (model.allowPaging && model.pageSettings.pageSizeMode === ej.TreeGrid.PageSizeMode.Root)
                        model.parentRecords = proxy._getParentRecords(model.updatedRecords, proxy, true);
                    if (model.allowSorting && model.sortSettings.sortedColumns.length > 0) {
                        proxy._sortingRecords();
                    }
                } else {
                    model.updatedRecords = model.enableVirtualization ? proxy.getExpandedRecords(model.flatRecords) : model.flatRecords.slice();
                    proxy._updateHasFilteredChildRecordsStatus(model.updatedRecords);
                    if (proxy._isHierarchyFiltering) 
                        proxy._clearFilterLevel(proxy.model.flatRecords)
                    if (model.allowPaging && model.pageSettings.pageSizeMode === ej.TreeGrid.PageSizeMode.Root)
                        model.parentRecords = proxy._getParentRecords(model.updatedRecords, proxy, true);
                    if (proxy._isExcelFilter) {
                        proxy._getExcelPredicate(args);
                    }
                }
                if (!model.allowSearching || proxy._searchString.length == 0)
                    proxy._updatedFilteredRecords(args);
                isupdatedHeight = true;
                //// Updating checkbox state of parent records when enableHierarchySelection is true
            }

            if ((model.allowSearching && proxy._searchString.length) || (args && args.requestType==="searching")) {

                if (model.showSummaryRow) {
                    model.updatedRecords = proxy._spliceSummaryRows(model.updatedRecords);
                    model.flatRecords = proxy._spliceSummaryRows(model.flatRecords);
                }
                var operator = proxy._searchString.length > 0 ? model.searchSettings.operator : "contains";
                proxy._queryManagar.search(proxy._searchString, proxy.getColumnFieldNames(), operator, true);
                if (model.filterSettings.filterHierarchyMode == "parent" || proxy._searchString.length > 0 || model.filterSettings.filteredColumns.length == 0) {
                    var isSearching = true;
                    if (model.isFromGantt) {
                        proxy._filteredRecords = model.flatRecords.filter(function (record) {
                            return record.hasChildRecords === false;
                        });
                    } else {
                        proxy._filteredRecords = model.flatRecords.slice();
                    }
                }
                else
                    isSearching = false;
                if (model.isFromGantt) {
                    ganttObject.model.searchSettings = {
                        fields: proxy._getColumnMappingNames(),
                        key: proxy._queryManagar.queries[0].e.searchKey,
                        operator: "contains",
                        ignoreCase: true
                    };  
                }           
                var dataManager = new ej.DataManager(proxy._filteredRecords);
                proxy._tempfilteredrecords = dataManager.executeLocal(proxy._queryManagar).result;
                proxy._updateFilteredRecords(proxy._tempfilteredrecords, isSearching);
                model.updatedRecords = proxy._filteredRecords;
                if (model.allowPaging && model.pageSettings.pageSizeMode === ej.TreeGrid.PageSizeMode.Root)
                    model.parentRecords = proxy._getParentRecords(model.updatedRecords, proxy, true);
                //Render parent records again while no childs are retrieved from server.
                if (proxy._isLoadonDemand && proxy._searchString.length == 0 && proxy._filteredRecords == 0)
                    model.updatedRecords = model.flatRecords;
                if (model.allowSorting && model.sortSettings.sortedColumns.length > 0) {
                    proxy._sortingRecords();
                }
                proxy._updatedFilteredRecords(args);
                isupdatedHeight = true;               
                proxy.clearSelection();

            }
            else if (model.allowSorting && sortcolumns.length > 0 || (args && args.requestType == "sorting")) {
                
                proxy._sortingRecords();
                if (model.showSummaryRow)
                    proxy._createSummaryRow(args);
                if (!isupdatedHeight)
                {
                    proxy.updateCollapsedRecordCount();
                    proxy.updateHeight();
                    isupdatedHeight = true;
                }
            }

            if (args && (args.requestType === "outdent" || args.requestType === "indent")) {
                args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;
                isupdatedHeight = true;
            }
            if (args && model.allowPaging && args.requestType == "delete") {
                var arg = {};
                if (!proxy._isLoadonDemand) {
                arg.totalRecordsCount = proxy.getExpandedRecords(model.flatRecords).length;                
                proxy.getPager().ejPager("option", arg).ejPager("refreshPager");
                var pagerModel = this.getPager().ejPager("model");
                if (proxy._currentPage() > pagerModel.totalPages)
                    proxy._currentPage(pagerModel.totalPages);
                isupdatedHeight = true;                
            }
            }

            if (!isupdatedHeight) {
                proxy._zerothLevelParentRecords = model.parentRecords.slice();
                model.updatedRecords = model.enableVirtualization ? ((model.enableCollapseAll && !args) ? proxy._zerothLevelParentRecords : proxy.getExpandedRecords(model.flatRecords)) : model.flatRecords.slice();
                proxy.updateCollapsedRecordCount();
                proxy.updateHeight();
            }

            if (!args) {
                args = {};
                if (model.allowPaging)
                    args.requestType = ej.TreeGrid.Actions.Paging;
                else
                    args.requestType = ej.TreeGrid.Actions.Refresh;
                args.inital = true;
            }
            

            proxy._updateCurrentViewData(args.requestType);

            if (!ej.isNullOrUndefined(model.printingEnabled) && model.printingEnabled && model.pageSettings.printMode=="currentPage") {
                var updatedRecords = this.getExpandedRecords(model.updatedRecords),
                    dataManager = new ej.DataManager(updatedRecords),
                    result = dataManager.executeLocal(model.query);
                model.currentViewData = result.result;
            }

            if ((!model.isFromGantt && args.requestType === "save") || args.requestType === "sorting" || args.requestType === "searching"
                || args.requestType === "delete" || args.requestType === "refreshDataSource" || args.requestType === "filtering"
                || args.requestType === "dragAndDrop" || args.requestType === "rowIndexUpdate" || (!args.inital && args.requestType === "paging")) {
				    proxy._isRefreshAddedRecord = false; /* flag new added record is sorted or filtered in treegrid */
                proxy.renderRecords(args);
                proxy._updateWrapRowHeights(null,"refreshAll");
                proxy._setScrollTop(args.requestType);
                proxy._updateFixedTableBorder(args.requestType);
                proxy.getScrollElement().ejScroller("refresh");
                proxy._updateScrollCss();
            }
        },

        _updatedFilteredRecords:function(args){
            var proxy = this, model = proxy.model;
            if (args && (args.requestType == "filtering" || args.requestType == "searching") && proxy._checkboxSelection &&
         model.selectionSettings.enableHierarchySelection) {
                if (args.requestType == "filtering" || args.requestType == "searching") {
                    $.each(model.updatedRecords, function (index, record) {
                        if (record.hasChildRecords)
                            proxy._updateParentSelection(record);

                    });
                    proxy.selectAllRows();
                }
            }

            if (model.showSummaryRow)
                proxy._createSummaryRow(args);
            if (args) {
                if (model.showTotalSummary)
                    proxy._updateTotalSummaryRow(args);
            }
            proxy.updateCollapsedRecordCount();
            proxy.updateHeight();         
            if (model.enableAltRow) {
                proxy.updateAltRow();
            }
        },
         //Remove parent & children records which available in the data collection 
        _removeChildRecords: function(data, record){
            var proxy = this, model = proxy.model,
                childRecords = ej.isNullOrUndefined(record.childRecords) ? [] : record.childRecords;
            var recordIndex = data.indexOf(record);
            if(recordIndex > -1){
                record.checkboxState = "unchecked";
                data.splice(recordIndex, 1);
            }
            for(var i = 0; i < childRecords.length; i++){                
                if(childRecords[i].hasChildRecords)
                    proxy._removeChildRecords(data, childRecords[i]);
                else{
                    recordIndex = data.indexOf(childRecords[i]);
                    if(recordIndex > -1){
                        childRecords[i].checkboxState = "unchecked";
                        data.splice(recordIndex, 1);
                    }
                }
            }
            if(record.parentItem && model.selectionSettings.enableHierarchySelection){                
                proxy._updateParentSelection(record.parentItem);
            }
        },
        //remove child record in data source on self reference
        _removeChildItem: function (record) {
            var currentRecord, proxy = this, idx;
            for (var i = 0; i < record.childRecords.length; i++) {
                currentRecord = record.childRecords[i];
                if (this.dataSource() instanceof ej.DataManager) {
                    if (this.dataSource().dataSource.offline && this.dataSource().dataSource.json) {
                        idx = this.dataSource().dataSource.json.indexOf(currentRecord.item);
                        if(idx!==-1)
                            this.dataSource().dataSource.json.splice(idx, 1);

                        if (currentRecord.hasChildRecords) {
                            proxy._removeChildItem(currentRecord);
                        }
                    }
                    else if (proxy._isDataManagerUpdate) {
                        idx = proxy._jsonData.indexOf(currentRecord.item);
                        if (idx !== -1)
                            proxy._jsonData.splice(idx, 1);
                        if (currentRecord.hasChildRecords) {
                            proxy._removeChildItem(currentRecord);
                        }
                    }
                } else {
                    idx = this.dataSource().indexOf(currentRecord.item);
                    if (idx !== -1)
                        this.dataSource().splice(idx, 1);

                    if (currentRecord.hasChildRecords) {
                        proxy._removeChildItem(currentRecord);
                    }
                }
            }
        },

        //Update checkbox columnName to change dynamically 
        _updateCheckboxColumnName: function(){
            var proxy = this,
                model = proxy.model;

            // Checkbox column is decide by showCheckbox API or default 
            if (proxy._checkboxSelection && !(model.rowTemplateID || model.altRowTemplateID)) {
                var column = model.columns.filter(function (column) {
                    return column['showCheckbox'] == true;
                });
                proxy._addCheckboxColumn = this._getDefaultCheckboxColumn();
                proxy._checkboxColumnName = column.length > 0 ? column[0].isTemplateColumn ? null : column[0].field : null;
                if (ej.isNullOrUndefined(proxy._checkboxColumnName)) {
                    var treeGridIndexColumn = model.columns[model.treeColumnIndex];
                    if (!model.columns[0].nonColumn) {
                        for (var i = 0; i < model.columns.length; i++) {
                            // to remove the existing default checkbox column
                            if (model.columns[i].nonColumn == true) {
                                model.columns.splice(i, 1);
                            }
                        }
                        model.columns.splice(0, 0, proxy._addCheckboxColumn);
                        proxy._checkboxColumnName = proxy._addCheckboxColumn.field;
                    }
                    else
                        proxy._checkboxColumnName = model.columns[0].field;
                    if (treeGridIndexColumn)
                        model.treeColumnIndex = model.columns.indexOf(treeGridIndexColumn);
                }
                if (proxy._frozenColumnsLength > 0 && !model.columns[0]['isFrozen'] &&
                        proxy._checkboxColumnName == proxy._addCheckboxColumn.field) {
                    model.columns[0]['isFrozen'] = true;
                    proxy._frozenColumnsLength += 1;
                    proxy._frozenColumns.splice(0, 0, model.columns[0]);
                    model.scrollSettings.frozenColumns = this._frozenColumnsLength;
                }

            }
        },
        //To save batch edited values
        batchSave: function () {
            var proxy = this,
                model = proxy.model, cArgs = {},
                batchChanges = proxy.batchChanges,
                batchEditedRecords = batchChanges.changed,
                batchDeletedRecords = batchChanges.deleted,
                batchAddedRecords = batchChanges.added;
            cArgs.type = "beforeBatchSave";
            cArgs.batchChanges = batchChanges;
            if (proxy._trigger("beforeBatchSave", cArgs))
                return true;
            //Clear filtered columns
            var filteredColumns = model.filterSettings.filteredColumns;
            if (filteredColumns && filteredColumns.length > 0)
                proxy.clearFilter();
            //To remove batchEditedField collection from modified data
            for (var e = 0; e < batchEditedRecords.length; e++) {
                delete batchEditedRecords[e].batchEditedFields;
            }
            //To remove isBatchAdd property and batchEditedField collection from added data
            for (var a = 0; a < batchAddedRecords.length; a++) {
                delete batchAddedRecords[a].isBatchAdd;
                if (batchAddedRecords[a].batchEditedFields)
                    delete batchAddedRecords[a].batchEditedFields;
            }
            //To remove batch deleted rows
            if (batchDeletedRecords.length)
                proxy.deleteRow();
            if (proxy._batchEditStarts)
                proxy._batchEditStarts = false;
            proxy._cancelSaveTools();
            proxy._checkboxSelection && proxy._resetCheckboxState();
            proxy.processBindings();
            proxy.renderRecords();
            cArgs.requestType = "batchSave";
            cArgs.batchChanges = batchChanges;
            proxy._trigger("actionComplete", cArgs);
            proxy.batchChanges.changed = [];
            proxy.batchChanges.added = []; 
            proxy.batchChanges.deleted = [];
            proxy.batchChanges.updatedHierarchyRowIndexData = [];
            proxy._batchSaveCancel = "";
        },
        //To cancel batch edited values
        batchCancel: function () {
            var proxy = this,
                model = proxy.model,
                batchChanges = proxy.batchChanges,
                batchEditedRecords = batchChanges.changed,
                batchDeletedRecords = batchChanges.deleted,
                batchAddedRecords = batchChanges.added, args = {}, changedRecords = proxy._mergeRecords();
            //Clear filtered columns
            var filteredColumns = model.filterSettings.filteredColumns;
            if (filteredColumns && filteredColumns.length > 0)
                proxy.clearFilter();
            //Remove the batch added rows
            if (batchAddedRecords.length)
                proxy.deleteRow();
            //To reset the original value and remove batchEditedField collection from modified data
            for (var e = 0; e < batchEditedRecords.length; e++) {
                var fieldsData = batchEditedRecords[e].batchEditedFields;
                for (var field in fieldsData) {
                    batchEditedRecords[e][field] = fieldsData[field];
                    batchEditedRecords[e].item[field] = fieldsData[field];
                }
                delete batchEditedRecords[e].batchEditedFields;
            }
            //To remove isBatchDelete, isBatchAdd and batchEditedField collection from deleted data
            for (var a = 0; a < batchDeletedRecords.length; a++) {
                proxy._updateBatchDeletedRecord(batchDeletedRecords[a]);                
            }

            proxy.batchChanges.changed = [];
            proxy.batchChanges.added = [];
            proxy.batchChanges.deleted = [];
            proxy.batchChanges.updatedHierarchyRowIndexData = [];
            proxy._batchSaveCancel = "";
            if (proxy._batchEditStarts)
                proxy._batchEditStarts = false;
            proxy._cancelSaveTools();
            proxy._checkboxSelection && proxy._resetCheckboxState();
            if (model.showSummaryRow) {
                proxy._batchEditCancel = true;
                for (var i = 0; i < changedRecords.length; i++) {
                    args.record = changedRecords[i];
                    proxy._updateSummaryRow(args);
                }
            }
            if (model.showTotalSummary) {
                args.editType = "celledit";
                proxy._updateTotalSummaryRow(args);
            }
            proxy.processBindings();
            proxy.renderRecords();
            proxy._batchEditCancel = false;
        },

        //Reset checkbox state
        _resetCheckboxState: function () {
            var proxy = this, model = proxy.model;
            if (proxy._checkboxSelection) {
                var flatData = model.flatRecords;
                $.map(flatData, function (record) {
                    if (record.checkboxState == "checked" || record.checkboxState == "indeterminate")
                        record.checkboxState = "unchecked";
                });               
                proxy.selectAllRows();
            }
        },

        _updateBatchDeletedRecord: function (record) {
            var proxy = this, model = proxy.model;
            delete record.isBatchDelete;
            if (record.isBatchAdd) {
                //To delete the batch added row that is in batch delete collection
                proxy.batchChanges.added = [];
                proxy.batchChanges.added.push(record);
                proxy.deleteRow();
                return true;
            }
            if (record.batchEditedFields) {
                var fieldsData = record.batchEditedFields;
                for (var field in fieldsData) {
                    record[field] = fieldsData[field];
                }
                delete record.batchEditedFields;
            }
            if (record.hasChildRecords) {
                var childs = record.childRecords;
                for (var c = 0; c < childs.length; c++) {
                    proxy._updateBatchDeletedRecord(childs[c]);
                }
            }
        },
        //CHECK EDITING OPTIONS 
        _processEditing:function(){

            var proxy = this,
                model = proxy.model;

            if (proxy._checkboxSelection)
                proxy._updateCheckboxColumnName();

            if (model.editSettings.allowEditing || model.editSettings.allowAdding) {
                if (model.editSettings.editMode.toLowerCase() === "cellediting" || model.editSettings.editMode.toLowerCase() === "rowediting" || (model.editSettings.editMode.toLowerCase() == "batchediting" && (model.editSettings.batchEditSettings.editMode == "cell" || model.editSettings.batchEditSettings.editMode == "row")) || (model.editSettings.editMode.toLowerCase() === "normal" && model.selectionSettings.selectionMode.toLowerCase() === "cell")) {
                    proxy._addCellEditTemplate();
                } else if ((model.editSettings.editMode.toLowerCase() === "dialogediting" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog")) && (ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) || model.editSettings.dialogEditorTemplateID == "")) {
                    proxy._treeGridAddEditDialog();
                    if ($("#" + this._id + "_dialogEdit").length == 0)
                        this.element.append(this._renderEditDialog());
                } else if ((model.editSettings.editMode.toLowerCase() === "dialogediting" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog")) && (!ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) && model.editSettings.dialogEditorTemplateID != "")) {
                    proxy._treeGridAddEditDialogTemplate();
                    if ($("#" + this._id + "_dialogEdit").length == 0)
                        this.element.append(this._renderEditDialog());
                }
            }

        },

        _toolBarClick: function (sender) {

            //Skip the mouse right click and mouse wheel
            if ((sender.event.which && (sender.event.which == 3 || sender.event.which == 2)) ||
                     (sender.event.button && sender.event.button == 2))
                return false;

            var proxy = this,
               model = proxy.model,
               $gridEle = $(this.itemsContainer).closest(".e-treegrid"),
               gridInstance = $gridEle.ejTreeGrid("instance"),
               gridId = $gridEle.attr('id'),
                toolbarObj = $("#" + gridInstance._id + "_toolbarItems");
            
            if (sender.event == undefined && sender.target.tagName == "INPUT" && sender.currentTarget.id == gridId + "_search")
                return false;

            if (gridInstance.model.dateFormat.toLowerCase().indexOf("hh") != -1)
                $.isFunction($.fn.ejDateTimePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDateTimePicker("hide");
            else
                $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            var currentTarget = sender.currentTarget; var target = sender.target;
            var args = {
                itemName: sender.text,
                currentTarget: currentTarget,
                model: gridInstance.model,
                sender: sender
            };
            if ($gridEle.ejTreeGrid("instance")._trigger("toolbarClick", args))
                return false;
            $(toolbarObj).ejToolbar('deselectItem', sender.currentTarget);
            switch (sender.currentTarget.id) {

                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add", sender);
                    break;

                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit", sender);
                    break;

                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete", sender);
                    break;

                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update", sender);
                    break;

                case gridId + "_cancel":
                    gridInstance._toolbarOperation(gridId + "_cancel", sender);
                    break;

                case gridId + "_expandAll":
                    gridInstance._toolbarOperation(gridId + "_expandAll", sender);
                    break;

                case gridId + "_collapseAll":
                    gridInstance._toolbarOperation(gridId + "_collapseAll", sender);
                    break;
                case gridId + "_pdfExport":
                    gridInstance._toolbarOperation(gridId + "_pdfExport",sender);
                    break;
                case gridId + "_excelExport":
                    gridInstance._toolbarOperation(gridId + "_excelExport", sender);
                    break;
                case gridId + "_print":
                    gridInstance._toolbarOperation(gridId + "_print", sender);
                    break;
                case gridId + "_search":
                    if (gridInstance._isRowEdit)
                        gridInstance.cancelRowEditCell();
                    else if (gridInstance.model.isEdit)
                        gridInstance.cancelEditCell();
                    break;
            }
            return false;
        },
        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejTreeGrid("instance"), gridId = $gridEle.attr('id'), proxy = this;
            var selectedrow = this.selectedItem();
            var parentEle = searchEle.target;
            var index = proxy.model.allowPaging ? this._updatedPageData.indexOf(selectedrow) : this.selectedRowIndex();
            gridObject._exportTo = gridObject["export"];
            switch (operation) {

                case gridId + "_add":
                    if (proxy.model.selectionSettings.selectionMode == "cell") {
                        proxy.model.editSettings.rowPosition = "top";
                    }
                    if (proxy.model.editSettings.allowEditing && (proxy.model.editSettings.editMode == "dialogEditing" || (proxy.model.editSettings.editMode.toLowerCase() == "batchediting" && proxy.model.editSettings.batchEditSettings.editMode == "dialog"))) {
                        proxy._showTreeGridAddEditDialog(-1, "add");
                    } else {
                        proxy._startAdd(parentEle, null);
                    }
                    break;
                    
                case gridId + "_edit":
                    proxy.updateScrollBar();
                    if (proxy.model.editSettings.allowEditing && (proxy.model.editSettings.editMode == "dialogEditing" || (proxy.model.editSettings.editMode.toLowerCase() == "batchediting" && proxy.model.editSettings.batchEditSettings.editMode == "dialog"))) {
                        proxy._showTreeGridAddEditDialog(index);
                    } else {
                        proxy._editRow(index);
                    }
                    break;

                case gridId + "_delete":
                    proxy.deleteRow();
                    if (!proxy.model.editSettings.showDeleteConfirmDialog)
                        proxy._cancelSaveTools();
                    proxy._isRefreshAddedRecord = false;/* flag new added record is sorted or filtered in treegrid */
                    break;

                case gridId + "_update":
                    editMode = proxy.model.editSettings.editMode;
                    if (editMode.toLowerCase() == "cellediting") {
                        proxy._saveEditedElements();
                    }
                    else if (editMode.toLowerCase() == "rowediting")
                        proxy.saveRow();
                    else if (editMode.toLowerCase() == "batchediting") {                        
                        proxy.cancelEdit();
                        proxy._batchSaveCancel = "save";
                        proxy.model.editSettings.showConfirmDialog ? proxy._confirmDialog.find(".e-content").html(proxy._batchSaveConfirmText).end().ejDialog("open") : proxy.batchSave();
                    }
                    proxy._clearContextMenu();
                    this._removeDetailsRow();
                    proxy._focusTreeGridElement();

                    break;

                case gridId + "_cancel":
                    var editMode = proxy.model.editSettings.editMode;
                    if (editMode.toLowerCase() == "cellediting") {
                        if (proxy._isRowEdit)
                            proxy.cancelRowEditCell();
                        else proxy.cancelEditCell();
                    }
                    else if (editMode.toLowerCase() == "rowediting")
                        proxy.cancelRowEditCell();
                    else if (editMode.toLowerCase() === "batchediting") {                        
                        if (proxy._isRowEdit)
                            proxy.cancelRowEditCell();
                        else if (proxy.model.isEdit)
                            proxy.cancelEditCell();
                        proxy._batchSaveCancel = "cancel";
                        proxy.model.editSettings.showConfirmDialog ? proxy._confirmDialog.find(".e-content").html(proxy._cancelEditText).end().ejDialog("open") : proxy.batchCancel();
                    }
                    if (proxy.model.sizeSettings.height == "auto")
                        proxy._updateContentHeight();
                    proxy._cancelSaveTools();
                    proxy._clearContextMenu();
                    this._removeDetailsRow();
                    break;

                case gridId + "_expandAll":
                    proxy.expandAll();
                    break;

                case gridId + "_collapseAll":
                    proxy.collapseAll();
                    break;
                case gridId + "_pdfExport":
                    gridObject._exportTo(gridObject.model.exportToPdfAction, 'pdfExporting', gridObject.model.allowMultipleExporting);
                    break;
                case gridId + "_excelExport":
                    gridObject._exportTo(gridObject.model.exportToExcelAction, 'excelExporting', gridObject.model.allowMultipleExporting);
                    break;
                case gridId + "_print":
                    gridObject.print();
                    break;
                case gridId + "_search":
                    if (proxy._batchEditStarts && !(proxy._confirmDialog).is(":visible")) {
                        proxy._batchSaveLostWarning("Searching");
                        proxy._batchTempDetails.searchTarget = searchEle;
                        proxy._batchTempDetails.searchString = searchEle.value;
                        return true;
                    }
                    if (proxy._searchString != searchEle.value) {
                        proxy.search(searchEle.value);
                        proxy._updateToolbarItems();
                        proxy._isRefreshAddedRecord = false;
                    }
                    break;
            }
            proxy._refreshToolBar();
            return false;
        },

        //To render alert dialog for batch save lost
        _batchSaveLostWarning: function (action) {
            var proxy = this, model = proxy.model;
            if (model.editSettings.editMode.toLowerCase() == "batchediting") {
                if (model.isEdit)
                    proxy.saveCell();
                else if (proxy._isRowEdit)
                    proxy.saveRow();
                proxy._batchSaveCancel = "saveLost" + action;
                model.editSettings.showConfirmDialog ? proxy._confirmDialog.find(".e-content").html(proxy._batchSaveLostChangesText).end().ejDialog("open") : proxy.batchCancel();
            }
        },

       // To render a delete confirmation dialog
        _renderConfirmDialog: function () {
            var proxy = this;
            var $contentDiv = ej.buildTag('div.e-content', proxy.localizedLabels.deleteRecordText),
                $buttons = ej.buildTag('span.e-buttons', '<input type="button" class="e-flat e-btnsub" id=' + proxy._id + "_ConfirmDialogOK" + ' value="' + this.localizedLabels.okButtonText + '" /> '
                + "<input type='button' class='e-flat e-btncan' id=" + proxy._id + '_ConfirmDialogCancel' + " value='" + proxy.localizedLabels.cancelButtonText + "' />", { float: "right", padding: ".5em 0em" });
            proxy._confirmDialog = ej.buildTag('div#' + proxy._id + '_ConfirmDialog', { height: "auto", width: "auto", float: "left" });
            proxy._confirmDialog.append($contentDiv).append($buttons);
            proxy.element.append(proxy._confirmDialog);
            $buttons.find("input").ejButton({
                cssClass: proxy.model.cssClass,
                showRoundedCorner: true,
                size: "mini",
                click: $.proxy(proxy._triggerConfirm, proxy)
            });
            proxy._confirmDialog.ejDialog({ showOnInit: false,enableAnimation: false, showHeader: false, width: "auto", minWidth: 0, minHeight: 0, enableModal: true, enableResize: false });
		   },
		   
		// To confirm whether to delete a row or not   
		   _triggerConfirm: function (args) {
		       var proxy = this, model = proxy.model;
		       if (args !== undefined && args.model.text == proxy.localizedLabels.okButtonText) {
		           if (proxy._batchSaveCancel == "save") {
		               proxy.batchSave();
		           }
		           else if (proxy._batchSaveCancel == "cancel") {
		               proxy.batchCancel();
		           }
		           else if (proxy._batchSaveCancel == "saveLostDrag") {
		               proxy._batchSaveCancel = "cancel";
		               proxy.batchCancel();
		           }
		           else if (proxy._batchSaveCancel == "saveLostColumnMenu") {
		               proxy._batchSaveCancel = "cancel";
		               proxy.batchCancel();
		               proxy._renderColumnMenu(proxy._batchTempDetails.menuTarget);
		           }
		           else if (proxy._batchSaveCancel == "saveLostSearching") {
		               proxy._batchSaveCancel = "cancel";
		               proxy.batchCancel();
		               proxy.search(proxy._batchTempDetails.searchString);
		               proxy._updateToolbarItems();
		               proxy._isRefreshAddedRecord = false;
		           }
		           else if (proxy._batchSaveCancel == "saveLostPaging") {
		               proxy._batchSaveCancel = "cancel";
		               proxy.batchCancel();
		               if (proxy._prevPageNo == proxy._batchTempDetails.currentPage)
		                   return;
		               proxy._isNextPage = true;
		               if (proxy._isRowEdit)
		                   proxy.cancelRowEditCell();
		               else if (proxy.model.isEdit)
		                   proxy.cancelEditCell();
		               proxy.gotoPage(proxy._batchTempDetails.currentPage);
		           }
		           else if (proxy._batchSaveCancel == "saveLostFiltering") {
		               proxy._batchSaveCancel = "cancel";
		               proxy.batchCancel();
		               var batchDetails = proxy._batchTempDetails;
		               if (model.filterSettings.filterType == "filterbar") {
		                   proxy.filterColumn(batchDetails.fieldName, batchDetails.filterOperator, batchDetails.filterValue,
                               batchDetails.predicate, batchDetails.matchcase, batchDetails.actualFilterValue);
		               }
		               else if (model.filterSettings.filterType == "excel" || model.filterSettings.filterType == "menu") {
		                   proxy._onHeaderClick(batchDetails.filterTarget);
		               }
		           }
		           else proxy.deleteRow(null, proxy._isFromContextmenu, proxy._deleteRowIndex);
		           proxy._confirmDialog.ejDialog("close");
		           proxy._cancelSaveTools();
		       }
		       else {
		           proxy._confirmDialog.ejDialog("close");
		           if (proxy._batchSaveCancel == "saveLostFiltering") {
		               if (model.filterSettings.filterType == "filterbar") {
		                   var targetElem = $(proxy._batchTempDetails.filterTarget);
		                   if (targetElem.hasClass("e-numerictextbox"))
		                       targetElem.ejNumericTextbox({ value: null });
		                   else if (targetElem.hasClass("e-dropdownlist"))
		                       targetElem.ejDropDownList({ value: null });
		                   else targetElem.closest("input").val("");
		               }
		           }
		           if (proxy._batchSaveCancel == "saveLostSearching") {
		               if (!proxy._searchString) {
		                   var targetElem = $(proxy._batchTempDetails.searchTarget);
		                   if (targetElem.hasClass("e-ejinputtext"))
		                       targetElem.closest("input").val("");
		               }
		           }
		           if (proxy._batchSaveCancel == "saveLostPaging") {
		               var pagerModel = this.getPager().ejPager("model");
		               pagerModel.currentPage = proxy._prevPageNo;
		               proxy.getPager().ejPager("refreshPager");
		           }
		       }
		           proxy._focusTreeGridElement();
		       proxy._deleteRowIndex = -1;
		       proxy._isFromContextmenu = false;
		       proxy._batchSaveCancel = "";
		   },

        // To print the page
        print: function () {
            var args = {}, proxy = this, model = proxy.model;
            args.requestType = "print";
            proxy._trigger("actionBegin", args);
            var printWin = window.open('', 'print', "height=452,width=1024,tabbar=no"), $printDiv = ej.buildTag("div#" + proxy._id + "Print"),
                cloneTreeGridModel = $.extend({}, proxy.model), cloneTreeGridModelColumn = $.extend(true, [], model.columns),
                columns = cloneTreeGridModelColumn, length = columns.length;
            cloneTreeGridModel.toolbarSettings = $.extend({}, model.toolbarSettings);
            cloneTreeGridModel.toolbarSettings.showToolbar = cloneTreeGridModel.allowPaging = false;
            cloneTreeGridModel.selectedItem = null;
            cloneTreeGridModel.enableVirtualization = false;
            cloneTreeGridModel.query.queries = [];
            cloneTreeGridModel.printingEnabled = true;            
            //To remove the frozen columns while printing.
            if (this._frozenColumnsLength > 0) {
                for (var i = 0; i < length; i++) {
                    if (columns[i].isFrozen)
                        columns[i].isFrozen = false;
                }
                cloneTreeGridModel.columns = columns;
            }
            if (this._commandColumnLength > 0) {
                for (var i = 0; i < length; i++) {
                    if (columns[i].commands) {
                        columns.splice(i, 1);
                        length--;
                        i--;
                    }
                }
                cloneTreeGridModel.columns = columns;
            }
            //To check the page settings to print the current page or all the pages.
            if (model.allowPaging && model.pageSettings.printMode == ej.TreeGrid.PrintMode.CurrentPage) {
                cloneTreeGridModel.query.queries = [{ fn: "onPage", e:{ pageIndex: proxy._currentPage(), pageSize: model.pageSettings.pageSize }}];
                cloneTreeGridModel.dataBound = function () {
                    this._printProcess(printWin);
                }
            }
            else {                
                cloneTreeGridModel.dataBound = function () {
                    this._printProcess(printWin);
                }
            }
            $printDiv.ejTreeGrid(cloneTreeGridModel);            
        },
        //To call the print option for TreeGrid.
        _printProcess: function (printWin) {
            var proxy = this,
                elementClone = this.element,
                args = { requestType: "print", element: elementClone};                      
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar")
                elementClone.find(".e-filterbar").remove();
            $(elementClone).find(".e-gridheader").css({ "width": "auto" });
            this.clearSelection();
            elementClone.show();
            proxy._trigger("beforePrint", args);          
            ej.print(elementClone, printWin);
            proxy._trigger("actionComplete", args);
        },

		 // Perform Tree grid context menu operations.
        contextMenuOperations:function(option){
            var proxy = this,
                 model = proxy.model,
                 index = proxy._contextMenuSelectedIndex,
                 e = proxy._contextMenuEvent,
                 fieldName,
                 item = proxy._contextMenuSelectedItem,
                 editSettings = model.editSettings,
                 editMode = editSettings.editMode.toLowerCase();
            if(!this._isContextMenuItems)
                column = proxy._contextMenuColumnDetails;
            switch (option)
            {
                case "contextMenu_Unfreeze":
                    if ($(this).find(".e-expander").length == 0) {
                        if (!$(this).hasClass("e-disable")) {
                            if (proxy._checkboxSelection && proxy._frozenColumnsLength == 2) {
                                if (model.columns[0].nonColumn && model.columns[0].isFrozen)
                                    model.columns[0]['isFrozen'] = false;
                            }
                            proxy.freezeColumn(column.field, false);
                        }
                    }
                    break;
                case "contextMenu_Freeze":
                    if ($(this).find(".e-expander").length == 0) {
                        if (!$(this).hasClass("e-disable")) {
                            if (proxy._checkboxSelection) {
                                if (model.columns[0].nonColumn && !model.columns[0].isFrozen)
                                    model.columns[0]['isFrozen'] = true;
                            }
                            proxy.freezeColumn(column.field, true);
                        }
                    }
                    break;
                case "contextMenu_FreezePreceding":
                    if ($(this).find(".e-expander").length == 0) {
                        if (!$(this).hasClass("e-disable"))
                            proxy.freezePrecedingColumns(column.field);
                    }
                    break;
                case "contextMenu_SortDescending":
                    if (model.allowMultiSorting) proxy._multiSortRequest = true;
                    if ($(this).find(".e-expander").length == 0) {
                        if (!model.isFromGantt)
                            proxy.sortColumn(column.field, ej.sortOrder.Descending);
                        proxy._multiSortRequest = false;
                    }
                    break;
                case "contextMenu_SortAscending":

                    if (model.allowMultiSorting) proxy._multiSortRequest = true;
                    if ($(this).find(".e-expander").length == 0) {
                        if (!model.isFromGantt)
                            proxy.sortColumn(column.field, ej.sortOrder.Ascending);
                        proxy._multiSortRequest = false;
                    }
                    break;
                case "Delete":
                    proxy.deleteRow(null, true, proxy._contextMenuSelectedIndex);
                    proxy._cancelSaveTools();
                    proxy._clearContextMenu();
                    break;
                case "Above":
                    proxy._subContextMenuAction(item, index, "Above");
                    proxy._focusTreeGridElement();
                    break;
                case "Below":
                    proxy._subContextMenuAction(item, index, "Below");
                    proxy._focusTreeGridElement();
                    break;
                case "Add":
                    proxy._setInitialData();
                    proxy._clearContextMenu();
                    break;
                case "Edit":
                    if (editMode == "cellediting" || (editMode == "batchediting" && editSettings.batchEditSettings.editMode == "cell")) {
                        var column = model.columns[proxy.getCellIndex(e)];
                        if (!ej.isNullOrUndefined(column) && column.allowEditing != false) {
                            proxy._focusTreeGridElement();
                            proxy._cellEditingDetails.columnIndex = proxy.getCellIndex(e);
                            proxy._editedRowIndex = proxy._cellEditingDetails.rowIndex = proxy.getRowIndex(e);
                            fieldName = proxy._cellEditingDetails.columnIndex >= 0 && column.field;
                            if (fieldName) {
                                proxy.updateScrollBar();
                                model.editSettings.allowEditing &&
                                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName) && proxy._editAddTools();
                            }
                        }
                    }
                    else if (editMode == "rowediting" || (editMode == "batchediting" && editSettings.batchEditSettings.editMode == "row")) {
                        proxy.updateScrollBar();
                        proxy._editRow(index);
                    } else if (editMode == "dialogediting" || (editMode == "batchediting" && editSettings.batchEditSettings.editMode == "dialog")) {
                        proxy._showTreeGridAddEditDialog(index);
                    }
                    proxy._clearContextMenu();
                    break;
                case "Save":
                    if (editMode == "cellediting" || (editMode == "batchediting" && editSettings.batchEditSettings.editMode == "cell")) {
                        if (this.editFormValidate()) {
                            if (proxy._isRowEdit)
                                proxy._endEdit();
                            else proxy.saveCell();
                        }
                        else
                            return true;
                    }
                    else if (editMode == "rowediting" || (editMode == "batchediting" && editSettings.batchEditSettings.editMode == "row"))
                        proxy.saveRow();
                    proxy._clearContextMenu();
                    proxy._focusTreeGridElement();
                    break;
                case "Cancel":
                    if (editMode == "cellediting" || (editMode == "batchediting" && editSettings.batchEditSettings.editMode == "cell")) {
                        if (proxy._isRowEdit)
                            proxy.cancelRowEditCell();
                        else if (model.isEdit) proxy.cancelEditCell();
                    }
                    else if (editMode == "rowediting" || (editMode == "batchediting" && editSettings.batchEditSettings.editMode == "row"))
                        proxy.cancelRowEditCell();
                    proxy._clearContextMenu();
                    proxy._focusTreeGridElement();
                    break;
                default:
                    var eventArgs = {};
                    if (this._isContextMenuItems) {
                        eventArgs.data = item;
                        eventArgs.menuId = option;
                        proxy._triggerMenuEventHandler(option, eventArgs, true);
                        proxy._clearContextMenu();
                    }
                    else {
                        eventArgs.column = proxy._contextMenuColumnDetails;
                        eventArgs.menuId = option;
                        proxy._triggerMenuEventHandler(option, eventArgs, false);
                         proxy._clearContextMenu();
                    }
                    break;
            }
            if (model.enableAltRow && model.currentViewData[0])
                ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);            
        },
        //Change the text based on culture information.
        _setCultureInfo: function () {
            var proxy = this,
              model = proxy.model,
              culture = model.locale,
              localization = proxy.localizedLabel, 
              defaultLocalization = ej.TreeGrid.Locale["default"];
            if (!(model.predecessorTable || model.resourceTable))
                localization = ej.TreeGrid.Locale[culture];
            else
                localization = ej.Gantt.Locale[culture];
            //Tooltip Text to be displayed in the Toolbar
            proxy._toolboxTooltipTexts = (localization && localization["toolboxTooltipTexts"])?
                localization["toolboxTooltipTexts"] : defaultLocalization["toolboxTooltipTexts"];

            //Context Menu Text
            proxy._contextMenuTexts = (localization && localization["contextMenuTexts"]) ?
                localization["contextMenuTexts"] : defaultLocalization["contextMenuTexts"];

            if (proxy.model.isFromGantt) {
                proxy._columnMenuTexts = proxy.model.columnMenuTexts;
                proxy._deleteColumnText = proxy.model.deleteColumnText;
                proxy._okButtonText = proxy.model.okButtonText;
                proxy._cancelButtonText = proxy.model.cancelButtonText;
                proxy._columnDialogTexts = proxy.model.columnDialogTexts;
                proxy._columnDialogTitle = proxy.model.columnDialogTitle;
                proxy._confirmDeleteText = proxy.model.confirmDeleteText;
                proxy._editTypeText = proxy.model.editTypeText;
                proxy._textAlignTypeText = proxy.model.textAlignTypeText;
                proxy._clipModeText = proxy.model.clipModeText;
            }
            else {
                //Column Menu Text
                proxy._columnMenuTexts = (localization && localization["columnMenuTexts"]) ?
                    localization["columnMenuTexts"] : defaultLocalization["columnMenuTexts"];
                
                //Add Column Dialog Text to be displayed in popup window
                proxy._columnDialogTexts = (localization && localization["columnDialogTexts"]) ?
                    localization["columnDialogTexts"] : defaultLocalization["columnDialogTexts"];

                //Add Column Dialog Titles
                proxy._columnDialogTitle = (localization && localization["columnDialogTitle"]) ?
                    localization["columnDialogTitle"] : defaultLocalization["columnDialogTitle"];

                //Deleting the column texts 
                proxy._deleteColumnText = (localization && localization["deleteColumnText"]) ?
                    localization["deleteColumnText"] : defaultLocalization["deleteColumnText"];
                proxy._okButtonText = (localization && localization["okButtonText"]) ?
                    localization["okButtonText"] : defaultLocalization["okButtonText"];
                proxy._cancelButtonText = (localization && localization["cancelButtonText"]) ?
                    localization["cancelButtonText"] : defaultLocalization["cancelButtonText"];
                proxy._confirmDeleteText = (localization && localization["confirmDeleteText"]) ?
                    localization["confirmDeleteText"] : defaultLocalization["confirmDeleteText"];

                //edit Types text
                proxy._editTypeText = (localization && localization["editTypeTexts"]) ?
                    localization["editTypeTexts"] : defaultLocalization["editTypeTexts"];

                //Edit types text
                proxy._textAlignTypeText = (localization && localization["textAlignTypes"]) ?
                    localization["textAlignTypes"] : defaultLocalization["textAlignTypes"];

                //Clip mode types text
                proxy._clipModeText = (localization && localization["clipModeTexts"]) ?
                    localization["clipModeTexts"] : defaultLocalization["clipModeTexts"];

                //String to be displayed in drop down list for filtering blank items
                proxy._dropDownListBlanksText = (localization && localization["dropDownListBlanksText"]) ?
                    localization["dropDownListBlanksText"] : defaultLocalization["dropDownListBlanksText"];

                //String to be displayed in drop down list to clear filtered items
                proxy._dropDownListClearText = (localization && localization["dropDownListClearText"]) ?
                    localization["dropDownListClearText"] : defaultLocalization["dropDownListClearText"];

                //Text to be displayed in drop down list for TRUE value in boolean edit type column
                proxy._trueText = (localization && localization["trueText"]) ?
                    localization["trueText"] : defaultLocalization["trueText"];

                //Text to be displayed in drop down list for FALSE value in boolean edit type column
                proxy._falseText = (localization && localization["falseText"]) ?
                    localization["falseText"] : defaultLocalization["falseText"];

                //Batch editing confirm dialog texts 
                proxy._batchSaveConfirmText = (localization && localization["batchSaveConfirmText"]) ?
                    localization["batchSaveConfirmText"] : defaultLocalization["batchSaveConfirmText"];
                proxy._batchSaveLostChangesText = (localization && localization["batchSaveLostChangesText"]) ?
                    localization["batchSaveLostChangesText"] : defaultLocalization["batchSaveLostChangesText"];
                proxy._cancelEditText = (localization && localization["cancelEditText"]) ?
                    localization["cancelEditText"] : defaultLocalization["cancelEditText"];

                //EmptyRecord text to be displayed with dataSource is null
                proxy.model.emptyRecordText = (localization && localization["emptyRecord"]) ?
                    localization["emptyRecord"] : defaultLocalization["emptyRecord"];

                //Text to be displayed in filter menu
                proxy._filterMenuTexts = (localization && localization["filterMenuTexts"]) ?
                    localization["filterMenuTexts"] : defaultLocalization["filterMenuTexts"];
            }
        },

        
		
        //Public method to pass a datasource from customer side.

        //REFRESH THE TREEGRIDCONTENT
        

        refresh:function(datasource, query){
            var proxy = this, model = proxy.model;
            if (!ej.isNullOrUndefined(datasource)) {
                proxy.resetModelCollections();
                proxy._fieldNamesToQuery = [];
                proxy._assignedIdCollections = [];
                proxy._assignedParendIDs = [];
                model.dataSource = datasource;
                model.query = query;
                proxy.element.ejTreeGrid("destroy").ejTreeGrid(model);                
            }
            else {
                var args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;
                this.sendDataRenderingRequest(args);
            }
        },		
        /* get index for new record */
        _getNewIndex: function () {
            var proxy = this;
            proxy._maxRowIndex += 1;
            return (proxy._maxRowIndex);            
        },
        _getNewId: function () {
            var proxy = this,
                newId = proxy._lastInsertedId + 1;
            return newId;
        },
        _updateLastInsertedId: function (Id) {
            var proxy = this, newId = isNaN(parseInt(Id)) ? proxy._lastInsertedId < 1 ? 1 : proxy._lastInsertedId : parseInt(Id);
            proxy._lastInsertedId = proxy._lastInsertedId < newId ? newId : proxy._lastInsertedId;
        },
        _updateHierarchyRowIndexData: function (args, recordCollection, recordIndex) {
            var proxy = this, model = proxy.model, editMode = model.editSettings.editMode.toLowerCase();
            args.updatedHierarchyRowIndexData = [];
            for (var e = recordIndex; e < recordCollection.length; e++) {
                recordCollection[e].hierarchyRowIndex = e;
                if (model.hierarchyRowIndexMapping)
                    recordCollection[e].item[model.hierarchyRowIndexMapping] = e;                
                if (editMode != "batchediting")
                    args.updatedHierarchyRowIndexData.push(recordCollection[e]);
                else
                    proxy.batchChanges.updatedHierarchyRowIndexData.indexOf(recordCollection[e]) == -1 && proxy.batchChanges.updatedHierarchyRowIndexData.push(recordCollection[e]);
            }
        },
        //Add new record in treeGrid control
        _addRecord: function (data, rowPosition, isEdit, isFrom) {
            var proxy = this;

            /*Cancel if treegrid is in edit*/
            if (proxy.model.isEdit || proxy._isRowEdit) {
                proxy.cancelRowEditCell();
            }

            var model = this.model,
                selectedRowIndex = this.selectedRowIndex(),
                //selectedCellIndexes = this.selectedCellIndexes(),
                _cAddedRecord,
                updatedRecords = model.updatedRecords,
                flatRecords = model.flatRecords,
                parentRecords = model.parentRecords,
                selectedItem = model.selectionSettings.selectionMode == "row" ? this.selectedItem() : updatedRecords[proxy._rowIndexOfLastSelectedCell],
                columns = model.columns,
                dataSource = this.dataSource(),
                level = 0, insertIndex, validateArgs = {}, currentPageRowIndex = -1,
                isRecordAdded = false,
                isToolbarAdd = false,
                newIndex,                
                dataChildCount,
                editMode = model.editSettings.editMode.toLowerCase(),
                parentItem, nextBelowItemInflatRecords, nextBelowItemInUpdatedRecords,
                isParentItemAdd = false, args = {}, argsLevel, argsParentItem;

            if ($.type(dataSource) !== "array" && !(dataSource instanceof ej.DataManager)) {
                this.dataSource([]);
                dataSource = this.dataSource();
            }
            if (ej.isNullOrUndefined(data)) {
                data = null;
                for (var count = 0; count < columns.length; count++) {
                    if (!ej.isNullOrUndefined(columns[count].nonColumn))
                        continue;
                    data = proxy._restoreCustomData(data, "", columns[count].field);
                }
                isToolbarAdd = true;
            }
            if (editMode == "batchediting")
                data.isBatchAdd = true;
            if (model.idMapping && model.parentIdMapping && !ej.isNullOrUndefined(data[proxy.model.parentIdMapping]) && rowPosition != "child") {
                var parentRec = model.flatRecords.filter(function (item) {
                    return item[model.idMapping] == data[model.parentIdMapping];
                });
                if (parentRec.length > 0) {
                    selectedItem = parentRec[0];
                    rowPosition = ej.TreeGrid.RowPosition.Child;
                    isParentItemAdd = true;
                }
            }

            if ((selectedRowIndex === -1 && proxy._rowIndexOfLastSelectedCell == -1) && (rowPosition === ej.TreeGrid.RowPosition.Above
                 || rowPosition === ej.TreeGrid.RowPosition.Below
                 || rowPosition === ej.TreeGrid.RowPosition.Child) || !rowPosition || (model.selectedItems.length > 1 && !proxy._checkboxSelection)) {
                if (!isParentItemAdd)
                    rowPosition = ej.TreeGrid.RowPosition.Top;
            }
            if (rowPosition == ej.TreeGrid.RowPosition.Top || rowPosition == ej.TreeGrid.RowPosition.Bottom)
            {
                argsLevel = 0;
                argsParentItem = null;
            }
            else if (rowPosition == ej.TreeGrid.RowPosition.Above || rowPosition == ej.TreeGrid.RowPosition.Below) {
                argsLevel = selectedItem.level;
                argsParentItem = selectedItem.parentItem;
            }
            else if (rowPosition == ej.TreeGrid.RowPosition.Child) {
                argsLevel = selectedItem.level + 1;
                argsParentItem = selectedItem;
            }
            if (model.idMapping && model.parentIdMapping) {
                var id = data[model.idMapping];
                if (ej.isNullOrUndefined(id) || id == "" || isFrom == "contextMenu")
                    data[model.idMapping] = proxy._getNewId();
                else if (proxy._validateIdValue(id)) return false;
            }

            args.requestType = "add";
            args.data = data;
            args.level = argsLevel;
            args.rowPosition = rowPosition;
            args.parentItem = argsParentItem;
            if (!proxy._trigger("actionBegin", args)) {
                data = args.data;

                if (rowPosition != args.rowPosition)
                    rowPosition = args.rowPosition;

                if (data[model.childMapping] && data[model.childMapping].length)
                    delete data[model.childMapping]; // Remove the child items for externally adding row

                //Clear previously selected Item
                //proxy.clearSelection()
                proxy.selectRows(-1);
                newIndex = this._getNewIndex();

                switch (rowPosition) {
                    case ej.TreeGrid.RowPosition.Top:
                        level = 0;
                        parentItem = null;
                        _cAddedRecord = proxy._createRecord(data, level);
                        _cAddedRecord.index = newIndex;
                        
                        /*record update*/
                        flatRecords.splice(0, 0, _cAddedRecord);
                        updatedRecords.splice(0, 0, _cAddedRecord);
                        if (proxy._isHierarchyFiltering)
                            proxy._updateFilterParent.splice(0, 0, _cAddedRecord);
                        parentRecords.splice(0, 0, _cAddedRecord);

                        //Update hierarchyRowIndex for level 0 records
                        proxy._updateHierarchyRowIndexData(args, parentRecords, 0);

                        /*data source update*/
                        if (proxy._isDataManagerUpdate) {
                            proxy._jsonData.splice(0, 0, _cAddedRecord.item);
                        } else {
                            dataSource.splice(0, 0, _cAddedRecord.item);
                        }
                        insertIndex = 0;
                        break;

                    case ej.TreeGrid.RowPosition.Bottom:
                        level = 0;
                        parentItem = null;
                        _cAddedRecord = proxy._createRecord(data, level);
                        _cAddedRecord.index = newIndex;
                        
                        /*record update*/
                        flatRecords.push(_cAddedRecord);
                        updatedRecords.push(_cAddedRecord);
                        if (proxy._isHierarchyFiltering)
                            proxy._updateFilterParent.push(_cAddedRecord);
                        //ids.push(newIndex);
                        parentRecords.push(_cAddedRecord);
                        /*data source update*/
                        if (proxy._isDataManagerUpdate) {
                            proxy._jsonData.push(_cAddedRecord.item);
                        } else {
                            dataSource.push(_cAddedRecord.item);
                        }
                        insertIndex = updatedRecords.indexOf(_cAddedRecord);
                        break;

                    case ej.TreeGrid.RowPosition.Above:
                        level = selectedItem.level;
                        parentItem = selectedItem.parentItem;
                        _cAddedRecord = proxy._createRecord(data, level, parentItem);
                        _cAddedRecord.index = newIndex;
                        if (proxy._isHierarchyFiltering && selectedItem.filterLevel > -1)
                            _cAddedRecord.filterLevel = selectedItem.filterLevel;

                        var childIndex, recordIndex, updatedCollectionIndex;
                        /*Record Updates*/
                        recordIndex = flatRecords.indexOf(selectedItem);
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem);
                        flatRecords.splice(recordIndex, 0, _cAddedRecord);
                        updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                        if (proxy._isHierarchyFiltering)
                            proxy._updateFilterParent.splice(updatedCollectionIndex, 0, _cAddedRecord);
                        //ids.splice(recordIndex, 0, newIndex);

                        if (!ej.isNullOrUndefined(parentItem)) {
                            childIndex = parentItem.childRecords.indexOf(selectedItem);
                            /*Child collection update*/
                            parentItem.childRecords.splice(childIndex, 0, _cAddedRecord);
                            if (!model.parentIdMapping) {
                                parentItem.item[model.childMapping].splice(childIndex, 0, _cAddedRecord.item);
                            }
                            else {
                                _cAddedRecord.item[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                                _cAddedRecord[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                                if (proxy._isDataManagerUpdate) {
                                    proxy._jsonData.push(_cAddedRecord.item);

                                } else {
                                    dataSource.push(_cAddedRecord.item);
                                }
                            }
                            //Update hierarchyRowIndex for level 1 and above records
                            var childRecords = parentItem.childRecords;
                            proxy._updateHierarchyRowIndexData(args, childRecords, childIndex);
                        } else {
                            /* Parent records collection and data source update*/
                            parentRecords.splice(parentRecords.indexOf(selectedItem), 0, _cAddedRecord);
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.splice(proxy._jsonData.indexOf(selectedItem.item), 0, _cAddedRecord.item);
                            }
                            else {
                                dataSource.splice(dataSource.indexOf(selectedItem.item), 0, _cAddedRecord.item);
                            }
                            //Update hierarchyRowIndex for level 0 records
                            var pIndex = parentRecords.indexOf(selectedItem);
                            proxy._updateHierarchyRowIndexData(args, parentRecords, pIndex);
                        }
                        insertIndex = updatedCollectionIndex;
                        break;
                    case ej.TreeGrid.RowPosition.Below:
                        level = selectedItem.level;
                        parentItem = selectedItem.parentItem;
                        _cAddedRecord = proxy._createRecord(data, level, parentItem);
                        _cAddedRecord.index = newIndex;
                        if (proxy._isHierarchyFiltering && selectedItem.filterLevel>-1)
                            _cAddedRecord.filterLevel = selectedItem.filterLevel;

                        //find next item position
                        var currentItemIndex = flatRecords.indexOf(selectedItem);

                        if (selectedItem.hasChildRecords) {
                            dataChildCount = proxy.getChildCount(selectedItem, 0);
                            recordIndex = currentItemIndex + dataChildCount + 1;
                            updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + proxy._getVisibleChildRecordCount(selectedItem, 0, updatedRecords) + 1;

                        } else {
                            recordIndex = currentItemIndex + 1;
                            updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + 1;
                        }

                        /* Record collection update */
                        flatRecords.splice(recordIndex, 0, _cAddedRecord);
                        updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                        if (proxy._isHierarchyFiltering)
                            proxy._updateFilterParent.splice(updatedCollectionIndex, 0, _cAddedRecord);
                        //ids.splice(recordIndex, 0, newIndex);

                        /* data Source update */
                        if (!ej.isNullOrUndefined(parentItem)) {
                            childIndex = parentItem.childRecords.indexOf(selectedItem);
                            /*Child collection update*/
                            parentItem.childRecords.splice(childIndex + 1, 0, _cAddedRecord);
                            if (!model.parentIdMapping) {
                                parentItem.item[model.childMapping].splice(childIndex + 1, 0, _cAddedRecord.item);
                            }
                            else {
                                _cAddedRecord.item[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                                _cAddedRecord[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                                if (proxy._isDataManagerUpdate) {
                                    proxy._jsonData.push(_cAddedRecord.item);

                                } else {
                                    dataSource.push(_cAddedRecord.item);
                                }
                            }
                            //Update hierarchyRowIndex for level 1 and above records
                            var childRecords = parentItem.childRecords;
                            proxy._updateHierarchyRowIndexData(args, childRecords, childIndex + 1);
                        } else {
                            /* Parent records collection and data source update*/
                            parentRecords.splice(parentRecords.indexOf(selectedItem) + 1, 0, _cAddedRecord);
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.splice(proxy._jsonData.indexOf(selectedItem.item) + 1, 0, _cAddedRecord.item);
                            }
                            else {
                                dataSource.splice(dataSource.indexOf(selectedItem.item) + 1, 0, _cAddedRecord.item);
                            }
                            //Update hierarchyRowIndex for level 0 records
                            var pIndex = parentRecords.indexOf(selectedItem);
                            proxy._updateHierarchyRowIndexData(args, parentRecords, pIndex + 1);
                        }
                        insertIndex = updatedCollectionIndex;
                        break;
                    case ej.TreeGrid.RowPosition.Child:
                        level = selectedItem.level + 1;
                        parentItem = selectedItem;
                        _cAddedRecord = proxy._createRecord(data, level, parentItem);
                        _cAddedRecord.index = newIndex;
                        if (proxy._isHierarchyFiltering && selectedItem.filterLevel > -1)
                            _cAddedRecord.filterLevel = selectedItem.filterLevel+1;
                        
                        //find next item position
                        var currentItemIndex = flatRecords.indexOf(selectedItem);

                        if (selectedItem.hasChildRecords) {
                            dataChildCount = proxy.getChildCount(selectedItem, 0);
                            recordIndex = currentItemIndex + dataChildCount + 1;
                            //Expand Add record's parent item 
                            if (!selectedItem.expanded || !proxy.getExpandStatus(selectedItem)) {
                                var expanargs = {};
                                expanargs.expanded = true;
                                expanargs.data = selectedItem;
                                proxy._isInAdd = true;
                                /* refresh Added row on expand collapse */
                                if (proxy._isRefreshAddedRecord) {
                                    expanargs.data.expanded = true;
                                    proxy.updateExpandStatus(args.data, args.expanded);
                                    proxy.refreshContent();
                                    proxy._isRefreshAddedRecord = false;
                                } else {
                                    proxy._expandRecord(selectedItem);
                                }
                                updatedRecords = model.updatedRecords;
                                proxy._isInAdd = false;                              
                            }
                            updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + proxy._getVisibleChildRecordCount(selectedItem, 0, updatedRecords) + 1 - (model.showSummaryRow ? model.summaryRows.length:0);

                        } else {
                            if (proxy._isLoadonDemand)
                                selectedItem.isChildRetrieved = true;
                            if (!proxy.getExpandStatus(selectedItem))
                                proxy._expandRecord(selectedItem.parentItem);                            
                            selectedItem.hasChildRecords = true;
                            selectedItem.childRecords = [];
                            selectedItem.expanded = true;
                            if (!model.parentIdMapping) {
                                selectedItem.item[model.childMapping] = [];
                            }
                            selectedItem.isMilestone = false;
                            recordIndex = currentItemIndex + 1;
                            updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + 1;
                        }

                        /* Record collection update */
                        flatRecords.splice(recordIndex, 0, _cAddedRecord);
                        if (proxy._isHierarchyFiltering)
                            proxy._updateFilterParent.splice(updatedCollectionIndex, 0, _cAddedRecord);
                        if (proxy._isLoadonDemand) {
                            if (selectedItem.isChildRetrieved)
                                updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                        }
                        else
                            updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                        //ids.splice(recordIndex, 0, newIndex);

                        /* data Source update */
                        if (!ej.isNullOrUndefined(parentItem)) {
                            childIndex = parentItem.childRecords.indexOf(selectedItem);
                            /*Child collection update*/
                            parentItem.childRecords.splice(childIndex + 1, 0, _cAddedRecord);
                            if (!model.parentIdMapping) {
                                parentItem.item[model.childMapping].splice(childIndex + 1, 0, _cAddedRecord.item);
                            }
                            else {
                                _cAddedRecord.item[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                                _cAddedRecord[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                                if (proxy._isDataManagerUpdate) {
                                    proxy._jsonData.push(_cAddedRecord.item);
                                } else {
                                    dataSource.push(_cAddedRecord.item);
                                }
                            }
                            //Update hierarchyRowIndex for level 1 and above records
                            var childRecords = parentItem.childRecords;
                            proxy._updateHierarchyRowIndexData(args, childRecords, 0);
                        }
                        insertIndex = updatedCollectionIndex;
                        break;
                }
                if (model.allowPaging) {
                    //Goto next page if New record insertIndex exit than pagesize
                    if (rowPosition == ej.TreeGrid.RowPosition.Below || rowPosition == ej.TreeGrid.RowPosition.Child) {
                        if ((model.pageSettings.pageSizeMode !== "root" || ej.isNullOrUndefined(selectedItem.parentItem))) {
                            var targetData = proxy._updatedPageData[proxy._updatedPageData.length - 1],
                                currentIndex = proxy._updatedPageData.length == model.pageSettings.pageSize ?
                                model.updatedRecords.indexOf(targetData) : insertIndex;
                            if (insertIndex > currentIndex) {
                                proxy._currentPage(this._currentPage() + 1);
                                currentPageRowIndex = 0;
                            }
                        }
                    }
                        //Goto first page
                    else if (rowPosition == ej.TreeGrid.RowPosition.Top && (proxy._currentPage() > 1 || proxy._currentPage() < 1)) {
                        proxy._currentPage(1);
                        currentPageRowIndex = 0;
                    }
                        //Goto last page
                    else if (rowPosition == ej.TreeGrid.RowPosition.Bottom && proxy._currentPage() < model.pageSettings.totalPages) {
                        proxy._currentPage(model.pageSettings.totalPages);
                        currentPageRowIndex = insertIndex;
                    }

                }
                //For update selectedItems if Add record once after filtered
                if ((model.allowFiltering && model.filterSettings.filteredColumns.length > 0) || proxy._searchString.length > 0)
                    proxy._flatFilteredRecords.push(_cAddedRecord);

                if (!isToolbarAdd && (ej.TreeGrid.RowPosition.Above || ej.TreeGrid.RowPosition.Below)) {
                    var args = {},
                        currentValue = {},
                        columns = model.columns,
                        length = columns.length;
                    if (!model.allowPaging)
                        currentPageRowIndex = insertIndex;
                    args.rowIndex = insertIndex;
                    args.requestType = "addNewRow";
                    args.addedRow = data;

                    if (editMode == "batchediting" || !proxy._trigger("actionComplete", args)) {
                        proxy._updateAction(args);
                        proxy._renderAddedRow(insertIndex, _cAddedRecord);
                        if (model.showSummaryRow) {
                            if (args.requestType == "addNewRow" && rowPosition == "child") {
                                proxy._createAndRenderSummaryRecords(args);
                            }
                            proxy._updateSummaryRow(args);
                        }
                        if (model.showTotalSummary)
                            proxy._updateTotalSummaryRow(args);
                        isRecordAdded = true;
                        if (model.idMapping && model.parentIdMapping) {
                            proxy. _updateLastInsertedId(_cAddedRecord[model.idMapping]);                            
                        }                        
                    }
                    else {
                        updatedRecords.splice(insertIndex, 1);
                    }
                }
                else {
                    proxy._renderAddedRow(insertIndex, _cAddedRecord);
                    isRecordAdded = true;
                    proxy._emptyRowIndex = model.selectionSettings.selectionMode == "cell" ? insertIndex : null;
                }
                if (editMode == "batchediting" && isRecordAdded && !isToolbarAdd) {
                    proxy.batchChanges.added.push(_cAddedRecord);
                    proxy._batchEditStarts = true;
                }
                /*Set flag for refresh newly added record in non virtalization mode*/
                if (model.enableVirtualization === false &&
                    (model.sortSettings.sortedColumns.length > 0 || model.filterSettings.filteredColumns.length > 0)) {
                    proxy._isRefreshAddedRecord = true;
                }
                if (model.allowPaging)
                    updatedRecords = proxy._updatedPageData;
                /* select add row */
                var currentRowIndex = isRecordAdded ? updatedRecords.indexOf(_cAddedRecord) : selectedRowIndex;
                
                if (currentRowIndex == -1)
                    currentRowIndex = selectedRowIndex;

                if (model.allowSelection && model.selectionSettings.selectionMode == "row") {
                    
                    if (!proxy._rowSelectingEventTrigger(this._previousIndex, currentRowIndex)) {  
						proxy.selectRows(currentRowIndex);					
                        if (model.selectionSettings.enableSelectAll)
                            proxy.selectAllRows();
                        if (proxy._checkboxSelection && model.selectionSettings.enableHierarchySelection && _cAddedRecord.parentItem)
                            proxy._updateParentSelection(_cAddedRecord.parentItem);
                        proxy._rowSelectedEventTrigger(currentRowIndex);
                    }
                }
                if (!model.allowSelection && model.selectionSettings.selectionMode == "row") {
                    proxy.selectRows(currentRowIndex);
                }
                if (model.selectionSettings.selectionMode == "cell")
                    proxy._rowIndexOfLastSelectedCell = updatedRecords.indexOf(_cAddedRecord);
                /* scroll to new added record */
                proxy.updateScrollBar();

                if (isEdit == "isEditRow") {
                    proxy._editRow(updatedRecords.indexOf(_cAddedRecord), _cAddedRecord);
                }
                else {
                    ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(_cAddedRecord));
                    proxy._cancelSaveTools();
                    if (model.enableAltRow)
                        ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);
                }
                //refresh parent item for expand collapse icons
                if (_cAddedRecord.parentItem && rowPosition == ej.TreeGrid.RowPosition.Child && (_cAddedRecord.parentItem.childRecords.length == 1 || !_cAddedRecord.parentItem.hasFilteredChildRecords)) {
                    _cAddedRecord.parentItem.hasFilteredChildRecords = true;
                    ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(_cAddedRecord.parentItem));
                }
                if (model.allowTextWrap && proxy._$frozenTableContent.length > 0) {
                    proxy._updateWrapRowHeights("total,header");
                    proxy.updateHeight();
                } else if (model.allowTextWrap && model.enableVirtualization) {
                    proxy.updateHeight();
                }
            } else {
                if (isFrom == "dialog")
                    return false;
            }
            if (proxy._commandColumnLength > 0) {
                if (ej.isNullOrUndefined(isEdit)) {
                    var tr = ej.TreeGrid.getRowByIndex(proxy, currentRowIndex)
                    proxy._refreshUnboundTemplate(tr);
                }
                if (rowPosition == ej.TreeGrid.RowPosition.Child) {
                    var parentIndex = updatedRecords.indexOf(_cAddedRecord.parentItem),
                    parentTr = ej.TreeGrid.getRowByIndex(proxy, parentIndex);
                    proxy._refreshUnboundTemplate(parentTr);
                }
            }
            return true;
        },

        //validate the taskId to avoid duplicate value
        _validateIdValue: function (dataID) {
            var proxy = this, model = proxy.model,
                matchedItem = [],
                fRecords = model.flatRecords;
            if (proxy._isLoadonDemand) {
                matchedItem = $.map(proxy._assignedIdCollections, function (value) {
                    if (value == dataID)
                        return value;
                });
            }
            else {
                matchedItem = $.map(fRecords, function (record) {
                    if (record[model.idMapping] == dataID)
                        return record;
                });
            }
            if (matchedItem.length)
                return true;
            else return false;
        },         
         
        // Method to trigger the actionbegin event while close the Add/Edit dialog
        _closeTreegridAddEditDialog: function (args, type) {
            var proxy = this;
            var eventArgs = {};
            this._off($("#" + this._id + "_dialogEdit"),
                "click keypress", "#EditFormDialog_" + this._id + "_Save ,#EditFormDialog_" + this._id + "_Cancel", proxy._buttonClick);
            if (proxy._isTreeAddEditDialogSave)
                proxy._isTreeAddEditDialogSave = false;
            else {
                if (type === "Add")
                    eventArgs.requestType = "closeAddDialog";
                else
                    eventArgs.requestType = "closeEditDialog";
                return proxy._trigger("actionBegin", eventArgs);
            }
        },

        // Public method to show the Add dialog/Add template dialog
        showAddDialog: function () {
            var proxy = this;
            if (proxy.model.editSettings.allowAdding && proxy.model.editSettings.editMode == "dialogEditing")
                proxy._showTreeGridAddEditDialog(null, "add");
        },

        // Public method to show the Edit dialog/Edit template dialog
        showEditDialog: function (index) {
            var proxy = this;
            if (proxy.model.editSettings.allowEditing && proxy.model.editSettings.editMode == "dialogEditing")
                proxy._showTreeGridAddEditDialog(index);
        },

        // Method to open/show the ADD/EDIT dialog from all possible ways (double/single click edit, toolbar click, public method, context menu)
        _showTreeGridAddEditDialog: function (index, addrow) {
            var temp = document.createElement('div'), proxy = this,
            $temp = $(temp), controlPosition = proxy.getOffsetRect($("#" + proxy._id)[0]),
            currentData = [], columns = proxy.model.columns, contextMenuLabel = proxy._contextMenuTexts;
            if ((ej.isNullOrUndefined(index) || index == -1) && !addrow) {
                if (proxy.model.selectionSettings.selectionMode == "cell")
                    index = proxy.selectedCellIndexes()[0].rowIndex;
                else
                    index = proxy.selectedRowIndex();
            }
            currentData[0] = proxy.model.allowPaging ? this._updatedPageData[index] : proxy.model.updatedRecords[index];
            if (addrow && addrow.toLowerCase() === "add") {
                if (ej.isNullOrUndefined(currentData[0])) {
                    currentData[0] = null;
                    for (var count = 0; count < columns.length; count++) {
                        currentData[0] = proxy._restoreCustomData(currentData[0], "", columns[count].field);
                    }
                    if (proxy.model.parentIdMapping)
                        currentData[0][proxy.model.idMapping] = proxy._getNewId();
                }
                $temp.addClass("e-dialogaddrow");
            } else {
                $temp.addClass("e-dialogeditedrow");
            }
            if (ej.isNullOrUndefined(proxy.model.editSettings.dialogEditorTemplateID) || proxy.model.editSettings.dialogEditorTemplateID == "")
                temp.innerHTML = $.render[this._id + "_JSONDialogEditingTemplate"](currentData);
            else if (!ej.isNullOrUndefined(proxy.model.editSettings.dialogEditorTemplateID) && proxy.model.editSettings.dialogEditorTemplateID != "")
                temp.innerHTML = $.render[this._id + "_JSONdialogTemplateMode"](currentData);

            if (this.model.editSettings.editMode == "dialogEditing" || (proxy.model.editSettings.editMode.toLowerCase() == "batchediting" && proxy.model.editSettings.batchEditSettings.editMode == "dialog")) {
                $("#" + this._id + "_dialogEdit").html($(temp));
                var model = {};
                model.cssClass = this.model.cssClass;
                model.enableRTL = false;
                model.enableResize = false;
                model.showOnInit = false;
                model.content = "#" + this._id;
                if ($(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive) {
                    model.width = $("#" + proxy._id).width() + "px";
                    model.height = $("#" + proxy._id).height() + "px";
                    model.enableModal = false;
                    model.isResponsive = false;
                    model.allowDraggable = false;
                } else {
                    model.width = "auto";
                    model.height = "auto";
                    model.enableModal = true;
                    model.isResponsive = false;
                    model.allowDraggable = true;
                }
                model.allowKeyboardNavigation = false;
                if (addrow && addrow.toLowerCase() === "add") {
                    model.title = contextMenuLabel["addRowText"];
                    model.beforeClose = function (args) {
                        if (proxy._closeTreegridAddEditDialog(args, "Add"))
                            args.cancel = true; // Cancel the add dialog box close.
                            proxy._focusTreeGridElement();
                    };
                }
                else {
                    model.title = contextMenuLabel["editText"];
                    model.beforeClose = function (args) {
                        if (proxy._closeTreegridAddEditDialog(args, "Edit"))
                            args.cancel = true; // Cancel the add dialog box close.
                            proxy._focusTreeGridElement();
                    };
                }
                $("#" + this._id + "_dialogEdit").ejDialog(model);
                var dialogArgs = {};
                dialogArgs.data = currentData[0];
                if (addrow && addrow.toLowerCase() === "add") {
                    dialogArgs.requestType = "beforeOpenAddDialog";
                    dialogArgs.rowPosition = proxy.model.editSettings.rowPosition;
                }
                else
                    dialogArgs.requestType = "beforeOpenEditDialog";
                dialogArgs.element = $("#" + proxy._id + "_dialogEdit");
                if (!proxy._trigger("actionBegin", dialogArgs)) {
                    var args = {};
                    args.columnIndex = index;
                    args.data = dialogArgs.data;
                    args.requestType = "beginEdit";
                    proxy._refreshEditForm(args);
                    dialogArgs.data = args.data;
                    if (addrow && addrow.toLowerCase() === "add")
                        dialogArgs.requestType = "openAddDialog";
                    else
                        dialogArgs.requestType = "openEditDialog";
                    dialogArgs.element = $("#" + proxy._id + "_dialogEdit");
                    if (!proxy._trigger("actionBegin", dialogArgs)) {
                        $("#" + this._id + "_dialogEdit").ejDialog("open");
                        proxy._currentEditRecordDialogIndex = this.model.currentViewData.indexOf(args.data);
                        if ($(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive) {
                            $("#" + proxy._id + "_dialogEdit_wrapper").css({ 'left': controlPosition.left + "px", 'top': controlPosition.top + "px" });
                            $("#" + proxy._id + "_dialogEdit_wrapper").removeClass("e-shadow").removeClass("e-treegridadaptivedialog");
                            $("#" + proxy._id + "_dialogEdit").removeClass("e-shadow");
                        } else {
                            $("#" + proxy._id + "_dialogEdit_wrapper").addClass("e-shadow").addClass("e-treegridadaptivedialog");
                            $("#" + proxy._id + "_dialogEdit").addClass("e-shadow");
                        }
                        this._on($("#" + this._id + "_dialogEdit"),
                       "click keypress", "#EditFormDialog_" + this._id + "_Save ,#EditFormDialog_" + this._id + "_Cancel", proxy._buttonClick);
                        $("#" + proxy._id + "_dialogEdit").ejDialog("refresh");
                    }
                }
            }
            if ($.isFunction($.validator)) {
                proxy._initValidator();
                proxy.setValidation();
            }
        },

        //Public method for adding new record
        addRow: function (data, rowPosition) {
            var proxy = this, model = proxy.model;
            /* Cancel edited cell before sort the column*/
            proxy._cancelEditState();

            if (!data) {
                proxy._addRecord(null, rowPosition, "isEditRow");
            } else {
                if (proxy._addRecord(data, rowPosition)) return true;
                else
                    return false;
            } 
        },

        _startAdd: function (parentEle, rowPos) {
            var proxy = this, model = this.model, rowPosition;
            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                proxy._clearMultiSelectPopup();
            }
            proxy._addRecord(null, model.editSettings.rowPosition, "isEditRow");
        },
        _getRecordIndexByItem: function (item, collection) {
            var record = ej.DataManager(collection).executeLocal(ej.Query().where("item", ej.FilterOperators.equal, item));
            if (record.length > 0)
                return collection.indexOf(record[0]);
            else
                return -1;
        },
        /* update the reord's expand status in expanded state*/
        _expandRecord: function (record)
        {
            if (this._isLoadonDemand && !record.isChildRetrieved)
                return;
            var parentItem = record, args = {};
            while (parentItem != null) {
                if (parentItem.expanded == false) {
                    args.expanded = true;
                    args.data = parentItem;
                    if (!parentItem.parentItem)
                        this._isInExpandCollapseAll = false;
                    else
                        this._isInExpandCollapseAll = true;
                    ej.TreeGrid.sendExpandCollapseRequest(this, args);
                }
                parentItem = parentItem.parentItem;
            }
        },

        /* Public method for scroll to the corresponding offset value */
        scrollOffset: function (left, top) {
            var scrollerObj = this.getScrollElement().data("ejScroller"),
                proxy = this;

            proxy._saveEditedElements();

            if ((typeof top == "number" || typeof parseInt(top) == "number")) {
                top = parseInt(top);
                if (top >= 0 && this.getMaxScrollHeight() >= top)
                    scrollerObj.scrollY(top, true);
            }
            if (typeof left == "number" || typeof parseInt(left) == "number") {
                left = parseInt(left);
                if (left >= 0 && this.getMaxScrollWidth() >= left)
                    scrollerObj.scrollX(left, true);
            } 
        },
        /* Public method for get scrollTop value*/
        getScrollTopOffset: function () {
            return this.getScrollElement().ejScroller("model.scrollTop");
        },
        /* Public method for get scrollLeft value*/
        getScrollLeftOffset: function () {
            return this.getScrollElement().ejScroller("model.scrollLeft");
        },
        /* Public method for scroll to first row of treegrid*/
        scrollToTop:function()
        {
            var model = this.model,
                updatedRecords = this.getUpdatedRecords(), record, rowIndex = -1;
            if (!this.isVScroll()) return;

            this._saveEditedElements();

            updatedRecords = model.allowPaging ? updatedRecords : this.getExpandedRecords(model.updatedRecords);
            if (updatedRecords.length > 0) {
                rowIndex = 0;
                record = updatedRecords[0];
                rowIndex = updatedRecords.indexOf(record);
                this.updateScrollBar(rowIndex);
                if (model.allowSelection && !this._rowSelectingEventTrigger(this.selectedRowIndex(), rowIndex)) {
                    this.selectRows(rowIndex);
                    this._rowSelectedEventTrigger(rowIndex);
                }
            }
        },
        /* Public method for scroll to last row of treegrid*/
        scrollToBottom: function ()
        {
            var model = this.model,
                updatedRecords, record, rowIndex = -1;
            if (!this.isVScroll()) return;

            this._saveEditedElements();

            updatedRecords = model.allowPaging ? this.getUpdatedRecords() : this.getExpandedRecords(model.updatedRecords);

            if (updatedRecords.length > 0) {
                rowIndex = updatedRecords.length - 1;
                record = updatedRecords[rowIndex];
                rowIndex = model.allowPaging ? updatedRecords.indexOf(record) : model.updatedRecords.indexOf(record);
                this.updateScrollBar(rowIndex);
                if (model.allowSelection && !this._rowSelectingEventTrigger(this.selectedRowIndex(), rowIndex)) {
                    this.selectRows(rowIndex);
                    this._rowSelectedEventTrigger(rowIndex);
                }
            }
        },

        //ENABLE AND DISABLE THE CORRESPONDING TOOLS WHILE ADDING AND EDITING THE ROW RECORD.
        _editAddTools: function () {
            var proxy = this, model = proxy.model;
            var toolbar = $("#" + proxy._id + "_toolbarItems");
            var editSettings = proxy.model.editSettings,
            toolbarItems = model.toolbarSettings.toolbarItems;
            var disableToolItems = [];
            if (editSettings.allowAdding && toolbarItems.indexOf("add") !== -1 && !$(toolbar).find(".e-addnew").parent("li").hasClass("e-disable") && editSettings.beginEditAction != "click")
                disableToolItems.push($(toolbar).find(".e-addnew").parent()[0]);
            if (editSettings.allowEditing && toolbarItems.indexOf("edit") !== -1 && !$(toolbar).find(".e-edit").parent("li").hasClass("e-disable"))
                disableToolItems.push($(toolbar).find(".e-edit").parent()[0]);
            if (editSettings.allowDeleting && toolbarItems.indexOf("delete") !== -1 && !$(toolbar).find(".e-delete").parent("li").hasClass("e-disable") && model.editSettings.beginEditAction != "click")
                disableToolItems.push($(toolbar).find(".e-delete").parent()[0]);
            disableToolItems.forEach(function (element) {
                $(toolbar).ejToolbar('disableItem', element);
            });

            var enabledToolItems = [];
            if (editSettings.editMode.toLowerCase() != "batchediting" || (editSettings.editMode.toLowerCase() == "batchediting" && proxy._batchEditStarts)) {
                if (toolbarItems.indexOf("cancel") !== -1)
                    enabledToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
                if (toolbarItems.indexOf("update") !== -1)
                    enabledToolItems.push($(toolbar).find(".e-save").parent()[0]);
            }
            enabledToolItems.forEach(function (element) {
                $(toolbar).ejToolbar('enableItem', element);
            });

        },
        //ENABLE AND DISABLE THE CORRESPONDING TOOLS WHILE CANCEL AND SAVING THE ROW RECORD.
        _cancelSaveTools: function () {
            var proxy = this,
                model = proxy.model,
                editSettings = proxy.model.editSettings,
                toolbar = $("#" + proxy._id + "_toolbarItems"),
                parentRecords = [],
                enabledToolItems,
                disableToolItems,
                toolbarItems = model.toolbarSettings.toolbarItems;
            if (!model.isFromGantt) {
                enabledToolItems = [];
                disableToolItems = [];
                if (editSettings.allowAdding) {
                    if (toolbarItems.indexOf("add") !== -1)
                        enabledToolItems.push($(toolbar).find(".e-addnew").parent()[0]);
                } else {
                    disableToolItems.push($(toolbar).find(".e-addnew").parent()[0]);
                }

                if (editSettings.allowEditing && toolbarItems.indexOf("edit") !== -1) {
                    if (proxy.selectedRowIndex() != -1 && model.allowSelection && (model.selectedItems.length == 1 || proxy._checkboxSelection)  && editSettings.beginEditAction != "click")
                        enabledToolItems.push($(toolbar).find(".e-edit").parent()[0]);
                    else if (!$(toolbar).find(".e-edit").parent("li").hasClass("e-disable"))
                        disableToolItems.push($(toolbar).find(".e-edit").parent()[0]);
                } else {
                    disableToolItems.push($(toolbar).find(".e-edit").parent()[0]);
                }
                if (editSettings.allowDeleting && toolbarItems.indexOf("delete") !== -1) {
                    if ((proxy._checkboxSelection || (proxy.selectedRowIndex() != -1 && model.allowSelection)) && model.selectedItems.length > 0)
                        enabledToolItems.push($(toolbar).find(".e-delete").parent()[0]);
                    else if (!$(toolbar).find(".e-delete").parent("li").hasClass("e-disable"))
                        disableToolItems.push($(toolbar).find(".e-delete").parent()[0]);
                } else {
                    disableToolItems.push($(toolbar).find(".e-delete").parent()[0]);
                }

                parentRecords = proxy._getParentRecords(model.updatedRecords, proxy);

                if (parentRecords.length <= 0) {
                    if (toolbarItems.indexOf("expandAll") !== -1 && !$(toolbar).find(".e-expandall").parent("li").hasClass("e-disable"))
                        disableToolItems.push($(toolbar).find(".e-expandall").parent()[0]);
                    if (toolbarItems.indexOf("collapseAll") !== -1 && !$(toolbar).find(".e-collapseall").parent("li").hasClass("e-disable"))
                        disableToolItems.push($(toolbar).find(".e-collapseall").parent()[0]);
                }
                else {
                    if (toolbarItems.indexOf("expandAll") !== -1)
                        enabledToolItems.push($(toolbar).find(".e-expandall").parent()[0]);
                    if (toolbarItems.indexOf("collapseAll") !== -1)
                        enabledToolItems.push($(toolbar).find(".e-collapseall").parent()[0]);
                }
                if (editSettings.beginEditAction != "click" || (editSettings.beginEditAction == "click" && !model.isEdit)) {
                    if (editSettings.editMode.toLowerCase() != "batchediting" || (editSettings.editMode.toLowerCase() == "batchediting" && !proxy._batchEditStarts)) {
                        if (toolbarItems.indexOf("cancel") !== -1 && !$(toolbar).find(".e-cancel").parent("li").hasClass("e-disable"))
                            disableToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
                        if (toolbarItems.indexOf("update") !== -1 && !$(toolbar).find(".e-save").parent("li").hasClass("e-disable"))
                            disableToolItems.push($(toolbar).find(".e-save").parent()[0]);
                    }
                    else if (editSettings.editMode.toLowerCase() == "batchediting" && proxy._batchEditStarts) {
                        if (toolbarItems.indexOf("cancel") !== -1 && $(toolbar).find(".e-cancel").parent("li").hasClass("e-disable"))
                            enabledToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
                        if (toolbarItems.indexOf("update") !== -1 && $(toolbar).find(".e-save").parent("li").hasClass("e-disable"))
                            enabledToolItems.push($(toolbar).find(".e-save").parent()[0]);
                    }
                }

                disableToolItems.forEach(function (element) {
                    $(toolbar).ejToolbar('disableItem', element);
                });
                enabledToolItems.forEach(function (element) {
                    $(toolbar).ejToolbar('enableItem', element);
                });
            }
            else {
                var toolbar = $("#" + proxy._id.replace("ejTreeGrid", "") + "_toolbarItems");
                var enabledToolItems = [],disableToolItems = [];
                if (model.readOnly == true) {
                    disableToolItems = [];
                    disableToolItems.push($(toolbar).find(".e-addnewitem").parent()[0]);
                    disableToolItems.push($(toolbar).find(".e-edititem").parent()[0]);
                    disableToolItems.push($(toolbar).find(".e-edititem").parent()[0]);
                }
                else {
                    if (editSettings.allowAdding && toolbarItems.indexOf("add") !== -1)
                        enabledToolItems.push($(toolbar).find(".e-addnewitem").parent()[0]);

                    if (editSettings.allowEditing && toolbarItems.indexOf("edit") !== -1 && model.allowSelection && editSettings.beginEditAction != "click") {
                        if (proxy.selectedRowIndex() != -1 && model.selectedItems.length == 1)
                            enabledToolItems.push($(toolbar).find(".e-edititem").parent()[0]);
                        else
                            disableToolItems.push($(toolbar).find(".e-edititem").parent()[0]);
                    }
                    if (editSettings.allowDeleting && toolbarItems.indexOf("delete") !== -1 && model.allowSelection) {
                        if (proxy.selectedRowIndex() != -1)
                            enabledToolItems.push($(toolbar).find(".e-deleteitem").parent()[0]);
                        else
                            disableToolItems.push($(toolbar).find(".e-deleteitem").parent()[0]);
                    }
                    if (editSettings.allowIndent && (toolbarItems.indexOf("indent") !== -1 || toolbarItems.indexOf("outdent") !== -1) && model.allowSelection) {
                        if (proxy.selectedRowIndex() == -1) {
                            disableToolItems.push($(toolbar).find(".e-indent").parent()[0]);
                            disableToolItems.push($(toolbar).find(".e-outdent").parent()[0]);
                        }
                    }
                    $(toolbar).ejToolbar('enableItem', enabledToolItems);

                  
                    if (toolbarItems.indexOf("cancel") !== -1)
                        disableToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
                    if (toolbarItems.indexOf("update") !== -1)
                        disableToolItems.push($(toolbar).find(".e-saveitem").parent()[0]);
                }
                disableToolItems.forEach(function (element) {
                    $(toolbar).ejToolbar('disableItem', element);

                });
            }
        },

        
        _checkIsEmptyRow: function (item) {
            var column = this.model.columns,
                itemIndex;
                    for (itemIndex = 0; itemIndex < column.length ; itemIndex++) {
                        var value = this._getCustomCellValue(item, column[itemIndex].field);
                        if (value === " " || ej.isNullOrUndefined(value) || value === "" || value === false)
                            continue;
                        return false;
                    }
            return true;
        },
        /*Method to trigger begin edit event*/   
        _beginEdit: function () {
            var proxy = this, args = {};
            proxy._trigger("beginEdit", args);
            proxy._isinBeginEdit = true;           
        },

        /*Method to save edited/added row*/
        _endEdit: function () {
            var proxy = this;

            proxy._isinBeginEdit = false;
            var index = this.selectedRowIndex();
            var tr = ej.TreeGrid.getRowByIndex(proxy, index);
            if (this.editFormValidate()) {
                if ($('tr').hasClass("e-addedrow") || $('tr').hasClass("e-rowedit"))
                    proxy.saveRow();
                else
                    proxy.saveCell();
            }
            else
                return true;

        },
        //Refresh the treegrid content after expandAll or collapseAll
        _refreshTreeGridOnExpandCollapseAll:function(args)
        {
            var proxy = this,
                model = this.model;
            if (model.enableVirtualization) {
                if (model.enableAltRow)
                    proxy.updateAltRow();
                proxy.sendDataRenderingRequest(args);
                if (model.allowTextWrap) {
                    if (proxy._$frozenTableContent.length > 0)
                        proxy._updateRowHeight();
                    proxy._setScrollTop(args.requestType);
                    proxy.updateHeight();
                    if (proxy.getScrollElement().hasClass("e-scroller")) {
                        proxy.getScrollElement().ejScroller("refresh");
                    }
                } else {
                    proxy._setScrollTop();
                    proxy.updateHeight();
                }
            }             
            else {
                if (model.allowPaging) {
                    var arg = {},
                        pagerModel;
                    arg.totalRecordsCount = this.getExpandedRecords(model.updatedRecords).length;
                    proxy.getPager().ejPager("option", arg).ejPager("refreshPager");
                    pagerModel = this.getPager().ejPager("model");
                    proxy._currentPage(pagerModel.currentPage);
                    arg.requestType = "paging";
                    proxy.processBindings(arg);
                }
                //If need refresh chart if record is added after searching or sorting
                if (proxy._isRefreshAddedRecord) {
                    proxy.processBindings();
                    this.renderRecords();
                    proxy._isRefreshAddedRecord = false;
                    return;
                }
                if (model.allowTextWrap && proxy._$frozenTableContent.length > 0) {
                    proxy._updateRowHeight();
                    proxy.getScrollElement().ejScroller("refresh");
                    var movableScrollTop = proxy.getScrollElement().ejScroller("model.scrollTop");
                    proxy._getFrozenScrollElement().scrollTop(movableScrollTop);
                } else {
                    proxy.updateHeight();
                }
                if (model.enableAltRow && model.currentViewData.length > 0)
                    ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);
            }
        },
        //expand or collapse all inner level child records on expandAll or collapseAll
        _expandCollapseInnerLevelRecord: function (record, expanded) {
            var proxy = this,
                length = record.childRecords.length,
                tempArgs = {};

            for (var count = 0; count < length; count++) {
               if (record.childRecords[count].hasChildRecords) {
                    tempArgs.data = record.childRecords[count];
                    tempArgs.expanded = expanded;
                    tempArgs.recordIndex = proxy.model.updatedRecords.indexOf(tempArgs.data);
                    tempArgs.requestType = ej.TreeGrid.Actions.ExpandCollapse;
                    if (!proxy._triggerExpandingCollapsingEvent(tempArgs)) {
                        proxy._expandCollapseInnerLevelRecord(record.childRecords[count], expanded);
                        if (record.childRecords[count].expanded != expanded) {
                            if (proxy._isLoadonDemand && !record.childRecords[count].isChildRetrieved) {
                                proxy._isInExpandCollapseAll = true;
                                tempArgs.requestType = ej.TreeGrid.Actions.ExpandCollapse;
                                var isNotExpanded = ej.TreeGrid.expandRowsonDemand(proxy, record.childRecords[count], tempArgs);
                                if (isNotExpanded) {
                                    proxy._allParentItems.splice(proxy._allParentItems.indexOf(record.childRecords[count]), 1);
                                    record.childRecords[count].expanded = false;
                                    continue;
                                }
                                else {
                                    return true;
                                }
                            }
                            else 
                                ej.TreeGrid.sendExpandCollapseRequest(proxy, tempArgs);
                        }
                    }
                }
            }
        },
        _expandParentLevelRecord: function (record, expanded) {
            var proxy = this,               
                tempArgs = {};
            var parentExpandState = proxy.getExpandStatus(record);
            if (!record.expanded || !parentExpandState) {
                tempArgs.data = record;
                tempArgs.expanded = expanded;
                tempArgs.recordIndex = proxy.model.updatedRecords.indexOf(record);
                if (!proxy._triggerExpandingCollapsingEvent(tempArgs))
                ej.TreeGrid.sendExpandCollapseRequest(proxy, tempArgs);
                if (tempArgs.data.parentItem) {
                    proxy._expandParentLevelRecord(tempArgs.data.parentItem, expanded);
                }
            }
        },
        // expand the specified level record
        expandAtLevel: function (index) {
            var proxy = this, model = proxy.model, args = {}, treegridRecord, parentRecords;
            if (proxy._isHierarchyFiltering)
                return
            if (model.allowPaging && model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root && model.enableVirtualization) {                
                proxy._expandCollapseSettings.atLevel = "expandAtLevel";
                proxy._expandCollapseSettings.level = index;
                proxy._expandCollapsePages = [];
                proxy._isInExpandCollapseAll = true;
                proxy._expandPageLevelData();
                args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
                if (model.flatRecords.length > 0)
                    proxy._refreshTreeGridOnExpandCollapseAll(args);
                proxy._isInExpandCollapseAll = false;
                proxy.updateHeight();
            }
            else {
                parentRecords = proxy._getParentRecords(proxy.model.updatedRecords, proxy);
                for (var count = 0; count < parentRecords.length; count++) {
                    treegridRecord = parentRecords[count];
                    if (treegridRecord.level == index) {
                        args.data = treegridRecord;
                        args.expanded = true;
                        args.recordIndex = model.updatedRecords.indexOf(treegridRecord);
                        if (proxy._isLoadonDemand && !treegridRecord.isChildRetrieved) {
                            args.recordLevel = index;
                            args.isExpandingParticularLevelParent = true;
                            args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
                            var isNotExpanded = ej.TreeGrid.expandRowsonDemand(proxy, treegridRecord, args);
                            if (isNotExpanded) {
                                proxy._allParentItems.splice(proxy._allParentItems.indexOf(treegridRecord), 1);
                                treegridRecord.expanded = false;
                                continue;
                            }
                            else {
                                return;
                            }
                        }
                        else {
                            if (!proxy._triggerExpandingCollapsingEvent(args))
                                ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        }
                        // expand its parent if it is collapsed.
                        if (treegridRecord.parentItem)
                            proxy._expandParentLevelRecord(treegridRecord.parentItem, args.expanded);

                        if (treegridRecord.hasChildRecords)
                            proxy._expandCollapseInnerLevelRecord(treegridRecord, false);
                    }
                }
                if (model.allowPaging && !model.enableVirtualization) {
                    if (!ej.isNullOrUndefined(proxy.getPager())) {
                        proxy.gotoPage();
                    }
                }
            }
        },
        // collapse the specified level record
        collapseAtLevel: function (index) {
            var proxy = this, args = {},
                model = proxy.model,
                treegridRecord,
                parentRecords;
            if (proxy._isHierarchyFiltering)
                return
            if (model.allowPaging && model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root && model.enableVirtualization) {
                proxy._expandCollapseSettings.atLevel = "collapseAtLevel";
                proxy._expandCollapseSettings.level = index;
                proxy._expandCollapsePages = [];
                proxy._isInExpandCollapseAll = true;
                proxy._collapsePageLevelData();
                args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
                if (model.flatRecords.length > 0)
                    proxy._refreshTreeGridOnExpandCollapseAll(args);
                proxy._isInExpandCollapseAll = false;
                proxy.updateHeight();
            }
            else {
                parentRecords = proxy._getParentRecords(model.updatedRecords, proxy);
                for (var count = 0; count < parentRecords.length; count++) {
                    treegridRecord = parentRecords[count];
                    if (treegridRecord.level == index && treegridRecord.expanded) {
                        var parentExpandState = proxy.getExpandStatus(treegridRecord);
                        if (parentExpandState) {
                            args.data = treegridRecord;
                            args.expanded = false;
                            args.recordIndex = model.updatedRecords.indexOf(treegridRecord);
                            if (!proxy._triggerExpandingCollapsingEvent(args))
                                ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        }
                    }
                }
                if (model.allowPaging && !model.enableVirtualization) {
                    if (!ej.isNullOrUndefined(proxy.getPager())) {
                        proxy.gotoPage();
                    }
                }
            }
        },
        //To collapse current page records alone
        _collapsePageLevelData: function () {
            var proxy = this, args = {},
                model = proxy.model,                
                level = proxy._expandCollapseSettings.level,                
                parentRecords = proxy._getParentRecords(proxy._updatedPageData, proxy),
                countLength = parentRecords.length,
                pageIndex = proxy._currentPage(),
                parentExpandState,
                treegridRecord;                                                    
            $.inArray(pageIndex, proxy._expandCollapsePages) == -1 && proxy._expandCollapsePages.push(pageIndex);
            for (var count = 0; count < countLength; count++) {
                treegridRecord = parentRecords[count];
                parentExpandState = proxy.getExpandStatus(treegridRecord);
                if (treegridRecord && treegridRecord.level == level) {                    
                    args.data = treegridRecord;
                    args.recordIndex = model.updatedRecords.indexOf(treegridRecord),
                    args.expanded = false;
                    if (!proxy._triggerExpandingCollapsingEvent(args)) {
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        if (!parentExpandState)
                            proxy._expandParentLevelRecord(treegridRecord.parentItem, true);
                    }
                }
            }
        },
        //To expand current page records alone
        _expandPageLevelData: function () {
            var proxy = this, args = {},
                model = proxy.model,                
                level = proxy._expandCollapseSettings.level,                
                parentRecords = proxy._getParentRecords(proxy._updatedPageData, proxy),
                countLength = parentRecords.length,
                pageIndex = proxy._currentPage(),
                treegridRecord;                                                    
            $.inArray(pageIndex, proxy._expandCollapsePages) == -1 && proxy._expandCollapsePages.push(pageIndex);            
            for (var count = 0; count < countLength; count++) {
                treegridRecord = parentRecords[count];
                if (treegridRecord.level == level) {
                    args.data = treegridRecord;
                    args.expanded = true;
                    args.recordIndex = model.updatedRecords.indexOf(treegridRecord);
                    if (!proxy._triggerExpandingCollapsingEvent(args)) {
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        // expand its parent if it is collapsed.
                        if (treegridRecord.parentItem)
                            proxy._expandParentLevelRecord(treegridRecord.parentItem, args.expanded);
                        if (treegridRecord.hasChildRecords)
                            proxy._expandCollapseInnerLevelRecord(treegridRecord, false);
                    }
                }
            }
        },
        // expand/collapse the specified record using its index
        expandCollapseRow: function (index) {
            var proxy = this, args = {}, treegridRecord, records;
            index = parseInt(index);
            records = proxy.model.allowPaging ? proxy.getUpdatedRecords() : proxy.model.updatedRecords;
            if (records.length > index && index >= 0) {
                treegridRecord = records[index];
                if (treegridRecord.hasChildRecords) {
                    args.data = treegridRecord;
                    if (treegridRecord.expanded)
                        args.expanded = false;
                    else
                        args.expanded = true;
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                }
            }
        },

        //Expand the child records from parent.
        expandAll: function () {

            var proxy = this,
                model = proxy.model,
                count = 0,
                treegridRecord,
                args = {};
            if (!proxy.editFormValidate()) return true;
            proxy._expandCollapseSettings.state = "expandAll";
            proxy._expandCollapseSettings.atLevel = "";
            proxy._expandCollapseSettings.level = 0;
            proxy._expandCollapsePages = [];
            if (proxy._isRowEdit)
                proxy.cancelRowEditCell();
            else if(proxy.model.isEdit)
                proxy.cancelEditCell();

            args.requestType = ej.TreeGrid.Actions.ExpandCollapse;            
            proxy._isInExpandCollapseAll = true;
            //loop excecute for iterate the GanttRecords preseent in the Gantt Control
            proxy._expandAllData();                        
            if (model.flatRecords.length > 0)
                proxy._refreshTreeGridOnExpandCollapseAll(args);            
            proxy._isInExpandCollapseAll = false;
            proxy.updateHeight();
        },

        _expandAllData: function () {
            var proxy = this, model = proxy.model,
                count = 0, treegridRecord, args = {};
            args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
            if (model.allowPaging && model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root && model.enableVirtualization) {
                var pageSize = model.pageSettings.pageSize,
                    pageIndex = proxy._currentPage(),
                    countLength = pageSize * pageIndex,
                    initCount = pageSize * (pageIndex - 1);
                proxy._expandCollapsePages.push(pageIndex);
                for (count = initCount; count < countLength; count++) {
                    treegridRecord =proxy._isHierarchyFiltering ? proxy._filterParentRecords[count] : proxy.model.parentRecords[count];
                    if (treegridRecord) {
                        args.data = treegridRecord;
                        args.recordIndex = model.updatedRecords.indexOf(treegridRecord);
                        args.expanded = true;
                        if (treegridRecord.hasChildRecords) {
                            if (!proxy._triggerExpandingCollapsingEvent(args)) {
                                var isServerCalled = proxy._expandCollapseInnerLevelRecord(treegridRecord, args.expanded);
                                if (isServerCalled)
                                    return;
                                if (treegridRecord.expanded != args.expanded)
                                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                            }
                        }
                    }
                }
            }
            else {
                var parentRecord = proxy._isHierarchyFiltering ? proxy._filterParentRecords : proxy.model.parentRecords;
                for (count = 0; count < parentRecord.length; count++) {
                    treegridRecord = parentRecord[count];
                    args.data = treegridRecord;
                    args.recordIndex = model.updatedRecords.indexOf(treegridRecord);
                    args.expanded = true;
                    if (treegridRecord.hasChildRecords) {
                        if (!proxy._triggerExpandingCollapsingEvent(args)) {
                            if (proxy._isLoadonDemand && !treegridRecord.isChildRetrieved) {
                                proxy._isInExpandCollapseAll = true;
                                var isNotExpanded = ej.TreeGrid.expandRowsonDemand(proxy, treegridRecord, args);
                                if (isNotExpanded) {
                                    proxy._allParentItems.splice(proxy._allParentItems.indexOf(treegridRecord), 1);
                                    treegridRecord.expanded = false;
                                    continue;
                                }
                                else {
                                    return;
                                }
                            }
                            else {
                                var isServerCalled = proxy._expandCollapseInnerLevelRecord(treegridRecord, args.expanded);
                                if (isServerCalled)
                                    return;
                                if (treegridRecord.expanded != args.expanded)
                                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                            }
                        }
                    }
                }
            }
        },
           

        _refreshToolBar: function () {
            var $toolbar = $("#" + this._id + "_toolbarItems");
            $toolbar.find("li").removeClass("e-hover");
        },

        //RENDER TOOL BAR
        _renderToolbar: function () {
            var proxy = this,
                model = proxy.model,
                element = proxy.element;
            if (!model.isFromGantt) {
                if (proxy.model.toolbarSettings.showToolbar) {
                    element.append(proxy._renderToolbarTemplate());
                }
                if (proxy.model.isResponsive) {
                    $("#" + proxy._id + "_toolbarItems").addClass("e-treegridadaptivetoolbar");
                }
            }
        },

        //RENDER TOOLBAR TEMPLATE
        _renderToolbarTemplate: function () {
            var proxy = this,
                model = proxy.model,
                $div = ej.buildTag("div.e-treegridtoolbar#" + proxy._id + "_toolbarItems", "", { 'height': '36px', 'border': '1px solid' }, { "data-unselectable": "on" });
            if(!ej.isNullOrUndefined(model.toolbarSettings.toolbarItems) && model.toolbarSettings.toolbarItems.length){
                var $ul = ej.buildTag("ul", "", {},{});
                proxy._renderLi($ul);
                $div.append($ul);
            }
            if (!ej.isNullOrUndefined(model.toolbarSettings.customToolbarItems) && model.toolbarSettings.customToolbarItems.length) {
                var $customUl = ej.buildTag("ul", "", {}, {});
                proxy._renderCustomLi($customUl);
                $div.append($customUl);
            }
            var helper = {};
            helper.click = proxy._toolBarClick;
            helper.cssClass = proxy.model.cssClass;
            helper.rtl = false;
            helper.itemSeparator = false;
            helper.itemLeave = proxy._toolbarItemLeave;
            helper.width = proxy._gridWidth;
            helper.fields = {
                id: "",
                tooltipText: "",
                imageUrl: "",
                text: "",
                imageAttributes: "",
                spriteCSS: "",
                htmlAttributes: "",
            };
            $div.ejToolbar(helper);
            $div.ejToolbar("disableItem", proxy._disabledToolItems);
            proxy._disabledToolItems = $();
            return $div;
        },        
        //to renderCustom Li
        _renderCustomLi: function ($ul) {
            var proxy = this,
            model = proxy.model,
            $li,$item,
            customToolbarItems = model.toolbarSettings.customToolbarItems;
            for (var i = 0; i < customToolbarItems.length; i++) {
                if (customToolbarItems[i]["text"] || customToolbarItems[i]["templateID"]) {                   
                        if (customToolbarItems[i]["text"]) {
                            var customToolbar = customToolbarItems[i]["tooltipText"] ? customToolbarItems[i]["tooltipText"] : customToolbarItems[i]["text"];
                            $li = ej.buildTag("li", "", {}, { id: proxy._id + "_" + customToolbarItems[i]["text"], title: customToolbar });
                            $item = ej.buildTag("a.e-toolbaricons e-icon e-treegridcustomtoolbaritem", "", {}).addClass(customToolbarItems[i]["text"]);
                            $li.addClass("e-treegridtoolbaritem");
                        }
                        else {                          
                            var customToolbar = customToolbarItems[i]["tooltipText"] ? customToolbarItems[i]["tooltipText"] : customToolbarItems[i]["templateID"].replace("#", "");
                            $li = ej.buildTag("li", "", {}, { id: proxy._id + "_" + customToolbarItems[i]["templateID"].replace("#", ""), title: customToolbar });
                            $item = $(customToolbarItems[i]["templateID"]).hide().html();
                }                
                $li.html($item);              
                $ul.append($li);
                }
            }
        },
        //Render list to display the tools in toolbar.
        _renderLi: function ($ul) {
            var proxy = this,
                toolBarItems = proxy.model.toolbarSettings.toolbarItems,
                length = toolBarItems.length,
                i = 0,
                $li;
            for (i; i < length; i++) {

                $li = ej.buildTag("li.e-treegridtoolbaritem", "", {}, {
                    id: proxy._id + "_" + toolBarItems[i]
                });             
                proxy._renderLiContent($li, toolBarItems[i]);
                $ul.append($li);
            }
        },

        //RENDER TOOLBAR ICONS
        _renderLiContent: function ($li, item) {
            var $a, $input, selectionMode = this.model.selectionSettings.selectionMode;
            var selectedRowIndex = this.selectedRowIndex();
            var editSetting = this.model.editSettings;
            switch (item) {

                case "add":
                    $a = ej.buildTag("a.e-addnew e-toolbaricons e-icon e-treegrid-add", "", {});
                    var addTitle = this._toolboxTooltipTexts["addTool"];
                    $li.attr("title", addTitle);
                    if (!this.model.editSettings.allowAdding)
                        this._disabledToolItems.push($li.get(0));
                    break;

                case "edit":
                    $a = ej.buildTag("a.e-edit e-toolbaricons e-icon e-treegrid-edit", "", {});
                    var editTitle = this._toolboxTooltipTexts["editTool"];
                    $li.attr("title", editTitle);
                    if (!this.model.editSettings.allowEditing || (selectedRowIndex == -1 || (selectedRowIndex != -1 && selectionMode == "cell")) || !this.model.allowSelection || this.model.editSettings.beginEditAction == "click") {
                        this._disabledToolItems.push($li.get(0));
                    }

                    break;

                case "delete":
                    var deleteTitle = this._toolboxTooltipTexts["deleteTool"];
                    $li.attr("title", deleteTitle);
                    $a = ej.buildTag("a.e-delete e-toolbaricons e-icon e-treegrid-delete", "", {});

                    if (!this.model.editSettings.allowDeleting || (selectedRowIndex == -1 || (selectedRowIndex != -1 && selectionMode == "cell")) || !this.model.allowSelection) {
                        this._disabledToolItems.push($li.get(0));
                    }

                    break;

                case "update":
                    var updateTitle = this._toolboxTooltipTexts["updateTool"];
                    $li.attr("title", updateTitle);
                    $a = ej.buildTag("a.e-save e-toolbaricons e-icon e-treegrid-save e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "cancel":
                    var cancelTitle = this._toolboxTooltipTexts["cancelTool"];
                    $li.attr("title", cancelTitle);
                    $a = ej.buildTag("a.e-cancel e-toolbaricons e-icon e-treegrid-cancel e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "expandAll":
                    var expandAllTitle = this._toolboxTooltipTexts["expandAllTool"];
                    $li.attr("title", expandAllTitle);
                    $input = ej.buildTag("a.e-expandall e-toolbaricons e-icon e-treegrid-expandall", "", {}, {});
                    $li.append($input);
                    break;

                case "collapseAll":
                    var collapseAllTitle = this._toolboxTooltipTexts["collapseAllTool"];
                    $li.attr("title", collapseAllTitle);
                    $input = ej.buildTag("a.e-collapseall e-toolbaricons e-icon e-treegrid-collapseall", "", {}, {});
                    $li.append($input);
                    break;

                case "pdfExport":
                    var pdfExportTitle = this._toolboxTooltipTexts["pdfExportTool"];
                    $li.attr("title", pdfExportTitle);
                    $input = ej.buildTag("a.e-pdfIcon e-toolbaricons e-icon", "", {}, {});
                    $li.append($input);
                    break;
                case "excelExport":
                    var excelExportTitle = this._toolboxTooltipTexts["excelExportTool"];
                    $li.attr("title", excelExportTitle);
                    $input = ej.buildTag("a.e-excelIcon e-toolbaricons e-icon", "", {}, {});
                    $li.append($input);
                    break;
                case "print":
                    var printTitle = this._toolboxTooltipTexts["printTool"];
                    $li.attr("title", printTitle);
                    $input = ej.buildTag("a.e-printIcon e-toolbaricons e-icon", "", {}, {});
                    $li.append($input);
                    break;
                case "search":
                    var searchTitle = this._toolboxTooltipTexts["searchTool"], proxy = this, value = proxy.model.searchSettings.key;
                    $a = ej.buildTag("a.e-searchitem e-toolbaricons e-disabletool e-icon e-searchfind", "", { 'position': 'absolute', 'padding': '4px', 'padding-left': '7px' });
                    var $div = ej.buildTag('div.e-filterdiv e-searchinputdiv', "", { 'display': 'inline-table', 'width': '83%' });
                    $input = ej.buildTag("input.e-ejinputtext e-searchinput", "", { 'width': '100%' }, { type: "text", placeholder: searchTitle, value:value });
                    var $span = ej.buildTag('span.e-searchcancel e-cancel e-icon e-hide e-spanstyle', "", { 'right': '1%', 'padding-top': '0px', 'top': '0px' });
                    $li.attr("title", searchTitle);
                    $li.addClass('e-search');
                    $div.append($input).append($span);
                    $li.append($div);
                    $li.css({ 'width': '172px' });
                    this.model.allowSearching = true;
                    $input.keydown($.proxy(this._keyDown, this)).focus(function () { proxy.element.find("span.e-searchcancel").removeClass("e-hide"); });
                    $a.click($.proxy(this._keyDown, this));                    
                    $div.focusout(function () {
                        if (proxy.element.find("input.e-searchinput").val() == "")
                            $("span.e-searchcancel").addClass("e-hide");
                    });                    
                    $span.click(function () {
                        proxy.element.find("input.e-searchinput").val("");
                        $(this).addClass("e-hide");
                        proxy.search("");
                    }).mouseover(function () { this.style.cursor = 'pointer'; });
                    break;
            }

            $li.append($a);
        },

        //update the toolbar items when dynamically update the editSettings property
        _updateToolbarItems: function () {
            var proxy = this,
                model = proxy.model,
                toolbarItems = model.toolbarSettings.toolbarItems,
                editSettings = model.editSettings,
                toolbar = $("#" + proxy._id + "_toolbarItems"),
                selectedRowIndex = proxy.selectedRowIndex(),
                editMode = editSettings.editMode,
                disableToolItems = [],
                enableToolItems = [];
                if (editMode.toLowerCase() == "cellediting" || editMode.toLowerCase() == "dialogediting") {
                    /* We can able perform rowEditing using toolbar, whether the editMode also in cellEditing.
                       So cancel the editing based on edit type.*/
                    if (proxy._isRowEdit)
                        proxy.cancelRowEditCell();
                    else if (model.isEdit)
                        proxy.cancelEditCell();
                }
                else if (editMode.toLowerCase() == "rowediting")
                    proxy.cancelRowEditCell();
                proxy._cancelSaveTools();
        },


        //#region TREEGRIDHEADER RENDERIGN

        //TREEGRID HEADER RENDER INITIALIZATION
        _renderAfterColumnInitialize:function(){
            var proxy = this;
            proxy.element.append(proxy._renderGridHeader());
        },

        //TO CHECK FILTER FORM ELEMENTS EDIT TYPE
        filterEditType: function (column, element, filterValue) {

            var proxy = this,
                model = proxy.model,
                columnField = column.field.split('.').join("");
               
            if (ej.isNullOrUndefined(column["filterEditType"]))
                column["filterEditType"] = column["editType"];

            switch (column["filterEditType"]) {
                case "stringedit":
                    element.html(ej.buildTag('input.e-ejinputtext e-filtertext e-field e-filterwidth', "", {}, { id: proxy._id + "_" + columnField + "_filterbarcell", name: column.field, value: filterValue }));
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", { "width": column.width - 15 }, { type: "text", id: proxy._id + "_" + columnField + "_filterbarcell", name: column.field, value: filterValue });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + column["filterEditType"] + ' e-field', "", {}, { type: "text", id: proxy._id + "_" + columnField + "_filterbarcell", name: column.field, value: filterValue });
                    element.append($datePicker);
                    break;
                case "booleanedit":
                case "dropdownedit":
                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist e-field', "", {}, { type: "text", id: proxy._id + "_" + columnField + "_filterbarcell", name: column.field, value: filterValue });
                    element.append($dropDownList);
                    break;
            }
        },


        //RENDER FILTER BAR ELEMENTS ACCORDING TO ITS TYPE
        _filteringElements: function (args) {
            var proxy = this,
                model=proxy.model,
                elementFocused = false,
                $formElement,
                form, length,
                $element,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat,
                formatVal,
            selectedItem = model.selectedItem;

            if (args.requestType == "filtering") {
                form = document.getElementById(proxy._id + "_filterBar" + args.fieldname.split('.').join(""));
            }
            
            column = customParams = ej.TreeGrid.getColumnByField(model.columns, args.fieldname);
            $formElement = $(form).find("input,select");
            length = $formElement.length;
            for (var i=0; i < length; i++) {
                $element = $formElement.eq(i);
                params = {};
                //inputWidth = column.width ? column.width - 2 : proxy.model.commonWidth - 2;
                var inputWidth = $element.closest("div").width()-2;
                if ($element.hasClass("e-numerictextbox")) {
                    width = inputWidth;
                    value = $element.val();
                    params.width = width;                    
                    params.showSpinButton = true;
                    params.cssClass = model.cssClass;
                    params.locale = model.locale;
                    params.height = 28;
                    params.focusIn = $.proxy(proxy._cancelEditState, proxy, $element);
                    //for parent editing 
                    var tempId = $element[0].id;
                    tempId=tempId.replace(proxy._id, '');
                    if (model.filterSettings.filterBarMode == "immediate")
                        params.change = $.proxy(proxy._filterBarHandler, proxy, $element);
                    else
                        params.change = null;
                    var numericObject = $element.data("ejNumericTextbox");
                    if (ej.isNullOrUndefined(numericObject)) {
                        if (value.length)
                            params.value = parseFloat(value);
                        else
                            params.value = null;
                    }
                    customParams = ej.TreeGrid.getColumnByField(proxy.model.columns, $element.prop("name"));
                    if (!ej.isNullOrUndefined(customParams) && !ej.isNullOrUndefined(customParams["editParams"]))
                        $.extend(params, customParams["editParams"]);
                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(proxy._id, ""));
                }
                else if ($element.hasClass("e-datepicker")) {
                    width = inputWidth;
                    params.width = width;                    
                    params.cssClass = model.cssClass;
                    params.dateFormat = model.dateFormat;
                    params.locale = model.locale;
                    params.height = 28;
                    params.focusIn = $.proxy(proxy._cancelEditState, proxy, $element);
                    var tempId = $element[0].id;
                    tempId = tempId.replace(proxy._id, '');
                    //To filter the records once the date seleted.
                    if (model.filterSettings.filterBarMode == "immediate")
                        params.select = $.proxy(proxy._filterBarHandler, proxy, $element);
                    else
                        params.select = null;
                    column = ej.TreeGrid.getColumnByField(proxy.model.columns, $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);

                    if (model.dateFormat.toLowerCase().indexOf("hh") != -1)
                        $element.ejDateTimePicker(params);
                    else
                        $element.ejDatePicker(params);
                    proxy._preventDoubleClick(column); //To prevent double click event
                }
                else if ($element.hasClass("e-datetimepicker")) {
                    width = inputWidth;
                    params = {
                        width: width,
                        rtl: model.rtl,
                        locale: model.locale,
                        cssClass: model.cssClass,
                        showButton: false,
                        focusIn : $.proxy(proxy._cancelEditState, proxy, $element),
                    };
                    params.height = 28;
                    if (model.filterSettings.filterBarMode == "immediate")
                        params.close = $.proxy(proxy._filterBarHandler, proxy, $element);
                    else
                        params.close = null;
                    column = ej.TreeGrid.getColumnByField(proxy.model.columns, $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);

                    $element.ejDateTimePicker(params);
                    proxy._preventDoubleClick(column); //To prevent double click event

                }
                else if ($element.hasClass("e-dropdownlist")) {
                    column = ej.TreeGrid.getColumnByField(model.columns, args.fieldname);
                    var text = (column.editParams && column.editParams.fields && column.editParams.fields.text) ? column.editParams.fields.text : "text";
                    var value = (column.editParams && column.editParams.fields && column.editParams.fields.value) ? column.editParams.fields.value : "value";
                    var clear, emptyString, dropData = [], blank, selectAll, dropDatatrue, dropDatafalse;
                    //Collecting the drop down data for drop down filter bar
                    if (column.dropdownData && column.dropdownData.length > 0)
                        dropData = $.extend(true, [], column.dropdownData);
                    else {
                        //Default drop down data for boolean editType columns
                        if (column.editType == "booleanedit") {
                            dropData = [];
                            dropDatatrue = {};
                            dropDatatrue[text] = proxy._trueText;
                            dropDatatrue[value] = "true";
                            dropData.push(dropDatatrue);
                            dropDatafalse = {};
                            dropDatafalse[text] = proxy._falseText;
                            dropDatafalse[value] = "false";
                            dropData.push(dropDatafalse);
                        }
                        else
                            dropData = proxy._dropDownArrayCollection(column, args.fieldname);  //Gathering dropdown data from datasource                       
                    }
                    //To append selectAll(clear) label in the drop down list                   
                    //clear = { text: proxy._dropDownListClearText, value: "" };
                    clear = {};
                    clear[text] = proxy._dropDownListClearText;
                    clear[value] = proxy._dropDownListClearText;
                    dropData.splice(0, 0, clear);
                    //To append (Blanks) label in the drop down list
                        blank = column.allowFilteringBlankContent == undefined ? true : column.allowFilteringBlankContent;
                    if (blank) {
                        //emptyString = { text: proxy._dropDownListBlanksText, value: proxy._dropDownListBlanksText };
                        emptyString = {};
                        emptyString[text] = proxy._dropDownListBlanksText;
                        emptyString[value] = proxy._dropDownListBlanksText;
                        dropData.push(emptyString);
                    }
                    var param = {}, inputValue = $element.val(), $dropDownObj, dropObject = $("#" + $element[0].id).data("ejDropDownList");
                    if (dropObject) {
                        $dropDownObj = $(dropObject.popupList);
                    }
                    
                    param.width = inputWidth;
                    param.showCheckbox = false;
                    param.dataSource = dropData;
                    param.height = 28;
                    param.popupShown = $.proxy(proxy._cancelEditState, proxy, $element);
                    param.fields = { text: text, value: value };
                    //To filter the records once the item has been selected.
                    if (model.filterSettings.filterBarMode == "immediate")
                        param.popupHide = $.proxy(proxy._filterBarHandler, proxy, $element);
                    else
                        param.popupHide = null;
                    $element.ejDropDownList(param);
                    //Apply italic font style for "(Clear Filter)" and "(Blanks)" label
                    if ($dropDownObj) {
                        $dropDownObj.find(".e-ul :first-child").find(".e-ddltxt").css("font-style", "italic");
                        if(blank)
                            $dropDownObj.find(".e-ul :last-child").find(".e-ddltxt").css("font-style", "italic");
                    }
                    if (inputValue == proxy._dropDownListClearText)
                        inputValue = "";
                    $element.ejDropDownList("selectItemByValue", inputValue);//To retain the selected value in dropdown    
                    proxy._preventDoubleClick(column); //To prevent double click event                 
                } else {
                    switch ($element.prop('tagName')) {
                        case "INPUT":
                            $element.width(inputWidth - 4);
                            if (column.filterEditType == "stringedit")
                                proxy._on($("#" + proxy._id + "_" + column.field.split('.').join("") + "_filterbarcell"), "focus", proxy._cancelEditState);
                            break;

                        case "SELECT":
                            $element.width(inputWidth).height(15);
                            break;
                    }
                }
                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                    if (!proxy._isEnterKeyPressed) {
                        elementFocused = true;
                    }
                }
            }
        },

        // Method for retriving drop down data to filterbar from data source
        _dropDownArrayCollection: function (column, fieldItems) {
            var proxy = this,
                model = proxy.model,
                fieldName = fieldItems,
                dropDownRecords = [],
                uniqueDropDownRecords = [],
                dropdownData = [],
                columnEditType = column.editType;

            dropDownRecords = $.map(model.flatRecords, function (record) {
                if (!ej.isNullOrUndefined(record.item[fieldName]))
                    return record.item[fieldName];
            });
            $.each(dropDownRecords, function (i, el) {
                el=$.trim(el); 
                if ($.inArray(el, uniqueDropDownRecords) === -1) uniqueDropDownRecords.push(el);
            });
            if (columnEditType != "numericedit")
                uniqueDropDownRecords.sort(proxy.stringCompare);
            else {
                uniqueDropDownRecords = uniqueDropDownRecords.sort(function (a, b) {
                    return a - b;
                });
            }
            $.each(uniqueDropDownRecords, function (i, val) {
                dropdownData.push({ text: val, value: val });
            });
            return dropdownData;
        },

        stringCompare: function (s1, s2) {
            var str1 = [], str2 = [];

            s1.replace(/(\d+)|(\D+)/g, function (_, $1, $2) { str1.push([$1 || Infinity, $2 || ""]) });
            s2.replace(/(\d+)|(\D+)/g, function (_, $1, $2) { str2.push([$1 || Infinity, $2 || ""]) });
    
            while (str1.length && str2.length) {
                var st1 = str1.shift();
                var st2 = str2.shift();
                var val = (st1[0] - st2[0]) || st1[1].localeCompare(st2[1]);
                if(val) return val;
            }

            return str1.length - str2.length;
        },
        // Method to prevent double click event during dropDown list and date picker filtering
        _preventDoubleClick: function (col) {
            var proxy = this,
                filterBoxTool = $("#" + proxy._id + "_" + col.field.split('.').join("") + "_filterbarcell_popup");
            if (filterBoxTool.length != 0) {
                $(filterBoxTool).on('click', function (e) {
                    if (ej.isTouchDevice())
                        proxy._off(proxy.element, "doubletap ", ".e-gridcontent", proxy._editdblClickHandler);
                    else
                        proxy._off(proxy.element, "dblclick ", ".e-gridcontent", proxy._editdblClickHandler);
                    setTimeout(function () {
                        if (ej.isTouchDevice())
                            proxy._on(proxy.element, "doubletap ", ".e-gridcontent", proxy._editdblClickHandler);
                        else
                            proxy._on(proxy.element, "dblclick ", ".e-gridcontent", proxy._editdblClickHandler);
                    }, 300);
                });
            }
        },
        // METHOD TO RENDER FILDERING ELEMENTS
        _renderFiltering: function (columns, showDetailColumn) {
            var proxy = this, model = this.model,
            totColumns = model.columns,
            filteredColumns = model.filterSettings.filteredColumns;
            var $tr = ej.buildTag('tr.e-filterbar', "", {}, {});
            if (proxy.model.allowFiltering) {
                $.each(totColumns, function (i, column) {
                    if (ej.isNullOrUndefined(column["allowFiltering"]))
                        column["allowFiltering"] = true;
                });
                var filterEnabled = false;
                //To check whether atleast any one of the column's allowFiltering has been enabled
                $.each(totColumns, function (i, column) {
                    if (column.allowFiltering == true) {
                        filterEnabled = true;
                        return false;
                    }
                });
                if (filterEnabled) {
                    for (var column = 0; column < columns.length; column++) {

                    var $th = ej.buildTag('th.e-filterbarcell .e-editValue', "", {}, {});
                    columns[column]["visible"] === false && $th.addClass("e-hide");
                    var $div = ej.buildTag('div.e-filterdiv', "", {}, { id: proxy._id + "_filterBar" + columns[column].field.split('.').join("") });
                        $tr.append($th.append($div));
                        //Condition to check whether allow filtering enabled for the individual column before rendering filtering elements
                        if (columns[column].allowFiltering == false)
                            continue;
                        else {
                            var filterValue = "";
                            if (filteredColumns.length > 0) {
                                for (var fcolumn = 0; fcolumn < filteredColumns.length; fcolumn++) {
                                    if (columns[column].field == filteredColumns[fcolumn].field) {
                                        filterValue = filteredColumns[fcolumn].value;
                                    }
                                }
                            }
                       
                            this.filterEditType(columns[column], $div, filterValue);
                        var $span = ej.buildTag('span.e-cancel e-icon e-hide e-spanstyle');
                        $div.append($span);
                    }                   
                }          
                }
            }
            /* add details column to last in column header */
            if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn && showDetailColumn) {
                $tr.append(ej.buildTag('th.e-filterbarcell e-detailheadercell', '<div style="width:' + proxy._detailsColumnWidth + 'px;"></div>'));
            }
            return $tr;
        },


        _getIndentCol: function () {
            return ej.buildTag("col", "", { width: this._detailsColumnWidth + "px" });
        },

        //METHOD FOR RENDERING TREEGRID HEADER
        _renderGridHeader: function () {

            var proxy = this,
                model = proxy.model,
                doc = document, temp, $frozenDiv, $movableDiv,
                $innerDiv = ej.buildTag('div.e-gridheadercontainer', "", { "overflow": "hidden" }, { "data-unselectable": "on" }),
                $div = model.isFromGantt ? ej.buildTag('div.e-gridheader#' + proxy._id + "e-gridheader", "", {
                    "border-left-style": "solid",
                    "border-left-width": "1px"
                }, "") :
                    ej.buildTag('div.e-gridheader#' + proxy._id + "e-gridheader", "",
                {
                    "border-top-style": "solid",
                    "border-left-style": "solid",
                    "border-right-style": "solid",
                    "border-top-width": "1px",
                    "border-left-width": "1px",
                    "border-right-width": "1px",
                    "width": proxy._gridWidth,
                }, "");
            if (model.allowFiltering == true && model.filterSettings.filterType == ej.TreeGrid.FilterType.FilterBar)
                $innerDiv.addClass("e-headercontent-material-filterbar");
            $innerDiv.addClass("e-headercontent");
            proxy.setGridHeaderContent($div);
            proxy._$gridHeaderContainer = $innerDiv;
            proxy._hiddenColumns = [];
            proxy._visibleColumns = [];
            proxy.columnsWidthCollection = [];
            proxy._removePrevHeaderTemplates();
            if (this._frozenColumnsLength > 0) {
                $innerDiv.addClass("e-frozen-header-container");
                $frozenDiv = ej.buildTag("div.e-frozenheaderdiv#e-frozenheaderdiv" + proxy._id, this._renderGridHeaderInternalDesign(model.columns.slice(0, this._frozenColumnsLength), false, true));
                if (model.allowFiltering == true && model.filterSettings.filterType == ej.TreeGrid.FilterType.FilterBar)
                    $frozenDiv.addClass("e-filterbar-header-material");
                if (model.showStackedHeader && model.stackedHeaderRows.length>0)
                    $frozenDiv.addClass("e-stacked-header-material");
                $movableDiv = ej.buildTag("div.e-movableheader#e-movableheader" + proxy._id, ej.buildTag("div.e-movableheaderdiv#e-movableheaderdiv" + proxy._id, this._renderGridHeaderInternalDesign(this.model.columns.slice(this._frozenColumnsLength), true)));
                $innerDiv.append($frozenDiv).append($movableDiv);
            } else {
                $innerDiv.append(this._renderGridHeaderInternalDesign(this.model.columns, true));
                $innerDiv.removeClass("e-frozen-header-container");
            }
            $div.html($innerDiv);
            this.setGridHeaderTable(this.getHeaderContent().find(".e-table"));
            return $div;
        },
        _removePrevHeaderTemplates: function () {
            $("[id^='" + this._id + "'][type='text/x-treegridheadertemplate']").remove();
            this.headerAngularTemplate && (this.headerAngularTemplate= null);
        },
        getPager: function () {
            return this._gridPager;
        },

        setTreeGridPager: function (value) {
            this._gridPager = value;
        },

        _renderGridPager: function () {
            var proxy = this,
                $div = $(document.createElement('div')),
                pagerModel = {}, pageSettings = proxy.model.pageSettings;
            $div.attr({ "id": this._id + "Pager" });
            pageSettings.change = $.proxy(proxy._pagerClickHandler, proxy);
            if (ej.isNullOrUndefined(pageSettings.totalRecordsCount))
                pageSettings.totalRecordsCount = proxy._isLoadonDemand ? proxy._pageRecordCountOnDemand : pageSettings.totalRecordsCount;
            $.extend(pagerModel, pageSettings);
            pagerModel.locale = proxy.model.locale;
            delete pagerModel.template;
            if (proxy._currentPage() < 0)
                proxy._currentPage(1);
            pagerModel.currentPage = this._prevPageNo = proxy._currentPage();
            pagerModel.masterObject = proxy;
            pagerModel.cssClass = proxy.model.cssClass;
            proxy.setTreeGridPager($div);
            $div.ejPager(pagerModel);
            $div.ejPager("refreshPager");
            pagerModel = $div.ejPager("model");
            pageSettings.totalPages = pagerModel.totalPages;
            if (proxy._currentPage() !== pagerModel.currentPage)
                proxy._currentPage(pagerModel.currentPage);
            //Create custom Paging Template
            if (!ej.isNullOrUndefined(pageSettings.template) && pageSettings.template.length > 0) {
                $div.children().remove();                
                var $customDiv = ej.buildTag('div.e-pagercontainer'),
                $template = this._createTemplateElement(pageSettings, true);
                $customDiv.append($template.innerHTML);
                $div.append($customDiv);
            }
            return $div;
        },
		// Tempalte to render header checkbox selection
        _addHeaderCheckboxTemplate: function () {
            var model = this.model, template,
                flatRecordsCount = model.flatRecords.length,
                selectedItemsCount = model.selectedItems.length;

            if (flatRecordsCount > 0 && flatRecordsCount == selectedItemsCount) {
                template = '<div><span class="e-chkbox-wrap e-widget" style="margin-left:5px;padding-right:10px;color:#c8c8c8;">' +
                        '<div class="e-chkbox-small"><span class="e-chk-act">' +
                        '<span class="e-checkbox e-checkmark e-chk-image e-icon e-headerCheckbox"' +
                'id =' + this._id + '_headerCheckbox></span></span></div></span></div>';
            }
            else if (flatRecordsCount > selectedItemsCount && selectedItemsCount > 0) {
                template = '<div><span class="e-chkbox-wrap e-widget" style="margin-left:5px;padding-right:10px;color:#c8c8c8;">' +
                        '<div class="e-chkbox-small"><span class="e-chk-indeter">' +
                        '<span class="e-checkbox e-stop e-chk-image e-icon e-headerCheckbox"' +
                'id =' + this._id + '_headerCheckbox></span></span></div></span></div>';
            }
            else {
                template = '<div><span class="e-chkbox-wrap e-widget" style="margin-left:5px;padding-right:10px;color:#c8c8c8;">' +
                        '<div class="e-chkbox-small"><span class="e-chk-inact">' +
                        '<span class="e-checkbox e-chk-image e-icon e-headerCheckbox"' +
                'id =' + this._id + '_headerCheckbox></span></span></div></span></div>';
            }
            return template;
        },
        _createStackedRow: function (stackedHeaderRow, frozenHeader) {
            var $tr = ej.buildTag('tr.e-columnheader e-stackedHeaderRow'),
                sHeader = [], sCss = [], tAl = [], tp = [], proxy = this,
                model = proxy.model, columnLength = model.columns.length,
                column, sColumn = stackedHeaderRow.stackedHeaderColumns, _column, className,
                colsPanList = [], frzCol = model.scrollSettings.frozenColumns, spanC, colSpan, j,
                frozenColspanList = [], forzenColumn = 0, index = 0, frzHideCol = 0, c, $th, $div;
            for (var c = 0; c < columnLength; c++) {
                column = model.columns[c];
                if (column.visible != false || (column == model.columns[model.treeColumnIndex] && column.visible == false)) {
                    var headerText = '', cssClass = '', txtAlign = '', ttp = "";
                    for (var col = 0; col < sColumn.length; col++) {
                        _column = Array.isArray(sColumn[col].column) ? sColumn[col].column : $.map(sColumn[col].column.split(","), $.trim);
                        className = "e-row" + $.inArray(stackedHeaderRow, model.stackedHeaderRows) + "-column" + col;
                        if ($.inArray(column.field, _column) != -1) {
                            headerText = sColumn[col].headerText;
                            cssClass = sColumn[col]["cssClass"];
                            txtAlign = sColumn[col].textAlign;
                            ttp = sColumn[col]["tooltip"] ? " e-stackedtooltip " + className : '';
                        }
                    }
                    sHeader.push(headerText);
                    sCss.push(cssClass);
                    tAl.push(txtAlign);
                    tp.push(ttp);
                }
            }
            for (var i = 0; i < sHeader.length; i++) {
                colSpan = 1;
                for (j = i + 1; j < sHeader.length; j++) {
                    if (sHeader[i] == sHeader[j]) {
                        colSpan++;
                    }
                    else
                        break;
                }
                colsPanList.push({ spanCount: colSpan, headerText: sHeader[i], cssClass: sCss[i], txtAlign: tAl[i], tooltip: tp[i] });
                i += colSpan - 1;
            }
            if (frzCol > 0) {              
                for (var i = 0; i < columnLength; i++) {
                    col = model.columns[i];
                    if (i < frzCol && col.visible == false)
                        frzHideCol++;
                }
                forzenColumn = frzCol - frzHideCol;
                while (forzenColumn > 0) {
                    spanC = colsPanList[index].spanCount;
                    if (colsPanList[index].spanCount < forzenColumn) {
                        frozenColspanList.push(colsPanList[index])
                        if (!frozenHeader)
                            colsPanList.splice(index, 1);
                        else
                            index++;
                    }
                    else if (colsPanList[index].spanCount > forzenColumn) {
                        colsPanList[index].spanCount = colsPanList[index].spanCount - forzenColumn
                        if (frozenHeader)
                            frozenColspanList.push({ spanCount: forzenColumn, headerText: colsPanList[index].headerText });
                    }
                    else {
                        frozenColspanList.push(colsPanList[index])
                        if (!frozenHeader)
                            colsPanList.splice(index, 1);
                    }
                    forzenColumn -= spanC;
                }
                if (frozenHeader)
                    colsPanList = frozenColspanList
            }
            else {
               
            }
         
            for (var c = 0; c < colsPanList.length; c++) {
                $th = ej.buildTag('th.e-headercell e-stackedHeaderCell', "", {}, { 'colspan': colsPanList[c].spanCount });
                $div = ej.buildTag('div.e-headercelldiv  e-default' + colsPanList[c].tooltip, colsPanList[c].headerText, {"textAlign": colsPanList[c].txtAlign}, {});
                $th.append($div);
                $tr.append($th);
                if (colsPanList[c]["cssClass"] != undefined)
                    $th.addClass(colsPanList[c]["cssClass"]);
                if (model.allowColumnResize || model.allowColumnReordering)
                    $th.addClass("e-resizer");
            }
            if (model.detailsTemplate && model.showDetailsRow && model.showDetailsRowInfoColumn && frozenHeader!= true)
                $tr.append(ej.buildTag('th.e-headercell e-detailheadercell', '<div></div>'));
            return $tr;
        },

        //Get Header element by field name
        getHeaderCell: function (field) {
            var model = this.model,
                columnIndex = model.columns.indexOf(this.getColumnByField(field)),
                headerTd = null;
            if (!ej.isNullOrUndefined(columnIndex) && columnIndex != -1)
                headerTd = this.getHeaderTable().find("thead tr:not('.e-stackedHeaderRow')").
                find(".e-headercell").not(".e-detailheadercell").eq(columnIndex);
            return headerTd;
        },

        //creating a template element for angularjs template
        _createHeaderTemplateElement: function (column) {

            var idText = (column.field ? column.field: "") + $.inArray(column, this.model.columns) + "_HeaderTemplate",
                property = "headerTemplateID",
                template = column[property],
                quickReg = /^#([\w-]*)/, scriptReg = /^<script/i, scripEle;

            var match = quickReg.exec(template),
                options = {
                    name: "SCRIPT",
                    type: "text/x-treegridheadertemplate",
                    text: template,
                    id: (this._id + idText).replace(/(\\|[^0-9A-z-_])/g, "")
                };

            if (match && match[1])
                scripEle = document.getElementById(match[1]);
            else {
                if (scriptReg.test(template))
                    scripEle = $(template).get(0);
                else
                    scripEle = ej.buildTag(options.name, options.text).get(0);
            }
            scripEle.id = scripEle.id || options.id;
            scripEle.type = scripEle.type || options.type;
            $("#" + scripEle.id).remove();
            $("body").append(scripEle);
            return scripEle;
        },

        _renderGridHeaderInternalDesign: function (columns, showDetailColumn, frozenHeader)
        {
            var proxy = this,
             model = proxy.model,
             doc = document,
             $table = ej.buildTag('table.e-table e-zerospace'),
             $thead = ej.buildTag('thead'),
             $tbody = ej.buildTag('tbody.e-hide'),
             $columnHeader = ej.buildTag('tr.e-columnheader'),
             $colGroup = $(doc.createElement('colgroup')),
             $rowBody = $(doc.createElement('tr')),
             columnCount = 0,
             length = columns.length,
             $headerCell,
             bodyCell,
             $headerCellDiv, 
             col, $filterHeader,
             $columnFilter, stackedHeaderLength = model.stackedHeaderRows.length;

            //CHECK SORTING ENABLED OR NOT
            if (model.allowSorting || model.showColumnChooser || model.allowColumnReordering) {
                $columnHeader.css({ 'cursor': 'pointer' });
            }

            if (model.headerTextOverflow === "wrap")
                proxy.element.addClass("e-wrap");
            else
                proxy.element.removeClass("e-wrap");

            if (model.showStackedHeader) {
                for (var index = 0; index < stackedHeaderLength; index++) {
                    var $tr = proxy._createStackedRow(model.stackedHeaderRows[index], frozenHeader);
                    $thead.append($tr);
                }
            }

            for (columnCount; columnCount < length; columnCount++) {
                if (columns[columnCount]) {
                    var headerTextAlign = columns[columnCount].headerTextAlign ? columns[columnCount].headerTextAlign
                                                           : (columns[columnCount].textAlign ? columns[columnCount].textAlign : ej.TextAlign.Left);
                    // set padding right 35px for displaying column menu icon
                    if ((model.showColumnChooser || (!model.showColumnChooser && model.allowFiltering && (model.filterSettings.filterType == "menu" || model.filterSettings.filterType == "excel"))) && columns[columnCount].field != "checkboxState") {
                        if (!ej.isNullOrUndefined(columns[columnCount].allowFiltering) && !columns[columnCount].allowFiltering && !proxy.model.showColumnChooser) {
                            $headerCell = ej.buildTag('th.e-headercell', "", {}, {});
                        } else {
                            $headerCell = ej.buildTag('th.e-headercell', "", {}, {});
                            $headerCell.addClass("e-column-menu-padding");
                        }
                    }
                    else {
                        $headerCell = ej.buildTag('th.e-headercell', "", {}, {});
                    }
                    if (model.allowFiltering && model.filterSettings.filterType == ej.TreeGrid.FilterType.FilterBar) {
                        $headerCell.addClass("e-filterbar-material");
                    }
                    bodyCell = doc.createElement('td');

                    $headerCellDiv = ej.buildTag(
                        'div.e-headercelldiv', "",
                        { "text-align": headerTextAlign },
                        { 'data-ej-mappingname': columns[columnCount]["field"], 'data-unselectable': "on" }
                    );
                    if (columns[columnCount]["clipMode"] && columns[columnCount]["clipMode"] == ej.TreeGrid.ClipMode.Ellipsis) {
                        $headerCellDiv.addClass("e-treegridtrimcell");
                    }
                    // add checkbox that name of column have in proxy._checkboxColumnName
                    if (proxy._checkboxSelection && model.selectionSettings.enableSelectAll) {
                        if (proxy._checkboxColumnName == columns[columnCount]["field"])
                            $headerCellDiv.append(proxy._addHeaderCheckboxTemplate());                        
                    }
                    if (columns[columnCount]["field"] == "checkboxState")
                        $headerCell.css({ cursor: "default" });

                    var headerText = ej.buildTag('span',columns[columnCount]["headerText"] == undefined ?
                        columns[columnCount]["headerText"] = columns[columnCount]["field"] :
                        columns[columnCount]["headerText"]);

                    if (model.headerTextOverflow == "wrap") {
                        $headerCellDiv.addClass("e-textwrapheaderdiv");
                    }
                    $headerCellDiv.append(headerText);
                    $headerCell.append($headerCellDiv);
                    // Add column menu icon in header columns
                    if (proxy.model.showColumnChooser && !columns[columnCount].nonColumn) {
                        var $columnChooser = ej.buildTag('div.e-columnmenu-icon e-icon e-columnicon', "", { "cursor": "pointer" });
                        if (model.headerTextOverflow == "wrap")
                            $columnChooser.addClass("e-textwrapcolumnicon");
                        $columnChooser.data("isClicked", false);
                        $headerCell.append($columnChooser);
                    } else if ((!model.showColumnChooser && model.allowFiltering && (model.filterSettings.filterType == "menu" || model.filterSettings.filterType == "excel") && (ej.isNullOrUndefined(columns[columnCount].allowFiltering) || columns[columnCount].allowFiltering)) && columns[columnCount].field != "checkboxState") {
                        if (proxy._filteredColumnValueID.indexOf(columns[columnCount].field) != -1)
                            $columnFilter = ej.buildTag('div.e-column-filtered-icon e-icon e-filtericon', "", { "cursor": "pointer" });
                        else
                            $columnFilter = ej.buildTag('div.e-column-filter-icon e-icon e-filtericon', "", { "cursor": "pointer" });
                        if (proxy.model.filterSettings.filteredColumns) {
                            for (var x = 0; x < proxy.model.filterSettings.filteredColumns.length; x++) {
                                if (proxy.model.filterSettings.filteredColumns[x].field == columns[columnCount].field && proxy.model.filterSettings.filteredColumns[x].value.toString() != "")
                                    $columnFilter = ej.buildTag('div.e-column-filtered-icon e-icon e-filtericon', "", { "cursor": "pointer" });
                            }
                        }
                        $columnFilter.data("isClicked", false);
                        $headerCell.append($columnFilter);
                    }
                    
                    col = doc.createElement('col');

                    $rowBody.append(bodyCell);
                    $columnHeader.append($headerCell);
                    $colGroup.append(col);


                    //CHECK VISIBLE PROPERTY OF THE COLUMN
                    if (columns[columnCount]["visible"] === false && columns[columnCount] != model.columns[model.treeColumnIndex]) {
                        $headerCell.addClass("e-hide") && $(col).css("display", "none") &&
                        proxy._hiddenColumns.push(columns[columnCount].headerText);

                    } else {
                        proxy._visibleColumns.push(columns[columnCount].headerText);
                        columns[columnCount]["visible"] = true;
                    }

                    if (columns[columnCount]["allowSorting"] === false) {
                        $headerCell.css({ 'cursor': 'normal' });
                    }

                    if (!ej.isNullOrUndefined(columns[columnCount]["headerTemplateID"])) {
                        $headerCellDiv.html(proxy._renderEjTemplate("#" + this._createHeaderTemplateElement(columns[columnCount]).id, columns[columnCount], 0, columns[columnCount], "treegridheadertemplate")).parent().addClass("e-headertemplate");
                    }
                    if (columns[columnCount]["allowCellSelection"] == undefined) {
                        columns[columnCount]["allowCellSelection"] = true;
                    }
                    if (columns[columnCount]["showInColumnChooser"] == undefined) {
                        columns[columnCount]["showInColumnChooser"] = true;
                    }
                    if (columns[columnCount]["width"] == undefined && model.commonWidth !== undefined) {
                        proxy.columnsWidthCollection.push(model.commonWidth);
                        columns[columnCount]["width"] = model.commonWidth;
                    } else {
                        proxy.columnsWidthCollection.push(parseFloat(columns[columnCount]["width"]));
                        columns[columnCount]["width"] = parseFloat(columns[columnCount]["width"]);
                    }

                    proxy._fieldColumnNames[columns[columnCount].headerText] = columns[columnCount].field;
                    proxy._headerColumnNames[columns[columnCount].field] = columns[columnCount].headerText;
                }
            }
            //Updating the column properties to Gantt's model.columns also
            if (model.isFromGantt)
                proxy.updateToGanttColumns();

            /* add detail column to last*/
            if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn && showDetailColumn) {
                $columnHeader.append(ej.buildTag('th.e-headercell e-detailheadercell', '<div style="width:' + proxy._detailsColumnWidth + 'px;"></div>', {}, {}));
                $colGroup.append(this._getIndentCol());
            }

            // Condition to check whether filtering enabled to render filter elements.
            if (model.allowFiltering && !model.isFromGantt && model.filterSettings.filterType == "filterbar") {
                $filterHeader = this._renderFiltering(columns, showDetailColumn);
            }            
            $thead.append($columnHeader);
            if ($filterHeader)
                $thead.append($filterHeader);
            $table.append($colGroup).append($thead);
            return $table;
        },

        //#endregion

        //#region TREEGRIDCONTENT RENDERING
        //TREEGRID CONTENT RENDER INITIALIZATION
        _initGridRender: function () {

            var proxy = this;
            proxy._addInitTemplate();
            proxy._renderGrid();

            //CHECK COLUMN RESIZE IS ENABLED OR NOT AND INITIALIZE THE RESIZER
            if (proxy.model.allowColumnResize) {
                proxy._resizer = new ej.gridFeatures.gridResize(proxy);
            }

        },
        //Create a summary row.
        _summaryRow: function () {
            var proxy = this, model = proxy.model;
            if (model.showSummaryRow) {
                proxy._createSummaryRow();
                var refreshArgs = {};
                refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.sendDataRenderingRequest(refreshArgs);
                proxy.updateCollapsedRecordCount();
                proxy._updateWrapRowHeights("header");
                proxy.updateHeight();
                if (model.allowTextWrap || model.headerTextOverflow == "wrap") {
                    proxy._updateGridContentHeight();
                }
            }
            if (model.showTotalSummary) {
                var summaryLength = $("#" + proxy._id + "-footersummaryrow").length;
                proxy._createTotalSummaryRow();
                if (summaryLength == 0)
                    proxy._viewPortHeight = proxy._viewPortHeight - proxy._$totalSummaryRowContainer.outerHeight();
                if (model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns && !model.isFromGantt)
                    this._updateTableWidth();
                this._updateHeaderScrollLeft();
            }
        },


        //ADD GRIDCONTENT TO THE TREEGRID
        _renderGrid: function () {
            
            var proxy = this;
            proxy._renderGridContent().insertAfter(proxy.element.children(".e-gridheader"));
            

        },
        _renderFrozenSummary: function ()
        {
            this._$footerContainer.find("#e-frozenfooterdiv" + this._id + " .e-table colgroup").replaceWith(this._getMetaColGroup()[0]);
            this._$footerContainer.find("#e-movablefooterdiv" + this._id + " .e-table colgroup").replaceWith(this._getMetaColGroup()[1]);
            this._$footerContainer.find("#e-frozenfooterdiv" + this._id + " .e-table tbody").html($.render[this._id + "_JSONFrozenTemplate"](this._footerSummaryRecord));
            this._$footerContainer.find("#e-movablefooterdiv" + this._id + " .e-table tbody").html($.render[this._id + "_Template"](this._footerSummaryRecord));
        },

        _renderSummaryByFrozenDesign: function () {
            var $div = $(document.createElement('div')), col = this._getMetaColGroup().find("col"), colgroups = {};
            colgroups["colgroup1"] = $div.append(ej.buildTag("colgroup").append(col.splice(0, this._frozenColumnsLength))).html();
            colgroups["colgroup2"] = $div.html(ej.buildTag("colgroup").append(col)).html();
            if (!$.templates[this._id + "_FrozenSummaryTemplate"])
                this.addFrozenSummaryTemplate();
            return $.render[this._id + "_FrozenSummaryTemplate"](colgroups);
           
        },
        _spliceSummaryRows: function (updatedRecords) {
            var proxy = this,
                model = proxy.model, records=[];
            var nonSummaryRowChildRecords = updatedRecords.filter(function (data) {
                if (!data.isSummaryRow)
                    records.push(data);
            });
            return records;
        },
        addFrozenSummaryTemplate: function () {
            var template = "<div class='e-frozenfooterdiv' id='e-frozenfooterdiv" + this._id +"'>"
            + "<table class='e-table e-zerospace' style='width:100%' id='" + this._id + "frozensummarye-table'>{{:colgroup1}}<tbody>"
            + "</tbody></table></div>"
            + "<div class='e-movablefooter' id='e-movablefooter" + this._id + "'><div class='e-movablefooterdiv' id='e-movablefooterdiv" + this._id + "'><table class='e-table e-zerospace' id='" + this._id + "movablesummarye-table'>{{:colgroup2}}<tbody>"
            + "</tbody></table></div></div>", templates = {};
            templates[this._id + "_FrozenSummaryTemplate"] = template;
            $.templates(templates);
        },

        //collapse the Summary record
        expandCollapseTotalSummary: function (expanded) {
            var proxy = this,
                model = proxy.model;            
            if (model.showTotalSummary && model.collapsibleTotalSummary) {
                if (proxy._footerSummaryRecord[0].summaryExpanded == expanded) return;
                var args = {};
                args.expanded = expanded;
                args.requestType = expanded ? "summaryExpanding" : "summaryCollapsing";
                args.data = proxy._footerSummaryRecord[0];
                args.columnName = model.columns[this._summaryColumnIndex].headerText;
                args.cellValue = !ej.isNullOrUndefined(args.data) ? args.data[model.columns[this._summaryColumnIndex].field] : "";
                args.isFromUserAction = proxy._isInManualExpandCollapse;
                var $firstSummaryRow = $(proxy.element).find(".e-firstfootersummaryrow"),
                 $collapsedRows = $(proxy.element).find(".e-footersummaryrow");
                if (!expanded) {
                    proxy._trigger("collapsing", args);
                    if (args.cancel) return;
                    $collapsedRows.css({ 'display': 'none' });
                    $firstSummaryRow.css({ 'display': 'table-row' });
                    $firstSummaryRow.find(".e-summarytreegridexpand").removeClass('e-summarytreegridexpand').addClass('e-summarytreegridcollapse');
                    proxy._footerSummaryRecord[0].summaryExpanded = false;
                    var tempArgs = {};                 
                    tempArgs.expanded = expanded;
                    tempArgs.data = proxy._footerSummaryRecord[0];
                    tempArgs.columnName = model.columns[this._summaryColumnIndex].headerText;
                    tempArgs.cellValue = !ej.isNullOrUndefined(args.data) ? args.data[model.columns[this._summaryColumnIndex].field] : "";
                    tempArgs.requestType = "summaryCollapsed";
                    tempArgs.isFromUserAction = proxy._isInManualExpandCollapse;
                    proxy._trigger("collapsed", tempArgs);
                }
                else {
                    proxy._trigger("expanding", args)
                    if (args.cancel) return;
                    $collapsedRows.css({ 'display': 'table-row' });
                    $firstSummaryRow.find(".e-summarytreegridcollapse").removeClass('e-summarytreegridcollapse').addClass('e-summarytreegridexpand');
                    proxy._footerSummaryRecord[0].summaryExpanded = true;
                    var tempArgs = {};                    
                    tempArgs.expanded = expanded;
                    tempArgs.data = proxy._footerSummaryRecord[0];
                    tempArgs.columnName = model.columns[this._summaryColumnIndex].headerText;
                    tempArgs.cellValue = !ej.isNullOrUndefined(args.data) ? args.data[model.columns[this._summaryColumnIndex].field] : "";
                    tempArgs.requestType = "summaryExpanded";
                    tempArgs.isFromUserAction = proxy._isInManualExpandCollapse;
                    proxy._trigger("expanded", tempArgs)
                }
                proxy._$totalSummaryRowContainer.ejScroller({ height: expanded ? model.rowHeight : model.totalSummaryHeight });
                proxy._updateGridContentHeight();
                proxy._updateSummaryHeight();
            }
        },

        //Add a total summary row after treegrid content.
        _createTotalSummaryRow: function () {
            var proxy = this, $summaryDiv, $table, model = proxy.model, flatRecords = model.flatRecords,
                summaryRows = model.summaryRows, summaryRowLength = summaryRows.length,
                updatedRecords = model.updatedRecords, parentRecords = model.parentRecords,
                columns = model.columns, $tbody = ej.buildTag('tbody');
            model.totalSummaryHeight = model.totalSummaryHeight < model.rowHeight ? model.rowHeight : model.totalSummaryHeight;
            proxy._footerSummaryRecord = [];
            proxy._footerSummaryRows = [];
            if (model.collapsibleTotalSummary && $("#" + proxy._id + "-footersummaryrow").length != 0 && $("#" + proxy._id + "-footersummaryrow").find(".e-summarytreegridcollapse").length != 0)
                proxy._persistCollapseState = true;
            $("#" + proxy._id + "-footersummaryrow").remove();
            $summaryDiv = ej.buildTag('div.e-footersummaryrowdiv#' + proxy._id + '-footersummaryrow', '',
                {
                    "overflow": "hidden",
                    "cursor": "default"
                });
            var $footerContainer = ej.buildTag('div.e-footercontainer#e-footercontainer' + proxy._id, '',
               {
                   "overflow": "hidden"
               });
            proxy._flatChildRecords = proxy._spliceSummaryRows(model.updatedRecords);

            for (var summaryRowIndex = 1; summaryRowIndex <= summaryRowLength; summaryRowIndex++) {
                var item = {};
                for (var columnLength = 0; columnLength < columns.length; columnLength++) {
                    item[columns[columnLength].field] = "";
                }
                item = proxy._createSummaryItem(item, summaryRows[summaryRowIndex - 1]);
                var record = proxy._createRecord(item, 0);
                if (summaryRowIndex == 1) {
                    record.isFirstFooterSummary = true;
                    if (model.collapsibleTotalSummary) {
                        record.summaryExpanded = true;
                        }
                }
                record.footerSummaryRowRecord = true;
                record.index = summaryRowIndex;
                proxy._footerSummaryRecord.push(record);
                //GridRow = $.render[this._id + "_Template"](record);
                //proxy._footerSummaryRows.push(GridRow);
                //$tbody.append(GridRow);
            }
            proxy._$footerContainer = $footerContainer;
            if (this._frozenColumnsLength > 0) {
                $footerContainer.html(this._renderSummaryByFrozenDesign());
                this._renderFrozenSummary();
            } else {
                var $table = ej.buildTag('table.e-table e-zerospace#' + proxy._id + "summarye-table", "",
                { top: "0px" });
                $table.append(proxy.getContentTable().find('colgroup').clone()).append($tbody);
                $tbody.html($.render[this._id + "_Template"](proxy._footerSummaryRecord));
                $footerContainer.append($table);
            }
            proxy._$totalSummaryRowContainer = $summaryDiv;
            proxy._$footertableContent = $footerContainer.find(".e-table");
            $summaryDiv.append($footerContainer);
            //Always display after TotalSummaryRow
            if (model.allowPaging) {
                proxy.element.append($("#" + this._id + "Pager").before($summaryDiv));
            }
            else
                proxy.element.append($summaryDiv);

            proxy._flatChildRecords = [];
            this._setWidthToFooters();
            if (this._$totalSummaryRowContainer.height() <= model.totalSummaryHeight) {
                if (proxy.isVScroll())
                    proxy._$totalSummaryRowContainer.addClass("e-scrollcss");
                model.totalSummaryHeight = this._$totalSummaryRowContainer.height();
            }
            else {
                proxy._$totalSummaryRowContainer.ejScroller({ height: model.sizeSettings.height == "auto" ? 0 : model.totalSummaryHeight });
            }
            if (model.sizeSettings.height == "auto")
                $("#" + proxy._id + "-footersummaryrow").width(proxy._gridWidth - 2);
            proxy._updateSummaryTitleColumn();           
            if (proxy._persistCollapseState == true) {
                proxy.expandCollapseTotalSummary(false);
                proxy._persistCollapseState = false;
            }
            if (model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns) {
                this._updateTableWidth();
            }
        },
        _getFooterRows: function ()
        {
            var gridFooterRows = $();
            if (this.model.showTotalSummary && this._$footertableContent) {
                gridFooterRows = this._$footertableContent.get(0).rows;
                if (this._frozenColumnsLength > 0)
                    gridFooterRows = [gridFooterRows, this._$footertableContent.get(1).rows];
            }
            return $(gridFooterRows);
        },
        //update footer container width for frozen columns
        _setWidthToFooters: function () {
            var width = this._gridWidth;
            if (this._frozenColumnsLength > 0) {
                var frozenWidth = this._getFrozenColumnWidth();
                this._$totalSummaryRowContainer.find("#e-frozenfooterdiv" + this._id).width(width).next().css("margin-left", frozenWidth + "px");
                this._$totalSummaryRowContainer.find("#e-frozenfooterdiv" + this._id).outerWidth(frozenWidth)
                    .end().find("#e-movablefooterdiv" + this._id).css("width", "100%");
            }
        },
        _excludeSummaryRows: function () {

            var $gridRows = $(), model = this.model;
            if (this.getTreeGridRows() && this.getTreeGridRows().length > 0) {
                    if (this._frozenColumnsLength > 0)
                        $gridRows = [$(this.getTreeGridRows()[0]).not(".e-summaryrow"), $(this.getTreeGridRows()[1]).not(".e-summaryrow")];
                    else
                        $gridRows = $(this.getTreeGridRows()).not(".e-summaryrow");
            }
            return $gridRows;
        },

        //Add a summary row for every parent item in Tree grid.
        _createSummaryRow: function (args) {
            var proxy = this, model = proxy.model, parentRecords,
                recordIndex,
                item = {},
                docsummaryRow,
                totalChildRecordsLength,
                summaryRows = model.summaryRows,
                summaryRowLength = summaryRows.length,
                columns = model.columns;
            if (args && (args.requestType == "dragAndDrop" || args.requestType == "addNewRow" ||
                 args.requestType == "sorting")) {
                parentRecords = proxy._getParentRecords(model.updatedRecords, proxy);
                //Remove the summaryRow records from the childRecords collection of parentRecords
                if (args.requestType == "dragAndDrop") {
                    for (var i = 0; i < parentRecords.length; i++) {
                        var nonSummaryRowChildRecords = [];
                        nonSummaryRowChildRecords = parentRecords[i].childRecords.filter(function (data) {
                            if (!data.isSummaryRow)
                                return true;

                        });
                        parentRecords[i].childRecords = nonSummaryRowChildRecords.slice();
                    }
                }
            }
            else
                parentRecords = proxy._isHierarchyFiltering ? proxy._findFilteredParentRecords(proxy._flatFilteredRecords) : proxy._parentRecords;
            var length = parentRecords.length;
            if (args && ej.isNullOrUndefined(args.filterCollection))
                args.filterCollection = [];
            if (model.enableCollapseAll && model.enableVirtualization && !args)
                model.updatedRecords = model.flatRecords.slice();
            model.updatedRecords = proxy._spliceSummaryRows(model.updatedRecords); 
            model.flatRecords = proxy._spliceSummaryRows(model.flatRecords);
            model.summaryRowRecords = [];
            //$(proxy.element).find(".e-summaryrow").remove();                       
            //this._$gridRows = proxy._excludeSummaryRows();
            proxy._flatChildRecords = [];
            for (var parentIndex = 0; parentIndex < length; parentIndex++) {
                var parentRecord = parentRecords[parentIndex];
                totalChildRecordsLength = proxy._getChildRecordsLength(parentRecord, model.updatedRecords);              
                if (totalChildRecordsLength == 0)
                    continue;
                for (var summaryRowIndex = 1; summaryRowIndex <= summaryRowLength; summaryRowIndex++) {
                    for (var columnLength = 0; columnLength < columns.length; columnLength++) {
                        item[columns[columnLength].field] = "";
                    }
                    item = proxy._createSummaryItem(item, summaryRows[summaryRowIndex - 1], parentRecord);
                    var currentIndex = model.updatedRecords.indexOf(parentRecord) + totalChildRecordsLength + summaryRowIndex,
                        summaryRecord = proxy._createRecord(item, currentIndex);
                    if (proxy._isHierarchyFiltering && model.enableVirtualization)
                        summaryRecord.isDisplay = true;
                    proxy._isSummaryRow = true;
                    summaryRecord.level = parentRecord.level + 1;
                    summaryRecord.parentItem = parentRecord;
                    summaryRecord.isSummaryRow = true;
                    summaryRecord.index = model.flatRecords.length + 1;
                    if (model.enableVirtualization) {
                        if (proxy._isHierarchyFiltering)
                            model.updatedRecords = proxy._updateFilterParent;
                        model.updatedRecords.splice(currentIndex, 0, summaryRecord);
                            if (!args || (args && args.filterCollection))
                                model.summaryRowRecords.push(model.updatedRecords[currentIndex]);
                    }
                    else {
                        model.updatedRecords.splice(currentIndex, 0, summaryRecord);
                        model.summaryRowRecords.push(model.updatedRecords[currentIndex]);
                    }
                    currentIndex = model.flatRecords.indexOf(parentRecord) + totalChildRecordsLength + summaryRowIndex;
                    model.flatRecords.splice(currentIndex, 0, summaryRecord);                    
                    parentRecord.childRecords.push(summaryRecord);
                    item = {};
                    if (summaryRowIndex == summaryRowLength) {
                        summaryRecord.isLastSummary = true;
                    }
                }
                proxy._flatChildRecords = [];
            }
            proxy._isSummaryRow = false;
            proxy._summaryRowsCount = parentRecords.length;
            if (model.enableVirtualization) {
                model.updatedRecords = proxy.getExpandedRecords(model.updatedRecords);
                if (model.enableAltRow)
                    proxy.updateAltRow();
            }
            this._updateCurrentViewData();
        },
        //Get all inner parent records for Virtualization is true
        _getInnerParentRecords: function (records, proxy, parentRecords) {
            var recordLength = records.childRecords.length;

            for (var count = 0; count < records.childRecords.length; count++) {
                
                if (records.childRecords[count].hasChildRecords) {
                    if (parentRecords.indexOf(records.childRecords[count]) == -1)
                        parentRecords.push(records.childRecords[count]);
                    proxy._getInnerParentRecords(records.childRecords[count], proxy, parentRecords);
                }
            }           
        },

        //Get all parent records from updated records.
        _getParentRecords: function (updatedRecords, proxy, rootParent) {
            var recordLength = updatedRecords.length, record, parentRecords = [];
            for (var length = 0; length < recordLength; length++) {
                record = updatedRecords[length];
                if (rootParent) {
                    if (ej.isNullOrUndefined(record.parentItem))
                        parentRecords.push(record);
                }
                else if (record.childRecords && record.childRecords.length > 0) {
                    if ((proxy.model.enableVirtualization) || (proxy.model.allowPaging && proxy.model.showSummaryRow)) {
                        if (parentRecords.indexOf(record) == -1)
                            parentRecords.push(record);
                        proxy._getInnerParentRecords(record, proxy, parentRecords);
                    }
                    else
                        parentRecords.push(record);
                }
                else if (proxy._isLoadonDemand && record.hasChildRecords && parentRecords.indexOf(record) == -1)
                    parentRecords.push(record);
            }
            return parentRecords;
        },
        //To get all child and grand child of one parent record when the datasource is in hierarchy form.
        _getChildRecordsLength: function (parentRecord, updatedRecords) {
            var proxy = this,
                recordLength = updatedRecords.length, record;
            for (var length = 0; length < recordLength; length++) {
                record = updatedRecords[length];
                if (record.summaryRow)
                    continue;
                if (parentRecord == record.parentItem) {
                    proxy._flatChildRecords.push(record);
                    if (record.hasChildRecords)
                        proxy._getChildRecordsLength(record, updatedRecords);
                }
            }
            return proxy._flatChildRecords.length;
        },

        //Assign the values for item properties to create a summary record.
        _createSummaryItem: function (item, targetSummaryRow, parentRecord) {
            var proxy = this,
                summaryColumns = targetSummaryRow.summaryColumns,
                summaryColumnsLength = summaryColumns.length;
            for (var columnLength = 0; columnLength < summaryColumnsLength; columnLength++) {
                var targetSummaryColumn = summaryColumns[columnLength],
                    prefix = targetSummaryColumn.prefix,
                    suffix = targetSummaryColumn.suffix,
                    displayColumn = targetSummaryColumn.displayColumn;

                if (ej.isNullOrUndefined(displayColumn))
                    displayColumn = targetSummaryColumn.dataMember;
                if (ej.isNullOrUndefined(prefix))
                    prefix = "";
                if (ej.isNullOrUndefined(suffix))
                    suffix = "";
                item.summaryColumn = targetSummaryColumn;
                item.summaryRow = targetSummaryRow;

                for (var key in item) {
                    if (key == displayColumn) {
                        if (item[key])
                            item[key] = item[key] + " " + prefix + proxy._getSummaryValues(targetSummaryColumn, proxy._flatChildRecords) + suffix;
                        else
                            item[key] = prefix + proxy._getSummaryValues(targetSummaryColumn, proxy._flatChildRecords) + suffix;
                    }
                }
            }
            return item;
        },

        //Calculate the summary values.
        _getSummaryValues: function (summaryColumn, summaryData) {
            var dataMember = summaryColumn.dataMember, format = summaryColumn.format,
                jsonData = summaryData;
            var dbMgr, $value= "";
            if (jsonData instanceof ej.DataManager) {
                dbMgr = jsonData;
                jsonData = jsonData.dataSource.json;
            } else
                dbMgr = ej.DataManager(jsonData);
            switch (summaryColumn.summaryType) {
                case ej.TreeGrid.SummaryType.Sum:
                    $value = ej.sum(jsonData, dataMember);
                    break;
                case ej.TreeGrid.SummaryType.Average:
                    $value = ej.avg(jsonData, dataMember);
                    $value = (isNaN($value) || ej.isNullOrUndefined($value)) ? "" : $value;
                    break;
                case ej.TreeGrid.SummaryType.Maximum:
                    var obj = ej.max(jsonData, dataMember);
                    $value = parseInt(this._getCustomCellValue(obj, summaryColumn.dataMember));
                    $value = (isNaN($value) || ej.isNullOrUndefined($value)) ? "" : $value;
                    break;
                case ej.TreeGrid.SummaryType.Minimum:
					var filteringData = jsonData.filter(function (data) {
                        if (data[dataMember] !== "")
                            return true;
                    });
                    var obj = ej.min(filteringData, dataMember);
                    $value = parseInt(this._getCustomCellValue(obj, summaryColumn.dataMember));
                    $value = (isNaN($value) || ej.isNullOrUndefined($value)) ? "" : $value;
                    break;
                case ej.TreeGrid.SummaryType.MaximumDate:
                    $value = this._getMaxDate(jsonData, dataMember);
                    $value = ej.isNullOrUndefined($value) ? "" : $value;
                    break;
                case ej.TreeGrid.SummaryType.MinimumDate:
                    $value = this._getMinDate(jsonData, dataMember);
                    $value = ej.isNullOrUndefined($value) ? "" : $value;
                    break;
                case ej.TreeGrid.SummaryType.Count:
                    $value = jsonData.length;
                    break;
                case ej.TreeGrid.SummaryType.TrueCount:
                    var predicate = ej.Predicate(summaryColumn.dataMember, "equal", true);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.TreeGrid.SummaryType.FalseCount:
                    var predicate = ej.Predicate(summaryColumn.dataMember, "equal", false);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.TreeGrid.SummaryType.Custom:
                    var fn = summaryColumn.customSummaryValue;
                    if (fn) {
                        if (typeof fn === "string")
                            fn = ej.util.getObject(fn, window);
                        if ($.isFunction(fn))
                            $value = fn.call(this, summaryColumn, jsonData);
                        else
                            $value = summaryColumn.customSummaryValue;
                    }
                    break;
            }
            if (!ej.isNullOrUndefined(format))
                $value = this.formatting(format, $value, this.model.locale);
            return $value;
        },
        // To Get Maximum Date of Particular Datamember in JSON data.
        _getMaxDate: function (jsonData, dataMember) {
            var max = 0;
            var maxDate = null;
            for (var i = 0; i < jsonData.length; i++) {
                var item = jsonData[i],
                    date = new Date(item[dataMember]),
                    time = date.getTime();
                if (time > max) {
                    max = time;
                    maxDate = item[dataMember];
                }
            }

            return maxDate;
        },
        // To Get Minimum Date of Particular Datamember in JSON data.
        _getMinDate: function (jsonData, dataMember) {
            if (jsonData.length == 0) return null;
            var firstMinDate = new Date(jsonData[0][dataMember]),
                min = firstMinDate.getTime();
            var minDate = jsonData[0][dataMember];
            for (var i = 0; i < jsonData.length; i++) {
                var item = jsonData[i],
                    date = new Date(item[dataMember]),
                    time = date.getTime();
                if (time < min) {
                    min = time;
                    minDate = item[dataMember];
                }
            }

            return minDate;
        },
        /*Create new summary records and re render all records*/
        _createAndRenderSummaryRecords: function (args)
        {
            var proxy = this;
            proxy._createSummaryRow(args);
            var refreshArgs = {};
            refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
            proxy.sendDataRenderingRequest(refreshArgs);
            proxy.updateCollapsedRecordCount();
            proxy.updateHeight();
        },
        _mergeRecords: function () {
            var proxy=this, batchChanges = proxy.batchChanges,editedChanges=batchChanges.changed, addedChanges=batchChanges.added, mergeCollection=[];
            for (var i = 0; i < editedChanges.length; i++)
            {
                if (!mergeCollection[editedChanges[i]]) {
                    mergeCollection.push(editedChanges[i]);
                }
            }
            for (var i = 0; i < addedChanges.length; i++) {
                if (!mergeCollection[addedChanges[i]]) {
                    mergeCollection.push(addedChanges[i]);
                }
            }
            return mergeCollection;
        },
       
        //Update the summary after add, edit or delete the record.
        _updateSummaryRow: function (args) {
            var proxy = this, model = proxy.model, summaryRow,
                editedRecord, editedRecordIndex, updatedSummaryRecords = [],
                _pRecords = [], cellvalue,
            updatedRecords = proxy.getUpdatedRecords();

            if (args && args.requestType == "dragAndDrop") {
                args.rowIndex = updatedRecords.indexOf(proxy._droppedRecord);
                if(args.rowIndex == -1) return;
            }
            if (args && args.requestType == "delete") {
                proxy._updateAfterDeleteRecord(args);
                return;
            }

            if (args && args.editType == "celledit") {
                if (args.columnObject.editType == "numericedit") {
                    cellvalue = parseInt(args.value);
					args.value = isNaN(cellvalue) ? "" : cellvalue;
                    args.data[args.columnName] = args.value;
                }
                args.currentValue = args.value;
                var rowIndex = proxy.getIndexByRow(args.rowElement);
                var data = model.enableVirtualization ? model.currentViewData[rowIndex] : updatedRecords[rowIndex];
                args.rowIndex = updatedRecords.indexOf(data);
            }
            if ((args.currentValue != args.previousValue) || args.requestType == "dragAndDrop"
                || args.requestType == "addNewRow" || proxy._batchEditCancel) {
                editedRecord = args.requestType == "addNewRow" ? model.updatedRecords[args.rowIndex] : updatedRecords[args.rowIndex];
                if (proxy._batchEditCancel)
                    editedRecord = args.record;
                if (editedRecord.parentItem)
                    _pRecords = proxy._getpRecords(editedRecord, proxy);
                if (ej.isNullOrUndefined(_pRecords))
                    return;
                for (var parentLength = 0; parentLength < _pRecords.length; parentLength++) {
                    var summaryRowRecords = model.summaryRowRecords, recordindex, columnIndex,
                        prevUpdatedCols = [], emptyDisplayCol;
                    proxy._getChildRecordsLength(_pRecords[parentLength], updatedRecords);
                    $.each(summaryRowRecords, function (index, record) {
                        summaryRow = record.summaryRow;
                        if (record.parentItem === _pRecords[parentLength]) {
                            prevUpdatedCols = [];
                            var summaryColumnLength = summaryRow.summaryColumns.length;
                            for (var length = 0; length < summaryColumnLength; length++) {
                                var summaryColumn = summaryRow.summaryColumns[length],
                                    prefix = summaryColumn.prefix,
                                    suffix = summaryColumn.suffix;
                                if (ej.isNullOrUndefined(prefix))
                                    prefix = "";
                                if (ej.isNullOrUndefined(suffix))
                                    suffix = "";
                                if (args.editType == "rowedit") {
                                    args.columnName = summaryColumn.dataMember;
                                    var columnList = model.columns,
                                        dataMemberColumnIndex = proxy.getColumnIndexByField(args.columnName);
                                    //To convert empty string to integer while adding the new row.    
                                    if (model.columns[dataMemberColumnIndex].editType == "numericedit")
                                        editedRecord[args.columnName] = parseInt(editedRecord[args.columnName]);
                                }
                                var displayColumn = summaryColumn.displayColumn;
                                if (!displayColumn)
                                    displayColumn = summaryColumn.dataMember;
                                recordindex = model.currentViewData.indexOf(record);
                                columnIndex = proxy.getColumnIndexByField(displayColumn);
                                args.data = record;
                                if (columnIndex != -1) {
                                if (proxy._frozenColumnsLength > 0) {
                                    if (proxy._frozenColumnsLength > columnIndex)
                                        args.rowElement = $(proxy.getRows()[0][recordindex]);
                                    else {
                                        args.rowElement = $(proxy.getRows()[1][recordindex]);
                                        columnIndex -= proxy._frozenColumnsLength;
                                    }
                                } else {
                                    args.rowElement = $(proxy.getRows()[recordindex]);
                                }
                                args.cellElement = args.rowElement.find(".e-rowcell").eq(columnIndex);

                                var value = proxy._getSummaryValues(summaryColumn, proxy._flatChildRecords);
                                var summaryCellValue = prefix + value + suffix;
                                emptyDisplayCol = prevUpdatedCols.indexOf(displayColumn) != -1 ? false : true;
                                if(!emptyDisplayCol) {
                                    $(args.cellElement).append(' ' + summaryCellValue);
                                    record[displayColumn] = record[displayColumn] + " " + summaryCellValue;
                                }
                                else {
                                    if ($(args.cellElement).find(".e-summarytitle").length != 0) {
                                        var $titleClone = $(args.cellElement).find(".e-summarytitle").clone();
                                        $(args.cellElement).empty().html(summaryCellValue).prepend($titleClone);
                                    }
                                    else
                                        $(args.cellElement).empty().html(summaryCellValue);
                                    record[displayColumn] = summaryCellValue;
                                    prevUpdatedCols.push(displayColumn);
                                }                                                
                                updatedSummaryRecords.push(record);
                                proxy._updateWrapRowHeights("total,header", "virtualRefresh", recordindex, "savecell");
                                }
                            }
                        }
                    });
                    proxy._flatChildRecords = [];
                    proxy._summaryRowCount = _pRecords.length;
                }
            }
        },
        //Update the Total summary after add, edit or delete the record.
        _updateTotalSummaryRow: function (args) {
            if (args.requestType == "dragAndDrop" || args.isDragAndDropDelete)
                return;
            var proxy = this, model = proxy.model, summaryRows = model.summaryRows, summaryColumns, summaryRow,
                summaryColumnsLength, record, summaryRowLength = summaryRows.length, displayColumn, recordindex, columnIndex;
            for (var length = 0; length < summaryRowLength; length++) {
                summaryRow = summaryRows[length];
                summaryColumns = summaryRow.summaryColumns;
                summaryColumnsLength = summaryColumns.length;
                record = proxy._footerSummaryRecord[length];
                var prevUpdatedCols = [], emptyDisplayCol;
                for (var columnLength = 0; columnLength < summaryColumnsLength; columnLength++) {
                    var summaryColumn = summaryColumns[columnLength],
                        prefix = summaryColumn.prefix,
                        suffix = summaryColumn.suffix;
                    if (ej.isNullOrUndefined(prefix))
                        prefix = "";
                    if (ej.isNullOrUndefined(suffix))
                        suffix = "";
                    if (args.editType == "rowedit" || args.editType == "celledit")
                        args.columnName = summaryColumn.dataMember;
                    if (!displayColumn)
                        displayColumn = summaryColumn.dataMember;
                    if (summaryColumn.dataMember == args.columnName || args.requestType == "delete"
                        || args.requestType == "filtering" || args.requestType == "addNewRow" || args.requestType == "searching") {
                        displayColumn = summaryColumn.displayColumn;
                        recordindex = proxy._footerSummaryRecord.indexOf(record);
                        columnIndex = proxy.getColumnIndexByField(displayColumn);
                        args.data = record;
                        if (columnIndex != -1) {
                        if (this._frozenColumnsLength > 0) {
                            if (this._frozenColumnsLength > columnIndex)
                                args.rowElement = this._getFooterRows()[0][recordindex];
                            else {
                                args.rowElement = this._getFooterRows()[1][recordindex];
                                columnIndex -= this._frozenColumnsLength;
                            }
                        } else {
                            args.rowElement = this._getFooterRows()[recordindex];
                        }
                        args.cellElement = args.rowElement.childNodes[columnIndex];
                        var footerUpdatedRecords = proxy._spliceSummaryRows(model.updatedRecords),
                        value = proxy._getSummaryValues(summaryColumn, footerUpdatedRecords);
                        var summaryCellValue = prefix + value + suffix;                        
                        emptyDisplayCol = prevUpdatedCols.indexOf(displayColumn) !=-1 ? false : true;
                        if (!emptyDisplayCol) {
                            if ($(args.cellElement).find(".e-summarytitle").length > 0 || $(args.cellElement).find(".e-icon").length > 0) {
                                $(args.cellElement).find(".e-summarycell").append(' ' + summaryCellValue);
                            }
                            else {
                                $(args.cellElement).append(' ' + summaryCellValue);
                            }
                            record[displayColumn] = record[displayColumn] + " " + summaryCellValue;
                        }
                        else {
                            if ($(args.cellElement).find(".e-summarytitle").length != 0 ||
                                    $(args.cellElement).find(".e-icon").length > 0) {
                                var $titleClone = $(args.cellElement).find(".e-summarytitle, .e-icon").clone();
                                $(args.cellElement).empty().html(summaryCellValue).prepend($titleClone);
                            }
                            else
                                $(args.cellElement).empty().html(summaryCellValue);
                            prevUpdatedCols.push(displayColumn);
                            record[displayColumn] = summaryCellValue;
                        }
                    }
                        }
                }
            }
            if (model.allowTextWrap) {
                proxy._updateTotalSummaryHeight();
            }
        },
        //Update summary row after deleted the record.
        _updateAfterDeleteRecord: function (args) {
            var proxy = this, model = proxy.model, summaryRow,
                editedRecord, editedRecordIndex, updatedSummaryRecords = [],
                _pRecords, emptyDisplayCol, prevUpdatedCols,
                deletedRecord = args.data;
            if (deletedRecord.parentItem)
                _pRecords = proxy._getpRecords(deletedRecord, proxy);
            if (ej.isNullOrUndefined(_pRecords))
                return;
            for (var parentLength = 0; parentLength < _pRecords.length; parentLength++) {
                var summaryRowRecords = model.summaryRowRecords,
                    childRecords = _pRecords[parentLength].childRecords;
                if (childRecords.length == 0)
                    continue;
                proxy._getChildRecordsLength(_pRecords[parentLength], model.updatedRecords);
                $.each(summaryRowRecords, function (index, record) {
                    summaryRow = record.summaryRow;
                    if (record.parentItem === _pRecords[parentLength]) {
                        prevUpdatedCols = [];
                        var summaryColumnLength = summaryRow.summaryColumns.length;
                        for (var length = 0; length < summaryColumnLength; length++) {
                            var summaryColumn = summaryRow.summaryColumns[length],
                                displayColumn = summaryColumn.displayColumn,
                            recordindex = model.currentViewData.indexOf(record),
                            columnIndex = proxy.getColumnIndexByField(displayColumn);
                            var prefix = summaryColumn.prefix,
                             suffix = summaryColumn.suffix;
                            if (ej.isNullOrUndefined(prefix))
                                prefix = "";
                            if (ej.isNullOrUndefined(suffix))
                                suffix = "";
                            args.data = record;
                            if (proxy._frozenColumnsLength > 0) {
                                if (proxy._frozenColumnsLength > columnIndex)
                                    args.tr = $(proxy.getRows()[0][recordindex]);
                                else {
                                    args.tr = $(proxy.getRows()[1][recordindex]);
                                    columnIndex -= proxy._frozenColumnsLength;
                                }
                            } else {
                                args.tr = $(proxy.getRows()[recordindex]);
                            }
                            args.tr = $(proxy.getRows()[recordindex]);
                            args.cellElement = args.tr.find(".e-rowcell").eq(columnIndex);
                            var value = proxy._getSummaryValues(summaryColumn, proxy._flatChildRecords);
                            var summaryCellValue = prefix + value + suffix;
                            emptyDisplayCol = prevUpdatedCols.indexOf(displayColumn) != -1 ? false : true;
                            if (!emptyDisplayCol) {
                                $(args.cellElement).append(' ' + summaryCellValue);
                                record[displayColumn] = record[displayColumn] + " " + summaryCellValue;
                            }
                            else {
                                if ($(args.cellElement).find(".e-summarytitle").length != 0) {
                                    var $titleClone = $(args.cellElement).find(".e-summarytitle").clone();
                                    $(args.cellElement).empty().html(summaryCellValue).prepend($titleClone);
                                }
                                else
                                    args.cellElement.empty().html(summaryCellValue);
                                record[displayColumn] = summaryCellValue;
                                prevUpdatedCols.push(displayColumn);
                            }
                        }
                        updatedSummaryRecords.push(record);
                    }
                });
                proxy._flatChildRecords = [];
            }
        },
        //To get parent and grand Parent of one Child record.
        _getpRecords: function (record, proxy) {
            var recordParents = [];
            do {
                record = record.parentItem;
                recordParents.push(record);
            } while (record.parentItem);
            return recordParents;
        },

        //RENDER THE GRIDCONTENT
        _renderGridContent:function(){
            
            var proxy = this,
                doc = document,
                model = proxy.model,
                $tbody = $(doc.createElement('tbody')),
                $div = ej.buildTag("div.e-gridcontent#" + proxy._id + "e-gridcontent", "",
                {                    
                    "height": proxy._viewPortHeight
                }),
                $innderDiv = ej.buildTag('div.e-gridcontainer#' + proxy._id + "e-gridcontainer", "", {}, {}),
                //Insert e-contet div for ejScroller
                $scrolllerContentDiv = ej.buildTag('div', {}, { "width": "auto", "height": "auto" });
                $scrolllerContentDiv.append($innderDiv);
            //CHECK VIRTUALIZATION ENABLED OR NOT
            if (model.enableVirtualization || model.allowPaging) {
                $innderDiv.css({ 'height': proxy._totalHeight });
            }

            if (this._frozenColumnsLength > 0) {
                $innderDiv.html(proxy._renderByFrozenDesign());
                proxy.setGridContentTable($innderDiv.find(".e-table"));

            }
            else {
                var $table = ej.buildTag('table.e-table e-zerospace#' + proxy._id + "e-table", "",
                    { top: "0px", "position": "relative" });
                $table.append(proxy.getHeaderTable().find('colgroup').clone()).append($tbody);
                $innderDiv.html($table);
                proxy.setGridContentTable($table);
            }
            $div.html($scrolllerContentDiv);
            proxy._$gridContent = $div;
            //CHECK DATASOURCE IS DEFINED OR NOT
            if (this.dataSource() === null || this.model.currentViewData.length == 0) {
                if (this._frozenColumnsLength > 0) {
                    var hiddenCol = proxy._getFrozenHiddenCol(),
                        $emptyTd = ej.buildTag('td', model.emptyRecordText, {}, { colSpan: (this._frozenColumns.length - hiddenCol.frozen) });
                    $innderDiv.find("#e-frozencontentdiv" + proxy._id + " .e-table tbody").append($(doc.createElement("tr")).addClass("e-emptyText").height(30).append($emptyTd));
                    $emptyTd = ej.buildTag('td', model.emptyRecordText, { visibility: "hidden" }, { colSpan: (this._unFrozenColumns.length - hiddenCol.moveable) });
                    $innderDiv.find("#e-movablecontentdiv" + proxy._id + " .e-table tbody").append($(doc.createElement("tr")).addClass("e-emptyText").height(30).append($emptyTd));
                } else {
                    var $emptyTd = ej.buildTag('td', model.emptyRecordText, {}, { colSpan: (proxy.model.columns.length - proxy._hiddenColumns.length) });
                    $tbody.append($(doc.createElement("tr")).addClass("e-emptyText").height(30).append($emptyTd));
                    $innderDiv.css('height', model.rowHeight);
                }
            } else
            {
                proxy.renderRecords();
            }

            if (!model.isFromGantt) {
                $div.css({
                    "border-bottom-style": "solid",
                    "border-bottom-width": "1px",
                    "border-left-style": "solid",
                    "border-left-width": "1px",
                    "border-right-style": "solid",
                    "border-right-width": "1px",
                    "width": "auto",
                    "height": "auto",
                    "cursor": "default"
                });
            } else {
                $div.css({
                    "border-left-style": "solid",
                    "border-left-width": "1px"
                });
            }
            return $div;

        },

        _getFrozenHiddenCol: function () {
            var proxy = this, columns = proxy.model.columns, hiddenCol = {}; hiddenCol.frozen = 0; hiddenCol.moveable = 0;
            if (proxy._hiddenColumns.length) {
                for (var colIndex = 0; colIndex < columns.length; colIndex++) {
                    if (!columns[colIndex].visible && columns[colIndex].isFrozen)
                        hiddenCol.frozen = hiddenCol.frozen + 1;
                    else if (!columns[colIndex].visible && !columns[colIndex].isFrozen)
                        hiddenCol.moveable = hiddenCol.moveable + 1;
                }
            }
            return hiddenCol;
        },

        //AFTER INITIALIZATION END RENDERING
        _initialEndRendering: function () {

            var proxy = this, model = proxy.model, currentSelectingRecord,
                expandedRecords = proxy.getExpandedRecords(model.updatedRecords),
                index = proxy.selectedRowIndex(), cellIndexes = proxy.selectedCellIndexes();

            if (model.allowSelection) {
                if (model.selectionSettings.selectionMode == "row" && index != -1 && !model.isFromGantt) {
                if (model.showSummaryRow) {
                    var recordsWithOutSummary = proxy._spliceSummaryRows(model.updatedRecords),
                        currentSelectedRecord = recordsWithOutSummary[index];
                    index = model.updatedRecords.indexOf(currentSelectedRecord);                    
                }
                if (!proxy._rowSelectingEventTrigger(this._previousIndex, index)) {
                    proxy.selectRows(index);
                    proxy._cancelSaveTools();
                    proxy._rowSelectedEventTrigger(index);

                }
            }
                if (model.selectionSettings.selectionMode == "cell" && cellIndexes.length > 0) {
                    proxy.selectedRowIndex(-1);
                    proxy.selectCells(cellIndexes);
                    proxy._cancelSaveTools();
                }
                }

            /* treegrid rendering completed here at load time*/
            proxy._isRendered = true;
            this._trigger("dataBound", {});
            proxy._trigger("refresh");
            proxy._eventBindings();
            proxy._hideCollapsedDetailsRows();
            proxy.getScrollElement().ejScroller("refresh");
            proxy._updateScrollCss();
        },

        //#endregion
        
        //#region TEMPLATE

        //ADD CELLEDITING TEMPLATE
        _addCellEditTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                $outerDiv = ej.buildTag('div', "", { display: 'none' }, { id: proxy._id + "_CellEditTemplate" }),
                $innerDiv, columnField;

            for (columnCount = 0; columnCount < columns.length; columnCount++) {
                if (columns[columnCount]) {
                    columnField = proxy._getFieldValue(columns[columnCount].field, columnCount);
                    if (proxy._isCommandColumn(columns[columnCount])) {
                        $innerDiv = ej.buildTag('td.e-rowcell e-unboundcell', "", {}, { id: columnField + "_CellEdit" });
                        var column = columns[columnCount];
                        $("#" + this._id + columnCount + "_UnboundTemplate").remove();
                        var divElement = this._createCommandElement(column, columnCount);
                        $innerDiv.addClass("e-unboundcell").html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                        }
                    else {            
                        $innerDiv = ej.buildTag('div', "", {}, { id: columnField + "_CellEdit" });
                        ej.TreeGrid._initCellEditType(proxy, $innerDiv, proxy._id, columnCount);
                    }
                    $outerDiv.append($innerDiv);
                }
            }

            if ($outerDiv.children().length) {
                proxy._cellEditTemplate = $outerDiv;
            }

        },

        _getState: function () {
            if (this.data.parentItem)
                return false;
            else
                return true;
        },

        _getCellIndex: function (array, findItem) {
            var proxy = this, cellIndexArray = [];
            for (var i = 0; i < array.length; i++) {
                cellIndexArray[i] = array[i][findItem];
            }
            return cellIndexArray;
        },
        _getSelectedCellClass: function (data, columnCount) {
            var proxy = this, model = proxy.model,
                columnIndex = parseInt(columnCount),
                selectedCellDetails = proxy._selectedCellDetails;
            if (model.selectionSettings.selectionMode == "cell" && selectedCellDetails.length > 0) {
                    proxy._getSelectedCellsinARow(data.index, selectedCellDetails);
                if (proxy._selectedCellsinARow.length > 0) {
                    if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)
                        var currentSelectingCellIndex = proxy._getCellIndex(proxy._selectedCellsinARow,"cellIndex").indexOf(columnIndex);
                    else
                        var currentSelectingCellIndex = proxy._selectedCellsinARow.map(function (e) { return e.cellIndex }).indexOf(columnIndex);                    
                    if (currentSelectingCellIndex != -1) {
                        if (proxy._selectedCellsinARow[currentSelectingCellIndex].cellIndex == columnIndex)
                            return "e-selectingcell";
                    }
                }
            }
            return "";
        },

        //Return true if batch edited data
        _getBatchEditState: function (data, columnName) {
            var proxy = this, model = proxy.model;
            if (proxy._checkboxSelection && columnName == "checkboxState") {
                return false;
            }
            else if (data.isBatchAdd || (data.batchEditedFields && data.batchEditedFields[columnName] != null)) {
                return true;
            }
            else false;
        },
        _getSelectedCellsinARow: function (recordIndex, selectedCellDetails) {
            var proxy = this;
            proxy._selectedCellsinARow = [];
            selectedCellDetails.forEach(function (selectedcell) {
                if (selectedcell.data.index == recordIndex)
                    proxy._selectedCellsinARow.push(selectedcell);
            });
            proxy._currentRecordIndex = recordIndex;
        },
        _getrowName: function () {
            var rowClass = "gridrowIndex",
                proxy = this;
            rowClass += proxy.data.index.toString() + "level" + proxy.data.level.toString();
            return rowClass;
        },
        /* get details row col span with showDetailROwInfoColumn API value */
        _getDetailsColSpan:function(isFrozenTable)
        {
            var model = this.model, visibleColumns = [];
            if (this._frozenColumnsLength > 0) {
                if (!isFrozenTable) {
                    visibleColumns = this._unFrozenColumns.filter(function (value) {
                        return value.visible !== false;
                    });
                    return visibleColumns.length;
                } else {
                    visibleColumns = this._frozenColumns.filter(function (value) {
                        return value.visible !== false;
                    });
                    return visibleColumns.length;
                }
            }
            return model.showDetailsRowInfoColumn ? this._visibleColumns.length + 1 : this._visibleColumns.length;
        },
        /* expand status for details rows */
        _getDetailsExpandStatus: function (data) {
            var proxy = this;
            if (proxy.getExpandStatus(data) && data.isDetailsExpanded) {
                return true;
            } else
            {
                return false;
            }
        },
        _createDetailsRowTemplate: function (isFrozenTable) {
            var proxy = this,
                model = this.model,
                helpers = {};

            helpers["_" + proxy._id + "getDetailsColSpan"] = $.proxy(proxy._getDetailsColSpan, proxy);
            helpers["_" + proxy._id + "getDetailsExpandStatus"] = $.proxy(proxy._getDetailsExpandStatus, proxy);
            helpers["_" + proxy._id + "getExpandStatus"] = $.proxy(proxy.getExpandStatus, proxy);
            
            $.views.helpers(helpers);

            var tr = '{{if !isSummaryRow && !footerSummaryRowRecord}}' + "<tr class='e-detailsrow e-treegridrows detailsrow{{:~_" + proxy._id + "rowClassName()}} {{if isDetailsExpanded}}" +
                      "e-detailsrowexpanded{{else}}e-detailsrowcollapsed{{/if}}{{if !~_" + proxy._id + "getExpandStatus(#data) && isDetailsExpanded}} e-collapsedrowexpandeddetailsrow{{/if}}' " +
                      "style='{{if ~_" + proxy._id + "getDetailsExpandStatus(#data)}}display:table-row;{{else}}visibility:none;{{/if}}height:" + model.detailsRowHeight + "px;'>" +
                      "<td class='e-detailsrowcell' colspan='{{:~_" + proxy._id + "getDetailsColSpan(" + isFrozenTable + ")}}'>" +
                      "<div class='e-detailscellwrapper' style='height:" + (model.detailsRowHeight - 1) + "px;'>";

            var templateString = "";
            if (model.detailsTemplate && model.detailsTemplate.length > 0) {

                if (document.getElementById(model.detailsTemplate))
                {
                    templateString = "#" + model.detailsTemplate;
                } else
                {
                    templateString = model.detailsTemplate;
                }
            }
            if (!isFrozenTable) {
                $("#treeGridDetailsRowTemplateElement" + proxy._id).remove();
                var scriptElement = this._createDetailRowTemplateElement(templateString);
                tr += "{{:~_" + proxy._id + "treeGridTemplating('" + scriptElement.id + "')}}";
            }
            tr += "</div></td></tr>";
            tr += "{{/if}}";
            if (!isFrozenTable) {
                var templates = {};
                templates[proxy._id + "_detailRowTemplate"] = tr;
                $.templates(templates);
            }
            return tr;

        },

        _SelectState:function(args){
            var proxy = this,
                model = this.model;
            if (model.allowSelection)
                return true;
            else
                return false;
        },
        _unboundClickHandler: function (e) {
            var proxy = this,
              model = this.model, form, isAddRow,
            $target = $(e.target).closest("button");
            if (!($target.hasClass("e-deletebutton") || $target.hasClass("e-editbutton") || $target.hasClass("e-savebutton") || $target.hasClass("e-cancelbutton"))) {
                return
            }
            if ($target.hasClass("e-button") && ($target.hasClass("e-disable") || $target.prop("disabled"))) return;
            if ($(e.target).hasClass("e-unboundcelldiv"))
                return;

            if (proxy.model.editSettings.editMode == "dialogEditing") {
                form = $("#" + proxy._id + "EditFormDialog");
            } else {
                form = $("#" + proxy._id + "EditForm");
            }
            var index = ($target.hasClass("e-savebutton") || $target.hasClass("e-cancelbutton")) ? proxy.getIndexByRow(form.closest("tr")) : proxy.getIndexByRow($target.closest("tr"));
            proxy.selectRows(proxy.getUpdatedRecords().indexOf(model.currentViewData[index]));
            if ($target.hasClass("e-editbutton") && model.editSettings.allowEditing) {
                if (proxy._isRowEdit) {
                    proxy.cancelEdit();
                }
                if (model.editSettings.editMode != "dialogEditing") {
                    var columnIndex = proxy.getCellIndex(e);
                    proxy._editRow(proxy.getUpdatedRecords().indexOf(model.currentViewData[index]), null, columnIndex);
                }
                else {
                    proxy.showEditDialog(proxy.getUpdatedRecords().indexOf(model.currentViewData[index]));
                }
            } else if ($target.hasClass("e-deletebutton")) {
                proxy.deleteRow(null, true);
            }
            else if ($target.hasClass("e-savebutton")) {
                    proxy.saveRow(); 
            }
            else if ($target.hasClass("e-cancelbutton")) {
                proxy.cancelEdit();
            }
        },
        _createCommandElement: function (column, count) {
            var divElement = document.createElement("div");
            divElement.id = this._id + count + "_UnboundTemplate";
            var $div = ej.buildTag("div.e-unboundcelldiv"), commands = column.commands;
            for (var unbounType = 0; unbounType < commands.length; unbounType++) {
                if (!ej.isNullOrUndefined(commands[unbounType].type)) {
                    var $button = ej.buildTag("button.e-" + commands[unbounType].type.replace(/\s+/g, "") + "button", "", {}, { type: "button" });
                    $button.val(commands[unbounType].type);
                    $div.append($button);
                }
            }
            $("body").append($(divElement).html($div).hide());
            return divElement;
        },
        _refreshUnboundTemplate: function ($target) {
            var proxy = this,
                model = proxy.model;
            if (proxy._commandColumnLength > 0) {
                for (var column = 0; column < model.columns.length; column++) {
                    if (proxy._isCommandColumn(model.columns[column])) {
                        var $unboundDivs = $target.find(".e-unboundcell").find(".e-unboundcelldiv");
                        var commands = $.extend(true, [], model.columns[column].commands);
                        for (var j = 0; j < commands.length; j++) {
                            if (ej.isNullOrUndefined(commands[j].buttonOptions))
                                commands[j].buttonOptions = {};
                            commands[j].buttonOptions.width = ej.isNullOrUndefined(commands[j].buttonOptions.width) ? "52" : commands[j].buttonOptions.width;
                            commands[j].buttonOptions.height = ej.isNullOrUndefined(commands[j].buttonOptions.height) ? "22" : commands[j].buttonOptions.height;
                            commands[j].buttonOptions.showRoundedCorner = ej.isNullOrUndefined(commands[j].buttonOptions.showRoundedCorner) ? true : commands[j].buttonOptions.showRoundedCorner;
                            commands[j].buttonOptions.size = ej.isNullOrUndefined(commands[j].buttonOptions.size) ? "small" : commands[j].buttonOptions.size;
                            if (!ej.isNullOrUndefined(commands[j].type)) {
                                var $buttons = $unboundDivs.find(".e-" + commands[j].type.replace(/\s+/g, "") + "button");

                                if ($target.find(".e-unboundeditedcell").length > 0) {
                                    if (commands[j].type == "save" || commands[j].type == "cancel")
                                        $buttons.show();
                                    else {
                                        $buttons.hasClass("e-deletebutton") && $buttons.hide();
                                        $buttons.hasClass("e-editbutton") && $buttons.hide();
                                    }
                                } else {
                                    if (commands[j].type == "save" || commands[j].type == "cancel")
                                        $buttons.hide();
                                    else {
                                        $buttons.hasClass("e-deletebutton") && $buttons.show();
                                        $buttons.hasClass("e-editbutton") && $buttons.show();
                                    }
                                }
                                $buttons.ejButton(commands[j].buttonOptions);
                            }
                        }
                    } else
                        continue;
                }
            }
        },
        _unboundTemplateRendering: function (unboundTemplateId) {
            return $("#" + unboundTemplateId).html();
        },
        _collapsibleSummary: function (columnCount, data) {
            var proxy = this,
                model = proxy.model,
                columns = model.columns;
            if (model.showTotalSummary && model.collapsibleTotalSummary && proxy._summaryColumnIndex == parseInt(columnCount) && data.isFirstFooterSummary) {
                return true;
            }
        },
        _summaryIndendCell: function (columnCount) {
            var proxy = this,
               model = proxy.model,
               columns = model.columns;
            if (model.showTotalSummary && model.collapsibleTotalSummary && proxy._summaryColumnIndex == parseInt(columnCount)) {
                if (this._summaryColumnIndex != 0) {
                    return true;
                }
                else if (this._summaryColumnIndex == 0) {
                    return false;
                }
            }
        },
        _getsummaryExpandStatus: function (data) {
            var proxy = this;
            if (!ej.isNullOrUndefined(data.summaryExpanded) && !data.summaryExpanded) proxy._summaryExpanded = false;
            if (this.model.collapsibleTotalSummary && !proxy._summaryExpanded) {
                if (data.isFirstFooterSummary)
                    return 'table-row';
                else
                    return 'none';
            }
            else
                return 'table-row';
        },
         
        //ADD TEMPLATE FOR RENDERING GRIDCONTENT
        _addInitTemplate:function(){

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                length = columns.length,
                columnName,
                column,
                tdCell,
                summaryTitle, isTextWrap = false,
                helpers = {};
            proxy._commandColumnLength = 0;

            //CHECK THE COLUMNS ARRAY CONTAIN THE COLUMN OBJECT
            if (columns.length === 0) {
                return false;
            }
            if (this._frozenColumnsLength > 0) {
                this.addFrozenTemplate();
                if (model.showTotalSummary)
                    this.addFrozenSummaryTemplate();
            }
            helpers["_" + proxy._id + "rowClassName"] = ej.TreeGrid._getrowClassName;
            helpers["_" + proxy._id + "SummaryRowtdClassName"] = ej.TreeGrid._getSummaryRowtdClassName;
            helpers["_" + proxy._id + "summaryIndendCell"] = $.proxy(proxy._summaryIndendCell, proxy);
            helpers["_" + proxy._id + "collapsibleSummary"] = $.proxy(proxy._collapsibleSummary, proxy);
            helpers["_" + proxy._id + "isSelectedCell"] = $.proxy(proxy._getSelectedCellClass, proxy);
            helpers["_" + proxy._id + "batchEditState"] = $.proxy(proxy._getBatchEditState, proxy);
            helpers["_" + proxy._id + "summaryTitle"] = proxy._getSummaryTitle;
            // helpers for drag row indicator icon class
            helpers["_" + proxy._id + "rowName"] = proxy._getrowName;
            helpers["_" + proxy._id + "getState"] = proxy._getState;
            helpers["_" + proxy._id + "SelectState"] = $.proxy(proxy._SelectState, proxy);
            helpers["_" + proxy._id + "summaryExpandStatus"] = $.proxy(proxy._getsummaryExpandStatus, proxy);
            helpers["_" + proxy._id + "expandStatus"] = $.proxy(proxy._getExpandStatusRecord, proxy);         
            helpers["_" + proxy._id + "cellValue"] = $.proxy(proxy._getCellValue, proxy);
            helpers["_" + proxy._id + "TemplateCellValue"] = $.proxy(proxy._getTemplateCellValue, proxy);
            helpers["_" + proxy._id + "isTreeColumnIndex"] = $.proxy(proxy._isTreeColumnIndex, proxy);
            helpers["_" + proxy._id + "getIndentWidth"] = $.proxy(proxy._getIndentWidth, proxy);
            helpers["_" + proxy._id + "checkColumn"] = $.proxy(proxy._getCellColumn, proxy);
            helpers["_" + proxy._id + "formatting"] = $.proxy(proxy.formatting, proxy);
            helpers["_" + proxy._id + "treeGridTemplating"] = ej.proxy(proxy._gridTemplate, null, proxy);
            helpers["_" + proxy._id + "getCheckboxState"] = $.proxy(proxy._getCheckboxState, proxy);
            helpers["_" + proxy._id + "UnboundTemplate"] = $.proxy(proxy._unboundTemplateRendering, proxy);
            helpers["_" + proxy._id + "isTrimCell"] = $.proxy(proxy._checkIsTrimCell, proxy);
            helpers["_" + proxy._id + "getRowHeight"] = $.proxy(proxy._getTemplateRowHeight, proxy);

            $.views.helpers(helpers);

            if (model.allowTextWrap)
                isTextWrap = proxy.model.allowTextWrap;

            if (model.parseRowTemplate && (model.rowTemplateID.length > 0 || model.altRowTemplateID.length > 0)) {
                
                var altRowTemplateElement = ej.buildTag("tbody", "", {}, {});
                var rowTemplateElement = ej.buildTag("tbody", "", {}, {});
                var rowTemplateHTML = "";
                var altRowTemplateHTML = "";

                var altRowTemplateScript = model.altRowTemplateID.length > 0 ? $("#" + model.altRowTemplateID): $("#" + model.rowTemplateID);
                var rowTemplateScript = model.rowTemplateID.length > 0 ? $("#" + model.rowTemplateID): $("#" + model.altRowTemplateID);
                var altRowTemplate = altRowTemplateScript[0].innerHTML;
                var rowTemplate = rowTemplateScript[0].innerHTML;

                $(altRowTemplateElement[0]).html(altRowTemplate);
                $(rowTemplateElement[0]).html(rowTemplate);

                if (rowTemplateElement[0].childNodes.length > 0) {
                    for (var i = 0; i < rowTemplateElement[0].childNodes.length; i++) {
                        if (rowTemplateElement[0].childNodes[i].nodeName == 'TR') {

                            rowTemplateElement[0].className = rowTemplateElement[0].childNodes[i].className;
                            rowTemplateElement[0].style.cssText = rowTemplateElement[0].childNodes[i].style.cssText;

                            if (rowTemplateElement[0].childNodes[i].childNodes.length > 0) {
                                var tdCount = 0;
                                for (var j = 0; j < rowTemplateElement[0].childNodes[i].childNodes.length; j++) {

                                    var childNode = rowTemplateElement[0].childNodes[i].childNodes[j];

                                    if (childNode.nodeName == 'TD') {
                                        var innerHtml = childNode.innerHTML,
                                            templateId = proxy._id + "RowTemplateElement" + tdCount;

                                        $("#" + templateId).remove();
                                        var scriptElement = this._createRowTemplateElement(innerHtml, templateId, rowTemplateScript[0].type);
                                        childNode.innerHTML = "{{:~_" + proxy._id + "treeGridTemplating('" + scriptElement.id + "')}}";
                                        childNode.className += " e-rowcell e-templatecell";


                                        if (columns[tdCount]) {
                                            column = columns[tdCount];
                                            columnName = column.field;
                                            var visible = column.visible == undefined ? true : column.visible;
                                            //To create a seperate div in summary cell to display a summary title.
                                            var summaryTitlediv = "";
                                            if (column.headerText == proxy._visibleColumns[0]) {
                                                summaryTitlediv = '{{if isSummaryRow || footerSummaryRowRecord}}' + "<div " +
                                                    "class='e-summarytitle{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}} {{if " + isTextWrap + "}} e-textwrapcell{{/if}}'" + "style='float:left;display:block;'>" + "{{:~_" + proxy._id + "summaryTitle()}}"
                                                     + "</div>" + '{{/if}}';
                                            }

                                            if (tdCount == model.treeColumnIndex) {
                                                var indentDiv = "<div class='e-inner-treecolumn-container' style='min-height:20px;' data-unselectable='on'>" +
                                                "{{if hasChildRecords && hasFilteredChildRecords}}" +
                                                "<div class='e-dragintend'" +
                                                "{{if filterLevel > -1}}" +
                                                "style='height:1px; float:left; width:{{:((filterLevel)*" + proxy.treeIndentLevelWidth + ")}}" + "px; display:inline-block;'>" +
                                                "{{else}}" +
                                                "style='height:1px; float:left; width:{{:((level)*" + proxy.treeIndentLevelWidth + ")}}" + "px; display:inline-block;'>" +
                                                "{{/if}}"+
                                                 //indicator icons included for drag and drop
                                                "{{if !~_" + proxy._id + "getState() || !hasChildRecords }}" +
                                                "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                                "<span class='e-aboveIcon e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='e-belowIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='e-childIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='e-cancelIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "</div>" +
                                                "{{else !hasChildRecords || (!hasFilteredChildRecords && hasChildRecords)}}" +
                                                "<div " +
                                                "class='e-dragintend'" +
                                                "{{if filterLevel > -1}}" +
                                                "style='height:1px; float:left; width:{{:(filterLevel*" + proxy.treeIndentLevelWidth + ")+24}}" + "px; display:inline-block;'>" +
                                                "{{else}}" +
                                                "style='height:1px; float:left; width:{{:(level*" + proxy.treeIndentLevelWidth + ")+24}}" + "px; display:inline-block;'>" +
                                                "{{/if}}" +
                                                //indicator icons included for drag and drop
                                                "{{if !~_" + proxy._id + "getState() || !hasChildRecords || !hasFilteredChildRecords }}" +
                                                "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                                "<span class='e-aboveIcon iconMargin e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='e-belowIcon iconMargin e-icon' style='float:right;display:none;'></span>" +
                                                "<span class='e-childIcon iconMargin e-icon' style='float:right;display:none;'></span>" +
                                                "<span class='e-cancelIcon iconMargin e-icon' style='float:right;display:none;'></span>" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "{{if ~_" + proxy._id + "getState() && hasChildRecords && hasFilteredChildRecords}}" +
                                                "<div class='e-intendparent'>" +
                                                "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                                "<span class='e-aboveIcon e-icon'  style='position:relative;float:left;display:none;'></span>" +
                                                "<span class='e-belowIcon e-icon' style='float:left;display:none;'></span>" +
                                                "<span class='e-childIcon e-icon' style='float:left;display:none;'></span>" +
                                                "<span class='e-cancelIcon e-icon' style='float:left;display:none;'></span>" +
                                                "</div>" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "<div class='{{if expanded && hasFilteredChildRecords}}e-treegridexpand e-icon{{else hasChildRecords && hasFilteredChildRecords}}e-treegridcollapse e-icon{{/if}}' " +
                                                "style='float:left;display:inline-block;font-size:10px;'>" +
                                                "</div>" +
                                                "<div class='e-cell{{if " + (column.showCheckbox && (this.model.selectionSettings.selectionType == ej.TreeGrid.SelectionType.Checkbox)) + "}} e-treecheckboxcolumn{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}' style='overflow: hidden;'>";

                                                childNode.style.cssText = "padding-left:0px;";

                                                rowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected && ~_" + proxy._id + "SelectState()}} e-selectionbackground{{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}} e-hide{{/if}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}} e-summaryintend{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                    + childNode.style.cssText + "'>" + indentDiv + "{{else}}'>{{/if}}" +
                                                     "{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + tdCount + "',#data)}}" +
                                                    "<div class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                                    "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                                    "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                                    "{{/if}}" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}' +
                                                    "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                                    "<div class='e-summarycell'" +
                                                    "style='overflow: hidden;padding-left:25px;'>" +
                                                    "{{/if}}";
                                                     rowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}" + '{{/if}}';
                                                     rowTemplateHTML += "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                                     "</div>" +
                                                     "{{/if}}" +
                                                     "</div></div></td>";
                                            }
                                            else {
                                                rowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected && ~_" + proxy._id + "SelectState()}} e-selectionbackground{{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}} e-hide{{/if}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}} e-summaryintend{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                    + childNode.style.cssText + "{{/if}}'>" +
                                                    "{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + tdCount + "',#data)}}" +
                                                    "<div  " +
                                                    "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                                    "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                                    "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                                    "{{/if}}" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}'+
                                                    "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                                    "<div class='e-summarycell'" +
                                                    "style='overflow: hidden;padding-left:25px;'>" +
                                                    "{{/if}}";
                                                rowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}" + '{{/if}}';
                                                rowTemplateHTML += "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "</td>";
                                            }
                                            if (this._frozenColumnsLength > 0 && this._frozenColumnsLength == tdCount + 1) {
                                                var frozenRowTemplate = "";
                                                frozenRowTemplate += rowTemplateHTML;
                                                frozenRowTemplate += "</tr>";
                                                rowTemplateHTML = "";
                                            }
                                            tdCount++;

                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if (altRowTemplateElement[0].childNodes.length > 0) {
                    for (var i = 0; i < altRowTemplateElement[0].childNodes.length; i++) {
                        if (altRowTemplateElement[0].childNodes[i].nodeName == 'TR') {
                            altRowTemplateElement[0].className = altRowTemplateElement[0].childNodes[i].className;
                            altRowTemplateElement[0].style.cssText = altRowTemplateElement[0].childNodes[i].style.cssText;
                            var tdCount = 0;

                            for (var j = 0; j < altRowTemplateElement[0].childNodes[i].childNodes.length; j++) {

                                var childNode = altRowTemplateElement[0].childNodes[i].childNodes[j];

                                if (childNode.nodeName == 'TD') {

                                    var innerHtml = childNode.innerHTML,
                                        templateId = proxy._id + "AltRowTemplateElement" + tdCount;
                                    $("#" + templateId).remove();
                                    var scriptElement = this._createRowTemplateElement(innerHtml, templateId, altRowTemplateScript[0].type);
                                    childNode.innerHTML = "{{:~_" + proxy._id + "treeGridTemplating('" + scriptElement.id + "')}}";

                                    childNode.className += " e-rowcell e-templatecell";
                                    if (columns[tdCount]) {
                                        column = columns[tdCount];
                                        columnName = column.field;
                                        var visible = column.visible == undefined ? true : column.visible;
                                        //To create a seperate div in summary cell to display a summary title.
                                        var summaryTitlediv = "";
                                        if (column.headerText == proxy._visibleColumns[0]) {
                                            summaryTitlediv = '{{if isSummaryRow || footerSummaryRowRecord}}' + "<div " +
                                                "class='e-summarytitle{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}} {{if " + isTextWrap + "}} e-textwrapcell{{/if}}'" + "style='float:left;display:block;'>" + "{{:~_" + proxy._id + "summaryTitle()}}"
                                                 + "</div>" + '{{/if}}';
                                        }
                                        if (tdCount == model.treeColumnIndex) {
                                            var indentDiv = "<div class='e-inner-treecolumn-container' style='min-height:20px;'>" +
                                            "{{if hasChildRecords && hasFilteredChildRecords}}" +
                                            "{{if filterLevel > -1}}" +
                                            "<div class='e-dragintend' style='height:1px; float:left;width:{{:((filterLevel)*" + proxy.treeIndentLevelWidth + ")}}" + "px; display:inline-block;'>" +
                                            "{{else}}" +
                                            "<div class='e-dragintend' style='height:1px; float:left;width:{{:((level)*" + proxy.treeIndentLevelWidth + ")}}" + "px; display:inline-block;'>" +
                                            "{{/if}}" +
                                            //indicator icons included for drag and drop
                                            "{{if !~_" + proxy._id + "getState() || !hasChildRecords }}" +
                                            "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                            "<span class='e-aboveIcon e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                            "<span class='e-belowIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                            "<span class='e-childIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                            "<span class='e-cancelIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                            "</div>" +
                                            "{{/if}}" +
                                            "</div>" +
                                            "{{else !hasChildRecords || (!hasFilteredChildRecords && hasChildRecords)}}" +
                                                "<div " +
                                                "class='e-dragintend'" +
                                                "{{if filterLevel > -1}}" +
                                                "style='height:1px; float:left; width:{{:(filterLevel*" + proxy.treeIndentLevelWidth + ")+24}}" + "px; display:inline-block;'>" +
                                                "{{else}}" +
                                                "style='height:1px; float:left; width:{{:(level*" + proxy.treeIndentLevelWidth + ")+24}}" + "px; display:inline-block;'>" +
                                                "{{/if}}" +
                                                //indicator icons included for drag and drop
                                                "{{if !~_" + proxy._id + "getState() || !hasChildRecords || !hasFilteredChildRecords}}" +
                                                "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                                "<span class='e-aboveIcon iconMargin e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='e-belowIcon iconMargin e-icon' style='float:right;display:none;'></span>" +
                                                "<span class='e-childIcon iconMargin e-icon' style='float:right;display:none;'></span>" +
                                                "<span class='e-cancelIcon iconMargin e-icon' style='float:right;display:none;'></span>" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "</div>" +
                                                "{{/if}}" +
                                            "{{if ~_" + proxy._id + "getState() && hasChildRecords && hasFilteredChildRecords}}" +
                                            "<div class='e-intendparent'>" +
                                            "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                            "<span class='e-aboveIcon e-icon'  style='position:relative;float:left;display:none;'></span>" +
                                            "<span class='e-belowIcon e-icon' style='float:left;display:none;'></span>" +
                                            "<span class='e-childIcon e-icon' style='float:left;display:none;'></span>" +
                                            "<span class='e-cancelIcon e-icon' style='float:left;display:none;'></span>" +
                                            "</div>" +
                                            "</div>" +
                                            "{{/if}}" +
                                            "<div class='{{if expanded&& hasFilteredChildRecords}}e-treegridexpand e-icon{{else hasChildRecords && hasFilteredChildRecords}}e-treegridcollapse e-icon{{/if}}' " +
                                            "style='float:left;display:inline-block;font-size:10px;'></div>" +
                                            "<div class='e-cell{{if " + column.showCheckbox + "}} e-treecheckboxcolumn{{/if}}{{if  ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}} {{if " + isTextWrap + "}} e-textwrapcell{{/if}}' style='overflow:hidden'>";

                                            childNode.style.cssText = "padding-left:0px;";

                                            altRowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected && ~_" + proxy._id + "SelectState()}} e-selectionbackground{{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}} e-hide{{/if}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}} e-summaryintend{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                 + childNode.style.cssText + "'>" + indentDiv + "{{else}}'>{{/if}}" +
                                                    "{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + tdCount + "',#data)}}" +
                                                    "<div  " +
                                                    "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                                    "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                                    "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                                    "{{/if}}" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}'+
                                                     "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                                    "<div class='e-summarycell'" +
                                                    "style='overflow: hidden;padding-left:25px;'>" +
                                                    "{{/if}}";
                                            altRowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}" + '{{/if}}';
                                            altRowTemplateHTML += "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                            "</div>" +
                                            "{{/if}}" +
                                            "</div></div></td>";

                                        }
                                        else {
                                            altRowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected && ~_" + proxy._id + "SelectState()}} e-selectionbackground{{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}} e-hide{{/if}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}} e-summaryintend{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                    + childNode.style.cssText + "{{/if}}'>" +
                                                    "{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + tdCount + "',#data)}}" +
                                                    "<div  " +
                                                    "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                                    "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                                    "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                                    "{{/if}}" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}'+
                                                     "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                                    "<div class='e-summarycell'" +
                                                    "style='overflow: hidden;padding-left:25px;'>" +
                                                    "{{/if}}";
                                            altRowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}" + '{{/if}}';
                                            altRowTemplateHTML += "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + tdCount + ")}}" +
                                                     "</div>" +
                                                     "{{/if}}" + "</td>";
                                        }
                                        if (this._frozenColumnsLength > 0 && this._frozenColumnsLength == tdCount + 1) {
                                            var frozenAltRowTemplate = "";
                                            frozenAltRowTemplate += altRowTemplateHTML;
                                            frozenAltRowTemplate += "</tr>";
                                            altRowTemplateHTML = "";
                                        }
                                        tdCount++;
                                    }
                                }
                            }
                        }
                    }
                }
               
                /* add details column to end of headers */
                if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn) {
                    altRowTemplateHTML += "<td class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-selectionbackground{{/if}} {{if isDetailsExpanded}}e-detailsrowexpand{{else}}e-detailsrowcollapse{{/if}}  " +
                                          "e-detailsrowcell{{if isLastSummary}} e-lastsummaryrow{{/if}}'>{{if !isSummaryRow && !footerSummaryRowRecord}}<div class='e-detailsiconcell e-icon {{if isDetailsExpanded}}e-detailsinfoiconhide{{else}}e-detailsinfoiconshow{{/if}}'></div>{{/if}}";
                    rowTemplateHTML += "<td class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-selectionbackground{{/if}} {{if isDetailsExpanded}}e-detailsrowexpand{{else}}e-detailsrowcollapse{{/if}}  " +
                                        "e-detailsrowcell{{if isLastSummary}} e-lastsummaryrow{{/if}}'>{{if !isSummaryRow && !footerSummaryRowRecord}}<div class='e-detailsiconcell e-icon {{if isDetailsExpanded}}e-detailsinfoiconhide{{else}}e-detailsinfoiconshow{{/if}}'></div>{{/if}}";
                }

                var trRow = "{{if isAltRow}}" + "<tr class='{{:~_" + proxy._id + "rowClassName()}} " +
                    "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} e-treegridrowcollapse {{/if}} e-templaterow "
                    + altRowTemplateElement[0].className + "' style='{{if !isSummaryRow && !footerSummaryRowRecord}}" + altRowTemplateElement[0].style.cssText + "{{else}}height:" + model.rowHeight + "px;{{/if}}display:{{if footerSummaryRowRecord}}{{:~_" + proxy._id + "summaryExpandStatus(#data)}}{{else}}{{:~_" + proxy._id + "expandStatus(#data)}}{{/if}}'" + "'>" + altRowTemplateHTML + "</tr>"

                    + "{{else}}" + "<tr class='{{:~_" + proxy._id + "rowClassName()}} " +
                    "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} e-treegridrowcollapse {{/if}} e-templaterow "
                    + rowTemplateElement[0].className + "' style='{{if !isSummaryRow && !footerSummaryRowRecord}}" + rowTemplateElement[0].style.cssText + "{{else}}height:" + model.rowHeight + "px;{{/if}}display:{{if footerSummaryRowRecord}}{{:~_" + proxy._id + "summaryExpandStatus(#data)}}{{else}}{{:~_" + proxy._id + "expandStatus(#data)}}{{/if}}'" + "'>" + rowTemplateHTML + "</tr>" +
                    "{{/if}}";


                var frozenTrRow = "{{if isAltRow}}" + "<tr class='{{:~_" + proxy._id + "rowClassName()}} " +
                    "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} e-treegridrowcollapse {{/if}} e-templaterow "
                    + altRowTemplateElement[0].className + "' style='{{if !isSummaryRow && !footerSummaryRowRecord}}" + altRowTemplateElement[0].style.cssText + "{{else}}height:" + model.rowHeight + "px;{{/if}} display:{{if footerSummaryRowRecord}}{{:~_" + proxy._id + "summaryExpandStatus(#data)}}{{else}}{{:~_" + proxy._id + "expandStatus(#data)}}{{/if}}'" + "'>" + frozenRowTemplate + "</tr>"

                    + "{{else}}" + "<tr class='{{:~_" + proxy._id + "rowClassName()}} " +
                    "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} e-treegridrowcollapse {{/if}} e-templaterow "
                    + rowTemplateElement[0].className + "' style='{{if !isSummaryRow && !footerSummaryRowRecord}}" + rowTemplateElement[0].style.cssText + "{{else}}height:" + model.rowHeight + "px;{{/if}} display:{{if footerSummaryRowRecord}}{{:~_" + proxy._id + "summaryExpandStatus(#data)}}{{else}}{{:~_" + proxy._id + "expandStatus(#data)}}{{/if}}'" + "'>" + frozenAltRowTemplate + "</tr>" +
                    "{{/if}}";
            } else if (!model.parseRowTemplate && (model.rowTemplateID.length > 0 || model.altRowTemplateID.length > 0)) {
                /*Render rowTemplates without parsing*/
                if (model.rowTemplateID && model.altRowTemplateID) {
                    trRow = "{{if isAltRow}}" + $.trim($("#" + model.rowTemplateID).html()) + "{{else}}" + $.trim($("#" + model.altRowTemplateID).html()) + "{{/if}}";
                }
                else if (model.rowTemplateID.length > 0) {
                    trRow = $.trim($("#" + model.rowTemplateID).html());
                } else if (model.altRowTemplateID.length > 0) {
                    trRow = $.trim($("#" + model.altRowTemplateID).html());
                }   
            }

            else {

                var trRow = "<tr class='e-treegridrows {{:~_" + proxy._id + "rowClassName()}} ";

                if (model.enableAltRow) {
                    trRow += "{{if isAltRow}}e-alt-row{{/if}} ";
                }

              
                var tdCellCollections = "";
                for (columnCount = 0; columnCount < length; columnCount++) {
                    if (columns[columnCount]) {
                        columnName = columns[columnCount].field;
                        column = columns[columnCount];
                        var visible = columns[columnCount].visible == undefined ? true : columns[columnCount].visible;
                        var textAlign = columns[columnCount].textAlign;
                        //To create a seperate div in summary cell to display a summary title.
                        summaryTitlediv = "";
                        if (column.headerText == proxy._visibleColumns[0]) {
                            summaryTitlediv = '{{if isSummaryRow || footerSummaryRowRecord}}' + "<div " +
                                "class='e-summarytitle{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}} {{if " + isTextWrap + "}} e-textwrapcell{{/if}}'" + "style='float:left;display:block;'>" + "{{:~_" + proxy._id + "summaryTitle()}}"
                                 + "</div>" + '{{/if}}';
                        }

                        //CHECK THE template COLUMN OR NOT
                        if (column["isTemplateColumn"] && (column["angularTemplate"] || column["templateID"] || column["template"])) {

                            trRow += "e-templaterow ";
                            var headerText = column['headerText'];
                            if (!ej.isNullOrUndefined(headerText) && !ej.isNullOrUndefined(headerText.match(/(\\|[^0-9A-z-_])/g)))
                                headerText = headerText.replace(/(\\|[^0-9A-z-_])/g, "");
                            $("#" + proxy._id + headerText + $.inArray(column, this.model.columns) + "_Template").remove(); //Remove if any template already rendered
                            var scriptElement = this._createTemplateElement(column);
                            tdCell = "<td " +
                                 "class='{{if !isSummaryRow && !footerSummaryRowRecord}} e-intend{{/if}} e-rowcell e-templatecell{{if isSelected && ~_" + proxy._id + "SelectState()}} e-selectionbackground{{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}} e-hide{{/if}}{{:~_" + proxy._id + "SummaryRowtdClassName()}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}} e-summaryintend{{/if}}{{if ~_" + proxy._id + "batchEditState(#data,'" + columnName + "')}}e-updatedtd e-tgupdatenotify{{/if}}'" +
                                 "style='text-Align:" + textAlign + ";'" +
                                 "role='row'>" +
                                 "{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + columnCount + "',#data)}}" +
                                 "<div  " +
                                "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                "{{/if}}" +
                                 '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                 "{{:~_" + proxy._id + "treeGridTemplating('" + scriptElement.id + "','" + columnCount + "')}}" +
                                 '{{else}}' + summaryTitlediv + '{{/if}}'+
                                  "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                 "<div class='e-summarycell'" +
                                 "style='overflow: hidden;padding-left:25px;'>" +
                                 "{{/if}}";
                            tdCell += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}" + '{{/if}}';
                            tdCell += "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                "</div>" +
                                "{{/if}}</td>";
                            //EXPAND COLLAPSE COLUMN BASED ON THE TREECOLUMNINDEX
                        } else if (columnCount === model.treeColumnIndex) {

                            tdCell = "<td " +
                                 "class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-rowcell e-selectionbackground e-intend {{else}}e-rowcell {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if !isSummaryRow && !footerSummaryRowRecord}} e-intend{{/if}} {{/if}} {{:~_" + proxy._id + "SummaryRowtdClassName()}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}} e-summaryintend{{/if}}{{if ~_" + proxy._id + "batchEditState(#data,'" + columnName + "')}}e-updatedtd e-tgupdatenotify{{/if}} {{if hasChildRecords && hasFilteredChildRecords}} e-parentcell{{/if}}'" +
                                 "style='padding-left:{{if ~_" + proxy._id + "isTreeColumnIndex('" + columnName + "') && isSummaryRow }}{{:((level)*" + proxy.treeIndentLevelWidth + ")}}" + "px{{/if}};background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                 "{{:cellBackgroundColor}}{{else}}none{{/if}};'" +
                                 "role='row'>" +
                                 "{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + columnCount + "',#data)}}" +
                                 "<div  " +
                                  "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                  "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                  "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                  "{{/if}}" +
                                 '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                 "<div class='e-inner-treecolumn-container' style='min-height:20px;'>" +
                                 "{{if hasChildRecords && hasFilteredChildRecords}}" +
                                 "<div " +
                                 "class='e-dragintend'" +
                                 "{{if filterLevel > -1}}" +
                                 "style='height:1px; float:left; width:{{:((filterLevel)*" + proxy.treeIndentLevelWidth + ")}}" + "px; display:inline-block;'>" +
                                 "{{else}}" +
                                 "style='height:1px; float:left; width:{{:((level)*" + proxy.treeIndentLevelWidth + ")}}" + "px; display:inline-block;'>" +
                                 "{{/if}}" +
                                 //indicators include for drag and drop
                                 "{{if !~_" + proxy._id + "getState() || !hasChildRecords}}" +
                                  "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                 "<span class='e-aboveIcon {{if level == 1 && hasChildRecords}} e-firstLevIconMargin {{/if}}  e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='e-belowIcon {{if level == 1 && hasChildRecords}} e-firstLevIconMargin {{/if}}  e-icon' style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='e-childIcon {{if level == 1 && hasChildRecords}} e-firstLevIconMargin {{/if}}  e-icon' style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='e-cancelIcon {{if level == 1 && hasChildRecords}} e-firstLevIconMargin {{/if}} e-icon' style='position:relative;float:right;display:none;'></span>" +
                                 "</div>" +
                                 "{{/if}}"+
                                 "</div>" +
                                 "{{else !hasChildRecords || (!hasFilteredChildRecords && hasChildRecords)}}" +
                                 "<div " +
                                 "class='e-dragintend'" +
                                 "{{if filterLevel > -1}}" +
                                 "style='height:1px; float:left; width:{{:(filterLevel*" + proxy.treeIndentLevelWidth + ")+24}}" + "px; display:inline-block;'>" +
                                 "{{else}}" +
                                 "style='height:1px; float:left; width:{{:(level*" + proxy.treeIndentLevelWidth + ")+24}}" + "px; display:inline-block;'>" +
                                 "{{/if}}" +
                                   //indicator icons included for drag and drop
                                 "{{if !~_" + proxy._id + "getState() || !hasChildRecords || !hasFilteredChildRecords}}" +
                                 "<div class={{:~_" + proxy._id + "rowName()}} >"+
                                 "<span class='e-aboveIcon e-iconMargin e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='e-belowIcon e-iconMargin e-icon' style='float:right;display:none;'></span>" +
                                 "<span class='e-childIcon e-iconMargin e-icon' style='float:right;display:none;'></span>" +
                                 "<span class='e-cancelIcon e-iconMargin e-icon' style='float:right;display:none;'></span>" +
                                 "</div>" +
                                  "{{/if}}" +
                                 "</div>" +
                                 "{{/if}}" +
                                 //indicator icons included for drag and drop
                                 "{{if ~_" + proxy._id + "getState() && hasChildRecords && hasFilteredChildRecords}}" +
                                 "<div " +
                                 "class='e-intendparent'>" +
                                 "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                 "<span class='e-aboveIcon e-icon'  style='position:relative;float:left;display:none;'></span>" +
                                 "<span class='e-belowIcon e-icon' style='float:left;display:none;'></span>" +
                                 "<span class='e-childIcon e-icon' style='float:left;display:none;'></span>" +
                                 "<span class='e-cancelIcon e-icon' style='float:left;display:none;'></span>" +
                                 "</div>" +
                                 "</div>" +
                                 "{{/if}}" +
                                 "<div " +
                                 "class='{{if expanded && hasFilteredChildRecords}}e-treegridexpand e-icon{{else hasChildRecords && hasFilteredChildRecords}}" +
                                 "e-treegridcollapse e-icon{{/if}}'" +
                                 "style='float:left;display:inline-block; font-size:10px;'>" +
                                 "</div>" +
                                 "<div class='e-cell{{if " + (column.showCheckbox && (this.model.selectionSettings.selectionType == ej.TreeGrid.SelectionType.Checkbox)) + "}} e-treecheckboxcolumn{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}} {{if " + isTextWrap + "}} e-textwrapcell{{/if}}'" +
                                 "style='overflow: hidden;padding-left:{{if ~_" + proxy._id + "isTreeColumnIndex('" + columnName + "') && isSummaryRow}}{{:(level*" + proxy.treeIndentLevelWidth + ")+24}}" + "px{{/if}};'>" +
                                '{{/if}}' +
                                "{{:~_" + proxy._id + "getCheckboxState(#data, '" + columnName + "', '" + proxy._checkboxColumnName + "')}}";
                                
                            if (!ej.isNullOrUndefined(column["format"]))
                                tdCell += "{{if !isSummaryRow && !footerSummaryRowRecord}}{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "'),'" + this.model.locale + "')}}{{else}}{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}{{/if}}";
                            else if (model.isFromGantt && column.mappingName === model.resourceInfoMapping) {
                                helpers = {};
                                helpers["_" + proxy._id + "resourceName"] = proxy._getResourceName;
                                $.views.helpers(helpers);
                                tdCell += "{{:~_" + proxy._id + "resourceName('" + proxy._id + "Object','"
                                 + model.resourceNameMapping + "','"
                                 + model.resourceUnitMapping + "','" + columnName + "')}}" +
                                "</div>" +
                                "</div>" +
                                "</td>";
                            } else if (model.isFromGantt && column.mappingName === model.predecessorMapping) {

                                helpers = {};
                                helpers["_" + proxy._id + "predecessor"] = $.proxy(proxy._getPredecessorsValue, proxy);
                                $.views.helpers(helpers);

                                tdCell += "{{:~_" + proxy._id + "predecessor(#data)}}" +
                                "</div>" +
                                "</div>" +
                                "</td>";
                            }
                            else {                               
                                tdCell += summaryTitlediv +
                                "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" + 
                                 "<div class='e-summarycell'" +
                                 "style='overflow: hidden;padding-left:25px;'>" +
                                "{{/if}}" +
                                "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}" +
                                "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                "</div>" +
                                "{{/if}}" +
                                '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                "</div>" +
                                "</div>" +
                                "</td>" +
                                '{{/if}}';
                                summaryTitlediv = "";
                            }
                            //CHECK RESOURCE COLUMN FROM GANTT OR NOT
                        } else if (model.isFromGantt && column.mappingName === model.resourceInfoMapping) {
                            helpers = {};
                            helpers["_" + proxy._id + "resourceName"] = proxy._getResourceName;
                            $.views.helpers(helpers);

                            tdCell = "<td  " +
                                 "class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}}e-hide{{/if}}'" +
                                 "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                 "{{:cellBackgroundColor}}{{else}}none{{/if}};'" +
                                 "role='row'>" +
                                 "{{:~_" + proxy._id + "resourceName('" + proxy._id + "Object','"
                                 + model.resourceNameMapping + "','"
                                 + model.resourceUnitMapping + "','" + columnName + "')}}" +
                                 "</td>";
                            //CHECK PREDECESSOR COLUMN FROM GANTT OR NOT
                        } else if (model.isFromGantt && column.mappingName === model.predecessorMapping) {

                            helpers = {};
                            helpers["_" + proxy._id + "predecessor"] = $.proxy(proxy._getPredecessorsValue, proxy);
                            $.views.helpers(helpers);

                            tdCell = "<td " +
                                "class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}}e-hide{{/if}}' " +
                                "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                "{{:cellBackgroundColor}}{{else}}none{{/if}};'" +
                                "role='row'>" +
                                "{{:~_" + proxy._id + "predecessor(#data)}}" +
                                "</td>";

                        } else if (model.isFromGantt && column.mappingName === model.durationMapping) {

                            helpers = {};
                            helpers["_" + proxy._id + "getDurationString"] = $.proxy(proxy._getDurationString, proxy);
                            $.views.helpers(helpers);

                            tdCell = "<td " +
                                "class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}}e-hide{{/if}}' " +
                                "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                "{{:cellBackgroundColor}}{{else}}none{{/if}};'" +
                                "role='row'>" +
                                "{{:~_" + proxy._id + "getDurationString(#data)}}" +
                                "</td>";

                        }
                        else if (column["editType"] == "booleanedit" && proxy._checkboxSelection && columnName == "checkboxState") {
                            tdCell = "<td " +
                                "class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-rowcell e-selectionbackground{{else}}e-rowcell{{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}}e-hide{{/if}}{{:~_" + proxy._id + "SummaryRowtdClassName()}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}} e-summaryintend {{/if}}{{if ~_" + proxy._id + "batchEditState(#data,'" + columnName + "')}}e-updatedtd e-tgupdatenotify{{/if}}'" +
                                "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                "{{:cellBackgroundColor}}{{else}}none{{/if}};text-Align: center;'" +
                                "role='row'" +
                                "> {{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + columnCount + "',#data)}}" +
                                 "<div  " +
                                  "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                  "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                  "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                  "{{/if}}" + summaryTitlediv +
                                  "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                 "<div class='e-summarycell'" +
                                 "style='overflow: hidden;padding-left:25px;'>" +
                                 "{{/if}}" +
                                  "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "', '" + proxy._checkboxColumnName + "')}}" +
                                "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                "</div>" +
                                "{{/if}}";
                        }
                        else if (proxy._isCommandColumn(column)) {                            
                            $("#" + this._id + columnCount + "_UnboundTemplate").remove();
                            var divElement = proxy._createCommandElement(column, columnCount);                            
                            tdCell = "<td " +
                           "class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-rowcell e-unboundcell e-selectionbackground {{else}}e-rowcell e-unboundcell {{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}}e-hide{{/if}}{{:~_" + proxy._id + "SummaryRowtdClassName()}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}} e-summaryintend{{/if}}'" +
                           "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                           "{{:cellBackgroundColor}}{{else}}none{{/if}};text-Align:" + textAlign + "'" +
                           "role='row'> {{if !isSummaryRow && !footerSummaryRowRecord}}{{:~_" + proxy._id + "UnboundTemplate('" + divElement.id + "')}}{{else}}{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + columnCount + "',#data)}}" +
                                 "<div  " +
                                  "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                  "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                  "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                  "{{/if}}" + summaryTitlediv +
                                  "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                 "<div class='e-summarycell'" +
                                 "style='overflow: hidden;padding-left:25px;'>" +
                                "{{/if}}" +
                                  "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "', '" + proxy._checkboxColumnName + "')}}{{/if}}" +
                                  "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                "</div>" +
                                "{{/if}}";
                            proxy._commandColumnLength = proxy._commandColumnLength + 1;
                            tdCell += "</td>";
                        }
                        else {
                            tdCell = "<td " +
                                "class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}} {{:~_" + proxy._id + "isSelectedCell(#data,'" + columnCount + "')}} {{if " + !visible + "}}e-hide{{/if}} {{:~_" + proxy._id + "SummaryRowtdClassName()}}{{if isLastSummary}} e-lastsummaryrow{{/if}}{{if ~_" + proxy._id + "isTrimCell('" + column['clipMode'] + "')}} e-treegridtrimcell{{/if}}{{if " + isTextWrap + "}} e-textwrapcell{{/if}}{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}} e-summaryintend{{/if}}{{if ~_" + proxy._id + "batchEditState(#data,'" + columnName + "')}}e-updatedtd e-tgupdatenotify{{/if}}'" +
                                "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                "{{:cellBackgroundColor}}{{else}}none{{/if}};text-Align:" + textAlign + "'" +
                                "role='row'" +
                                ">" +
                                "{{:~_" + proxy._id + "getCheckboxState(#data, '" + columnName + "', '" + proxy._checkboxColumnName + "')}}{{if footerSummaryRowRecord && ~_" + proxy._id + "collapsibleSummary('" + columnCount + "',#data)}}" +
                                 "<div  " +
                                  "class= '{{if summaryExpanded}} e-summarytreegridexpand e-icon" +
                                  "{{else}} e-summarytreegridcollapse e-icon {{/if}}'" +
                                  "style='float:left;display:inline-block; font-size:10px;'></div>" +
                                  "{{/if}}" + summaryTitlediv +
                                  "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                 "<div class='e-summarycell'" +
                                 "style='overflow: hidden;padding-left:25px;'>" +
                                "{{/if}}";

                            if (!ej.isNullOrUndefined(column["format"]))
                                tdCell += "{{if !isSummaryRow && !footerSummaryRowRecord}}{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "'),'" + this.model.locale + "')}}{{else}}{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}{{/if}}";
                            else {
                                tdCell += "{{:~_" + proxy._id + "TemplateCellValue(#data,'" + columnName + "')}}";

                            }    
                           
                            tdCell+= "{{if footerSummaryRowRecord && ~_" + proxy._id + "summaryIndendCell(" + columnCount + ")}}" +
                                "</div>" +
                                "{{/if}}</td>";
                            summaryTitlediv = "";
                        }

                        tdCellCollections += tdCell;
                    }
                    if (this._frozenColumnsLength > 0 && this._frozenColumnsLength == columnCount + 1) {
                        var frozenTrRow = "";
                        frozenTrRow += tdCellCollections;
                        frozenTrRow += "</tr>";
                        tdCellCollections = "";
                    }
                }

                /* add details column to end of header cell*/
                if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn) {
                    tdCellCollections += "<td class='{{if isSelected && ~_" + proxy._id + "SelectState()}}e-selectionbackground{{/if}} {{if isDetailsExpanded}}e-detailsrowexpand{{else}}e-detailsrowcollapse{{/if}}  " +
                                         "e-detailsrowcell{{if isLastSummary}} e-lastsummaryrow{{/if}}'>{{if !isSummaryRow && !footerSummaryRowRecord}}<div class='e-detailsiconcell e-icon {{if isDetailsExpanded}}e-detailsinfoiconhide{{else}}e-detailsinfoiconshow{{/if}}'></div>{{/if}}";
                }

                trRow += "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} " +
                      "e-treegridrowcollapse {{/if}}" +
                      "{{if isBatchDelete}}e-batchdisable {{/if}}'" +
                      "style='background-color:{{if rowBackgroundColor }}{{:rowBackgroundColor}}{{else}}none{{/if}}; height:{{:~_" + proxy._id + "getRowHeight(#data)}}px;" +
                      "display:{{if footerSummaryRowRecord}}{{:~_" + proxy._id + "summaryExpandStatus(#data)}}{{else}}{{:~_" + proxy._id + "expandStatus(#data)}}{{/if}}'" +
                      "role='treegrid'>";
                frozenTrRow = trRow + frozenTrRow;
                trRow += tdCellCollections;

                if (model.isFromGantt) {
                    trRow += "<td class='e-extendcolumn'></td></tr>";
                } else {
                    trRow += "</tr>";
                }
            }

            if (model.showDetailsRow && model.detailsTemplate)
            {
                trRow += "{{if isDetailsExpanded}}" + proxy._createDetailsRowTemplate() + "{{/if}}";
                frozenTrRow += "{{if isDetailsExpanded}}" + proxy._createDetailsRowTemplate(true) + "{{/if}}";
            }
            proxy._summaryExpanded = true;
            var templates = {};
            templates[proxy._id + "_Template"] = trRow;
            templates[proxy._id + "_JSONFrozenTemplate"] = frozenTrRow;

            $.templates(templates);

            

        },
        
        //#endregion template

        //#region template METHODS

        _gridTemplate: function (self, templateId, columnIndex) {
            var column;
            if (!ej.isNullOrUndefined(columnIndex))
                column = self.model.columns[columnIndex];
            return self._renderEjTemplate("#" + templateId, this.data, this.data.index, column);
        },

        //method for calculating the indent width for expander column
        _getIndentWidth:function(data){
            var level = data.level;
            return (level * this.treeIndentLevelWidth) + "px";
		},


        //creating template element for angulartemplate in details row template
        _createDetailRowTemplateElement: function (detailTemplate) {
            var quickReg = /^#([\w-]*)/,
                scripEle,
                match = quickReg.exec(detailTemplate), scriptReg = /^<script/i;

            var options = {
                name: "SCRIPT",
                type: "text/x-template",
                text: detailTemplate,
                id: "treeGridDetailsRowTemplateElement" + this._id
            };

            if (match && match[1])
                scripEle = document.getElementById(match[1]);
            else {
                if (scriptReg.test(detailTemplate))
                    scripEle = $(detailTemplate).get(0);
                else
                    scripEle = ej.buildTag(options.name, options.text).get(0);
            }

            scripEle.id = scripEle.id || options.id;
            scripEle.type = scripEle.type || options.type;
            $("body").append(scripEle);
            return scripEle;
        },


        //creating template element for angulartemplate in row template
        _createRowTemplateElement: function (innerHtml, id, type) {
            var scriptElement = document.createElement("script");
            scriptElement.id = id;
            scriptElement.type = type || "text/x-template";
            scriptElement.text = innerHtml;

            $("body").append(scriptElement);
            return scriptElement;
        },

        //creating a template element for angularjs template
        _createTemplateElement: function (column, text) {

            var idText = text ? "Pager" : column.headerText + $.inArray(column, this.model.columns) + "_Template",
                property = "template" in column ? "template" : "templateID" in column ? "templateID" : "angularTemplate",
                template = column[property],
                quickReg = /^#([\w-]*)/, scriptReg = /^<script/i, scripEle;

            if (property == "templateID") {
                if (template.indexOf("#") == -1)
                    template = "#" + template;
            }

            var match = quickReg.exec(template),
                options = {
                    name: "SCRIPT",
                    type: "text/x-template",
                    text: template,
                    id: (this._id + idText).replace(/(\\|[^0-9A-z-_])/g, "")
                };

            if (match && match[1])
                scripEle = document.getElementById(match[1]);
            else {
                if (scriptReg.test(template)) // branch here to handle tmpl string with SCRIPT. 
                    scripEle = $(template).get(0);
                else
                    scripEle = ej.buildTag(options.name, options.text).get(0);
            }
            scripEle.id = scripEle.id || options.id;
            scripEle.type = scripEle.type || options.type;
            $("#" + scripEle.id).remove();
            $("body").append(scripEle);
            return scripEle;
        },

        //Get checkbox state while rendering the treegrid
        _getCheckboxState: function (data, columnName, checkboxColumnName) {
            if (columnName != checkboxColumnName)
                return;
            if (data['isSummaryRow'] == true || data.summaryColumn || data.summaryRow)
                return;
            var checkboxState;
            if (data.checkboxState == "indeterminate") {
                checkboxState = '<span class="e-chkbox-wrap e-widget" style="padding-right:10px;color:#c8c8c8;">' +
                '<div class="e-chkbox-small">' +
                '<span class="e-chk-indeter"><span class="e-checkbox e-chk-image e-icon e-stop" id =' + this._id + '_checkbox' + data.ejRowId + '></span></span></div></span>';
            }
            else if (data.checkboxState == "unchecked") {
                checkboxState = '<span class="e-chkbox-wrap e-widget" style="padding-right:10px;color:#c8c8c8;">' +
                    '<div class="e-chkbox-small">' +
                    '<span class="e-chk-inact"><span class="e-checkbox e-chk-image e-icon" id =' + this._id + '_checkbox' + data.ejRowId + '></span></span></div></span>';
            }
            else {
                checkboxState = '<span class="e-chkbox-wrap e-widget" style="padding-right:10px;color:#c8c8c8;">' +
                    '<div class="e-chkbox-small">' +
                    '<span class="e-chk-act"><span class=" e-checkbox e-chk-image e-icon e-checkmark" id =' + this._id + '_checkbox' + data.ejRowId + '></span></span></div></span>';
            }
            return checkboxState;
        },

        // Healper method to check the clip mode of the cell is in 'ellipsis'.
        _checkIsTrimCell: function (clipMode) {
            var proxy = this;
            if (clipMode && clipMode == "ellipsis")
                return true;
            else
                return false;
        },
        /*Method to get row for resource view Gantt*/
        _getTemplateRowHeight: function (data) {
            if (this.model.isFromGantt && this.model.viewType == "resourceView")
                return data.eOverlapIndex * this.model.rowHeight;
            else
                return this.model.rowHeight;
        },

        _getCustomCellValue: function (data, columnName) {
            if (!ej.isNullOrUndefined(data)) {
                var splits = (columnName || "").split("."), cellValue;
                if (splits.length <= 1)
                    cellValue = data[columnName];
                else {
                    cellValue = data[splits[0]];
                    if (ej.isNullOrUndefined(cellValue)) {
                        //if (typeof cellValue === "undefined")
                        //    this._restoreCustomData(data, "", columnName);
                        return null;
                    }
                    for (var splitKeyIndex = 1; splitKeyIndex < splits.length; splitKeyIndex++) {
                        cellValue = cellValue[splits[splitKeyIndex]];
                        if (ej.isNullOrUndefined(cellValue)) {
                            //if (typeof cellValue === "undefined")
                            //    this._restoreCustomData(data, "", columnName);
                            return null;
                        }
                    }
                }
                return cellValue;
            }
        },

        _restoreCustomData: function (data, value, fieldName) {
            var proxy = this, splitIndex = 0,dataIndex = 0;
            if (ej.isNullOrUndefined(fieldName)) {
                var fieldName = proxy._cellEditingDetails.fieldName;
            }
            var splits = (fieldName || "").split("."), cellValue = {};
            if (splits.length <= 1) {
                if (!ej.isNullOrUndefined(data)) {
                    data[fieldName] = value;
                } else {
                    data = {};
                    data[fieldName] = value;
                }
            }
            else {
                if (!ej.isNullOrUndefined(data)) {
                    cellValue = data[splits[0]];
                    if (ej.isNullOrUndefined(cellValue)) {
                        cellValue = {};
                        data[splits[0]] = cellValue;
                    }
                    dataIndex = 1;
                } else {
                    data = {};
                    cellValue = data;
                    dataIndex = 0;
                }
                for (splitIndex = dataIndex; splitIndex < (splits.length - 1) ; splitIndex++) {
                    if (ej.isNullOrUndefined(cellValue[splits[splitIndex]])) {
                        cellValue[splits[splitIndex]] = {};
                    }
                    cellValue = cellValue[splits[splitIndex]];
                }
                cellValue[splits[splitIndex]] = value;
            }
                return data;
        },

        //Get the cell value while rendering the tree grid.
        _getTemplateCellValue:function(data,columnName, checkboxColumn)
        {
            var proxy = this,
                column = proxy.getColumnByField(columnName),
                editType = column.editType, i, dropdownValue,
                cellValue = proxy._getCustomCellValue(data, columnName);
               var value, text;
            if (!ej.isNullOrUndefined(cellValue)) {
                cellValue = cellValue;
            } else {
                cellValue = data.item && data.item[columnName];
            }
            if (editType == ej.TreeGrid.EditingType.Boolean && columnName == checkboxColumn) {
                cellValue = proxy._getCheckboxState(data, columnName, checkboxColumn);
            } else if (!data['isSummaryRow'] && !data["footerSummaryRowRecord"] && editType == ej.TreeGrid.EditingType.Boolean && columnName != checkboxColumn && (ej.isNullOrUndefined(column.displayAsCheckbox) || column.displayAsCheckbox)) {
                if (cellValue == "true" || cellValue == true)
                    cellValue = "<input class='e-treegrid-chekboxcell' type='checkbox' disabled='disabled' checked/>";
                else
                    cellValue = "<input class='e-treegrid-chekboxcell' type='checkbox' disabled='disabled'/>";
            }
            if (proxy.model.isFromGantt) {
                if (columnName == "work")
                    cellValue += " " + proxy.model.workUnit;
                if (columnName == "taskMode")
                    cellValue = !data.isAutoSchedule;
                if (columnName == "unit")
                    cellValue += "" + "%";
                if (columnName == "duration")
                    cellValue = proxy._getDurationString(data);
                if (columnName == "startDate" || columnName == "endDate" || columnName == "duration") {
                    if (ej.isNullOrUndefined(cellValue) || cellValue == "") {
                        cellValue = column.showNullText ? proxy.model.nullText : "" ;
                    }
                }                    
            }
            if (editType == ej.TreeGrid.EditingType.Dropdown && !data['isSummaryRow'] && !data["footerSummaryRowRecord"])
            {
                var editParams = column.editParams;
                cellValue = (cellValue != null || cellValue != undefined) ? cellValue : "";
                if(editParams && editParams.fields)
                {
                    text = editParams.fields.text ? editParams.fields.text : "text";
                    value = editParams.fields.value ? editParams.fields.value : "value";
                }
                else
                {
                    value = "value";
                    text = "text";
                }
                if (editParams && (editParams.showCheckbox || editParams.multiSelectMode)) {
                    var cellValueCollection, length, dropdownValue,
                       dropdownDataCollection = column.dropdownData,
                   delimiterChar = (editParams && !ej.isNullOrUndefined(editParams.delimiterChar)) ? editParams.delimiterChar : ",";
                    if (typeof (cellValue) == 'string' && cellValue.indexOf(delimiterChar) > -1) {
                        cellValueCollection = cellValue.split(delimiterChar);
                        length = cellValueCollection.length;
                    }
                    else
                        cellValueCollection = cellValue;
                    cellValue = "";
                    if (column.dropdownData) {
                        if (length) {
                            for (var i = 0; i < length; i++) {                                
                                dropdownValue = $.map(dropdownDataCollection, function (dropdownData) {
                                    if (!ej.isNullOrUndefined(dropdownData[value])) {
                                        if (dropdownData[value].toString() == cellValueCollection[i].toString())
                                            return dropdownData[text];
                                    }
                                    else {
                                        if (dropdownData[text] == cellValueCollection[i])
                                            return dropdownData[text];
                                    }
                                });
                                dropdownValue[0] = dropdownValue[0] ? dropdownValue[0] : "";
                                cellValue += i == 0 ? dropdownValue[0] : delimiterChar + dropdownValue[0];
                            }
                        }
                        else {
                            $.map(dropdownDataCollection, function (dropdownData) {
                                if ((!ej.isNullOrUndefined(dropdownData[value]) && cellValueCollection.toString() === dropdownData[value].toString()) || cellValueCollection === dropdownData[text]) {
                                    cellValue = dropdownData[text];
                                }
                            });
                        }
                    }
                }
                else {
                    if (column.dropdownData) {
                        var dropdownDataCollection = column.dropdownData;
                        dropdownValue = $.map(dropdownDataCollection, function (dropdownData) {
                            if (!ej.isNullOrUndefined(dropdownData[value])) {
                                if (dropdownData[value].toString() == cellValue.toString())
                                    return dropdownData[text];
                            }
                            else {
                                if (dropdownData[text].toString() == cellValue)
                                    return dropdownData[text];
                            }
                        });
                        dropdownValue[0] = ej.isNullOrUndefined(dropdownValue[0]) ? "" : dropdownValue[0];
                        cellValue = dropdownValue[0];
                    }
                }
            }           
            return cellValue;
        },

        _getCellValue: function (data, columnName) {

            var cellValue = this._getCustomCellValue(data, columnName);

            if (cellValue || cellValue===0) {
                return cellValue;
            } else {
                return data.item && data.item[columnName];
            }


        },
        _isCommandColumn: function (column) {
            var proxy = this, model = proxy.model,
            commands = column.commands;
            if ((!ej.isNullOrUndefined(commands) && commands.length == 0 )|| ej.isNullOrUndefined(commands))
                return false;
            else
                return true;
        },
        _isTreeColumnIndex: function (columnName) {
            var proxy = this, model = proxy.model,
            currentColumn = model.columns[model.treeColumnIndex];
            if (currentColumn && currentColumn.field == columnName)
                return true;
            else
                return false;
        },
        _getSummaryTitle: function () {
            var proxy = this;
            var summaryTitleValue = proxy.data.summaryRow.title;
            if (summaryTitleValue)
                return summaryTitleValue;
            else
                return proxy.data.item && proxy.data.item.summaryRow.title;
        },

        _getCellColumn: function (columnName,data) {
            var proxy = this;

            if (data.treeMappingName.indexOf(columnName) !== -1) {
                return columnName;
            }
            else {
                return "none";
            }


        },

        //GET RESOURCE NAME FOR RESOURCE COLUMN IN THE GANTT CONTROL
        _getResourceName: function (gridObject, mappingName, unitMappingName, columnName) {

            var proxy = this,
                data = proxy.data[columnName],
                count = 0,
                resourceString, resourceUnit,
                length = data && data.length,
                resourceName = [];

            if (data && data.length > 0) {
                for (count; count < length; count++) {
                    resourceString = data[count][mappingName];
                    resourceUnit = data[count][unitMappingName];
                    if (resourceUnit != 100)
                        resourceString += "[" + resourceUnit + "%]";                    
                    resourceName.push(resourceString);
                }
            }
            return resourceName;
        },

        _getDurationString: function (data) {
            var val = "";

            if (!ej.isNullOrUndefined(data.duration) && data.duration !== "") {
                if (typeof data.duration == "string") {
                    var duration = parseFloat(data.duration);
                    val += !isNaN(duration) ? parseFloat(duration.toFixed(2)) + " " : "";
                }
                else
                    val += parseFloat(data.duration.toFixed(2)) + " ";
                if (data.durationUnit != null) {
                    var multiple = data.duration != 1;
                    if (data.durationUnit == "day")
                        val += multiple ? this.model.durationUnitTexts.days : this.model.durationUnitTexts.day;
                    else if (data.durationUnit == "hour")
                        val += multiple ? this.model.durationUnitTexts.hours : this.model.durationUnitTexts.hour;
                    else
                        val += multiple ? this.model.durationUnitTexts.minutes : this.model.durationUnitTexts.minute;
                }
            }
            else {
                var column = this.getColumnByField("duration");
                val = column = column.showNullText ? this.model.nullText : val;
            }
            return val;
        },
        //GET PREDECESSOR VALUE FOR PREDECESSOR COLUMN IN THE GANTT CONTROL
        _getPredecessorsValue: function (data) {
            var predecessors = data.predecessor,
                returnVal = "",
                resultString = "";
            if (predecessors) {
                var length = predecessors.length;
                for (var i = 0; i < length; i++) {
                    var cPredecessor = predecessors[i],
                        temp = "";
                    if (cPredecessor.from != data.taskId) {
                        temp = cPredecessor.from + cPredecessor.predecessorsType;
                        if (cPredecessor.offset != 0) {
                            temp += cPredecessor.offset > 0 ? ("+" + cPredecessor.offset + " ") : (cPredecessor.offset + " ");
                            var multiple = Math.abs(cPredecessor.offset) != 1;
                            if (cPredecessor.offsetDurationUnit == "day")
                                temp += multiple ? this.model.durationUnitTexts.days : this.model.durationUnitTexts.day;
                            else if (cPredecessor.offsetDurationUnit == "hour")
                                temp += multiple ? this.model.durationUnitTexts.hours : this.model.durationUnitTexts.hour;
                            else
                                temp += multiple ? this.model.durationUnitTexts.minutes : this.model.durationUnitTexts.minute;
                        }
                        if (resultString.length > 0)
                            resultString = resultString + "," + temp;
                        else
                            resultString = temp;
                    }
                }
            }
            return resultString;
        },

        //GET THE EXPAND COLLAPSE STATUS OF TABLE ROW
        _getExpandStatusRecord: function (data) {

            var proxy = this;
            if (proxy.getExpandStatus(data) || (proxy._isHierarchyFiltering && data.isDisplay)) {
                return 'table-row';

            }

            return 'none';
        },

        //#endregion TEMPLATE METHODS

        //#region CONVERT HIERARCHICAL RECORDS TO FLAT RECORDS
        //CREATE FLATRECORDS FROM HIERARCHICAL DATASOURCE
        _createRecords: function (dataSource) {

            var proxy = this,
                model = proxy.model,
                ids=proxy.model.ids,
                flatRecords = model.flatRecords,
                length = dataSource.length,
                count = 0,
                expanded = true,
                parentRecord,
                parentRecords = model.parentRecords;

            if (!model.isFromGantt && model.enableCollapseAll) 
                expanded = false;            

            for (count; count < length; count++) {

                parentRecord = proxy._createRecord(dataSource[count], 0, null, expanded);
                parentRecord["hierarchyRowIndex"] = count;
                proxy._storedIndex++;

                //CHECK ALTROW ENABLED OR NOT
                
                parentRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                

                //CHECK SORTING ENABLED OR NOT
                
                parentRecords.push(parentRecord);
                

                parentRecord.index = proxy._storedIndex;
                ids[proxy._storedIndex] = parentRecord.index;
                flatRecords.push(parentRecord);
                if (parentRecord.hasChildRecords)
                    proxy._parentRecords.push(parentRecord);
                parentRecord.childRecords && proxy._addNestedRecords(parentRecord.childRecords);

                if (this.dataSource() instanceof ej.DataManager && proxy._isLoadonDemand) {
                    if (parentRecord.item[model.hasChildMapping] || proxy._assignedParendIDs.indexOf(parentRecord.item[model.idMapping]) != -1) {
                        parentRecord.hasChildRecords = true;
                        parentRecord.childRecords = [];
                        parentRecord.isChildRetrieved = false;
                    }
                }
            }
            if (model.allowPaging && !proxy._isRemoteData) {
                proxy._updateTotalRecordsCount();
            }
            /* assign records length as max index */
            proxy._maxRowIndex = this.model.flatRecords.length;
        },

        _updateTotalRecordsCount : function(){
            var proxy = this,
                model = proxy.model,
                flatRecords = model.flatRecords;
            if (!ej.isNullOrUndefined(model.pageSettings.totalRecordsCount) && model.pageSettings.totalRecordsCount <= flatRecords.length) {
                var totalRecordsCount = model.pageSettings.totalRecordsCount;
                model.flatRecords = flatRecords.slice(0, totalRecordsCount);
                if (model.flatRecords[totalRecordsCount - 1].hasChildRecords) {
                    model.flatRecords[totalRecordsCount - 1].hasChildRecords = false;
                    model.flatRecords[totalRecordsCount - 1].expanded = false;
                    model.flatRecords[totalRecordsCount - 1].childRecords = null;
                    model.flatRecords[totalRecordsCount - 1].hasFilteredChildRecords = false;
                }
            }
            else if (ej.isNullOrUndefined(model.pageSettings.totalRecordsCount))
                model.pageSettings.totalRecordsCount = flatRecords.length;
        },

        //CREATE RECORD OBJECT
        _createRecord: function (data, level, parentItem, expanded) {

            var proxy = this,
                record,
                childDataSource = data[proxy.model.childMapping], model = this.model;

            //To create and maintain record collections for multi select dropdownedit of columns
            record = $.extend({}, data);
            var tempData;
            if (model.parentIdMapping && proxy._createdAt === "load") {
                var id = data[model.idMapping],
                   index = proxy._taskIds.indexOf(id);
                record.item = (index > -1) ? proxy._retrivedData[index] : [];              
            }
            else {

                if (this.dataSource() instanceof ej.DataManager && this.dataSource().dataSource.json && this.dataSource().dataSource.offline && proxy._createdAt === "load") {

                    if (model.parentIdMapping) {
                        var id = data[model.idMapping],
                           index = proxy._taskIds.indexOf(id);
                        record.item = (index > -1) ? proxy._retrivedData[index] : [];
                    }
                    else {
                        record.item = data;
                    }
                }
                else {
                    record.item = data;
                }
            }
            record.parentItem = parentItem;
			record.childRecords = (childDataSource && childDataSource.length > 0) && proxy._createChildRecords(childDataSource, level + 1, record);
			record.hasChildRecords = (childDataSource && childDataSource.length > 0) ? true : false;
			if (model.expandStateMapping && record.hasChildRecords) {
			    if (!ej.isNullOrUndefined(data[model.expandStateMapping])) {
			        record[model.expandStateMapping] = record.item[model.expandStateMapping] = data[model.expandStateMapping];
			    } else if (ej.isNullOrUndefined(data[model.expandStateMapping]) && !model.enableCollapseAll) {
			        record[model.expandStateMapping] = record.item[model.expandStateMapping] = data[model.expandStateMapping] = true;
			    } else if (ej.isNullOrUndefined(data[model.expandStateMapping]) && model.enableCollapseAll) {
			        record[model.expandStateMapping] = record.item[model.expandStateMapping] = data[model.expandStateMapping] = false;
			    }
			    var expandStateMapping = data[model.expandStateMapping];
			}
			else
			    var expandStateMapping = true;
			record.expanded = (expanded !== undefined && childDataSource !== undefined && record.hasChildRecords == true
                && expandStateMapping) ? expanded : expandStateMapping && childDataSource ? childDataSource.length > 0 : false;
            record.dragState = true;
            record.isSelected = false;
            record.hasFilteredChildRecords = true;//Used to Indicates Parent records can expandable or not on Filtering operation
            record.level = level;
            record.canDrag = true;
            record.canDrop = true;
            record.cellBackgroundColor = null;
            record.rowBackgroundColor = null;
            record.textWrapRowHeight = model.rowHeight;
            record.treeMappingName = [];
            if (model.showDetailsRow && model.detailsTemplate) {
                record.isDetailsExpanded = model.showDetailsRowInfoColumn ? false : true;
            } else {
                record.isDetailsExpanded = false;
            }
            if (!model.isFromGantt)
                record.ejRowId = ++proxy._idGenerator;
            record.checkboxState = "unchecked";
            delete record[model.childMapping];
            return record;
        },


        //CREATE CHILD GANTT RECORDS
        _createChildRecords: function (childDataSource, level, parentItem) {

            var proxy = this,
                model=proxy.model,
                records = [],
                count = 0,
                length = childDataSource.length,
                record = null,
                expanded,
                childRecord;

            if (!model.isFromGantt && model.enableCollapseAll)
                expanded = false;

            for (count = 0; count < length; count++) {

                record = childDataSource[count];

                if (record) {
                    childRecord = proxy._createRecord(record, level, parentItem, expanded);
                    childRecord["hierarchyRowIndex"] = count;
                    records.push(childRecord);
                }
            }

            return records;

        },


        //ADD NESTED RECORDS TO FLATRECORDS
        _addNestedRecords: function (childDataSource) {

            var proxy = this,
                model = proxy.model,
                ids=proxy.model.ids,
                flatRecords = model.flatRecords,
                updatedRecords = model.updatedRecords,
                count = 0,
                length = childDataSource.length,
                records = [],
                record;
                

            for (count = 0; count < length; count++) {

                record = childDataSource[count];
                proxy._storedIndex++;

                //CHECK ALTROW ENABLED OR NOT
                
                record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                

                record.index = proxy._storedIndex;
                ids[proxy._storedIndex] = record.index;
                flatRecords.push(record);
                updatedRecords.push(record);
                if (record.hasChildRecords)
                    proxy._parentRecords.push(record);
                records = record.childRecords;

                if (records) {

                    var j = 0,
                        recordlength = records.length,
                        childRecord = null;

                    for (j = 0; j < recordlength; j++) {

                        childRecord = records[j];
                        proxy._storedIndex++;     
                        childRecord.index = proxy._storedIndex;
                        ids[proxy._storedIndex] = childRecord.index;
                        //CHECK ALTROW ENABLED OR NOT
                        
                        childRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        
                        flatRecords.push(childRecord);
                        updatedRecords.push(record);
                        if (childRecord.hasChildRecords)
                            proxy._parentRecords.push(childRecord);
                        childRecord.childRecords && proxy._addNestedRecords(childRecord.childRecords);
                    }
                }
            }
        },

        //EVENT FOR COLUMN RESIZE
        _enableColumnResizeEvents: function () {

            var proxy = this;

            if (proxy.model.allowColumnResize) {
                proxy._on(proxy.element, "mousemove", proxy._mouseMove);
            } else {
                proxy._off(proxy.element, "mousemove", proxy._mouseMove);
            }
        },

        //#endregion CONVERT HIERARCHICAL RECORDS TO FLAT RECORDS

        //#region METHOD FOR EVENTS

        //INITIALIZE THE EVENTS
        _wireEvents: function () {

            var proxy = this,
                model = proxy.model,
                matched = jQuery.uaMatch(navigator.userAgent);

            //proxy._$gridContent.bind("scroll", $.proxy(proxy._onScroll, proxy));
            if (ej.isTouchDevice()) {
                proxy._on(proxy.element, "tap", proxy._onRecordClick);// ".e-gridcontent",
                proxy._on(proxy.element, "doubletap", ".e-gridcontent > div", proxy._onRecordDoubleClick);
                proxy._on(proxy.element, "tap", proxy._onClick);
            }
            else {
                proxy._on(proxy.element, "click", proxy._onRecordClick);// ".e-gridcontent",
                proxy._on(proxy.element, "dblclick", ".e-gridcontent > div", proxy._onRecordDoubleClick);
                proxy._on(proxy.element, "click", proxy._onClick);
            }            
            proxy._on(proxy.element, "keydown", proxy._keyDown);
            proxy._on(proxy.element, "click", "#" + proxy._id + "e-gridheader", proxy._onHeaderClick);
            proxy._on(proxy.element, "mousedown touchstart", "#" + proxy._id + "e-gridheader", proxy._headerMouseDown);
            proxy._on($(document), "mousedown touchstart", proxy._mouseDownHandler);
            proxy._on(proxy.element, "mousedown touchstart", ".e-gridcontent", proxy._contentMouseDown);            
            
            if (this.model.isResponsive)
                this._on(this.element, "swipeleft swiperight", ".e-gridcontent div > .e-table", $.proxy(this._swipeAction, this));

            if (proxy.model.allowColumnResize)
                proxy._on(proxy.element, "mousemove", proxy._mouseMove);
            //Method binding for filtering data.
            if (model.filterSettings.filterBarMode == "immediate") {
                proxy._on(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox")._on(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
            }
            proxy._on(proxy.element, "focus click", ".e-filterbarcell input", proxy._filterBarClose);
            proxy._on(proxy.element, "click", ".e-filterbarcell .e-cancel", proxy._filterBarClose);
            if (proxy.model.showGridCellTooltip || proxy.model.showGridExpandCellTooltip) {
                proxy._on(proxy.element, "mousemove", ".e-rowcell", proxy._mouseHover);
            }
            if (proxy.model.showGridCellTooltip) {
                proxy._on(proxy.element, "mousemove", ".e-headercelldiv", proxy._headermouseHover);
            }
            proxy._on(proxy.element, "mouseleave", ".e-rowcell", proxy._cellMouseLeave);
            proxy._on(proxy.element, "mouseleave", proxy._cellMouseLeave);            
            proxy._on(proxy.element, "mouseleave", ".e-headercelldiv", proxy._headerMouseLeave);
            proxy._on(proxy.element, "mouseleave", proxy._headerMouseLeave);
            

            //MOUSE WHEEL EVENT FOR GANTT CONTROL
            if (model.isFromGantt || this._frozenColumnsLength) {
                proxy._on(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);
            }

            if (model.enableResize && model.isResponsive && !model.isFromGantt) {
                proxy._on($(window), "resize", proxy._windowResize);
            }
            proxy._enableEditingEvents();
            this._on(this.element, "click", ".e-unboundcelldiv button", this._unboundClickHandler);
            //Mouse right click event for tree grid


            if (model.contextMenuSettings.showContextMenu) {
                proxy._on(proxy.element, "keyup contextmenu taphold", this._preventContextMenu);
            }
        },
        /* prevent context menu action by menu option key*/
        _preventContextMenu: function (e) {
            
            if (e.keyCode == 93 || e.type == "contextmenu" || e.type == "taphold") {
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        },
        _mouseDownHandler:function(e)
        {
            var proxy = this,
                target = $(e.target);
            //Check the whether the mouse down on column menu and column menu items
            if (target.closest("div.e-tgheaderContextmenu").length == 0 &&target.closest("div.e-columnmenuitem").length == 0 && target.closest("div.e-columnMenuListDiv").length == 0 && ((!target.hasClass("e-columnmenu-icon") && !target.hasClass("e-column-filter-icon") && !target.hasClass("e-column-filtered-icon")) || $(e.target).closest("#" + proxy._id).length == 0) && target.closest("div.e-" + proxy._id + "filterStringSelector").length == 0 && target.closest("div#" + proxy._id + "filterStringDropDown_popup_list_wrapper").length == 0
                && target.closest("div#" + proxy._id + "filterStringInputBox_suggestion").length == 0 && target.closest("div.e-" + proxy._id + "filterNumericSelector").length == 0 && target.closest("div#" + proxy._id + "filterNumericDropDown_popup_list_wrapper").length == 0 &&
                target.closest("div.e-" + proxy._id + "filterDatepickerSelector").length == 0 && target.closest("div#" + proxy._id + "filterDatepickerDropDown_popup_list_wrapper").length == 0 &&
                target.closest("div#e-" + proxy._id + "filterDatepickerInputBox").length == 0 && target.closest("div.e-" + proxy._id + "filterDatetimepickerSelector").length == 0 && target.closest("div#" + proxy._id + "filterDatetimepickerDropDown_popup_list_wrapper").length == 0 &&
                target.closest("div#" + proxy._id + "filterDatetimepickerInputBox_popup").length == 0 && target.closest("div.e-" + proxy._id + "filterDropdownSelector").length == 0 && target.closest("div#" + proxy._id + "filterDropdownDropDown_popup_list_wrapper").length == 0 && target.closest("div.e-" + proxy._id + "filterBooleanSelector").length == 0 &&
                target.closest("div.e-excelfilter").length == 0 && !proxy._isExcelFilterClick(target))
                proxy._clearColumnMenu();
            //Check the whether the mouse down on context menu items and header context menu items
            if (!(target.hasClass("e-tgheaderContextmenu"))) {
                if (target.closest("div.e-menuitem").length == 0 && target.closest("div.e-columnmenuselection").length == 0 && target.closest("div.e-columnMenuListDiv").length == 0)
                    proxy._clearContextMenu();
            }
            //Check the detail template items on mouse down event
            if (target.closest("div.e-detailscellwrapperfly").length == 0)
                proxy._removeDetailsRow(e);            
        },

        _isExcelFilterClick: function (target)
        {
            var proxy = this,
                id = proxy._id;
            if (target.closest("#" + id + "number_CustomDrop1_popup").length == 0 && target.closest("#" + id + "number_CustomDrop2_popup").length == 0 &&
               target.closest("#" + id + "datetime_CustomDrop1_popup").length == 0 && target.closest("#" + id + "datetime_CustomDrop2_popup").length == 0 &&
               target.closest("#" + id + "date_CustomDrop1_popup").length == 0 && target.closest("#" + id + "date_CustomDrop2_popup").length == 0 &&
               target.closest("#" + id + "string_CustomDrop1_popup").length == 0 && target.closest("#" + id + "string_CustomDrop2_popup").length == 0 &&
               target.closest("#" + id + "datetime_CustomValue2_popup").length == 0 && target.closest("#" + id + "datetime_CustomValue1_popup").length == 0 &&
               target.closest("#e-" + id + "date_CustomValue2").length == 0 && target.closest("#e-" + id + "date_CustomValue1").length == 0 &&
                target.closest("#" + id + "string_CustomValue1_suggestion").length == 0 && target.closest("#" + id + "string_CustomValue1_suggestion").length == 0)
                return false;
            else
                return true;
        },
        // Auto scroll the content if mouse move out of grid area while dragging a row.
        _mouseout: function (e) {
            var proxy = this,
                model = proxy.model,
                topvalue, bottomScrollPos, topScrollPos,
                position = proxy._gridPosition,
                top = position.top, contentHeight = proxy._gridHeight,
                height = proxy._$gridHeaderContent.outerHeight() + top;

            if (model.toolbarSettings.showToolbar) {
                var id = model.isFromGantt ? proxy._id.replace("ejTreeGrid", "") : proxy._id;
                height += $("#" + id + "_toolbarItems").length > 0 ? $("#" + id + "_toolbarItems").outerHeight() : 0;
            }            
            if (model.allowPaging) {
                contentHeight = contentHeight - proxy.element.find(".e-pager").first().outerHeight();
            }
            if (model.showTotalSummary) {
                contentHeight = contentHeight - proxy._$totalSummaryRowContainer.outerHeight();
            }
            if (proxy._dragMouseDown) {
                var posx = position.left, posy = e.pageY;

                if (proxy.model.isFromGantt) {
                    var id = proxy._id.replace("ejTreeGrid", "");
                    var ganttbody = $("#"+id).find(".e-ganttviewerbodyContianer"),
                        isVscroll = ganttbody.ejScroller("isVScroll"),
                        max = isVscroll ? ganttbody.find(".e-content").get(0).scrollHeight - 1 : 0;
                    bottomScrollPos = ganttbody.ejScroller("isHScroll") ? (contentHeight + top - (model.rowHeight / 2) - 18) :
                                        (contentHeight + top - (model.rowHeight / 2));
                    }
                else{
                    var max = proxy.isVScroll() ? proxy.getMaxScrollHeight() + 1 : 0;
                    bottomScrollPos = proxy.getScrollElement().ejScroller("isHScroll") ? (contentHeight + top - (model.rowHeight / 2) - 18) :
                                           (contentHeight + top - (model.rowHeight / 2));
                }
                    topScrollPos = height + (model.rowHeight/2);
                   
                    //condition to whether the mouse pointer below the content.
                if (posy > bottomScrollPos) {
                        //Condition to check whether the scroller reaches the end.

                        if (max > (proxy._scrollTop)) {
                            if (!proxy._timerDragDown) {

                                //Timer started for scrolling the content till the end.
                                proxy._timerDragDown = window.setInterval(function () {
                                    if (max > proxy._scrollTop) {
                                        topvalue = proxy._scrollTop + 20;
                                        topvalue = topvalue > max ? max : topvalue;
                                        if (proxy.model.isFromGantt)
                                            ganttbody.ejScroller("scrollY", topvalue, true);
                                        else
                                            proxy.getScrollElement().ejScroller("scrollY", topvalue, true);                       
                                    }                                    
                                }, 100);
                            }
                        }
                        else {
                            proxy._timerDragDown && (proxy._timerDragDown = window.clearInterval(proxy._timerDragDown));
                        }
                    }
                    else {
                    proxy._timerDragDown && (proxy._timerDragDown = window.clearInterval(proxy._timerDragDown));
                    }
                    //condition to check if the mouse points above to content area.
                if (posy < topScrollPos && proxy._scrollTop != 0) {
                        if (proxy._scrollTop > 0) {
                            if (!proxy._timerDragUp) {
                                proxy._timerDragUp = window.setInterval(function () {
                                    if (proxy._scrollTop > 0) {
                                        topvalue = proxy._scrollTop - 20;                                        
                                        if (topvalue < 0)
                                            topvalue = 0;
                                        if (proxy.model.isFromGantt)
                                            ganttbody.ejScroller("scrollY", topvalue, true);
                                        else
                                            proxy.getScrollElement().ejScroller("scrollY", topvalue, true);
                                    }
                                }, 100);
                            }
                        }
                        else {
                            proxy._timerDragUp && (proxy._timerDragUp = window.clearInterval(proxy._timerDragUp));
                        }
                    }
                    else {
                    proxy._timerDragUp && (proxy._timerDragUp = window.clearInterval(proxy._timerDragUp));
                    }                
            }
        },       
        // This method is to get the row range to enable the drag row indicators consequently.
        getRange: function (record, index,$tr) {
            
            var proxy = this,

                model = proxy.model, rowHeight, position,
                roundOff = 0, height,rowTop = 0;
            rowHeight = $tr.length == 0 ? model.rowHeight : $tr[0].offsetHeight,
            position = proxy.getOffsetRect($("#" + proxy._id)[0]),
                height = proxy._$gridHeaderContent.outerHeight() + position.top;
                rowHeight += proxy._detailsRowHeight;
            if (!model.enableVirtualization)
                proxy._scrollTop = proxy.getScrollElement().ejScroller("scrollTop");
            if (model.toolbarSettings.showToolbar)
                height += $("#" + proxy._id + "_toolbarItems").length > 0 ? $("#" + proxy._id + "_toolbarItems").outerHeight() : 0;
            if (model.allowTextWrap) {
                if ($tr.length == 0) {
                    $tr = proxy._lastRow;
                }
                rowTop = $tr.offset().top;
            }
            else if (proxy._scrollTop != 0) {
                roundOff = rowHeight - proxy._scrollTop % rowHeight;
                rowTop = (index * rowHeight) + height + roundOff;
            } else {
                rowTop = (index * rowHeight) + height + roundOff;
            }
            // 7 is to equalize the height.
            var rowBottom = $tr.length == 0 ? rowTop + model.rowHeight : rowTop + $tr[0].offsetHeight,
                difference = rowBottom - rowTop,
                divide = difference / 3,
                topRowSegment = rowTop + divide,
                middleRowSegment = topRowSegment + divide,
                bottomRowSegment = middleRowSegment + divide,
                position = {
                    topRowSegment: topRowSegment,
                    middleRowSegment: middleRowSegment,
                    bottomRowSegment: bottomRowSegment
                };
            return position;
        },
        //Child validation to display the child indicator.
        _childValidation: function (record) {            
            var childIndex,
                proxy = this,
                draggedRecord = proxy._draggedRecord,
                parentItem = draggedRecord.parentItem,
                childRecords;
            if (parentItem) {
                childRecords = parentItem.childRecords;
                childIndex = childRecords.indexOf(this._draggedRecord);
                if (record == parentItem && childIndex != -1)
                    this._childItem = true;
            }
        },
        _isColumnHidable: function (column) {
            var model = this.model, visibleFrozenColumn = [], visibleUnfrozenColumn = [], returnValue = true;

            visibleFrozenColumn = this._frozenColumns.filter(function (value) {
                return value.visible == true;
            });
            visibleUnfrozenColumn = this._unFrozenColumns.filter(function (value) {
                return value.visible == true;
            });
            if (visibleFrozenColumn.length == 1 && visibleFrozenColumn[0].field == column.field) {
                returnValue = false;
            }
            if (visibleUnfrozenColumn.length == 1 && visibleUnfrozenColumn[0].field == column.field) {
                returnValue = false;
            }
            return returnValue;
        },
        _updateColumnMenuVisibility: function () {
            var model = this.model, visibleFrozenColumn = [], visibleUnfrozenColumn = [];
            if (model.showColumnChooser || model.contextMenuSettings.showContextMenu) {
                visibleFrozenColumn = this._frozenColumns.filter(function (value) {
                    return value.visible == true;
                });
                visibleUnfrozenColumn = this._unFrozenColumns.filter(function (value) {
                    return value.visible == true;
                });
                if (visibleFrozenColumn.length == 1) {
                    var checkBoxData = $("#" + this._id + "_columnMenu_" + visibleFrozenColumn[0].field.split('.').join("")).data("ejCheckBox");
                    if (checkBoxData)
                        checkBoxData.option("enabled", false);
                }
                if (visibleUnfrozenColumn.length == 1) {
                    var checkBoxData = $("#" + this._id + "_columnMenu_" + visibleUnfrozenColumn[0].field.split('.').join("")).data("ejCheckBox");
                    if (checkBoxData)
                        checkBoxData.option("enabled", false);
                }
            }
        },
        //to check valid drop
        _validDrop: function (draggedRecords, currentRecord) {
            var proxy = this;
            draggedRecords.filter(function (item) {
                if (item.hasChildRecords && currentRecord.dragState == true) {
                    var valid = $.inArray(currentRecord, item.childRecords);
                    if (valid == -1) {
                        proxy._validDrop(item.childRecords, currentRecord);
                    }
                    else {
                        currentRecord.dragState = false;
                        return;
                    }
                }
            });
        },

        //This method is to render the tooltip while dragging.
        _renderToolTip: function (e, element, args, $target) {
     
            var proxy = this,
                model = this.model,
                positions = [],
                doc = document,
                item,
                columns = proxy.model.columns,
                tooltipItemLength = model.dragTooltip.tooltipItems.length,
                 recordIndex,
                $tr = $target.closest('tr');

            // disable all the icons initially.
            $(".e-dragintend").find(".e-icon").css({ "display": "none" });
            $(".e-intendparent").find(".e-icon").css({ "display": "none" });
            $(".e-error").css({ "display": "none" });
            $(".e-treegridexpand").css({ "display": "inline-block" });
            $(".e-treegridcollapse").css({ "display": "inline-block" });

            if (proxy._isHierarchyFiltering)
                $("#" + proxy._id + "e-gridcontent").css({ 'cursor': 'no-drop' });
            //condition for allow down record on empty records
            if (!proxy._isHierarchyFiltering) {
                if ($tr.length == 0 && ($target.hasClass("e-movablecontainer") || $target.hasClass("e-gridcontent") || $target.hasClass("e-content"))) {
                    recordIndex = proxy._recordIndex;
                
                if (!proxy._currentRecord.parentItem) {
                    if (proxy._currentRecord.hasChildRecords)
                        if (proxy._currentRecord.expanded)
                            proxy._lastRow.find(".e-treegridexpand").css({ "display": "none" });
                        else
                            proxy._lastRow.find(".e-treegridcollapse").css({ "display": "none" });
                }
            }
        }
            else {
                recordIndex = proxy.getIndexByRow($tr);
                proxy._currentRecord = model.currentViewData[recordIndex]
            }

            var positions = {}, records, dragRecIndex, index,
            updatedRecords = proxy.getUpdatedRecords();
            proxy._insertAbove = false;
            proxy._insertAsChild = false;
            proxy._insertBelow = false;
            proxy._childItem = false;
            
          item = proxy._draggedRecord;

            if (recordIndex != proxy._draggedIndex)
                proxy._displayContextMenu = false;
                        
      // to get the record index to find its mouse position.            
            var records = proxy.getExpandedRecords(updatedRecords);
                proxy.getVisibleRange();
                var currentViewData = records.slice(proxy._visibleRange.top, proxy._visibleRange.bottom);
                if (proxy._scrollTop != 0) {
                    dragRecIndex = updatedRecords.indexOf(proxy._draggedRecord) - 1;
                    index = currentViewData.indexOf(proxy._currentRecord) - 1;
                }
                else {
                    dragRecIndex = updatedRecords.indexOf(proxy._draggedRecord);
                    var index = currentViewData.indexOf(proxy._currentRecord);
                }
            var currentIndex = updatedRecords.indexOf(proxy._currentRecord),
            isNextRecord = proxy._currentRecord && proxy._checkPrevNextRecord(proxy._draggedRecord, currentIndex, proxy._currentRecord.level, "next"),
            isPrevRecord = proxy._currentRecord && proxy._checkPrevNextRecord(proxy._draggedRecord, currentIndex, proxy._currentRecord.level, "prev"),
            posx = proxy._gridPosition.left, posy = e.pageY;

            if (proxy._currentRecord)
                positions = proxy.getRange(proxy._currentRecord, index, $tr);
                                           
            var isTopRowSegment = posy <= positions.topRowSegment,
                isMiddleRowSegment = (posy > positions.topRowSegment && posy <= positions.middleRowSegment),
                isBottomRowSegment = (posy > positions.middleRowSegment && posy <= positions.bottomRowSegment);
            /* check current record or else render previous records tootip*/
            if (proxy._currentRecord) {
                proxy.data = proxy._currentRecord;
                var classname = this._getrowName();
                proxy._childValidation(proxy._currentRecord);
                // This condition is to disapper the expand collapse icon if it is top most level.
                if (!proxy._isHierarchyFiltering) {
                    if (!proxy._currentRecord.parentItem) {
                        if (proxy._currentRecord.hasChildRecords)
                            if (proxy._currentRecord.expanded)
                                $tr.find(".e-treegridexpand").css({ "display": "none" });
                            else
                                $tr.find(".e-treegridcollapse").css({ "display": "none" });
                    }
                    else {
                        if (proxy._currentRecord.hasChildRecords)
                            if (proxy._currentRecord.expanded)
                                $tr.find(".e-treegridexpand").css({ "display": "inline-block" });
                            else
                                $tr.find(".e-treegridcollapse").css({ "display": "inline-block" });
                    }
                    proxy._validDrop(proxy._draggedRecords, proxy._currentRecord);
                    //To get the dropPosition
                    if (proxy._currentRecord.dragState == true && args.canDrop != false) {
                        if (isTopRowSegment && !isPrevRecord) {
                            args.dropPosition = "insertAbove";
                        }
                        else if (isMiddleRowSegment && proxy._childItem == false) {
                            args.dropPosition = "insertAsChild";
                        }
                        else if (isMiddleRowSegment && proxy._childItem == true) {
                            args.dropPosition = "invalidPosition";
                        }
                        else if (isBottomRowSegment && !isNextRecord || $tr.length == 0) {
                            args.dropPosition = "insertBelow";
                        }
                        else {
                            args.dropPosition = "invalidPosition";
                        }
                    }
                    else {
                        args.dropPosition = "invalidPosition";
                    }
                    proxy._trigger("rowDrag", args);
                    //To render the indicators according to its position.
                    if (proxy._currentRecord.dragState == true && args.canDrop != false) {
                        if (isTopRowSegment && !isPrevRecord) {
                            $(proxy.element).find("." + classname).find(".e-aboveIcon").css({ "display": "block" });
                            proxy._insertAbove = true;
                        }
                        else if (isMiddleRowSegment && proxy._childItem == false) {
                            $(proxy.element).find("." + classname).find(".e-childIcon").css({ "display": "block" });
                            proxy._insertAsChild = true;
                        }
                        else if (isMiddleRowSegment && proxy._childItem == true) {
                            $(proxy.element).find("." + classname).find(".e-cancelIcon").css({ "display": "block" });
                            proxy.cancelDrop = true;
                        }
                        else if (isBottomRowSegment && !isNextRecord || $tr.length == 0) {
                            $(proxy.element).find("." + classname).find(".e-belowIcon").css({ "display": "block" });
                            proxy._insertBelow = true;
                        }
                    }
                    else {
                        $(proxy.element).find("." + classname).find(".e-cancelIcon").css({ "display": "block" });
                        proxy.cancelDrop = true;
                    }
                }
            }

            $(proxy._dragmouseOverTooltip).css({
                "top": (posy + 20) + "px", "left": (posx + 15) + "px",
                'position': 'absolute',
                'z-index': proxy.getMaxZIndex() + 1,
                'padding': '0',
                'border-right': '0',
                'border-bottom': '0',
                'border-radius': '3px',
                'height': "auto",
            });                            
        },

        //Used for check whether the dragged record is previous or next to current record based on position argument
        //If it is previous or next to current record, this method return true otherwise return false.
        _checkPrevNextRecord:function(record,index,level, position)
        {
            var proxy = this,
                model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords(),
                currentIndex = index;
            position == "next" ? index++ : index--;
            while(updatedRecords[index])
            {
                if (updatedRecords[index].level == level)
                {
                    if (updatedRecords[index] == record && record.parentItem == updatedRecords[currentIndex].parentItem)
                        return true;
                    return false;
                }
                position == "next" ? index++ : index--;
            }
            return false;
        },
        //Tooltip template for row drag and drop
        _createDragTooltipTemplate: function () {
            var proxy = this,
                td,               
                template = proxy.model.dragTooltip.tooltipTemplate,
                parentTr;
            if (template) {
                var templateString = template.charAt(0);
             
                if (templateString == "#")
                     parentTr = $(proxy.model.dragTooltip.tooltipTemplate)[0].innerHTML;
                else
                     parentTr = proxy.model.dragTooltip.tooltipTemplate;
            }            
            var templates = {};
            templates[proxy._id + "dragtooltipTemplate"] = parentTr;
            $.templates(templates);
        },       

        //render the cancel icon if drag over its child.
		_updateDragStateFalse: function (record, count) {
		    var currentRecord, proxy = this, length;
		    record.dragState = false;		   
		    if (record.hasChildRecords) {
		        length = record.childRecords.length;
		        for (var i = 0; i < length; i++) {
		            currentRecord = record.childRecords[i];
		            currentRecord.dragState = false;
		            if (currentRecord.hasChildRecords) {
		                count = proxy._updateDragStateFalse(currentRecord, count);
		            }
		        }
		    }
		    return;
		},

        //Turn all the records dragState to true once dragging finished.
		_updateDragStateTrue: function (record, count) {
		    var currentRecord, proxy = this, length;
		    if (!ej.isNullOrUndefined(record)) {
		        record.dragState = true;
		        if (record.hasChildRecords) {
		            length = record.childRecords.length;
		            for (var i = 0; i < length; i++) {
		                currentRecord = record.childRecords[i];
		                currentRecord.dragState = true;
		                if (currentRecord.hasChildRecords) {
		                    count = proxy._updateDragStateTrue(currentRecord, count);
		                }
		            }
		        }
		    }
		    return;
		},

		// Get XY coordinates for touch and non-touch device
		_getCoordinate: function (evt) {
		    var coor = evt;
		    if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend" || evt.type== "taphold" || evt.type == "tap")
		        coor = evt.originalEvent.changedTouches[0];
		    return coor;
		},

        // Get current target element by mouse position
        // window.pageXOffset && window.pageYOffset is used to find the accurate element position in IPad/IPhone
		_getElementByPosition: function (x, y) {
		    return $(document.elementFromPoint((x - window.pageXOffset), (y - window.pageYOffset)));
		},      

        //Mouse down action for to display a Drag tooltip   
		dragRecord: function (e) {
            /*verify ctlkey and shift key and mouse button*/
		    if (e.ctrlKey || e.shiftKey || e.which == 2 || e.which == 3)
		        return true;
		    
		    var proxy = this,
               model=proxy.model,
               recordIndex,
               args = {},               
               coord = proxy._getCoordinate(e),
               $target = $(e.target),
               $tr = $target.closest('tr'),
               record,
               tooltipbody,
		       updatedRecords = proxy.getUpdatedRecords(), setCanDrag = true;

		    window.clearTimeout(this._dragTooltipTimer);
		    this._dragTooltipTimer = null;

		    if (!proxy._mouseDown) {		       
		        return true;
		    }
                        
		    if (!$tr.hasClass("e-editedrow") && proxy.editFormValidate()) {
                if (model.selectionSettings.selectionMode == "cell")
                    proxy.clearSelection();
		       
		        proxy._clearColumnMenu();		        
		        proxy._removeDetailsRow();
		        proxy._saveCellHandler(e);
		        

		        proxy._posx = coord.pageX;
		        proxy._posy = coord.pageY;

		        if ($target.hasClass("e-summaryrowcell") || $target.hasClass("e-footersummaryrowcell")
                || $target.hasClass("e-summarytitle") || $target.closest("tr").hasClass("e-detailsrowexpanded"))
		            return;
		        if (!model.enableVirtualization)
		            proxy._currentIndex = proxy.getIndexByRow($tr);
		        else {
		            proxy._currentIndex = proxy.getIndexByRow($tr);
		            record = model.currentViewData[proxy._currentIndex];
		            proxy._currentIndex = updatedRecords.indexOf(record);
		        }
		        proxy._draggedIndex = recordIndex = proxy.getIndexByRow($tr);
		        proxy._draggedRecord = proxy.model.currentViewData[recordIndex];
		        if (ej.isNullOrUndefined(proxy._draggedRecord)) return;
		        args.draggedRowIndex = recordIndex;
		        args.draggedRow = proxy._draggedRecord;
                //to get dragged row collection for selected items
		        if (!model.isFromGantt && ((model.selectionSettings.selectionType == "checkbox" && model.selectedItems.length > 1 && model.currentViewData[proxy.getIndexByRow($tr)].checkboxState == "checked") || (model.selectionSettings.selectionType == "multiple" && model.selectedItems.length > 1))) {
		            proxy._draggedRecords = $.extend([], model.selectedItems).reverse();
		        }
		        else
		            proxy._draggedRecords.push(proxy._draggedRecord);
		        args.draggedRecords = proxy._draggedRecords;

		        proxy._trigger("rowDragStart", args);
		        if (args.draggedRow && args.draggedRow.isSummaryRow) {
		            args.cancel = true;
		            args.draggedRow.canDrag = false
		        }
		        if (args.draggedRow.canDrag == false) setCanDrag = false;
		        $.extend([], proxy._draggedRecords).filter(function (item) {
		            if (item.canDrag == false) {		               
		                proxy._draggedRecords.splice(proxy._draggedRecords.indexOf(item), 1);
		                item.canDrag = true;		                
		            }
		        });
		        proxy._gridPosition = proxy.getOffsetRect($("#" + proxy._id)[0]);

		        if (proxy._frozenColumnsLength > 0 && recordIndex != -1) {
		            var tempTr, index = $tr.index();
		            if ($tr.closest(".e-movablecontentdiv").length > 0)
		                tempTr = $(proxy._gridRows[0])[recordIndex];
		            else
		                tempTr = $(proxy._gridRows[1])[recordIndex];
		            $tr = $tr.add(tempTr);
		        }

		        var posx = proxy._gridPosition.left, posy = coord.pageY,
		         width = model.isFromGantt ? model.ganttElement.width() : proxy._gridWidth,
		         element = $tr.clone();
		        width = width - 50;//to render tooltip inside the control container element
		        //Remove the hided columns.
		        element.find(".e-hide").remove();
                var tooltipItemLength = model.dragTooltip.tooltipItems.length,
		        tooltiptable = ej.buildTag("table.e-zerospace", "", { 'padding': '1', "width": width - 1, "border-collapse": "collapse" });
		        //to render the tooltip according the option given in the model.
		        if (proxy.model.dragTooltip.showTooltip) {
		            if (proxy._dragTooltip == false) {
		                if (!proxy.model.dragTooltip.tooltipTemplate && proxy.model.dragTooltip.tooltipItems.length == 0) {

		                    tooltipbody = ej.buildTag("tbody", "", {}, {});
		                    element.find(".e-treegridexpand").css({ "display": "none" });
		                    element.find("td.e-unboundcell").css({ "display": "none" });
		                    element.find("td.e-detailsrowcell").css({ "display": "none" });
		                    element.find(".e-treegridcollapse").css({ "display": "none" });
		                    $(element.find(".e-dragintend")).css({ "width": 0 });
		                    element.find(".e-intend").removeClass("e-intend");
		                    element.find(".e-rowcell ").removeClass("e-rowcell ").addClass("e-dragrowcell");
		                    element.addClass("e-dragtooltipbackground");
		                    if (model.isFromGantt)
		                        element.find("td:last").remove();
		                    if (element.length == 2 && this._frozenColumnsLength > 0) {
		                        $(element[0]).append($(element[1].childNodes));
		                        element.splice(1, 1);
		                    }
		                    tooltipbody.append(element);
		                    tooltiptable.append(tooltipbody);
		                    proxy._dragmouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#tooltiptreegrid" + proxy._id + "", tooltiptable,
                                {
                                    "top": (posy + 20) + "px", "left": (posx + 15) + "px",
                                    'position': 'absolute',
                                    'overflow': 'hidden',
                                    'z-index': proxy.getMaxZIndex() + 1,
                                    'padding': '0',
                                    'border-radius': '3px',
                                    'border-right': '0',
                                    'border-bottom': '0',
                                    'width': width,
                                }, {});
		                    model.cssClass && $(proxy._dragmouseOverTooltip).addClass(model.cssClass);
		                    $(proxy._dragmouseOverTooltip).addClass("e-dragtooltipbackground").addClass("e-dragrowcell");		                    
		                    proxy._dragTooltip = true;
		                }
		                else if (tooltipItemLength != 0) {
		                    var columns = model.columns;
		                    tooltipbody = ej.buildTag("tbody", "", {}, {});
		                    var tooltiptr = ej.buildTag("tr.e-dragtooltipbackground", "", {}, {}),
		                        columnTooltipValue;
		                    for (var j = 0; j < tooltipItemLength; j++) {
		                        for (var i = 0; i < columns.length; i++) {
		                            //Skip the columns which is not visible in TreeGrid.
		                            if (columns[i].visible == false)
		                                continue;
		                            if (model.isFromGantt)
		                                columnTooltipValue = columns[i].mappingName
		                            else
		                                columnTooltipValue = columns[i].field;
		                            if (model.dragTooltip.tooltipItems[j] == columnTooltipValue) {
		                                var value = proxy._getCustomCellValue(proxy._draggedRecord, columns[i].field),
		                                    $td;
		                                if (columns[i].format)
		                                    value = proxy.formatting(columns[i].format, value, model.locale)
		                                if (!ej.isNullOrUndefined(value)) {
		                                    $td = ej.buildTag("td.e-dragrowcell", value.toString(), { "padding": "5px", "width": proxy.columnsWidthCollection[i], "height": model.rowHeight }, {});
		                                    tooltiptr.append($td);
		                                }
		                                tooltiptr.append($td);
		                                break;
		                            }
		                        }
		                    }
		                    tooltipbody.append(tooltiptr);
		                    tooltiptable.append(tooltipbody);
		                    proxy._dragmouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#tooltiptreegrid" + proxy._id + "", tooltiptable,
                                {
                                    "top": (posy + 20) + "px", "left": (posx + 15) + "px",
                                    'position': 'absolute',
                                    'z-index': proxy.getMaxZIndex() + 1,
                                    'padding': '0',
                                    'border-right': '0',
                                    'border-bottom': '0',
                                    'border-radius': '3px',
                                    'height': "auto",
                                    'width': width,
                                    'overflow': 'hidden'
                                }, {});
		                    model.cssClass && $(proxy._dragmouseOverTooltip).addClass(model.cssClass);
		                    $(proxy._dragmouseOverTooltip).addClass("e-dragtooltipbackground").addClass("e-dragrowcell");		                    
		                    proxy._dragTooltip = true;
		                }
		                else if (proxy.model.dragTooltip.tooltipTemplate) {
		                    proxy._dragmouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#tooltiptreegrid" + proxy._id + "", "",
                                {
                                    "top": (posy + 20) + "px", "left": (posx + 15) + "px",
                                    'position': 'absolute',
                                    'z-index': proxy.getMaxZIndex() + 1,
                                    'padding': '0',
                                    'border-right': '0',
                                    'border-bottom': '0',
                                    'border-radius': '3px',
                                    'height': "auto"
                                }, {});
		                    model.cssClass && $(proxy._dragmouseOverTooltip).addClass(model.cssClass);
		                    proxy.tooltipState = "TemplateID";
		                    tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "dragtooltipTemplate"](proxy._draggedRecord), {}, {});
		                    proxy._dragmouseOverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
		                    proxy._dragTooltip = true;
		                }                                       
		            }
		        }
		        
		        if (proxy._draggedRecord) {		            
		            proxy._draggedElement = $tr;
		            proxy._updateDragStateFalse(proxy._draggedRecord);		                                
		            proxy._dragMouseDown = true;		            
                        //if(ej.isTouchDevice() || (ej.browserInfo().name == "chrome" && e.type == "touchstart"))
                        //    $(document.body).bind("touchmove", $.proxy(proxy.dragToolTip, proxy));
		            //else
		            if (e.type == "touchstart") {
		                if (!proxy._isHierarchyFiltering)
		                    $(document.body).append(proxy._dragmouseOverTooltip);
		                proxy._off(proxy.element, "touchmove", "#" + proxy._id + "e-gridcontent", proxy._touchScroll);
		                proxy._on(proxy.element, "touchmove", ".e-treegridrows, .e-templatecell", proxy.dragToolTip);
		            }		                
		            else{
		                proxy._off(proxy.element, "mousemove", "#" + proxy._id + "e-gridcontent", proxy._touchScroll);
		                proxy._on(proxy.element, "mousemove", ".e-gridcontent, .e-templatecell", proxy.dragToolTip);
                    }
		        }

		        if (args.cancel || !setCanDrag) {
		            proxy._dragMouseDown = false;
		            proxy._updateDragStateTrue(proxy._draggedRecord);
		            proxy._dragTooltip = false;
		            proxy._dragmouseOverTooltip && proxy._dragmouseOverTooltip.remove();
		            proxy._dragmouseOverTooltip = null;
		            if (e.type == "touchstart")
		                
		                proxy._off(proxy.element, "touchmove", ".e-treegridrows, .e-templatecell", proxy.dragToolTip);
		            else
		                proxy._off(proxy.element, "mousemove", ".e-gridcontent, .e-templatecell", proxy.dragToolTip);
		        }
		    }
		},

        //Mouse move action for to move the drag tooltip
		dragToolTip: function (e) {
		    var proxy = this,
                model = proxy.model,
                currentViewData = model.currentViewData,
                args = {},
                draggedrows = [],
                length,
                recordIndex, item,
                $trClone, $target, $tr,
                $tr_summary,
                coord = proxy._getCoordinate(e);
            e.preventDefault();
            
            var diffX = Math.abs(coord.pageX - proxy._posx),
                diffY = Math.abs(coord.pageY - proxy._posy);

            if (ej.isNullOrUndefined(proxy._posx) || ej.isNullOrUndefined(proxy._posy) || (diffX < 10 && diffY < 10))
                return;                        
            
            if (proxy._dragMouseDown == true && proxy.model.allowDragAndDrop == true) {
                if (proxy._batchEditStarts) {
                    proxy._batchSaveLostWarning("Drag");
                    return true;
                }
                if (e.type == "touchmove")
                    $target = proxy._getElementByPosition(coord.pageX, coord.pageY);
                else {
                    $target = $(e.target);
                    $(document.body).append(proxy._dragmouseOverTooltip);
                }
                $tr = $target.closest('tr');
                //to select row while mousedown for drag drop without row selection
                if (model.allowSelection == true && !ej.isNullOrUndefined(proxy._draggedRecord) && proxy._draggedRecord.isSelected == false) {
                    proxy.clearSelection(-1);
                    proxy._selectDraggedRow($tr);
                    proxy._cancelSaveTools();
                }

            //condition for when down record on empty records
                if ($tr.length == 0 && ($target.hasClass("e-movablecontainer") || $target.hasClass("e-gridcontent") || $target.hasClass("e-content"))) {
                    if (model.enableVirtualization) {
                        proxy._currentRecord = currentViewData[currentViewData.length - 1];
                    proxy._insertBelow = true;
                    proxy._lastRow = ej.TreeGrid.getRowByIndex(proxy, currentViewData.length - 1);
                    proxy._recordIndex = currentViewData.length - 1;
                }
                    else {
                        var collapsedRecords = proxy.getExpandedRecords(currentViewData);
                        proxy._currentRecord = collapsedRecords[collapsedRecords.length - 1];
                    if (model.allowPaging) {
                        proxy._lastRow = ej.TreeGrid.getRowByIndex(proxy, currentViewData.length - 1);
                    }
                    else {
                        var lastRecordIndex = model.updatedRecords.indexOf(proxy._currentRecord);
                        proxy._lastRow = ej.TreeGrid.getRowByIndex(proxy, lastRecordIndex);
                    }
                    proxy._recordIndex = proxy._indexOfLastRow = proxy.getIndexByRow(proxy._lastRow);
                }
                recordIndex = proxy._recordIndex;
            }
            else {
                var recordIndex = proxy.getIndexByRow($tr);
                proxy._currentRecord = model.currentViewData[recordIndex]
                }
                var updatedRecords = proxy.getUpdatedRecords();

              if (recordIndex != -1) {
                  item = proxy.model.currentViewData[recordIndex];
                  $tr = ej.TreeGrid.getRowByIndex(proxy, proxy._currentIndex);
                  $trClone = $tr.clone();
                    //Remove the hided columns.
                    $trClone.find(".e-hide").remove();
                    proxy._dragTargetItem = item;
                    args.targetRowIndex = recordIndex;
                    args.targetRow = item;
                    args.canDrop = true;
                    args.draggedRow = proxy._draggedRecord;
                    args.draggedRowIndex = updatedRecords.indexOf(proxy._draggedRecord);
                    args.draggedRecords = proxy._draggedRecords;
                    proxy._renderToolTip(coord, $trClone, args, $target);                 
                    if (args.targetRow && args.targetRow.isSummaryRow) {
                        proxy._draggedRecord.canDrop = false;
                        $(".e-dragintend").find(".e-icon").css({ "display": "none" });
                        $(".e-intendparent").find(".e-icon").css({ "display": "none" });
                    }
                    else if (args.canDrop == false) {
                        proxy._draggedRecord.canDrop = false;
                    }
                    else {
                        proxy._draggedRecord.canDrop = true;
                    }
                } else if (proxy._draggedRecord && proxy._dragMouseDown) {
                    item = null;
                    $tr = ej.TreeGrid.getRowByIndex(proxy, proxy._currentIndex);
                    $trClone = $tr.clone();
                    $tr_summary = $target.closest('tr'),
                    args.targetRowIndex = null;
                    args.targetRow = null;
                    proxy._dragTargetItem = null;
                    args.canDrop = true;
                    args.draggedRow = proxy._draggedRecord;
                    args.draggedRowIndex = proxy._draggedRecord.index;
                    args.dropPosition = "invalidPosition";
                    proxy._trigger("rowDrag", args);
                    if (!$tr_summary.hasClass("e-footersummaryrow"))
                        proxy._renderToolTip(coord, $trClone, args, $target);
                }
                proxy._mouseout(coord);
            }
        },

        _mouseupHandler: function(e){
            var proxy = this,
                model = proxy.model;
            
            if (e.type == "touchend")
                proxy._off(proxy.element, "touchmove", "#" + proxy._id + "e-gridcontent", proxy._touchScroll);
            else
                proxy._off(proxy.element, "mousemove", "#" + proxy._id + "e-gridcontent", proxy._touchScroll);

            proxy._off($(document), "mouseup touchend", proxy._mouseupHandler);
            proxy._mouseDown = false;
            proxy._isTouchEvent = false;
            window.clearTimeout(proxy._dragTooltipTimer);
            if (model.allowDragAndDrop && proxy._dragMouseDown)
                proxy.dragMouseUp(e);

            if (model.contextMenuSettings.showContextMenu && proxy._contextMenuHandler && proxy._displayContextMenu) {
                proxy._rightClick(e);
            }
            else
                proxy._contextMenuHandler = false;
            proxy._displayContextMenu = true;
            proxy._dy = proxy._dx = 0;
        },
        _performDragAndDrop: function (draggedRecord, droppedRecord,recordIndex,args,arg ) {
            var proxy = this,
                model = proxy.model,
                editMode = model.editSettings.editMode.toLowerCase(),
                childRecords, droppedRecordIndex,
                count = 0,
                length = model.selectedItems.length,
                dragLength = proxy._draggedRecords.length,
                draggedRecords = [],
                recordCollection = {};
            for (var j = 0; j < dragLength; j++) {
                draggedRecord = proxy._draggedRecord = proxy._draggedRecords[j];
                var flatRecords = model.flatRecords;
                args.previousParentItem = draggedRecord.parentItem ? $.extend({}, draggedRecord.parentItem) : draggedRecord.parentItem;
                if (!ej.isNullOrUndefined(draggedRecord.parentItem)) {
                    args.previousParentItem.childRecords = $.extend([], draggedRecord.parentItem.childRecords);
                    args.parentChildState = draggedRecord.parentItem.hasChildRecords;
                    args.parentExpandedState = draggedRecord.parentItem.expanded;
                }
                if (args.requestType != "rowIndexUpdate") {
                    arg.targetRow = $.extend({}, proxy._droppedRecord);
                    arg.targetRow.item = $.extend({}, proxy._droppedRecord.item);
                    arg.targetRowIndex = model.updatedRecords.indexOf(droppedRecord);
                    arg.draggedRow = $.extend({}, proxy._draggedRecord);
                    arg.draggedRow.item = $.extend({}, proxy._draggedRecord.item);
                    arg.draggedRowIndex = model.updatedRecords.indexOf(draggedRecord);
                }
                args.previousItem = ej.isNullOrUndefined(arg) ? $.extend({}, proxy._draggedRecord) : arg.draggedRow;
                args.previousItemIndex = flatRecords.indexOf(proxy._draggedRecord);
                args.previousTarget = proxy._droppedRecord;
                args.previousTargetItemIndex = flatRecords.indexOf(proxy._droppedRecord);
                args.clonedTarget = $.extend({}, proxy._droppedRecord);
                args.hasChildRecords = droppedRecord.hasChildRecords;
                args.expanded = droppedRecord.expanded;
                args.childIndex = args.previousParentItem ? args.previousParentItem.childRecords.indexOf(draggedRecord) : 0;
                args.previousLevel = draggedRecord.level;
                var draggedrows = [];

                if (droppedRecord.dragState == true) {
                    proxy._updateDragStateTrue(draggedRecord);

                    if (proxy._insertAbove == true || proxy._insertBelow == true || proxy._insertAsChild == true) {
                        if (!proxy.model.isEdit && !proxy._isRowEdit)
                            proxy._cancelSaveTools();
                        for (var i = 0; i < length; i++)
                            proxy.clearSelection(model.selectedItems[i]);

                        proxy._deleteDragRow();
                        var recordIndex1 = flatRecords.indexOf(droppedRecord);
                        // Update checkbox state of parent record of dragged record
                        if (proxy._checkboxSelection && model.selectionSettings.enableHierarchySelection) {
                            draggedRecord.parentItem && proxy._updateParentSelection(draggedRecord.parentItem);
                        }

                        //Condition to splice above to the dropped record.
                        if (proxy._insertAbove == true) {
                            args.index = recordIndex1;
                            draggedRecord.parentItem = flatRecords[recordIndex1].parentItem;
                            draggedRecord.level = flatRecords[recordIndex1].level;
                            flatRecords.splice(recordIndex1, 0, draggedRecord);
                            model.updatedRecords.splice(recordIndex1, 0, draggedRecord);
                            if (model.isFromGantt)
                                model.ids.splice(recordIndex1, 0, (draggedRecord.taskId).toString());
                            if (draggedRecord.hasChildRecords) {
                                var level = 1;
                                proxy._updateChildRecord(draggedRecord, recordIndex1);
                                proxy._updateChildRecordLevel(draggedRecord, level);
                            }
                            if (droppedRecord.parentItem) {
                                childRecords = droppedRecord.parentItem.childRecords;
                                droppedRecordIndex = childRecords.indexOf(droppedRecord);
                                childRecords.splice(droppedRecordIndex, 0, draggedRecord);
                                //Update hierarchyRowIndex for level 1 and above records
                                proxy._updateHierarchyRowIndexData(args, childRecords, droppedRecordIndex);
                            }
                        }

                            //condition to splice below to the dropped record.
                        else if (proxy._insertBelow == true) {
                            args.index = recordIndex1 + 1;
                            if (!droppedRecord.hasChildRecords) {
                                flatRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                model.updatedRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                if (model.isFromGantt)
                                    model.ids.splice(recordIndex1 + 1, 0, (draggedRecord.taskId).toString());
                            }
                            else {
                                count = proxy.getChildCount(droppedRecord, 0);
                                flatRecords.splice(recordIndex1 + count + 1, 0, draggedRecord);
                                model.updatedRecords.splice(recordIndex1 + count + 1, 0, draggedRecord);
                                if (model.isFromGantt)
                                    model.ids.splice(recordIndex1 + count + 1, 0, (draggedRecord.taskId).toString());
                            }
                            draggedRecord.parentItem = flatRecords[recordIndex1].parentItem;
                            draggedRecord.level = flatRecords[recordIndex1].level;
                            if (draggedRecord.hasChildRecords) {
                                var level = 1;
                                proxy._updateChildRecordLevel(draggedRecord, level);
                                proxy._updateChildRecord(draggedRecord, recordIndex1 + count + 1);
                            }
                            if (droppedRecord.parentItem) {
                                childRecords = droppedRecord.parentItem.childRecords;
                                droppedRecordIndex = childRecords.indexOf(droppedRecord) + 1;
                                childRecords.splice(droppedRecordIndex, 0, draggedRecord);
                                //Update hierarchyRowIndex for level 1 and above records
                                proxy._updateHierarchyRowIndexData(args, childRecords, droppedRecordIndex);
                            }                            
                        }

                            // Condition to insert as child to the dropped Record.
                        else if (proxy._insertAsChild == true) {
                            args.index = recordIndex1 + 1;
                            if (!draggedRecord.hasChildRecords) {
                                flatRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                model.updatedRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                if (model.isFromGantt)
                                    model.ids.splice(recordIndex1 + 1, 0, (draggedRecord.taskId).toString());
                                proxy._recordLevel(recordIndex, args);
                            }
                            else {
                                flatRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                model.updatedRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                if (model.isFromGantt)
                                    model.ids.splice(recordIndex1 + 1, 0, (draggedRecord.taskId).toString());
                                proxy._recordLevel(recordIndex, args);
                                if (droppedRecord.hasChildRecords)
                                    droppedRecord.expanded = proxy._isLoadonDemand ? droppedRecord.isChildRetrieved : droppedRecord.expanded;
                                proxy._updateChildRecord(draggedRecord, recordIndex1 + 1, droppedRecord.expanded);
                            }
                        }
                    }
                    //update checkbox state of parent record and header of dropped record
                    if (proxy._checkboxSelection && model.selectionSettings.enableHierarchySelection) {
                        draggedRecord.parentItem && proxy._updateParentSelection(draggedRecord.parentItem);
                        proxy.selectAllRows();
                    }
                    if (draggedRecord.parentItem == null) {
                        var parentRecords = model.parentRecords,
                            newPIndex = parentRecords.indexOf(proxy._droppedRecord);                        
                        if (proxy._insertBelow == true)
                            parentRecords.splice(newPIndex + 1, 0, draggedRecord);
                        else if (proxy._insertAbove == true)
                            parentRecords.splice(newPIndex, 0, draggedRecord);
                        //Update hierarchyRowIndex for level 0 records
                        var pIndex = proxy._insertBelow == true ? newPIndex + 1 : newPIndex;
                        proxy._updateHierarchyRowIndexData(args, parentRecords, pIndex);
                    }
                }
                draggedRecords.push(draggedRecord);
            }
            recordCollection.draggedRecords = draggedRecords;
            recordCollection.draggedRecord = draggedRecord;
            recordCollection.droppedRecord = droppedRecord;
            return recordCollection;
        },
        _updatedMovedRow: function (draggedRecord, droppedRecord, dropPosition, args, rindex1, draggedRecords) {
            var proxy = this,
                model = proxy.model;            
            args.draggedRow = draggedRecord;
            args.targetRow = droppedRecord;
            args.droppedPosition = dropPosition;
            if (!model.isFromGantt && draggedRecords) {
                for (var i = 0; i < draggedRecords.length; i++) {
                    proxy._draggedRecord = draggedRecords[i];
                    proxy._updateDataSource();
                }
            }
            else
                proxy._updateDataSource();
            /*update summary rows collection*/
            if (model.showSummaryRow && proxy._insertAsChild == true)
                proxy._createSummaryRow(args);
            if (model.enableWBS) {
                var ddSiblings;
                if (draggedRecord.parentItem)
                    ddSiblings = draggedRecord.parentItem.childRecords;
                else {
                    var flatData = model.flatRecords,
                        Level0 = flatData.filter(function (item) {
                            return item && item.level == 0;
                        });
                    ddSiblings = Level0;
                }
                var ddIndex = ddSiblings.indexOf(draggedRecord),
                    selectedRecords = ddSiblings.slice(ddIndex, ddSiblings.length),
                    parentVal = draggedRecord.parentItem ? draggedRecord.parentItem.WBS : null,
                    lastVal = ddIndex + 1;
                proxy.reCalculateWBS(selectedRecords, lastVal, parentVal);
            }
            if (model.isFromGantt) {
                var rindex2 = model.flatRecords.indexOf(draggedRecord),
                    rindex = rindex2 < rindex1 ? rindex2 : rindex1;
                proxy.updateSerialNumber(rindex);
                if (model.enableSerialNumber) {
                    if (dropPosition == "insertAsChild")
                        rindex -= 1;
                    proxy.updateSerialNumberPredecessors(rindex);
                }
            }
            proxy.processBindings(args);
            if (model.enableAltRow && model.currentViewData.length > 0)
                ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);
        },
        _navigatePager:function(rowIndex){
            var proxy = this, model = proxy.model,
            pageSize = model.pageSettings.pageSize,pageIndex;
            if (model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root) {
                var rootParent = proxy._getRootParent(model.updatedRecords[rowIndex]);
                rowIndex = model.parentRecords.indexOf(rootParent);
            }
            //rowIndex + 1 to equate the record count in a page
            pageIndex = Math.ceil((rowIndex + 1) / pageSize);            
            proxy.gotoPage(pageIndex);
        },
        _getRootParent: function (draggedRecord) {
            var proxy = this;
            if (ej.isNullOrUndefined(draggedRecord.parentItem)) {
                return draggedRecord;
            }
            else {
                return  proxy._getRootParent(draggedRecord.parentItem);
            }            
        },
        moveRow: function (fromIndex, toIndex, position) {
            var proxy = this, model = proxy.model,
                 args = {}, newArgs = {};
            proxy._cancelEditState();
            if (model.editSettings.editMode.toLowerCase() == "batchediting" && proxy._batchEditStarts)
                return false;
            if (ej.isNullOrUndefined(toIndex)) {
                if (position == "above" || position == "child")
                    toIndex = fromIndex - 1;
                else if (position == "below")
                    toIndex = fromIndex + 1;
            }
            var draggedRecord = model.updatedRecords[fromIndex],
            droppedRecord = model.updatedRecords[toIndex];
            //Skip the summary row while moving records as child
            if (droppedRecord && droppedRecord.isSummaryRow) {
                droppedRecord = proxy._getNextRecord(toIndex, model.updatedRecords, position);
            }
            if (ej.isNullOrUndefined(draggedRecord) || ej.isNullOrUndefined(droppedRecord) || draggedRecord.isSummaryRow == true || droppedRecord.isSummaryRow == true || !proxy.getExpandStatus(draggedRecord) || !proxy.getExpandStatus(droppedRecord)) return false;
            proxy._draggedRecord = draggedRecord;
            proxy._droppedRecord = droppedRecord;
            proxy._draggedRecords.push(draggedRecord);
            args.currentRecord = draggedRecord;
            args.targetRecord = droppedRecord;
            args.currentRowIndex = fromIndex;
            args.targetRowIndex = toIndex;
            args.dropPosition = position;
            args.requestType = "rowIndexUpdate";
            proxy._moveRow = true;
            if (!proxy._trigger("actionBegin", args)) {
                proxy._validDrop(proxy._draggedRecords, droppedRecord);
                if (droppedRecord.dragState == true) {
                    if (position == "above")
                        proxy._insertAbove = true;
                    else if (position == "below")
                        proxy._insertBelow = true;
                    else if (position == "child" && $.inArray(draggedRecord, droppedRecord.childRecords) == -1)
                        proxy._insertAsChild = true;
                    else {
                        proxy._resetPrivateProperties();
                        return false;
                    }
                    proxy._performDragAndDrop(draggedRecord, droppedRecord, fromIndex, args);
                    proxy._updatedMovedRow(draggedRecord, droppedRecord, position, args);
                    if (model.allowPaging) {
                        proxy._navigatePager(model.updatedRecords.indexOf(draggedRecord));
                    }
                    proxy.updateScrollBar(proxy.getUpdatedRecords().indexOf(draggedRecord));
                    //to select row while mousedown for drag drop without row selection
                    if (model.allowSelection == true) {
                        proxy.clearSelection(-1);
                        proxy._selectDraggedRow();
                        proxy._cancelSaveTools();
                    }
                    newArgs.currentRecord = draggedRecord;
                    newArgs.targetRecord = droppedRecord;
                    newArgs.currentRowIndex = model.updatedRecords.indexOf(draggedRecord);
                    newArgs.targetRowIndex = model.updatedRecords.indexOf(droppedRecord);
                    newArgs.dropPosition = args.dropPosition;
                    newArgs.requestType = args.requestType;
                    proxy._trigger("actionComplete", newArgs);

                    var cArgs = {};
                    cArgs.draggedRowIndex = newArgs.currentRowIndex;
                    cArgs.targetRowIndex = newArgs.targetRowIndex;
                    cArgs.requestType = "dragAndDrop";
                    proxy._updateAction(cArgs);
                    proxy._resetPrivateProperties();
                    return true;
                }
                else {
                    droppedRecord.dragState = true;
                    proxy._resetPrivateProperties();
                    return false;
                }
            }

        },
        //Mouse up event for drag and drop
        dragMouseUp: function (e) {
            var args = {},
                arg = {},
                newArgs={},
                proxy = this,
                model = proxy.model, $target, $tr, draggedRecords,
                draggedRecord = proxy._draggedRecord,
                currentRecord = proxy._currentRecord, droppedRecord,
                canDrop,
                dropPosition, coord = proxy._getCoordinate(e),
                rindex1 = model.flatRecords.indexOf(draggedRecord);
            draggedRecord.isDragged = true;
            proxy._timerDragDown && (proxy._timerDragDown = window.clearInterval(proxy._timerDragDown));
            proxy._timerDragUp && (proxy._timerDragUp = window.clearInterval(proxy._timerDragUp));
            
            // unbind the drag events once row dropped
            if (model.allowDragAndDrop) {
                if (e.type == "touchstart")
                    proxy._off(proxy.element, "touchmove", ".e-treegridrows, .e-templatecell", proxy.dragToolTip);
                else
                    proxy._off(proxy.element, "mousemove", ".e-gridcontent, .e-templatecell", proxy.dragToolTip);

                if (ej.isNullOrUndefined(draggedRecord) || proxy._draggedRecords.length > 1)
                    canDrop = true;
                else
                    canDrop = draggedRecord.canDrop;
            }

            window.clearTimeout(proxy._dragTooltipTimer);
            proxy._dragMouseDown = false;
            $(proxy._dragmouseOverTooltip).remove();
            proxy._dragTooltip = false;

            if (e.type == "touchend")
                $target = proxy._getElementByPosition(coord.pageX, coord.pageY);
            else
                $target = $(e.target);
          

            $tr = $target.closest('tr');
            if (proxy._isHierarchyFiltering)
                $("#" + proxy._id + "e-gridcontent").css({ 'cursor': 'default' });
            $(".e-dragintend").find(".e-icon").css({ "display": "none" });
            $(".e-intendparent").find(".e-icon").css({ "display": "none" });
            var targetItem = model.currentViewData[proxy.getIndexByRow($tr)];
            if (this.cancelDrop == true && !ej.isNullOrUndefined(targetItem) && targetItem.parentItem == null) {
                var targetTr = ej.TreeGrid.getRowByIndex(this, model.updatedRecords.indexOf(targetItem));
                if (targetItem.expanded)
                    targetTr.find(".e-treegridexpand").css({ "display": "inline-block" });
                else
                    targetTr.find(".e-treegridcollapse").css({ "display": "inline-block" });
            }
            proxy._dragMouseLeave = true;
            if ($target.closest("#" + proxy._id).length == 1 && $target.closest(".e-gridcontent").length == 1 && !ej.isNullOrUndefined(draggedRecord) && proxy._dropCancel == false) {
                
                if (model.allowDragAndDrop && canDrop) {
                    proxy._dragMouseLeave = false;
                    var compareIndex;
                    //condition for allow down record on empty records
                    if ($tr.length == 0) {
                        recordIndex = proxy._indexOfLastRow;
                        compareIndex = recordIndex;
                        if (model.enableVirtualization) {
                            proxy._droppedRecord = model.currentViewData[model.currentViewData.length - 1];
                        }
                        else {
                            if (model.currentViewData[model.currentViewData.length - 1].expanded) {
                                proxy._insertBelow = true;
                                proxy._droppedRecord = model.currentViewData[model.currentViewData.length - 1];
                            }
                            else {
                                var collapsedRecords = proxy.getExpandedRecords(model.currentViewData);
                                proxy._droppedRecord = collapsedRecords[collapsedRecords.length - 1];
                            }
                        }
                    }
                    else {
                        var recordIndex = proxy.getIndexByRow($tr);
                        proxy._droppedRecord = model.currentViewData[recordIndex];
                        compareIndex = model.updatedRecords.indexOf(proxy._droppedRecord);
                    }

                  
                    if (compareIndex != proxy._currentIndex && recordIndex != -1 && proxy._draggedRecords.length != 0) {
                        if (proxy._droppedRecord.dragState == true) {
                            //to splice the record while we drop on selected item itself
                            if ($.inArray(proxy._droppedRecord, proxy._draggedRecords) != -1) {
                                var index = proxy._draggedRecords.indexOf(proxy._droppedRecord);
                                proxy._draggedRecords.splice(index, 1);
                            }                           
                            //to maintain hierarchy while dropping multiple records using checkbox
                            if (model.selectionSettings.selectionType == "checkbox" && model.selectionSettings.enableHierarchySelection == true) {
                                var checkedRecords = $.extend([], proxy._draggedRecords);
                                checkedRecords.filter(function (item) {
                                    if (item.parentItem && item.parentItem.checkboxState == "checked") {
                                        proxy._draggedRecords.splice(proxy._draggedRecords.indexOf(item), 1);
                                    }                                   
                                });
                            }
                        }
                        if (proxy._insertAbove == true && proxy._draggedRecords.length > 1)
                            proxy._draggedRecords.reverse();

                        if (proxy._insertAbove) dropPosition = "insertAbove";
                        else if (proxy._insertBelow) dropPosition = "insertBelow";
                        else if (proxy._insertAsChild) dropPosition = "insertAsChild";

                        newArgs.draggedRecords = proxy._draggedRecords;
                        newArgs.draggedRow = draggedRecord;
                        newArgs.draggedRow.item = draggedRecord.item;
                        newArgs.targetRow = proxy._droppedRecord;
                        newArgs.draggedRowIndex = model.updatedRecords.indexOf(draggedRecord);
                        newArgs.targetRowIndex = model.updatedRecords.indexOf(proxy._droppedRecord);
                        newArgs.dropPosition = dropPosition;
                        droppedRecord = proxy._droppedRecord;
                        if (!proxy._trigger("rowDropActionBegin", newArgs)) {
                            //to splice the canDrop false item from dragged records collection
                            var clonedDraggedRecord = $.extend([], proxy._draggedRecords);
                            clonedDraggedRecord.filter(function (item) {
                                if (item.canDrop == false) {
                                    proxy._draggedRecords.splice(proxy._draggedRecords.indexOf(item), 1);
                                    item.canDrop = true;
                                }
                                if (proxy._draggedRecords.length == 0) droppedRecord.dragState = false;
                            });
                            var recordCollection = this._performDragAndDrop(draggedRecord, droppedRecord, recordIndex, args, arg);
                            draggedRecord = recordCollection.draggedRecord;
                            draggedRecords = recordCollection.draggedRecords;
                            droppedRecord = recordCollection.droppedRecord;                           
                            args.requestType = "dragAndDrop";
                            if (droppedRecord.dragState == true && !proxy._isHierarchyFiltering) {
                                proxy._updatedMovedRow(draggedRecord, droppedRecord, dropPosition, args, rindex1,draggedRecords);
                                if (!model.isFromGantt)
                                    arg.draggedRecords = draggedRecords;
                                arg.draggedRow = args.draggedRow;                                                            
                                arg.draggedRow.item = args.draggedRow.item;
                                arg.draggedRowIndex = model.updatedRecords.indexOf(draggedRecord);
                                arg.targetRowIndex = model.updatedRecords.indexOf(droppedRecord);
                                arg.requestType = dropPosition;
                                args.isFromRevertDragging = false;
                                proxy._trigger("rowDragStop", arg);
                                if (arg.cancel) {
                                    args.isFromRevertDragging = true;
                                    proxy._revertDragging(args);
                                }
                                else {
                                    var cArgs = {};                                    
                                    cArgs.draggedRecords = draggedRecords;
                                    cArgs.draggedRowIndex = arg.draggedRowIndex;
                                    cArgs.targetRowIndex = arg.targetRowIndex;
                                    cArgs.requestType = "dragAndDrop";
                                    proxy._updateAction(cArgs);
                                }

                            
                                if (model.allowSelection) {
                                    if (model.selectionSettings.selectionMode == "row") {
                                        //to maintain selection for two rows with drop on selected record or canDrop is false
                                        if (!model.isFromGantt && ((model.selectionSettings.selectionType == "checkbox" && $.inArray(draggedRecord, model.selectedItems) != -1) || (model.selectionSettings.selectionType == "multiple")) && model.selectedItems.length > 1) {
                                           var multiDragRecord = model.selectedItems.filter(function (item) {
                                                if (item.isDragged == true) {
                                                    item.isDragged = false;
                                                    return item
                                                }
                                            });
                                           var rowIndex = proxy.getUpdatedRecords().indexOf(multiDragRecord[0]);
                                            proxy.clearSelection(rowIndex);
                                            proxy._multiSelectCtrlRequest = true;
                                            if (rowIndex != -1 && model.allowSelection && !(proxy._rowSelectingEventTrigger(this.selectedRowIndex(), rowIndex))) {
                                                proxy.selectRows(rowIndex);
                                                proxy._rowSelectedEventTrigger(rowIndex);
                                            }
                                        }                                        
                                        else
                                            proxy._selectDraggedRow($tr);                                        
                                    }
                                    if (model.selectionSettings.selectionMode == "cell")
                                        proxy.updateScrollBar(model.updatedRecords.indexOf(proxy._draggedRecord));
                                }
                            }
                            else
                                //If drag state false then drop will cancel so have change the drag state.
                                proxy._updateDragStateTrue(draggedRecord);
                        }                                                  
                    }
                    else {
                        // If drop record not found means condition failed and drag state should change
                        proxy._updateDragStateTrue(draggedRecord);
                        if (!proxy.model.isEdit && !proxy._isRowEdit)
                            proxy._cancelSaveTools();
                    }
                }
                else {
                    // if canDrop true then drop will cancelled and state should change
                    proxy._updateDragStateTrue(draggedRecord);
                    proxy._resetPrivateProperties();
                    return 0;
                }
            }
            else
                // if esc key pressed or user drops a record other the tree grid content then 
                //drop cancelled and have to change the drag state.
                proxy._updateDragStateTrue(draggedRecord);
            // to enable expand and collapse icon of top most record if user mouse up outside of the grid area. 
            if (!ej.isNullOrUndefined(proxy._currentRecord)) {
                if (proxy._currentRecord.parentItem == null) {
                    $tr = model.allowPaging ? ej.TreeGrid.getRowByIndex(proxy, proxy.getUpdatedRecords().indexOf(proxy._currentRecord)) : ej.TreeGrid.getRowByIndex(proxy, model.updatedRecords.indexOf(proxy._currentRecord));
                    if (proxy._currentRecord.expanded)
                        $tr.find(".e-treegridexpand").css({ "display": "inline-block" });
                    else
                        $tr.find(".e-treegridcollapse").css({ "display": "inline-block" });
                }
            }
            //Update the summary row after perform the drag and drop action.
            if (model.showSummaryRow && args.requestType == "dragAndDrop" && proxy._insertAsChild != true && !proxy._isHierarchyFiltering)
                proxy._updateSummaryRow(args);

            proxy._dropCancel = false;
            if (!ej.isNullOrUndefined(droppedRecord)) droppedRecord.dragState = true;            
            proxy._resetPrivateProperties();
            if (!ej.isNullOrUndefined(draggedRecord) && !proxy.model.isEdit && !proxy._isRowEdit)
                proxy._cancelSaveTools();                        
            if (!ej.isNullOrUndefined(draggedRecord) && draggedRecord.isDragged == true) draggedRecord.isDragged = false;
        },

        // Mouse down action to start a Drag Column
        dragColumn: function (e) {
            
            var $target = $(e.target),
               proxy = this,
            coord = proxy._getCoordinate(e);

            proxy._posx = coord.pageX
            proxy._posy = coord.pageY;

            /*verify ctlkey and shift key and mouse button*/
            if (e.ctrlKey || e.shiftKey || e.which == 2 || e.which == 3)
                return true;

            var model = proxy.model,
               columns = model.columns,
               columnIndex,
               args = {};

            if ($target.css("cursor") == "col-resize") return; // Return if mousedown action on column resizer

            if ($target.closest(".e-headercelldiv") && !$target.hasClass("e-columnmenu-icon")) {
                var columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname");
                if (ej.isNullOrUndefined(columnFieldName) && ej.isNullOrUndefined(this.getColumnByField(columnFieldName))) return;

                proxy._saveEditedElements();

                if ($target.hasClass("e-checkbox"))
                    return false;

                proxy._draggedColumn = this.getColumnByField(columnFieldName);
                columnIndex = columns.indexOf(proxy._draggedColumn);
                args.draggedColumnIndex = columnIndex;
                args.draggedColumn = proxy._draggedColumn;
                args.requestType = "columnDragStart";
                if (!ej.isNullOrUndefined(proxy._draggedColumn.headerTemplateID)) {
                    proxy._draggedColumnTemplate = $target.closest(".e-headertemplate");
                }
                else {
                    proxy._draggedColumnTemplate = null;
                }
                if (!ej.isNullOrUndefined(proxy._draggedColumn.nonColumn)) return;
                proxy._trigger("columnDragStart", args);
                if (args.cancel) return;
                if (model.selectionSettings.selectionMode == "cell")
                    proxy.clearSelection();
                proxy._headerDragStatus = "mousedown";                
                proxy._dragMouseDown = true;
                proxy._mouseDragged = false;
                var gridPos = proxy.getOffsetRect(proxy.element[0]);
                proxy._gridLeft = gridPos.left;
                proxy._gridTop = gridPos.top;
                proxy._touchstart = true;
                // Mousemove event bind for Drag a column                
                if (e.type != "touchstart") {
                    proxy._on(proxy.element, "mousemove", proxy.dragColumnToolTip);
                }
            }
        },

        //Mousemove action to dragging tooltip
        dragColumnToolTip: function (e) {
            var proxy = this,
                model = proxy.model,
                args = {},
                $target,
                columns = model.columns,
                columnIndex, item, column = {},
                coord = proxy._getCoordinate(e),
                posx = coord.pageX, posy = coord.pageY;
                        
            if (proxy._touchstart) {
                proxy._touchstart = false;
                return;
            }
            if (e.type == "touchmove") {
                e.preventDefault();
                $target = proxy._getElementByPosition(coord.pageX, coord.pageY);
            }
            else
                $target = $(e.target);

            if (proxy._headerDragStatus.length > 0 && model.allowColumnReordering == true) {

                if (model.selectionSettings.selectionMode == "cell" && proxy._headerDragStatus != "mousemove")
                    proxy.clearSelection();
                var $hcell = $target.closest(".e-headercell"),
                    columnFieldName = $hcell.find(".e-headercelldiv").attr("data-ej-mappingname"),
                    elementRight = proxy._gridLeft + proxy._gridWidth - 10,
                    elementHeight = proxy._gridHeight + proxy._gridTop, xpos = posx, ypos = posy;

                if (!ej.isNullOrUndefined(proxy._dragmouseOverTooltip)) {
                    var tooltipMidPos = proxy._dragmouseOverTooltip["width"] / 2,
                        tooltipHeight = proxy._dragmouseOverTooltip["height"] + 5;
                    if ((xpos - tooltipMidPos) < proxy._gridLeft)
                        xpos = proxy._gridLeft;
                    else if ((xpos + tooltipMidPos) > elementRight)
                        xpos = elementRight - (tooltipMidPos*2) - 4;
                    else
                        xpos = xpos - tooltipMidPos;
                    if (ypos + tooltipHeight > elementHeight)
                        ypos = elementHeight - tooltipHeight;
                }

                if (!ej.isNullOrUndefined(columnFieldName) && !ej.isNullOrUndefined(this.getColumnByField(columnFieldName))) {

                    var targetColumn = this.getColumnByField(columnFieldName);
                    proxy._currentIndex = columns.indexOf(targetColumn);
                    args.targetColumnIndex = proxy._currentIndex;
                    args.draggedColumnIndex = columns.indexOf(proxy._draggedColumn);
                    args.draggedColumn = proxy._draggedColumn;
                    args.targetColumn = targetColumn;
                    args.requestType = "columnDrag";
                    args.canDrop = true;

                    proxy._trigger("columnDrag", args);
                    if (args.cancel) return;

                    args.direction = (args.draggedColumnIndex >= args.targetColumnIndex) ? "left" : "right";
                    column.pos = proxy.getOffsetRect($hcell[0]);
                    column.height = $($hcell).height();
                    column.width = $($hcell).outerWidth();
                    column.pos.top = proxy.getOffsetRect($hcell.closest(".e-gridheader")[0]).top;
                    column.display = "block";
                    if(args.direction == "left" &&proxy._gridLeft > column.pos.left)
                        column.pos.left = proxy._gridLeft;
                    else if (args.direction == "right" && elementRight < (column.pos.left + column.width))
                        column.pos.left = elementRight - 10;
                    else
                        column.pos.left = ((args.draggedColumnIndex > args.targetColumnIndex)) ? (column.pos.left - 2) : (column.pos.left + column.width - 2);
                    args.column = column;
                    
                    proxy._autoHscroll(posx, posy, args);
                    if (args.draggedColumnIndex != args.targetColumnIndex) {
                        proxy._displayContextMenu = false;
                    }
                    if ((args.draggedColumnIndex == args.targetColumnIndex) || !ej.isNullOrUndefined(args.targetColumn.nonColumn) ||
                        ((column.pos.left - 8) > (proxy._gridWidth + proxy._gridLeft))) {
                        column.display = "none";
                    }
                    if (proxy._headerDragStatus != "mousemove" && !proxy._dragTooltip) {

                        var height = proxy._gridHeight - 1;
                        if (proxy.model.toolbarSettings.showToolbar) {
                            height -= !ej.isNullOrUndefined($("#" + proxy._id + "_toolbarItems").outerHeight()) ? $("#" + proxy._id + "_toolbarItems").outerHeight() : 0;
                        }
                        height -= proxy.getScrollElement().ejScroller("isHScroll") ? 18 : 0;

                        var tooltipcolumn = ej.buildTag('div', proxy._draggedColumn.headerText, { "margin-top": "5px" });
                        if (!ej.isNullOrUndefined(proxy._draggedColumnTemplate) && !ej.isNullOrUndefined(proxy._draggedColumn) && !ej.isNullOrUndefined(proxy._draggedColumn.headerTemplateID))
                            tooltipcolumn[0].innerHTML = proxy._draggedColumnTemplate[0].innerHTML;
                        proxy._dragmouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#columnreordertooltip" + proxy._id + "", tooltipcolumn,
                            {
                                "top": (posy+10) + "px", "left": (posx - 57) + "px",//contextmenu action not triggered beacause touch event bind on tooltip, so adjust the tooltip top position
                                'position':'absolute',
                                'z-index': proxy.getMaxZIndex() + 1
                            }, {});
                        model.cssClass && proxy._dragmouseOverTooltip.addClass(model.cssClass);
                        proxy._reordervline = ej.buildTag("div.e-vline e-js#reorderline" + proxy._id + "", {
                            "height": height,
                            "width": 0,
                            "border-width": 0,
                            "position": "absolute",
                            "top": column.pos.top + "px",
                            "left": column.pos.left + "px",
                            "border-left-width": "3px",
                            "border-style": "solid",
                            "display": column.display,
                        });
                        model.cssClass && proxy._reordervline.addClass(model.cssClass);
                        $(document.body).append(proxy._dragmouseOverTooltip);
                        $(document.body).append(proxy._reordervline);
                        $(proxy._dragmouseOverTooltip).addClass("e-headertooltip");
                        $(proxy._reordervline).addClass("e-headervline");
                        proxy._dragmouseOverTooltip["width"] = $(proxy._dragmouseOverTooltip).outerWidth();
                        proxy._dragmouseOverTooltip["height"] = $(proxy._dragmouseOverTooltip).outerHeight()
                        proxy._reordervline["height"] = model.sizeSettings.height == "auto" ? $("#" + proxy._id + "e-gridcontent").height() + $("#" + proxy._id + "e-gridheader").height() : height;
                        proxy._dragTooltip = true;
                    }
                    else {
                        $(proxy._reordervline).css({
                            "height": proxy._reordervline["height"],
                            "width": 0,
                            "border-width": 0,
                            "position": "absolute",
                            "top": column.pos.top + "px",
                            "left": column.pos.left + "px",
                            "border-left-width": "3px",
                            "border-style": "solid",
                            "display": column.display,
                        });

                        $(proxy._dragmouseOverTooltip).css({
                            'top': ypos + "px",
                            'left': xpos + "px",
                        });
                    }
                    proxy._headerDragStatus = "mousemove";
                }
                else {
                    if (!ej.isNullOrUndefined(proxy._dragmouseOverTooltip)) {
                        $(proxy._dragmouseOverTooltip).css({
                            'top': ypos + "px",
                            'left': xpos + "px",
                        });
                    }
                }
            }
        },      

        //Public method to reorder the Column position.
        reorderColumn: function (fieldName, targetColumnIndex) {
            var proxy = this, model = proxy.model,
                columns = model.columns, draggedColumn, 
                draggedColumnIndex, targetColumn, width, treeColumn,summaryTreeColumn,
                frozenWidth;

            if (model.showTotalSummary && model.collapsibleTotalSummary) summaryTreeColumn = columns[proxy._summaryColumnIndex];
            treeColumn = columns[model.treeColumnIndex];
            draggedColumn = proxy.getColumnByField(fieldName);
            draggedColumnIndex = columns.indexOf(draggedColumn);
            width = proxy.columnsWidthCollection[draggedColumnIndex];
            
            targetColumn = columns[targetColumnIndex];
            if (ej.isNullOrUndefined(draggedColumn) || ej.isNullOrUndefined(targetColumn) || targetColumn && targetColumn.nonColumn) return;
           
            if (proxy._frozenColumnsLength > 0) {

                frozenWidth = this._getFrozenColumnWidth();
                // Drag from non-freeze column and drop on frozen column
                if (proxy._frozenColumnsLength <= draggedColumnIndex && proxy._frozenColumnsLength > targetColumnIndex && proxy._frozenColumns.indexOf(draggedColumn) == -1) {                    
                    frozenWidth = frozenWidth + (this.columnsWidthCollection[draggedColumnIndex] + 18);
                    if (frozenWidth > this._gridWidth)
                        return false;
                    draggedColumn['isFrozen'] = true;
                    proxy._frozenColumns.splice(targetColumnIndex, 0, draggedColumn);
                }
                    // Drag from frozen column and drop on non-freeze column
                else if (proxy._frozenColumnsLength > draggedColumnIndex && proxy._frozenColumnsLength <= targetColumnIndex && proxy._frozenColumns.indexOf(draggedColumn) != -1) {                   
                    draggedColumn['isFrozen'] = false;
                    proxy._frozenColumns.splice(draggedColumnIndex, 1);
                    if (proxy._checkboxSelection && proxy._frozenColumns.length == 1 && proxy._frozenColumns[0]['nonColumn']) {
                        proxy._frozenColumns[0]['isFrozen'] = false;
                        proxy._frozenColumns.splice(0, 1);
                    }

                }
            }

            columns.splice(draggedColumnIndex, 1);
            proxy.columnsWidthCollection.splice(draggedColumnIndex, 1);
            columns.splice(targetColumnIndex, 0, draggedColumn);
            proxy.columnsWidthCollection.splice(targetColumnIndex, 0, width);
            
            model.treeColumnIndex = columns.indexOf(treeColumn);
            if (model.showTotalSummary && model.collapsibleTotalSummary) proxy._summaryColumnIndex = columns.indexOf(summaryTreeColumn);
            proxy._refreshFrozenColumns();
            proxy._updateWrapRowHeights(null,"virtualRefresh");
        },

        _refreshStackedHeader: function () {
            var proxy = this, model = proxy.model;
            if (model.showStackedHeader) {
                var stackedRows = model.stackedHeaderRows;
                for (var i = 0; i < stackedRows.length; i++) {
                    if (proxy._frozenColumnsLength != 0) {
                        var frznHeader = $(proxy.getHeaderContent().find(".e-frozenheaderdiv")),
                            movHeader = $(proxy.getHeaderContent().find(".e-movableheader")),
                            newFrzn = proxy._createStackedRow(stackedRows[i], true),
                            newMov = proxy._createStackedRow(stackedRows[i], false);
                        $(frznHeader.find("tr.e-stackedHeaderRow")[i]).replaceWith(newFrzn);
                        $(movHeader.find("tr.e-stackedHeaderRow")[i]).replaceWith(newMov);
                    }
                    else {
                        var stackedTR = proxy._createStackedRow(stackedRows[i], false);
                        if (proxy.getHeaderTable().find("tr.e-stackedHeaderRow")[i])
                            $(proxy.getHeaderTable().find("tr.e-stackedHeaderRow")[i]).replaceWith(stackedTR);
                        else
                            stackedTR.insertBefore(proxy.getHeaderTable().find("tr.e-columnheader:last"));
                    }
                }
                var args = {};
                args.requestType = "refresh";
                proxy._updateScrollCss();
            }
        },
        //Horizontal auto scroll while drag towards the end
        _autoHscroll: function (posx, posy, args) {
            var proxy = this,
                model = proxy.model,
                maxWidth,
                isHscroll,
                columns = model.columns,
                frozenWidth = 0,
                columnsWidth = proxy.columnsWidthCollection,
            gridContent = proxy.getScrollElement();

            isHscroll = gridContent.ejScroller("isHScroll");

            maxWidth = isHscroll ? gridContent.find(".e-content").get(0).scrollWidth : 0;
            proxy._viewPortWidth = gridContent.find(".e-content").width();

            if (proxy._frozenColumnsLength > 0) {
                frozenWidth = this._getFrozenColumnWidth();
            }
            var rightScrollPos = Math.round(((proxy._gridLeft + proxy._gridWidth) * 10)/ 100) + 18,
                leftScrollPos = Math.round((proxy._gridLeft * 10) / 100) + 18,
                topvalue = 0;

            if (((proxy._gridLeft + proxy._gridWidth) - rightScrollPos) < posx) {
                if (!proxy._timerDragDown) {

                    //Timer started for scrolling the content till the end.
                    proxy._timerDragDown = window.setInterval(function () {
                        if (maxWidth > (proxy._scrollLeft + proxy._viewPortWidth)) {
                            topvalue = proxy._scrollLeft;
                            topvalue = topvalue + 50;
                            proxy.getScrollElement().ejScroller("scrollX", topvalue, true);
                        }
                    }, 300);
                }
            }
            else {
                proxy._timerDragDown && (proxy._timerDragDown = window.clearInterval(proxy._timerDragDown));
            }            

            if ((posx < (proxy._gridLeft + frozenWidth + leftScrollPos)) && ( posx > (proxy._gridLeft + frozenWidth)) && proxy._scrollLeft != 0) {
                if (proxy._scrollLeft > 0) {
                    if (!proxy._timerDragUp) {
                        proxy._timerDragUp = window.setInterval(function () {
                            if (proxy._scrollLeft > 0) {
                                topvalue = proxy._scrollLeft;
                                topvalue = topvalue - 50;
                                if (topvalue < 0)
                                    topvalue = 0;
                                proxy.getScrollElement().ejScroller("scrollX", topvalue, true);
                            }
                        }, 300);
                    }
                }
                else {
                    proxy._timerDragUp && (proxy._timerDragUp = window.clearInterval(proxy._timerDragUp));
                }
            }
            else {
                proxy._timerDragUp && (proxy._timerDragUp = window.clearInterval(proxy._timerDragUp));
            }
        },

        _resetPrivateProperties: function () {
            this._timerDragDown && (this._timerDragDown = window.clearInterval(this._timerDragDown));
            this._timerDragUp && (this._timerDragUp = window.clearInterval(this._timerDragUp));
            var proxy = this, model = proxy.model;
            proxy.cancelDrop = false;
            proxy._draggedRecord = null;            
            proxy._draggedRecords = [];
            proxy._dragMouseDown = false;
            proxy._timerDragDown = null;
            proxy._timerDragUp = null;
            proxy._childItem = false;
            proxy._dragMouseLeave = false;
            proxy._moveRow = false;
            proxy._currentRecord = null;
            proxy._currentIndex = null;
            proxy._droppedRecord = null;
            proxy._insertAsChild = false;
            proxy._insertAbove = false;
            proxy._insertBelow = false;
            if (model.allowDragAndDrop && model.enableVirtualization && (ej.browserInfo().name == "chrome" || ej.isTouchDevice())) {
                proxy._dummyElement && $(proxy._draggedElement).remove();
                proxy._dummyElement = false;
                proxy._draggedElement = null;
            }          
            proxy._dragTooltipTimer = null;
            proxy._dragColumnTooltipTimer = null;
            proxy._dragmouseOverTooltip = null;
        },
        // To select the dragged Record after drag stopped.
        _selectDraggedRow: function ($tr) {
            var proxy = this, model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords(),
                rowIndex = updatedRecords.indexOf(proxy._draggedRecord);
            if (rowIndex != -1) {
                proxy._focusTreeGridElement();
                if (model.allowSelection && model.selectionSettings.selectionMode == "row" && !(proxy._rowSelectingEventTrigger(this.selectedRowIndex(), rowIndex))) {
                    proxy.selectRows(rowIndex);
                    proxy._rowSelectedEventTrigger(rowIndex);
                }
            }
        },
        //Add child records to the data source
        _addChildItem: function (record) {
            var currentRecord, proxy = this, length=0;
            length = record.childRecords.length;
            for (var i = 0; i < length; i++) {
                currentRecord = record.childRecords[i];
                if (proxy.dataSource() instanceof ej.DataManager) {
                    if (proxy.dataSource().dataSource.offline && proxy.dataSource().dataSource.json) {
                        proxy.dataSource().dataSource.json.push(currentRecord.item);
                        if (currentRecord.hasChildRecords) {
                            proxy._addChildItem(currentRecord);
                        }
                    }
                } else {
                    proxy.dataSource().push(currentRecord.item);                     
                    if (currentRecord.hasChildRecords) {
                        proxy._addChildItem(currentRecord);
                    }
                }
            }
        },
        //Update the changes in DataSource.
        _updateDataSource: function () {
            var proxy = this,
                model = proxy.model,
                draggedRecord = proxy._draggedRecord,
                droppedRecord = proxy._droppedRecord,
                tempDataSource, idx, childIndex;
            if (proxy.dataSource() instanceof ej.DataManager) {
                if (proxy.dataSource().dataSource.offline && proxy.dataSource().dataSource.json) {
                    tempDataSource = proxy.dataSource().dataSource.json;
                }
            }
            else 
                tempDataSource = proxy.dataSource();
            
            if (tempDataSource && !droppedRecord.parentItem) {
                idx = tempDataSource.indexOf(droppedRecord.item);
                if (proxy._insertAbove) {
                    if (!model.parentIdMapping)
                        tempDataSource.splice(idx, 0, draggedRecord.item);
                }
                else if (proxy._insertBelow) {
                    if (!model.parentIdMapping)
                        tempDataSource.splice(idx + 1, 0, draggedRecord.item);
                }
            }
            else if (!model.parentIdMapping && droppedRecord.parentItem) {
                if (proxy._insertAbove || proxy._insertBelow) {
                    droppedRecord.parentItem.item[proxy.model.childMapping] = [];
                    var record = droppedRecord.parentItem.childRecords;
                    for (var i = 0; i < record.length; i++) {
                        droppedRecord.parentItem.item[proxy.model.childMapping].splice(i, 0, record[i].item)
                    }
                }               
            }

            if (model.parentIdMapping) {
                if (draggedRecord.parentItem) {
                    if (proxy._insertAbove || proxy._insertBelow) {
                        if (model.isFromGantt == true) {
                            draggedRecord.item[model.parentIdMapping] = droppedRecord.item[proxy.model.parentIdMapping];
                        }
                        else {
                            draggedRecord[model.parentIdMapping] = droppedRecord[proxy.model.parentIdMapping];
                            draggedRecord.item[model.parentIdMapping] = droppedRecord[proxy.model.parentIdMapping];
                        }
                    }
                    else {
                        if (model.isFromGantt == true) {
                            draggedRecord.item[model.parentIdMapping] = droppedRecord.item[proxy.model.idMapping];
                        }
                        else {
                            draggedRecord[model.parentIdMapping] = droppedRecord[proxy.model.idMapping];
                            draggedRecord.item[model.parentIdMapping] = droppedRecord[proxy.model.idMapping];
                        }
                    }
                }
                else {
                    draggedRecord.item[model.parentIdMapping] = null;
                    draggedRecord[model.parentIdMapping] = null;
                }

                if (proxy.dataSource() instanceof ej.DataManager) {
                    if (proxy.dataSource().dataSource.offline && proxy.dataSource().dataSource.json) {
                        proxy.dataSource().dataSource.json.push(draggedRecord.item);
                    }
                } else {

                    if (proxy._insertAbove || proxy._insertBelow || proxy._insertAsChild)
                        proxy.dataSource().push(draggedRecord.item);
                    if (draggedRecord.hasChildRecords) {
                        proxy._addChildItem(draggedRecord);
                    }
                }

            }
        },

        //This method is to delete the dragged record 
        _deleteDragRow: function ($tr) {
            var proxy = this,
                model = proxy.model,
                 deletedRowIndex,
                 deletedRow,
                 dataRows,
                 isUpdateWBS,
                 targetRowIndex,
                 args = {},
                 updatedRecords = proxy.getUpdatedRecords(),
                 flatRecords = model.flatRecords;
            if (updatedRecords.indexOf(proxy._draggedRecord) != -1) {
                deletedRowIndex = updatedRecords.indexOf(proxy._draggedRecord);
                deletedRow = updatedRecords[deletedRowIndex];
            }
            else
            {
                deletedRowIndex = flatRecords.indexOf(proxy._draggedRecord);
                deletedRow = flatRecords[deletedRowIndex];
            }
            args.tr = $tr;
            args.data = deletedRow;
            args.requestType = ej.TreeGrid.Actions.Delete;
            args.isDragAndDropDelete = true;                        
            proxy._removeRecords(args);                       
            proxy._previousIndex = -1;
        },
        // To revert the changes if drop get cancelled.
        _revertDragging: function (args) {
            var proxy = this,
                model=proxy.model,
                flatRecords = model.flatRecords;
            proxy._draggedRecord = args.draggedRow;
            proxy._deleteDragRow();
            if (!proxy._draggedRecord.hasChildRecords) {
                flatRecords.splice(args.previousItemIndex, 0, proxy._draggedRecord);
                if (model.isFromGantt)
                    model.ids.splice(args.previousItemIndex, 0, (proxy._draggedRecord.taskId).toString());
                flatRecords[args.previousItemIndex].parentItem = args.previousItem.parentItem;
                if (!ej.isNullOrUndefined(args.previousParentItem)) {
                    flatRecords[args.previousItemIndex].parentItem.childRecords.splice(args.childIndex, 0, proxy._draggedRecord);
                    flatRecords[args.previousItemIndex].parentItem.hasChildRecords = true;
                    flatRecords[args.previousItemIndex].parentItem.hasFilteredChildRecords = true;
                    flatRecords[args.previousItemIndex].parentItem.expanded = true;
                }
                flatRecords[args.previousItemIndex].level = args.previousLevel;
            }
            else {
                flatRecords.splice(args.previousItemIndex, 0, proxy._draggedRecord);
                if (model.isFromGantt)
                    model.ids.splice(args.previousItemIndex, 0, (proxy._draggedRecord.taskId).toString());
                if (!ej.isNullOrUndefined(flatRecords[args.previousItemIndex].parentItem)) {
                    flatRecords[args.previousItemIndex].parentItem = args.previousParentItem;
                    if (!ej.isNullOrUndefined(args.previousParentItem))
                        flatRecords[args.previousItemIndex].parentItem.childRecords.splice(args.childIndex, 0, proxy._draggedRecord);
                }
                var recordIndex = model.flatRecords.indexOf(proxy._draggedRecord);
                proxy._updateChildRecord(proxy._draggedRecord,recordIndex, args.previousItem.index);
                proxy._draggedRecord.level = args.previousLevel;
                proxy._updateChildRecordLevel(proxy._draggedRecord, args.previousLevel);
            }
            if (model.parentIdMapping) {
                flatRecords[args.previousItemIndex][model.parentIdMapping] = args.previousItem[model.parentIdMapping];
                flatRecords[args.previousItemIndex].item[model.parentIdMapping] = args.previousItem[model.parentIdMapping];
            }
            if (!ej.isNullOrUndefined(args.previousItem.parentItem)) {
                args.previousItem.parentItem.hasChildRecords = args.parentChildState;
                args.previousItem.parentItem.expanded = args.parentExpandedState;
                if (model.childMapping && args.previousItem.parentItem.item[proxy.model.childMapping])
                    args.previousItem.parentItem.item[proxy.model.childMapping].splice(args.childIndex, 0, args.previousItem.item);
            }
            if (model.isFromGantt) {
                flatRecords[args.previousTargetItemIndex] = args.previousTarget;
                if (args.previousTarget.isAutoSchedule == true) {
                    args.previousTarget.startDate = args.previousTarget.item[args.model.startDateMapping] = new Date(args.clonedTarget.startDate);
                    args.previousTarget.endDate = new Date(args.clonedTarget.endDate);
                    args.previousTarget.duration = args.clonedTarget.duration;
                    args.previousTarget.status = args.clonedTarget.status;
                    if (args.model.endDateMapping) {
                        args.previousTarget.item[args.model.endDateMapping] = new Date(args.clonedTarget.endDate);
                    }
                    if (args.model.progressMapping) {
                        args.previousTarget.item[args.model.progressMapping] = args.clonedTarget.status;
                    }
                    if (args.model.durationMapping) {
                        args.previousTarget.item[args.model.durationMapping] = args.clonedTarget.duration;
                    }
                    args.previousTarget.expanded = args.clonedTarget.expanded;
                    var ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");
                    ganttObject._updateTargetGanttRecord(args.previousTarget);
                }
                else {
                    args.previousTarget.manualStartDate = new Date(args.clonedTarget.manualStartDate);
                    args.previousTarget.manualEndDate = new Date(args.clonedTarget.manualEndDate);
                    args.previousTarget.manualDuration = args.clonedTarget.manualDuration;
                    args.previousTarget.status = args.clonedTarget.status;
                    if (args.model.progressMapping) {
                        args.previousTarget.item[args.model.progressMapping] = args.clonedTarget.status;
                    }
                    args.previousTarget.expanded = args.clonedTarget.expanded;
                    var ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");
                    ganttObject._updateGanttRecord(args.previousTarget);
                }
            }
            if (model.enableWBS) {
                var ddSiblings,
                    draggedRecord = proxy._draggedRecord;
                if (draggedRecord.parentItem)
                    ddSiblings = draggedRecord.parentItem.childRecords;
                else {
                    var flatData = model.flatRecords,
                        Level0 = flatData.filter(function (item) {
                            return item && item.level == 0;
                        });
                    ddSiblings = Level0;
                }
                var ddIndex = ddSiblings.indexOf(draggedRecord),
                    selectedRecords = ddSiblings.slice(ddIndex, ddSiblings.length),
                    parentVal = draggedRecord.parentItem ? draggedRecord.parentItem.WBS : null,
                    lastVal = ddIndex + 1;
                proxy.reCalculateWBS(selectedRecords, lastVal, parentVal);
            }
            proxy.processBindings(args);
            if (model.currentViewData.length > 0)
                ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);
        },

        //To splice the child records if user drag and dropped the parent item.
        _updateChildRecord: function (record, count,expandState){
            var currentRecord,
                proxy = this,
                model = proxy.model,
                length;
            if (!record.hasChildRecords)
                return 0;
            length = record.childRecords.length;
            for (var i = 0; i < length; i++) {
                currentRecord = record.childRecords[i];
                count++;
                model.flatRecords.splice(count, 0, currentRecord);
                model.updatedRecords.splice(count, 0, currentRecord);
                if (model.isFromGantt)
                    model.ids.splice(count, 0, (currentRecord.taskId).toString());
                if (currentRecord.hasChildRecords) {
                    count = proxy._updateChildRecord(currentRecord, count);
                }
            }
            return count;
        },
        //This method is to leveling the records when it is inserted as child.
        _recordLevel: function (recordIndex, args) {
            var j = 0, proxy = this,
                model = proxy.model,
                draggedRecord=proxy._draggedRecord,
                droppedRecord = proxy._droppedRecord,
                childItem = proxy.model.childMapping;
            if (!droppedRecord.hasChildRecords) {
                droppedRecord.hasChildRecords = true;
                droppedRecord.hasFilteredChildRecords = true;
                if (proxy._isLoadonDemand)
                    droppedRecord.isChildRetrieved = true;
                if (ej.util.isNullOrUndefined(droppedRecord.childRecords) || $.isEmptyObject(droppedRecord.childRecords)) {
                    droppedRecord.childRecords = [];
                    if (!model.parentIdMapping)
                        droppedRecord.item[childItem] = [];
                }
            }
            if (proxy._insertAsChild) {
                draggedRecord.parentItem = droppedRecord;
                droppedRecord.childRecords.splice(0, 0, draggedRecord);
                //Update hierarchyRowIndex for level 1 and above records
                var childRecords = droppedRecord.childRecords;
                proxy._updateHierarchyRowIndexData(args, childRecords, 0);
                if (!ej.isNullOrUndefined(draggedRecord) && !ej.isNullOrUndefined(droppedRecord.item[childItem]) && !model.parentIdMapping) {
                    droppedRecord.item[childItem].splice(0, 0, draggedRecord.item);
                }
                if (!draggedRecord.hasChildRecords)
                    draggedRecord.level = droppedRecord.level + 1;
                else {
                    var level = 1;
                    draggedRecord.level = droppedRecord.level + 1;
                    proxy._updateChildRecordLevel(draggedRecord, level);
                }
                if (!droppedRecord.expanded && droppedRecord.hasChildRecords && model.isFromGantt && !model.enableVirtualization) {
                    proxy.updateExpandStatus(droppedRecord, true);
                }
                droppedRecord.expanded = true;               
                if (proxy._isLoadonDemand && !droppedRecord.isChildRetrieved)
                    droppedRecord.expanded = false;                
                proxy._parentRecords.push(droppedRecord);
            }
        },
        // This method is to level the parent record at the end of it hierarchy.
        _updateChildRecordLevel: function (record,level) {
            var proxy = this,
                length=0,
                currentRecord;
            level++;
            if (!record.hasChildRecords)
                return 0;
            length = record.childRecords.length;
            for (var i = 0; i < length ; i++) {
                currentRecord = record.childRecords[i];
                currentRecord.level = record.parentItem ? record.parentItem.level + level : record.level + 1;
                if (currentRecord.hasChildRecords) {
                    level--;
                    level = proxy._updateChildRecordLevel(currentRecord, level);
                }
            }
            return level;
        },
                   
        //To renter cancel icon in filtering elements.
        _filterBarClose: function (e) {
            var $target = $(e.target);
            if (e.type == "click" && $target.hasClass("e-cancel")) {
                var $targetText = $target.prev();
                    $targetText.focus().val("");
                $targetText.prop('checked', false);
                $targetText.trigger("keyup");
                e.stopPropagation();
            }
            if (e.type == "focusin") { 
                $target = $(e.target).next();
                this.getHeaderTable().find(".e-cancel").addClass("e-hide");
                $target.removeClass("e-hide");
            }
        },

        _checkIdenticalValue: function (val, currentFilterColumn) {
            var proxy = this,
                model = proxy.model,
                flag,
                filteredColumns = model.filterSettings.filteredColumns,
                columnLength = filteredColumns.length;
            for (var col = 0; col < columnLength; col++) {
                if (filteredColumns[col].value === val && filteredColumns[col].field == currentFilterColumn.field) {
                    flag = true;
                    break;
                }               
            }
            return flag;            
        },
        //To wireup method while user enter text to the filter textbox.
        _filterBarHandler: function (e) {
            var proxy = this,
                $target,
                model = proxy.model,filteringColumn;
            //cancel the edited cell and row
            if (proxy.model.isEdit)
                proxy.cancelEditCell();
            else if (proxy._isRowEdit)
                proxy.cancelRowEditCell();
            /* flag new added record is sorted or filttered in treegrid */
            proxy._isRefreshAddedRecord = false;

            /* clear selction*/
            proxy.clearAllSelection();

            if (e.target == undefined)
                $target =$(e);
            else
                $target = $(e.target);
            
            if (this.model.allowFiltering ) {
                var cols=proxy.model.columns;
                var matchcase;
                matchcase = proxy._cellEditingDetails;
                var key = proxy._queryManagar.queries[0], $element;
                //Get input element onEnter event
                if ($target.hasClass("e-ddl"))
                    $element = $target.find("input.e-field");
                else
                    $element = $target;
                proxy._fieldName = $element[0].id.replace(proxy._id + "_", "").replace("_filterbarcell", "");
                this._currentFilterColumn = ej.TreeGrid.getColumnByField(model.columns, proxy._fieldName);
                proxy._fieldName = this._currentFilterColumn.field;
                var currentFilterCol = this._currentFilterColumn, currentDateformat;
                if ($target.hasClass("e-checkbox"))
                    proxy._filterString = $target[0].checked;
                else if ($target.hasClass("e-ddl")) {
                    // To get the value from dropdown filter (onEnter) mode
                    $target = $target.find("input.e-dropdownlist");
                    var dropDownElement = $("#" + $target[0].id).data("ejDropDownList");
                    if (dropDownElement.selectedIndexValue == -1 || (proxy._checkIdenticalValue(dropDownElement._selectedValue, this._currentFilterColumn)))
                        return true;
                    proxy._filterString = dropDownElement._selectedValue;
                }
                else if ($target.hasClass("e-dropdownlist")) {
                    // To get the value from dropdown filter (immediate) mode
                    var dropdownobject = $("#" + $target[0].id).data("ejDropDownList");
                    if (dropdownobject.selectedIndexValue == -1 || (proxy._checkIdenticalValue(dropdownobject._selectedValue, this._currentFilterColumn)))
                        return true;
                    if (dropdownobject._selectedValue==proxy._dropDownListClearText)
                        proxy._filterString = "";
                    else
                        proxy._filterString = dropdownobject._selectedValue;
                }
                else if ($target.hasClass("e-numerictextbox")) {
                    var numericObject = $("#" + $target[0].id).data("ejNumericTextbox"),
                        modelValue = numericObject.model.value == null ? "" : (numericObject.model.value).toString();

                    if (ej.isNullOrUndefined(e.type)) {
                        var filteredColumns = model.filterSettings.filteredColumns,
                            length = filteredColumns.length,
                            flag = false;
                        if (modelValue == "" && length == 0)
                            flag = true;
                        for (var col = 0; col < length; col++) {
                            var value = ej.isNullOrUndefined(filteredColumns[col].value) ? "" : filteredColumns[col].value.toString();
                            if (value == modelValue && filteredColumns[col].field == this._currentFilterColumn.field) {
                                flag = true;
                                break;
                            }
                        }
                        if (flag) return true;
                    }
                    proxy._filterString = $target.closest("input").val();
                }
                else
                    proxy._filterString = $target.closest("input").val();
                if ((this._currentFilterColumn.filterEditType == "datepicker" || this._currentFilterColumn.filterEditType == "datetimepicker") && proxy._filterString != "") {
                    if (proxy._filterString) {
                        if (currentFilterCol.format)
                            currentDateformat = proxy._getDateFormat(this._currentFilterColumn.format);
                        else
                            currentDateformat = proxy.model.dateFormat;
                        if (ej.isNullOrUndefined(ej.parseDate(proxy._filterString, currentDateformat, model.locale)))
                            proxy._filterString = "";
                        else
                            proxy._filterString = ej.parseDate(proxy._filterString, currentDateformat, model.locale);
                    }
                }
                if (proxy._filterString == "") {
                    filteringColumn = model.filterSettings.filteredColumns.filter(function (column) {
                        if (column.field === proxy._fieldName)
                            return true;
                    });
                    if (!filteringColumn.length) {
                        proxy._resizeFilteringElements();
                        return true;
                    }
                }
               
                proxy._validateFilterValue(proxy._filterString);
                if (proxy._batchEditStarts)
                    proxy._batchTempDetails.filterTarget = $target;
                proxy.filterColumn(proxy._fieldName, this._operator, proxy._filterString, this._predicate);                
                proxy._storePreviousFilteredValue(proxy._fieldName, proxy._filterString, this._operator);
                if (proxy.model.currentViewData.length > 0 && proxy.model.enableAltRow)
                    ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);                
                proxy._cancelSaveTools();
            }
        },
        //Method to validate search string type and assign operators accordingly.
        _validateFilterValue: function (_value) {
            var proxy = this;
            this._predicate = "and";
            switch (this._currentFilterColumn.filterEditType) {
                case "numericedit":
                    this._operator = ej.FilterOperators.equal;
                    break;
                case "datepicker":
                    this._operator = ej.FilterOperators.equal;
                    
                    var _format;
                    if (this._currentFilterColumn.format == "" || this._currentFilterColumn.format == undefined)
                        _format = ej.preferredCulture().calendar.patterns.d; //System Date format
                    else
                        _format = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                    if (this._currentFilterbarValue != "")
                        this._currentFilterbarValue = ej.format(_value, _format, this.model.locale);
                    break;
                case "datetimepicker":
                    this._operator = ej.FilterOperators.equal;
                  
                    var _format;
                    if (this._currentFilterColumn.format == "" || this._currentFilterColumn.format == undefined)
                        _format = ej.preferredCulture().calendar.patterns.f; //System DateTime format
                    else
                        _format = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                    if (this._currentFilterbarValue != "")
                        this._currentFilterbarValue = ej.format(_value, _format, this.model.locale);
                    break;
                case "stringedit":
                    this._operator = ej.FilterOperators.startsWith;
                    break;
                case "dropdownedit":
                    var filterColumn = this._currentFilterColumn;
                    if (filterColumn.editType == "dropdownedit" && filterColumn.editParams && filterColumn.editParams.showCheckbox)
                        this._operator = ej.FilterOperators.contains;
                    else
                        this._operator = ej.FilterOperators.equal;
                    break;
                case "booleanedit":
                    if (proxy._filterString == true || proxy._filterString == "1")
                        proxy._filterString = true;
                    else if (proxy._filterString == "false" || proxy._filterString == "0")
                        proxy._filterString = false;
                    this._operator = ej.FilterOperators.equal;
                    break;
                default:
                    this._operator = ej.FilterOperators.equal;
            }
        },

        _getStackedColumnByTarget: function (target) {
            var cls = (target.get(0) || {}).className, match = /e-row([0-9])-column([0-9])/.exec(cls),
                rIndx = match[1], cIndx = match[2], key = [rIndx, "stackedHeaderColumns", cIndx].join(".");

            return ej.getObject(key, this.model.stackedHeaderRows);
        },

        // Method to render header tooltip and header template tooltip.
        _headermouseHover: function (e) {
            e.preventDefault();
            var proxy = this,
                model = proxy.model;
            if (model.showGridCellTooltip) {
                if (!proxy._dragTooltip) {
                    var $target = $(e.target),
                        posx = 0,
                        posy = 0,
                        column = $target.closest('div.e-headercelldiv'),
                        tooltiptable, tooltipbody, textContent, tooltipWidth,
                        tooltipHeight, containerWidth, containerHeight,
                        currentColumn, columnFieldName, stackedHeaderColumn;
                    proxy._headerMouseLeave();
                    if (!e) e = window.event;
                    if (e.originalEvent.pageX || e.originalEvent.pageY) {
                        posx = e.originalEvent.pageX;
                        posy = e.originalEvent.pageY;
                    }
                    else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                        posx = e.originalEvent.clientX + document.body.scrollLeft
                            + document.documentElement.scrollLeft;
                        posy = e.originalEvent.clientY + document.body.scrollTop
                            + document.documentElement.scrollTop;
                    }
                    textContent = column.clone();
                    if (textContent.find(".e-ascending").length > 0) {
                        textContent.find(".e-ascending").remove();
                    } else if (textContent.find(".e-descending").length > 0) {
                        textContent.find(".e-descending").remove();
                    }
                    textContent.find(".e-chkbox-wrap").remove();
                    if ($target.closest(".e-headercell").hasClass("e-headertemplate")) {
                        textContent = textContent.clone();
                    } else {
                        textContent = textContent.find("span").clone();
                    }
                    if ($target.hasClass("e-stackedtooltip")) {
                        stackedHeaderColumn = proxy._getStackedColumnByTarget($target);
                        if (!ej.isNullOrUndefined(stackedHeaderColumn["tooltip"]) && stackedHeaderColumn["tooltip"].slice(0, 1) !== "#")
                            textContent = stackedHeaderColumn["tooltip"]; // $target.clone();
                        else {
                            var t = $(stackedHeaderColumn["tooltip"]).clone();
                            textContent = t.html();
                        }
                    }
                    tooltiptable = ej.buildTag("table.e-tooltiptable e-zerospace", "", { 'padding': '0' });
                    tooltipbody = ej.buildTag("tbody", textContent, {}, {});
                    tooltiptable.append(tooltipbody);
                    var isNonEmptyText = $(textContent).text().replace(/[ \t\r]+/g, "").length > 0 ? true : false;
                    if (tooltipbody) {
                        columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname");
                        currentColumn = this.getColumnByField(columnFieldName);
                        if ((currentColumn && (ej.isNullOrUndefined(currentColumn['headerTooltip'])
                            || currentColumn['headerTooltip'] == "") && model.showGridCellTooltip && isNonEmptyText)
                            || $target.hasClass("e-stackedtooltip")) {
                                proxy._headerMouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#headertooltip" + proxy._id + "", tooltiptable,
                                    {
                                        "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                        'position': 'absolute',
                                        'z-index': proxy.getMaxZIndex() + 1,
                                        'padding': '0',
                                        'border-radius': '3px'
                                    }, {});
                                model.cssClass && (proxy._headerMouseOverTooltip.addClass(model.cssClass));
                                //Providing a time delay to render the tooltip to improve UI
                                proxy._headerTooltipTimer = setTimeout(function () {
                                    $(document.body).append(proxy._headerMouseOverTooltip);
                                    var element = $("#" + proxy._id.replace("ejTreeGrid", "")),
                                        position = proxy.getOffsetRect(element[0]);
                                    containerWidth = $(element).width() + position.left;
                                    containerHeight = $(element).height() + position.top;
                                    tooltipWidth = $(proxy._headerMouseOverTooltip).width();
                                    tooltipHeight = $(proxy._headerMouseOverTooltip).height();
                                    // Calculation for reposition the tooltip when tooltip is exist the container height or width
                                    if ((posx + tooltipWidth + 15) >= containerWidth) {
                                        posx = posx - tooltipWidth - 15;
                                    }
                                    if ((posy + tooltipHeight + 10) >= containerHeight) {
                                        posy = posy - tooltipHeight - 10;
                                    }
                                    $(proxy._headerMouseOverTooltip).css({ "top": posy, "left": posx });
                                }, 700);
                            
                        } else if (currentColumn && !ej.isNullOrUndefined(currentColumn['headerTooltip']) && currentColumn['headerTooltip'] != "") {
                            var dataObj = {};
                            dataObj["column"] = currentColumn;
                            proxy._createColumnTooltipTemplate(currentColumn['headerTooltip']);
                            proxy._headerMouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#headertooltip" + proxy._id + "", "",
                                {
                                    "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                    'position': 'absolute',
                                    'z-index': proxy.getMaxZIndex() + 1,
                                    'padding': '0',
                                    'border-radius': '3px'
                                }, {});
                            model.cssClass && (proxy._headerMouseOverTooltip.addClass(model.cssClass));
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "columnTooltipTemplate"](dataObj), {}, {});
                            proxy._headerMouseOverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                            var isNonEmptyText = $(tooltipbody).text().replace(/[ \t\r]+/g, "").length > 0 ? true : false;
                            if (isNonEmptyText){
                            proxy._headerTooltipTimer = setTimeout(function () {
                                $(document.body).append(proxy._headerMouseOverTooltip);
                                if (!model.isFromGantt) {
                                    var position = proxy.getOffsetRect(proxy.element[0]);
                                    containerWidth = $(proxy.element).width() + position.left;
                                    containerHeight = $(proxy.element).height() + position.top;
                                }
                                else {
                                    var element = $("#" + proxy._id.replace("ejTreeGrid", "")),
                                        position = proxy.getOffsetRect(element[0]);
                                    containerWidth = $(element).width() + position.left;
                                    containerHeight = $(element).height() + position.top;
                                }
                                tooltipWidth = $(proxy._headerMouseOverTooltip).width();
                                tooltipHeight = $(proxy._headerMouseOverTooltip).height();
                                // Calculation for reposition the tooltip when tooltip is exist the container height or width
                                if ((posx + tooltipWidth + 15) >= containerWidth) {
                                    posx = posx - tooltipWidth - 15;
                                }
                                if ((posy + tooltipHeight + 10) >= containerHeight) {
                                    posy = posy - tooltipHeight - 10;
                                }
                                $(proxy._headerMouseOverTooltip).css({ "top": posy, "left": posx });
                            }, 700);
                            }
                        }
                    }
                }
            }
        },
        
        _mouseHover: function (e) {            
            var proxy = this,
                model = proxy.model;
            if (model.showGridCellTooltip) {
                if (this._dragMouseDown != true) {
                    var $target = $(e.target),
                        posx = 0,
                        posy = 0,
                        $ganttGridRows = proxy.getRows(),
                        row = $target.closest('tr'),
                        column = $target.closest('td.e-rowcell'),
                        recordIndex = this.getIndexByRow(row),
                        item = model.currentViewData[recordIndex],
                        tooltiptable, tooltipbody, textContent, tooltipWidth,
                        tooltipHeight, containerWidth, containerHeight,
                        columns = model.columns,
                        currentColumn;

                    if (row.hasClass("e-footersummaryrow")) {
                        recordIndex = proxy._getFooterRows().index(row);
                        item = proxy._footerSummaryRecord[recordIndex];
                    }
                    
                    if (recordIndex >= 0 || row.hasClass("e-footersummaryrow")) {
                        var columnIndex = proxy.getCellIndex(e);
                        proxy._cellMouseLeave();

                        currentColumn = columns[columnIndex];
                        if (!e) e = window.event;
                        if (e.originalEvent.pageX || e.originalEvent.pageY) {
                            posx = e.originalEvent.pageX;
                            posy = e.originalEvent.pageY;
                        }
                        else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                            posx = e.originalEvent.clientX + document.body.scrollLeft
                                + document.documentElement.scrollLeft;
                            posy = e.originalEvent.clientY + document.body.scrollTop
                                + document.documentElement.scrollTop;
                        }
                        if (item || row.hasClass("e-footersummaryrow")) {
                            if (columnIndex != model.treeColumnIndex) {
                                var clonedCellElement = column.clone();
                                clonedCellElement.find(".e-chkbox-wrap").remove();
                                textContent = clonedCellElement.children().length ? clonedCellElement.children().clone() : clonedCellElement.clone(); // Copying the DOM element of particular cell.
                            }
                            else {
                                if (column.find(".e-cell").length > 0) {
                                    textContent = column.find(".e-cell").clone();
                                    textContent.find(".e-chkbox-wrap").remove();
                                }
                                else if (column.hasClass("e-summaryrowcell")) {
                                    if (column.find(".e-summarytitle").length > 0) {
                                        textContent = column.find(".e-summarytitle").clone();
                                    } else {
                                        textContent = column.clone();
                                        $(textContent).css({ "cssText": "padding-left: 0px;" });
                                    }
                                }
                            }
                            textContent && textContent.addClass("e-treegrid-textWrap");
                            tooltiptable = ej.buildTag("table.e-tooltiptable e-zerospace", "", { 'padding': '0' });

                            if (!model.showGridExpandCellTooltip || (model.showGridExpandCellTooltip && columnIndex == proxy.model.treeColumnIndex)) {
                                tooltipbody = ej.buildTag("tbody", textContent, {}, {});
                            }
                            tooltiptable.append(tooltipbody);

                            var isNonEmptyText = $(textContent).text().replace(/[ \t\r]+/g, "").length > 0 ? true : false;

                            if (tooltipbody && $(column).find("#" + proxy._id + "EditForm").length == 0 && !proxy._isCommandColumn(currentColumn)) {
                                if ((!model.cellTooltipTemplate && (ej.isNullOrUndefined(currentColumn['tooltip']) || currentColumn['tooltip'] == "")) || (model.cellTooltipTemplate && !ej.isNullOrUndefined(currentColumn['tooltip']) && currentColumn['tooltip'] == "")) {
                                    if (!(model.showGridExpandCellTooltip) && isNonEmptyText
                                        || (model.showGridExpandCellTooltip && isNonEmptyText)) {
                                        var z_index = $("#" + this._id + "detailscellwrapper").length > 0 ? 110 : 5;
                                        proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#tooltipgantt" + proxy._id + "", tooltiptable,
                                            {
                                                "top": (posy + 10) + "px", 
                                                'left': 'auto',
                                                'position': 'absolute',
                                                'z-index': proxy.getMaxZIndex() + 1,
                                                'padding': '0',
                                                'border-radius': '3px'
                                            }, {});
                                        model.cssClass && (proxy._mouseOverTooltip.addClass(model.cssClass));
                                        //Providing a time delay to render the tooltip to improve UI
                                        proxy._tooltipTimer = setTimeout(function () {
                                            $(document.body).append(proxy._mouseOverTooltip);
                                            if (!model.isFromGantt) {
                                                var position = proxy.getOffsetRect(proxy.element[0]);
                                                containerWidth = $(proxy.element).width() + position.left;
                                                containerHeight = $(proxy.element).height() + position.top;
                                            }
                                            else {
                                                var element = $("#" + proxy._id.replace("ejTreeGrid", "")),
                                                    position = proxy.getOffsetRect(element[0]);
                                                containerWidth = $(element).width() + position.left;
                                                containerHeight = $(element).height() + position.top;
                                            }
                                            tooltipWidth = $(proxy._mouseOverTooltip).width();
                                            tooltipHeight = $(proxy._mouseOverTooltip).height();
                                            // Calculation for reposition the tooltip when tooltip is exist the container height or width
                                            var windowWidth = $(window).width(),
                                                leftSide = position.left + posx,
                                                rightSide = windowWidth - (position.left + posx);
                                            if ((posx + tooltipWidth + 15) >= containerWidth && (leftSide > rightSide)) {
                                                    posx = containerWidth - tooltipWidth - 15;
                                                    if (posx < 0)
                                                        posx = position.left;
                                                }
                                            if ((posy + tooltipHeight + 10) >= containerHeight) {
                                                posy = posy - tooltipHeight - 10;
                                            }
                                            $(proxy._mouseOverTooltip).css({ "top": posy, "left": posx });
                                        }, 700);
                                    }
                                }
                                else if (model.cellTooltipTemplate || (!ej.isNullOrUndefined(currentColumn['tooltip']) && currentColumn['tooltip'] != "")) {
                            var z_index = $("#" + this._id + "detailscellwrapper").length > 0 ? 110 : 5,
                                dataObj = {};
                            dataObj["record"] = item;
                            dataObj["column"] = columns[columnIndex];
                            if (!ej.isNullOrUndefined(currentColumn['tooltip']))
                                proxy._createColumnTooltipTemplate(currentColumn['tooltip']);
                                proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt e-js#tooltipgantt" + proxy._id + "", "",
                                    {
                                        "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                        'position': 'absolute',
                                        'z-index': proxy.getMaxZIndex() + 1,
                                        'padding': '0',
                                        'border-radius': '3px'
                                    }, {});
                                model.cssClass && (proxy._mouseOverTooltip.addClass(model.cssClass));
                                proxy.tooltipState = "Template";
                                var tooltipbody;
                                if (!ej.isNullOrUndefined(currentColumn['tooltip']))
                                    tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "columnTooltipTemplate"](dataObj), {}, {});
                                    else
                                    tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](dataObj), {}, {});
                                proxy._mouseOverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                                var isNonEmptyText = $(tooltipbody).text().replace(/[ \t\r]+/g, "").length > 0 ? true : false;
                                if (isNonEmptyText){
                                proxy._tooltipTimer = setTimeout(function () {
                                    $(document.body).append(proxy._mouseOverTooltip);
                                    if (!model.isFromGantt) {
                                        var position = proxy.getOffsetRect(proxy.element[0]);
                                        containerWidth = $(proxy.element).width() + position.left;
                                        containerHeight = $(proxy.element).height() + position.top;
                                    }
                                    else {
                                        var element = $("#" + proxy._id.replace("ejTreeGrid", "")),
                                            position = proxy.getOffsetRect(element[0]);
                                        containerWidth = $(element).width() + position.left;
                                        containerHeight = $(element).height() + position.top;
                                    }
                                    tooltipWidth = $(proxy._mouseOverTooltip).width();
                                    tooltipHeight = $(proxy._mouseOverTooltip).height();
                                    // Calculation for reposition the tooltip when tooltip is exist the container height or width
                                    if ((posx + tooltipWidth + 15) >= containerWidth) {
                                        posx = posx - tooltipWidth - 15;
                                    }

                                    if ((posy + tooltipHeight + 10) >= containerHeight) {
                                        posy = posy - tooltipHeight - 10;
                                    }
                                    $(proxy._mouseOverTooltip).css({ "top": posy, "left": posx });
                                }, 700);
                                    }
                            }

                    }
                }
            }
        }

     }
 },
        _cellMouseLeave: function () {

            var proxy = this;
            if (proxy._mouseOverTooltip != null) {
                $("#tooltipgantt").remove();
                window.clearTimeout(proxy._tooltipTimer);
                $(proxy._mouseOverTooltip).remove();
            }
        },

        // Method to clear the header tooltip.
        _headerMouseLeave: function () {
            var proxy = this;
            if (proxy._headerMouseOverTooltip != null) {
                $("#headertooltip").remove();
                window.clearTimeout(proxy._headerTooltipTimer);
                $(proxy._headerMouseOverTooltip).remove();
            }
        },

        disableTooltip: function () {
            this._cellMouseLeave();
            this._headerMouseLeave();
        },
       
        _createTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                helpers = {},
                template = proxy.model.cellTooltipTemplate,
                parentTr = null;
            helpers["_" + proxy._id + "getTaskName"] = proxy._tooltipTaskName;
            $.views.helpers(helpers);
            var templateString = "";
            if (template) {

                if (document.getElementById(template)) {
                    templateString = $("#" + template)[0].innerHTML;
                } else {
                    templateString = template;
                }
                 parentTr = templateString;
                
            }
            else {
                var parentTr = "<tr class='e-tooltip_rowcell'>";

                td = "{{if ~_" + proxy._id + "getTaskName()}}" +
                    "<td class='e-tooltiptaskname' style='height:auto;width:auto;font-weight:normal;'>{{:ttiptaskname}}</td>{{/if}}";

                parentTr += td;
                parentTr += "</tr>";
            }
            var templates = {};
            templates[proxy._id + "tooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _tooltipTaskName: function () {
            return this.data.ttiptaskname;
        },

        // Method to create cell tooltip template.
        _createColumnTooltipTemplate: function (columnTemplate) {
            var proxy = this, parentTr = null, templateString = "";
            if (columnTemplate) {
                if (document.getElementById(columnTemplate)) {
                    templateString = $("#" + columnTemplate)[0].innerHTML;
                } else if (columnTemplate.slice(0, 1) === "#" && document.getElementById(columnTemplate.slice(1, columnTemplate.length))) {
                    templateString = $(columnTemplate)[0].innerHTML;
                } else {
                    templateString = columnTemplate;
                }
                parentTr = templateString;
            }
            var templates = {};
            templates[proxy._id + "columnTooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        getMaxZIndex: function () {
            var divs = document.getElementsByTagName('div'),
                highest = 0,
                divLength = divs.length;
            if (this._maxZIndex == 0) {
                for (var i = 0; i < divLength; i++) {
                    var zindex = divs[i].style.zIndex,
                    checkIndex = parseInt(zindex);
                    if (checkIndex > highest && zindex != "auto" && !isNaN(checkIndex)) {
                        highest = checkIndex;
                    }
                }
                this._maxZIndex = highest;
            }
            return this._maxZIndex;
        },

        //SCROLL EVENT FOR VIRTUALIZATION
        _onScroll: function (args) {

            var proxy = this,
                model = this.model,
                hScrollDist, vScrollDist;
            proxy._popupHide();
            proxy._clearContextMenu();
         $("#" + this._id + "detailscellwrapper").length > 0 && this._removeDetailsRow();

            if (proxy.model.showGridCellTooltip) {
                proxy._cellMouseLeave();
                proxy._headerMouseLeave();
            }

            if (!model.isFromGantt && proxy._dragTooltip && (!proxy._timerDragDown && !proxy._timerDragUp)) {
                args.cancel = true;
                return;
            }

            if (!proxy.isVScroll() && !proxy.model.isFromGantt && args.source === "wheel") {
                args.cancel = true;
                return;
            }                        

            if (args.scrollLeft != undefined) {
                proxy._scrollLeft = args.scrollLeft;
                hScrollDist = Math.abs(proxy._scrollLeft - proxy._prevScrollLeft);
            }

            if (args.scrollTop != undefined) {
                proxy._scrollTop = args.scrollTop; //Maintain the scroll top for dynamic update of virtualization and non-virtualization mode.
                vScrollDist = Math.abs(proxy._scrollTop - proxy._preVScrollTop);
            }            

            if (hScrollDist) {
                proxy._updateHeaderScrollLeft(proxy._scrollLeft);
                proxy._prevScrollLeft = proxy._scrollLeft;
            }
            
            if (args.scrollTop != undefined) {

                /*Cancel if treegrid is in edit*/
                if (proxy.model.isEdit) {
                    proxy.cancelEditCell();
                    proxy._focusTreeGridElement();
                }
                else if (proxy._isRowEdit) {
                    proxy.cancelRowEditCell();
                    proxy._focusTreeGridElement();
                }

                proxy._vScrollDist = Math.abs(proxy._scrollTop - proxy._prevScrollTop);
                proxy._preVScrollTop = proxy._scrollTop;
                //CHECK VIRTUALIZATION IS ENABLED OR NOT
                if (proxy.model.enableVirtualization) {

                    proxy._vScrollDir = proxy._prevScrollTop <= proxy._scrollTop ? 1 : -1;
                    //console.log("VScrollDist:" + proxy._vScrollDist);
                    if (model.allowPaging && !proxy.model.sizeSettings.height) {
                        return;
                    }else if (proxy._vScrollDist && (proxy._scrollTop != proxy._prevScrollTop)) {
                        proxy._updateCurrentViewData(ej.TreeGrid.Actions.Scroll);
                        proxy._prevScrollTop = proxy._scrollTop;
                    }
                }
                proxy._scrollTop = args.scrollTop;
                if(this._frozenColumnsLength)
                    proxy.getContent().find("#e-frozencontainer" + proxy._id).scrollTop(proxy._scrollTop);
                proxy._vScrollDist = 0;//resetting this property to prevent unwanted rendering 
            }
        },

        editFormValidate: function () {
            if ($.isFunction($.validator)) {
                if (this._frozenColumnsLength > 0) {
                    var forms = this.element.find(".e-treegridform");
                    this.setValidation();
                    if (forms.length > 1) {
                        var form1, form2;
                        form1 = forms.eq(0).validate().form();
                        form2 = forms.eq(1).validate().form();
                        if (!(form1 && form2))
                            return false;
                        else
                            return true;
                    }
                    else if (forms.length > 0)
                        return forms.validate().form();
                    else
                        return true;
                }
                else if ($("#" + this._id + "EditFormDialog").length > 0) {
                    this.setValidation();
                    return $("#" + this._id + "EditFormDialog").validate().form();
                }
                else if ($("#" + this._id + "EditForm").length > 0) {
                    this.setValidation();
                    return $("#" + this._id + "EditForm").validate().form();
                }
                else
                    return true;
            }
            return true;
        },

        //To check previous and current value for row edit
        _checkModfiedCellValue: function (orginalValue, modifiedValue, column) {
            var proxy = this, model = proxy.model, flag = false;
            if ((column["editType"] == "datepicker" || column["editType"] == "datetimepicker")) {
                if (orginalValue instanceof Date && modifiedValue instanceof Date) {
                    if (column["editType"] == "datepicker") {
                        var oldDate = new Date(orginalValue), newDate = new Date(modifiedValue);
                        if (oldDate.setHours(0, 0, 0, 0) != newDate.setHours(0, 0, 0, 0))
                            flag = true;
                    }
                    else {
                        if (orginalValue.getTime() != modifiedValue.getTime()) {
                            flag = true;
                        }
                    }
                }
                else if ((orginalValue !== modifiedValue))
                    flag = true;

            }
            else if (column["editType"] == "dropdownedit") {
                if (orginalValue !== modifiedValue)
                    flag = true;
            }
            else {
                orginalValue = ej.isNullOrUndefined(orginalValue) ? "" : orginalValue;
                modifiedValue = ej.isNullOrUndefined(modifiedValue) ? "" : modifiedValue;
                if (model.isFromGantt && column["field"] == "duration") {
                    var values = modifiedValue.match(/(\d*\.*\d+|[A-z]+)/g);
                    if (values && values.length <= 2)
                        modifiedValue = parseFloat(values[0]);
                }
                if (orginalValue instanceof Array && modifiedValue instanceof Array) {
                    $.each(orginalValue, function (key, value) {
                        var index = $.inArray(value, modifiedValue);
                        if (index == -1) {
                            flag = true;
                        }
                    });
                }
                else if (orginalValue.toString() !== modifiedValue.toString())
                    flag = true;
            }
            return flag;
        },
        
        //Save the edited row
        saveRow: function () {
            var proxy = this,
           model = proxy.model, form, index, isDialogEdit, editType,
           editMode = model.editSettings.editMode.toLowerCase();

            if (proxy.model.editSettings.allowEditing && (editMode == "dialogediting" || (editMode == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog"))) {
                form = $("#" + proxy._id + "EditFormDialog");
                index = proxy._currentEditRecordDialogIndex;
                isDialogEdit = true;
            } else {
                form = $("#" + proxy._id + "EditForm");
                index = this.getIndexByRow(form.closest("tr"));
            }

            if (form) {
                if (!proxy.editFormValidate()) return true;
            }

            var frozenForm = $("#" + proxy._id + "EditFrozenForm"),
                formElement = frozenForm.add(form).find("td"),
                isAddRow = form.closest("tr").hasClass("e-addedrow"),
                columns = model.columns,
                columnLength = columns.length,
                currentItem = model.currentViewData[index],
                value, args = {}, currentValue = ej.copyObject({}, currentItem.item),
                isLocalDataUpdate = true, isValueModified = false;

            proxy._focusTreeGridElement();

            if (form.length <= 0 && index == -1)
                return;

            for (var i = 0; i < columnLength; i++) {
                var column = columns[i], fieldName = column["field"];
                if (fieldName == "" || (proxy._checkboxSelection && column["nonColumn"])) continue;

                if (column.visible || ((model.editSettings.editMode == "dialogEditing" || (proxy.model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode.toLowerCase() == "dialog")) && (!ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) && model.editSettings.dialogEditorTemplateID != ""))) {
                    editType = ej.isNullOrUndefined(column["editType"]) ? "stringedit" : column["editType"];

                    value = proxy.getCurrentEditCellDataForRowEdit(fieldName, editType, i);
                    value = (ej.isNullOrUndefined(value) || (column.editType == ej.TreeGrid.EditingType.Numeric && isNaN(value))) ? null : value;
                    //To avoid duplicate ID for TreeGrid on self reference while row editing
                    if (model.idMapping && model.parentIdMapping && fieldName == model.idMapping) {
                        if (proxy._validateIdValue(value))
                            value = currentItem[fieldName]; //Replacing the old value
                    }
                    if (!proxy._checkModfiedCellValue(ej.getObject(fieldName, currentValue), value, column)) continue;

                    if (fieldName.indexOf(".") != -1)
                        ej.createObject(fieldName, value, currentValue);
                    else
                        currentValue[fieldName] = value;
                    isValueModified = true;
                    if (editMode == "batchediting" && !isAddRow) {
                        //Take backup of original field values
                        proxy._createBatchEditedFields(currentItem, fieldName, value, column);
                        proxy._batchEditStarts = true;
                        proxy._updateBatchChanges(currentItem);
                    }
                }
            }
            if (isAddRow) {
                proxy._isEmptyRow = proxy._checkIsEmptyRow(currentValue);
                args.rowIndex = proxy.selectedRowIndex();
                args.previousValue = currentItem.item;
                args.requestType = "addNewRow";
                args.addedRow = proxy._isEmptyRow ? null : currentValue;
                args.rowElement = form.closest("tr");                
                if (!proxy._isEmptyRow) {
                    if (editMode == "batchediting") {
                        proxy.batchChanges.added.push(currentItem);
                        proxy._batchEditStarts = true;
                    }
                    else {
                        if (proxy._trigger("actionComplete", args)) return true;
                    }
                    ej.copyObject(currentItem, currentValue);
                    ej.copyObject(currentItem.item, currentValue);
                    isLocalDataUpdate = proxy._updateAction(args);
                    if (model.idMapping && model.parentIdMapping)
                        proxy._updateLastInsertedId(currentItem[model.idMapping]);
                }
            }
            else {
                args.requestType = "update";
                args.rowIndex = proxy.getUpdatedRecords().indexOf(currentItem);
                args.currentValue = currentValue;
                args.previousValue = currentItem.item;

                if (isDialogEdit) {
                    args.element = $("#" + proxy._id + "_dialogEdit");
                    if (!proxy._isRemoteData) {
                        isValueModified = true;
                    }
                    if (isValueModified && proxy._trigger("actionBegin", args)) return true;

                    isLocalDataUpdate = proxy._updateAction(args);
                    if (model.idMapping && model.parentIdMapping)
                        proxy._updateLastInsertedId(args.currentValue[model.idMapping]);

                    for (var colIndex = 0 ; colIndex < columns.length ; colIndex++) {
                        if (columns[colIndex]["editType"] == ej.TreeGrid.EditingType.Dropdown || columns[colIndex]["editType"] == ej.TreeGrid.EditingType.DatePicker || columns[colIndex]["editType"] == ej.TreeGrid.EditingType.DateTimePicker) {
                            var $element = $("#" + proxy._id + proxy._getFieldValue(columns[colIndex]["field"]));
                            $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        }
                    }
                }
                else {
                    args.rowElement = form.add(frozenForm).closest("tr");
                    if (isValueModified || !proxy._isRemoteData) {
                        if ((isValueModified && editMode == "batchediting") || (editMode != "batchediting" && !proxy._trigger("endEdit", args))) {
                            isLocalDataUpdate = proxy._updateAction(args);
                            if (model.idMapping && model.parentIdMapping)
                                proxy._updateLastInsertedId(args.currentValue[model.idMapping]);
                        }
                    }
                }
            }              

            if (isLocalDataUpdate)
                proxy._refreshAfterSaveRow(args, true);
        },

        _refreshAfterSaveRow: function (args, isValueModified) {
            var proxy = this, model = proxy.model,
                columns = model.columns,
                currentItem = proxy.getUpdatedRecords()[args.rowIndex],
                index = model.currentViewData.indexOf(currentItem);

            /* delete the row if new added row is not have any values */
            if (proxy._isEmptyRow && args.requestType == "addNewRow") {
                var deleteValueUpdated = false;
                if (!model.editSettings.allowDeleting) {
                    deleteValueUpdated = true;
                    model.editSettings.allowDeleting = true;
                }
                proxy.deleteRow(null, true, null);
                proxy._isEmptyRow = false;
                proxy._isEmptyRowDeleted = true;
                if (deleteValueUpdated) {
                    model.editSettings.allowDeleting = false;
                }
            } else {
                // Save the edited row           
                ej.TreeGrid.refreshRow(proxy, index);
                if (args.requestType == "addNewRow" && currentItem.parentItem && currentItem.parentItem.childRecords.length == 1) {
                    ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(currentItem.parentItem));
                }
                if (model.showSummaryRow) {
                    args.editType = "rowedit";
                    if (args.requestType == "addNewRow" && model.editSettings.rowPosition == "child") {
                        proxy._createAndRenderSummaryRecords(args);
                    }
                    proxy._updateSummaryRow(args);
                }
                if (model.showTotalSummary) {
                    args.editType = "rowedit";
                    proxy._updateTotalSummaryRow(args);
                }
                if (isValueModified && model.editSettings.editMode == "dialogEditing") {
                    //Client side event is triggerd for upadte the server side database
                    var eventArgs = {};
                    eventArgs.data = currentItem
                    eventArgs.item = currentItem.item;
                    eventArgs.requestType = "recordUpdate";
                    proxy._trigger("actionComplete", eventArgs);
                    if (model.idMapping && model.parentIdMapping) {
                        proxy._updateLastInsertedId(eventArgs.item[model.idMapping]);
                    }
                }
            }
            proxy._isRowEdit = false;
            if (model.currentViewData.length > 0 && model.enableAltRow && !model.enableVirtualization)
                ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);
            proxy._cancelSaveTools();
            var colObject = columns.filter(function (column) {
                if (column.filterEditType == "dropdownedit" && column.editType != "dropdownedit")
                    return true;
            });
            if (model.allowFiltering && colObject.length)
                proxy._resizeFilteringElements();
            if (proxy._commandColumnLength > 0) {
                var tr = ej.TreeGrid.getRowByIndex(proxy, proxy.getUpdatedRecords().indexOf(currentItem));
                proxy._refreshUnboundTemplate(tr);
                if (args.requestType == "addNewRow" && model.editSettings.rowPosition == ej.TreeGrid.RowPosition.Child) {
                    var parentTr = ej.TreeGrid.getRowByIndex(proxy, proxy.getUpdatedRecords().indexOf(currentItem.parentItem));
                    proxy._refreshUnboundTemplate(parentTr);
                }
            }
            if (model.allowTextWrap && proxy._$frozenTableContent.length > 0) {
                proxy._updateRowHeight(index);
                proxy._updateGridContentHeight();
                proxy.getScrollElement().ejScroller("refresh");
                var movableScrollTop = proxy.getScrollElement().ejScroller("model.scrollTop");
                proxy._getFrozenScrollElement().scrollTop(movableScrollTop);
                if (proxy.model.enableVirtualization) {
                    proxy.updateHeight();
                }
            } else if (model.allowTextWrap) {
                if (model.allowPaging && !model.sizeSettings.height) {
                    proxy.updateHeight();
                } else {
                    proxy.getScrollElement().ejScroller("refresh");
                }
            }
            if (model.allowTextWrap && proxy._$frozenTableContent.length == 0) {
                var height = proxy._getViewPortHeight();
                proxy._$gridContainer.css({
                    "height": height + "px",
                });
                $("#" + proxy._id + "e-gridcontent").css("height", height)
                proxy.getScrollElement().ejScroller("refresh");
            }
        },

        _updateAction: function (args) {
            var proxy = this, model = proxy.model,
                pkey = model.idMapping,
                index = (args.requestType == "dragAndDrop") ? args.draggedRowIndex : args.rowIndex,
                record = proxy.getUpdatedRecords()[index], isBatchAction = false,
                item, promise, bulkChanges = { added: [], changed: [], deleted: [] };

            if (proxy._isRemoteData) {
                var selectionType = model.selectionSettings.selectionType,
                    batchUrl = proxy._dataManager.dataSource.batchUrl;
                if(!ej.isNullOrUndefined(batchUrl) || selectionType == "multiple" || selectionType == "checkbox")
                    isBatchAction = true;
            }                

            if (args.requestType == "addNewRow") {
                if (proxy._isRemoteData){
                    if(isBatchAction)
                        bulkChanges.added.push(args.addedRow);
                    else
                        promise = proxy._dataManager.insert(args.addedRow, proxy.model.query);
                }
            }
            else if (args.requestType == "update") {
                if (proxy._isRemoteData){
                    if(isBatchAction)
                        bulkChanges.changed.push(args.currentValue);
                    else
                        promise = this._dataManager.update(pkey, args.currentValue, proxy.model.query);
                }                    
                else {
                    ej.copyObject(record, args.currentValue);
                    ej.copyObject(record.item, args.currentValue);
                }
            }
            else if (args.requestType == "dragAndDrop") {
                if (proxy._isRemoteData){
                    if (isBatchAction) {
                        var records = args.draggedRecords;
                        for (var i = 0; i < records.length; i++) {                            
                            bulkChanges.changed.push(records[i].item);
                        }
                    }
                    else
                        promise = this._dataManager.update(pkey, record.item, proxy.model.query);
                }                    
            }
            else if (args.requestType == "saveCellValue") {
                var item = args.data.item;
                if (proxy._isRemoteData && $.isPlainObject(item)) {
                    if(isBatchAction)
                        bulkChanges.changed.push(item);
                    else
                        promise = proxy._dataManager.update(pkey, item, proxy.model.query);
                }
            }
            else if (args.requestType == ej.TreeGrid.Actions.Delete) {
                if (proxy._isRemoteData) {
                    if (isBatchAction)
                        bulkChanges.deleted.push(proxy._getParentChildItem(args.deletedItems));
                    else {
                        var item = args.data.item;
                        promise = proxy._dataManager.remove(pkey, item[pkey], this.model.query);
                    }
                }
            }
            if (isBatchAction) {
                promise = proxy._dataManager.saveChanges(bulkChanges, pkey, this.model.query._fromTable, this.model.query);
            }

            if (promise && $.isFunction(promise.promise) && !proxy._dataManager.dataSource.offline) {
                proxy._waitingPopupObj.show();
                promise.done(function (e) {
                    if (args.requestType == ej.TreeGrid.Actions.Delete) {
                        proxy._updateDeleteAction(args);
                        proxy._cancelSaveTools();
                    }
                    else if (args.requestType == "addNewRow" || args.requestType == "update") {
                        // Replace record if data getting from server-side
                        var item = (args.requestType == "addNewRow") ? args.addedRow : args.currentValue;
                        if (!ej.isNullOrUndefined(e) && !ej.isNullOrUndefined(e.record) && $.isPlainObject(e.record))
                            ej.copyObject(item, e.record);
                        ej.copyObject(record, item);
                        ej.copyObject(record.item, item);
                        proxy._refreshAfterSaveRow(args, true);
                    }
                    else if (args.requestType == "saveCellValue")
                        proxy._refreshAfterSaveCell(args, true);
                    proxy._waitingPopupObj.hide();
                });
                promise.fail(function (e) {
                    args.error = (e && e.error) ? e.error : e;
                    if (args.requestType == "update")
                        proxy._refreshAfterSaveRow(args, true);
                    else if (args.requestType == "addNewRow") {
                        var recordIndex = proxy.getUpdatedRecords().indexOf(record);
                        proxy._isEmptyRow = true;
                        proxy.deleteRow(null, true, recordIndex);
                        proxy._isEmptyRow = false;
                    }
                    else if (args.requestType == "saveCellValue") {
                        args.data[args.columnName] = args.previousValue;
                        args.data.item[args.columnObject.field] = args.previousValue;
                        proxy.selectRows(proxy.selectedRowIndex());
                        ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(args.data));
                    }                    
                    proxy._waitingPopupObj.hide();
                    proxy._trigger("actionFailure", args);
                });
            }
            return !proxy._isRemoteData;
        },
        _getParentChildItem: function (records, items) {
            var proxy = this, length = records.length,
                items = ej.isNullOrUndefined(items) ? [] : items;
            for (var i = 0; i < length; i++) {
                var data = records[i];
                if (data.hasChildRecords)
                    proxy._getParentChildItem(data.childRecords, items);
                else
                    items.push(data.item);
            }
            return items;
        },
        _excludeDetailRows : function()
        {
            var $gridRows = $(), model = this.model;
            if (this.getTreeGridRows() && this.getTreeGridRows().length > 0) {
                if (model.showDetailsRow && model.detailsTemplate) {
                    if (this._frozenColumnsLength > 0)
                        $gridRows = [$(this.getTreeGridRows()[0]).not(".e-detailsrow"), $(this.getTreeGridRows()[1]).not(".e-detailsrow")];
                    else
                        $gridRows = $(this.getTreeGridRows()).not(".e-detailsrow");
                }
                else {
                    if (this._frozenColumnsLength > 0)
                        $gridRows = [$(this.getTreeGridRows()[0]), $(this.getTreeGridRows()[1])];
                    else
                        $gridRows = $(this.getTreeGridRows());
                }
            }
            return $gridRows;
        },

        getIndexByRow: function ($tr) {
            var $gridRows = this._excludeDetailRows(), rowIndex = -1;
            if (this._frozenColumnsLength > 0) {
                rowIndex = $($gridRows[0]).index($tr);
                if (rowIndex == -1)
                    rowIndex = $($gridRows[1]).index($tr);
                return rowIndex;
            }
            return $gridRows.index($tr);
        },
        /*get all treegrid rows with details rows*/
        getTreeGridRows: function () {
            return this._gridRows;
        },

        _keyDown: function (e) {
            var proxy = this;
            if (e.shiftKey && proxy._shiftKeyFirstElementDetails.firstElementRowIndex == -1) {
                proxy._shiftKeyFirstElementDetails.firstElementRowIndex = proxy._focusingRowIndex;
                proxy._shiftKeyFirstElementDetails.firstElementCellIndex = proxy._cellIndex;
            }
            else if (e.which == 13 && this._excelFilter != null && !ej.isNullOrUndefined(this._excelFilter._openedFltr) && this._excelFilter._openedFltr.length && this._excelFilter._openedFltr.is(":visible")) {
                var dlgID = this._id + this._excelFilter._$colType;
                if (this._excelFilter._openedFltr.hasClass("e-dlgcustom"))
                    dlgID += "Custom";
                this._excelFilter._openedFltr.find("#" + dlgID + "_OkBtn").trigger("click");
            }

            else if (e.which === 13 && $(e.target).hasClass("e-searchinput") && $(e.target).closest("#" + proxy._id + "_toolbarItems")) { //13 is key code for Enter Key 
                this._toolbarOperation(this._id + "_search", e.currentTarget);
                e.preventDefault();
                return false;//prevent post back
            }
            else if (e.type == "click" && $(e.target).hasClass("e-searchitem")) {
                this._toolbarOperation(this._id + "_search", $(e.currentTarget).prev()[0].firstChild);
                e.preventDefault();
                return false;
            }
        },

        /*focus treegrid element*/
        _focusTreeGridElement: function () {
            if (ej.browserInfo().name == "msie") {
                try { this.element[0].setActive(); } catch (e) { }
            }
            else {
                var scrollTop = $(window).scrollTop();
                this.element[0].focus();
                $(window).scrollTop(scrollTop);
            }
        },
        _onRecordClick:function(e){
            this._recordClickAction(e,"recordClick");
        },
        _onRecordDoubleClick:function(e){
            this._recordClickAction(e,"recordDoubleClick");
        },
        _recordClickAction: function (e,eventName) {
            var proxy = this,$target = $(e.target), model= proxy.model, column,
            $td = $target.closest("td"), args = {},$tr=$target.closest("tr");
            if (($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell") || ($target.closest("td").hasClass("e-rowcell"))
                || ($target.closest("td").hasClass("e-rowcell")) && $target.hasClass("e-cell")) && $tr.find(".e-editedcell").length == 0 && !$tr.hasClass("e-summaryrow") && !$td.hasClass("e-unboundcell")) {
                args.cell = $td;
                args.cellIndex = proxy.getCellIndex(e);
                args.row = $tr;
                args.rowIndex = proxy.getIndexByRow(args.row);
                args.data = proxy.model.currentViewData[args.rowIndex];
                column = model.columns[args.cellIndex];
                args.columnName = column.headerText;
                args.cellValue = !ej.isNullOrUndefined(args.data) ? args.data[column.field] : "";
                args.targetElement = $target;
                proxy._trigger(eventName, args);
            }
        },
        //add remove animation class for expand/collpase and header icons
        _addRemoveAnimateClass:function(element)
        {
            var proxy = this;
           this._$gridContentTable && this._$gridContentTable.find(".e-treegrid-animate").removeClass("e-treegrid-animate");
            proxy._$gridHeaderContainer && proxy._$gridHeaderContainer.find(".e-treegrid-animate").removeClass("e-treegrid-animate");
            $(element).addClass("e-treegrid-animate");
        },
        //CLICK HANDLER FOR PERFORM ROWSELECTION AND EXPANDCOLLAPSE THE RECORD
        _onClick: function (e) {

            var proxy = this,
                model = proxy.model,
                $target = $(e.target), currentRowIndex, args = {}, rowIndex = -1, expandRecord, id = $target.closest(".e-treegrid").attr("id"),
                updatedRecords = proxy.getUpdatedRecords();
            /* check inner treegrid click events and  */
            if ($target.closest(".e-treegrid").attr("id") !== this._id) return false;
            proxy._clearContextMenu();
            this._removeDetailsRow(e);            
            if (model.allowDragAndDrop)
                window.clearTimeout(proxy._dragTooltipTimer);
            if (model.allowColumnReordering) {
                window.clearTimeout(this._dragColumnTooltipTimer);
                this._dragColumnTooltipTimer = null;
            }
            proxy._cellIndex = proxy.getCellIndex(e);
            if (!$target.hasClass("e-filtertext") && !$target.hasClass("e-cancel"))
                proxy.getHeaderTable().find(".e-cancel").addClass("e-hide");
            if (model.selectionSettings.selectionMode == "cell" && model.editSettings.editMode != "dialogEditing") {
                model.editSettings.editMode = "cellEditing";
            }
            rowIndex = proxy.getRowIndex(e);
            if ((model.editSettings.beginEditAction == "click" || $target.hasClass('e-treegridexpand')
                || $target.hasClass("e-treegridcollapse")) && $target.closest('.e-scrollbar').length == 0 && $target.closest('.e-toolbaricons').length == 0 && $target.closest('.e-treegridtoolbaritem').length == 0) {
                if (model.editSettings.editMode.toLowerCase() == "cellediting" || (model.editSettings.editMode.toLowerCase() == "batchediting") && model.editSettings.batchEditSettings.editMode.toLowerCase() == "cell")
                    proxy._cellEditingDetails.rowIndex = rowIndex;
                expandRecord = model.currentViewData && model.currentViewData[rowIndex];
            }
            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && !proxy.model.isFromGantt && !($target.hasClass("e-downtail"))) {
                if ($target.hasClass("e-rowselect")) {
                    if (!$target.hasClass("e-spanclicked")) {
                        $target.addClass("e-spanclicked");
                    } else {
                        proxy._clearMultiSelectPopup();
                    }
                } else {
                    if (!$("#" + id + "_selectionpopup").find(".e-rowselect").hasClass("e-spanclicked")) {
                        this._multiSelectPopup.hide();
                    }
                }
            }
            proxy._saveCellHandler(e);
            if ($target.hasClass('e-summarytreegridexpand') || $target.hasClass('e-summarytreegridcollapse')) {
                args.expanded = $target.hasClass('e-summarytreegridexpand') ? false : true;
                proxy._isInManualExpandCollapse = true;
                proxy.expandCollapseTotalSummary(args.expanded);
                proxy._isInManualExpandCollapse = false;
                proxy._addRemoveAnimateClass($target);               
                return;
            }
            if ($target.hasClass("e-summaryrowcell") || $target.hasClass("e-footersummaryrowcell")
                || $target.hasClass("e-summarytitle"))
                return;
            if (($target.hasClass('e-treegridexpand') || $target.hasClass("e-treegridcollapse")) && rowIndex != -1  && proxy.editFormValidate()) {
                var $tr = $target.closest('tr'),
                    recordIndex = updatedRecords.indexOf(expandRecord),
                    args = {},
                    isExpandCollapseEnabeled;
                $target.removeClass("e-treegrid-animate");
                args.data = record;
                args.data = expandRecord;
                args.recordIndex = recordIndex;
                args.expanded = $tr.hasClass("e-treegridrowexpand") ? false : true;
                //Set the focus to the element when row or column not in edit mode.
                if ($("#" + proxy._id + "EditForm").length == 0 && ($target.hasClass("e-rowcell") ||  !$target.closest(".e-rowcell").hasClass("e-templatecell"))) {
                   proxy._focusTreeGridElement();
                } else {
                    proxy._cancelEditState();
                }
                if (args.expanded) {

                    isExpandCollapseEnabeled = proxy._trigger("expanding", args);

                } else {

                    isExpandCollapseEnabeled = proxy._trigger("collapsing", args);
                    
                }

                if (!isExpandCollapseEnabeled&&!model.isFromGantt) {

                    /* refresh Added row on expand collapse */
                    if (proxy._isRefreshAddedRecord) {
                        args.data.expanded = args.expanded;
                        proxy.updateExpandStatus(args.data, args.expanded);
                        proxy.refreshContent();
                        proxy.renderRecords();
                        if (model.allowTextWrap && proxy._frozenColumnsLength > 0) {
                            proxy._updateRowHeight();
                            if (model.sizeSettings.height == "auto")
                                proxy._updateContentHeight();
                            proxy.getScrollElement().ejScroller("refresh");
                            proxy._updateScrollCss();
                        }
                        proxy._isRefreshAddedRecord = false;
                    } else {
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    }
                }
                //To set animate class for clicked item
                if (model.allowPaging || model.enableVirtualization) {
                    var newUpdatedRecords = proxy.getUpdatedRecords(),
                        currentRowElement = ej.TreeGrid.getRowByIndex(this,newUpdatedRecords.indexOf(expandRecord));
                    currentRowElement && proxy._addRemoveAnimateClass($(currentRowElement).find(".e-treegridcollapse,.e-treegridexpand"));
                }else
                {
                    proxy._addRemoveAnimateClass($target);
                }
                
            } else if ($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell") || ($target.closest("td").hasClass("e-rowcell")) || $target.parent().hasClass("e-detailsrowcell") || $target.hasClass("e-detailsrowcell")
                || ($target.closest("td").hasClass("e-rowcell")) && $target.hasClass("e-cell")) {
                this._shiftSelectedRecord = null;
                if ($("#" + id + "_selectionpopup").is(":visible")) {
                    if ($("#" + id + "_selectionpopup").find(".e-rowselect").hasClass("e-spanclicked")) {
                        e.ctrlKey = true;
                        if (model.selectionSettings.selectionMode == ej.TreeGrid.SelectionMode.Row)
                        proxy._multiSelectCtrlRequest = true;
                    }
                    else {
                        e.ctrlKey = false; proxy._multiSelectCtrlRequest = false;
                    }
                }
                if (model.isFromGantt && proxy._ganttTouchTrigger) {
                    var args = {
                        target: $target,
                        ctrlKey:e.ctrlKey
                     }
                    proxy._trigger("getCtrlRequestValue", args);
                    e.ctrlKey = args.ctrlKey;
                }
                //Set the focus to the element when row or column not in edit mode.
                if ($("#" + proxy._id + "EditForm").length == 0 && ($target.hasClass("e-rowcell") || !$target.closest(".e-rowcell").hasClass("e-templatecell"))) {
                    proxy._focusTreeGridElement();
                }
                //Check selction is enabled or not
                
                    currentRowIndex = proxy.getIndexByRow($target.closest('tr'));
                    if (model.selectionSettings.selectionMode == ej.TreeGrid.SelectionMode.Row) {

                        if (proxy._checkboxSelection && ($("#" + proxy._id + "EditForm").length == 0 || model.editSettings.beginEditAction == "click") && proxy.editFormValidate()) {
                            var record = proxy.model.currentViewData[currentRowIndex],
                                recordIndex = updatedRecords.indexOf(record), isCheckbox = false;
                            if ($target.hasClass("e-checkbox"))
                                isCheckbox = true;
                            if (recordIndex != -1 && !proxy._rowSelectingEventTrigger(this.selectedRowIndex(), recordIndex, isCheckbox, e.type)) {
                                proxy.selectRows(recordIndex, null, isCheckbox);
                                proxy._cancelSaveTools();
                                proxy._rowSelectedEventTrigger(recordIndex, null, isCheckbox, e.type);
                            }
                        }
                if (model.selectionSettings.selectionType == ej.TreeGrid.SelectionType.Multiple) {

                    proxy._multiSelectCtrlRequest = e.ctrlKey? true: false;

                    if (e.shiftKey && model.allowSelection == true) {
                        currentRowIndex = proxy.getIndexByRow($target.closest('tr'));
                                record = model.currentViewData[currentRowIndex];
                                currentRowIndex = updatedRecords.indexOf(record);
                        proxy._multiSelectShiftRequest = true;
                        if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), currentRowIndex , false , e.type)) {
                                    if (proxy.getRows() && proxy._prevSelectedItem != null) {
                                        if (proxy.getExpandStatus(proxy._prevSelectedItem))
                                            proxy.selectRows(updatedRecords.indexOf(proxy._prevSelectedItem), currentRowIndex);
                                        else
                                            proxy.selectRows(currentRowIndex);
                            }
                                    else if (proxy.getRows() && proxy._prevSelectedItem == null) {
                                        proxy.selectRows(0, currentRowIndex);
                            }
                            proxy._cancelSaveTools();
                            if (model.isFromGantt && e.shiftKey) {
                                proxy._rowSelectedEventTrigger(currentRowIndex, !ej.isNullOrUndefined(proxy._prevSelectedItem) ? model.updatedRecords.indexOf(proxy._prevSelectedItem) : 0);
                            }
                            else {
                                proxy._rowSelectedEventTrigger(currentRowIndex, null, false, e.type);
                            }
                        }
                    }
                }
                if (!proxy._checkboxSelection &&!proxy._multiSelectShiftRequest && $("#" + proxy._id + "EditForm").length == 0 ) {

                    //recordIndex = proxy.getIndexByRow($target.closest('tr'));

                    if (rowIndex != -1) {

                        record = model.currentViewData[rowIndex];
                        currentRowIndex = updatedRecords.indexOf(record);
                       
                        if (model.allowSelection && !proxy._rowSelectingEventTrigger(this.selectedRowIndex(), currentRowIndex, false, e.type)) {
                            proxy.selectRows(currentRowIndex);
                            proxy._cancelSaveTools();
                            proxy._rowSelectedEventTrigger(currentRowIndex, null, false, e.type);
                        }                      
                    }
                }
                proxy._multiSelectShiftRequest = false;
                    }
                    if (model.selectionSettings.selectionMode == ej.TreeGrid.SelectionMode.Cell && $("#" + proxy._id + "EditForm").length == 0 && proxy._cellIndex != -1 &&
                        currentRowIndex != -1 && model.allowSelection) {
                        var frozenColumnRowIndex = proxy._cellIndex >= proxy._frozenColumnsLength ? 1 : 0,
                            targetRow;
                        if (proxy._frozenColumnsLength > 0) {
                            targetRow = proxy.getRows()[frozenColumnRowIndex][currentRowIndex];
                        }
                        else
                            targetRow = proxy.getRows()[currentRowIndex];
                        var columns = model.columns,
                            record = model.currentViewData && model.currentViewData[currentRowIndex],
                            targetIndex = updatedRecords.indexOf(record),
                            targetCellIndex = proxy._cellIndex >= proxy._frozenColumnsLength ? proxy._cellIndex - proxy._frozenColumnsLength : proxy._cellIndex,
                            target = targetRow.childNodes[targetCellIndex],
                            previousRowIndex = proxy._previousSelectedCellDetails.rowIndex,
                            previousCellIndex = proxy._previousSelectedCellDetails.cellIndex,
                            targetCell = $(target).closest("td"),
                            currentCellIndex;
                        proxy._focusingRowIndex = targetIndex;

                        if (currentRowIndex != -1) {

                            if (record.isSummaryRow)
                                return;
                            var args = {
                                rowIndex: targetIndex,
                                cellIndex: proxy._cellIndex,
                                rowElement: targetRow,
                                cellElement: targetCell[0]
                            };
                            
                            var selectedCellInfo = {
                                rowIndex: targetIndex,
                                cellIndex: proxy._cellIndex,
                                cellElement: targetCell[0],                                
                                data: record
                            };

                            if (!proxy._cellSelectingEventTrigger(args) && columns[proxy._cellIndex].allowCellSelection) {
                                if (model.selectionSettings.selectionType == "multiple" && (e.ctrlKey || e.shiftKey)) {
                                    //Select Multiple cells dynamically using Ctrl Key + mouse click.
                                    if (e.ctrlKey) {
                                        if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)
                                            currentCellIndex = proxy._getCellIndex(proxy._selectedCellDetails, "cellElement").indexOf(targetCell[0]);
                                        else
                                            currentCellIndex = proxy._selectedCellDetails.map(function (e) { return e.cellElement; }).indexOf(targetCell[0]);


                                        if (currentCellIndex == -1) {
                                            $(targetCell[0]).addClass("e-selectingcell");
                                                proxy._selectedCellDetails.push(selectedCellInfo);                                            
                                            model.selectedCellIndexes.push({ rowIndex: selectedCellInfo.rowIndex, cellIndex: selectedCellInfo.cellIndex });
                                                proxy._cellSelectedEventTrigger(args, proxy._previousSelectedCellDetails);
                                        }
                                        else {
                                            $(targetCell[0]).removeClass("e-selectingcell");
                                            proxy._selectedCellDetails.splice(currentCellIndex, 1);
                                            model.selectedCellIndexes.splice(currentCellIndex, 1);
                                        }                                       
                                    }
                                        //Select multiple cells using Shift Key + mouse Click.
                                    else if (e.shiftKey) {
                                        var args = {
                                            rowIndex: targetIndex,
                                            cellIndex: proxy._cellIndex,
                                        };
                                        proxy._shiftKeySelectCells(args);
                                        proxy._rowIndexOfLastSelectedCell = targetIndex;
                                        proxy._focusingRowIndex = proxy._shiftKeyFirstElementDetails.firstElementRowIndex;                                     
                                    }
                                }
                                    //Select single cell using mouse click.
                                else  {
                                    proxy._selectedCellDetails = [];
                                    model.selectedCellIndexes = [];
                                    $("#" + proxy._id).find(".e-selectingcell").removeClass("e-selectingcell");
                                    $(targetCell[0]).addClass("e-selectingcell");
                                    proxy._selectedCellDetails.push(selectedCellInfo);
                                    model.selectedCellIndexes.push({ rowIndex: selectedCellInfo.rowIndex, cellIndex: selectedCellInfo.cellIndex });                                   
                                    proxy._cellSelectedEventTrigger(args, proxy._previousSelectedCellDetails);
                                    //Clear the Details of shiftKeyFirstElement.
                                    proxy._shiftKeyFirstElementDetails = {
                                        firstElementRowIndex: -1,
                                        firstElementCellIndex: -1
                                    }
                                    proxy._updateSecondRowIndexBy = 1;
                                }
                                if (!e.shiftKey) {
                                    proxy._previousSelectedCellDetails = {
                                        rowElement: targetRow,
                                        cellElement: targetCell[0],
                                        cellIndex: args.cellIndex,
                                        rowIndex: targetIndex,
                                    };
                                    proxy._rowIndexOfLastSelectedCell = proxy._focusingRowIndex;
                                    proxy._isShiftKeyNavigation = false;
                                }
                            }
                            if (!e.shiftKey)
                                proxy._rowIndexOfLastSelectedCell = proxy._focusingRowIndex;
                            proxy._cancelSaveTools();
                            //Get back default values to perform keypressed event.                            
                            proxy._lastSelectedCellIndex = proxy._cellIndex;
                        }
                    }
                
                    if (proxy._isTouchTrigger && proxy.model.allowSelection && this.model.selectionSettings.selectionType == "multiple" && !proxy.model.isFromGantt) {
                        if (proxy._isTouchTrigger && this._multiSelectPopup != null && !this._multiSelectPopup.is(":visible") && this._multiSelectPopup.find(".e-rowselect").hasClass("e-spanclicked"))
                            this._multiSelectPopup.show();
                        if (proxy._isTouchTrigger && this._multiSelectPopup != null && !this._multiSelectPopup.find(".e-rowselect").hasClass("e-spanclicked")) {
                            if (!$target.hasClass("e-detailsrowcell") && !$target.parent("td").hasClass("e-detailsrowcell") && !$target.closest("tr").hasClass('e-detailsrow')) {
                                this._multiSelectPopup.removeAttr("style");
                                var offset = $target.closest(".e-rowcell").offset();
                                this._multiSelectPopup.offset({ top: 0, left: 0 }).offset({ left: (offset.left + (($target.closest(".e-rowcell").outerWidth() / 2) - (this._multiSelectPopup.width() / 2))), top: (offset.top - this._multiSelectPopup.outerHeight() - 10) }).show();
                            }
                        }
                        proxy._isTouchTrigger = false;
                    }
            }

            /* detail row show and hide */
            if (($target.hasClass("e-detailsrowcell") || $target.parent("td").hasClass("e-detailsrowcell")) && !$target.closest("tr").hasClass('e-detailsrow')) {
                if (!e.ctrlKey && !e.shiftKey) {
                    var editState=$("#" + this._id + "EditForm").length>0 ? this.editFormValidate()?true:false:true;                  
                    if (editState) {
                        if (proxy._isRowEdit) {
                            this.saveRow();
                        } else {
                            proxy._detailsExpandCollapse($target);
                        }
                    }
                }
            }

            if (proxy._isRowEdit)
                $target.focus();
            if (!$target.closest("#" + proxy._id + "_ColumnMenu").hasClass("e-columnmenu") && $target.closest("div.e-excelfilter").length == 0 && !$target.hasClass("e-overlay"))
            {
                proxy._clearColumnMenu();
            }
            if (model.editSettings.beginEditAction == "click" && $target.closest("#" + proxy._id + "EditForm").length == 0 &&
                !proxy._isShiftKeyNavigation && !proxy._isEmptyRowDeleted)
                proxy._editdblClickHandler(e);
            proxy._isEmptyRowDeleted = false;
        },
        //Check TreeGrid is in filtered state or not
        _isInFilteredState: function () {
            return ((this.model.allowFiltering && this.model.filterSettings.filteredColumns.length > 0) || this._searchString.length > 0);
        },
		// Method to check/uncheck all the records
        selectAllRows: function (checkAll) {
            var proxy = this, model = proxy.model, index = -1,
                data = proxy._isInFilteredState() ? proxy._flatFilteredRecords : model.flatRecords,
                updatedRecords = proxy.getUpdatedRecords(),
                length = 0;
            if (!model.selectionSettings.enableSelectAll && !proxy._isFromSetmodel)
                return;

            if (model.showSummaryRow)
                data = proxy._spliceSummaryRows(data);
            if (!ej.isNullOrUndefined(checkAll)) {                
                if (!checkAll)
                    data = proxy._isInFilteredState() ? proxy._flatFilteredRecords : $.extend([], model.selectedItems);

                for (var i = 0; i < data.length; i++) {
                    if (checkAll) {
                        if (data[i].checkboxState == "checked") continue;
                        data[i].checkboxState = "checked";
                        model.selectedItems.push(data[i]);
                        $('#' + this._id + "_checkbox" + data[i].ejRowId).removeClass("e-stop").addClass("e-checkmark");
                        $('#' + this._id + "_checkbox" + data[i].ejRowId).parent("span").removeAttr("class").addClass("e-chk-act");
                    }
                    else {                                             
                        index = model.selectedItems.indexOf(data[i]);
                        if (index > -1) {
                            data[i].checkboxState = "unchecked";
                            model.selectedItems.splice(index, 1);
                            $('#' + this._id + "_checkbox" + data[i].ejRowId).removeClass("e-checkmark");
                            $('#' + this._id + "_checkbox" + data[i].ejRowId).parent("span").removeAttr("class").addClass("e-chk-inact");
                            if (model.selectionSettings.enableHierarchySelection || proxy._isFromSetmodel)
                                proxy._updateParentSelection(data[i]);
                        }
                    }
                }                
                proxy._cancelSaveTools();
            }
            if (proxy._isInFilteredState() && proxy._flatFilteredRecords.length > 0 && model.selectedItems.length > 0) {
                var selectedFilteredRecords = proxy._flatFilteredRecords.filter(function (data) {
                    return data.checkboxState == "checked";
                });
                length = selectedFilteredRecords.length;
            }
            else
                length = model.selectedItems.length;
            if (length > 0 && data.length > 0) {
                if (length != data.length) {
                    $('#' + this._id + "_headerCheckbox").removeClass("e-checkmark").addClass("e-stop");
                    $('#' + this._id + "_headerCheckbox").parent("span").removeAttr("class").addClass("e-chk-indeter");
                }
                else {
                    $('#' + this._id + "_headerCheckbox").removeClass("e-stop").addClass("e-checkmark");
                    $('#' + this._id + "_headerCheckbox").parent("span").removeAttr("class").addClass("e-chk-act");
                }
            }
            else {
                $('#' + this._id + "_headerCheckbox").removeClass("e-stop e-checkmark");
                $('#' + this._id + "_headerCheckbox").parent("span").removeAttr("class").addClass("e-chk-inact");
            }
        },

        //Add/Remove records in selectedItems collection
        _updateSelectedItems: function (currentRecord, checkboxState) {
            var proxy = this, model = proxy.model, length = 0,
                selectedRowIndex = -1, recordIndex = -1,
                updatedRecords = proxy.getUpdatedRecords();
            recordIndex = updatedRecords.indexOf(currentRecord);
            currentRecord.checkboxState = checkboxState;
            selectedRowIndex = $.inArray(currentRecord, model.selectedItems);
            if (currentRecord.checkboxState == "unchecked") {
                selectedRowIndex != -1 && model.selectedItems.splice(selectedRowIndex, 1);
                if (recordIndex > -1) {
                    $('#' + this._id + "_checkbox" + currentRecord.ejRowId).removeClass("e-checkmark e-stop");
                    $('#' + this._id + "_checkbox" + currentRecord.ejRowId).parent("span").removeAttr("class").addClass("e-chk-inact");
                }
            }
            else if (currentRecord.checkboxState == "checked") {
                selectedRowIndex == -1 && model.selectedItems.push(currentRecord);
                if (recordIndex > -1) {
                    $('#' + this._id + "_checkbox" + currentRecord.ejRowId).removeClass("e-stop").addClass("e-checkmark");
                    $('#' + this._id + "_checkbox" + currentRecord.ejRowId).parent("span").removeAttr("class").addClass("e-chk-act");
                }
            }            
        },
        // Check/Uncheck parents & Children records Hierarchy selection
        _traverseSelection: function (record, checkboxState, ischildItem) {
            var proxy = this, length = 0, childRecords, updateParent = false, model = this.model;

                proxy._updateSelectedItems(record, checkboxState);                            

            if (!ischildItem && record.parentItem && model.selectionSettings.enableHierarchySelection) 
                proxy._updateParentSelection(record.parentItem);

            if (record.hasChildRecords && model.selectionSettings.enableHierarchySelection) {
                childRecords = record.childRecords;
                if (proxy._flatFilteredRecords.length > 0) {
                    var filteredChildRecords = childRecords.filter(function (record) {
                        return proxy._flatFilteredRecords.indexOf(record) > -1;
                    });
                    childRecords = filteredChildRecords;
                }
                length = childRecords.length;
                for (var count = 0; count < length; count++) {
                    if (childRecords[count].hasChildRecords)
                        proxy._traverseSelection(childRecords[count], checkboxState, true);
                    else if (!childRecords[count]['isSummaryRow']) {                        
                        proxy._updateSelectedItems(childRecords[count], checkboxState);
                    }
                }
            }
        },

        // Update parent checkbox state by its children checkbox state
        _updateParentSelection: function (parentRecord) {

            var proxy = this, length = 0, isChanged = false,
                model = this.model,
            childRecords = [];
            if (parentRecord && parentRecord.childRecords)
                childRecords = proxy._spliceSummaryRows(parentRecord.childRecords);
            if (proxy._flatFilteredRecords.length > 0 && model.selectionSettings.enableHierarchySelection) {
                var filteredChildRecords = childRecords.filter(function (record) {
                    return proxy._flatFilteredRecords.indexOf(record) > -1;
                });
                childRecords = filteredChildRecords;
            }
            length = childRecords && childRecords.length;
            var callMe = false;

            var indeter = childRecords.filter(function (record) {
                return record.checkboxState == "indeterminate";
            });
            var checkChildRecords = childRecords.filter(function (record) {
                return record.checkboxState == "checked";
            });

            if (indeter.length > 0 || (checkChildRecords.length > 0 && checkChildRecords.length != length)) {
                if (parentRecord.checkboxState == "checked") {
                    this._updateSelectedItems(parentRecord, "unchecked");
                    callMe = true;
                }
                else if (parentRecord.checkboxState == "unchecked") callMe = true;

                if (callMe) {
                    $('#' + this._id + "_checkbox" + parentRecord.ejRowId).addClass("e-stop");
                    $('#' + this._id + "_checkbox" + parentRecord.ejRowId).parent("span").removeAttr("class").addClass("e-chk-indeter");
                    parentRecord.checkboxState = "indeterminate";
                    if (parentRecord.parentItem) this._updateParentSelection(parentRecord.parentItem);
                }
            }
            else if (checkChildRecords.length == 0 && indeter.length == 0) {

                $('#' + this._id + "_checkbox" + parentRecord.ejRowId).removeClass("e-stop e-checkmark").parent("span").removeClass("e-chk-indeter");
                if (parentRecord.checkboxState == "checked")
                    this._updateSelectedItems(parentRecord, "unchecked");                
                else
                    parentRecord.checkboxState = "unchecked";
                parentRecord.parentItem && this._updateParentSelection(parentRecord.parentItem);
            }
            else {
                parentRecord.checkboxState = "checked";
                this._updateSelectedItems(parentRecord, "checked");
                if (parentRecord.parentItem) this._updateParentSelection(parentRecord.parentItem);
            }
        },

         /* remove child detail row wraper when detail row contains another treegrid*/
        _removeInnerDetailRows: function ($target, eventCheck)
        {
            /* check if target is in any of detail rows */
            var detailRows = $(".e-detailscellwrapperfly"),
                detailRowsLength = detailRows.length, count = 0, containsFlag = false;

            for (count; count < detailRowsLength; count++) {
                if ($.contains(detailRows[count], $target[0]) || $target == detailRows[count]) {
                    containsFlag = true;
                    break;
                }
            }

            var countFlag = false;
            if (ej.isNullOrUndefined(detailRows[count])) {
                count = detailRows.index($("#" + this._id + "detailscellwrapper"));
                countFlag = true;
            }
            var countFlagString = countFlag ? "" : ",";
            var parentTreeGridId = $(detailRows[count]).attr("parentTreeGridIds");
            var parentDetailWrapperElement = $("[parentTreeGridIds$='" + countFlagString + parentTreeGridId + "']"),
                length = parentDetailWrapperElement.length, i = 0;
            this._triggerDetailRowHideEvent(parentDetailWrapperElement, eventCheck);
        },

        /* trigger detail row collaspse event */
        _triggerDetailRowHideEvent: function (elements, eventCheck)
        {
            var proxy = this,
                model = this.model;
            var length = elements.length, i = 0;

            if (length > 0 && $("#" + this._id + "detailscellwrapper").length > 0)
            {
                var detailElement = $("#" + this._id + "detailscellwrapper"),
                    selectedItem = model.selectedItem;
                var detailsCollapseEventArgs = {};
                detailsCollapseEventArgs.rowData = selectedItem;

                if (!this._trigger("detailsHidden", detailsCollapseEventArgs) || eventCheck) {
                    for (i; i < length; i++) {
                        var treeGridContainerId = $(elements[i]).attr('id').replace("detailscellwrapper", "");
                       
                            $("#" + treeGridContainerId).find(".e-detailsrowexpand").removeClass("e-detailsrowexpand").addClass("e-detailsrowcollapse");
                            $("#" + treeGridContainerId).find(".e-detailsinfoiconhide").removeClass("e-detailsinfoiconhide").addClass("e-detailsinfoiconshow");
                        
                            $(elements[i]).remove();
                    }
                }
            }
        },
        /* remove all child detail row */
        _removeDetailsRow: function (e, eventCheck)
        {
            var model= this.model;
            
                if (e) {
                    var target = $(e.target);
                    var detailCellElement = target;
                    if (target.parent("td").hasClass("e-detailsrowcell"))
                        detailCellElement = target.parent("td");

                    if (target.closest(".e-popup").length == 0 && target[0] != $('html').get(0)) {

                        /* check if target is in any of detail rows */
                        var detailRows = $(".e-detailscellwrapperfly"),
                            detailRowsLength = detailRows.length, count = 0, containsFlag = false;

                        for (count; count < detailRowsLength; count++) {
                            if ($.contains(detailRows[count], target[0]) || target[0] == detailRows[count]) {
                                containsFlag = true;
                                break;
                            }
                        }

                        if (!target.hasClass("e-detailscellwrapperfly") && !detailCellElement.hasClass("e-detailsrowexpand") && !detailCellElement.hasClass("e-detailsrowcollapse")
                            && !containsFlag) {
                            this._removeInnerDetailRows($("#" + this._id + "detailscellwrapper"), eventCheck);
                        }

                        if (target.hasClass("e-detailscellwrapperfly") || detailCellElement.hasClass("e-detailsrowcollapse") || (!detailCellElement.hasClass("e-detailsrowexpand") && containsFlag)) {
                            var countFlag = false;
                            if (ej.isNullOrUndefined(detailRows[count])) {
                                count = detailRows.index($("#" + this._id + "detailscellwrapper"));
                                countFlag = true;
                            }
                            var countFlagString = countFlag ? "" : ",";
                            var parentTreeGridId = $(detailRows[count]).attr("parentTreeGridIds");
                            var parentDetailWrapperElement = $("[parentTreeGridIds$='" + countFlagString + parentTreeGridId + "']");
                            this._triggerDetailRowHideEvent(parentDetailWrapperElement, eventCheck);
                        }
                    }
                } else {
                    $("#" + this._id + "detailscellwrapper").length > 0 && this._removeInnerDetailRows($("#" + this._id + "detailscellwrapper")[0], eventCheck);
                    this._triggerDetailRowHideEvent($("#" + this._id + "detailscellwrapper"), eventCheck);
                }
            
        },

        /* show hide detail rows public method*/
        
        showHideDetailsRow:function(rowIndex)
        {
            var proxy = this, model = this.model;
            /* save if tree grid is in edit state */
            proxy._saveEditedElements();

            if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn && !ej.isNullOrUndefined(rowIndex)
                 && rowIndex <= model.updatedRecords.length && proxy.getExpandStatus(model.updatedRecords[rowIndex])) {
                if (rowIndex != this.selectedRowIndex()) {
                    if (rowIndex != -1 && $.inArray(rowIndex, proxy._selectedRowsIndexes == -1) && model.allowSelection && model.updatedRecords.length > rowIndex) {
                        if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), rowIndex)) {
                            proxy.selectRows(rowIndex);
                            proxy._rowSelectedEventTrigger(rowIndex);
                        }
                    }
                }

                this.updateScrollBar(rowIndex);
                var targetRowElement = $(ej.TreeGrid.getRowByIndex(proxy, rowIndex)), targetCellElement;
                if (targetRowElement.length > 0 && model.updatedRecords[rowIndex] && proxy.getExpandStatus(model.updatedRecords[rowIndex]))
                {
                    targetCellElement = $(targetRowElement).find(".e-detailsrowcell");
                    if ($(targetCellElement).length > 0 && $(targetCellElement).hasClass("e-detailsrowcell"))
                    {
                        if (this.element.find(".e-detailsrowexpand")[0] != targetCellElement[0])
                            this._removeDetailsRow();

                        this._detailsExpandCollapse($(targetCellElement));
                    }
                }
            }
        },
        /* expand collapse the detail row */
        _detailsExpandCollapse:function($target)
        {
            var proxy = this,
                model = this.model;
            
            if ($target.parent("td").hasClass("e-detailsrowcell"))
                $target = $target.parent("td");
            var rowElement = $target.closest('tr'),
                detailsIconElement = rowElement.find(".e-detailsrowcell");

            if (detailsIconElement.hasClass("e-detailsrowcollapse")) {

                var rowIndexValue = this.getIndexByRow(rowElement),
                   rowData = this.model.currentViewData[rowIndexValue];
                if (ej.isNullOrUndefined(rowData) || rowData.isSummaryRow)
                    return;
                var rowBottom = this.getOffsetRect(rowElement[0]),
                   detailtr = $($.render[proxy._id + "_detailRowTemplate"](rowData)),
                   detailsCellElement = $(detailtr).find(".e-detailscellwrapper").attr({ "id": this._id + "detailscellwrapper" }),
                   width = proxy._gridWidth - (proxy.element.find(".e-scrollcss").length > 0 ? 18 : 0) - 2;//2 - treegrid border

                /* add treegrid attribute for detail row wraaper element */
                var rowWrapperElement = this.element.parents().filter(".e-detailscellwrapperfly"),
                    wrapperElementID = this._id, tempTreeGridId = "";
                while (rowWrapperElement.length > 0) {
                    tempTreeGridId = rowWrapperElement.attr("id").replace("detailscellwrapper", "");
                    wrapperElementID += "," + tempTreeGridId;
                    rowWrapperElement = $("#" + tempTreeGridId).parents().filter(".e-detailscellwrapperfly");
                }

                detailsCellElement.attr("parentTreeGridIds", wrapperElementID);
                
                //Edge detection for detail rows
                var treeGridContentOffset = this.getOffsetRect(proxy._$gridContent[0]), topPosition, bottomPosition;
                treeGridContentOffset.bottom = treeGridContentOffset.top + proxy._$gridContent[0].offsetHeight;
                topPosition = rowBottom.top + rowElement[0].offsetHeight;
                bottomPosition = topPosition + model.detailsRowHeight;
                if (topPosition > treeGridContentOffset.bottom || bottomPosition > treeGridContentOffset.bottom)
                {
                    var tempTopPosition = rowBottom.top - model.detailsRowHeight;
                    if (tempTopPosition > treeGridContentOffset.top)
                        topPosition = tempTopPosition;
                }

                detailsCellElement.removeClass("e-detailscellwrapper").addClass("e-detailscellwrapperfly");
                model.cssClass && detailsCellElement.addClass(model.cssClass);
                var scrollerLeft = proxy.getScrollElement().ejScroller("isHScroll") ? proxy.getScrollElement().ejScroller("option", "scrollLeft") : 0;
                var maxScrollWidth = proxy.getMaxScrollWidth();
                if (maxScrollWidth > 0 && scrollerLeft > maxScrollWidth)
                    scrollerLeft = maxScrollWidth;
                var frozenTableWidth = 0;
                if (this._frozenColumnsLength > 0) {
                    frozenTableWidth = $(this.element).find("#e-frozencontentdiv" + this._id).width();
                }
                detailsCellElement.css({
                    "top": topPosition, "left": rowBottom.left + scrollerLeft - frozenTableWidth,
                    "z-index": proxy.getMaxZIndex() + 1, position: "absolute", width: width
                }).appendTo(this.element);
                this._trigger("refresh");
                var eventArgs = {};
                eventArgs.detailsElement = detailsCellElement;
                eventArgs.data = rowData;
                eventArgs.rowIndex = this.selectedRowIndex();
                this._trigger("detailsDataBound", eventArgs);

                /* details show/expand event trigger*/
                var expandEventArgs = {};
                expandEventArgs.detailsElement = detailsCellElement;
                expandEventArgs.data = rowData;
                expandEventArgs.rowIndex = this.selectedRowIndex();
                expandEventArgs.appendTarget = null;
                if (this._trigger("detailsShown", expandEventArgs)) {
                    this._removeInnerDetailRows($target);
                }else
                {
                    detailsIconElement.find(".e-detailsinfoiconshow").removeClass("e-detailsinfoiconshow").addClass("e-detailsinfoiconhide");
                    detailsIconElement.removeClass("e-detailsrowcollapse").addClass("e-detailsrowexpand");
                    if (expandEventArgs.appendTarget)
                    {
                        $(detailsCellElement).css({ "top": "auto", left: "auto", position: "auto" });
                        $(detailsCellElement).detach().appendTo($("#" + expandEventArgs.appendTarget));
                    }

                }
            }else
            {
                this._removeInnerDetailRows($target);
            }
        },

        //CLICK HANDLER FOR PERFORM THE SORTING ACTION
        _onHeaderClick: function (e) {

            var proxy = this,
                model = proxy.model;
            proxy._cellMouseLeave();
            proxy._headerMouseLeave();
            proxy._clearContextMenu();
            this._removeDetailsRow();
            if ($(e.target).is(".e-ascending, .e-descending")) e.target = e.target.parentNode;

            var $target = $(e.target),
                columnFieldName,
                columnSortDirection,
                column, canSort = true;

            proxy.getHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive");

            if (proxy._mouseDragged && model.allowColumnReordering) canSort = false;

            if (proxy._checkboxSelection && model.allowSelection && model.selectionSettings.enableSelectAll && $target.hasClass("e-headerCheckbox")) {
                proxy.selectAllRows(!$target.hasClass("e-checkmark") && !$target.hasClass("e-stop"));
                canSort = false;
            }

            if ($target.closest(".e-headercelldiv") && !$target.hasClass("e-columnmenu-icon") && !$target.hasClass("e-column-filter-icon") && !$target.hasClass("e-column-filtered-icon")) {
                columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname");
                if (!canSort || (ej.isNullOrUndefined(columnFieldName) && ej.isNullOrUndefined(this.getColumnByField(columnFieldName)) || this.getColumnByField(columnFieldName).allowSorting == false)) {

                    proxy._saveEditedElements();
                }
                else if (canSort && model.allowSorting) {
                    column = this.getColumnByField(columnFieldName);
                    if ($target.closest(".e-headercell").find('span').hasClass("e-ascending")) {
                        columnSortDirection = ej.sortOrder.Descending;
                    } else {
                        columnSortDirection = ej.sortOrder.Ascending;
                    }

                    if (model.allowMultiSorting && e.ctrlKey) {
                        proxy._multiSortRequest = true;
                    }
                    proxy._isRefreshAddedRecord = false;
                    if(model.isFromGantt)
                        proxy.sortColumn(column.mappingName ? column.mappingName : column.field, columnSortDirection);
                    else
                        proxy.sortColumn(column.field, columnSortDirection);
                    if (proxy.model.isFromGantt && this.selectedRowIndex() >= 0) {
                        var args = {};
                        args.data = proxy.model.selectedItem;
                        args.target = "ejTreeGrid";
                        args.recordIndex = this.selectedRowIndex();
                        proxy._trigger("rowSelected", args);
                    }
                }
            }            
                                            

            // Show the column menu / filter menu while clicking the column menu icon / filter menu icon

            if ($target.hasClass("e-column-filter-icon") || $target.hasClass("e-column-filtered-icon"))
            {
                if (proxy._batchEditStarts) {
                    proxy._batchSaveLostWarning("Filtering");
                    proxy._batchTempDetails.filterTarget = e;                    
                    return false;
                }
                columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname");
                column = proxy.getColumnByField(columnFieldName);
                proxy._checkFilterType(column);
                if (!$target.data("isClicked")) {
                    if ((model.filterSettings.filterType == "menu" && column.filterType != "excel") || (model.filterSettings.filterType == "excel" && column.filterType == "menu")) {
                        proxy._updateFilterMenuList(e);
                        $(".e-filtericon").data("isClicked", false);
                        $target.data("isClicked", true);
                    }
                    else {
                        var targetWidth = $target[0].offsetWidth,
                            targetHeight = $target[0].offsetHeight,
                            position = proxy.getOffsetRect($target[0]);

                        position.top = position.top + targetHeight;
                        position.left = position.left + targetWidth;

                        var containerOffset = proxy.getOffsetRect(proxy.element[0]),
                        rightPosWidth = containerOffset.left + proxy.element.width(),
                        xPos = position.left, yPos = position.top,
                        colType = column.filterEditType ? column.filterEditType : column.editType;
                        colType = colType.toLowerCase();

                        switch (colType) {
                            case "numericedit":
                                colType = "number";
                                break
                            case "stringedit":
                            case "maskedit":
                            case "dropdownedit":
                                colType = "string";
                                break;
                            case "booleanedit":
                                colType = "boolean";
                                break;
                            case "datepicker":
                            case "datetimepicker":
                                colType = colType.replace("picker", "");
                                break

                        }
                        if (model.filterSettings.filterType == "excel") {
                            proxy._saveEditedElements();
                        }
                        //alert($(window).width());
                        if ($(window).width() <= proxy._responsiveMinWidth && model.isResponsive && model.enableResize) {
                            proxy._renderResponsiveExcelFilter(column, colType);
                            $(".e-filtericon").data("isClicked", false);
                            proxy._mediaStatus = true;
                        }
                        else {
                            if ($.inArray(colType, proxy._excelColTypes) == -1) {    
                                proxy._clearFilterMenu();
                                $("#" + proxy._id + colType + "_excelDlg").remove();
                                $("#" + proxy._id + colType + "_CustomFDlg_wrapper").remove();
                                $("#" + proxy._id + colType + "_CustomFDlg").remove();
                                proxy._excelFilter._openedFltr = null;
                                proxy._excelColTypes.push(colType);
                                proxy._excelFilter.renderDialog(colType);
                            }
                            var localXFLabel = { True: proxy.localizedLabels.trueText, False: proxy.localizedLabels.falseText },
                                records = [];
                            $.grep(proxy.model.flatRecords, function (record) {
                                if (!record.isSummaryRow)
                                    records.push(record.item);
                            });
                            var dlgWidth = $("#" + proxy._id + colType + "_excelDlg").width();
                            if (rightPosWidth < xPos + dlgWidth)
                                xPos = xPos - dlgWidth - targetWidth;

                            proxy._excelFilter.openXFDialog({ field: column.field, enableResponsiveRow: true, displayName: column.headerText, dataSource: records, query: null, position: { X: xPos, Y: yPos }, dimension: { height: $(window).height(), width: $(window).width() }, cssClass: "resFilter", type: colType, format: column.format, localizedStrings: localXFLabel });
                            $(".e-filtericon").data("isClicked", false);
                            $target.data("isClicked", true);
                            proxy._customFilter = null;
                        }
                        proxy._filterIconTarget = e;
                    }
                }
                else {
                    proxy._clearColumnMenu();
                    $target.data("isClicked", false);
                }
                this._addRemoveAnimateClass($target);
            }
            else if ($target.hasClass("e-columnmenu-icon")) {
                if (!$target.data("isClicked")) {
                    proxy._renderColumnMenu(e);
                    $(".e-columnicon").data("isClicked", false);
                    $target.data("isClicked", true);
                }
                else{
                    proxy._clearColumnMenu();
                    $target.data("isClicked", false);
                }
                this._addRemoveAnimateClass($target);
            }
            else
                proxy._clearColumnMenu();
            if ($target.hasClass("e-checkbox"))
                return true;
            else if ($target.hasClass("e-columnmenu-icon") || $target.hasClass("e-column-filter-icon") || $target.hasClass("e-column-filtered-icon"))
                return false;
        },

        //Render the excel filter with responsive mode
        _renderResponsiveExcelFilter:function(column,colType)
        {
            var proxy = this,
               colIndex = $.inArray(colType, proxy._excelColTypes),
               windowWidth = $(window).width(),
               windowHeight = $(window).height();
            if (colIndex != -1) {
                $("#" + proxy._id + colType + "_excelDlg").remove();
                proxy._excelColTypes.splice(colIndex, 1);
                proxy._excelFilter._openedFltr = null;
            }
            proxy._excelFilter.renderDialog(colType);
            //Modify the excel filter UI for responsive mode
            var $headerDiv = ej.buildTag('div.e-treegird-filterDialogHeaderDiv', "", {'width':'100%'}),
                $titleSapn = ej.buildTag('div.e-treegrid-label', '<span>' + proxy._columnMenuTexts["menuFilter"] + '</span>', { 'padding-top': '10px', 'font-weight': 'bold', 'font-family': 'Segoe UI, Helvetica Neue, Ubuntu, Arial' });
            if ($(".e-filterMenuBtn").length > 0)
                $(".e-filterMenuBtn").remove();
            var $dlgBtn = ej.buildTag('div.e-filterMenuBtn'),
                $inputOk = ej.buildTag('input.e-responsiveFilterBtnLeft e-flat e-btnsub', "", {'margin-left':'15px'}),
                $inputCancel = ej.buildTag('input.e-responsiveFilterBtnRight e-flat e-btncan', "", {'margin-right':'12px'}),
                $id;
            $headerDiv.append($titleSapn);
            proxy._excelDlg = $id = "#" + proxy._id + colType + "_excelDlg";
                $inputOk.ejButton({
                    text: proxy.localizedLabels.okButtonText, type: 'button', click: function (sender) {
                        proxy._excelFilter._fltrBtnHandler();
                        $($id).css('display', 'none');
                    }
                });
                $inputCancel.ejButton({
                    text: proxy.localizedLabels.cancelButtonText, type: 'button',
                    click: function () {
                        $($id).css('display', 'none');
                        proxy.element.css('display', 'block')
                    }
                });
                $($id).children().not('.e-searchcontainer').remove();
                var excelObj = proxy._excelFilter,
                    $backIcon = ej.buildTag('div.e-treegrid-resFilterleftIcon', '', {'padding-top':'10px'}, { closeDialogue: $id.slice(1), openDialogue: 'responsiveFilter' }),
                    $spanIcon = ej.buildTag('span.e-icon e-treegrid-resIcon e-treegrid-responsiveBack', '', {}, { closeDialogue: $id.slice(1), openDialogue: 'responsiveFilter' });
                $backIcon.click(function (e) {
                    $.proxy(proxy._closeDivIcon(e), this);
                })
                $headerDiv.append($backIcon.append($spanIcon));
                var $closeIcon = ej.buildTag('div.e-treegrid-resFilterRigthIcon', '', {'padding-top':'10px'}, { closeDialogue: $id.slice(1), gridEle: true }),
                    $closeSpan = ej.buildTag('span.e-icon e-treegrid-resIcon e-responsiveCustomFilter', '', {}, { closeDialogue: $id.slice(1), gridEle: true });
                if (colType != 'boolean')
                    $headerDiv.append($closeIcon.append($closeSpan));
                var $searchContainer = $($id).css({'height': windowHeight - 20,'padding': '0px','width': "99.7%"}),
                    $searchBox = $searchContainer.find('.e-searchcontainer .e-searchbox').css('margin-top', '10px');
                $searchBox.children().css('margin-top', '10px');
                var $checkBoxDiv = $searchContainer.find('.e-checkboxlist'),
                    scrollHeight = windowHeight * (65 / 100);
                proxy._checkBoxDiv = $checkBoxDiv;
                $($searchContainer.find('.e-searchcontainer')).addClass('e-treegrid-resultSearch');
                var $btn = $searchContainer.find('.e-treegrid-resultSearch .e-btncontainer').remove();
                $inputOk.addClass('e-treegrid-responsiveFilterBtnLeft').css({ 'width': '43.6%' });
                $inputCancel.addClass('e-treegrid-responsiveFilterBtnRight').css('width', '43.6%');
                $searchContainer.find('.e-excelLabel').remove();
                var $labelDiv = ej.buildTag('div.e-excelLabel', column.headerText, { 'margin-top': '10px' }),
                    $searchBox = $searchContainer.find('.e-searchcontainer');
                $labelDiv.insertBefore($searchBox.find('.e-searchbox'));
                $searchContainer.prepend($headerDiv);
                $searchContainer.find(".e-searchbox span").css("top", "23px");
                $($id).append($dlgBtn.append($inputOk).append($inputCancel))
                $checkBoxDiv.ejScroller({ height: scrollHeight, width: "98%" }).ejScroller('refresh');
            var localXFLabel = { True: proxy.localizedLabels.trueText, False: proxy.localizedLabels.falseText },
                  records = [];
            $.grep(proxy.model.flatRecords, function (record) {
                if (!record.isSummaryRow)
                    records.push(record.item);
            });
            proxy._excelFilter.openXFDialog({ field: column.field, enableResponsiveRow: true, displayName: column.headerText, dataSource: records, query: null, position: { X: 0, Y: 0 }, dimension: { height: windowHeight, width: windowWidth }, cssClass: "resFilter", type: colType, format: column.format, localizedStrings: localXFLabel });
            $($id).insertAfter(proxy.element);
            $closeIcon.click(function (e) {
                //Responsive UI rendering for custom excel filter
                $.proxy(proxy._closeDivIcon(e), this);
                var height = windowHeight - 30, width = windowWidth;
                excelObj._openCustomFilter('customfilter');
                var $dlgClone = $("#" + proxy._id + colType + "_CustomFDlg").addClass('e-responsviesExcelFilter');
                proxy._customFilter = $dlgClone;
                if (ej.isNullOrUndefined($dlgClone.find('.e-treegird-filterDialogHeaderDiv')[0])) {
                    var $headerDivCustom = ej.buildTag('div.e-treegird-filterDialogHeaderDiv'),
                        $titleSapn = ej.buildTag('div.e-treegrid-label', '<span>' + excelObj.localizedLabels.title + '</span>', { 'font-weight': 'bold', 'font-family': 'Segoe UI, Helvetica Neue, Ubuntu, Arial' });
                    $headerDivCustom.append($titleSapn);
                    var $backIcon = ej.buildTag('div.e-treegrid-resFilterleftIcon', '', {}, { closeDialogue: proxy._id + colType + "_CustomFDlg", openDialogue: $id.slice(1) }),
                        $spanIcon = ej.buildTag('span.e-icon e-treegrid-resIcon e-treegrid-responsiveBack', '', {}, { closeDialogue: proxy._id + colType + "_CustomFDlg", openDialogue: $id.slice(1) });
                    $backIcon.click(function (e) {
                        $dlgClone.css('display', 'none');
                        $("#" + proxy._id + colType + "_CustomFDlg_wrapper").css("display", "none");
                        var localXFLabel = { True: proxy.localizedLabels.trueText, False: proxy.localizedLabels.falseText },
                               records = [];
                        $.grep(proxy.model.flatRecords, function (record) {
                            if (!record.isSummaryRow)
                                records.push(record.item);
                        });
                        proxy._excelFilter.openXFDialog({ field: column.field, enableResponsiveRow: false, displayName: column.headerText, dataSource: records , query: null, position: { X: 0, Y: 0 }, dimension: { height: windowHeight, width: windowWidth }, type: colType, format: column.format, localizedStrings: localXFLabel });
                    });
                    $headerDivCustom.append($backIcon.append($spanIcon));
                    var $closeIconCust = ej.buildTag('div.e-treegrid-resFilterRigthIcon', '', {}, { closeDialogue: proxy._id + colType + "_CustomFDlg", gridEle: true }),
                        $closeSpan = ej.buildTag('span.e-icon e-treegrid-resIcon e-treegrid-responsiveClose', '', {}, { closeDialogue: proxy._id + colType + "_CustomFDlg", gridEle: true });
                    $headerDivCustom.append($closeIconCust.append($closeSpan));
                    $closeIconCust.click(function (e) {
                        $dlgClone.css("display", "none");
                        $("#" + proxy._id + colType + "_CustomFDlg_wrapper").css("display", "none");
                        proxy.element.css('display', 'block');
                    });
                    $dlgClone.prepend($headerDivCustom);
                    $dlgClone.insertAfter(proxy.element);
                    $dlgClone.find('.e-dlgfields').css('width', '98%');
                    var $firstDiv = $dlgClone.find('.e-dlgfields:first').css('width', '92%').css({ 'margin-left': '6px', 'font-family': 'Segoe UI, Helvetica Neue, Ubuntu, Arial' });
                    $firstDiv.css('margin-top', '4%');
                    var colName = $dlgClone.find('.e-dlgfields').find('.e-fieldset legend').text(),
                        $labelDiv = ej.buildTag('div.e-responsiveLabelDiv', colName, { 'margin-left': '6px', 'font-family': 'Segoe UI, Helvetica Neue, Ubuntu, Arial' });
                    $labelDiv.insertAfter($firstDiv);
                    var $fieldSet = $dlgClone.find('.e-dlgfields').find('.e-fieldset').find('table').css('width', '100%');
                    $dlgClone.find('.e-dlgfields').find('.e-fieldset').replaceWith($fieldSet);
                    var $fieldDiv = $fieldSet.parent('div').addClass('e-responsiveExcelFilterFieldDiv'),
                        $ddl = $fieldSet.find('.e-dropdownlist')
                    $fieldSet.find('.e-dropdownlist').each(function (index, object) {
                        var ds = $(object).ejDropDownList('model.dataSource');
                        $(object).ejDropDownList('destroy').ejDropDownList({ width: "80%", dataSource: ds });
                    });
                    $fieldSet.find('.e-autocomplete').each(function (index, object) {
                        var model = $(object).ejAutocomplete('model');
                        $(object).ejAutocomplete('destroy').ejAutocomplete({ width: "90%", dataSource: model.dataSource, fields: model.fields, enableDistinct: true });
                    });
                    $fieldSet.find('.e-datepicker').each(function (index, object) {
                        var date = $(object).ejDatePicker('instance');
                        date.option("width", "90%");
                    });
                    $fieldSet.find('.e-datetimepicker').each(function (index, object) {
                        var date = $(object).ejDateTimePicker('instance');
                        date.option("width", "90%");
                    });
                    $fieldSet.find('.e-numerictextbox').each(function (index, object) {
                        var numberic = $(object).ejNumericTextbox('instance');
                        numberic.option("width", "90%");
                    });
                }
                var $okClone = $inputOk.clone(), $cancelClone = $inputCancel.clone();
                $okClone.ejButton({
                    text: proxy.localizedLabels.okButtonText, type: 'button', click: function (sender) {
                        proxy._excelFilter._openedFltr = $dlgClone;
                        proxy._excelFilter._fltrBtnHandler();
                        $dlgClone.css("display", "none");
                        $("#" + proxy._id + colType + "_CustomFDlg_wrapper").css("display", "none");

                    }
                });
                $cancelClone.ejButton({
                    text: proxy.localizedLabels.cancelButtonText, type: 'button', click: function () {
                        $dlgClone.css("display", "none");
                        $("#" + proxy._id + colType + "_CustomFDlg_wrapper").css("display", "none");
                    }
                });
                //$dlgBtn.children().hide();
                $dlgClone.append($dlgBtn.clone().append($okClone).append($cancelClone));
                $okClone.css('width', '43.6%');
                $cancelClone.css('width', '43.6%');
                var $btnContainer = $dlgClone.find('.e-dlgfields .e-btncontainer').remove();
                $btnContainer.find('input:first').addClass('e-responsiveFilterBtnLeft').css('width', '45.6%');
                $btnContainer.find('input:last').addClass('e-responsiveFilterBtnRight');
                $dlgClone.ejDialog({
                    enableModal: false, height: height, width: "100%", position: { X: 0, Y: 0 }, enableResize: false, showHeader: false}).ejDialog('open');
                var zIndex = proxy.getMaxZIndex() + 1,
                    overlayIndex = $("#" + proxy._id + colType + "_CustomFDlg_wrapper").css("z-index");
                zIndex = zIndex > overlayIndex ? zIndex : overlayIndex;
                $dlgClone.css({ "position": "absolute", "top": "0px", "left": "0px", "z-index": zIndex, "width": "95.5%" });
                proxy._mediaStatus = true;
            });
        },
        _closeDivIcon:function(sender)
        {
            var $div = $(sender.target);
            if (!ej.isNullOrUndefined($div.attr('closeDialogue'))) {
                var $dialog = $("#" + $div.attr('closeDialogue'));
                if (!ej.isNullOrUndefined($dialog.data('ejDialog')))
                    $dialog.ejDialog('close');
                else
                    $dialog.css('display', 'none');
            }
            if (!ej.isNullOrUndefined($div.attr('gridEle'))) {
                this.element.css('display', 'block');
            }
            if (!ej.isNullOrUndefined($div.attr('openDialogue'))) {
                if (this.model.enableResponsiveRow || $div.attr('closeDialogue').indexOf('Custom') != -1) {
                    var $dialog = $("#" + $div.attr('openDialogue'));
                    if (!ej.isNullOrUndefined($dialog.data('ejDialog')))
                        $dialog.ejDialog('open');
                    else
                        $dialog.css('display', 'block');
                }
                else
                    this.element.css('display', 'block');
            }
        },

        //Get predicate value based on fileredColumn value for excel filtering
        _getExcelPredicate: function (args) {
            var proxy = this,
                model = proxy.model,
                filteredColumns = $.extend(true, [], model.filterSettings.filteredColumns),
                length = filteredColumns.length,
                predicate;
            proxy._excelFilter.getPredicate(filteredColumns, null, true);
            var predicates = proxy._excelFilter._predicates[0];
            for (var i = 0; i < length; i++) {
                var obj = filteredColumns[i];
                if (obj.value == proxy._dropDownListBlanksText) {
                    var dropDownPredicate = ej.Predicate(obj.field, "equal", null, true);
                    dropDownPredicate = dropDownPredicate["or"](obj.field, "equal", "", true);
                    predicates[obj.field] = dropDownPredicate;
                    predicate = predicate != undefined ? predicate["and"](dropDownPredicate) : dropDownPredicate;
                }
                else {
                    if (!ej.isNullOrUndefined(predicate)) {
                        if (!obj.isComplex)
                            predicate = predicate[obj.predicate](obj.field, obj.operator, obj.value, !obj.matchcase);
                        else
                            predicate = predicate["and"](obj);
                    }
                    else
                        predicate = ej.Predicate(obj.field, obj.operator, obj.value, !obj.matchcase);
                }
                if (ej.isNullOrUndefined(args))
                    proxy._storePreviousFilteredValue(obj.field, obj.value, obj.operator);

            }
            return predicate;
        },
        //Render the filter menu based on filter type
        _checkFilterType:function(column)
        {
            var proxy = this,
                model = proxy.model,
                filterSettings = model.filterSettings,
                colType = column.filterEditType ? column.filterEditType : column.editType;
            colType = colType.toLowerCase();
            if (filterSettings.filterType == "excel" && column.filterType == "menu" && $.inArray(colType, proxy._menuColTypes) == -1) {
                var colFilterType, dropdownColumn;
                switch (colType) {
                    case "stringedit":
                        colFilterType = "String";
                        break;
                    case "numericedit":
                        colFilterType = "Numeric";

                        break;
                    case "datepicker":
                        colFilterType = "Datepicker";
                        break;
                    case "datetimepicker":
                        colFilterType = "Datetimepicker";
                        break;
                    case "dropdownedit":
                        colFilterType = "Dropdown";
                        dropdownColumn = column;
                        break;
                    case "booleanedit":
                        colFilterType = "Boolean";
                        break;
                    default:
                        colFilterType = "String";
                        colType = "stringedit";
                        break;
                }
                proxy._renderEdittypeFilterMenu(colFilterType, dropdownColumn);
                proxy._menuColTypes.push(colType);
            }
            else if ((filterSettings.filterType == "menu" && column.filterType == "excel")) {
                if (ej.isNullOrUndefined(proxy._excelFilter))
                    proxy._renderExcelFilter();
            }
        },
        // Method to trigger the column resizing events.
        _triggerColumnResize: function (event, _x, e) {
            var proxy = this, model=proxy.model;
            var _rowobj = proxy.getHeaderTable().find(".e-columnheader");
            var _childTH = _rowobj.find(".e-headercell");
            var cellIndex = proxy._cellIndex, newWidth;
            var target = $(proxy._target), columnIndex = [], col = [],
                currentCellIndex = 0,
                $target = $(e.target);   
            if (event == "columnResizeStart") {
                this._isFrozenStackedResize = false;
                proxy._orgX = _x;
                cellIndex = proxy._currentCell = proxy._cellIndex = proxy.getCellIndex(e);
                if (cellIndex != -1 && this._frozenColumnsLength > 0 && model.showStackedHeader && model.stackedHeaderRows.length > 0) {
                    if (($target.closest(".e-movablecontentdiv").length > 0 || $target.closest(".e-movableheaderdiv").length > 0 || $target.closest(".e-movablefooterdiv").length > 0)) {
                        currentCellIndex += this._frozenColumnsLength;
                    }
                    else if ($target.closest(".e-frozenheaderdiv").length > 0) {
                        if (!$target.hasClass("e-stackedHeaderCell"))
                            currentCellIndex += $(this.getHeaderTable()[0]).find(".e-stackedHeaderCell").length;
                        else
                            this._isFrozenStackedResize = true;
                    }
                }
            }
            if (proxy._currentCell != -1) {
                
                var _outerCell = _childTH[proxy._currentCell + currentCellIndex];
                var _oldWidth = _outerCell.offsetWidth;
                // condition for column resize start event
                if (event == "columnResizeStart") {
                    proxy._isStackedResize = $(_outerCell).hasClass("e-stackedHeaderCell");
                    var args = {};
                    args = { columnIndex: cellIndex, column: proxy.model.columns[cellIndex], target: $(_outerCell), oldWidth: _oldWidth };
                    return proxy._trigger("columnResizeStart", args);
                }
                    //condition for column resize end event
                else if (event == "columnResizeEnd") {
                    var _childth = _rowobj.find(".e-headercell").not(".e-detailheadercell").filter(":visible");
                    //to calculate old, new and extra width values for event argument
                    var _extra = _x - proxy._orgX;
                    proxy._newWidth = _oldWidth + _extra;
                    proxy.__oldWidth = _oldWidth;
                    var args = {};
                    args = { columnIndex: cellIndex, column: model.columns[cellIndex], target: $(_outerCell), oldWidth: _oldWidth, newWidth: proxy._newWidth, extra: _extra };
                    return proxy._trigger("columnResizeEnd", args);
                }
                    //condition for column resized event
                else {
                    if (this.model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns && !this.model.isFromGantt) {
                        this._updateTableWidth();
                    } else if (!this.model.isFromGantt && this.model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.NextColumn && this.model.showTotalSummary) {
                        this._colgroupRefresh();
                    }
                    var args = {};
                    //to calculate old, new and extra width values for event argument
                    var _extra = _x - proxy._orgX;
                    newWidth = _oldWidth + _extra;
                    args = { columnIndex: cellIndex, column: model.columns[cellIndex], oldWidth: proxy.__oldWidth, newWidth: proxy._newWidth };
                    return proxy._trigger("columnResized", args);
                }
            }
        },

        //HEADER MOUSE DOWN EVENT FOR COLUMN RESIZING
        _headerMouseDown: function (e) {

            var proxy = this,
                coord = proxy._getCoordinate(e),                
            $target, $div, model = proxy.model;            
            proxy._cellMouseLeave();            
            proxy._headerMouseLeave();
            if (!proxy.model.isFromGantt && model.contextMenuSettings.showContextMenu) {
                proxy._contextMenuEventBinding(e);
            }
            if (e.type == "touchstart") {
                $target = proxy._getElementByPosition(coord.pageX, coord.pageY);
                proxy._on(proxy.element, "touchmove", proxy._mouseMove);
                proxy._resizer && proxy._resizer._touchStart(e);
            } else
                $target = $(e.target);

            proxy._on($(document), "mouseup touchend", proxy._headerMouseUp);            

            if (model.allowColumnReordering) {
                if ($target.closest(".e-columnheader").length > 0)
                    proxy.dragColumn(e);
                if (e.type == "touchstart") {
                    proxy._dragColumnTooltipTimer = window.setTimeout(function () {
                        proxy._touchstart = false;
                        if (!proxy._dragTooltip && e.type == "touchstart") {
                            proxy.dragColumnToolTip(e);
                            if (proxy._dragTooltip) {
                                proxy._on(proxy.element, "touchmove", proxy.dragColumnToolTip);
                            }
                        }
                    }, 550);
                }
            }
            if (proxy.model.allowColumnResize) {
                if(model.isFromGantt)
                    proxy._on(proxy.element.closest(".e-gantt"), "mouseup touchend", proxy._mouseUp);                
                else
                    proxy._on(proxy.element, "mouseup touchend", proxy._mouseUp);

                $div = $target.closest('tr');
            
                this._target = $target;
                var _x = coord.pageX, _y = coord.pageY;            
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = coord.pageX;
                    _y = coord.pageY - document.body.scrollTop;
                }
                if ($div.length > 0 && $div[0].style.cursor == "col-resize" && !$target.hasClass("e-columnicon")) {
                    if ($target.is(".e-headercelldiv"))
                        $target = $($target[0].parentNode);
                    e.target = $target;

                    if (proxy._frozenColumnsLength > 0) {
                        var columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname"),
                         column = proxy.getColumnByField(columnFieldName);
                    }
                    // Event triggered when we start to resize the column
                    if (this._triggerColumnResize("columnResizeStart", _x, e))
                        return;
                }
                if (e.type != "touchstart")
                    proxy._resizer._mouseDown(e);
            }

            //if (($(e.target).hasClass("headercelldiv") && $(e.target).hasClass("headercell"))) {

            //    var $headercell = $(e.target).hasClass("headercelldiv") ? $(e.target).parent() : $(e.target);
            //    model.headerEffects && $headercell.addClass("headercellactive");

            //}
        },

        _headerMouseUp: function(e){
            var proxy = this, model = proxy.model,
                args = {}, coord = proxy._getCoordinate(e),
                $target;

            if (e.type == "touchend") {
                $target = proxy._getElementByPosition(coord.pageX, coord.pageY);
                proxy._off(proxy.element, "touchmove", proxy._mouseMove);
            }
            else
                $target = $(e.target);            
            proxy._off($(document), "mouseup touchend", proxy._headerMouseUp);
           
            if (model.contextMenuSettings.showContextMenu && proxy._contextMenuHandler && proxy._displayContextMenu) {
                proxy._headerRightClick(e);
            }
            else
                proxy._contextMenuHandler = false;
            proxy._displayContextMenu = true;
            if (proxy.model.allowColumnResize) {
                if (model.isFromGantt)
                    proxy._off(proxy.element.closest(".e-gantt"), "mouseup touchend", proxy._mouseUp);
                else
                    proxy._off(proxy.element, "mouseup touchend", proxy._mouseUp);
                this.element.find(".e-reSizeColbg").remove();
                proxy._resizer._expand = false;
            }

            if (model.allowColumnReordering) {
                window.clearTimeout(this._dragColumnTooltipTimer);
                this._dragColumnTooltipTimer = null;
                proxy._off(proxy.element, "mousemove touchmove", proxy.dragColumnToolTip);
                $(proxy._reordervline).remove();
                $(proxy._dragmouseOverTooltip).remove();
                if (proxy._posx != coord.pageX || proxy._posy != coord.pageY) proxy._mouseDragged = true;
                if (model.allowColumnReordering && $target.closest('.e-headercell').length > 0 && proxy._headerDragStatus == "mousemove") {
                    var columns = model.columns;
                    args.targetColumn = columns[proxy._currentIndex];
                    args.targetColumnIndex = proxy._currentIndex;
                    args.draggedColumn = proxy._draggedColumn;
                    args.draggedColumnIndex = columns.indexOf(proxy._draggedColumn);

                    if (ej.isNullOrUndefined(args.targetColumn.nonColumn) && !ej.isNullOrUndefined(args.targetColumnIndex) && args.targetColumnIndex != args.draggedColumnIndex) {
                        proxy._trigger("columnDrop", args);
                        if (!args.cancel) {
                            proxy.reorderColumn(args.draggedColumn.field, args.targetColumnIndex);
                        }
                    }
                }
            }
            proxy._resetHeaderPrivateProperties();
        },

        _resetHeaderPrivateProperties: function () {
            this._timerDragDown && (this._timerDragDown = window.clearInterval(this._timerDragDown));
            this._timerDragUp && (this._timerDragUp = window.clearInterval(this._timerDragUp));
            var proxy = this, model = proxy.model;                      
            proxy._draggedColumn = null;
            proxy._dragMouseDown = false;
            proxy._dragTooltip = false;
            proxy._headerDragStatus = "";
            proxy._timerDragDown = null;
            proxy._timerDragUp = null;
            proxy._currentIndex = null;                        
            proxy._dragmouseOverTooltip = null;
        },

        //TREE GRID CONTENT MOUSE DOWN EVENT

        _contentMouseDown: function (e) {
            var $target = $(e.target), 
                proxy = this, model = proxy.model, coord = proxy._getCoordinate(e), id = $target.closest(".e-treegrid").attr("id");
            proxy._contentPageY = coord.pageY, proxy._contentPageX = coord.pageX;
            
            proxy._on($(document), "mouseup touchend", proxy._mouseupHandler);

            //to update ctrlKey on drag and drop during multiselect icon clicked
            if (model.isFromGantt) {
                if (e.type == "touchstart")
                    proxy._ganttTouchTrigger = true;
                var args = {
                    ctrlKey: e.ctrlKey
                }
                proxy._trigger("touchHandler", args);
                e.ctrlKey = args.ctrlKey;
            }
            if (e.type == "touchstart") {
                proxy._isTouchEvent = true;            
                proxy._isTouchTrigger = true;
            }

            if ($("#" + id + "_selectionpopup").is(":visible")) {
                if ($("#" + id + "_selectionpopup").find(".e-rowselect").hasClass("e-spanclicked")) {
                    e.ctrlKey = true;
                    if (proxy.model.selectionSettings.selectionMode == ej.TreeGrid.SelectionMode.Row)
                    proxy._multiSelectCtrlRequest = true;
                }
                else {
                    e.ctrlKey = false; proxy._multiSelectCtrlRequest = false;
                }
            }
            if (!proxy.model.isFromGantt && model.contextMenuSettings.showContextMenu) {
                proxy._contextMenuEventBinding(e);
            }
            
            if (!($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vdown")
                  || $target.hasClass("e-vhandlespace") || $target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hdown")
                  || $target.hasClass("e-hhandlespace") || $target.closest(".e-editedcell").length > 0 || $target.closest(".e-unboundeditedcell").length > 0||$target.parent().hasClass("e-unboundcelldiv")))
            {                
                proxy._mouseDown = true;
                if (ej.isTouchDevice())
                    e.preventDefault();
                if (model.allowDragAndDrop) {
                    //Events binded for Row drag and drop
                    if ($target.closest(".e-treegridrows").length > 0 || $target.closest(".e-templatecell").length > 0) {
                        if (e.type == "touchstart") {
                            proxy._dragTooltipTimer = window.setTimeout(function () {
                                proxy.dragRecord(e);
                                if (proxy._multiSelectPopup && proxy._multiSelectPopup.is(":visible") && proxy._dragTooltip)
                                    proxy._clearMultiSelectPopup();

                            }, 550);
                        }
                        else
                            proxy.dragRecord(e);
                    }
                }
                if (!model.isFromGantt) {
                    if (e.type == "touchstart")
                        proxy._on(proxy.element, "touchmove", "#" + proxy._id + "e-gridcontent", proxy._touchScroll);                   
                }
                if (e.ctrlKey || e.shiftKey || e.which == 2 || e.which == 3)
                    return true;
            }
        },

        _touchScroll: function (e) {
            var proxy = this, model = proxy.model, args = {},
                coord = proxy._getCoordinate(e), $target, pixelX,pixelY,               
                prevX = proxy._contentPageX, prevY = proxy._contentPageY,
                posX = coord.pageX, posY = coord.pageY,
                dx, dy;
            proxy._isTouchTrigger = false;
            
            dx = posX - prevX;
            dy = posY - prevY;                        
            if (e.type == "touchmove") {                
                e.preventDefault();                
            }
            if ((proxy._mouseDown &&  !proxy._dragMouseDown)) {

                window.clearTimeout(this._dragTooltipTimer);
                this._dragTooltipTimer = null;
                window.clearTimeout(this._dragColumnTooltipTimer);
                this._dragColumnTooltipTimer = null;

                if (e.type == "touchmove")
                    $target = proxy._getElementByPosition(coord.pageX, coord.pageY);
                else
                    $target = $(e.target);

                if (!model.isFromGantt && $target.closest('div').find('.e-content')) {

                    if (prevY < posY)
                        pixelY = -Math.abs(dy);
                    else
                        pixelY = Math.abs(dy);

                    if (prevX < posX)
                        pixelX = -Math.abs(dx);
                    else
                        pixelX = Math.abs(dx);

                    proxy._contentPageX = posX;
                    proxy._contentPageY = posY;

                    proxy._updateScrollPosition($target, pixelX, pixelY);
                }
            }
        },

        _updateScrollPosition: function (target, px, py) {

            var proxy = this,
                scrollTop = proxy.getScrollElement().ejScroller("option", "scrollTop"),
                scrollLeft = proxy.getScrollElement().ejScroller("option", "scrollLeft"),
                deltaX = scrollLeft + px, deltaY = scrollTop + py,
                sHeight = proxy.isVScroll() ? proxy.getMaxScrollHeight() : null,
                sWidth = proxy.getScrollElement().ejScroller("isHScroll") ? proxy.getScrollElement().find(".e-content").get(0).scrollWidth : null;
            
            if (sHeight) {
                deltaY = sHeight < deltaY ? sHeight : deltaY < 0 ? 0 : deltaY;
                if ((deltaY > 0 && scrollTop > 0) || (deltaY < sHeight && scrollTop < sHeight)) {
                    proxy._$gridContent.scrollTop(0);
                    proxy.getScrollElement().ejScroller("scrollY", deltaY, true);
                }
            }

            if (sWidth) {
                deltaX = sWidth < deltaX ? sWidth : deltaX < 0 ? 0 : deltaX;
                if ((deltaX > 0 && scrollLeft > 0) || (deltaX < sWidth && scrollLeft < sWidth)) {
                    proxy._$gridContent.scrollLeft(0);
                    proxy.getScrollElement().ejScroller("scrollX", deltaX, true);
                }
            }                        
        },

        _mouseMove: function (e) {

            var proxy = this, model = this.model;
            window.clearTimeout(this._dragColumnTooltipTimer);
            this._dragColumnTooltipTimer = null;
            if (e.type == "touchmove") {                
                if (proxy._resizer && proxy._resizer._allowStart)
                    proxy._headerDragStatus = "";
            }
            if (proxy.model.allowColumnResize && proxy._headerDragStatus.length == 0) {
                proxy._resizer._mouseMove(e);
                if (proxy._resizer._expand)
                    proxy._cancelEditState();
                if ($(e.target).parent().css("cursor") !== "col-resize") {
                    if ((model.allowSorting || model.showColumnChooser || (!model.showColumnChooser && model.allowFiltering && model.filterSettings.filterType == "menu")) && !$(e.target).closest("th").hasClass("e-detailheadercell") && !$(e.target).closest("th").hasClass("e-stackedHeaderCell")) {
                        proxy.getHeaderTable().find(".e-columnheader").css({ 'cursor': 'pointer' });
                    } else {
                        proxy.getHeaderTable().find(".e-columnheader").css({ 'cursor': 'default' });
                    }
                }
            }
        },
        //to get the frozen width by using col group element
        _frozenWidth: function () {
            var proxy = this, frozenColumns, width;
            var frozenColumns = proxy.getContentTable().find('colgroup').find("col").slice(0, proxy.model.scrollSettings.frozenColumns),
                       width = 0;
            for (var i = 0; i < frozenColumns.length; i++)
                width += frozenColumns[i].style.display == 'none' ? 0 : parseInt(frozenColumns[i].style.width.replace("px", ""));
            return width;
        },


        _mouseUp: function (e) {

            var proxy = this,
                model = proxy.model;

            if (proxy._headerDragStatus == "mousemove") return;

            if (model.allowColumnResize) {
                
                if ($(e.target).hasClass('e-headercelldiv') || $(e.target).hasClass('e-reSizeColbg')) {
                    proxy._focusTreeGridElement();
                    proxy._clearColumnMenu();
                }
                if ($(e.target).hasClass("e-reSizeColbg") || $(e.target).closest(".e-gridheader").length > 0) {
                // if treegrid column resizer initiated
                if (proxy._resizer._expand) {
                    var _x = e.clientX, _y = e.clientY;
                    if (navigator.userAgent.indexOf("WebKit") != -1) {
                        _x = e.pageX;
                        _y = e.pageY;
                    }
                    if (e.type == "touchend") {
                        _x = e.originalEvent.changedTouches[0].clientX;
                        _y = e.originalEvent.changedTouches[0].clientY;
                    }
                    // Event triggered when we mouse the column resize
                    if (!ej.isNullOrUndefined(proxy._currentCell) && this._triggerColumnResize("columnResizeEnd", _x, e))
                    {
                        this.element.find(".e-reSizeColbg").remove();
                        proxy._resizer._expand = false;
                        if (model.isFromGantt)
                            proxy._off(proxy.element.closest(".e-gantt"), "mouseup touchend", proxy._mouseUp);
                        else
                            proxy._off(proxy.element, "mouseup touchend", proxy._mouseUp);
                        return;
                    }
                    //to cancel the column resize of frozen column if it exceeds the grid width
                    if (!ej.isNullOrUndefined(proxy._currentCell) && model.columns[proxy._currentCell].isFrozen == true && _x + 100 > this._gridWidth) {
                        return;
                    }
                    proxy._resizer._mouseUp(e);
                    //update columnWidthCollection after column resize
                    if (!ej.isNullOrUndefined(proxy._currentCell) && proxy._currentCell >= 0 && proxy._currentCell < model.columns.length) {
                        model.columns[proxy._currentCell] && (model.columns[proxy._currentCell].width = proxy._resizer._newWidth);
                    }
                    // Event triggered after column resized
                    !ej.isNullOrUndefined(proxy._currentCell) && this._triggerColumnResize("columnResized", _x, e);                    
                    if (model.allowTextWrap&&model.allowColumnResize)
                        proxy._updateRowHeight();
                    if (model.sizeSettings.height == "auto" && model.allowColumnResize) {
                        proxy.getScrollElement().ejScroller("refresh");
                        proxy._updateContentHeight();
                    }
                    proxy.getScrollElement().ejScroller("refresh");
                    proxy._updateHeaderScrollLeft();
                    proxy._updateScrollCss();
                    if (model.isFromGantt) {
                        proxy._trigger("refreshScrollCss");
                    }
                    }
                }else
                {
                    this.element.find(".e-reSizeColbg").remove();
                    proxy._resizer._expand = false;
                    if (model.isFromGantt)
                        proxy._off(proxy.element.closest(".e-gantt"), "mouseup touchend", proxy._mouseUp);
                    else
                        proxy._off(proxy.element, "mouseup touchend", proxy._mouseUp);
                }
            }
        },

        // resize the filtering elements after column resized.

        _resizeFilteringElements:function()
        {
            var proxy = this;
            if (proxy.model.allowFiltering) {
                var args = {}, colLength = proxy.model.columns.length;
                args.requestType = ej.TreeGrid.Actions.Filter;
                for (var item = 0; item < colLength ; item++) {
                    proxy.model.columns[item].width = proxy.columnsWidthCollection[item];
                    args.fieldname = proxy.model.columns[item].field;
                    this._filteringElements(args);
                }
            }
        },

        //ENABLE THE EDITING EVENTS
        _enableEditingEvents: function () {

            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing) {
                if (ej.isTouchDevice()) {
                    proxy._off(proxy.element, "doubletap ", ".e-gridcontent", proxy._editdblClickHandler);
                    proxy._on(proxy.element, "doubletap ", ".e-gridcontent", proxy._editdblClickHandler);
                }
                else {
                    proxy._off(proxy.element, "dblclick ", ".e-gridcontent", proxy._editdblClickHandler);
                    proxy._on(proxy.element, "dblclick ", ".e-gridcontent", proxy._editdblClickHandler);
                }
            }
            else {
                if (ej.isTouchDevice())
                    proxy._off(proxy.element, "doubletap ", ".e-gridcontent", proxy._editdblClickHandler);
                else
                    proxy._off(proxy.element, "dblclick ", ".e-gridcontent", proxy._editdblClickHandler);
            }

        },


        //ENABLE THE MOUSEDOWNEVENT HANDLER FOR SAVE CELL
        _saveCellHandler: function (e) {

            var $target = $(e.target),
                proxy = this, model = proxy.model, targetIndex;
            if (model.isFromGantt && $target.closest("tr").length) {
                var $targetElement = $target.closest("tr"),
                    $gridRows = proxy.getRows(),
                    rowIndex = this.getIndexByRow($targetElement),
                    record = model.currentViewData && model.currentViewData[rowIndex];
                targetIndex = model.updatedRecords.indexOf(record);             
            }
            //SKIP THE SCROLL BAR CLICK IN TREE GRID.
            if ($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vdown") || $target.hasClass("e-vhandlespace") || $target.hasClass("e-vscroll"))
                return;
            if ($target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hup") || $target.hasClass("e-hdown") || $target.hasClass("e-hhandlespace") || $target.hasClass("e-hscroll"))
                return;
          //  e.stopPropagation();
            if ($target.closest(".e-gridcontent").length > 0 || $target.hasClass(".e-gridcontent") || $target.hasClass('e-summarytreegridexpand') || $target.hasClass('e-summarytreegridcollapse')) {
                if (proxy._isRowEdit) {
                    if ($target.closest("form#" + proxy._id + "EditForm").length == 0 && $target.closest("form#" + proxy._id + "EditFrozenForm").length == 0 &&  $("#" + proxy._id + "EditForm").length > 0)
                        proxy.saveRow();
                 } else if ($target.closest(".e-popup").length == 0 && $target.closest(".e-rowcell").find("#" + proxy._id + "EditForm").length == 0
                     && $target.closest("form#" + proxy._id + "EditFrozenForm").length == 0 && $("#" + proxy._id + "EditForm").length > 0) {
                    proxy.saveCell();
                }
            }

            //Row selection for Gantt alone here
            //Why because, To select the clicked row after saving the edited cell (Due to predecessor and Row refresh)
            if (model.selectedMode == "row" && model.isFromGantt && model.updatedRecords.length && !ej.isNullOrUndefined(targetIndex) && !$target.hasClass('e-treegridexpand') && !$target.hasClass("e-treegridcollapse")) {
               if (this.selectedRowIndex() != targetIndex && !proxy._rowSelectingEventTrigger(this.selectedRowIndex(), targetIndex)) {
                    if (model.allowSelection)
                   proxy.selectRows(targetIndex);
                   proxy._rowSelectedEventTrigger(targetIndex);
               }
           }
        },

        //Mouse right click (Context Menu,header Context Menu) handler
        _contextMenuAction: function (e) {
          
            var proxy = this,
                $target, model = proxy.model,
                coord = proxy._getCoordinate(e);

            if(e.type == "taphold")
                $target = proxy._getElementByPosition(coord.pageX, coord.pageY);
            else
                $target = $(e.target);
            if (model.contextMenuSettings.showContextMenu) {
                if (proxy._multiSelectCtrlRequest)
                    proxy._multiSelectCtrlRequest = false;
                if (e.which == 3 && !proxy._isTouchEvent){

                    if ($target.closest(".e-gridcontent").length > 0) {
                        proxy._rightClick(e);
                    }
                    else {
                        if($target.closest(".e-gridheader").length>0)
                            proxy._headerRightClick(e);

                    }
                }
                else{
                    proxy._contextMenuHandler = true;
                    return true;
                }
            }
            else {
                e.stopPropagation();
            }            
        },

        _rightClick: function (e) {
            
            var proxy = this,
                model = proxy.model,
               $target = $(e.target),
               div = $target[0].parentNode,
               $ganttGridRows,
               row,
               recordIndexr,
               item, selectIndex,
               args = {},
               checkEditMenu = false,
               updatedRecords = proxy.getUpdatedRecords();
            /* check inner treegrid click events and  */
            if ($target.closest(".e-treegrid").attr("id") !== this._id) return false;

            if (model.selectionSettings.selectionMode == "cell" && model.editSettings.editMode != "dialogEditing") {
                model.editSettings.editMode = "cellEditing";
            }

            if (proxy.model.contextMenuSettings.showContextMenu) {
                proxy._clearContextMenu();
                this._removeDetailsRow();
                proxy.disableTooltip();
                if ($target.hasClass("e-summaryrowcell") || $target.hasClass("e-summarytitle"))
                    return;
                if ($target.hasClass("e-cell") || $target.hasClass("e-treegridexpand"))
                    div = $target.closest('tr');
                row = $target.closest('tr');
                recordIndexr = proxy.getIndexByRow(row);
                proxy.model.currentViewData = proxy.getCurrentViewData();
                item = proxy.model.currentViewData[recordIndexr];
                recordIndexr = updatedRecords.indexOf(item);
                args.data = item;
                args.recordIndex = recordIndexr;
                if (model.isEdit)
                    checkEditMenu = ($target.closest(".e-rowcell").find("form#" + proxy._id + "EditForm").length > 0) || ($target.closest(".e-rowcell").find("form#" + proxy._id + "EditFrozenForm").length > 0);
                else if (this._isRowEdit) {
                    checkEditMenu = ($target.closest(".e-rowedit").find("form#" + proxy._id + "EditForm").length > 0) || ($target.closest(".e-rowedit").find("form#" + proxy._id + "EditFrozenForm").length > 0) ||
                                    ($target.closest(".e-addedrow").find("form#" + proxy._id + "EditForm").length > 0) || ($target.closest(".e-addedrow").find("form#" + proxy._id + "EditFrozenForm").length > 0);
                }
                /* save edited value before rendering contextmenu on non edited cell*/
                if (!checkEditMenu) {
                    if(!this.editFormValidate())
                        return true;
                    proxy._saveEditedElements();
                }
                //Context menu for edited row or empty records
                if ((proxy.dataSource() == null || proxy.dataSource().length == 0 || checkEditMenu || model.flatRecords.length == 0)) {
                    proxy._contextMenuItems = [];
                    proxy._contextMenuItems = proxy._getContextMenuItems();
                    proxy._subContextMenuItems = [];
                    proxy._updateContextmenuOption(recordIndexr, item);
                    proxy._clearContextMenu();
                    if (proxy._contextMenuItems.length > 0)
                        proxy._renderContextMenu(e, recordIndexr, item, checkEditMenu);

                } else if (item) {
                    e.preventDefault();
                    proxy._contextMenuItems = [];
                    proxy._contextMenuItems = proxy._getContextMenuItems();
                    proxy._subContextMenuItems = [];
                    proxy._subContextMenuItems = proxy._getsubContextMenuItems();
                    proxy._updateContextmenuOption(recordIndexr, item);
                    proxy._clearContextMenu();

                   
                    if (model.selectionSettings.selectionMode == "row") {
                        if (model.allowSelection && !(proxy._rowSelectingEventTrigger(this.selectedRowIndex(), recordIndexr, false, e.type))) {
                            proxy.selectRows(recordIndexr);
                            proxy._rowSelectedEventTrigger(recordIndexr, null, false, e.type);
                        }
                    }
                    if (model.selectionSettings.selectionMode == "cell") {
                        var selectCellIndex = proxy.getCellIndex(e);
                        proxy.selectCells([{ rowIndex: recordIndexr, cellIndex: selectCellIndex }]);
                        proxy._rowIndexOfLastSelectedCell = recordIndexr;
                    }

                    if (proxy._contextMenuItems.length > 0)
                        proxy._renderContextMenu(e, recordIndexr, item, checkEditMenu);
                    if (proxy.editFormValidate()) proxy._cancelSaveTools();
                }
            }
            proxy._clearColumnMenu();
            proxy._contextMenuHandler = false;
            if (!proxy.model.isFromGantt && model.contextMenuSettings.showContextMenu) {
                if (ej.isTouchDevice())
                    proxy._off(proxy.element, "taphold", proxy._contextMenuAction);
                else
                    proxy._off(proxy.element, "contextmenu", proxy._contextMenuAction);
                
            }
            return;
        },

        //contextMenu Event Binding
        _contextMenuEventBinding: function (e) {
            var proxy = this;
            if (ej.isTouchDevice()) {
                proxy._off(proxy.element, "taphold", proxy._contextMenuAction);
                proxy._on(proxy.element, "taphold", proxy._contextMenuAction);
            }
            else {
                proxy._off(proxy.element, "contextmenu", proxy._contextMenuAction);
                proxy._on(proxy.element, "contextmenu", proxy._contextMenuAction);

            }


        },

        _headerRightClick: function (e) {
            
            var proxy = this,
                 model = proxy.model,
                  $target = $(e.target);
            
            e.preventDefault();
            if (model.contextMenuSettings.showContextMenu) {
                proxy._clearColumnMenu();
                proxy._removeDetailsRow();
                proxy.disableTooltip();
                proxy._headerContextMenuItems = [];

                proxy._headerContextMenuItems = proxy._getHeaderContextMenuItems(e);
                var columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname"),
            column = proxy.getColumnByField(columnFieldName);

                if (proxy._headerContextMenuItems.length > 0&& !column.nonColumn)
                    proxy._renderHeaderContextMenu(e);
                if (proxy.editFormValidate()) 
                    proxy._cancelSaveTools();
                proxy._contextMenuHandler = false;
                if (!model.isFromGantt && model.contextMenuSettings.showContextMenu) {
                    if (ej.isTouchDevice())
                        proxy._off(proxy.element, "taphold", proxy._contextMenuAction);
                    else
                        proxy._off(proxy.element, "contextmenu", proxy._contextMenuAction);

                }
                
            }
        },
        //DOUBLE CLICK EVENT HANDLER FOR DIALOG EDITING

        _editdblClickHandler: function (e) {

            var proxy = this,
                model = proxy.model,
                $form = $("#" + proxy._id + "EditForm"),
                args = {},
                $target = $(e.target),
                row = $target.closest('tr'),
                record,
                cellEditingDetails = proxy._cellEditingDetails;
            if ((model.isFromGantt && model.readOnly == true) || $target.hasClass("e-unboundcelldiv") || $target.hasClass("e-unboundcell") || $target.closest('td').hasClass("e-unboundcell") || e.ctrlKey) {
                return true;
            }
            
            var updatedRecords = proxy.getUpdatedRecords();
            if ($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vdown") || $target.hasClass("e-vhandlespace") ||
                $target.hasClass("e-treegridexpand") || $target.hasClass("e-treegridcollapse"))
                return;
            if ($target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hup") || $target.hasClass("e-hdown") || $target.hasClass("e-hhandlespace"))
                return;
            if ($target.hasClass("e-summaryrowcell") || $target.hasClass("e-footersummaryrowcell")
                || $target.hasClass("e-summarytitle") || $target.hasClass("e-lastsummaryrow"))
                return;
            /* check inner treegrid click events and  */
            if ($target.closest(".e-treegrid").length && $target.closest(".e-treegrid").attr("id") !== this._id) return false;            

            if ($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell") || ($target.closest("td").hasClass("e-rowcell"))
                || ($target.closest("td").hasClass("e-rowcell")) && $target.hasClass("e-cell")) {

                if (model.editSettings.editMode === "normal" && model.editSettings.allowEditing) {
                    if (model.viewType == "resourceView")
                        return true;
                    else {
                        args.requestType = ej.TreeGrid.Actions.BeginEdit;
                        proxy._trigger("actionBegin", args);
                    }
                }

                if (proxy._cellEditingDetails.cancelSave) {

                    proxy._cellEditingDetails.cancelSave = false;
                    return;

                }

                if ($form.length === 0 && proxy.getRows() !== null) {

                    if (model.editSettings.editMode.toLowerCase() == "cellediting" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode.toLowerCase() == "cell")) {
                        proxy._focusTreeGridElement();
                        proxy._cellEditingDetails.columnIndex = proxy.getCellIndex(e);
                        if (model.editSettings.beginEditAction != "click")
                        proxy._cellEditingDetails.rowIndex = proxy.getRowIndex(e);
                        record = model.currentViewData[proxy._cellEditingDetails.rowIndex];
                        if (record.summaryRow)
                            return;
                        model.selectedItem = record;                        
                        if (model.columns[cellEditingDetails.columnIndex].allowEditing == false || (model.viewType == "resourceView" && record.eResourceTaskType == "unassignedTask"))
                            return;
                        var fieldName = proxy._cellEditingDetails.columnIndex >= 0 && model.columns[proxy._cellEditingDetails.columnIndex].field;
                        if (fieldName) {
                            proxy.updateScrollBar();
                            model.editSettings.allowEditing && proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName);
                        }
                    }
                    else if (model.editSettings.editMode.toLowerCase() == "rowediting" || model.editSettings.editMode.toLowerCase() == "dialogediting" || (model.editSettings.editMode.toLowerCase() == "batchediting" && (model.editSettings.batchEditSettings.editMode.toLowerCase() == "row" || model.editSettings.batchEditSettings.editMode.toLowerCase() == "dialog"))) {
                        model.currentViewData = proxy.getCurrentViewData();
                        if (proxy.getCellIndex(e) != -1 && proxy.getRowIndex(e) != -1) {
                            proxy._cellEditingDetails.columnIndex = proxy.getCellIndex(e);
                            var rowIndex = proxy.getRowIndex(e);
                            if (model.enableVirtualization) {
                                var selectedItem = model.selectionSettings.selectionMode == "row" ? model.selectedItem : model.updatedRecords[proxy._rowIndexOfLastSelectedCell];
                                rowIndex = updatedRecords.indexOf(selectedItem);
                            }
                            if ((proxy.model.editSettings.editMode.toLowerCase() == "dialogediting" || (proxy.model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode.toLowerCase() == "dialog"))) {
                                proxy._showTreeGridAddEditDialog(rowIndex);
                            } else {
                                proxy.updateScrollBar();
                                proxy._editRow(rowIndex, undefined, proxy._cellEditingDetails.columnIndex);
                            }
                        }
                    }
                    $form = $("#" + proxy._id + "EditForm");
                    if ($form.length > 0)
                        proxy._editAddTools();

                    if ($.isFunction($.validator)) {
                        proxy._initValidator();
                    }
                }

                if (!proxy.model.allowSelection)
                    row.find(".e-active").removeClass("e-active").removeClass("e-selectionbackground");
                proxy._clearContextMenu();
                this._removeDetailsRow();
            }            
        },

        //To hide the datepicker and dropdown list.
        _popupHide: function (element) {
            var proxy = this,
             column = proxy.model.columns,
             object = null,
             dateobj = null,
             columnField = null,
             tdElement = element && $(element).closest("td");
            if (proxy.model.allowFiltering && proxy.model.filterSettings.filterType == "filterbar") {
                tdElement = element && $(element).closest("td, th");
            }
            var columnChooser = $("#" + proxy._id + "_ColumnMenu").length;
            if (columnChooser > 0) {
                if (!ej.isTouchDevice())
                proxy._clearColumnMenu();
            }
            for (var i = 0; i < column.length; i++) {
                object = null;

                if (column[i]) {
                    columnField = column[i].field.split('.').join("");
                    if (column[i].editType == "datepicker" || column[i].editType == "datetimepicker")
                        object = $("#" + proxy._id + columnField).data("ejDatePicker");
                    else if (column[i].editType == "dropdownedit") {
                        var dropdownobject = $("#" + proxy._id + columnField).data("ejDropDownList");
                        if (dropdownobject && $(tdElement).find("#" + dropdownobject._id).length == 0)
                            dropdownobject.model && dropdownobject.hidePopup();
                    }                    
                    if (object && $(tdElement).find("#" + object._id).length == 0)
                        object.hide();

                    if (column[i].filterEditType == "datepicker" || column[i].filterEditType == "datetimepicker") {
                        dateobj = $("#" + proxy._id + "_" + columnField + "_filterbarcell").data("ejDatePicker");
                        if (dateobj && $(tdElement).find("#" + dateobj._id).length == 0)
                            dateobj.hide();
                    }
                    if (column[i].filterEditType == "dropdownedit") {
                        var dropdownobject = $("#" + proxy._id + "_" + columnField + "_filterbarcell").data("ejDropDownList");
                        if (dropdownobject && $(tdElement).find("#" + dropdownobject._id).length == 0)
                            dropdownobject.hidePopup();
                    }
                }
            }
        },
        //PERFORM SCROLLING IN TREEGRID
        _mouseWheel: function (e) {

            var proxy = this,
                model = proxy.model,
                delta = null,
                ori = e;            
            proxy._popupHide();
            
            e = e.originalEvent;

            if ($(e.target).hasClass('e-footersummaryrowcell'))
                return;
            if (proxy.getScrollElement().height() > proxy._$gridContainer.height()) {
                proxy.getScrollElement().ejScroller("scrollY", 0, true);
                return;
            }

            if (e.wheelDelta) {
                delta = -e.wheelDelta / 120;
                if (window.opera) {
                    if (parseFloat(window.opera.version, 10) < 10)
                        delta = -delta;
                }
            } else if (e.detail) {
                delta = e.detail / 3;
            }
	
            if (!delta) return false;

            var scrollTop = proxy.getScrollElement().ejScroller("option", "scrollTop"),

                change = scrollTop + (delta * 57);  

            var h = this.model.isFromGantt ? parseInt(this.getRecordsHeight()) - (proxy._viewPortHeight - 18) :model.sizeSettings.height=="auto"?0: this.getMaxScrollHeight();
            if (change > h) {
                change = h;
            }
            if (change < 0) {
                change = 0;
            }
            if (scrollTop != change) {
                try {
                    e.preventDefault ? e.preventDefault() : ori.preventDefault();
                }
                catch (err) {
                }
            }
            proxy.updateScrollTop(change);
            return true;

        },
        _getCollapsedParentItem:function(record)
        {
            var parentRecord = record.parentItem;
            if (parentRecord) {
                if (this.getExpandStatus(parentRecord) === true) {
                    return parentRecord;
                } else {
                    return this._getCollapsedParentItem(parentRecord)
                }
            }
            else {
                return null;
            }
        },
        /*Method to get top position of record for update the scrollbar to required position*/
        _getRecordTopPosition: function (records, index) {
            var position = {}, height = 0, model = this.model;
            if (model.viewType == "resourceView") {

                for (var i = 0; i <= index; i++) {
                    if (i == index)
                        position.bottom = height + this._getTemplateRowHeight(records[i]);
                    else
                        height += this._getTemplateRowHeight(records[i]);
                }
                position.top = height;
            } else {
                position.top = index * (model.rowHeight + this._detailsRowHeight);
                position.bottom += (model.rowHeight + this._detailsRowHeight);
            }
            return position;
        },
        // Update the scroll bar while edit, expand and collapse the record for focusing the row.
        updateScrollBar: function (rowIndex) {
            var proxy = this,
                model = proxy.model, horizontalScrollBarHeight = 0,
                currentUpdatedRecords, recordIndex, rowTop,
                rowHeight, totalHeight,
                isInViewPortAbove, isInViewPortBelow,
                updatedRecords = proxy.getUpdatedRecords(), selectedItem;
            if (model.allowSelection) {
                selectedItem = model.selectionSettings.selectionMode == "row" ? this.selectedItem() : updatedRecords[proxy._rowIndexOfLastSelectedCell];
            }
            else {
                selectedItem = this.selectedItem();
            }
            currentUpdatedRecords = model.allowPaging ? updatedRecords : (proxy._isHierarchyFiltering && model.enableVirtualization) ? proxy._updateFilterParent : proxy.getExpandedRecords(updatedRecords);
            if (rowIndex || rowIndex == 0) {
                    recordIndex = currentUpdatedRecords.indexOf(updatedRecords[rowIndex]);
                } else {
                    recordIndex = currentUpdatedRecords.indexOf(selectedItem);
                }            

            if (model.allowTextWrap) {
                var updatedrecord = proxy.getUpdatedRecords(),
                    recordIndex = updatedrecord.indexOf(selectedItem);
                if (model.enableVirtualization) {
                    rowTop = proxy._getAvgTextWrapRowHeight("updateHeight", recordIndex).height;
                    rowHeight = updatedrecord[recordIndex].textWrapRowHeight + proxy._detailsRowHeight;
                } else {
                    var $tr = ej.TreeGrid.getRowByIndex(proxy, recordIndex);
                    if ($tr.length > 0) {
                        rowHeight = $tr.height() + proxy._detailsRowHeight;
                        rowTop = $tr[0].offsetTop;
                    } else {
                        rowHeight = model.rowHeight + proxy._detailsRowHeight;
                        rowTop = recordIndex * rowHeight;
                    }
                }
            } else {
                var rowPos = this._getRecordTopPosition(currentUpdatedRecords, recordIndex);
                rowHeight = model.rowHeight + this._detailsRowHeight;
                rowTop = rowPos.top;
            }

            var rowBottom = rowTop + rowHeight,
                scrollTop,
                containerWidth,
                scrollerExist = proxy.getScrollElement().children(".e-content").length;
   
            if (scrollerExist)
                scrollTop = proxy.getScrollElement().children(".e-content").scrollTop();
            else
                scrollTop = proxy.getScrollElement().scrollTop();

            if (model.allowTextWrap) {
                if (proxy.model.enableVirtualization) {
                    totalHeight = proxy._getAvgTextWrapRowHeight("updateHeight").height;
                } else {
                    totalHeight = proxy._$gridContentTable.height();
                }
            } else {
                totalHeight = this.getRecordsHeight();
            }
    
            if(proxy.getScrollElement().ejScroller("isHScroll")  || proxy.element.find(".e-borderbox").length > 0)
            {
                horizontalScrollBarHeight = 18; /* default horizontal scrollbar height*/
            }

            isInViewPortAbove = (rowTop < scrollTop);
            isInViewPortBelow = model.sizeSettings.height == "auto" ? false : (scrollTop + proxy._viewPortHeight - horizontalScrollBarHeight) < rowBottom;


            if (isInViewPortAbove || isInViewPortBelow) {

                if ((rowTop + proxy._viewPortHeight) > totalHeight) {
                    rowTop = totalHeight - proxy._viewPortHeight;

                    if (isInViewPortBelow)
                        rowTop += horizontalScrollBarHeight;
                }

                    var args = {
                        requestType: ej.TreeGrid.Actions.Scroll,
                         delta: rowTop
                    };
                    if (scrollerExist)
                        proxy.updateScrollTop(rowTop);
                    else {
                        proxy.getScrollElement().scrollTop(args.delta);
                        proxy._completeAction(args);
                    }
                    if (model.enableVirtualization) {
                       proxy._focusTreeGridElement();
                    }
            }
        },
        /*Select row on shift pluse down arrow key*/
        _selectedItemDown: function (action) {
            var proxy = this,
            model = this.model,
            fromIndex, toIndex, index,
            updatedRecords = proxy.getUpdatedRecords(),
            expandedRecords = proxy.getExpandedRecords(updatedRecords);

            if (proxy._prevSelectedItem && this._getExpandStatusRecord(proxy._prevSelectedItem))
                fromIndex = model.updatedRecords.indexOf(proxy._prevSelectedItem);
            else
                fromIndex = 0;
            if (proxy._shiftSelectedRecord && this._getExpandStatusRecord(proxy._shiftSelectedRecord)) {
                index = expandedRecords.indexOf(proxy._shiftSelectedRecord);
                toIndex = model.updatedRecords.indexOf(expandedRecords[index + 1]);
            }
            else {
                var index = expandedRecords.indexOf(model.updatedRecords[fromIndex]);
                toIndex = model.updatedRecords.indexOf(expandedRecords[index + 1]);
            }
            if (toIndex >= updatedRecords.length || toIndex == -1 )
                toIndex = updatedRecords.length - 1;
            proxy.updateScrollBar(toIndex);
            proxy._multiSelectShiftRequest = true;
            proxy.selectRows(fromIndex, toIndex);
            proxy._rowSelectedEventTrigger(fromIndex, toIndex, false, action);
            proxy._multiSelectShiftRequest = false;
            proxy._shiftSelectedRecord = model.updatedRecords[toIndex];
        },
        /*Select row on shift pluse up arrow key*/
        _selectedItemUp: function (action) {
            var proxy = this,
            model = this.model, shiftUpIndex, nextRowIndex,
            fromIndex, toIndex, index,
            updatedRecords = proxy.getUpdatedRecords(),
            expandedRecords = proxy.getExpandedRecords(updatedRecords);

            if (proxy._prevSelectedItem && this._getExpandStatusRecord(proxy._prevSelectedItem))
                fromIndex = model.updatedRecords.indexOf(proxy._prevSelectedItem);
            else
                fromIndex = 0;
            if (proxy._shiftSelectedRecord && this._getExpandStatusRecord(proxy._shiftSelectedRecord)) {
                index = expandedRecords.indexOf(proxy._shiftSelectedRecord);
                toIndex = model.updatedRecords.indexOf(expandedRecords[index - 1]);
            }
            else {
                var index = expandedRecords.indexOf(model.updatedRecords[fromIndex]);
                toIndex = model.updatedRecords.indexOf(expandedRecords[index - 1]);
            }
            if (toIndex < 0)
                toIndex = 0;
            proxy.updateScrollBar(toIndex);
            proxy._multiSelectShiftRequest = true;
            proxy.selectRows(fromIndex, toIndex);
            proxy._rowSelectedEventTrigger(fromIndex, toIndex, false, action);
            proxy._multiSelectShiftRequest = false;
            proxy._shiftSelectedRecord = model.updatedRecords[toIndex];
        },

        //KEY PRESSED EVENT FOR KEY BOARD INTERACTION IN TREEGRID CONTROL

        _keyPressed: function (action, target, e) {

            var proxy = this,
            model = this.model,
            lastRowIndex,
            selectingRowIndex,
            updatedRecords = proxy.getUpdatedRecords(), index, editType, str,
            expandedRecords = model.allowPaging ? updatedRecords : proxy.getExpandedRecords(model.updatedRecords), filterMenuDiv,
            currentSelectingRecord;
            this._removeDetailsRow();
            //CHECK THE ALLOWKEYBOARDNAVIGATION
            if (!model.allowKeyboardNavigation){
                return false;
            }

            if (this.model.editSettings.editMode == "dialogEditing" && $(target).closest("#" + this._id + "_dialogEdit").length)
                return true;
            var returnValue = false,
                $target = $(target),
            $form = $("#" + proxy._id + "EditForm");
            if (action !="downArrow" && action != "upArrow" && action != "saveRequest" && action != "moveCellLeft" && action != "moveCellRight"
                   && action != "cancelRequest" && $target.prop("tagName") == "INPUT") // Skip the keypress event for filter bar input box except enter
                return true;
            var headerContextMenu, headerColumnChooserMenu, isVisibleHeaderColumnChooserMenu, contextMenu, subContextMenu, columnMenu, columnChooserMenu, isVisible, isVisibleColumnChooserMenu, isVisibleFilterMenu;
            if (model.isFromGantt) {
                contextMenu = $("#" + proxy._id.replace("ejTreeGrid", "") + "_ContextMenu");
                subContextMenu = $("#" + proxy._id.replace("ejTreeGrid", "") + "_SubContextMenu");
            }
            else {
                contextMenu = $("#" + proxy._id + "_ContextMenu");
                subContextMenu = $("#" + proxy._id + "_SubContextMenu");
            }
            columnMenu = $("#" + proxy._id + "_ColumnMenu");
            columnChooserMenu = $("#" + proxy._id + "ccDiv_wrapper");
            headerContextMenu = $("#" + proxy._id + "_headerContextMenu");
            headerColumnChooserMenu = $("#" + proxy._id + "ccDiv_wrapper");
            if (proxy._columnMenuTarget) {
                var columnFieldName = $.trim(proxy._columnMenuTarget.prev("div.e-headercelldiv").attr("data-ej-mappingname")),
                column = proxy.getColumnByField(columnFieldName), filtertype;
                if (column) {
                    if (column.filterEditType) {
                        index = column.filterEditType.indexOf("edit");
                        filtertype = column.filterEditType;
                    }
                    else {
                        index = column.editType.indexOf("edit");
                        filtertype = column.editType;
                    }
                    if (index != -1) {
                        editType = filtertype.slice(0, index);
                        str = editType.charAt(0).toUpperCase() + editType.slice(1);
                        filterMenuDiv = $("#" + proxy._id + "filter" + str + "MenuDiv_wrapper");
                    } else {
                        str = filtertype.charAt(0).toUpperCase() + filtertype.slice(1);
                        filterMenuDiv = $("#" + proxy._id + "filter" + str + "MenuDiv_wrapper");
                    }
                }
                isVisibleFilterMenu = !ej.isNullOrUndefined(filterMenuDiv) && filterMenuDiv.is(":visible");
            }
            isVisible = subContextMenu.css("visibility") == "visible" ? true : false;
            isVisibleColumnChooserMenu = columnChooserMenu.is(":visible");
            isVisibleHeaderColumnChooserMenu = headerColumnChooserMenu.is(":visible");
            switch (action) {

                //save the current edited cellValue on Enter key press
                case "saveRequest":
                    if (contextMenu.length == 0 && columnMenu.length == 0 && headerContextMenu.length==0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) {
                            $target.blur();
                            proxy._endEdit();
                           proxy._focusTreeGridElement();
                        }
                        if (model.filterSettings.filterBarMode == "onEnter" && model.filterSettings.filterType == "filterbar" && model.allowFiltering && $target.closest("th").hasClass("e-filterbarcell"))
                            proxy._filterBarHandler(target);
                    }
                    else
                    {
                        if (contextMenu.length)
                        {
                            proxy._moveToNextMenuItem("save");
                        }
                        else if (headerContextMenu.length) {
                           if ($("#" + proxy._activeMenuItemId).attr("id") == "contextMenu_Columns") {
                                var option = null;
                                columnChooserMenuItem = headerColumnChooserMenu.find("div.e-columnmenuselection");
                                if (isVisibleColumnChooserMenu) {
                                    option = columnChooserMenuItem.data("column");
                                }
                                else
                                {
                                    var columnMenuItem = headerContextMenu.find("div.e-columnmenuselection");
                                    if (columnMenuItem.length > 0) {
                                        option = columnMenuItem[0].id;
                                    }
                                }
                                if (!ej.isNullOrUndefined(option)) {
                               column = proxy._contextMenuColumnDetails;
                                    switch (option) {
                                        case "contextMenu_Columns":
                                            proxy._showKeyboardColumnChooser(false, headerContextMenu, headerColumnChooserMenu, $("#contextMenu_Columns"));
                                            break;
                                        default:
                                            if (!columnChooserMenuItem.find("span").hasClass("e-disable")) {
                                                var isColumnVisible = columnChooserMenuItem.find("span").attr("aria-checked");
                                                isColumnVisible == "true" ? proxy.hideColumn(option) : proxy.showColumn(option);
                                            }
                                            break;
                                    }
                                }
                            }
                            else {
                                proxy._moveToNextMenuItem("save");
                            }
                        }

                        else if (columnMenu.length) {
                            var option = null,
                                columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection");                            
                            if (isVisibleColumnChooserMenu) {                                
                                option = columnChooserMenuItem.data("column");
                            }
                            else
                            {
                                var columnMenuItem = columnMenu.find("div.e-columnmenuselection");
                                if (columnMenuItem.length > 0) {
                                    option = columnMenuItem[0].id;
                                }
                            }
                            if (!ej.isNullOrUndefined(option))
                            {
                                $target = proxy._columnMenuTarget;
                                var columnFieldName = $.trim($target.prev("div.e-headercelldiv").attr("data-ej-mappingname")),
                                column = proxy.getColumnByField(columnFieldName);
                                switch(option)
                                {
                                    case proxy._id + "_SortAscendingChooser":
                                        
                                        if (proxy.model.allowMultiSorting) proxy._multiSortRequest = true;
                                        if (model.isFromGantt)
                                            proxy.sortColumn(column.mappingName ? column.mappingName : column.field, ej.sortOrder.Ascending);
                                        else
                                            proxy.sortColumn(column.field, ej.sortOrder.Ascending);
                                        proxy._multiSortRequest = false;
                                        break;
                                    case proxy._id + "_SortDescendingChooser":
                                       
                                        if (proxy.model.allowMultiSorting) proxy._multiSortRequest = true;
                                        if (model.isFromGantt)
                                            proxy.sortColumn(column.mappingName ? column.mappingName : column.field, ej.sortOrder.Descending);
                                        else
                                            proxy.sortColumn(column.field, ej.sortOrder.Descending);
                                        proxy._multiSortRequest = false;
                                        break;
                                    case proxy._id + "_FreezeColumnsChooser":
                                        if (column && !$(proxy._id + "_FreezeColumnChooser").hasClass("e-disable")) {
                                            if (proxy._checkboxSelection) {
                                                if (model.columns[0].nonColumn && !model.columns[0].isFrozen)
                                                    model.columns[0]['isFrozen'] = true;
                                            }
                                            proxy.freezeColumn(column.field, true);
                                        }
                                        break;
                                    case proxy._id + "_UnfreezeColumnsChooser":
                                        if (column && !$(proxy._id + "_UnfreezeColumnChooser").hasClass("e-disable")) {                                           
                                            if (proxy._checkboxSelection && proxy._frozenColumnsLength == 2) {
                                                if (model.columns[0].nonColumn && model.columns[0].isFrozen)
                                                    model.columns[0]['isFrozen'] = false;
                                            }
                                            proxy.freezeColumn(column.field, false);
                                        }
                                        break;
                                    case proxy._id + "_FreezePrecedingColumnsChooser":
                                        if (column && !$(proxy._id + "_FreezePrecedingColumnsChooser").hasClass("e-disable"))
                                            proxy.freezePrecedingColumns(column.field);
                                        break;
                                    case proxy._id + "_ColumnLeftChooser":
                                        if (column && !$(proxy._id + "_ColumnLeftChooser").hasClass("e-disable"))
                                            proxy.insertColumnChooser(column, "left");
                                        break;
                                    case proxy._id + "_ColumnRightChooser":
                                        if (column && !$(proxy._id + "_ColumnRightChooser").hasClass("e-disable"))
                                            proxy.insertColumnChooser(column, "right");
                                        break;
                                    case proxy._id + "_DeleteColumnChooser":
                                        if (column && !$(proxy._id + "_DeleteColumnChooser").hasClass("e-disable")) {
                                            proxy._targetColumnIndex = model.columns.indexOf(column);
                                            proxy._updateConfirmDialog.ejDialog("open");
                                            proxy._clearColumnMenu();
                                        }
                                        break;
                                    case proxy._id + "_RenameColumnChooser":
                                        if (column && !$(proxy._id + "_RenameColumnChooser").hasClass("e-disable")) {
                                            proxy._targetColumnIndex = model.columns.indexOf(column);
                                            proxy._renderColumnRenameDialog(column);
                                            proxy._columnRenameDialog.ejDialog("open");
                                            proxy._clearColumnMenu();
                                        }
                                        break;
                                    case proxy._id + "_FilterChooser":
                                        if (column && $("#" + proxy._id + "_ColumnMenu").find(".e-expander").hasClass("e-treegridadaptivesubicon")) {
                                            var idValue = filterMenuDiv[0].id.substring(proxy._id.length, filterMenuDiv[0].id.indexOf("MenuDiv"));
                                            proxy._showFilterMenu(null, idValue, null, null, null, null, null, null, column, null);
                                        }
                                        break;
                                    case proxy._id + "_ColumnsChooser":
                                        if ((column && $("#" + proxy._id + "_ColumnMenu").find(".e-expander").hasClass("e-treegridadaptivesubicon")) && !proxy.model.isFromGantt) {
                                            proxy._adaptiveColumnChooser();
                                        }
                                        break;
                                    default:
                                        if (!columnChooserMenuItem.find("span").hasClass("e-disable")) {
                                            var isColumnVisible = columnChooserMenuItem.find("span").attr("aria-checked");
                                            isColumnVisible == "true" ? proxy.hideColumn(option) : proxy.showColumn(option);
                                        }
                                        break;
                                }
                            }
                        }

                    }
                    break;

                    //cancel the cell editing in the current cell Esc key press
                case "cancelRequest":
                    if (contextMenu.length == 0 && columnMenu.length == 0 && headerContextMenu.length==0) {
                        if ($form.length > 0) {
                            var editMode = model.editSettings.editMode;
                            if (editMode.toLowerCase() == "cellediting" || (editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "cell")) {
                                if (proxy._isRowEdit)
                                    proxy.cancelRowEditCell();
                                else
                                    proxy.cancelEditCell();
                            }
                            else if (editMode.toLowerCase() == "rowediting" || (editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "row")) {
                                proxy.cancelRowEditCell();
                            }
                            proxy._cancelSaveTools();
                        }
                       proxy._focusTreeGridElement();
                        if (proxy._dragMouseDown == true) {
                            $(proxy._dragmouseOverTooltip).remove();
                            $(".e-dragintend").find(".e-icon").css({ "display": "none" });
                            $(".e-intendparent").find(".e-icon").css({ "display": "none" });
                            proxy._dropCancel = true;
                            proxy._dragMouseDown = false;
                            proxy._dragTooltip = false;
                            proxy._headerDragStatus = "";
                            /*hide drag icons for target row element*/
                            if (proxy._dragTargetItem && proxy._dragTargetItem.hasChildRecords && proxy._dragTargetItem.level ==0) {
                                var targetTr = ej.TreeGrid.getRowByIndex(this, updatedRecords.indexOf(proxy._dragTargetItem));
                                if (proxy._dragTargetItem)
                                    targetTr.find(".e-treegridexpand").css({ "display": "inline-block" });
                                else
                                    targetTr.find(".e-treegridcollapse").css({ "display": "inline-block" });
                            }
                            /*unbind mouse move event of row drag*/
                            proxy._off(proxy.element, "touchmove", ".e-treegridrows, .e-templatecell", proxy.dragToolTip);
                            proxy._off(proxy.element, "mousemove", ".e-gridcontent, .e-templatecell", proxy.dragToolTip);
                        }
                    }
                    else {
                        proxy._clearContextMenu();
                        proxy._clearColumnMenu();
                    }
                    break;
                case "editRecord":
                    if (model.isFromGantt && model.readOnly == true) return;
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (model.selectionSettings.selectionMode == "cell" && model.editSettings.editMode != "dialogEditing") {
                            model.editSettings.editMode = "cellEditing";
                        }
                        var editSettings = model.editSettings,
                            editMode = editSettings.editMode;
                        if ((editSettings.allowEditing && model.selectedItems.length == 1) || (editSettings.allowEditing && model.selectionSettings.selectionMode == "cell") ||
                            (editSettings.allowEditing && model.selectedItem && proxy._checkboxSelection)) {
                            if (editMode.toLowerCase() == "cellediting" || model.selectionSettings.selectionMode == "cell") {
                                var column = proxy.model.columns[proxy._cellIndex];
                                if (ej.isNullOrUndefined(column) || !column.visible || column.allowEditing == false)
                                    return;
                                if (model.selectionSettings.selectionMode === "cell")
                                    proxy._cellEditingDetails.rowIndex = this._rowIndexOfLastSelectedCell;
                                else
                                    proxy._cellEditingDetails.rowIndex = this.selectedRowIndex();
                                proxy._cellEditingDetails.columnIndex = proxy._cellIndex;//proxy.getCellIndex(e, target);
                                model.selectedItem = updatedRecords[this.selectedRowIndex()];
                                if (proxy._cellEditingDetails.columnIndex > -1) {
                                    proxy.updateScrollBar();
                                    var fieldName = column.field;
                                    if (proxy.model.editSettings.editMode == "dialogEditing")
                                        fieldName && proxy._showTreeGridAddEditDialog();
                                    else
                                        fieldName && proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName);
                                    proxy._editAddTools();
                                }
                            }
                            else if (editMode.toLowerCase() == "rowediting" && !proxy._isRowEdit && model.selectionSettings.selectionMode != "cell") {
                                var index = proxy.selectedRowIndex();
                                if (index >= 0) {                                   
                                    proxy.updateScrollBar();
                                    proxy._editRow(index);
                                }
                            } else if (editMode.toLowerCase() == "dialogediting" && model.selectionSettings.selectionMode != "cell") {
                                var index = proxy.selectedRowIndex();
                                if (index >= 0) {
                                    proxy._showTreeGridAddEditDialog(index);
                                }
                            }
                        }
                    }
                    break;
                    //Select first row of current Page if Paging is enabled
                case "topRowSelection":
                    if (model.allowPaging && contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit)
                            return true;
                        if (updatedRecords.length > 0) {
                            var topRowIndex = 0;
                            if (updatedRecords[0].isSummaryRow) {
                                var record = proxy._getNextRecord(topRowIndex +1, updatedRecords, action);
                                topRowIndex = updatedRecords.indexOf(record);
                            }
                            if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), topRowIndex, false, action)) {
                                proxy.selectRows(topRowIndex);
                                proxy._focusTreeGridElement();
                                proxy.updateScrollBar();
                                proxy._rowSelectedEventTrigger(topRowIndex, null, false, action);
                            }
                        }
                    }
                    break;
                    //Move to next page
                case "nextPage":
                    if (proxy._currentPage() < model.pageSettings.totalPages && proxy.model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root
                        && proxy._batchEditStarts && !(proxy._confirmDialog).is(":visible")) {
                        proxy._batchSaveLostWarning("Paging");
                        proxy._batchTempDetails.currentPage = proxy._currentPage() + 1;
                        return true;
                    }
                    if (model.allowPaging) {
                        var currentPage = proxy._currentPage();
                       proxy._focusTreeGridElement();
                        if (currentPage < model.pageSettings.totalPages)
                            proxy.gotoPage(currentPage + 1);
                    }
                    break;
                    //Move to previous page
                case "prevPage":
                    if (proxy._currentPage() !=1 && proxy.model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root
                        && proxy._batchEditStarts && !(proxy._confirmDialog).is(":visible")) {
                        proxy._batchSaveLostWarning("Paging");
                        proxy._batchTempDetails.currentPage = proxy._currentPage() - 1;
                        return true;
                    }
                    if (model.allowPaging) {
                        var currentPage = proxy._currentPage();
                       proxy._focusTreeGridElement();
                        if (currentPage != 1)
                            proxy.gotoPage(currentPage - 1);
                    }
                    break;
                    //Select last row of current Page if Paging is enabled
                case "bottomRowSelection":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit)
                            return true;
                        if (updatedRecords.length > 0) {

                            var bottomRowIndex = updatedRecords.length - 1;
                            if (updatedRecords[bottomRowIndex].isSummaryRow) {
                                var record = proxy._getNextRecord(bottomRowIndex - 1, updatedRecords, action);
                                bottomRowIndex = updatedRecords.indexOf(record);
                            }
                            if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), bottomRowIndex, false, action)) {
                                proxy.selectRows(bottomRowIndex);
                                proxy._focusTreeGridElement();
                                proxy.updateScrollBar();
                                proxy._rowSelectedEventTrigger(bottomRowIndex, null, false, action);
                            }
                        }

                    }
                    break;

                    //select the first row while press the Home key 
                case "firstRowSelection":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit)
                            return true;

                        if (updatedRecords.length > 0) {
                            if (model.selectionSettings.selectionMode == "row") {
                                if (model.allowPaging)
                                    proxy.gotoPage(1);
                                if (model.allowSelection && !(proxy._rowSelectingEventTrigger(this.selectedRowIndex(), 0, false, action))) {
                                    proxy.selectRows(0);
                                    proxy.updateScrollBar();
                                    proxy._focusTreeGridElement();
                                    proxy._rowSelectedEventTrigger(0, null, false, action);
                                }
                            }
                                //Select first cell of a row.
                            else if (model.selectionSettings.selectionMode == "cell") {
                                var rowIndex = proxy._focusingRowIndex,
                                    cellInfo = {
                                        rowIndex: rowIndex,
                                        cellIndex: 0
                                    };
                                proxy._isShiftKeyNavigation = false;
                                proxy._cellIndex = 0;
                                proxy.selectCells([cellInfo]);

                            }
                            proxy._cancelSaveTools();
                        }  
                    }
                    break;

                case "lastRowSelection":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit)
                            return true;
                        if (updatedRecords.length > 0) {
                            lastRowIndex = expandedRecords.length - 1;
                            currentSelectingRecord = expandedRecords[lastRowIndex];                            
                            if (currentSelectingRecord.isSummaryRow)
                                currentSelectingRecord = proxy._getNextRecord(lastRowIndex - 1, expandedRecords, action);
                            if (model.allowPaging && currentSelectingRecord) {
                                proxy.gotoPage(model.pageSettings.totalPages);
                                updatedRecords = proxy._updatedPageData;
                            }
                            selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                            if (model.selectionSettings.selectionMode == "row") {
                                if (model.allowSelection && !(proxy._rowSelectingEventTrigger(this.selectedRowIndex(), selectingRowIndex, false, action))) {
                                    proxy.selectRows(selectingRowIndex);
                                    proxy._focusTreeGridElement();
                                    proxy.updateScrollBar();
                                    proxy._rowSelectedEventTrigger(selectingRowIndex, null, false, action);
                                }
                            }
                                //Select last cell of a row.
                            else if (model.selectionSettings.selectionMode == "cell") {
                                var rowIndex = proxy._focusingRowIndex, columns = model.columns;
                                proxy._isShiftKeyNavigation = false;
                                proxy._cellIndex = proxy._getLastVisibleColumnIndex();// columns.length - 1;

                                var cellInfo = {
                                    rowIndex: rowIndex,
                                    cellIndex: proxy._cellIndex
                                };

                                proxy.selectCells([cellInfo]);
                            }
                            proxy._cancelSaveTools();
                        }
                    }
                    break;
                    //set the edit mode to previous cell by press shift +tab key
                case "moveCellLeft":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        //Row edit Element focusing for Shift + tab key press
                        if (proxy._isRowEdit) {
                            var cellIndex = $target.closest("td").prev("td").length > 0 ? $target.closest("td").prev("td")[0].cellIndex : -1;
                            if (this._frozenColumnsLength > 0 && $("#" + proxy._id + "EditForm").has($target).length > 0)
                                cellIndex = cellIndex + this._frozenColumnsLength;
                            var previousElement =cellIndex !=-1 && proxy._getPreviousElement($target, cellIndex);
                            if (previousElement.length)
                                proxy._focusElementsForRowEdit(previousElement);
                        } else if (model.editSettings.allowEditing && model.editSettings.editMode.toLowerCase() == "cellediting" && $form.length > 0 && proxy.editFormValidate()) {
                            $target.blur();
                            returnValue = proxy._moveCurrentCell("left");
                        }
                        if (model.allowSelection && model.selectionSettings.selectionMode == "cell" && !proxy._isRowEdit && (proxy._focusingRowIndex != 0 || proxy._cellIndex != 0)) {
                            if (proxy.model.isEdit) {
                                var record = model.currentViewData[proxy._cellEditingDetails.rowIndex];
                                proxy._focusingRowIndex = updatedRecords.indexOf(record);
                                proxy._cellIndex = proxy._cellEditingDetails.columnIndex;
                                var cellInfo = {
                                    rowIndex: proxy._focusingRowIndex,
                                    cellIndex: proxy._cellIndex
                                };
                                proxy.selectCells([cellInfo]);
                            }
                            else proxy._selectNextCell("left", action);
                    }
                    }
                    break;

                    //set the edit mode to next cell by tab key
                case "moveCellRight":
                        if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit) {
                                var cellIndex = $target.closest("td").next("td").length > 0 ? $target.closest("td").next("td")[0].cellIndex : -1;
                                cellIndex = cellIndex == -1 && $("#" + proxy._id + "EditFrozenForm").has($target).length > 0 ? $("#" + proxy._id + "EditForm").find('td').eq(0).index() : cellIndex;
                                if (this._frozenColumnsLength > 0 && cellIndex !=-1 && $("#" + proxy._id + "EditForm").has($target).length > 0)
                                    cellIndex = cellIndex + this._frozenColumnsLength;
                                var nextElement =cellIndex!=-1 && proxy._getNextElement($target, cellIndex);
                            if (nextElement.length)
                                proxy._focusElementsForRowEdit(nextElement);
                        } else if (model.editSettings.allowEditing && (model.editSettings.editMode == "cellEditing" || model.editSettings.editMode == "batchEditing" || model.selectionSettings.selectionMode == "cell")
                            && $target && $form.length > 0 && this.editFormValidate()) {
                            $target.blur();
                            returnValue = proxy._moveCurrentCell("right");
                        }
                        if (model.allowSelection && model.selectionSettings.selectionMode == "cell" && !proxy._isRowEdit && (proxy._focusingRowIndex != proxy.getRows().length - 1 || proxy._cellIndex != columnLength - 1)) {
                            if (proxy.model.isEdit) {
                                var record = model.currentViewData[proxy._cellEditingDetails.rowIndex];
                                proxy._focusingRowIndex = updatedRecords.indexOf(record);
                                proxy._cellIndex = proxy._cellEditingDetails.columnIndex;
                                var cellInfo = {
                                    rowIndex: proxy._focusingRowIndex,
                                    cellIndex: proxy._cellIndex
                                };
                                proxy.selectCells([cellInfo]);
                            }
                            else proxy._selectNextCell("right", action);
                    }
                    }
                    break;

                    //select the next row by press down arrow key
                case "downArrow":
                    if (model.allowSelection) {
                        if (contextMenu.length == 0 && columnMenu.length == 0 &&headerContextMenu.length==0) {
                            if (proxy._isRowEdit || proxy.model.isEdit)
                                return true;

                            if (updatedRecords.length > 0 && model.selectedItem && proxy.selectedRowIndex() > -1 && model.selectionSettings.selectionMode == "row") {
                                //get the last row index of the ganttchart control
                                lastRowIndex = updatedRecords.length - 1;
                                if (proxy.selectedRowIndex() != lastRowIndex) {
                                    var selectedItem = updatedRecords[proxy.selectedRowIndex()];
                                    selectingRowIndex = expandedRecords.indexOf(selectedItem);
                                    currentSelectingRecord = expandedRecords[selectingRowIndex + 1];
                                        //To skip the key Navigation from summary row.
                                    if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                                        currentSelectingRecord = proxy._getNextRecord(selectingRowIndex + 1, expandedRecords);
                                                                  
                                        selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                                        if (currentSelectingRecord && selectingRowIndex <= lastRowIndex) {
                                            if (model.allowSelection && !(proxy._rowSelectingEventTrigger(this.selectedRowIndex(), selectingRowIndex, false, action))) {
                                                proxy.selectRows(selectingRowIndex);
                                                proxy.updateScrollBar();
                                                proxy._focusTreeGridElement();
                                                proxy._rowSelectedEventTrigger(proxy.selectedRowIndex(), null, false, action);
                                            }
                                        }
                                }
                            proxy._cancelSaveTools();
                        }
                            else if (model.selectionSettings.selectionMode == "cell" && proxy.selectedCellIndexes().length > 0) {
                                var rowIndex = proxy._focusingRowIndex + 1, cellIndex = proxy._cellIndex,
                                    recordLength = updatedRecords.length;
                                proxy._isShiftKeyNavigation = false;
                                if (rowIndex < recordLength) {
                                    var selectedItem = updatedRecords[proxy._rowIndexOfLastSelectedCell];
                                    selectingRowIndex = expandedRecords.indexOf(selectedItem);
                                    currentSelectingRecord = expandedRecords[selectingRowIndex + 1];
                                    //To skip the key Navigation from summary row.
                                    if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                                        currentSelectingRecord = proxy._getNextRecord(selectingRowIndex + 1, expandedRecords);
                                    rowIndex = updatedRecords.indexOf(currentSelectingRecord);
                                    
                                    if (currentSelectingRecord) {
                                    cellInfo = {
                                        rowIndex: rowIndex,
                                        cellIndex: cellIndex
                                    };
                                        proxy._focusingRowIndex = rowIndex;
                                        proxy.selectCells([cellInfo]);
                                        proxy.updateScrollBar();
                                }
                            }
                        }
                        }
                        else {
                            if (contextMenu.length) {
                                proxy._moveToNextMenuItem("next");
                            }
                            else if (headerContextMenu.length) {
                               if (isVisibleHeaderColumnChooserMenu) {
                                   var headerColumnChooserMenuItem = headerColumnChooserMenu.find("div.e-columnmenuselection"),
                                      nextItem = headerColumnChooserMenuItem.next("div.e-columnMenuListDiv");
                                   if (nextItem.length > 0) {
                                       headerColumnChooserMenuItem.removeClass("e-columnmenuselection");
                                       nextItem.addClass("e-columnmenuselection");
                                   }
                               }
                               else {
                                   proxy._moveToNextMenuItem("next");
                                  
                                }
                            }
                            else if (columnMenu.length) {
                                if (isVisibleColumnChooserMenu) {
                                    var columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection"),
                                       nextItem = columnChooserMenuItem.next("div.e-columnMenuListDiv");
                                    if (nextItem.length > 0) {
                                        columnChooserMenuItem.removeClass("e-columnmenuselection");
                                        nextItem.addClass("e-columnmenuselection");
                                    }
                                }
                                else {
                                    var columnMenuItem = columnMenu.find("div.e-columnmenuselection"),
                                        nextItem = proxy._findNextColumnMenuItem(columnMenuItem, "next",true);
                                    if (nextItem.length > 0) {
                                        columnMenuItem.removeClass("e-columnmenuselection");
                                        nextItem.addClass("e-columnmenuselection");
                                        proxy._focusTreeGridElement();
                                        proxy._isCloseFilterMenu = true;
                                        proxy._clearFilterMenu();
                                    }
                                }
                            }
                        }
					}
                    break;
                // expand the child task by press right arrow
                case "rightArrow":
					if (model.allowSelection) {
                    if (contextMenu.length == 0 && columnMenu.length == 0 &&headerContextMenu.length==0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) return true;
                        var rowIndex = proxy.selectedRowIndex(),
                        record = updatedRecords[rowIndex],
                        args = {},
                        isExpandCollapseEnabeled;
                        args.data = record;
                        args.recordIndex = rowIndex;
                        args.expanded = true;
                        if (record) {
                            if (record.hasChildRecords && !record.expanded) {
                                if (proxy.selectedRowIndex() >= 0) {
                                    isExpandCollapseEnabeled = proxy._trigger("expanding", args);
                                    if (!isExpandCollapseEnabeled && !model.isFromGantt) {
                                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                                    }
                                    if (model.enableVirtualization)
                                       proxy._focusTreeGridElement();
                                    proxy.updateScrollBar();
                                }
                            }
                        }
                        proxy._cancelSaveTools();
                        if (model.allowSelection && model.selectionSettings.selectionMode == "cell" && (proxy._focusingRowIndex != proxy.getRows().length -1 || proxy._cellIndex != columnLength - 1)) {
                            proxy._selectNextCell("right", action);
                    }
                    }
                    else
                    {
                        if (contextMenu.length && !isVisible) {
                            proxy._moveToNextMenuItem("expand");
                        }
                        else if (headerContextMenu.length && !isVisibleHeaderColumnChooserMenu) {
                            proxy._moveToNextMenuItem("expand");
                            proxy._showKeyboardColumnChooser(false, headerContextMenu, headerColumnChooserMenu, $("#contextMenu_Columns"));
                        }
                        else if (columnMenu.length && !isVisibleColumnChooserMenu) {
                            proxy._showKeyboardColumnChooser(true, columnMenu, columnChooserMenu, $("#" + proxy._id + "_ColumnsChooser"));
                            var columnMenuItem = columnMenu.find("div.e-columnmenuselection"),
                             columnChooserMenuList = columnChooserMenu.find("div.e-columnMenuListDiv"),
                              columnListPosX = proxy._columnListPosX,
                                columnFilterPosX = proxy._columnFilterPosX,
                                columnChooserListIndex = proxy._columnChooserListIndex,
                                windowWidth = $(document).width();
                                if (columnMenuItem.length > 0 && columnMenuItem[0].id == proxy._id + "_FilterChooser" && !$("#" + proxy._id + "_ColumnMenu").find(".e-expander").hasClass("e-treegridadaptivesubicon")) {
                                if (model.filterSettings.filterType == "menu" && column.filterType != "excel") {
                                    var sliceFilterMenuDivID = filterMenuDiv[0].id.indexOf("_"),
                                        filterMenuDivID = "#" + filterMenuDiv[0].id.slice(0, sliceFilterMenuDivID),
                                        filterMenuInstance = $(filterMenuDivID).ejDialog("instance"), filterMenuPosition, filterMenuIndex;
                                        var proxy = this;
                                    var evntArgs = {};
                                    evntArgs.width = "auto",
                                    evntArgs.height = "auto",
                                    evntArgs.minWidth = "0px",
                                    evntArgs.minHeight = "0px",
                                    evntArgs.showHeader = false,
                                    evntArgs.position = { X: 1, Y: 1 },
                                    evntArgs.enableModal = false,
                                    evntArgs.isResponsive = false,
                                    evntArgs.enableResize = false,
                                    $(filterMenuDivID + "_wrapper").removeClass("e-dialog").removeClass("e-shadow").find("div.e-dialog-scroller").css("height", "100%").removeClass("e-widget-content");
                                $(filterMenuDivID).ejDialog(evntArgs);

                                    if (windowWidth < columnFilterPosX + $("#" + filterMenuDiv[0].id).width()) {
                                        columnFilterPosX = columnFilterPosX - columnMenu.width() - $("#" + filterMenuDiv[0].id).width();
                                    }
                                    filterMenuIndex = columnMenu.find("div.e-columnmenuselection").closest("li").index();
                                    filterMenuPosition = {
                                        X: columnFilterPosX,
                                        Y: proxy._columnMenuPopupY + $("#" + proxy._id + "_FilterChooser").position().top
                                    };
                                    filterMenuInstance.option({ "position": filterMenuPosition });
                                    filterMenuInstance.open();
                                    proxy._updatePreviousFilteredValue(column);
                                    filterMenuDiv.css("z-index", proxy.getMaxZIndex() + 1);
                                    proxy._focusTreeGridElement();
                                    proxy._renderedSubMenuType = filterMenuDivID + "Desktop";
                                }
                                else
                                {
                                    var element = !model.isFromGantt ? $(this.element) : $(this.element).closest(".e-gantt"),
                                        containerOffset = this.getOffsetRect(element[0]);
                                    containerOffset.right = containerOffset.left + $(element).width();
                                    proxy._openExcelFilter(column, columnFilterPosX, proxy._columnMenuPopupY, containerOffset, columnMenu.width());
                                }
                            }
                        }
                    }
					}
                    break;
				
                //collapse the child task by press left arrow 
                case "leftArrow":
					if (model.allowSelection) {
                    if (contextMenu.length == 0 && columnMenu.length == 0 && headerContextMenu.length==0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) return true;
                        var rowIndex = proxy.selectedRowIndex(),
                        record = updatedRecords[rowIndex],
                        args = {},
                        isExpandCollapseEnabeled;
                        args.data = record;
                        args.recordIndex = rowIndex;
                        args.expanded = false;
                        if (record) {
                            if (record.hasChildRecords && record.expanded) {
                                if (proxy.selectedRowIndex() >= 0) {
                                    isExpandCollapseEnabeled = proxy._trigger("collapsing", args);
                                    if (!isExpandCollapseEnabeled && !model.isFromGantt) {
                                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                                    }
                                    if (model.enableVirtualization)
                                       proxy._focusTreeGridElement();
                                    proxy.updateScrollBar();
                                }
                            }
                        }
                        proxy._cancelSaveTools();
                        if (model.allowSelection && model.selectionSettings.selectionMode == "cell" && (proxy._focusingRowIndex != 0 || proxy._cellIndex != 0)) {
                            proxy._selectNextCell("left", action);
                    }
                    }
                    else if (contextMenu.length)
                        proxy._moveToNextMenuItem("collapse");
                    else if (headerContextMenu.length) {
                        proxy._moveToNextMenuItem("collapse");
                        $("#" + proxy._id + "ccDiv").ejDialog("close");
                    }
                    else if (columnMenu.length && (isVisibleColumnChooserMenu || isVisibleFilterMenu)) {
                        $("#" + proxy._id + "ccDiv").ejDialog("close");
                        proxy._isCloseFilterMenu = true;
                        proxy._clearFilterMenu();
                    }
					}
                    break;
				
                    //select the previous row by press down arrow key
                case "upArrow":
					if (model.allowSelection) {
					    if (contextMenu.length == 0 && columnMenu.length == 0 &&headerContextMenu.length==0) {
					       if (proxy._isRowEdit || proxy.model.isEdit)
                                return true;

                            if (model.selectionSettings.selectionMode == "row" && this.selectedRowIndex() > 0 && updatedRecords.length > 0 && model.selectedItem) {
                            var selectedItem = updatedRecords[proxy.selectedRowIndex()];
                            selectingRowIndex = expandedRecords.indexOf(selectedItem);
                            currentSelectingRecord = expandedRecords[selectingRowIndex - 1];
                                if (currentSelectingRecord.isSummaryRow)
                                    currentSelectingRecord = proxy._getNextRecord(selectingRowIndex - 1, expandedRecords, action);
                                selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);

                                if (selectingRowIndex != -1 && !proxy._rowSelectingEventTrigger(this.selectedRowIndex(), selectingRowIndex, false, action)) {
                                    proxy.selectRows(selectingRowIndex);
                                    proxy.updateScrollBar();
                                    proxy._focusTreeGridElement();
                                    proxy._rowSelectedEventTrigger(proxy.selectedRowIndex(), null, false, action);
                                }
                            proxy._cancelSaveTools();
                        }
                            else if (model.selectionSettings.selectionMode == "cell") {
                                var rowIndex = proxy._focusingRowIndex - 1, cellIndex = proxy._cellIndex,
                                   recordLength = model.flatRecords.length;
                                proxy._isShiftKeyNavigation = false;
                                if (rowIndex != -1) {
                                    var selectedItem = updatedRecords[proxy._rowIndexOfLastSelectedCell];
                                    selectingRowIndex = expandedRecords.indexOf(selectedItem);
                                    currentSelectingRecord = expandedRecords[selectingRowIndex - 1];
                                    //To skip the key Navigation from summary row.
                                    if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                                        currentSelectingRecord = proxy._getNextRecord(selectingRowIndex - 1, expandedRecords, action);
                                    rowIndex = updatedRecords.indexOf(currentSelectingRecord);
                    
                                    if (currentSelectingRecord) {
                                   cellInfo = {
                                       rowIndex: rowIndex,
                                       cellIndex: cellIndex
                                   };
                                        proxy._focusingRowIndex = rowIndex;
                                    proxy.selectCells([cellInfo]);
                                        proxy.updateScrollBar();
                                    }

                    }
                            }
                        }
                        else {
                        if (contextMenu.length) {
                            proxy._moveToNextMenuItem("prev");
                        }
                        else if (headerContextMenu.length) {
                            if (isVisibleHeaderColumnChooserMenu) {
                                var headerColumnChooserMenuItem = headerColumnChooserMenu.find("div.e-columnmenuselection"),
                                   prevItem = headerColumnChooserMenuItem.prev("div.e-columnMenuListDiv");
                                if (prevItem.length > 0) {
                                    headerColumnChooserMenuItem.removeClass("e-columnmenuselection");
                                    prevItem.addClass("e-columnmenuselection");
                                }
                            }
                            else {
                                proxy._moveToNextMenuItem("prev");
                            }
                        }
                        else if (columnMenu.length) {
                            if (isVisibleColumnChooserMenu) {
                                var columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection"),
                                  prevItem = columnChooserMenuItem.prev("div.e-columnMenuListDiv");
                                if (prevItem.length > 0) {
                                    columnChooserMenuItem.removeClass("e-columnmenuselection");
                                    prevItem.addClass("e-columnmenuselection");
                                }
                            }
                            else {
                                var columnMenuItem = columnMenu.find("div.e-columnmenuselection"),
                                    prevItem = proxy._findNextColumnMenuItem(columnMenuItem, "prev",true);
                                if (prevItem.length > 0) {
                                    columnMenuItem.removeClass("e-columnmenuselection");
                                    prevItem.addClass("e-columnmenuselection");
                                    proxy._isCloseFilterMenu = true;
                                    proxy._clearFilterMenu();
                                }
                            }
                        }
                    }
					}
                    break;
                case "deleteRecord":
                    if (model.isFromGantt && model.readOnly == true) return;
                    if (model.editSettings.allowDeleting && contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) return true;
                        if (proxy.selectedRowIndex() >= 0) {
                            proxy.deleteRow();
                            proxy._cancelSaveTools();
                           proxy._focusTreeGridElement();
                        }
                    }
                    break;
                case "totalRowCollapse":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy.model.isFromGantt) {
                            args = {};
                            args.requestType = "collapseAll";
                            proxy._trigger("expandAllCollapseAllRequest", args);
                        }
                        else
                            proxy.collapseAll();
                        break;
                    }
                case "totalRowExpand":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy.model.isFromGantt) {
                            args = {};
                            args.requestType = "expandAll";
                            proxy._trigger("expandAllCollapseAllRequest", args);
                        }
                        else
                            proxy.expandAll();
                    }
                    break;
                case "spaceBar":
                var option = null;
                if (columnMenu.length && isVisibleColumnChooserMenu) {
                    var columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection");
                    option = columnChooserMenuItem.data("column");
                    if (!ej.isNullOrUndefined(option) && !columnChooserMenuItem.find("span").hasClass("e-disable")) {
                        var isColumnVisible = columnChooserMenuItem.find("span").attr("aria-checked");
                        isColumnVisible == "true" ? proxy.hideColumn(option) : proxy.showColumn(option);
                    }
                }
                if (proxy._checkboxSelection && $target.hasClass('e-treegrid') && $("#" + proxy._id + "EditForm").length == 0) {
                    if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), this.selectedRowIndex(), true, action)) {
                        proxy.selectRows(this.selectedRowIndex(), null, true);
                        proxy._cancelSaveTools();
                        proxy._rowSelectedEventTrigger(this.selectedRowIndex(), null, true, action);
                    }
                }
                break;
                case "shiftDownArrow":
                    if (model.allowSelection && model.selectionSettings.selectionType == "multiple") {
                        if (model.selectionSettings.selectionMode == "cell") {
                            var recordLength = updatedRecords.length,
                                selectedCellElementLength = proxy._selectedCellDetails.length,
                                nextRowIndex;

                            var selectedItem = updatedRecords[proxy._rowIndexOfLastSelectedCell];
                            selectingRowIndex = expandedRecords.indexOf(selectedItem);
                            currentSelectingRecord = expandedRecords[selectingRowIndex + 1];
                            //To skip the key Navigation from summary row.
                            if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                                currentSelectingRecord = proxy._getNextRecord(selectingRowIndex + 1, expandedRecords);
                            if (!currentSelectingRecord)
                                return;
                            nextRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                            if (proxy._focusingRowIndex > proxy._rowIndexOfLastSelectedCell) {
                                proxy._rowIndexOfLastSelectedCell = nextRowIndex;
                                proxy._selectedCellDetails.sort(function (a, b) {
                                    return parseFloat(a.rowIndex) - parseFloat(b.rowIndex);
                                });
                                proxy._getSelectedCellsinARow(proxy._selectedCellDetails[0].data.index, proxy._selectedCellDetails);
                                for (var i = 0; i < proxy._selectedCellsinARow.length; i++) {
                                    var selectedCellIndex = proxy._selectedCellDetails.indexOf(proxy._selectedCellsinARow[i]);
                                    $(proxy._selectedCellsinARow[i].cellElement).removeClass("e-selectingcell");
                                    proxy._selectedCellDetails.splice(selectedCellIndex, 1);
                                }
                                proxy.updateScrollBar();
                            }
                            else if (nextRowIndex < recordLength) {
                                var args = {
                                    rowIndex: nextRowIndex,
                                    cellIndex: proxy._lastSelectedCellIndex,
                                }
                                proxy.updateScrollBar();
                                proxy._shiftKeySelectCells(args);
                                proxy._rowIndexOfLastSelectedCell = nextRowIndex;
                            }
                        }
                        else {
                            proxy._selectedItemDown(action);
                        }
                    }
                    break;
                case "shiftUpArrow":
                    if (model.allowSelection && model.selectionSettings.selectionType == "multiple") {
                        if (model.selectionSettings.selectionMode == "cell") {
                            var selectedCellElementLength = proxy._selectedCellDetails.length;
                            var selectedItem = updatedRecords[proxy._rowIndexOfLastSelectedCell];
                            selectingRowIndex = expandedRecords.indexOf(selectedItem);
                            currentSelectingRecord = expandedRecords[selectingRowIndex - 1];
                            //To skip the key Navigation from summary row.
                            if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                                currentSelectingRecord = proxy._getNextRecord(selectingRowIndex - 1, expandedRecords, "upArrow");
                            if (!currentSelectingRecord)
                                return;
                            nextRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                            if (selectedCellElementLength > 0 && proxy._focusingRowIndex < proxy._rowIndexOfLastSelectedCell) {
                                proxy._rowIndexOfLastSelectedCell = nextRowIndex;
                                proxy._getSelectedCellsinARow(proxy._selectedCellDetails[selectedCellElementLength - 1].data.index, proxy._selectedCellDetails);
                                for (var i = 0; i < proxy._selectedCellsinARow.length; i++) {
                                    var selectedCellIndex = proxy._selectedCellDetails.indexOf(proxy._selectedCellsinARow[i]);
                                    $(proxy._selectedCellsinARow[i].cellElement).removeClass("e-selectingcell");
                                    proxy._selectedCellDetails.splice(selectedCellIndex, 1);
                                }
                                proxy.updateScrollBar();
                            }
                            else if (nextRowIndex >= 0) {
                                var args = {
                                    rowIndex: nextRowIndex,
                                    cellIndex: proxy._lastSelectedCellIndex,
                                }
                                proxy._rowIndexOfLastSelectedCell = nextRowIndex;
                                proxy.updateScrollBar();
                                proxy._shiftKeySelectCells(args);
                            }
                        }
                        else
                            proxy._selectedItemUp(action);
                    }
                    break;

                case "shiftRightArrow":
                    if (model.allowSelection && model.selectionSettings.selectionMode == "cell" && model.selectionSettings.selectionType == "multiple") {
                        var columnLength = model.columns.length, cellIndex = proxy._lastSelectedCellIndex + 1;
                        if (cellIndex < columnLength - 1 && (!model.columns[cellIndex].visible || !model.columns[cellIndex].allowCellSelection))
                            cellIndex = proxy.getUpNextVisibleColumnIndex(cellIndex);
                        if (cellIndex < columnLength) {

                            var args = {
                                rowIndex: proxy._rowIndexOfLastSelectedCell,
                                cellIndex: cellIndex,
                            }
                            proxy._shiftKeyFirstElementDetails.firstElementRowIndex = proxy._focusingRowIndex;
                            proxy._shiftKeySelectCells(args);
                            proxy._lastSelectedCellIndex = cellIndex;                            
                        }
                    }
                    //proxy.updateScrollBar(proxy._rowIndexOfLastSelectedCell);                    
                    break;
                case "shiftLeftArrow":
                    if (model.allowSelection && model.selectionSettings.selectionMode == "cell" && model.selectionSettings.selectionType == "multiple") {
                        var cellIndex = proxy._lastSelectedCellIndex - 1;
                        if (cellIndex != -1 && (!model.columns[cellIndex].visible || !model.columns[cellIndex].allowCellSelection))
                            cellIndex = proxy.getPreviousVisibleColumnIndex(cellIndex);
                        var args = {
                            rowIndex: proxy._rowIndexOfLastSelectedCell,
                            cellIndex: cellIndex,
                        }
                        if (cellIndex != -1) {
                            proxy._shiftKeyFirstElementDetails.firstElementRowIndex = proxy._focusingRowIndex;
                            proxy._shiftKeySelectCells(args);
                            proxy._lastSelectedCellIndex = cellIndex;
                        }
                    }
                    //proxy.updateScrollBar(targetIndex);                    
                    break;

                case "shiftHomeButton":
                    if (model.selectionSettings.selectionMode == "cell" && model.selectionSettings.selectionType == "multiple") {
                        var rowIndex = proxy._rowIndexOfLastSelectedCell,
                           args = {
                               rowIndex: rowIndex,
                               cellIndex: 0
                           };
                        proxy._cellIndex = 0;
                        proxy._lastSelectedCellIndex = 0;
                        proxy._shiftKeySelectCells(args);

                    }
                    break;
                case "shiftEndButton":
                    if (model.selectionSettings.selectionMode == "cell" && model.selectionSettings.selectionType == "multiple") {
                        var rowIndex = proxy._rowIndexOfLastSelectedCell, columns = model.columns;
                        proxy._cellIndex = columns.length - 1;
                        args = {
                            rowIndex: rowIndex,
                            cellIndex: proxy._cellIndex
                        };
                        proxy._lastSelectedCellIndex = proxy._cellIndex;
                        proxy._shiftKeySelectCells(args);

                    }
                    break;
                default:
                    returnValue = true;
            }

            return returnValue;
        },

        //keyboardcolumnchooser


        _showKeyboardColumnChooser: function (isFromColumnMenu,columnMenu, columnChooserMenu,columnId) {
            var proxy = this,
               model = proxy.model;
            var evntArgs = {};
            evntArgs.width = "auto";
            evntArgs.height = "auto";
            evntArgs.minWidth = "0px";
            evntArgs.minHeight = "0px";
            evntArgs.showHeader = false;
            evntArgs.position = { X: 1, Y: 1 };
            evntArgs.enableModal = false;
            evntArgs.isResponsive = false;
            evntArgs.enableResize = false;
            evntArgs.cssClass = model.cssClass;
            $("#" + proxy._id + "ccDiv_wrapper").addClass("e-treegridadaptivedialog");
            $("#" + proxy._id + "ccDiv_wrapper").removeClass("e-dialog").removeClass("e-shadow").find("div.e-dialog-scroller").css("height", "100%").removeClass("e-widget-content");
            $("#" + proxy._id + "ccDiv").ejDialog(evntArgs);
            $("#" + proxy._id + "ccDiv").css({ 'height': 'auto' });
            var columnMenuItem = columnMenu.find("div.e-columnmenuselection"),
                 columnChooserMenuList = columnChooserMenu.find("div.e-columnMenuListDiv"),
                   columnListPosX = proxy._columnListPosX,
                     columnMenuPopupY = proxy._columnMenuPopupY,
                      windowWidth = $(document).width(),
                      columnMenuWidth = isFromColumnMenu ? columnMenu.width() : proxy._headerContextMenuWidth;
            proxy._updateColumnMenuVisibility();
            if (columnMenuItem.length > 0 && columnMenuItem[0].id == columnId[0].id && !columnMenu.find(".e-expander").hasClass("e-treegridadaptivesubicon")) {
                var columnchooser = $("#" + proxy._id + "ccDiv").ejDialog("instance"),
                    columnListWidth = $("#" + proxy._id + "ccDiv_wrapper").width();
                columnchooser.open();
                if (windowWidth < columnListPosX + columnMenu.width()) {
                    columnListPosX = columnListPosX - columnMenuWidth - columnListWidth;
                }
                var position = {
                    X: columnListPosX,
                    Y: columnMenuPopupY + columnId.position().top
                };
                columnchooser.option({ "position": position });
                columnChooserMenu.css("z-index", proxy.getMaxZIndex() + 1);
                columnChooserMenuList.removeClass("e-columnmenuselection");
                columnChooserMenuList.eq(0).addClass("e-columnmenuselection");
                proxy._focusTreeGridElement();
                proxy._renderedSubMenuType = "ColumnDesktop";
            }
            else {
                return false;
            }
        },


        _getNextElement: function ($target, cellIndex) {
            var proxy = this,
                model = proxy.model,
                columns = model.columns;
            $target = $target.closest("td").next("td");
            if (this._frozenColumnsLength > 0 && $target.length == 0) {
                $target = $("#" + proxy._id + "EditForm").find('td').eq(0);
                cellIndex = this._frozenColumnsLength;
            }
            if ($target.length != 0) {
                if (!ej.isNullOrUndefined(columns[cellIndex]) && ej.isNullOrUndefined(columns[cellIndex].allowEditing))
                    return $target;
                else {
                    if (cellIndex + 1 < columns.length)
                        return proxy._getNextElement($target, cellIndex + 1);
                    else
                        return false;
                }
            } else {
                return false;
            }
        },

        _getPreviousElement: function ($target, cellIndex) {
            var proxy = this,
                model = proxy.model,
                columns = model.columns;
            $target = $target.closest("td").prev("td");
            if (this._frozenColumnsLength > 0 && $target.length == 0)
                $target = $("#" + proxy._id + "EditFrozenForm").find('td').last();            

            if ($target.length != 0 && cellIndex > -1) {
                if ((!ej.isNullOrUndefined(columns[cellIndex]) && ej.isNullOrUndefined(columns[cellIndex].allowEditing)) || columns[cellIndex].allowEditing)
                    return $target;
                else
                    return proxy._getPreviousElement($target, cellIndex - 1);
            } else {
                return false;
            }
        },
        _clearAllFilter: function (fieldName) {
            var proxy = this, column = proxy.model.filterSettings.filteredColumns[0];
            proxy.model.allowFiltering = true;
            proxy.model.filterSettings.filteredColumns = [];
            proxy.filterColumn(column.field, column.operator, "", "and");
            proxy._filteredColumnValue = [];
            proxy._filteredColumnValueID = [];
        },

        //Method to bind all the filter condition and splice the over writed condition.
        filterColumn: function (fieldName, filterOperator, filterValue, predicate, matchcase, actualFilterValue) {
           
            var proxy = this,
                model = proxy.model,
                filteredColumns = model.filterSettings.filteredColumns;
            if (!this.model.allowFiltering)
                return;
            if (proxy._batchEditStarts && !(proxy._confirmDialog).is(":visible")) {
                proxy._batchSaveLostWarning("Filtering");
                var details = proxy._batchTempDetails;
                details["fieldName"] = fieldName, details["filterOperator"] = filterOperator, details["filterValue"] = filterValue,
                details["predicate"] = predicate, details["matchcase"] = matchcase, details["actualFilterValue"] = actualFilterValue;                
                return false;
            }
            var args = {};
            args.requestType = ej.TreeGrid.Actions.Filter;
            args.currentFilterObject = [];           
            if (!Array.isArray(filterOperator))
                filterOperator = $.makeArray(filterOperator);
            if (!Array.isArray(filterValue))
                filterValue = $.makeArray(filterValue);
            var firstLoop = false;
            var filterCol = this._filterCollection;
            if (ej.util.isNullOrUndefined(this._currentFilterColumn))
                this._currentFilterColumn = this.getColumnByField(fieldName);
            for (var index = 0; index < filterOperator.length; index++) {
                    var filterObject = {
                        field: fieldName,
                        operator: filterOperator[index],
                        value: filterValue[index],
                        matchcase: matchcase,
                        predicate: predicate,
                        actualFilterValue: actualFilterValue
                    };
                this._$colType = "string";
                if (filteredColumns.length == 0 && filterObject.value !== "") {
                    if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                        this._setDateFilters(filterObject);
                    else
                        filteredColumns.push(filterObject);
                } else {
                    var proxy = this;
                    if (!firstLoop) {
                        var dataManger = ej.DataManager(filteredColumns);
                        var query = new ej.Query().where("field", ej.FilterOperators.equal, filterObject.field);
                        var object = dataManger.executeLocal(query);
                        for (var i = 0; i < object.length; i++) {
                            var objectIndex = $.inArray(object[i], filteredColumns)
                            if (objectIndex != -1)
                                filteredColumns.splice(objectIndex, 1);
                        }
                    }
                    if (filterObject.value !== "") {
                        if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                            this._setDateFilters(filterObject);
                        else
                            filteredColumns.push(filterObject);
                    }
                }
                firstLoop = true;
                args.currentFilterObject.push(filterObject);
            }
            args.filterCollection = filteredColumns;
            args.currentFilteringColumn = fieldName;
            args.fieldName = fieldName;
            var returnValue = this.processBindings(args);
            if (returnValue) {
                filteredColumns.reverse().splice(0, filterOperator.length);
                filteredColumns.reverse();
            }
            proxy._setFilterFieldValues(filterObject);
        },

        filterContent: function (filterObject) {
            var proxy = this,
                model = proxy.model,
                filteredColumns = model.filterSettings.filteredColumns,
                index;

            if (!(filterObject instanceof ej.Predicate))
                return false;
            var index = filteredColumns.indexOf(filterObject);
            if (index != -1) {
                filteredColumns.splice(index, 1);
            }
            else {
                var predicateArray = [], filterFields = [];
                predicateArray.push(filterObject.toJSON());
                proxy._addPredicateObject(predicateArray, filterFields);
                ej.merge(filteredColumns, filterFields);
            }
            var args = { requestType: "filtering" };
            proxy.processBindings(args);
            proxy._setFilterFieldValues(filterFields);
        },

        _addPredicateObject: function (filterObject, filteredColumns, condition) {
            var length = filterObject.length;

            for (var i = 0; i < length; i++) {
                var predicate = filterObject[i];
                if (predicate.isComplex) {
                    if (!ej.isNullOrUndefined(predicate.predicates))
                        this._addPredicateObject(predicate.predicates, filteredColumns, predicate.condition);
                }
                else {
                    var arg = {};
                    arg.field = predicate.field;
                    arg.operator = predicate.operator;
                    arg.value = predicate.value;
                    arg.matchcase = !predicate.ignoreCase;
                    arg.predicate = condition;

                    filteredColumns.push(arg);
                }
            }
        },

        _setFilterFieldValues: function (filteredField) {
            var proxy = this,
                model = proxy.model,
                filterType = model.filterSettings.filterType,
                column, filterColumns = [], length = 0;

            if (ej.isNullOrUndefined(filteredField.length))
                filterColumns.push(filteredField);
            else
                filterColumns = filteredField;

            length = filterColumns.length;
            for (var i = 0; i < length; i++) {
                var filterObject = filterColumns[i];
                column = proxy.getColumnByField(filterObject.field);
                if (!column)
                    continue;
                if (filterType == "menu" || filterType == "excel") {
                    if (!proxy.model.showColumnChooser) {
                        var $filterIcon = proxy.getHeaderTable().find("div[data-ej-mappingname='" + filterObject.field + "']").next(".e-filtericon");
                        if (filterObject.value.toString() != "" || filterObject.value.toString() == "false")
                            $filterIcon.removeClass("e-column-filter-icon").addClass("e-column-filtered-icon");
                        else
                            $filterIcon.removeClass("e-column-filtered-icon").addClass("e-column-filter-icon");
                    }
                    if (filterType == "menu") {
                        proxy._storePreviousFilteredValue(filterObject.field, filterObject.value, filterObject.operator);
                        proxy._filterString = filterObject.value;
                    }
                    else
                        proxy._filteredColumnValueID.push(filterObject.field);
                }
                else if (filterType == "filterbar") {
                    var filterBarCell = proxy.getHeaderTable().find("#" + proxy._id + "_" + filterObject.field.split('.').join("") + "_filterbarcell");
                    if (column.filterEditType == "numericedit")
                        $(filterBarCell).ejNumericTextbox({ value: filterObject.value });
                    else if (column.filterEditType == "datepicker")
                        $(filterBarCell).ejDatePicker({ value: filterObject.value });
                    else if (column.filterEditType == "datetimepicker")
                        $(filterBarCell).ejDateTimePicker({ value: filterObject.value });
                    else if (column.filterEditType == "dropdownedit")
                        $(filterBarCell).ejDropDownList({ value: filterObject.value });
                    else {
                        if (filterBarCell.val() == "" || (filterBarCell.val() != filterObject.value)) {
                            filterBarCell.val(filterObject.value);
                        }
                    }
                }
            }
        },

        _cellSelectingEventTrigger: function (selectedCellInfo) {
            var proxy = this,
               model = proxy.model,
               args = {}, targetIndex = selectedCellInfo.rowIndex,
               currentCellIndex = selectedCellInfo.cellIndex,
               updatedRecords = proxy.getUpdatedRecords();
            args.data = updatedRecords[targetIndex];
            args.targetRow = selectedCellInfo.rowElement;
            args.targetCell = selectedCellInfo.cellElement;
            args.rowIndex = targetIndex;
            args.cellIndex = currentCellIndex;
            return proxy._trigger('cellSelecting', args);
        },
        _cellSelectedEventTrigger: function (selectedCellInfo, previousSelectedCellDetails) {
            var proxy = this,
                model = proxy.model,
                args = {}, targetIndex = selectedCellInfo.rowIndex,
                currentCellIndex = selectedCellInfo.cellIndex,
                previousRowIndex = previousSelectedCellDetails.rowIndex,
                previousCellIndex = previousSelectedCellDetails.cellIndex,
                updatedRecords = proxy.getUpdatedRecords();

            //previousSelectedCellDetails
            args.PreviousTargetRow = previousSelectedCellDetails.rowElement;
            args.previousTargetCell = previousSelectedCellDetails.cellElement;
            args.previousData = updatedRecords[previousRowIndex];
            args.previousRowIndex = previousRowIndex;
            args.previousCellIndex = previousCellIndex;
            //currentSelectedCellDetails
            args.data = updatedRecords[targetIndex];           
            args.targetRow = selectedCellInfo.rowElement;
            args.targetCell = selectedCellInfo.cellElement;
            args.rowIndex = targetIndex;
            args.cellIndex = currentCellIndex;
            return proxy._trigger('cellSelected', args);
        },

        //ROWSELECTING EVENT METHOD
        _rowSelectingEventTrigger: function (previousIndex, currentIndex, isCheckboxUpdate, actionType) {

            var proxy = this,
                model = proxy.model,
                args = {},
                selectedItem,
                $gridRows,
                updatedRecords = proxy.getUpdatedRecords();
            if (!isCheckboxUpdate)
                isCheckboxUpdate = false;
                args.targetRow = ej.TreeGrid.getRowByIndex(proxy, currentIndex);
                args.previousTreeGridRow = ej.TreeGrid.getRowByIndex(proxy, previousIndex);
                args.data = updatedRecords[currentIndex];
                args.previousData = updatedRecords[previousIndex];
                args.recordIndex = currentIndex;
                args.previousIndex = previousIndex;
                args.actionType = !ej.isNullOrUndefined(actionType) ? actionType : null;
                if (proxy.model.isFromGantt) {
                    args.target = "ejTreeGrid";
                } else {
                    args.isCheckboxUpdate = isCheckboxUpdate;
                }
                var isCancel = proxy._trigger('rowSelecting', args);
				if (isCancel)
                    model.selectedItem = args.data;
                return isCancel;
        },


        //ROWSELECTED EVENT METHOD
        _rowSelectedEventTrigger: function (index, toIndex, isCheckboxUpdate, actionType) {

            var proxy = this, model = proxy.model,
                args = {}, updatedRecords = proxy.getUpdatedRecords();
            var record = $.extend({}, updatedRecords[index]);
            /*Remove childRecords and parentItem from record object, 
              To avoid the cyclic dependency when performing postback action in ASP.NET */
            delete record.childRecords;
            delete record.parentItem;

            args = {
                targetRow: ej.TreeGrid.getRowByIndex(proxy, index),
                recordIndex: index
            };
            args.actionType = !ej.isNullOrUndefined(actionType) ? actionType : null;
            if (!isCheckboxUpdate)
                isCheckboxUpdate = false;
            if (!ej.isNullOrUndefined(toIndex)) args.toIndex = toIndex;
            if (proxy.model.isFromGantt) {
                args.target = "ejTreeGrid";
                args.data = model.updatedRecords[index];
                args.selectedItems = model.selectedItems;
            }
            else {
                args.data = record;
                args.isCheckboxUpdate = isCheckboxUpdate;
            }

                proxy._trigger("rowSelected", args);
        },


        //QUERYCELLINFO,ROWDATABOUND EVENT BINDING FOR TREEGRID
        _eventBindings: function (data) {

            var proxy = this,
                model = proxy.model,
                count = 0,
                row, updatedRecords = proxy.getUpdatedRecords(),
                gridRows = proxy.getRows(),
                length;
            
            if (ej.isNullOrUndefined(data)) {
                length = this._frozenColumnsLength > 0 ? $(gridRows[0]).length : $(gridRows).length;
                data = model.currentViewData;
            }
            else
                length = data.length;
            if (proxy._commandColumnLength > 0) {
                this._refreshUnboundTemplate(this.getContentTable());
            }
            if (model.queryCellInfo != null || model.rowDataBound != null || model.detailsDataBound) {

                for (count; count < length; count++) {
                    var rowIndex = updatedRecords.indexOf(data[count]);
                    row = ej.TreeGrid.getRowByIndex(this, rowIndex);
                    proxy._rowEventTrigger(row,data[count]);
                }
            }
        },

        //ROWDATABOUND EVENT TRIGGER
        _rowEventTrigger: function (row, data) {

            var proxy = this,
                model = this.model;

            if (model.rowDataBound) {
                proxy._trigger("rowDataBound", {
                    rowElement: row,
                    data: data
                });
            }
           
            var tdCells = row.cells,
                $tdRowcells = $(row).find(".e-rowcell"),
                cellIndex = 0,
                length = $tdRowcells.length,
                column=null,
                columns = proxy.model.columns;
            if (model.queryCellInfo) {
                for (cellIndex; cellIndex < length; cellIndex++) {

                    if ($($tdRowcells[cellIndex]).hasClass("e-rowcell")) {
                        column = columns[cellIndex];
                    }

                    if (column) {
                        proxy._cellEventTrigger($tdRowcells[cellIndex], data, column);
                    }
                }
            }

            if (model.detailsDataBound && $(row).next("tr").hasClass("e-detailsrow")) {
                var detailsRowElement = $(row).next("tr");
                if (this._frozenColumnsLength > 0)
                    detailsRowElement = $(detailsRowElement[1]);
                this._trigger("detailsDataBound", { detailsElement: detailsRowElement, data: data });
            }
        },


        //QUERYCELLINFO EVENT TRIGGER
        _cellEventTrigger: function (cell, data, column) {

            var args = {
                cellElement: cell,
                data: data,
                column: column,

            };
            args.cellValue = args.column && this._getCustomCellValue(data, args.column.field);
            this._trigger("queryCellInfo", args);
        },


        //ACTION COMPLETE EVENT TRIGGER
        _completeAction: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns,
                ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");

            model.isEdit = false;

            if (model.allowPaging && args.requestType != ej.TreeGrid.Actions.Scroll)
                this._refreshGridPager();
            if ((ej.TreeGrid.Actions.Sorting == args.requestType && model.allowSorting)
                || (ej.TreeGrid.Actions.Refresh == args.requestType)) {

                if (model.allowSorting) {

                    if (!proxy._multiSortRequest) {
                        proxy.getHeaderTable().find(".e-columnheader").
                            find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();

                        proxy.getHeaderTable().find("[aria-sort]").removeAttr("aria-sort");
                    }

                    for (var i = 0; i < sortcolumns.length; i++) {
                        proxy._addSortElementToColumn(sortcolumns[i].field, sortcolumns[i].direction);
                    }

                }

                proxy._multiSortRequest = false;
            } else if (ej.TreeGrid.Actions.BeginEdit == args.requestType || ej.TreeGrid.Actions.Add == args.requestType) {

                var $form = $("#" + proxy._id + "EditForm");

                proxy.model.isEdit = true;
                proxy.setWidthToColumns();

                if (ej.TreeGrid.Actions.Add == args.requestType) {

                    $form.find(".e-field:disabled").not(".e-identity").removeAttr("disabled").removeClass("e-disable");

                }

                if ($.isFunction($.validator))
                    this._initValidator();
            }
            else if (model.isFromGantt && args.requestType === ej.TreeGrid.Actions.Scroll)
            {
                proxy._updateCurrentViewData();
            }
            if (model.isFromGantt)
                ganttObject.model.sortSettings = model.sortSettings;
            if ((!proxy._moveRow && model.editSettings.editMode.toLowerCase() != "batchediting") || args.requestType === ej.TreeGrid.Actions.Refresh) proxy._trigger("actionComplete", args);
        },


        //#endregion

        //#region METHOD FOR DATA PROCESSING

        //RENDERING THE DATA
        sendDataRenderingRequest: function (args) {

            var proxy = this, model = this.model,
                temp, data;
            if (proxy.model.currentViewData.length) {

                switch (args.requestType) {

                    case ej.TreeGrid.Actions.Delete:
                    case ej.TreeGrid.Actions.Refresh:
                    case ej.TreeGrid.Actions.Save:
                    case ej.TreeGrid.Actions.Filter:
                    case ej.TreeGrid.Actions.Sorting:
                    case ej.TreeGrid.Actions.Searching:
                    case ej.TreeGrid.Actions.ExpandCollapse:
                    case ej.TreeGrid.Actions.RefreshDataSource:
                    case ej.TreeGrid.Actions.DragAndDrop:
                    case ej.TreeGrid.Actions.Paging:
                    case ej.TreeGrid.Actions.Scroll:
                    case ej.TreeGrid.Actions.RowIndexUpdate:

                        temp = document.createElement('div');
                       

                        if (args.requestType != ej.TreeGrid.Actions.Scroll) {
                            if (this._frozenColumnsLength > 0) {
                                proxy._renderFrozenRecords();

                            }
                            else {
                                proxy.getContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                                var $tbody = proxy.getContentTable().children('tbody');
                                $tbody.empty();

                                temp.innerHTML = ['<table>', $.render[proxy._id + "_Template"](proxy.model.currentViewData), '</table>'].join("");

                                proxy.getContentTable().get(0).replaceChild(temp.firstChild.firstChild,
                                    proxy.getContentTable().get(0).lastChild);
                                proxy.setGridRows($(proxy.getContentTable().get(0).rows));
                            }
                        }
                        else {
                            var $tbody = proxy.getContentTable().children('tbody'),
                                                           currentViewData = proxy.model.currentViewData,
                                                           visibleRange = proxy._visibleRange,
                                                           length = this._frozenColumnsLength > 0 ? $($tbody[0]).children("tr:not(.e-detailsrow)").length : $tbody.children("tr:not(.e-detailsrow)").length, isTopToBottom = false;
                            if (currentViewData.length != 0 && $tbody.find(".e-emptyText").length != 0) {
                                $tbody.find(".e-emptyText").remove();
                            }
                            if (proxy._scrollTop > proxy._prevScrollTop) {
                                var removeLength = visibleRange.top - proxy._prevRTop,
                                     addLength = visibleRange.bottom - proxy._prevRBottom,
                                     data;
                                if (addLength >= 0) {
                                    data = addLength ? currentViewData.slice(-addLength) : [];
                                } else {
                                    if (this._frozenColumnsLength > 0) {
                                        $($tbody.get(0)).children(":visible:gt(" + (length + (addLength - 1)) + ")").remove();
                                        $($tbody.get(1)).children(":visible:gt(" + (length + (addLength - 1)) + ")").remove();
                                    } else {
                                        if (model.showDetailsRow && !model.showDetailsRowInfoColumn) {
                                            var childLength = 2 * (length + (addLength - 1));
                                            $tbody.children(":visible:gt(" + childLength + ")").remove();
                                        }
                                        else
                                            $tbody.children(":visible:gt(" + (length + (addLength - 1)) + ")").remove();    
                                    }
                                    data = [];
                                }
                                isTopToBottom = true;
                            }
                            else {
                                var removeLength = length - (proxy._prevRBottom - visibleRange.bottom + 1),
                                      addLength = proxy._prevRTop - visibleRange.top, data;
                                if (addLength >= 0) {
                                    data = currentViewData.slice(0, addLength);
                                } else {
                                    data = [];
                                }
                                removeLength = removeLength >= 0 ? removeLength : (-length - 1);
                                
                                isTopToBottom = false;
                            }                            
                            if (this._frozenColumnsLength > 0) {
                                proxy._renderFrozenRecords(data, addLength, removeLength, isTopToBottom);
                            }
                            else {
                                proxy.getContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                                if (isTopToBottom) {
                                    if (model.showDetailsRow && !model.showDetailsRowInfoColumn) {
                                        var childLength = 2 * removeLength;
                                        $tbody.children(":visible:lt(" + childLength + ")").remove();
                                    }
                                    else
                                        $tbody.children(":visible:lt(" + removeLength + ")").remove();
                                    var dataHtml = $.render[proxy._id + "_Template"](data);
                                    $tbody.append(dataHtml);
                                }
                                else {
                                    if (model.showDetailsRow && !model.showDetailsRowInfoColumn) {
                                        var childLength = 2 * removeLength + 1;
                                        $tbody.children(":visible:gt(" + childLength + ")").remove();
                                    }
                                    else
                                        $tbody.children(":visible:gt(" + removeLength + ")").remove();
                                    var dataHtml = $.render[proxy._id + "_Template"](data);
                                    $tbody.prepend(dataHtml);
                                }
                                proxy._gridRows = proxy.getContentTable().find("tr");
                                proxy.setGridRows(proxy._gridRows);
                            }
                        }
                        this._gridRows = this.getContentTable().get(0).rows;
                        if (this._frozenColumnsLength > 0)
                            this._gridRows = [this._gridRows, this.getContentTable().get(1).rows];

                        /* check treee grid rows are append in DOM and Trigger refresh and querycellInfo events */
                        if (proxy._$gridContent && proxy._isRendered){
                            proxy._trigger("refresh");
                            proxy._eventBindings(data);
                            this._hideCollapsedDetailsRows();
                        }
                        break;
                }

            } else {
                if (proxy._frozenColumnsLength > 0) {
                    var $emptyTd = ej.buildTag('td', model.emptyRecordText, {}, { colSpan: this._frozenColumns.length }), doc = document;
                    proxy.getContent().find("#e-frozencontentdiv" + proxy._id + " .e-table tbody").empty().append($(doc.createElement("tr")).addClass("e-emptyText").height(30).append($emptyTd));
                    $emptyTd = ej.buildTag('td', model.emptyRecordText, { visibility: "hidden" }, { colSpan: this._unFrozenColumns.length });
                    proxy.getContent().find("#e-movablecontentdiv" + proxy._id + " .e-table tbody").empty().append($(doc.createElement("tr")).addClass("e-emptyText").height(30).append($emptyTd));
                    if (model.updatedRecords.length == 0)
                        proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id + ",#e-frozencontentdiv" + proxy._id).css("height", "30px");
                } else {
                    proxy.getContentTable().find('tbody').empty().append(proxy._getEmptyTbody());
                    if (model.updatedRecords.length == 0)
                        proxy._$gridContainer && proxy._$gridContainer.css({ height: "30px" }); // for render no record text
                }
                proxy._gridRows = null;
                proxy.setGridRows(null);

            }

            if (proxy.model.showGridCellTooltip || proxy.model.showGridExpandCellTooltip) {
                proxy._cellMouseLeave();
                proxy._headerMouseLeave();
            }

            if (args.requestType == ej.TreeGrid.Actions.Scroll)
                args.requestType == ej.TreeGrid.Actions.Refresh;

            proxy._completeAction(args);

        },
        _renderFrozenRecords: function (data, addLength, removeLength, isTopToBottom)
        {
            /* Update colgroup of frozen and movable tables while rendering records*/
            this.getContent().find("#e-frozencontentdiv" + this._id + " .e-table colgroup").replaceWith(this._getMetaColGroup()[0]);
            this.getContent().find("#e-movablecontentdiv" + this._id + " .e-table colgroup").replaceWith(this._getMetaColGroup()[1]);
            var $tbody = this.getContentTable().children('tbody'), temp;
            

            if (ej.isNullOrUndefined(data)) {
                $tbody.empty();
                /*Render frozen Table*/
                temp = document.createElement('div');
                temp.innerHTML = ['<table>', $.render[this._id + "_JSONFrozenTemplate"](this.model.currentViewData),
                    '</table>'].join("");
                this.getContentTable().get(0).replaceChild(temp.firstChild.firstChild,
                    this.getContentTable().get(0).lastChild);
                /*Render movable Table*/
                temp = document.createElement('div');
                temp.innerHTML = ['<table>', $.render[this._id + "_Template"](this.model.currentViewData),
                    '</table>'].join("");
                this.getContentTable().get(1).replaceChild(temp.firstChild.firstChild,
                    this.getContentTable().get(1).lastChild);
            }
            else {
                if (isTopToBottom) {
                    if (this.model.showDetailsRow && !this.model.showDetailsRowInfoColumn) {
                        var childLength = 2 * removeLength;
                        $($tbody.get(0)).children(":visible:lt(" + childLength + ")").remove();
                        $($tbody.get(1)).children(":visible:lt(" + childLength + ")").remove();
                    }
                    else {
                        $($tbody.get(0)).children(":visible:lt(" + removeLength + ")").remove();
                        $($tbody.get(1)).children(":visible:lt(" + removeLength + ")").remove();
                    }
                    var frozenHtml = $.render[this._id + "_JSONFrozenTemplate"](data);
                    var moveHtml = $.render[this._id + "_Template"](data);
                    $($tbody.get(0)).append(frozenHtml);
                    $($tbody.get(1)).append(moveHtml);
                }
                else {
                    if (this.model.showDetailsRow && !this.model.showDetailsRowInfoColumn) {
                        var childLength = 2 * removeLength + 1;
                        $($tbody.get(0)).children(":visible:gt(" + childLength + ")").remove();
                        $($tbody.get(1)).children(":visible:gt(" + childLength + ")").remove();
                    }
                    else {
                        $($tbody.get(0)).children(":visible:gt(" + removeLength + ")").remove();
                        $($tbody.get(1)).children(":visible:gt(" + removeLength + ")").remove();
                    }
                    var frozenHtml = $.render[this._id + "_JSONFrozenTemplate"](data);
                    var moveHtml = $.render[this._id + "_Template"](data);
                    $($tbody.get(0)).prepend(frozenHtml);
                    $($tbody.get(1)).prepend(moveHtml);
                }
            }

            this.setGridContentTable(this.getContent().find(".e-table"));
        },

        _renderByFrozenDesign: function () {
            var $div = $(document.createElement('div')), col = this._getMetaColGroup().find("col"), colgroups = {};
            colgroups["colgroup1"] = $div.append(ej.buildTag("colgroup").append(col.splice(0, this._frozenColumnsLength))).html();
            colgroups["colgroup2"] = $div.html(ej.buildTag("colgroup").append(col)).html();
            return $.render[this._id + "_FrozenTemplate"](colgroups);
           
        },
        addFrozenTemplate: function () {
            var template = "<div class='e-frozencontainer' id='e-frozencontainer" + this._id + "' style='overflow:hidden;float:left;'><div class='e-frozencontentdiv' id='e-frozencontentdiv" + this._id + "' style='width:100%;'>"
            + "<table class='e-table e-zerospace' style='width:100%' id='" + this._id + "frozene-table'>{{:colgroup1}}<tbody>"
            + "</tbody></table></div></div>"
            + "<div class='e-movablecontainer' id='e-movablecontainer" + this._id + "'><div class='e-movablecontent' id='e-movablecontent" + this._id + "'><div class='e-movablecontentdiv' id='e-movablecontentdiv" + this._id + "'><table class='e-table e-zerospace' id='" + this._id + "movablee-table'>{{:colgroup2}}<tbody>"
            + "</tbody></table></div></div></div>", templates = {};
            templates[this._id + "_FrozenTemplate"] = template;
            $.templates(templates);
        },
       

        //Public method to change the allowSorting value dynamically.
        sortSetting: function (value) {
            var proxy = this, model = this.model,
                ascendingIcon = $("#" + proxy._id).find(".e-ascending.e-icon"),
                descendingIcon = $("#" + proxy._id).find(".e-descending.e-icon");
            model.allowSorting = value;
            if (!model.allowSorting)
                proxy.clearSorting();
                if (ascendingIcon.length > 0) {
                    for (var length = 0 ; length < ascendingIcon.length; length++) {
                        $(ascendingIcon[length]).css("visibility", "hidden");
                    }
                }                                      
            if (descendingIcon.length > 0)
                for (var length = 0 ; length < descendingIcon.length; length++) {
                    $(descendingIcon[length]).css("visibility", "hidden");
                }
        },
        //Public method to change the treeColumnIndex value dynamically.
        columnIndex:function(value){
            var proxy = this, model = proxy.model,
                value = parseInt(value),
                columns = model.columns;
            if (!isNaN(value) && value >= 0 && value < columns.length && columns[value].visible && !proxy._isCommandColumn(columns[value])) {
                if (!columns[value].nonColumn)
                    model.treeColumnIndex = value;
                else if (columns[value + 1] && columns[value + 1].visible && !proxy._isCommandColumn(columns[value + 1]))
                    model.treeColumnIndex = value + 1;
                else
                    return;
                proxy._addInitTemplate();
                this.renderRecords();
                if (model.showColumnChooser)
                    proxy._renderColumnChooserList(true);
                proxy._updateWrapRowHeights("header", "refreshAll");
            }            
        },


        //SORTING THE DATA BASED ON THE PARTICULAR FIELDS
        
        sortColumn: function (columnName, columnSortDirection) {

            var proxy = this,
                model = proxy.model,
                sortColumns = model.sortSettings.sortedColumns,
                args = {},
                column,
                count = 0,
                length = sortColumns.length;
            /* Cancel edited cell before sort the column*/
            proxy._cancelEditState();
            proxy.clearSelection();
            if (model.isFromGantt)
                column = ej.TreeGrid.getColumnByMappingName(model.columns, columnName);
            if (ej.isNullOrUndefined(column))
                column = this.getColumnByField(columnName);
            if (!model.allowSorting || ej.isNullOrUndefined(column) || column.allowSorting == false) {
                return;
            }

            if (!proxy._multiSortRequest) {
                model.sortSettings.sortedColumns = [];
            }

            args.requestType = ej.TreeGrid.Actions.Sorting;
            args.columnName = columnName;

            // proxy._cSortedColumn = proxy.getFieldNameByHeaderText(columnName);

            proxy._cSortedColumn = columnName;

            proxy._cSortedDirection = args.columnSortDirection = columnSortDirection === undefined ?
                ej.sortOrder.Ascending : columnSortDirection;

            for (count; count < length; count++) {
                
                if (sortColumns[count].field == proxy._cSortedColumn) {
                    sortColumns.splice(count, 1);
                    break;
                }

            }

            model.sortSettings.sortedColumns.push({
                field: proxy._cSortedColumn,
                direction: proxy._cSortedDirection
            });

            var returnValue = proxy.processBindings(args);

            if (returnValue) {

                proxy._cSortedDirection = proxy._cSortedColumn = null;

            }
            if (proxy.selectedRowIndex() >= 0 && proxy._isRendered) {
                proxy.selectedRowIndex(proxy._sortedRecords.indexOf(this.selectedItem()));
            }
            proxy._clearColumnMenu();
            proxy._clearContextMenu();
        },


        //Clear the sorting
        clearSorting: function()
        {
            var proxy = this,
                model = proxy.model;
            model.sortSettings.sortedColumns = [];
            var args = {};
            args.requestType = ej.TreeGrid.Actions.Sorting;
            proxy.processBindings(args);
        },


        //DELETE THE PARTICULAR RECORD AND MULTIPLE RECORD
        deleteRow: function ($tr, isFromContextmenu, index) {

            var proxy = this, model = proxy.model,
                editMode = model.editSettings.editMode.toLowerCase(),
                updatedRecords = proxy.getUpdatedRecords(),
                deletedRows = [];

            if (!model.editSettings.allowDeleting)
                return false;
            if (model.editSettings.showDeleteConfirmDialog && !(proxy._confirmDialog).is(":visible") && editMode != "batchediting") {
                if (!proxy._isEmptyRow) {
                    proxy._isFromContextmenu = isFromContextmenu;
                    proxy._deleteRowIndex = index;
                    proxy._confirmDialog.ejDialog("open");
                    return;
                }
            }
            var args = {};
            args.requestType = ej.TreeGrid.Actions.Delete;
            args.isFromContextmenu = isFromContextmenu;

            if (isFromContextmenu) {
                if (model.isFromGantt)
                    args.data = updatedRecords[index];
                else if (!model.allowSelection && (!ej.isNullOrUndefined(index) && index > -1))
                    args.data = updatedRecords[index];
                else
                    args.data = model.selectionSettings.selectionMode == "row" ? model.selectedItem :
                                    proxy._rowIndexOfLastSelectedCell > -1 ? updatedRecords[proxy._rowIndexOfLastSelectedCell] :
                                        updatedRecords[proxy._emptyRowIndex];

                deletedRows.push(args.data);
                // Delete empty row when save new record
                if (proxy._isEmptyRow) {
                    args.deletedItems = deletedRows;
                    proxy._updateDeleteAction(args);
                    return true;
                }
            }
            else {
                var deletedRows = $.extend([], model.selectedItems);
                args.data = model.selectedItem;
            }
            if (!deletedRows.length) deletedRows.push(args.data);
            args.deletedItems = deletedRows;
            if (editMode == "batchediting") {
                if (proxy._batchSaveCancel == "save") {
                    args.data = {};
                    args.deletedItems = proxy.batchChanges.deleted;
                }
                else if (proxy._batchSaveCancel == "cancel") {
                    args.data = {};
                    args.deletedItems = proxy.batchChanges.added;
                }
                else {
                    for (var d = 0; d < deletedRows.length; d++) {
                        proxy._batchDeleteAction(deletedRows[d]);
                        proxy._updateBatchDeletedRecords(deletedRows[d]);
                    }
                    proxy.clearSelection();
                    proxy._batchEditStarts = true;
                    proxy._cancelSaveTools();
                    return true;
                }                
            }
            if (proxy._trigger("actionBegin", args)) return true;
            else {
                if (proxy._updateAction(args)) {
                    proxy._updateDeleteAction(args);
                }
            }
        },
        _batchDeleteAction: function (data) {
            var proxy = this,
                model = proxy.model,
                updatedRecords = model.allowPaging ? proxy._updatedPageData : model.updatedRecords,
                index = updatedRecords.indexOf(data);
            data.isBatchDelete = true,
            proxy.refreshRow(index);
            if (data.hasChildRecords) {
                var childs = data.childRecords;
                for (var c = 0; c < childs.length; c++)
                    this._batchDeleteAction(childs[c]);
            }
        },
        //To update deleted collection of batch edit
        _updateBatchDeletedRecords: function (deletedRecord) {
            var proxy = this,
                batchChangedGrp = proxy.batchChanges.changed,
                batchAddedGrp = proxy.batchChanges.added;
            if (batchAddedGrp.length > 0 && batchAddedGrp.indexOf(deletedRecord) != -1) {
                batchAddedGrp.splice(batchAddedGrp.indexOf(deletedRecord), 1);
            }
            else if (batchChangedGrp.length > 0 && batchChangedGrp.indexOf(deletedRecord) != -1) {
                batchChangedGrp.splice(batchChangedGrp.indexOf(deletedRecord), 1);
            }
            if (proxy.batchChanges.deleted.indexOf(deletedRecord) == -1)
                proxy.batchChanges.deleted.push(deletedRecord);
        },

        _updateDeleteAction: function (args) {
            var proxy = this, model = proxy.model,
                cArgs = {}, editMode = model.editSettings.editMode.toLowerCase();

            var length = args.deletedItems.length;
            for (var del = 0; del < length; del++) {
                args.data = args.deletedItems[del];
                proxy._selectedCellDetails = [];
                proxy._removeRecords(args);
            }
            if (editMode == "batchediting" && (proxy._batchSaveCancel == "save" || proxy._batchSaveCancel == "cancel")) return true;

            proxy._ensureDataSource(args);
            proxy.updateHeight();
            if (model.currentViewData.length > 0 && model.enableAltRow)
                ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);

            if (proxy._checkboxSelection) {
                model.selectedCellIndexes = [];
                proxy._prevSelectedItem = null;
                if (!args.isFromContextmenu)
                    model.selectedItems = [];
                model.selectedItem = null;
                proxy.selectedRowIndex(-1);
                proxy.selectAllRows();
            }
            else {
                model.selectedItems = [];
                model.selectedCellIndexes = [];
                model.selectedItem = null;
                proxy.selectedRowIndex(-1);
                proxy._prevSelectedItem = null;
                if (proxy._multiSelectPopup && proxy._multiSelectPopup.is(":visible") && !proxy.model.isFromGantt)
                    proxy._clearMultiSelectPopup();
            }
            if (proxy._isLoadonDemand && model.allowPaging)
                proxy._refreshDataSource();
        },

		//Delete selected records
        _removeRecords: function(args){
            var proxy = this,
                model = proxy.model,
                editMode = model.editSettings.editMode.toLowerCase(),
                isUnassignedTaskUpdate = false,
                dataSource =this.dataSource();
            if (args && (model.editSettings.allowDeleting || args.isDragAndDropDelete) && args.requestType === ej.TreeGrid.Actions.Delete) {
                var deletedRow = args.data,
                    recordIndex, rowindex;

                if (model.isFromGantt && proxy.model.enableWBS) {
                    var dataRows, targetRowIndex, isUpdateWBS = false;
                    if (deletedRow && deletedRow.parentItem) {
                        dataRows = deletedRow.parentItem.childRecords,
                        targetRowIndex = dataRows.indexOf(deletedRow);
                    } else if (model.flatRecords.length) {
                        var flatData = model.flatRecords,
                            Level0 = flatData.filter(function (item) {
                                return item && item.level == 0;
                            });
                        dataRows = Level0;
                        targetRowIndex = dataRows.indexOf(deletedRow);
                    }
                    if (targetRowIndex != dataRows.length - 1)
                        isUpdateWBS = true;
                }

                recordIndex = model.flatRecords.indexOf(deletedRow);
                if (model.viewType == "resourceView") {
                    var eventArgs = { data: args.data };
                    proxy._trigger("updateResource", eventArgs);// update resource values on resour or group delete action in resource view Gantt
                    proxy.dataSource(eventArgs.returnObj.dataSource);
                    proxy._jsonData = eventArgs.returnObj.jsondata;
                    if (eventArgs.returnObj.tasks.length > 0)
                        isUnassignedTaskUpdate = true;
                }
                if (deletedRow.parentItem) {

                    var childRecords = deletedRow.parentItem.childRecords,
                    childIndex;

                    if (childRecords && childRecords.length > 0) {
                        childIndex = childRecords.indexOf(deletedRow);

                        deletedRow.parentItem.childRecords.splice(childIndex, 1);
                        //Update hierarchyRowIndex for level 1 and above records
                        args.updatedHierarchyRowIndexData = [];
                        for (var e = childIndex; e < childRecords.length; e++) {
                            childRecords[e].hierarchyRowIndex = e;
                            if (model.hierarchyRowIndexMapping)
                                childRecords[e].item[model.hierarchyRowIndexMapping] = e;
                            if (!deletedRow.isBatchAdd) {
                                if (editMode != "batchediting")
                                    args.updatedHierarchyRowIndexData.push(childRecords[e]);
                                else
                                    proxy.batchChanges.updatedHierarchyRowIndexData.indexOf(childRecords[e]) == -1 && proxy.batchChanges.updatedHierarchyRowIndexData.push(childRecords[e]);
                            }
                        }
                        //Delete the summary row of one parent after deleted the all child records of the parent
                        if (model.showSummaryRow) {
                            var summaryRowLength = model.summaryRows.length;
                            if (summaryRowLength == childRecords.length) {
                                deletedRow.parentItem.childRecords.splice(0, summaryRowLength);
                                model.flatRecords.splice(recordIndex, summaryRowLength);
                            }
                        }
                        if (!model.parentIdMapping && model.viewType != "resourceView")
                            deletedRow.parentItem.item[model.childMapping].splice(childIndex, 1);
                    }
                }

                if (proxy._checkboxSelection && !args.isDragAndDropDelete) {
                    if (args.isFromContextmenu)
                        proxy._removeChildRecords(model.selectedItems, args.data);
                    else {
                        while (model.selectedItems.length > 0) {
                            proxy._removeChildRecords(model.selectedItems, model.selectedItems[0]);
                        }
                    }
                }

                //Delete from parent collection if item is parent
                var parentIndex = model.parentRecords.indexOf(deletedRow);
                if (parentIndex!== -1)
                {
                    model.parentRecords.splice(parentIndex, 1);
                    //Update hierarchyRowIndex for level 0 records
                    var parentRecords = model.parentRecords;
                    args.updatedHierarchyRowIndexData = [];
                    for (var e = parentIndex; e < parentRecords.length; e++) {
                        parentRecords[e].hierarchyRowIndex = e;
                        if (model.hierarchyRowIndexMapping)
                            parentRecords[e].item[model.hierarchyRowIndexMapping] = e;
                        if (!deletedRow.isBatchAdd) {
                            if (editMode != "batchediting")
                                args.updatedHierarchyRowIndexData.push(parentRecords[e]);
                            else
                                proxy.batchChanges.updatedHierarchyRowIndexData.indexOf(parentRecords[e]) == -1 && proxy.batchChanges.updatedHierarchyRowIndexData.push(parentRecords[e]);
                        }
                    }
                }
                
                if (recordIndex !== -1) {
                    var deletedRecordCount = proxy.getChildCount(deletedRow, 0);
                    model.flatRecords.splice(recordIndex, deletedRecordCount + 1);
                    model.ids && model.ids.splice(recordIndex, deletedRecordCount + 1);
                    if (isUnassignedTaskUpdate) {
                        model.flatRecords.push.apply(model.flatRecords, eventArgs.returnObj.tasks);
                        model.ids && model.ids.push.apply(model.ids, eventArgs.returnObj.ids);
                        model.parentRecords.push.apply(model.parentRecords, eventArgs.returnObj.tasks);
                    }
                    if (model.viewType == "resourceView" && deletedRow.eResourceTaskType == "resourceTask" && deletedRow.parentItem && deletedRow.parentItem.childRecords.length == 0) {
                        var pIndex = model.flatRecords.indexOf(deletedRow.parentItem);
                        model.flatRecords.splice(pIndex, 1);
                        model.ids && model.ids.splice(pIndex, 1);
                        pIndex = model.parentRecords.indexOf(deletedRow.parentItem);
                        if (pIndex != -1)
                            model.parentRecords.splice(pIndex, 1);
                    }
                }
                model.updatedRecords = model.enableVirtualization ? proxy.getExpandedRecords(model.flatRecords) : model.flatRecords.slice();
                proxy.updateCollapsedRecordCount();                                
                
                if (!model.parentIdMapping && model.viewType != "resourceView")
                {
                    if (dataSource instanceof ej.DataManager) {
                        if (dataSource.dataSource.offline && dataSource.dataSource.json) {
                            rowindex = dataSource.dataSource.json.indexOf(deletedRow.item);
                            if (rowindex !== -1)
                                dataSource.dataSource.json.splice(rowindex, 1);
                        }
                        else if (proxy._isDataManagerUpdate) {
                            rowindex = proxy._jsonData.indexOf(deletedRow.item);
                            proxy._jsonData.splice(rowindex, 1);
                        }
                    } else {
                        rowindex = dataSource.indexOf(deletedRow.item);
                        if (rowindex !== -1)
                            dataSource.splice(rowindex, 1);
                    }
                }
                else if (model.viewType != "resourceView")
                {
                    //remove child item
                    if (deletedRow.hasChildRecords && deletedRow.childRecords.length > 0)
                    {
                        proxy._removeChildItem(deletedRow);
                    }
                    //remove deleting item

                    if (dataSource instanceof ej.DataManager) {
                        if (dataSource.dataSource.offline && dataSource.dataSource.json) {
                            var idx = dataSource.dataSource.json.indexOf(deletedRow.item);
                            if (idx !== -1)
                                dataSource.dataSource.json.splice(idx, 1);
                        }
                        else if (proxy._isDataManagerUpdate) {
                            var idx = proxy._jsonData.indexOf(deletedRow.item);
                            if (idx !== -1)
                                proxy._jsonData.splice(idx, 1);
                        }
                    }
                    else
                    {
                        var idx = dataSource.indexOf(deletedRow.item);
                        if (idx !== -1)
                            dataSource.splice(idx, 1);
                    }
                }
                
                if (deletedRow.parentItem)
                    if (deletedRow.parentItem.childRecords && deletedRow.parentItem.childRecords.length == 0) {
                        deletedRow.parentItem.expanded = false;
                        deletedRow.parentItem.hasChildRecords = false;
                        deletedRow.parentItem.hasFilteredChildRecords = true;
                    }
                //Update the summary and total summary row after deleted a record.
                if (model.showSummaryRow)
                    proxy._updateSummaryRow(args);
                if (model.showTotalSummary)
                    proxy._updateTotalSummaryRow(args);

                if (isUpdateWBS) {
                    var targetRow;
                    if (deletedRow.parentItem)
                        targetRow = dataRows[targetRowIndex];//Here we are having the updated 'dataRows'
                    else {
                        //for level 0 records
                        dataRows.splice(targetRowIndex, 1);
                        targetRow = dataRows[targetRowIndex];
                    }
                    proxy.updateWBSdetails(targetRow);
                }
                if (model.isFromGantt) {
                    proxy.updateSerialNumber(recordIndex);
                    model.enableSerialNumber && proxy.updateSerialNumberPredecessors(recordIndex);
                }
            }
        },
        /* To update serial number predecessor for target records */
        updateSerialNumberPredecessors: function (index) {
            if (index == -1) return;
            var proxy = this, model = proxy.model,
                flatDatas = model.flatRecords, dataLength = flatDatas.length,
                targetRecords, idCollection = [];            
            for (var i = index; i < dataLength; i++) {
                if (flatDatas[i].predecessor) {
                    var linkSet = flatDatas[i].predecessor, taskId = flatDatas[i].taskId;
                    for (var s = 0; s < linkSet.length; s++) {
                        if (linkSet[s].to) {
                            var toId = +linkSet[s].to != taskId ? +linkSet[s].to : null;
                            (toId && idCollection.indexOf(toId) == -1) && idCollection.push(toId);
                        }
                    }
                }
            }
            if (idCollection.length) {
                targetRecords = flatDatas.filter(function (record) {
                    if (idCollection.indexOf(record.taskId) > -1)
                        return record;
                });
                var tlength = targetRecords.length;
                for (var t = 0; t < tlength; t++) {
                    var targetRecord = targetRecords[t];
                    proxy.predecessorToSerialPredecessor(targetRecord);
                }
            }            
        },
        /*Changing Predecessor to Serial number predecessor*/
        predecessorToSerialPredecessor: function (tRecord) {
            var proxy = this, model = proxy.model,
                flatDatas = model.flatRecords,
                ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt"),
                prdcList = tRecord["predecessorsName"] ? tRecord["predecessorsName"].split(',') : [],
                snoPred = null, pr, values;
            for (var p = 0; p < prdcList.length; p++) {
                pr = prdcList[p];
                values = pr.split('+');
                if (pr.indexOf('-') >= 0)
                    values = pr.split('-');
                var ref = values[0].match(/(\d+|[A-z]+)/g);
                if (ref && ref.length) {
                    var refId = +ref[0], refType = ref[1], refRecord;
                    refRecord = flatDatas.filter(function (record) {
                        return record && record.taskId == refId;
                    });
                    var newOne = refRecord[0] && refRecord[0]["serialNumber"] + refType + (values[1] ? "+" + values[1] : "");
                    if (newOne)
                        snoPred = snoPred ? (snoPred + "," + newOne) : newOne;
                }
            }
            tRecord["serialNumberPredecessor"] = snoPred;
            tRecord["item"]["serialNumberPredecessor"] = snoPred;
            ganttObject.refreshGanttRecord(tRecord);
        },        
        /*Update the WBS value of all the records that are below the newly added row*/
        updateWBSdetails: function (delTargetRow) {
            var proxy = this,
                model = proxy.model,
                dataW = delTargetRow,
                newWBS = dataW["WBS"];

            //Updating the WBS value of other records that are below the newly added row
            var childItems, markedIndex, markedRecords, dataS, dataLength;
            if (dataW.parentItem) {
                childItems = dataW.parentItem.childRecords;
                markedIndex = childItems.indexOf(dataW);
                dataS = childItems;
                dataLength = dataS.length;
            } else if (model.flatRecords.length) {
                var flatData = model.flatRecords,
                    Level0 = flatData.filter(function (item) {
                        return item && item.level == 0;
                    });
                dataS = Level0;
                markedIndex = dataS.indexOf(dataW);
                dataLength = dataS.length;
            }
            markedRecords = dataS.slice(markedIndex, dataLength);
            if (markedRecords.length) {
                var lastVal = newWBS.lastIndexOf('.') != -1 ? parseInt(newWBS.substr(newWBS.lastIndexOf('.') + 1)) : parseInt(newWBS),
                    parentVal = dataW.parentItem ? dataW.parentItem.WBS : null;
                lastVal--;
                proxy.reCalculateWBS(markedRecords, lastVal, parentVal);
            }
        },

        /* To recalculate the WBS value */
        reCalculateWBS: function (selectedRecords, lastDigit, parentWBS) {
            var proxy = this, model = proxy.model,
                cViewData = model.currentViewData, final,
                ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt"),
                previousVaue = "";

            if (parentWBS) {
                for (var i = 0; i < selectedRecords.length; i++) {
                    final = lastDigit + i;
                    previousVaue = selectedRecords[i]["WBS"];
                    selectedRecords[i]["WBS"] = parentWBS + "." + final;
                    selectedRecords[i]["item"]["WBS"] = parentWBS + "." + final;
                    if (model.enableWBSPredecessor)
                        proxy._getPredecessorIds(selectedRecords[i]);
                    if (previousVaue != selectedRecords[i]["WBS"])
                        ganttObject.refreshGanttRecord(selectedRecords[i]);
                    if (selectedRecords[i].hasChildRecords)
                        proxy._updateChildWBS(selectedRecords[i])
                }
            }
            else {
                for (var i = 0; i < selectedRecords.length; i++) {
                    final = (lastDigit + i).toString();
                    previousVaue = selectedRecords[i]["WBS"];
                    selectedRecords[i]["WBS"] = final;
                    selectedRecords[i]["item"]["WBS"] = final;
                    if (model.enableWBSPredecessor)
                        proxy._getPredecessorIds(selectedRecords[i]);
                    if (previousVaue != selectedRecords[i]["WBS"])
                        ganttObject.refreshGanttRecord(selectedRecords[i]);
                    if (selectedRecords[i].hasChildRecords)
                        proxy._updateChildWBS(selectedRecords[i])
                }
            }
        },

        _updateChildWBS: function (parentRecord) {
            var proxy = this, model = proxy.model,
                cViewData = model.currentViewData,
                childRecs = parentRecord.childRecords,
                pWBS = parentRecord["WBS"],
                ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt"),
                previousValue = "";
            for (var c = 0; c < childRecs.length; c++) {
                previousValue = childRecs[c]["WBS"];
                childRecs[c]["WBS"] = pWBS + "." + (c + 1);
                childRecs[c]["item"]["WBS"] = pWBS + "." + (c + 1);
                if (model.enableWBSPredecessor)
                    proxy._getPredecessorIds(childRecs[c]);
                if (previousValue != childRecs[c]["WBS"])
                    ganttObject.refreshGanttRecord(childRecs[c]);
                if (childRecs[c].hasChildRecords) {
                    proxy._updateChildWBS(childRecs[c]);
                }
            }
        },

        /*Get all the predecessor rows that are matching the taskId*/
        _getPredecessorIds: function (pData) {
            var proxy = this,
                model = proxy.model,
                dataTID = pData.taskId,
                flatDatas = model.flatRecords,
                updatedRecs = model.updatedRecords,
                targetRecords = flatDatas.filter(function (item) {
                    if (item["predecessorsName"]) {
                        var prdcList = item["predecessorsName"].split(',');
                        for (var p = 0; p < prdcList.length; p++) {
                            var ref = prdcList[p].match(/(\d+|[A-z]+)/g),
                                refId = +ref[0];
                            if (refId == dataTID)
                                return item;
                        }
                    }
                });
            for (var t = 0; t < targetRecords.length; t++) {
                proxy.updateWBSPredecessor(targetRecords[t]);
                proxy.refreshRow(updatedRecs.indexOf(targetRecords[t]));
            }
        },

        //To update Serial number
        updateSerialNumber: function (index) {
            if (index == -1) return;
            var proxy = this, model = proxy.model,
                flatRecords = model.flatRecords,
                isSerialUpdate = true,
                ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");
            for (var i = index; i < flatRecords.length; i++) {
                var record = flatRecords[i];
                record.serialNumber = i + 1;
                record.item["serialNumber"] = i + 1;
                ganttObject.refreshGanttRecord(record,null,isSerialUpdate);
            }
        },

        //SEARCH THE TASK FROM TREE GRID CONTROL

        

        search: function (searchString) {

            var proxy = this,
                args = {},
                searchSettings = proxy.model.searchSettings;
            if (!proxy.model.allowSearching) return;
            args.requestType = ej.TreeGrid.Actions.Searching;
            args.keyValue = searchString;

            proxy._searchString = searchString;
            if (searchString != "") {
                searchSettings.key = searchString.toLowerCase() == this.localizedLabels.trueText.toLowerCase() ? "true" : searchString.toLowerCase() == this.localizedLabels.falseText.toLowerCase() ? "false" : searchString;
                searchSettings.fields = this.model.searchSettings.fields.length != 0 ? this.model.searchSettings.fields : this.getColumnFieldNames();
            }
            else
            {
                searchSettings.key = "";
                searchSettings.fields = [];
            }
            proxy.processBindings(args);
        },

        //SEND SAVE REQUEST
        endEdit: function () {

            var proxy = this, model = proxy.model;

            if (proxy.model.editSettings.editMode.toLowerCase() == "cellediting" && !proxy._isRowEdit) {
                if (!this.editFormValidate())
                    return true;
                return proxy.saveCell();
            } else if (proxy.model.editSettings.editMode.toLowerCase() == "dialogediting" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog")) {
                return proxy.saveRow();
            }

            return true;

        },
        updateReadOnly:function(bool){
            var proxy=this;
            proxy.model.readOnly = bool;           
           
        },
        updateShowGridExpandCellTooltip: function (bool) {
            var proxy = this;
            proxy.model.showGridExpandCellTooltip = bool;
            proxy.refreshContent();
        },
        updateShowGridCellTooltip: function (bool) {
            var proxy = this;
            proxy.model.showGridCellTooltip = bool;
            proxy.refreshContent();
        },

        //#endregion

        //#region PUBLIC METHODS

        //CALCULATE THE VISIBLE RANGE OF RECORDS TO DISPLAYED IN CURRENT VIEWPORT
        getVisibleRange: function (type) {

            var proxy = this,
                model = this.model,
                calcHeight = (model.rowHeight + proxy._detailsRowHeight),
                topIndex, bottomIndex, top, coeff, testWrapRow,
                updatedRecords = (proxy._isHierarchyFiltering && proxy.model.enableVirtualization) ? proxy._updateFilterParent : proxy.getUpdatedRecords();
            if (model.sizeSettings.height == "auto") {
                proxy._viewPortHeight = proxy._getViewPortHeight();
                if (proxy._viewPortHeight != 0) {
                    proxy._visibleRange = {
                        top: 0,
                        bottom: proxy.getExpandedRecords(proxy.getUpdatedRecords()).length
                    };
                    return
                }
            }
            if (model.allowTextWrap) {
                testWrapRow = proxy._getVisibleTextWrapRows(proxy._scrollTop);
                top = testWrapRow.top;
                if (type == "scroll" && proxy._scrollTop > proxy._prevScrollTop) {
                    testWrapRow.bottom = testWrapRow.bottom + 4;
                }
                bottomIndex = testWrapRow.bottom;
                proxy._offset = testWrapRow.offSet;
            } else if (model.viewType == "resourceView") {
                var range = this._getVisibleRangeForResourceGantt(proxy._scrollTop);
                top = range.top;
                bottomIndex = range.bottom;
                proxy._offset = range.topOffset;
            }
            else {
                top = proxy._scrollTop / calcHeight;
                bottomIndex = proxy._getRowPosition(proxy._scrollTop + proxy._viewPortHeight);
                coeff = top - Math.floor(top);
                proxy._offset = coeff * calcHeight;
            }
            topIndex = Math.floor(top);
                topIndex = Math.max(0, topIndex);
                bottomIndex = Math.min(updatedRecords.length, bottomIndex);

                proxy._visibleRange = {
                    top: topIndex,
                    bottom: bottomIndex
                };

            return proxy._visibleRange;

        },
        /*calculate visible range for resource view gantt*/
        _getVisibleRangeForResourceGantt: function (scrollTop) {
            var returnObj = {},
                model = this.model, height = 0,
                bottomPos = scrollTop + this._viewPortHeight,
                rowHeight = 0, argsTop = scrollTop,
                topIndex, topOffset, bottomIndex,
                records = model.updatedRecords;
            for (var i = 0; i < records.length; i++) {
                rowHeight = this._getTemplateRowHeight(records[i]);
                if (argsTop == 0 && ej.isNullOrUndefined(topIndex)) {
                    topIndex = 0;
                    topOffset = 0;
                    scrollTop * -1;
                }
                else if (scrollTop < 0 && ej.isNullOrUndefined(topIndex)) {
                    topIndex = i - 1;
                    topOffset = this._getTemplateRowHeight(records[topIndex]) - (scrollTop * -1)
                }
                if (bottomPos <= 0 && ej.isNullOrUndefined(bottomIndex)) {
                    bottomIndex = i;
                }
                scrollTop -= rowHeight;
                bottomPos -= rowHeight;
                if (!ej.isNullOrUndefined(topIndex) && !ej.isNullOrUndefined(bottomIndex))
                    break;
            }
            returnObj.top = topIndex;
            returnObj.topOffset = topOffset;
            returnObj.bottom = ej.isNullOrUndefined(bottomIndex) ? records.length : bottomIndex;;
            return returnObj;
        },
        //get the index of visible top row and bottom row.
        _getVisibleTextWrapRows: function (scrollTop) {
            var proxy = this, model = this.model, length = 0, offSet = 0, top = 0, topFlag = true, updateHeight = 0, bottomFlag = true,
                textWrapRow = {}, bottom = 0, updatedRecord, start = 0, height = 0;
            if (model.allowPaging) {
                updatedRecord = proxy._updatedPageData;
                length = proxy._updatedPageData.length;
            } else {
                updatedRecord = model.updatedRecords;
                length = updatedRecord.length;
            }
            for (var x = start; x < length; x++) {
                updateHeight = updateHeight + updatedRecord[x].textWrapRowHeight;
                if (model.detailsTemplate && !model.showDetailsRowInfoColumn) {
                    if (!updatedRecord[x].isSummaryRow)
                        updateHeight = updateHeight + proxy._detailsRowHeight;
                }
                if ((updateHeight) >= scrollTop && topFlag) {
                    top = x;
                    offSet = parseInt(scrollTop) - parseInt(updateHeight - updatedRecord[x].textWrapRowHeight);
                    topFlag = !topFlag;
                }
                if (!topFlag) {
                    height = height + updatedRecord[x].textWrapRowHeight;
                }
                if (((updateHeight) >= (scrollTop + proxy._viewPortHeight) && bottomFlag) || (bottomFlag && x == (updatedRecord.length - 1))) {
                    bottom = x + 1;
                    bottomFlag = !bottomFlag;
                    break;
                }                                                                            
            }
            if (height < proxy._viewPortHeight) {
                if (parseInt(((proxy._viewPortHeight - height) / model.rowHeight)) > 0) {
                    top = top - parseInt(((proxy._viewPortHeight - height) / model.rowHeight));
                } else {
                    top = top - 1;
                }
            }
            textWrapRow = {
                top: top,
                offSet: offSet,
                bottom: bottom,
            };
            return textWrapRow;
        },

        //get the average height and the total height of text wrap rows.
        _getAvgTextWrapRowHeight: function (type, rowIndex) {
            var textWrapRow = {};
            if (this.getRows().length > 0) {
            var proxy = this, model = this.model, length = 0, recordIndex = 0, index = 0, isEnd = false,
            height = 0, currentViewData = model.currentViewData,updatedRecord, $gridRows;
            if (!model.allowPaging) {
                updatedRecord = model.updatedRecords;
            } else if (model.allowPaging) {
                updatedRecord = proxy._updatedPageData;
            }
            if ($("#" + proxy._id).find(".e-frozencontainer").length > 0) {
                if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn)
                    $gridRows = proxy.getTreeGridRows()[0];
                else
                    $gridRows = proxy.getRows()[0];
            } else {
                if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn)
                    $gridRows = proxy.getTreeGridRows();
                else
                    $gridRows = proxy.getRows();
            }
            if (type == "updateHeight" && ej.isNullOrUndefined(rowIndex)) {
                length = updatedRecord.length;
            } else if (type == "updateHeight" && !ej.isNullOrUndefined(rowIndex)) {
                length = rowIndex;
            } else {
                length = $gridRows.length;
            }
            if ($gridRows.length > 0) {
                for (var x = 0; x < length; x++) {
                    if (type == "getAvgHeight") {
                        index = proxy.getIndexByRow($gridRows[x]);
                        if (currentViewData[index]) {
                            if (currentViewData[index].textWrapRowHeight != $($gridRows[x]).height() && $($gridRows[x]).height() >= model.rowHeight)
                                currentViewData[index].textWrapRowHeight = $($gridRows[x]).height();
                            recordIndex = updatedRecord.indexOf(currentViewData[index]);
                            if (recordIndex > (updatedRecord.length - 5) && !proxy._textWrapNoCalculate) {
                                isEnd = true;
                            }
                        }
                        height = height + $($gridRows[x]).height();
                    } else if (type == "updateHeight") {
                        height = height + updatedRecord[x].textWrapRowHeight;
                    }
                }
            }
            if (type == "getAvgHeight") {
                height = height / length;
                textWrapRow = {
                    avgHeight: height,
                    isEnd: isEnd,
                };
            } else {
                textWrapRow = {
                    height: height,
                };
            }
            } else {
                textWrapRow = {
                    avgHeight: this.model.rowHeight,
                    isEnd: true,
                    height: this.model.rowHeight,
                };
            }
            return textWrapRow;
        },

        /* set width to frozen and movable header columns to prevent
        improper view port height calculation while load time*/
        _setWidthToHeaders: function ()
        {
            var width = this._gridWidth;
            if (this._frozenColumnsLength > 0) {
                var totalWidth = 0, frozenWidth, headerColumns = this.getHeaderContent().find('.e-headercell');
                for (var i = 0; i < this.columnsWidthCollection.length; i++) {
                    totalWidth += this.columnsWidthCollection[i];
                    if (this._frozenColumnsLength - 1 == i)
                        frozenWidth = Math.ceil(totalWidth);
                    if (this.model.headerTextOverflow) {
                        $(headerColumns[i]).css({ "width": this.columnsWidthCollection[i] + "px" });
                    }
                }
                this.getHeaderContent().find("#e-frozenheaderdiv" + this._id).width(width).next().css("margin-left", frozenWidth + "px");
                this.getHeaderContent().find("#e-frozenheaderdiv" + this._id).outerWidth(frozenWidth)
                    .end().find("#e-movableheaderdiv" + this._id).css("width", "100%");
            }
        },
        //SET THE COLUMNWIDTH TO EACH COLUMN IN TREEGRID
        setWidthToColumns: function () {

            var proxy = this,
                model = proxy.model,
                $cols1 = proxy.getContentTable().children("colgroup").find("col"),
                $cols2 = proxy.getHeaderTable().children("colgroup").find("col"),
                width = proxy._gridWidth,
                colCount = model.columns.length;

            for (var i = 0; i < $cols2.length; i++) {

                if (!ej.isNullOrUndefined(proxy.columnsWidthCollection[i])) {

                    $cols1.eq(i).width(proxy.columnsWidthCollection[i]);
                    $cols2.eq(i).width(proxy.columnsWidthCollection[i]);
                    }
            }
            /* width fro details col group cell */
            if (!model.isFromGantt) {
                if ($cols1.length > colCount) {
                    $cols1.eq(colCount).width(proxy._detailsColumnWidth);
                }

                if ($cols2.length > colCount) {
                    $cols2.eq(colCount).width(proxy._detailsColumnWidth);
                }
            }

            /* Update column width in total summary table*/
            if (model.showTotalSummary && this._$totalSummaryRowContainer) {
                var gridheaderCol = $(this.getHeaderTable()).find('colgroup');
                var footerColClone = $(gridheaderCol).clone();
                if (this._frozenColumnsLength > 0) {
                    if (this._$footertableContent) {
                        $(this._$footertableContent).find('colgroup').remove();
                        $(footerColClone[0]).prependTo(this._$footertableContent[0]);
                        $(footerColClone[1]).prependTo(this._$footertableContent[1]);
                    }
                }
                else {
                    if (this._$footertableContent) {
                        $(this._$footertableContent).find('colgroup').remove();
                        $(footerColClone).prependTo(this._$footertableContent);
                    }
                }
            }

            if (this._frozenColumnsLength > 0) {
                var frozenWidth = this._getFrozenColumnWidth();
                if (frozenWidth < this._gridWidth) {
                    this.getContent().find("#e-movablecontainer" + this._id).css("margin-left", frozenWidth + "px");
                    this.getContent().find("#e-frozencontainer" + this._id).outerWidth(frozenWidth);
                    if (this.getScrollElement().data("ejScroller")) {
                        this.getScrollElement().ejScroller("option", "width", this._gridWidth - frozenWidth - this._totalBorderWidth - 1);
                        this.getScrollElement().ejScroller("option", "scrollTop", proxy._scrollTop);
                        this.getScrollElement().ejScroller("option", "scrollLeft", proxy._scrollLeft);
                    }
                    this.getHeaderContent().find("#e-frozenheaderdiv" + this._id).next().css("margin-left", frozenWidth + "px");
                    this.getHeaderContent().find("#e-frozenheaderdiv" + this._id).outerWidth(frozenWidth)
                        .end().find("#e-movableheaderdiv" + this._id).css("width", "100%");
                    if (this._$totalSummaryRowContainer) {
                        this._$totalSummaryRowContainer.find("#e-frozenfooterdiv" + this._id).width(width).next().css("margin-left", frozenWidth + "px");
                        this._$totalSummaryRowContainer.find("#e-frozenfooterdiv" + this._id).outerWidth(frozenWidth)
                            .end().find("#e-movablefooterdiv" + this._id).css("width", "100%");
                        if (!this.mobileDevice() && this.getBrowserDetails().browser == "safari")
                            this._$totalSummaryRowContainer.find("#e-movablefooter" + this._id).css("margin-left", "auto");
                    }
                    if (!this.mobileDevice() && this.getBrowserDetails().browser == "safari")
                        this.getHeaderContent().find("#e-movableheader" + this._id).add(this.getContent().find("#e-movablecontainer" + this._id)).css("margin-left", "auto");
                }
            }
            if (model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns && !model.isFromGantt)
                this._updateTableWidth();
        },
        //Set width for table according to column resize mode
        _updateTableWidth: function (isUpdateCssOnly) {
            var totWidth = 0, $headerTable, $contentTable, $totalSummaryTable,
                model = this.model, frozenColumnWidth = this._getFrozenColumnWidth(),
                columnLength = model.columns.length;
            for (var i = 0; i < columnLength; i++) {
                if (model.columns[i].isFrozen || model.columns[i].visible != false)
                    totWidth += model.columns[i].width;
            }
            if (this._frozenColumnsLength > 0) {
                totWidth -= frozenColumnWidth;
                $headerTable = $(this.getHeaderTable()[1]);
                $contentTable = $(this.getContentTable()[1]);
                $totalSummaryTable = model.showTotalSummary ? $(this._$footertableContent[1]) : null;
            } else {
                $headerTable = this.getHeaderTable();
                $contentTable = this.getContentTable();
                $totalSummaryTable = model.showTotalSummary ? $(this._$footertableContent[0]) : null;
            }
            if (model.showDetailsRow && model.showDetailsRowInfoColumn && model.detailsTemplate)
                totWidth += this._detailsColumnWidth; // 35 denoted detail row icon column
            //when window resize no need to update the table width values
            if (!isUpdateCssOnly) {
                $headerTable.width(totWidth);
                $contentTable.width(totWidth);
                $totalSummaryTable && $totalSummaryTable.width(totWidth);
            }
            if (totWidth < (this._gridWidth - frozenColumnWidth) && model.columns[columnLength - 1].visible) {
                $headerTable.addClass('e-tableLastCell');
                if (model.currentViewData.length > 0) {
                    $contentTable.addClass('e-tableLastCell');
                }
                else
                    $contentTable.removeClass('e-tableLastCell');
                $contentTable.addClass('e-fixed-border');
             //   $totalSummaryTable && $totalSummaryTable.addClass('e-tableLastCell');
            } else {
                $headerTable.removeClass('e-tableLastCell');
                $contentTable.removeClass('e-tableLastCell');
                $contentTable.removeClass('e-fixed-border');
              //  $totalSummaryTable && $totalSummaryTable.removeClass('e-tableLastCell');
            }
        },
        _convertHtml: function (text) {
            return text
                .replace(/&/g, "&amp;")
                .replace(/</g, "&lt;")
                .replace(/>/g, "&gt;")
                .replace(/"/g, "&quot;")
                .replace(/'/g, "&#039;");
        },

        //ENTER EDIT MODE INTO THE TREEGRID CELL
        cellEdit: function (index, fieldName) {

            var proxy = this,
                $form,
                model = proxy.model,
                row, rowIndex = index,
                updatedRecords = proxy.getUpdatedRecords(), selectedItem;
            if (model.allowSelection == true) {
                selectedItem = model.selectionSettings.selectionMode == "row" ? this.selectedItem() : updatedRecords[proxy._rowIndexOfLastSelectedCell];
            }
            else {
                selectedItem = this.selectedItem();
            }
            if (proxy.model.showGridCellTooltip) {
                proxy._cellMouseLeave();
            }

            proxy.model.isEdit && proxy.saveCell();

            $form = $("#" + proxy._id + "EditForm");

            if ($form.length > 0) return true;

            if (!selectedItem) return false;

            if (model.isFromGantt && selectedItem.hasChildRecords) {
                if (fieldName === "status" ||
                (fieldName === "startDate" && selectedItem.isAutoSchedule) ||
                (fieldName === "endDate" && selectedItem.isAutoSchedule) ||
                (fieldName === "duration" && selectedItem.isAutoSchedule) ||
                 fieldName === "work" || fieldName === "effortDriven" || fieldName === "taskType" ||
                 fieldName === "predecessor" ||
                 fieldName === "taskId")
                    return false;
            }
            if (model.isFromGantt) {
                if (selectedItem.isAutoSchedule && selectedItem.taskType == "fixedWork" && fieldName == "effortDriven")
                    return false;
                if (!selectedItem.isAutoSchedule && (fieldName == "taskType" || fieldName == "effortDriven"))
                    return false;
            }
            if (model.enableVirtualization || model.selectionSettings.selectionMode === "cell" ) {
                rowIndex = updatedRecords.indexOf(selectedItem);
                index = model.currentViewData.indexOf(selectedItem);
            }

            row = ej.TreeGrid.getRowByIndex(this, rowIndex);
            if (row && $(row).find(".e-rowcell").length > 0) {
                var $targetTr = $(row),
                columnIndex = proxy.getColumnIndexByField(fieldName),
                $targetTd = $targetTr.find(".e-rowcell").eq(columnIndex),
                column = proxy.model.columns[columnIndex],
                data = selectedItem,
                cellValue = proxy._getCustomCellValue(data, column.field),
                beginEditArgs = {},
                args = {
                    validationRules: ej.isNullOrUndefined(column.validationRules) ? {} : $.extend(true, {},
                        column.validationRules),
                    columnName: column.field,
                    value: !ej.isNullOrUndefined(cellValue) ? cellValue : ej.isNullOrUndefined(data.item[fieldName]) ? null : data.item[fieldName],
                    data: data,
                    columnObject: column,
                    cell: $targetTd
                },
                isEditable = true;
                if (column.field == "notesText") {
                    args.value = !ej.isNullOrUndefined(data[fieldName]) ? data[fieldName] : data.item && data.item[fieldName];
                    args.value = args.value ? proxy._convertHtml(args.value) : args.value;
                }
                else {
                    args.value = !ej.isNullOrUndefined(cellValue) ? cellValue : ej.isNullOrUndefined(data.item[fieldName]) ? null : data.item[fieldName];
                }
            if (model.isFromGantt && args.columnName === "predecessor") {

                var predecessor = args.data.item[proxy.model.predecessorMapping];
                args.value = predecessor ? predecessor : "";

            }
            if (model.isFromGantt && args.columnName === "taskMode") {
                var taskModeValue = !args.data.isAutoSchedule;
                args.value = taskModeValue;
            }

            beginEditArgs.data = data;
            beginEditArgs.columnIndex = columnIndex;
            beginEditArgs.rowElement = $targetTr;
            beginEditArgs.cellElement = $targetTd;

          

            proxy._cellEventTrigger($targetTd, data, args.columnObject);

                if (proxy._trigger("beginEdit", beginEditArgs)) return false;               

                if ($targetTr.hasClass("e-insertedrow") && (!args.columnObject.isTemplateColumn)) isEditable = true;

                if (isEditable) {

                    $.extend(proxy._cellEditingDetails, {
                        rowIndex: index,
                        cellValue: args.value,
                        columnIndex: columnIndex,
                        fieldName: fieldName,
                        cellEditType: args.columnObject.editType,
                        data: data
                    });
                }
                args.columnIndex = columnIndex;
                proxy._renderCellEditObject(args, $targetTd);
                $targetTr.addClass("e-editedrow");
                args.cell && args.cell.addClass("e-editedcell");
                proxy._updateHorizontalScrollBar(fieldName);
                proxy._updateWrapRowHeights("total,header", null, proxy._cellEditingDetails.rowIndex, "cellEdit");
                return true;
            }
        },

        _updateHorizontalScrollBar: function (fieldName)
        {
            var proxy = this,
                gridContent = proxy.getScrollElement(),
                isHorizontalScroll = gridContent.ejScroller("isHScroll");
            //Check if the horizontal scrollbar is exists or not
            if (isHorizontalScroll) {
                var model = proxy.model,
                columns = model.columns,
                columnLength = columns.length,
                columnIndex = proxy.getColumnIndexByField(fieldName),                
                width = 0, startWidth = 0,
                endWidth = 0, left = 0,
                viewPortWidth = gridContent.find(".e-content").width(),
                scrollLeft = gridContent.ejScroller("option", "scrollLeft"),
                form = $("#" + proxy._id + "EditForm"), row,
                cells=[];                
                if (model.selectionSettings.selectionMode == "cell" && form.length == 0)
                    cells = $("#" + proxy._id).find(".e-selectingcell");//$(proxy._selectedCellDetails[proxy._selectedCellDetails.length - 1].cellElement).clone();
                else {
                    row = $(form).closest("tr.e-treegridrows");
                    cells = $(row).find("td.e-rowcell");
                }
                if (proxy._frozenColumnsLength > 0 && columnIndex < proxy._frozenColumnsLength)
                    return false;

                //Calculate the start and end width for edited cell
                for (var index = proxy._frozenColumnsLength; index <= columnIndex; index++) {
                    if (model.selectionSettings.selectionMode == "cell" && form.length == 0)
                        var cell = cells[cells.length - 1];
                    else
                        var cell = $(cells[index - proxy._frozenColumnsLength]);
                    if (index == columnIndex) {
                        startWidth = width;
                        endWidth = width + $(cell).width() + parseFloat($(cell).css("padding-left")) + parseFloat($(cell).css("padding-right")) + 1; // 1 for border
                    }
                    if (columns[index].visible) {
                        width = width + $(cell).width() + parseFloat($(cell).css("padding-left")) + parseFloat($(cell).css("padding-right")) + 1; // 1 for border
                    }
                }
                // Check if the edited cell in viewport.
                if (startWidth < scrollLeft || endWidth > scrollLeft + viewPortWidth) {
                    if (startWidth < scrollLeft) {
                        left = scrollLeft - (scrollLeft - startWidth) - 3;
                    } else {
                        left = scrollLeft + (endWidth - (scrollLeft + viewPortWidth) + 3);// 3 for slightly adjust the left scrollbar
                    }
                    gridContent.ejScroller("option", { "scrollLeft": left });
                
                }
            }
        },
        updateAllowColumnResize: function (bool) {
            var proxy = this,
                model = proxy.model;
            model.allowColumnResize = bool;
            if (model.allowColumnResize)
                proxy._resizer = new ej.gridFeatures.gridResize(proxy);
            proxy._enableColumnResizeEvents();
        },

        //Get predecessor string from serial number predecssor string
        _predecessorFromSerialPredecessor: function (serialPredecessor) {
            var proxy = this, model = proxy.model, values, serialNo,
                flatRecords = model.flatRecords, idValue, defaultPredecessor = [], defaultPredecessorString = "";
            if (!ej.isNullOrUndefined(serialPredecessor) && serialPredecessor.length > 0) {
                serialPredecessor.split(',').forEach(function (el) {
                    values = el.split('+');                  
                    if (el.indexOf('-') >= 0)
                        values = el.split('-');                        
                    serialNo = values[0].match(/(\d+|[A-z]+)/g);
                    if (serialNo && serialNo[0]) {
                        $.map(flatRecords, function (item) {
                            if (item.serialNumber == serialNo[0]) {
                                idValue = item.taskId;
                                return false;
                            }
                        });
                        idValue && defaultPredecessor.push(idValue + (serialNo[1] ? serialNo[1] : "") + (values[1] ? "+" + values[1] : ""));
                    }
                });
                defaultPredecessorString = defaultPredecessor.length && defaultPredecessor.join(",");
            }
            return defaultPredecessorString;
        },
        
        //validate predecessor string

        _validatePredecessorString: function (string) {
            var proxy = this, model = proxy.model, result, values, offsetvalue, match,
                flatDataLength = model.flatRecords.length, isExist = false;
            if (string.length > 0) {
                string.split(',').forEach(function (el) {
                    values = el.split('+');
                    offsetvalue = '+';
                    if (el.indexOf('-') >= 0) {
                        values = el.split('-');
                        offsetvalue = '-';
                    }
                    match = values[0].match(/(\d+|[A-z]+)/g);
                    if (!match)
                        return false;
                    if (model.enableSerialNumber) {
                        if (+match[0] <= flatDataLength)
                            isExist = true;
                    }
                    else {
                        if(model.ids.indexOf(match[0]) != -1)
                            isExist = true;
                    }
                    if (match && isExist) {
                        if (match[1]) {
                            if (match[1].toUpperCase() == 'FS' || match[1].toUpperCase() == 'FF' || match[1].toUpperCase() == 'SF' || match[1].toUpperCase() == 'SS') {
                            }
                            else {
                                result = false;
                            }
                        }
                    } else {
                        result = false;
                    }
                });
            } else {
                return true;
            }
            if (result === undefined || result)
                return true;
            else
                return false;
        },

        //SAVE THE EDITED CELL
        
        saveCell: function () {

            var proxy = this,
                model = proxy.model,
                $form = $("#" + proxy._id + "EditForm");

            if ($form.length > 0) {

                if (!proxy.editFormValidate()) return true;
                if (proxy._isRowEdit)
                    return;
                if (proxy._cellEditingDetails.columnIndex >= 0) {
                    proxy._focusTreeGridElement();

                    var recordIndex = proxy.getUpdatedRecords().indexOf(proxy._cellEditingDetails.data),
                        $targetTr = ej.TreeGrid.getRowByIndex(this, recordIndex),
                        $targetTd = $form.closest("td"),
                        formattedValue, args = {},
                        column = model.columns[proxy._cellEditingDetails.columnIndex],
                        $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName.split('.').join("")),
                        cellElement, isLocalData = true;

                    args = {
                        columnName: column.field,
                        value: proxy.getCurrentEditCellData(),
                        data: proxy._cellEditingDetails.data,
                        previousValue: proxy._cellEditingDetails.cellValue,
                        rowIndex: proxy._cellEditingDetails.rowIndex,
                        columnObject: column,
                        cellElement: $targetTd,
                        rowElement: $targetTr,
                    };

                    if (proxy._cellEditingDetails.cellEditType == "datetimepicker" ||
                        proxy._cellEditingDetails.cellEditType == "dropdownedit" ||
                        proxy._cellEditingDetails.cellEditType == "datepicker") {

                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");

                    }

                    if (!ej.isNullOrUndefined(column.format)) {

                        formattedValue = proxy.formatting(column.format, args.value, model.locale);
                        args.cellElement.empty().html(formattedValue);

                    } else {

                        if (args.columnObject.type == "boolean") {

                            var cellData = {};
                            cellData[args.columnObject.field] = args.value;
                            args.cellElement.empty().html(
                                $($.templates.Grid_JSONTemplate.render(cellData))[0].cells[proxy._cellEditingDetails.columnIndex].innerHTML
                            );

                        } else if (args.columnObject.field == "notesText") {
                            args.cellElement.empty().html(proxy._convertHtml(args.value));
                        }
                        else args.cellElement.empty().html(args.value);
                    }
                    var editType = proxy._cellEditingDetails.cellEditType,
                        fieldName = proxy._cellEditingDetails["fieldName"],
                        isValueModified = proxy._checkModfiedCellValue(args.previousValue, args.value, args.columnObject);


                    //To avoid duplicate ID for TreeGrid on self reference while editing ID value
                    if (!model.isFromGantt && model.idMapping && model.parentIdMapping && fieldName == model.idMapping) {
                        if (proxy._validateIdValue(args.value)) {
                            args.value = args.previousValue
                            isValueModified = false;
                        }
                    }
                    if (model.isFromGantt)
                        proxy._setGanttCellValue(args, fieldName, isValueModified);

                    else if (isValueModified || !proxy._isRemoteData) {
                        if (isValueModified && proxy.model.editSettings.editMode.toLowerCase() == "batchediting") {
                            //Take backup of original field values
                            !args.data.isBatchAdd && proxy._createBatchEditedFields(args.data, fieldName, args.value, args.columnObject);
                            proxy._batchEditStarts = true;
                        }
                        if (fieldName.indexOf(".") != -1)
                            ej.createObject(fieldName, args.value, args.data[fieldName]);
                        else
                            args.data[fieldName] = args.value;

                        args.data.item[fieldName] = args.data[fieldName]
                        args.requestType = "saveCellValue";
                        isLocalData = proxy._updateAction(args);

                    }
                    if (isLocalData)
                        proxy._refreshAfterSaveCell(args, true);
                }
            }
        },

        _createBatchEditedFields: function (data, fieldItem, newValue, columnObject) {
            var proxy = this, model = proxy.model, isDifferent;
            if (!data.batchEditedFields) {
                data.batchEditedFields = {};
                data.batchEditedFields[fieldItem] = data[fieldItem] ? data[fieldItem] : "";
            }
            else if (data.batchEditedFields && ej.isNullOrUndefined(data.batchEditedFields[fieldItem]))
                data.batchEditedFields[fieldItem] = data[fieldItem];
            else if (data.batchEditedFields && !ej.isNullOrUndefined(data.batchEditedFields[fieldItem])) {
                //To check whether edited value is same as Original value
                isDifferent = proxy._checkModfiedCellValue(data.batchEditedFields[fieldItem], newValue, columnObject);
                if (!isDifferent)
                    delete data.batchEditedFields[fieldItem];
            }
            proxy._updateBatchChanges(data);
        },

        //Update batch changes collection
        _updateBatchChanges: function (targetData) {
            var proxy = this,
                model = proxy.model,                
                batchChangedGrp = proxy.batchChanges.changed,
                batchAddedGrp = proxy.batchChanges.added;
            //Insert into batchChanges collection
            if (!$.isEmptyObject(targetData.batchEditedFields)) {
                if (batchAddedGrp.indexOf(targetData) == -1 && batchChangedGrp.indexOf(targetData) == -1) {
                    batchChangedGrp.push(targetData);
                }
            }
            else {
                if (batchChangedGrp.indexOf(targetData) != -1)
                    batchChangedGrp.splice(batchChangedGrp.indexOf(targetData), 1);
            }
        },

        _refreshAfterSaveCell: function (args, isValueModified) {
            var proxy = this, model = proxy.model,
                cellElement;

            if (this._frozenColumnsLength > 0) {
                if (!model.allowSelection)
                    args.data.isSelected = false;
                if (proxy._cellEditingDetails.columnIndex < this._frozenColumnsLength)
                    cellElement = $($.render[proxy._id + "_JSONFrozenTemplate"](args.data))[0].cells[proxy._cellEditingDetails.columnIndex];
                else
                    cellElement = $($.render[proxy._id + "_Template"](args.data))[0].cells[(proxy._cellEditingDetails.columnIndex - this._frozenColumnsLength)];
            } else {
                if (!model.allowSelection)
                    args.data.isSelected = false;
                cellElement = $($.render[proxy._id + "_Template"](args.data))[0].cells[proxy._cellEditingDetails.columnIndex];
            }
            args.cellElement.removeClass("e-editedcell").replaceWith(cellElement);
            args.cellElement = $(cellElement);

            /* refresh details Row */
            if (model.showDetailsRow && !model.showDetailsRowInfoColumn && model.detailsTemplate) {
                this._refreshDetailsRow(model.currentViewData.indexOf(args.data));
            }

            proxy._trigger("refresh");
            if (!args.data.treeMappingName)
                args.data.treeMappingName = [];
            args.data.treeMappingName.length == 0 && args.data.treeMappingName.push(args.columnName);
            if ((isValueModified || args.isModified) && model.editSettings.editMode.toLowerCase() != "batchediting" && proxy._trigger("endEdit", args)) {
                args.data[args.columnName] = args.previousValue;
                args.data.item[args.columnObject.field] = args.previousValue;
                proxy.selectRows(this.selectedRowIndex());
                ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(args.data));
            }
            else {
                if (model.idMapping && model.parentIdMapping) {
                    proxy._updateLastInsertedId(args.data[model.idMapping]);
                }
            }
            model.isEdit = false;
            proxy._cellEventTrigger(args.cellElement[0], args.data, args.columnObject);
            $(args.rowElement).removeClass("e-editedrow");

            if (model.showSummaryRow) {
                args.editType = "celledit";
                proxy._updateSummaryRow(args);
            }
            if (model.showTotalSummary) {
                args.editType = "celledit";
                proxy._updateTotalSummaryRow(args);
            }
            if (model.allowTextWrap) {
                if (proxy._$frozenTableContent.length > 0) {
                    proxy._updateRowHeight(proxy._cellEditingDetails.rowIndex, "savecell");
                    proxy._updateGridContentHeight();
                    proxy.getScrollElement().ejScroller("refresh");
                    var movableScrollTop = proxy.getScrollElement().ejScroller("model.scrollTop");
                    proxy._getFrozenScrollElement().scrollTop(movableScrollTop);
                    if (model.enableVirtualization) {
                        proxy.updateHeight();
                    }
                } else {
                    proxy.getScrollElement().ejScroller("refresh");
                }
                if (model.allowPaging && !model.sizeSettings.height) {
                    proxy.updateHeight();
                }
            }
            if (model.allowTextWrap && proxy._$frozenTableContent.length == 0) {
                var height = proxy._getViewPortHeight();
                proxy._$gridContainer.css({
                    "height": height + "px",
                });
                $("#" + proxy._id + "e-gridcontent").css("height", height)
                if (model.sizeSettings.height == "auto")
                    proxy.getScrollElement().ejScroller("refresh");
            }
            proxy._cancelSaveTools();
            var colObject = args.columnObject;
            if (model.allowFiltering && colObject && model.filterSettings.filterType == "filterbar" && colObject.filterEditType == "dropdownedit" && colObject.editType != "dropdownedit")
                proxy._resizeFilteringElements();
        },

        _setGanttCellValue: function (args, fieldName, isValueModified) {
            var proxy = this, model = proxy.model;

            if (fieldName === "taskMode") {
                isValueModified = (args.value !== args.previousValue) ? true : false;
            }

            if (fieldName === "predecessor") {

                var eventArg = {}, stringResult, logicResult;
                args.value = $.trim(args.value);
                stringResult = proxy._validatePredecessorString(args.value);
                if (stringResult) {
                    eventArg.predecessorString = args.value.split(',');
                    eventArg.currentRecord = args.data;
                    eventArg.requestType = "validatePredecessor";
                    proxy._trigger("actionComplete", eventArg);
                    logicResult = eventArg.result;
                    if (!logicResult) {
                        args.value = args.previousValue;
                    }
                }
                else {
                    args.value = args.previousValue;
                }
                var ganttPredecessor = [],
                prevPredecessors = [],
                modifiedPredecessors = [],
                count = 0,
                length, predecessor;
                args.previousValue = args.data.predecessor;
                prevPredecessors = args.previousValue;
                if (args.value.length > 0) {
                    var ganttProxy = $("#" + this._id.replace("ejTreeGrid", "")).data("ejGantt");
                    modifiedPredecessors = args.data._calculatePredecessor(args.value, model.durationUnitEditText, model.durationUnit, ganttProxy);
                }
                length = modifiedPredecessors.length;
                var index = 0,
                    prevPrdecessorlength = args.previousValue && args.previousValue.length;
                for (index = 0; index < prevPrdecessorlength; index++) {
                    predecessor = prevPredecessors[index];
                    if (predecessor.from === args.data.taskId.toString()) {
                        ganttPredecessor.push(predecessor);
                    }
                }
                for (count = 0; count < length; count++) {
                    ganttPredecessor.push(modifiedPredecessors[count]);
                }
                args.data[proxy._cellEditingDetails["fieldName"]] = ganttPredecessor;
                if (ganttProxy && !ganttProxy._isValidPredecessorString) {
                    args.value = proxy._getPredecessorsValue(args.data);
                    ganttProxy._isValidPredecessorString = true;
                }
                args.data.item[model.predecessorMapping] = args.value;
                args.data.predecessorsName = args.value;
                if (model.enableWBS && model.enableWBSPredecessor)
                    proxy.updateWBSPredecessor(args.data);

            }
            else if (fieldName === "serialNumberPredecessor") {

                var eventArg = {}, stringResult, logicResult, defaultPredecessor;
                args.value = $.trim(args.value);
                args.value = proxy._checkSerialPredecessor(args.value);
                stringResult = proxy._validatePredecessorString(args.value);
                if (stringResult) {
                    eventArg.predecessorString = args.value.split(',');
                    eventArg.currentRecord = args.data;
                    eventArg.requestType = "validateSerialPredecessor";
                    proxy._trigger("actionComplete", eventArg);
                    logicResult = eventArg.result;
                    if (!logicResult) {
                        args.value = args.previousValue;
                    }
                }
                else {
                    args.value = args.previousValue;
                }
                defaultPredecessor = proxy._predecessorFromSerialPredecessor(args.value);
                var ganttPredecessor = [],
                prevDefaultPredecessors = [],
                modifiedPredecessors = [],
                count = 0,
                length, predecessor;
                prevDefaultPredecessors = args.data.predecessor;
                args.prevDefaultPredecessors = prevDefaultPredecessors;
                args.previousValue = args.data.serialNumberPredecessor;
                if (defaultPredecessor.length > 0) {
                    var ganttProxy = $("#" + this._id.replace("ejTreeGrid", "")).data("ejGantt");
                    modifiedPredecessors = args.data._calculatePredecessor(defaultPredecessor, model.durationUnitEditText, model.durationUnit, ganttProxy);
                }
                length = modifiedPredecessors.length;
                var index = 0,
                    prevPrdecessorlength = prevDefaultPredecessors && prevDefaultPredecessors.length;
                for (index = 0; index < prevPrdecessorlength; index++) {
                    predecessor = prevDefaultPredecessors[index];
                    if (predecessor.from === args.data.taskId.toString()) {
                        ganttPredecessor.push(predecessor);
                    }
                }
                for (count = 0; count < length; count++) {
                    ganttPredecessor.push(modifiedPredecessors[count]);
                }
                args.data.predecessor = ganttPredecessor;
                args.data.item[model.predecessorMapping] = defaultPredecessor;
                args.data.predecessorsName = defaultPredecessor;
                args.data.item["serialNumberPredecessor"] = args.value;
                args.data.serialNumberPredecessor = args.value;
                if (model.enableWBS && model.enableWBSPredecessor)
                    proxy.updateWBSPredecessor(args.data);
            }
            else if (fieldName === "duration") {
                //validate text from duration field
                var val = args.value,
                duration = args.data.duration,
                durationUnit = args.data.durationUnit;

                if (val == "") {
                    duration = val;
                    if (model.allowUnscheduledTask)
                        duration = null;
                    else
                        isValueModified = false;                    
                }
                else {
                    var values = val.match(/(\d*\.*\d+|[A-z]+)/g);
                    if (values && values.length <= 2) {
                        duration = parseFloat(values[0]);
                        durationUnit = values[1] ? values[1].toLowerCase() : "";
                        if (model.durationUnitEditText.minute.indexOf(durationUnit) != -1)
                            durationUnit = ej.Gantt.DurationUnit.Minute;
                        else if (model.durationUnitEditText.hour.indexOf(durationUnit) != -1)
                            durationUnit = ej.Gantt.DurationUnit.Hour;
                        else if (model.durationUnitEditText.day.indexOf(durationUnit) != -1)
                            durationUnit = ej.Gantt.DurationUnit.Day;
                        else
                            durationUnit = model.durationUnit;
                    }
                }
                if (!isNaN(duration) && (isValueModified || args.data.durationUnit.toLowerCase() != durationUnit)) {
                    args.data.item[model.durationMapping] = args.data.duration = duration;
                    args.data.durationUnit = durationUnit;
                    isValueModified = true;
                } else {
                    args.data.item[model.durationMapping] = args.previousValue;
                    isValueModified = false;
                }

            } else if (isValueModified) {

                switch (fieldName) {

                    case "taskId":
                        args.data.item[model.idMapping] = args.value;
                        break;
                    case "taskName":
                        args.data.item[model.taskNameMapping] = args.value;
                        break;
                    case "startDate":
                    case "endDate":
                        if (!model.allowUnscheduledTask && (!args.value || args.value == "")) {
                            isValueModified = false;
                            args.value = args.previousValue;
                        }
                        else if (fieldName == "startDate")
                            args.data.item[model.startDateMapping] = args.value;
                        else if (fieldName == "endDate")
                            args.data.item[model.endDateMapping] = args.value;
                        break;
                    case "resourceInfo":
                        args.data.item[model.resourceInfoMapping] = proxy._getResourceId(args.value);
                        break;
                    case "status":
                        args.data.item[model.progressMapping] = args.value;
                        break;
                    case "notesText":
                        args.data.item[model.notesMapping] = args.value;
                        args.data["notes"] = args.value;
                        args.data["notesText"] = args.value;
                        break;
                    case "taskMode":
                        args.data.item[model.taskSchedulingModeMapping] = args.value;
                        args.data["isAutoSchedule"] = args.value == true ? false : true;
                        break;
                    case "work":
                        if (model.workMapping)
                            args.data.item[model.workMapping] = args.value;
                        else
                            args.data.item[proxy._cellEditingDetails.fieldName] = args.value;
                        break;
                    case "baselineStartDate":
                        args.data.item[model.baselineStartDateMapping] = args.value;
                        break;
                    case "baselineEndDate":
                        args.data.item[model.baselineEndDateMapping] = args.value;
                        break;
                    case "eResourceName":
                        if (args.data.eResourceTaskType == "groupTask")
                            args.data.item[model.groupNameMapping] = args.value;
                        else if (args.data.eResourceTaskType == "resourceTask")
                            args.data.item[model.resourceNameMapping] = args.value;
                        args.data.eResourceName = args.value;
                        break;
                    default:
                        args.data.item[proxy._cellEditingDetails.fieldName] = args.value;
                        break;
                }
                args.data[fieldName] = args.value;
            }

            if ((args.columnName === "startDate" ||
                args.columnName === "endDate" ||
                args.columnName === "duration" ||
                args.columnName === "taskMode" ||
                args.columnName === "predecessor" ||
                args.columnName === "serialNumberPredecessor" ||
                args.columnName == "status") ||
                args.columnName == "baselineStartDate" ||
                args.columnName == "baselineEndDate") {

                args.isModified = isValueModified;
            }
        },

        /*Check and update string case*/
        _checkSerialPredecessor: function (string) {
            var newString = "", sNo, pArray = [], values;
            if (string.length > 0) {
                string.split(',').forEach(function (pl) {
                    values = pl.split('+');
                    if (pl.indexOf('-') >= 0)
                        values = pl.split('-');
                    sNo = values[0].match(/(\d+|[A-z]+)/g);
                    pArray.push(sNo[0] + (sNo[1] ? sNo[1].toUpperCase() : "FS") + (values[1] ? "+" + values[1] : ""));
                });
                newString = pArray.join(",")
            }
            return newString;
        },
        
        /*Update WBS predecessors value*/
        updateWBSPredecessor: function (preData) {
            var proxy = this,
                model = proxy.model,
                prdc = preData["predecessorsName"],
                wbspred = null;
            if (prdc && prdc.length > 0) {
                var prdcList = prdc.split(',');
                for (var p = 0; p < prdcList.length; p++) {
                    var ref = prdcList[p].match(/(\d+|[A-z]+)/g),
                        refId = +ref[0],
                        refType = ref[1],
                        refRecord;
                    refRecord = model.flatRecords.filter(function (record) {
                        return record && record.taskId == refId;
                    });
                    var newOne;
                    if (refType)
                        newOne = refRecord[0] && refRecord[0]["WBS"] + refType;
                    else
                        newOne = refRecord[0] && refRecord[0]["WBS"];
                    if (newOne)
                        wbspred = wbspred ? (wbspred + "," + newOne) : newOne;
                }
                preData["WBSPredecessor"] = wbspred;
                preData["item"]["WBSPredecessor"] = wbspred;
            }
            else {
                preData["WBSPredecessor"] = "";
                preData["item"]["WBSPredecessor"] = "";
            }
        },

        //GET THE CURRENT CELL DATA
        getCurrentEditCellData: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                column, cellDate;

            if ($("#" + proxy._id + "EditForm").length) {

                var $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName.split('.').join("")),
                    cellValue;
                column = columns[proxy._cellEditingDetails.columnIndex];
                switch (proxy._cellEditingDetails.cellEditType) {

                    case ej.TreeGrid.EditingType.String:
                        cellValue = $element.val();
                        break;

                    case ej.TreeGrid.EditingType.Numeric:
                        cellValue = parseFloat($element.val());
                        if ((model.isFromGantt && (column.field == "work" || column.field == "status" || column.field == "duration") || column.field == "unit")) {
                            cellValue = isNaN(cellValue) ? proxy._cellEditingDetails.cellValue : cellValue;
                        } else {
                            cellValue = $element.ejNumericTextbox("model.value");
                        }
                        break;

                    case ej.TreeGrid.EditingType.Maskedit:
                        cellValue = $element.ejMaskEdit("model.value");
                        break;
                    case ej.TreeGrid.EditingType.Dropdown:
                        if (model.isFromGantt && proxy._cellEditingDetails.fieldName === "resourceInfo") {
                            cellValue = proxy._getSelectedItem($element.ejDropDownList("model.selectedItems"));
                        }
                        else if (model.isFromGantt && proxy._cellEditingDetails.fieldName === "taskMode") {
                            cellValue = $element.ejDropDownList("model.value") == "true" ? true : false;
                        }
                        else
                            cellValue = ($element.ejDropDownList("model.showCheckbox") == false && $element.ejDropDownList("model.multiSelectMode") == ej.MultiSelectMode.None) ? $element.ejDropDownList("model.itemValue") : $element.ejDropDownList("getSelectedValue");
                        break;

                    case ej.TreeGrid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:
                        if (column && !column.format) {
                            cellValue = ej.format($element.ejDatePicker("model.value"), this.model.dateFormat, this.model.locale)
                        }
                        else {
                            cellValue = $element.ejDatePicker("model.value");
                        }
                        if (cellValue == "" && model.isFromGantt)
                            cellValue = proxy._cellEditingDetails.cellValue;
                        break;

                    case ej.TreeGrid.EditingType.DateTimePicker:

                        var dateTimePickerValue = $element.ejDateTimePicker("model.value"),
                            dateTimePickerFormat = $element.ejDateTimePicker("model.dateTimeFormat"),
                            dateTimePickerLocale = $element.ejDateTimePicker("model.locale");

                        if (column && !column.format) {
                            cellValue = ej.format($element.ejDateTimePicker("model.value"), this.model.dateFormat, this.model.locale)
                        }
                        else {
                            cellValue = $element.ejDateTimePicker("model.value");
                        }
                        cellDate = ej.format($element.val(), dateTimePickerFormat, dateTimePickerLocale);
                        var tempValue = ej.format(dateTimePickerValue, dateTimePickerFormat, dateTimePickerLocale);
                        if (cellDate !== tempValue && model.isFromGantt && !model.allowUnscheduledTask)
                            cellValue = proxy._cellEditingDetails.cellValue;
                        break;
                    case "edittemplate":
                        var temp1 = column.editTemplate.read;
                        if (typeof temp1 == "string")
                            temp1 = ej.util.getObject(temp1, window);
                        cellValue = temp1($element);
                        break;

                }
                if (typeof cellValue == "string" && cellValue.length &&
                    model.columns[proxy._cellEditingDetails.columnIndex].type == "number") {

                    cellValue = parseFloat(cellValue);

                }

                return cellValue;
            }

            return null;
        },
        //Get the value from edited row

        getCurrentEditCellDataForRowEdit: function (field,type,cellIndex) {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                column, field = field.split('.').join(""), length, $element, isDialogEdit;
            if (proxy.model.editSettings.allowEditing && (proxy.model.editSettings.editMode == "dialogEditing" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog"))) {
                length = $("#" + proxy._id + "EditFormDialog").length;
                $element = $("#" + proxy._id + field);
                isDialogEdit = true;
            } else {
                length = $("#" + proxy._id + "EditForm").length;
                $element = $("#" + proxy._id + field);
            }

            if (length) {

                var cellValue, dateValue;
                column = columns[cellIndex];

                switch (type) {

                    case ej.TreeGrid.EditingType.String:
                        cellValue = $element.val();
                        break;

                    case ej.TreeGrid.EditingType.Numeric:
                        cellValue = parseFloat($element.val());
                        if ($element.data("ejNumericTextbox"))
                            cellValue = $element.ejNumericTextbox("model.value");
                        else
                            cellValue = cellValue;
                        break;

                    case ej.TreeGrid.EditingType.Dropdown:
                        if ($element.data("ejDropDownList"))
                            cellValue = ($element.ejDropDownList("model.showCheckbox") == false && $element.ejDropDownList("model.multiSelectMode") == ej.MultiSelectMode.None) ? $element.ejDropDownList("model.itemValue") : $element.ejDropDownList("getSelectedValue");
                        else
                            cellValue = $element.val();
                        if (!isDialogEdit)
                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        break;

                    case ej.TreeGrid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:

                        if ($element.hasClass("e-datepicker")) {
                            if (model.dateFormat.toLowerCase().indexOf("hh") == -1)
                                dateValue = $element.ejDatePicker("model.value");
                            else {
                                dateValue = $element.ejDateTimePicker("model.value");
                            }
                        } else {
                           dateValue = $element.val();
                        }

                        if (column && !column.format) {
                            cellValue = proxy.getFormatedDate(dateValue);
                        }
                        else {
                            cellValue = dateValue;
                        }
                       
                        if (!isDialogEdit)
                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        break;

                    case ej.TreeGrid.EditingType.DateTimePicker:

                        if ($element.hasClass("e-datetimepicker")) {
                            if (column && !column.format) {
                                cellValue = ej.format($element.ejDateTimePicker("model.value"), this.model.dateFormat, this.model.locale)
                            }
                            else {
                                cellValue = $element.ejDateTimePicker("model.value");
                            }
                        } else {
                            cellValue = $element.val();
                        }
                        if (!isDialogEdit)
                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        break;

                    case ej.TreeGrid.EditingType.Maskedit:
                        cellValue = $element.ejMaskEdit("model.value");
                        break;
                    case "edittemplate":
                        var temp1 = column.editTemplate.read;
                        if (typeof temp1 == "string")
                            temp1 = ej.util.getObject(temp1, window);
                        cellValue = temp1($element);
                        break;

                }
                if (typeof cellValue == "string" && cellValue.length &&
                    model.columns[cellIndex].type == "number") {
                    cellValue = parseFloat(cellValue);
                }

                return cellValue;
            }

            return null;
        },

        _getResourceId:function(dataSource){

            var resourceIds = [],
                count = 0, length = dataSource.length;

            for (count = 0; count < length; count++) {
                resourceIds.push(dataSource[count][this.model.resourceIdMapping]);
            }
            return resourceIds;
        },

        _getSelectedItem: function (indexArray) {

            var proxy = this, model = proxy.model,
                count = 0, index, 
                length = indexArray.length,
                resourceUnitMapping = model.resourceUnitMapping,
                resourceIdMapping = model.resourceIdMapping,
                dropDownData = model.columns[proxy._cellEditingDetails.columnIndex].dropdownData,
                selectedItemResourceInfo = model.selectedItem.resourceInfo,
                selectedItems = [];

            if (dropDownData) {

                for (count = 0; count < length; count++) {                    
                    if (selectedItemResourceInfo) {
                        for (var newIndex = 0; newIndex < selectedItemResourceInfo.length; newIndex++) {
                            if (selectedItemResourceInfo[newIndex][resourceIdMapping] === indexArray[count] + 1) {
                                index = newIndex;
                                break;
                            }
                            else
                                index = -1;
                        }
                    }
                    if (!ej.isNullOrUndefined(index) && index != -1)
                        selectedItems.push(selectedItemResourceInfo[index]);
                    else {
                        $.each(dropDownData, function (index, resourceInfo) {
                            if (indexArray[count] === index) {
                                var ganttRecordResource = $.extend({}, resourceInfo);
                                ganttRecordResource[resourceUnitMapping] = 100;
                                selectedItems.push(ganttRecordResource);
                            }
                        });
                    }
                }
            }

            return selectedItems;
        },       
        //GET THE FORMATED DATE 
        getFormatedDate: function (date) {
            return ej.format(date, this.model.dateFormat, this.model.locale);
        },
        //Get formated string from date object
        getDateFromFormat: function (date) {

            if (typeof date === "object") {
                return new Date(date);
            }
            if (date) {
                return ej.parseDate(date, this.model.dateFormat, this.model.locale) == null ?
                    new Date(date) : ej.parseDate(date, this.model.dateFormat, this.model.locale);
            }
        },

        //GET THE FORMATTED TEXT
        formatting: function (formatstring, str, locale) {
            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
            locale = ej.preferredCulture[locale] ? locale : "en-US";
            var s = formatstring;
            var frontHtmlidx, FrontHtml, RearHtml, lastidxval;
            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            FrontHtml = frontHtmlidx[0];
            RearHtml = lastidxval[1];
            if (typeof (str) == "string" && $.isNumeric(str))
                str = Number(str);
            if (formatstring.indexOf("{0:") != -1) {
                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                var formatVal = toformat.exec(formatstring);
                if (formatVal != null && str != null) {
                    if (FrontHtml != null && RearHtml != null)
                        str = FrontHtml + ej.format(str, formatVal[2], locale) + RearHtml;
                    else
                        str = ej.format(str, formatVal[2], locale);
                } else if (str != null)
                    str = str;
                else
                    str = "";
                return str;
            } else if (this.data != null && this.data.Value == null) {
                $.each(this.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });
                return s;
            } else {
                return this.data.Value;
            }
        },


        //GET THE COLUMNINDEX BY USIGN THE FIELD NAME
        getColumnIndexByField: function (fieldName) {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                column = 0;

            for (column; column < length; column++) {
                if (columns[column]["field"] == fieldName)
                    break;
            }

            return column == columns.length ? -1 : column; // if field name not available in columns collection, it return -1 value.
        },

        getUpNextVisibleColumnIndex: function (index, action) {
            var proxy = this, columns = proxy.model.columns;
            if (!index)
                index = 0;
            for (index; index <= columns.length; index++) {
                if (index == columns.length && (action == "rightArrow" || action == "moveCellRight")) {
                    index = 0;
                    proxy._focusingRowIndex += 1;
                }
                if (columns[index].visible && columns[index].allowCellSelection)
                    return columns.indexOf(columns[index]);
            }
        },
        getPreviousVisibleColumnIndex: function (index, action) {
            var proxy = this, columns = proxy.model.columns;
            if (!index)
                index = columns.length - 1;
            for (index; index < columns.length; index--) {
                if (index < 0 && (action == "leftArrow" || action == "moveCellLeft")) {
                    index = columns.length - 1;
                    proxy._focusingRowIndex -= 1;
                }
                if (columns[index].visible && columns[index].allowCellSelection)
                    return columns.indexOf(columns[index]);
            }
        },
        //GET THE BROWSER DETAILS
        getBrowserDetails: function () {
            var b = navigator.userAgent.match(/(firefox|chrome|opera|msie|safari)\s?\/?(\d+(.\d+)*)/i);
            if (!!navigator.userAgent.match(/Trident\/7\./))
                return { browser: "msie", version: jQuery.uaMatch(navigator.userAgent).version };
            return { browser: b[1].toLowerCase(), version: b[2] };
        },

        //Check whether the Browser is Mobile Device's or not.
        mobileDevice: function () {
            if (navigator.userAgent.match(/Android/i) || navigator.userAgent.match(/webOS/i)
                || navigator.userAgent.match(/iPhone/i) || navigator.userAgent.match(/iPad/i)
                || navigator.userAgent.match(/iPod/i) || navigator.userAgent.match(/BlackBerry/i)
                || navigator.userAgent.match(/Windows Phone/i)) {
                return true;
            }
            else {
                return false;
            }
        },


        //GET THE FIELDNAME OF COLUMN BY USING THE HEADERTEXT
        getFieldNameByHeaderText: function (headerText) {

            var proxy = this;

            if (ej.isNullOrUndefined(proxy._fieldColumnNames[headerText])) return null;

            return proxy._fieldColumnNames[headerText];
        },


        //CLEAR THE SELECTION

        

        clearSelection: function (index) {

            var proxy = this,
                model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords();

            if (this.selectedRowIndex() != -1 || (this.selectedRowIndex() == -1 && proxy._previousIndex != -1) || index == -1) {
                                                 //To clear the row selection of previously selected row via set model (selectedRowIndex)
                var $gridRows,
                    index;
                if (this._frozenColumnsLength) {
                    $gridRows = $(proxy.getRows()[0]).add(proxy.getRows()[1]);
                } else {
                    $gridRows = proxy.getRows();
                }     
                if (!ej.isNullOrUndefined(index) && index != -1) {
                    ej.TreeGrid.getRowByIndex(proxy, index).find(".e-selectionbackground").removeClass("e-selectionbackground").removeClass("e-active");
                    var record = updatedRecords[index];
                    record && (record.isSelected = false);
                    }
                else {

                    if ($gridRows) {
                        $gridRows.find(".e-rowcell,.e-detailsrowcell").removeClass("e-selectionbackground").removeClass("e-active");
                    }
                    var dataManager = new ej.DataManager(model.flatRecords);

                    var query = new ej.Query();
                    query.search(true, ['isSelected'], ej.FilterOperators.Equals, true);

                    var records = dataManager.executeLocal(query);
                    var length = records && records.length;

                    if (length > 0) {

                        var count = 0;

                        for (count = 0; count < length; count++) {
                            records[count].isSelected = false;                            
                        }

                    }
                }

                if (proxy._checkboxSelection) {
                    proxy.selectedRowIndex(-1);
                    model.selectedItem = null;              
                    proxy._cancelSaveTools();
                }
                  //to remove particular index for the selectedItems collection while clearing particular index
                else if (model.selectedItems.length > 0 && index != -1 && !ej.isNullOrUndefined(index)) {
                    var selectedIndex = $.inArray(updatedRecords[index], model.selectedItems);
                    if (selectedIndex != -1) {
                        model.selectedItems.splice(selectedIndex, 1);
                        updatedRecords[index].isSelected = false;
                        model.selectedItem = model.selectedItems.length > 0 ? model.selectedItems[model.selectedItems.length - 1] : null;
                        proxy.selectedRowIndex(updatedRecords.indexOf(model.selectedItem));
                    }
                }
                //Clearing the selected item and index while clearing the row selection itself
                else if (!model.selectedItems.length || (model.selectedItems.length && !proxy._multiSelectCtrlRequest) || index == -1 || ej.isNullOrUndefined(index)) {
                    proxy.selectedRowIndex(-1);
                    model.selectedItem = null;//For single selection only
                    model.selectedItems = [];
                    proxy._cancelSaveTools();
                }               

                if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible") && !$gridRows.find(".e-rowcell,.e-detailsrowcell").hasClass("e-selectionbackground") && !$("#" + proxy._id + "_selectionpopup").find(".e-rowselect").hasClass("e-spanclicked")) {
                    proxy._clearMultiSelectPopup();
                }
            }
            if (model.selectionSettings.selectionMode == "cell") {
                proxy._selectedCellDetails = [];
                model.selectedCellIndexes = [];
                $("#" + proxy._id).find(".e-selectingcell").removeClass("e-selectingcell");

                proxy._rowIndexOfLastSelectedCell = -1;
                proxy._lastSelectedCellIndex = -1;
                proxy._focusingRowIndex = -1;
            }
            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && ej.isNullOrUndefined(index) && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                proxy._clearMultiSelectPopup();
            }
        },

        //To hide the selection popup
        _clearMultiSelectPopup: function () {
            var proxy = this;
            $("#" + proxy._id + "_selectionpopup").find(".e-rowselect").removeClass("e-spanclicked");
            proxy._multiSelectPopup.hide();
            proxy._multiSelectCtrlRequest = false;
            proxy._isTouchTrigger = false;
        },

        //COUNT THE CHILDRECORDS COUNT
        getChildRecordsCount: function (record) {

            var proxy = this,
                childRecords = record.childRecords,
                count = 0,
                length = childRecords ? childRecords.length : 0;

            for (count; count < length; count++) {

                if (proxy._filteredRecords.length > 0) {

                    var recordIndex = proxy._filteredRecords.indexOf(childRecords[count]);

                    if (recordIndex != -1) {

                        ++proxy._removedCount;

                        if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                            proxy.getChildRecordsCount(childRecords[count]);
                        }

                    }
                } else {

                    ++proxy._removedCount;

                    if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                        proxy.getChildRecordsCount(childRecords[count]);
                    }

                }
            }

            return proxy._removedCount;
        },


        //GET THE EXPAND STATUS OF RECORD
        getExpandStatus: function (record) {

            var parentRecord = record.parentItem;

            if (parentRecord != null) {

                if (parentRecord.expanded === false) {
                    return false;
                } else if (parentRecord.parentItem) {

                    if (parentRecord.parentItem.expanded === false) {
                        return false;
                    } else {
                        return this.getExpandStatus(parentRecord.parentItem);
                    }

                } else return true;
            } else return true;
        },


        //RETURN THE COLUMN FIELD NAMES
        getColumnFieldNames: function () {

            var columnNames = [],
                model = this.model,
                columns = model.columns,
                count = 0,
                fields = model.searchSettings.fields,
                length = columns.length;

            if (fields.length > 0)
                return fields;

            for (count; count < length; count++) {

                if (columns[count]["field"] && (columns[count]["field"] != "resourceInfo") && (columns[count]["field"] != "predecessor")) { 
                    columnNames.push(columns[count]["field"]);
                }
                else if (columns[count]["field"] == "resourceInfo") {
                    
                    columnNames.push("resourceNames");
                }
                else if (columns[count]["field"] == "predecessor")
                {
                    
                    columnNames.push("predecessorsName");
                }

            }

            return columnNames;
        },

        _getColumnMappingNames: function () {

            var columnNames = [],
                columns = this.model.columns,
                count = 0,
                length = columns.length;

            for (count; count < length; count++) {

                if (columns[count]["field"] && (columns[count]["field"] != "resourceInfo") && (columns[count]["field"] != "predecessor")

                    && (columns[count]["field"] != "work") && (columns[count]["field"] != "taskType") && (columns[count]["field"] != "effortDriven")) {
                    columnNames.push(columns[count]["mappingName"]);
                }
                else if (columns[count]["field"] == "resourceInfo") {

                    columnNames.push("resourceNames");
                }
                else if (columns[count]["field"] == "work") {

                    columnNames.push(columns[count]["field"]);
                }
                else if (columns[count]["field"] == "taskType") {

                    columnNames.push(columns[count]["field"]);
                }
                else if (columns[count]["field"] == "effortDriven") {

                    columnNames.push(columns[count]["field"]);
                }
                else if (columns[count]["field"] == "predecessor") {

                    columnNames.push("predecessorsName");
                }

            }

            return columnNames;
        },

        //RENDERED  RECORDS
        renderRecords: function (args) {

            var proxy = this,
                model = proxy.model;

            if (!args) {

                args = {};
                if (!model.isFromGantt && model.enableCollapseAll) {
                    args.requestType = ej.TreeGrid.Actions.ExpandCollapse;                   
                    proxy._isInExpandCollapseAll = true;                                    
                }
                else
                    args.requestType = ej.TreeGrid.Actions.Refresh;

            }

            this.sendDataRenderingRequest(args);
            proxy._isInExpandCollapseAll = false;
        },       

        _shiftKeySelectCells: function (args, targetIndex) {
            var proxy = this, model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords(),
                //Get First Row Details for shift key functions.
                firstRowElementIndex = proxy._shiftKeyFirstElementDetails.firstElementRowIndex,
                firstRowCellIndex = ej.isNullOrUndefined(proxy._shiftKeyFirstElementDetails.firstElementCellIndex) ? -1 : 
                                    proxy._shiftKeyFirstElementDetails.firstElementCellIndex,
                secondRowElementIndex = args.rowIndex,
                secondRowCellIndex = args.cellIndex,
                shiftKeyRowIndexes = [firstRowElementIndex, secondRowElementIndex],
                shiftKeyCellIndexes = [firstRowCellIndex, secondRowCellIndex];

            //Sort the two Selected row and cell Indexes.
            shiftKeyRowIndexes.sort(function (a, b) {
                return (a - b);
            });
            shiftKeyCellIndexes.sort(function (a, b) {
                return (a - b);
            });
            proxy._isShiftKeyNavigation = true;
            proxy._selectedCellDetails = [];
            model.selectedCellIndexes = [];
            $("#" + proxy._id).find(".e-selectingcell").removeClass("e-selectingcell");

            for (var rowIndex = shiftKeyRowIndexes[0]; rowIndex <= shiftKeyRowIndexes[1]; rowIndex++) {
                var record = updatedRecords && updatedRecords[rowIndex],
                    currentRowIndex = model.currentViewData.indexOf(record);
                if (ej.isNullOrUndefined(record) || record.isSummaryRow)
                    continue;
                for (var cellIndex = shiftKeyCellIndexes[0]; cellIndex <= shiftKeyCellIndexes[1]; cellIndex++) {
                    proxy.selectCells([{ rowIndex: rowIndex, cellIndex: cellIndex }]);
                }
            }
        },

        selectCells: function (indexes, preservePreviousSelectedCells) {
            var proxy = this,
                model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords(),
                $gridRows = proxy.getRows(), $targetRow, targetCell, record, length = indexes.length, columns=model.columns;

            if (model.selectionSettings.selectionType == "single")
                length = 1;
            if ((!preservePreviousSelectedCells || ej.isNullOrUndefined(preservePreviousSelectedCells)) &&
                proxy._isShiftKeyNavigation == false) {
            proxy._selectedCellDetails = [];
                model.selectedCellIndexes = [];
            $("#" + proxy._id).find(".e-selectingcell").removeClass("e-selectingcell");
            }
            if (model.selectionSettings.selectionMode == "cell" && model.allowSelection && $gridRows.length > 0) {
                for (var index = 0; index < length; index++) {
                    if (indexes[index].rowIndex == -1)
                        continue;
                    var record = updatedRecords && updatedRecords[indexes[index].rowIndex],
                        currentRowIndex = model.currentViewData.indexOf(record);
                    if (ej.isNullOrUndefined(record) || record.isSummaryRow)
                        continue;
                    var frozenColumnRowIndex = indexes[index].cellIndex >= proxy._frozenColumnsLength ? 1 : 0,
                       targetCellIndex = indexes[index].cellIndex >= proxy._frozenColumnsLength ? indexes[index].cellIndex - proxy._frozenColumnsLength : indexes[index].cellIndex;
                    if (proxy._frozenColumnsLength > 0) {
                        $targetRow = $gridRows[frozenColumnRowIndex][currentRowIndex];
                    }
                    else
                        $targetRow = $gridRows[currentRowIndex];

                    if (!ej.isNullOrUndefined(record) && ej.isNullOrUndefined($targetRow))
                        $targetRow = $($.render[proxy._id + "_Template"](record))[0];

                    targetCell = $targetRow.childNodes[targetCellIndex];
                    if (indexes[index].cellIndex != -1 && columns[indexes[index].cellIndex].allowCellSelection && columns[indexes[index].cellIndex].visible &&
                           !proxy._cellSelectingEventTrigger({ rowIndex: indexes[index].rowIndex, cellIndex: indexes[index].cellIndex, rowElement: $targetRow, cellElement: targetCell })) {
                        $(targetCell).addClass("e-selectingcell");

                        var selectedCellInfo = {
                            rowIndex: indexes[index].rowIndex,
                            cellIndex: indexes[index].cellIndex,
                            cellElement: targetCell,
                            data: record
                        };
                        var currentCellIndex;
                        if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)
                            currentCellIndex = proxy._getCellIndex(proxy._selectedCellDetails, "cellElement").indexOf(targetCell);
                        else
                            currentCellIndex = proxy._selectedCellDetails.map(function (e) { return e.cellElement; }).indexOf(targetCell);

                        
                        if (currentCellIndex == -1) {
                            proxy._selectedCellDetails.push(selectedCellInfo);
                            model.selectedCellIndexes.push({ rowIndex: selectedCellInfo.rowIndex, cellIndex: selectedCellInfo.cellIndex });
                        }
                        proxy._cellSelectedEventTrigger({ rowIndex: indexes[index].rowIndex, cellIndex: indexes[index].cellIndex, rowElement: $targetRow, cellElement: targetCell }, proxy._previousSelectedCellDetails);
                        proxy._previousSelectedCellDetails = {
                            rowElement: $targetRow,
                            cellElement: targetCell,
                            cellIndex: indexes[index].cellIndex,
                            rowIndex: indexes[index].rowIndex,
                        }
                        proxy._updateHorizontalScrollBar(columns[indexes[index].cellIndex].field);
                    }
                }
            }
            if (proxy._isShiftKeyNavigation == false) {
            //For selecting multiple cells with Shift key.
            proxy._shiftKeyFirstElementDetails.firstElementRowIndex = proxy._focusingRowIndex;
            proxy._shiftKeyFirstElementDetails.firstElementCellIndex = proxy._cellIndex;
            //Get back Default values for key board navigation.           
            proxy._rowIndexOfLastSelectedCell = selectedCellInfo ? selectedCellInfo.rowIndex : proxy._rowIndexOfLastSelectedCell;
            proxy._lastSelectedCellIndex = proxy._cellIndex;
            }
        },

        //SELECT THE ROW IN TREEGRID
        selectRows: function (rowIndex, toIndex, isCheckbox) {

            var proxy = this,
                model = proxy.model;

            var $gridRows = proxy.getRows(),
            selectedItem;
            if (model.selectionSettings.selectionMode == "cell" && rowIndex != -1)
                return;
            if (ej.TreeGrid.getRowByIndex(proxy, rowIndex).hasClass("e-summaryrow"))
                return;
            if ((model.editSettings.allowEditing || model.editSettings.allowAdding) && !proxy._isRowEdit && model.editSettings.beginEditAction != "click") {
                if ($("#" + proxy._id + "EditForm").length > 0 && proxy.endEdit()) {
                    $gridRows = proxy.getRows();
                    return;
                }
            }
            if (this._frozenColumnsLength > 0)
                $gridRows = $(proxy.getRows()[0]);

             var updatedRecords = proxy.getUpdatedRecords();

            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(rowIndex)) {

                rowIndex = ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex;        

                switch (model.selectionSettings.selectionType) {

                    case ej.TreeGrid.SelectionType.Checkbox:
                        var currentRecord = updatedRecords[rowIndex];

                        proxy.clearSelection(-1)
                        if (isCheckbox) {
                            var checkboxState = (currentRecord.checkboxState == "unchecked") ? "checked" : "unchecked";
                            proxy._traverseSelection(currentRecord, checkboxState, false);
                            proxy.selectAllRows();
                        }

                        if (updatedRecords[rowIndex] && !updatedRecords[rowIndex].isSelected) {
                            if (model.allowSelection)
                                ej.TreeGrid.getRowByIndex(proxy, rowIndex).find('.e-rowcell,.e-detailsrowcell').addClass(" e-selectionbackground e-active");
                            updatedRecords[rowIndex].isSelected = true;
                            model.selectedItem = updatedRecords[rowIndex];
                        }
                        if (proxy._prevSelectedItem && proxy._prevSelectedItem != model.selectedItem) {
                            proxy._prevSelectedItem.isSelected = false;
                        }
                        proxy._prevSelectedItem = updatedRecords[rowIndex];
                        proxy.selectedRowIndex(rowIndex);
                        model.selectedItems = proxy.getSelectedRecords();
                        return false;

                        case ej.TreeGrid.SelectionType.Multiple:                        
                        if (proxy._multiSelectCtrlRequest && rowIndex != -1) {
                            var currentRecord = updatedRecords[rowIndex],
                            selectedRowIndex = $.inArray(currentRecord, model.selectedItems);
                            proxy._prevSelectedItem = updatedRecords[rowIndex];
                                if (selectedRowIndex != -1) {
                                    proxy.clearSelection(rowIndex);
                            }
                            if (selectedRowIndex == -1) {
                                model.selectedItems.push(currentRecord);
                                    if (model.allowSelection)
                                        ej.TreeGrid.getRowByIndex(proxy, rowIndex).attr("aria-selected", "true").find('.e-rowcell,.e-detailsrowcell').addClass(" e-selectionbackground e-active").attr("tabindex", "0");
                                currentRecord.isSelected = true;
                                model.selectedItem = updatedRecords[rowIndex];
                                }
                                break;
                            }
                            

                        case ej.TreeGrid.SelectionType.Single:
                            proxy.clearSelection(-1);
                            model.selectedItem = null;
                            if (rowIndex <= updatedRecords.length - 1) {
                            (rowIndex != -1) && model.selectedItems.push(updatedRecords[rowIndex]);
                            proxy._prevSelectedItem = updatedRecords[rowIndex];
                            proxy._shiftSelectedRecord = null;
                                if (model.allowSelection)
                                    ej.TreeGrid.getRowByIndex(proxy, rowIndex).find('.e-rowcell,.e-detailsrowcell').addClass(" e-selectionbackground e-active");
                                if (updatedRecords[rowIndex]) {
                                    updatedRecords[rowIndex].isSelected = true;  //Selection issue Changes
                                    model.selectedItem = updatedRecords[rowIndex];
                                }
                                
                            if (proxy._prevSelectedItem && proxy._prevSelectedItem != model.selectedItem) {
                                proxy._prevSelectedItem.isSelected = false;
                                }

                            }
                            break;
                    }
                if (model.selectedItems.length && model.selectedItems.indexOf(updatedRecords[rowIndex]) != -1) {
                    proxy._prevSelectedItem = updatedRecords[rowIndex];
                    proxy._shiftSelectedRecord = null;
                    if (this.selectedRowIndex() != rowIndex) {
                        this.selectedRowIndex(rowIndex);
                    }
              
                }
                else if (model.selectedItems.length == 0 && this.selectedRowIndex() != -1) {
                    this.selectedRowIndex(-1);
            } else {
                    this.selectedRowIndex(model.updatedRecords.indexOf(model.selectedItems[model.selectedItems.length - 1]));
                }

            } else {

                if (model.selectionSettings.selectionType == ej.TreeGrid.SelectionType.Multiple) {
                proxy.clearSelection(-1);
                   var records = rowIndex - toIndex < 0 ? updatedRecords.slice(rowIndex, toIndex + 1) : updatedRecords.slice(toIndex, rowIndex + 1),
                    row, updatedRowIndex, tempRows;
                    for (var count=0; count < records.length; count++) {
                        if (proxy.getExpandStatus(records[count])) {
                            records[count].isSelected = true;
                            updatedRowIndex = updatedRecords.indexOf(records[count]);
                            row = ej.TreeGrid.getRowByIndex(proxy, updatedRowIndex);
                    if (this._frozenColumnsLength) {
                                tempRows = $(row[0]).add(row[1]);
                                row = row[0];
                    } else {
                                tempRows = row;
                    }
                            tempRows && $(tempRows).find('.e-rowcell,.e-detailsrowcell').addClass(" e-selectionbackground e-active");
                    model.selectedItem = model.updatedRecords[updatedRowIndex];
                    this.selectedRowIndex(updatedRowIndex);
                    }
                }

                }
            }

            model.selectedItems = proxy.getSelectedRecords();
            //to multiselect chart side while rowselecting on grid side
            if(!model.isFromGantt)proxy._multiSelectCtrlRequest = false;
            if (model.isFromGantt) {
                proxy._cancelSaveTools();
            }
            return false;
        },

        /* Get all selected records from treegrid*/
        getSelectedRecords: function () {
            var proxy = this,
                model = this.model,
                filteredRecords,
                selectionType = model.selectionSettings.selectionType,
                selectedRecords = [];

            if ((model.allowFiltering && model.filterSettings.filteredColumns.length > 0) || proxy._searchString.length > 0) {
                if (model.isFromGantt || selectionType == ej.TreeGrid.SelectionType.Single)
                    filteredRecords = model.updatedRecords;
                else if (selectionType == ej.TreeGrid.SelectionType.Multiple || selectionType == ej.TreeGrid.SelectionType.Checkbox)
                    filteredRecords = proxy._flatFilteredRecords;
            }
            else
                filteredRecords = model.flatRecords;

            if (selectionType == "checkbox") {
                selectedRecords = filteredRecords.filter(function (record) {
                    return record.checkboxState == "checked";
                });
            }
            else {
                selectedRecords = filteredRecords.filter(function (record) {
                    return record.isSelected;
                });
            }
            return selectedRecords.slice();
        },
        /* Clear all selected rows and cells*/
        clearAllSelection: function () {
            var proxy = this,
                model = this.model;
            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                proxy._clearMultiSelectPopup();
            }
            proxy.selectRows(-1);
            model.selectedItem = null;

            //To clear cell Selections
            proxy._selectedCellDetails = [];
            model.selectedCellIndexes = [];            
            proxy._rowIndexOfLastSelectedCell = -1;
            proxy._lastSelectedCellIndex = -1;
            proxy._focusingRowIndex = -1;
            $("#" + proxy._id).find(".e-selectingcell").removeClass("e-selectingcell");

            proxy._cancelSaveTools();
        },
        //GET THE CURRENTCELLINDEX
        getCellIndex: function (e, target) {

            var $target = $(e.target),
                cellIndex = -1,
                model =this.model;

            if (target) $target = $(target);
            if ($target.hasClass("e-detailsrowcell") || $target.closest("td").hasClass("e-detailsrowcell"))
                return -1;
            if ($target.prop("tagName") == "TD") {
                cellIndex =  $target.index();
            } else if ($target.prop('tagName') == "DIV" && $target.hasClass('e-cell')) {

                var $td = $target.closest('td');
                cellIndex = $td.index();

            } else if ($target.prop('tagName') == "DIV" && $($target.closest('td')[0]).hasClass("e-templatecell")) {

                var $td = $target.closest('td');
                cellIndex = $td.index();
            }
            else if ($($target.closest('td')[0]).hasClass("e-templatecell") && $($target.closest("tr")[0]).hasClass("e-templaterow")) {

                var $td = $target.closest('td');
                cellIndex = $td.index();
            }
            else if ($target.parent().prop("tagName") == "TD") {
                var $td = $target.closest('td');
                cellIndex = $target.parent().index();
            }
            else if ($target.prop("tagName") == "TH" || ($target.prop("tagName") == "DIV" && $($target.closest("th")[0]).css("cursor") == "col-resize")) {
                var $th,
                    $targetElement = $target.prop("tagName") == "TH" ? $($target) : $target.closest("th");
                var location = $target[0].getBoundingClientRect();
                var scrollLeft = navigator.userAgent.indexOf("WebKit") != -1 ? document.body.scrollLeft : document.documentElement.scrollLeft;
                if (this._orgX < location.left + 5 + scrollLeft)
                    cellIndex = $targetElement.prevAll(":visible:first")[0] ? $targetElement.prevAll(":visible:first")[0].cellIndex : $targetElement.index();
                else
                    cellIndex = $targetElement.index();
            }
            else if ($target.closest("td.e-rowcell").length > 0) {
                var $td = $target.closest('td.e-rowcell');
                cellIndex = $td.index();
            }

            /*get cellIndex with frozen columns*/
            if (cellIndex != -1 && this._frozenColumnsLength > 0 && ($target.closest(".e-movablecontentdiv").length > 0 || $target.closest(".e-movableheaderdiv").length > 0 || $target.closest(".e-movablefooterdiv").length > 0))
                cellIndex += this._frozenColumnsLength;
            return cellIndex;
        },


        //GET THE CURRENT ROW INDEX
        getRowIndex: function (e) {

            var proxy = this,
                $target = $(e.target),
                $gridRows = proxy.getRows(),
                index = -1,
                model = this.model;

            if ($target.closest('.e-gridcontainer').length > 0) {
                if (this._frozenColumnsLength > 0) {
                    if ($target.closest(".e-frozencontentdiv").length > 0 || $target.closest(".e-frozencontainer").length > 0)
                        $gridRows = $($gridRows[0]);
                    else if ($target.closest(".e-movablecontentdiv").length > 0 || $target.closest(".e-movablecontainer").length > 0)
                        $gridRows = $($gridRows[1]);
                        else
                            return index;
                    }
                    index = $gridRows.index($target);
                    if (index == -1) {
                        var row = $target.closest('tr');
                        index = $gridRows.index(row);

                    }       

                return index;
            }
        },

        //Cancel the edited cell and row
        _cancelEditState: function (e) {
            var proxy = this;
            if (!ej.isNullOrUndefined(e)) {
                /*Change scrollLeft of content while header is auto scrolled on filtering element focus*/
                var scrollLeft = proxy.getScrollElement().ejScroller("option", "scrollLeft"),
                    headerScrollLeft = 0;
                if (this._frozenColumnsLength > 0)
                    headerScrollLeft = proxy._$gridHeaderContainer.find("#e-movableheader" + this._id).scrollLeft();
                else
                    headerScrollLeft = proxy._$gridHeaderContainer.scrollLeft();
                if (scrollLeft != headerScrollLeft) {
                    proxy.getScrollElement().ejScroller("option", "scrollLeft", headerScrollLeft);
                    if (proxy.model.showTotalSummary && proxy._$footerContainer) {
                        if (this._frozenColumnsLength > 0)
                            proxy._$footerContainer.find("#e-movablefooter" + this._id).scrollLeft(headerScrollLeft);
                        else
                            proxy._$footerContainer.scrollLeft(headerScrollLeft);
                    }
                }
            }
            if (proxy.model.isEdit)
                proxy.cancelEditCell();
            else if (proxy._isRowEdit)
                proxy.cancelRowEditCell();
        },

        //cancel the editMode of the cell
        cancelEditCell: function (e) {

            var proxy = this,
                model = proxy.model,
                tr,
                data = proxy.model.selectedItem,
                cell, cellIndex = proxy._cellEditingDetails.columnIndex,
                args = {}, column = model.columns[cellIndex];
            if (model.isFromGantt && !model.isEdit)
                return true;
            args = {
                columnName: column.field,
                value: proxy.getCurrentEditCellData(),
                data: proxy._cellEditingDetails.data,
                previousValue: proxy._cellEditingDetails.cellValue,
                columnObject: column,                
            };
            if (this._frozenColumnsLength > 0) {
                if (this._frozenColumnsLength <= proxy._cellEditingDetails.columnIndex) {
                    cellIndex = proxy._cellEditingDetails.columnIndex - this._frozenColumnsLength;
                    tr = $(proxy.getRows()[1])[proxy._cellEditingDetails.rowIndex];
                    if ($(tr).hasClass("e-editedrow"))
                        $(tr).removeClass("e-editedrow");
                    cell = tr.cells[cellIndex];
                    $(cell).removeClass("e-editedcell").empty().html(
                       $($.render[proxy._id + "_Template"](args.data))[0].cells[cellIndex].innerHTML
                    );
                } else {
                    tr = $(proxy.getRows()[0])[proxy._cellEditingDetails.rowIndex];
                    if ($(tr).hasClass("e-editedrow"))
                        $(tr).removeClass("e-editedrow");
                    cell = tr.cells[cellIndex];
                    $(cell).removeClass("e-editedcell").empty().html(
                      $($.render[proxy._id + "_JSONFrozenTemplate"](args.data))[0].cells[cellIndex].innerHTML
                    );
                }
            } else {
                tr = $(proxy.getRows())[proxy._cellEditingDetails.rowIndex];
                if ($(tr).hasClass("e-editedrow"))
                    $(tr).removeClass("e-editedrow");
                cell = tr.cells[cellIndex];
                $(cell).removeClass("e-editedcell").empty().html(
                    $($.render[proxy._id + "_Template"](args.data))[0].cells[cellIndex].innerHTML
                );
            }
            proxy._cellEventTrigger($(cell)[0], args.data, args.columnObject);
            proxy.model.isEdit = false;
            proxy._cancelSaveTools();
            proxy._trigger("refresh");
        },

        // To destroy the edit forms and popup menu
        _destroyWidgetElements:function()
        {
            var proxy = this, model = this.model, column = model.columns, columnLength = column.length,
                columnField;
            for (var i = 0; i < columnLength; i++) {
                columnField = proxy._getFieldValue(column[i].field,i);
                var $element = $("#" + proxy._id + columnField);
                $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
            }
        },

        //Cancel row edit.
        cancelRowEditCell: function () {

            var proxy = this, model = this.model,
            //  To delete the empty added row when allowDeleting is false.
            temp = model.editSettings.allowDeleting,
            form = $(document.getElementById(proxy._id + "EditForm")), $tr,
            frozenForm = $("#" + proxy._id + "EditFrozenForm"), $frozenTr;
            if (form.length > 0)
                this._destroyWidgetElements();
            $tr = $(form.closest("tr"));
            $frozenTr = frozenForm.closest("tr");

            var rowIndex = proxy.getIndexByRow($tr),
            data = model.currentViewData[rowIndex];
            if (rowIndex == -1 && form.length == 0)
                return;
            
            if ($tr.hasClass('e-addedrow')) {
                if (!proxy.model.editSettings.allowDeleting)
                    proxy.model.editSettings.allowDeleting = true;
            }
            
            //Cancel the edited row.
            if ($tr.hasClass('e-editedrow')) {
                $tr.removeClass("e-editedrow").empty().html(
                    $($.render[proxy._id + "_Template"](data))[0].innerHTML);
                if ($frozenTr.length > 0) {
                    $frozenTr.removeClass("e-editedrow").empty().html(
                        $($.render[proxy._id + "_JSONFrozenTemplate"](data))[0].innerHTML);
                }
                proxy._rowEventTrigger($tr, data);
            }
            else if ($tr.hasClass('e-rowedit')) {
                $tr.removeClass("e-rowedit").empty().html(
                    $($.render[proxy._id + "_Template"](data))[0].innerHTML);
                if ($frozenTr.length > 0) {
                    $frozenTr.removeClass("e-rowedit").empty().html(
                        $($.render[proxy._id + "_JSONFrozenTemplate"](data))[0].innerHTML);
                }
                proxy._rowEventTrigger($tr, data);
            }               
            //Cancel the newly added row.
            else if ($tr.hasClass('e-addedrow')) {
                proxy._isRefreshAddedRecord = false;
                proxy._isEmptyRow = true;
                proxy.deleteRow(null, true, null);
                proxy._isEmptyRow = false;
            }
            model.isEdit = proxy._isRowEdit = false;
            model.editSettings.allowDeleting = temp;
            proxy._updateWrapRowHeights("total,header", null, rowIndex);
            proxy._cancelSaveTools();
            proxy._trigger("refresh");
            if (proxy._commandColumnLength > 0) {
                var tr = ej.TreeGrid.getRowByIndex(proxy, proxy.getUpdatedRecords().indexOf(model.currentViewData[rowIndex]))
                proxy._refreshUnboundTemplate(tr);
            }
        },

        //RERENDERED THE EDITED RECORD
        refreshRow: function (index) {
            var proxy = this,
                model = proxy.model,
                updatedRecords = model.allowPaging ? proxy._updatedPageData : model.updatedRecords,
                data = updatedRecords[index], eventArgs = {},
                editIndex = proxy.getIndexByRow($("#" + proxy._id + "EditForm").closest("tr"));
            if (model.enableVirtualization || model.allowPaging)
            {
                index = model.currentViewData.indexOf(data);
            }
            if (data && index > -1) {
                ej.TreeGrid.refreshRow(this, index);

                //var currentRefreshingRow = $(proxy.getRows()[index]);

                //if (model.showDetailsRow && model.detailsTemplate && data.isDetailsExpanded) {
                //    var detailsRow = currentRefreshingRow.next("tr");
                //    if (detailsRow.hasClass("e-detailsrow"))
                //        detailsRow.remove();
                //}
                //currentRefreshingRow.replaceWith($($.render[proxy._id + "_Template"](data)));
                //proxy.setGridRows($(proxy.getContentTable().get(0).rows));

                //eventArgs.rowElement = $(proxy.getRows()[index])[0];
                //eventArgs.data = data;
                //proxy._trigger("refresh");
                //proxy._refreshedCellEventTrigger(eventArgs);

                // Check whether the edited index and refreshed index are same
                // If it is same either rowEdit or cellEdit is enable in particular row.
                // So update the _isRowEdit and isEdit value is false, and also update the toolbar item.s
                if (editIndex == index) {
                    proxy._isRowEdit = proxy.model.isEdit = false;
                    proxy._cancelSaveTools();
                }
            }
        },
        /* refresh the details row while cell editing */
        _refreshDetailsRow: function (index) {
            var proxy = this,
                model = this.model,
                data = proxy.model.currentViewData[index], eventArgs = {},
                gridRows = {};

            if (data) {
                if (this._frozenColumnsLength > 0)
                    gridRows = $(proxy.getRows()[1]);
                else
                    gridRows = $(proxy.getRows());

                var currentRefreshingRow = $(gridRows[index]).next("tr");
                if (currentRefreshingRow.hasClass("e-detailsrow")) {
                    currentRefreshingRow.remove();
                    var detailsRow = $($.render[proxy._id + "_detailRowTemplate"](data));
                    $(gridRows[index]).after(detailsRow);
                    proxy.setGridRows($(proxy.getContentTable().get(0).rows));
                    this._gridRows = this.getContentTable().get(0).rows;
                    if (this._frozenColumnsLength > 0)
                        this._gridRows = [this._gridRows, this.getContentTable().get(1).rows];
                    proxy._trigger("refresh");
                    if (model.detailsDataBound) {
                        this._trigger("detailsDataBound", { detailsElement: detailsRow, data: data });
                    }
                }
            }
        },
        //Trigger rowDataBound and queryCellInfo event when row is refreshed
        _refreshedCellEventTrigger:function(eventArgs)
        {
            var proxy = this, model = this.model;
            if (model.queryCellInfo != null || model.rowDataBound != null || model.detailsDataBound != null) {
                if (model.rowDataBound != null){
                    proxy._trigger("rowDataBound", {
                        rowElement: eventArgs.rowElement,
                        data: eventArgs.data
                    });
                }

                var tdCells = eventArgs.rowElement.cells,
                $tdRowcells = $(eventArgs.rowElement).find(".e-rowcell"),
                cellIndex = 0,
                length = tdCells.length,
                column = null,
                columns = proxy.model.columns;

                if (model.queryCellInfo != null) {

                    for (cellIndex; cellIndex < length; cellIndex++) {
                        if ($(tdCells[cellIndex]).hasClass("e-rowcell")) {
                            column = columns[$tdRowcells.index(tdCells[cellIndex])];
                        }
                        if (column) {
                            proxy._cellEventTrigger(tdCells[cellIndex], eventArgs.data, column);
                        }
                    }
                }
                if (model.detailsDataBound && $(eventArgs.rowElement).next("tr").hasClass("e-detailsrow")) {
                    var detailsRowElement = $(eventArgs.rowElement).next("tr");
                    if (this._frozenColumnsLength > 0)
                        detailsRowElement = $(detailsRowElement[1]);
                    this._trigger("detailsDataBound", { detailsElement: $(eventArgs.rowElement).next("tr"), data: eventArgs.data });
                }
            }
        },

        _refreshGridPager: function () {
            if (this.getPager() != null) {
                var proxy = this,
                    model = proxy.model,
                     pagerModel = this.getPager().ejPager("model"), args = {};
                args.currentPage = this._currentPage();
                var updatedRecord = proxy._isHierarchyFiltering ? proxy.getExpandedRecords(proxy._filterParentRecords).length : model.updatedRecords.length
                if (model.currentViewData.length == 0 && updatedRecord > 0) {
                    if (pagerModel.totalPages != 1) {
                        proxy.gotoPage(args.currentPage - 1);
                        return;
                    }                   
                }
                args.totalRecordsCount = proxy._isLoadonDemand ? proxy._pageRecordCountOnDemand :
                    proxy._isRemoteData ? model.pageSettings.totalRecordsCount : proxy._gridRecordsCount;
                this.getPager().ejPager("option", args).ejPager("refreshPager");                
                model.pageSettings.totalPages = pagerModel.totalPages || null;
                if (!proxy._isRemoteData)
                    model.pageSettings.totalRecordsCount = pagerModel.totalRecordsCount || null;
                proxy._updatePagerTextAlignment();
            }
        },
        /* get visible child record count in udpated records */
        _getVisibleChildRecordCount: function (data, count, collection) {
            var proxy = this, model = this.model, childRecords, length;

            if (data.hasChildRecords) {
                childRecords = data.childRecords;
                length = childRecords.length;
                for (var i = 0; i < length; i++) {
                    if (collection.indexOf(childRecords[i]) !== -1)
                        count++;
                    if (childRecords[i].hasChildRecords) {
                        count = proxy._getVisibleChildRecordCount(childRecords[i], count, collection);
                    }
                }
            }
            else {
                if (collection.indexOf(data) !== -1) {
                    count++;
                }
            }
            return count;
        },
        //Insert templated row in DOM for newly added record
        _renderAddedRow: function (index, data) {
            var proxy = this, model = this.model;
            proxy._updateCurrentViewData();
            if (model.enableVirtualization || model.allowPaging) {
                proxy.updateAltRow();
                var tempArgs = {};
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.sendDataRenderingRequest(tempArgs);
            }
            else {
                proxy.renderNewAddedRow(index, data);
            }
            proxy.updateHeight();
            proxy._updateFixedTableBorder();
        },

        //insert new item in DOM 
        renderNewAddedRow:function(index, data)
        {
            var proxy = this, model = this.model, length,
                gridRows = proxy.getRows(),
                addedGridRow = $.render[this._id + "_Template"](data),
                isFrozenColumn = this._frozenColumnsLength > 0,
                addedFrozenRow,
                frozenTableRows;
            if (isFrozenColumn) {
                gridRows = $(proxy.getRows()[1]);
                frozenTableRows = $(proxy.getRows()[0]);
                addedFrozenRow = $.render[this._id + "_JSONFrozenTemplate"](data);
            }


            if (!ej.isNullOrUndefined(gridRows) && gridRows.length > 0) {
                if (gridRows.length == index) {
                    var insertPositionRow = gridRows.eq(index - 1),
                        insertPositionFrRow;
                    if (isFrozenColumn)
                        insertPositionFrRow = frozenTableRows.eq(index - 1);
                    /*check whether the next row is detail row or not */
                    if($(insertPositionRow).next("tr").hasClass("e-detailsrow"))
                    {
                        $(insertPositionRow).next("tr").after(addedGridRow);
                        isFrozenColumn && $(insertPositionFrRow).next("tr").after(addedFrozenRow);
                    }else
                    {
                        $(insertPositionRow).after(addedGridRow);
                        isFrozenColumn && $(insertPositionFrRow).after(addedFrozenRow);
                    }
                } else {
                    gridRows.eq(index).before(addedGridRow);
                    isFrozenColumn && frozenTableRows.eq(index).before(addedFrozenRow);
                }
            } else {
                if (this._frozenColumnsLength > 0)
                    proxy._renderFrozenRecords();
                else {
                    proxy.getContentTable().find('tbody').empty().append(proxy._getEmptyTbody());
                    var temp = document.createElement('div');
                    proxy.getContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                    var $tbody = proxy.getContentTable().children('tbody');
                    $tbody.empty();
                    temp.innerHTML = ['<table>', $.render[proxy._id + "_Template"](proxy.model.currentViewData),
                        '</table>'].join("");
                    proxy.getContentTable().get(0).replaceChild(temp.firstChild.firstChild,
                        proxy.getContentTable().get(0).lastChild);
                }
            }
            this._gridRows = this.getContentTable().get(0).rows;
            if (this._frozenColumnsLength > 0)
                this._gridRows = [this._gridRows, this.getContentTable().get(1).rows];

            proxy.setGridRows($(proxy.getContentTable().get(0).rows));
          
        },


        
        onScrollHelper: function (scrolltop, isFromChart) {
            var proxy = this;
            proxy._isFromChart = isFromChart;
            proxy.getScrollElement().ejScroller("scrollY", scrolltop, true);
            var isHorizontalScroll = proxy.getScrollElement().ejScroller("isHScroll");
            if (!isHorizontalScroll && this.model.isFromGantt)
                proxy.getScrollElement().scrollTop(scrolltop);
        },

        isVScroll: function () {
            if (this.getScrollElement().hasClass("e-scroller")) {
                return this.getScrollElement().ejScroller("isVScroll");
            }
            else
            {
                return false;
            }
        },

        //REFRESH THE SCROLLER FOR TREEGRIDCONTENT
        refreshScroller:function(panSize){
            
            var proxy = this;
            proxy._$gridContent.removeClass("e-borderbox");
            
            var scrollTop = proxy.getScrollElement().ejScroller("option", "scrollTop");
            var scrollLeft = proxy.getScrollElement().ejScroller("option", "scrollLeft");

            proxy.getScrollElement().ejScroller("option", { "persist": true });
            proxy.getScrollElement().ejScroller("option", { "width": panSize });
            
            var maxScrollWidth = proxy.getMaxScrollWidth();
            if (scrollLeft > maxScrollWidth)
                scrollLeft = maxScrollWidth > 0 ? maxScrollWidth : 0;

            proxy._$gridContent.css("height", proxy._viewPortHeight);
            proxy.getScrollElement().ejScroller("refresh");
            var isHorizontalScroll = proxy.getScrollElement().ejScroller("isHScroll");
            proxy.getScrollElement().ejScroller("option", { "scrollTop": scrollTop });
            isHorizontalScroll && proxy.getScrollElement().ejScroller("option", { "scrollLeft": scrollLeft });
            proxy._$gridHeaderContainer.scrollLeft(scrollLeft);

            if (!isHorizontalScroll)
                this.getScrollElement().scrollTop(scrollTop);
            else {
                this.getScrollElement().scrollTop(0);
            }
        },
        getMaxScrollHeight: function () {
            var proxy = this;
            return proxy.getScrollElement().children(".e-content").children().height() - this.getScrollElement().children(".e-content").height();
        },
        getMaxScrollWidth:function()
        {
            var proxy=this;
            return proxy.getScrollElement().children(".e-content").find(".e-table").width() - this.getScrollElement().children(".e-content").width();
        },
        refreshHeight:function(){

            var proxy = this;
            proxy.getScrollElement().ejScroller("refresh");            

        },

		// Get the column using header text
        // argument headerText is column header text value

        getColumnByHeaderText: function (headerText) {
            var proxy = this;
            var length = proxy.model.columns.length;
            var columns = proxy.model.columns;
            for (var column = 0; column < length; column++) {
                if (columns[column]["headerText"] == headerText)
                    break;
            }
            return column == columns.length ? null : columns[column];
        },

        //Hide the column by using header text
        //argument headerText is column header text value
        
        hideColumn: function (headerText) {
            var proxy = this,
                model = proxy.model,
                column = this.getColumnByHeaderText(headerText),
                index = $.inArray(column, model.columns),
                hiddenColumnIndex = $.inArray(headerText, proxy._hiddenColumns);

            /* check this is single column in frozen or movable column*/
            if (column && !proxy._isColumnHidable(column))
                return;
            if (index != -1 && index != model.treeColumnIndex && hiddenColumnIndex == -1) {
                /* Cancel edited cell before sort the column*/
                proxy._cancelEditState();
                proxy._hiddenColumns.push(headerText);
                proxy.model.columns[index].visible = false;
               
				//Table column width not properly set in firefox, so just remove and add the e-table class from both header and content table
                proxy.getHeaderTable().removeClass("e-table");
                proxy.getContentTable().removeClass("e-table");
                var headerColgroup = proxy.getHeaderTable().find('colgroup').find("col");
                $(headerColgroup).eq(index).hide();
                var headerRows = proxy.getHeaderTable().find("tr").not(".e-stackedHeaderRow").find(".e-headercell");
                $(headerRows).eq(index).addClass("e-hide");
                if (model.allowFiltering && model.filterSettings.filterType == "filterbar") {
                    var headerrows = proxy.getHeaderTable().find('thead').find("th.e-filterbarcell");
                    $(headerrows).eq(index).addClass("e-hide");
                }
                if (proxy.model.columns[index].isFrozen == true) {
                    proxy.setWidthToColumns();
                }
                var gridrows = proxy.getContentTable().find("colgroup").find("col"),
                    isFrozenTable = true, cellIndex = index, footerRows;
                $(gridrows).eq(index).hide();
                gridrows = proxy.getRows();
                footerRows = this._getFooterRows();
                //hidding the specified column
                if (this._frozenColumnsLength > 0) {
                    if (index < this._frozenColumnsLength) {
                        gridrows = $(proxy.getRows()[0]);
                        footerRows = $(footerRows[0]);
                        if (proxy._gridRows && $(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                            var tempColSpan = parseInt($(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                            $(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan - 1);
                        } else if (proxy.model.currentViewData.length == 0 && (ej.isNullOrUndefined(proxy._gridRows) || proxy._gridRows.length == 0)) {
                            var tempColSpan = parseInt($(proxy.getContentTable()[0]).find('tbody').find('td').attr("colspan"));
                            $(proxy.getContentTable()[0]).find('tbody').find('td').attr("colspan", tempColSpan - 1);
                        }
                    }
                    else {
                        gridrows = $(proxy.getRows()[1]);
                        footerRows = $(footerRows[1]);
                        isFrozenTable = false;
                        cellIndex = index - this._frozenColumnsLength;
                        if (proxy._gridRows && $(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                            var tempColSpan = parseInt($(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                            $(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan - 1);
                        } else if (proxy.model.currentViewData.length == 0 && (ej.isNullOrUndefined(proxy._gridRows) || proxy._gridRows.length == 0)) {
                            var tempColSpan = parseInt($(proxy.getContentTable()[1]).find('tbody').find('td').attr("colspan"));
                            $(proxy.getContentTable()[1]).find('tbody').find('td').attr("colspan", tempColSpan - 1);
                        }
                    }
                } else if (proxy.model.currentViewData.length == 0 && (ej.isNullOrUndefined(proxy._gridRows) || proxy._gridRows.length == 0)) {
                    proxy.getContentTable().find('tbody').find('td').attr("colspan", (proxy.model.columns.length - proxy._hiddenColumns.length));
                } else {
                    if (proxy._gridRows && $(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                        var tempColSpan = parseInt($(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                        $(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan - 1);
                    }
                }
                
                gridrows.each(function () {
                    var td = $(this).children("td").eq(cellIndex);
                    $(td).addClass("e-hide");
                    if (model.showSummaryRow && $(td).hasClass('e-summaryrowcell') && isFrozenTable)
                        proxy._updateSummaryRowTitleCell(headerColgroup, td[0].parentElement);
                });
                //Hide the specific column from total summary row.
                if (model.showTotalSummary) {
                    var footerColGroup = proxy._$footertableContent.find("colgroup").find("col");
                    $(footerColGroup).eq(index).hide();
                    footerRows.each(function () {
                        var footerSummarytd = $(this).children("td").eq(cellIndex);
                        $(footerSummarytd).addClass("e-hide");
                        if ($(footerSummarytd).hasClass('e-footersummaryrowcell'))
                            proxy._updateSummaryRowTitleCell(footerColGroup, footerSummarytd[0].parentElement);
                    });
                }
                proxy.getHeaderTable().addClass("e-table");
                proxy.getContentTable().addClass("e-table");
                var gridContent = proxy.getScrollElement();
                if (model.isFromGantt) {
                    proxy._trigger("refreshScrollCss");
                }
                if (model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns && !model.isFromGantt)
                    this._updateTableWidth();
                gridContent.ejScroller("refresh");
                proxy._updateScrollCss();                
                proxy._updateHeaderScrollLeft();
                /* update scrollTop in gantt when treegrid horizontal scroller not exist*/
                if (model.isFromGantt && !gridContent.ejScroller("isHScroll")) {
                    gridContent.scrollTop(gridContent.data("ejScroller").scrollTop());
                }
                index = $.inArray(headerText, proxy._visibleColumns);
                if (index != -1)
                    proxy._visibleColumns.splice(index, 1);
                if (model.showColumnChooser || model.contextMenuSettings.showContextMenu) {
                    proxy._renderColumnChooserList(true, headerText);
                    proxy._updateColumnMenuVisibility();
                }
                proxy._addInitTemplate();
                if (model.showTotalSummary && model.collapsibleTotalSummary) {
                    proxy._updateSummaryTitleColumn();
                }
                if (model.editSettings.allowEditing && model.editSettings.editMode == "dialogEditing" && (ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) || model.editSettings.dialogEditorTemplateID == "")) {
                    proxy._treeGridAddEditDialog();
                } else if (model.editSettings.allowEditing && model.editSettings.editMode == "dialogEditing" && (!ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) && model.editSettings.dialogEditorTemplateID != "")) {
                    proxy._treeGridAddEditDialogTemplate();
                }
            }
            if (this.model.showStackedHeader)
                this._refreshStackedHeader();
            //Updating the column properties to Gantt's model.columns
            if (model.isFromGantt)
                proxy.updateToGanttColumns();
        },

        //Clear the filter based on fieldName.
        //clearFilter method without argumentment means, it clear the all filter in tree grid.
        clearFilter: function (fieldName) {
            var proxy = this,
                model = proxy.model;
            if (ej.isNullOrUndefined(fieldName)) {
                var filteredColumns = model.filterSettings.filteredColumns,
                    length = filteredColumns && filteredColumns.length;
                //Loop for clearing all the filter in tree grid
                while (filteredColumns && filteredColumns.length > 0) {
                    proxy.clearFilter(filteredColumns[0].field);
                }
                proxy._filteredColumnValue = [];
                proxy._filteredColumnValueID = [];
            } else {
                var column = proxy.getColumnByField(fieldName),
                    filteredColumn = $.grep(model.filterSettings.filteredColumns, function (filterColumn, index) {
                        return filterColumn.field === fieldName;
                    });
                if (column && filteredColumn.length > 0) {
                    /*Skip the filterColumn method for numeric textbox in immediate filterbar mode.
                      Because while set the empty value to numeric text box, it automatically triiger filterBarHanlder method*/
                    if (!(model.filterBarMode == "immediate" && column.filterEditType == "numericedit")) {
                        proxy._filterString = "";
                        proxy._fieldName = "";
                        proxy.filterColumn(fieldName, "equal", "", "and");
                        proxy._storePreviousFilteredValue(proxy._fieldName, proxy._filterString, this._operator);
                    }
                    proxy._clearFilterElementValue(column);
                    if ((model.filterSettings.filterType == "menu" && column.filterType != "excel") || (model.filterSettings.filterType == "excel" && column.filterType == "menu")) {
                        var filteredColumnIdIndex = proxy._filteredColumnValueID.indexOf(column.field);
                        proxy._filteredColumnValue.splice(filteredColumnIdIndex, 1);
                        proxy._filteredColumnValueID.splice(filteredColumnIdIndex, 1);
                    }
                    else if(model.filterSettings.filterType == "excel" && column.filterType !="menu" || (model.filterSettings.filterType == "menu" && column.filterType == "excel"))
                    {
                        var filteredColumnIdIndex = proxy._filteredColumnValueID.indexOf(column.field);
                        proxy._filteredColumnValueID.splice(filteredColumnIdIndex, 1);
                    }
                    proxy._isExcelFilter && proxy._excelFilter._predicates[0]&& delete proxy._excelFilter._predicates[0][column.field];
                    !model.showColumnChooser && proxy.getHeaderTable().find("div[data-ej-mappingname='"+column.field+"']").next(".e-filtericon").removeClass("e-column-filtered-icon").addClass("e-column-filter-icon");
                }
            }
        },

        //Get the column details based on the given field in tree grid
        getColumnByField: function (fieldName) {
            var proxy = this,
              model = proxy.model,
              columns = model.columns;
            for (var index = 0; index < columns.length; index++) {
                if (columns[index]["field"] == fieldName)
                    break;
            }
            return index == columns.length ? null : columns[index]; //If field name not available in column collection, it returns null value.
        },

        //Set the empty value to filter input elements
        _clearFilterElementValue: function (column) {
            var proxy = this, columnField = column.field.split('.').join(""),
                element = $("#" + proxy._id + "_" + columnField + "_filterbarcell");
            if (element && element.length) {
                switch (column.filterEditType) {
                    case "stringedit":
                        $(element).val("");
                        break;
                    case "booleanedit":
                        $(element).prop("checked", false);
                        break;
                    case "numericedit":
                        $(element).ejNumericTextbox("instance").option("value", "");
                        break;
                    case "dropdownedit":
                        $(element).ejDropDownList("instance").option("value", "");
                        break;
                    case "datepicker":
                        $(element).ejDatePicker("instance").option("value", "");
                        break;
                    case "datetimepicker":
                        $(element).ejDateTimePicker("instance").option("value", "");
                        break;
                }
            }
        },

        //To maintain the summary row title after hide/show the column using columnChooser.
        _updateSummaryRowTitleCell: function (colgroup, summaryRow) {
            var length = colgroup.length;
            for (var cell = 0; cell < length; cell++) {
                if ($(colgroup[cell]).css('display') == 'none')
                    continue;
                var summaryTitleElement = $(summaryRow).find(".e-summarytitle")[0],
                    currentColumnSummaryTitle = $(summaryRow.childNodes[cell]).find(".e-summarytitle")[0];
                if (currentColumnSummaryTitle)
                    return;
                else
                    $(summaryRow.childNodes[cell]).prepend(summaryTitleElement);
                return;

            }

        },
        //to update the frozen column width on resizing
        _updateFrozenColumnOnResize: function (index) {
            var proxy = this,
                model = proxy.model,
                columns = model.columns;
            var frozenWidth = this._getFrozenColumnWidth();
            if (frozenWidth + 18 > this._gridWidth) {
                var max_width = 0,
                    max_index = -1;
                for (var i = 0; i < proxy._frozenColumnsLength; i++) {
                    if (columns[i].visible == true) {
                        if (max_width < columns[i].width) {
                            max_width = columns[i].width;
                            max_index = i;
                        }
                    }
                }
                //update max column width
                if (columns[index].width < (max_width - model.commonWidth)) {
                    if (columns[max_index].width == max_width) {
                        columns[max_index].width = proxy.columnsWidthCollection[max_index] = columns[max_index].width - columns[index].width;
                    }
                }
                else {
                    //if column doesnot have max width reduce all frozen widths
                    if (model.columns[index].width > model.commonWidth)
                        model.columns[index].width = model.commonWidth;
                    var modifiedFrozenWidth = Math.round(columns[index].width / proxy._frozenColumnsLength);
                    for (var i = 0; i < proxy._frozenColumnsLength; i++) {
                        if (columns[i].isFrozen == true && columns[i].visible == true) {
                            columns[i].width = proxy.columnsWidthCollection[i] = columns[i].width - modifiedFrozenWidth;
                        }
                    }
                }

            }            
        },

        //Show the column by using header text
        // argument headerText is column header text value

        
        showColumn: function (headerText) {
            var proxy = this,
                model = proxy.model,
                column = this.getColumnByHeaderText(headerText),
                index = $.inArray(column, model.columns),
                visibleColumnIndex = $.inArray(headerText, proxy._visibleColumns);
            if (index != -1 && visibleColumnIndex == -1) {
                /* Cancel edited cell before sort the column*/
                proxy._cancelEditState();
                proxy._visibleColumns.push(headerText);
                proxy.model.columns[index].visible = true;
                var headerColgroup = proxy.getHeaderTable().find('colgroup').find("col");
                $(headerColgroup).eq(index).show();
                var headerRows = proxy.getHeaderTable().find("tr").not(".e-stackedHeaderRow").find(".e-headercell");
                $(headerRows).eq(index).removeClass("e-hide");
                if (model.allowFiltering && model.filterSettings.filterType == "filterbar") {
                    var headerrows = proxy.getHeaderTable().find('thead').find("th.e-filterbarcell");
                    $(headerrows).eq(index).removeClass("e-hide");
                }
                if (proxy.model.columns[index].isFrozen == true) {
                    proxy._updateFrozenColumnOnResize(index);                  
                    proxy.setWidthToColumns();
                }               
                var gridrows = proxy.getContentTable().find("colgroup").find("col"),
                    isFrozenTable = true, cellIndex = index, footerRows;
                $(gridrows).eq(index).show();
                gridrows = proxy.getRows();
                footerRows = this._getFooterRows();

                if (this._frozenColumnsLength > 0) {
                    if (index < this._frozenColumnsLength) {
                        gridrows = $(proxy.getRows()[0]);
                        footerRows = $(footerRows[0]);
                        if (proxy._gridRows && $(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                            var tempColSpan = parseInt($(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                            $(proxy.getTreeGridRows()[0]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan + 1);
                        }
                    }
                    else {
                        gridrows = $(proxy.getRows()[1]);
                        footerRows = $(footerRows[1]);
                        isFrozenTable = false;
                        cellIndex = index - this._frozenColumnsLength;
                        if (proxy._gridRows && $(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                            var tempColSpan = parseInt($(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                            $(proxy.getTreeGridRows()[1]).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan + 1);
                        }
                    }
                } else {
                    if (proxy._gridRows && $(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").length > 0) {
                        var tempColSpan = parseInt($(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan"));
                        $(proxy.getTreeGridRows()).find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", tempColSpan + 1);
                    }
                }


				//Display the specified column
                gridrows.each(function () {
                    var td = $(this).children("td").eq(cellIndex);
                    $(td).removeClass("e-hide");
                    if (model.showSummaryRow && $(td).hasClass('e-summaryrowcell') && isFrozenTable)
                        proxy._updateSummaryRowTitleCell(headerColgroup, td[0].parentElement);
                });
                //Show the specific column from total summary row.
                if (model.showTotalSummary) {
                    var footerColGroup = proxy._$footertableContent.find("colgroup").find("col");
                    $(footerColGroup).eq(index).show();
                    footerRows.each(function () {
                        var footerSummarytd = $(this).children("td").eq(cellIndex);
                        $(footerSummarytd).removeClass("e-hide");
                        if ($(footerSummarytd).hasClass('e-footersummaryrowcell'))
                            proxy._updateSummaryRowTitleCell(footerColGroup, footerSummarytd[0].parentElement);
                    });
                }
                proxy._addInitTemplate();
                if (model.showTotalSummary && model.collapsibleTotalSummary) {
                    proxy._updateSummaryTitleColumn();
                }
                var gridContent = proxy.getScrollElement();
                if (model.isFromGantt) {
                    proxy._trigger("refreshScrollCss");
                }
                if (model.columnResizeSettings.columnResizeMode == ej.TreeGrid.ColumnResizeMode.FixedColumns && !model.isFromGantt)
                    this._updateTableWidth();
                gridContent.ejScroller("refresh");
                proxy._updateScrollCss();
                proxy._updateHeaderScrollLeft();
                /* update scrollTop in gantt when treegrid horizontal scroller not exist*/
                if (model.isFromGantt && !gridContent.ejScroller("isHScroll")) {
                    gridContent.scrollTop(gridContent.data("ejScroller").scrollTop());
                }
                index = $.inArray(headerText, proxy._hiddenColumns);
                if (index != -1)
                    proxy._hiddenColumns.splice(index, 1);
                if (model.showColumnChooser || model.contextMenuSettings.showContextMenu) {
                    proxy._renderColumnChooserList(true, headerText);
                    proxy._updateColumnMenuVisibility();
                }
                if (model.editSettings.allowEditing && model.editSettings.editMode == "dialogEditing" && (ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) || model.editSettings.dialogEditorTemplateID == "")) {
                    proxy._treeGridAddEditDialog();
                } else if (model.editSettings.allowEditing && model.editSettings.editMode == "dialogEditing" && (!ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) && model.editSettings.dialogEditorTemplateID != "")) {
                    proxy._treeGridAddEditDialogTemplate();
                }
            }
            if (this.model.showStackedHeader)
                this._refreshStackedHeader();
            //Updating the column properties to Gantt's model.columns
            if (model.isFromGantt)
                proxy.updateToGanttColumns();
        },

        // Get the hidden columns detail in tree grid
        getHiddenColumns: function () {
            var proxy = this,
                model = proxy.model,
                hiddenColumn = proxy._hiddenColumns,
                length = hiddenColumn.length,
                hiddenColumns = [];
            for (var i = 0; i < length; i++)
                hiddenColumns.push(proxy.getColumnByHeaderText(hiddenColumn[i])); //Get the column by using header text
            return hiddenColumns;

        },

        // Get the column details based on the given column index in tree grid
        getColumnByIndex: function (index) {
            var proxy = this,
                model = proxy.model,
                columns = model.columns;
            if (columns.length > index)
                return columns[index];
            return null;
        },

        // Get the visible columns detail in tree grid
        getVisibleColumns: function () {
            var proxy = this,
                model = proxy.model,
                visibleColumn = proxy._visibleColumns,
                length = visibleColumn.length,
                visibleColumns = [];
            for (var i = 0; i < length; i++)
                visibleColumns.push(proxy.getColumnByHeaderText(visibleColumn[i])); //Get the column by using header text
            return visibleColumns;

        },

        _getLastVisibleColumnIndex:function(){
            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                length = columns.length - 1,
                index = -1;
            for (var i = length; i >= 0; i--) {
                if (columns[i].visible != false) {
                    index = i;
                    break;
                }
            }
            return index;
        },
        //Cancel the edit cell or row based on editing.
        cancelEdit: function () {
            if (this._isRowEdit)
                this.cancelRowEditCell();
            else if (this.model.isEdit)
                this.cancelEditCell();
        },

        //Rerender the tree grid content.
        redraw: function () {
            this.resetModelCollections();
            this._clearColumnMenu();
            this._clearContextMenu();
            this._removeDetailsRow();
            this.element.empty();
            /*Remove all events binded in treegrid*/
            this.element.off();
            var matched = jQuery.uaMatch(navigator.userAgent),
                proxy = this, model = this.model;

            proxy._off(proxy.element, "click", proxy._onClick);
            proxy._off(proxy.element, "keydown", proxy._keyDown);
            proxy._off(proxy.element, "click", "#" + proxy._id + "e-gridheader", proxy._onHeaderClick);
            proxy._off(proxy.element, "mousedown touchstart", "#" + proxy._id + "e-gridheader", proxy._headerMouseDown);
            proxy._off($(document), "mousedown touchstart", proxy._mouseDownHandler);
            proxy._off(proxy.element, "mousedown touchstart", ".e-gridcontent", proxy._contentMouseDown);
            proxy._off(proxy.element, "click", proxy._onRecordClick);// ".e-gridcontent",
            proxy._off(proxy.element, "dblclick", ".e-gridcontent > div", proxy._onRecordDoubleClick);

            //Method binding for filtering data.
            if (model.filterSettings.filterBarMode == "immediate") {
                proxy._off(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox")._off(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
            }
            proxy._off(proxy.element, "focus click", ".e-filterbarcell input", proxy._filterBarClose);
            proxy._off(proxy.element, "click", ".e-filterbarcell .e-cancel", proxy._filterBarClose);
            if (proxy.model.showGridCellTooltip && !proxy.model.showGridExpandCellTooltip) {

                if (ej.browserInfo().name == "msie") {
                    proxy._off(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
                }
                else {
                    proxy._off(proxy.element, "mousemove", ".e-rowcell", proxy._mouseHover);
                }
            }
            else {
                proxy._off(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
            }
            if (proxy.model.showGridCellTooltip) {

                if (ej.browserInfo().name == "msie") {
                    proxy._off(proxy.element, "mouseenter", ".e-headercelldiv", proxy._headermouseHover);
                }
                else {
                    proxy._off(proxy.element, "mousemove", ".e-headercelldiv", proxy._headermouseHover);
                }
            }
            proxy._off(proxy.element, "mouseleave", ".e-rowcell", proxy._cellMouseLeave);
            proxy._off(proxy.element, "mouseleave", proxy._cellMouseLeave);
            proxy._off(proxy.element, "mouseleave", ".e-headercelldiv", proxy._headerMouseLeave);
            proxy._off(proxy.element, "mouseleave", proxy._headerMouseLeave);

            //MOUSE WHEEL EVENT FOR GANTT CONTROL
            if (model.isFromGantt || this._frozenColumnsLength) {
                proxy._off(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);
            }

            if (model.enableResize && model.isResponsive && !model.isFromGantt) {
                proxy._off($(window), "resize", proxy._windowResize);
            }
            if (ej.isTouchDevice())
                proxy._off(proxy.element, "doubletap ", ".e-gridcontent", proxy._editdblClickHandler);
            else
                proxy._off(proxy.element, "dblclick ", ".e-gridcontent", proxy._editdblClickHandler);
            //Mouse right click event for tree grid


            if (model.contextMenuSettings.showContextMenu) {
                proxy._off(proxy.element, "keyup contextmenu taphold", this._preventContextMenu);
            }                        
            
            /*Redraw all elements in treegrid toolbar, header,content and summary*/
            this._init();
        },

        // Public method for removing column menu
		clearColumnMenu:function()
        {
            var proxy=this;
            proxy._clearColumnMenu();
		},

		"export": function (action, serverEvent, multipleExport) {
		    var proxy = this,
                model = proxy.model;
		   
		    proxy.contextMenuOperations("Cancel"); // Cancel the edit row or empty newly added row.
		    var modelClone = proxy._getExportModel(model);
		    var attr = { action: action, method: 'post', "data-ajax": "false" };
		    $('form#' + proxy._id + 'export').remove();
		    var form = ej.buildTag('form#'+proxy._id+'export', "", null, attr);
		    if (ej.raiseWebFormsServerEvents) {
		        var args = { model: modelClone, originalEventType: serverEvent };
		        var clientArgs = { model: JSON.stringify(modelClone) };
		        ej.raiseWebFormsServerEvents(serverEvent, args, clientArgs);
		    }
		    else {
		        var treegridObjectArray = {};
		        if (multipleExport) {
		            $('body').find('.e-treegrid').each(function (index, object) {
		                var gridObject = $(object).data('ejTreeGrid');
		                if (!ej.isNullOrUndefined(gridObject)) {
		                    var gridmodel = gridObject.model,
		                        modelClone = proxy._getExportModel(gridmodel);
		                        treegridObjectArray[index] = JSON.stringify(modelClone);
		                        var inputAttr = { name: 'TreeGridModel', type: 'hidden', value: JSON.stringify(modelClone) };
		                        var input = ej.buildTag('input', "", null, inputAttr);
		                        form.append(input);
		                }
		            });
		        }
		        else {
		            var inputAttr = { name: 'TreeGridModel', type: 'hidden', value: JSON.stringify(modelClone) };
		            var input = ej.buildTag('input', "", null, inputAttr);
		            form.append(input);
		            form.append(this);
		        }
		        $('body').append(form);
		        form.submit();
		    }
		    return true;
		},

        //#endregion

        //#region PRIVATE METHODS

		_getExportModel: function (treegridModel) {
		    var proxy = this,
		        model = treegridModel,
		        updatedRecords = model.updatedRecords,
                currentViewData = model.currentViewData,
                flatRecords = model.flatRecords,
                parentRecords = model.parentRecords,
                summaryRowRecords = model.summaryRowRecords,
                selectedItem = model.selectedItem,
                selectedItems = model.selectedItems;
		    // Delete the below internal items in model for avoid the circular reference while extend the model
		    delete model.updatedRecords;
		    delete model.currentViewData;
		    delete model.flatRecords;
		    delete model.parentRecords;
		    delete model.summaryRowRecords;
		    delete model.selectedItem;
		    delete model.selectedItems;
		    var modelClone = $.extend(true, {}, model);
		    model.updatedRecords = updatedRecords;
		    model.currentViewData = currentViewData;
		    model.flatRecords = flatRecords;
		    model.parentRecords = parentRecords;
		    model.selectedItem = selectedItem;
		    model.selectedItems = selectedItems;
		    model.summaryRowRecords = summaryRowRecords;
		    var columns = modelClone.columns,
                columnLength = columns.length,
                stackedHeaderRows = modelClone.stackedHeaderRows,
                filteredColumns = modelClone.filterSettings.filteredColumns,
                filterColumnLength = filteredColumns.length;
		    for (var i = 0; i < columnLength; i++) {
		        if (modelClone.columns[i].editType != undefined) {
		            switch (columns[i].editType) {
		                case "stringedit":
		                case "edittemplate":
		                    columns[i].editType = "string";
		                    break;
		                case "numericedit":
		                    columns[i].editType = "numeric";
		                    break;
		                case "dropdownedit":
		                    columns[i].editType = "dropdown";
		                    break;
		                case "booleanedit":
		                    columns[i].editType = "boolean";
		                    break;
		                default:
		                    break;
		            }
		        }
		        if (columns[i].filterEditType != undefined) {
		            switch (columns[i].filterEditType) {
		                case "stringedit":
		                    columns[i].filterEditType = "string";
		                    break;
		                case "numericedit":
		                    columns[i].filterEditType = "numeric";
		                    break;
		                case "dropdownedit":
		                    columns[i].filterEditType = "dropdown";
		                    break;
		                case "booleanedit":
		                    columns[i].filterEditType = "boolean";
		                    break;
		                default:
		                    break;
		            }
		        }
		    }
		    for (var i = 0; i < filterColumnLength; i++) {
		        if (filteredColumns[i].operator == "equal")
		            filteredColumns[i].operator = "equals";
		        else if (modelClone.filterSettings.filteredColumns[i].operator == "notequal")
		            filteredColumns[i].operator = "notequals";
		    }
		    if (model.showStackedHeader && stackedHeaderRows.length > 0) {
		        for (var row = 0; row < stackedHeaderRows.length; row++) {
		            var sColumn = stackedHeaderRows[row].stackedHeaderColumns;
		            for (var col = 0; col < sColumn.length; col++) {
		                var _column = Array.isArray(sColumn[col].column) ? sColumn[col].column : $.map(sColumn[col].column.split(","), $.trim);
		                sColumn[col].column = _column;
		            }
		        }
		    }
		    var ignoreOnExport = proxy.ignoreOnExport,
                igonreLength = ignoreOnExport.length;
		    if (proxy.ignoreOnExport) {
		        for (var i = 0; i < igonreLength; i++) {
		            delete modelClone[ignoreOnExport[i]];
		        }
		    }
		    return modelClone;
		},

		_getNonContentHeight: function(){
		    var proxy = this, model = proxy.model,
                height = 0, borderTopWidth = 0;
		    borderTopWidth = proxy.model.isFromGantt ? 0 : 1;
		    height = proxy._$gridHeaderContent.outerHeight() + parseFloat(borderTopWidth);
		    if (proxy.model.toolbarSettings.showToolbar) {
		        height += !ej.isNullOrUndefined($("#" + proxy._id + "_toolbarItems").outerHeight()) ?$("#" + proxy._id + "_toolbarItems").outerHeight():0;
		    }
		    if (model.allowPaging)
		        height += proxy.element.find('.e-pager').outerHeight();		    	    

		    return height;
                
		},
        //CALCULATE THE VIEWPORTHEIGHT
		_getViewPortHeight: function () {
		    var proxy = this, model = proxy.model, height;
		    if (model.sizeSettings.height == "auto") {
		        if (proxy._frozenColumnsLength > 0) {
		            height = $("#" + proxy._id + "e-gridcontent").find(".e-hscrollbar").length == 1 ? $("#" + proxy._id + "movablee-table").height() + 18 : $("#" + proxy._id + "movablee-table").height();
		        }
		        else {
		            height = $("#" + proxy._id + "e-gridcontent").find(".e-hscrollbar").length == 1 ? $("#" + proxy._id + "e-table").height() + 18 : $("#" + proxy._id + "e-table").height();
		        }
		        return height;
		    } else {
		        return (this.element.height() - this._getNonContentHeight());
		    }
		},

        updateViewPortHeight: function () {
            var proxy = this;
            proxy._viewPortHeight = proxy._getViewPortHeight();
        },

        _updateScrollCss: function () {
            var proxy = this,
                model = proxy.model;
            if (!model.isFromGantt) {
                var isVscroll = proxy.isVScroll(),
                    isPaddingExist = proxy._$gridHeaderContent.hasClass("e-scrollcss");
                var isSafari = /Safari/.test(navigator.userAgent) && /Apple Computer/.test(navigator.vendor);
                if (isSafari)
                    proxy._$gridHeaderContent.removeClass("e-gridheader");
                if (isVscroll) {
                    if (!this.mobileDevice())
                    proxy._$gridHeaderContent.addClass("e-scrollcss");
                    if (model.showTotalSummary && proxy._$totalSummaryRowContainer && proxy._$totalSummaryRowContainer.height() <= model.totalSummaryHeight && proxy._$totalSummaryRowContainer.find(".e-vscrollbar").length == 0) {
                        if (!this.mobileDevice()) {
                            proxy._$totalSummaryRowContainer.addClass("e-scrollcss");                            
                            proxy._$totalSummaryRowContainer.width(proxy._$gridContent.width() - 17);
                        }
                        else
                            proxy._$totalSummaryRowContainer.width(proxy._$gridContent.width());
                    }
                } else {
                    proxy._$gridHeaderContent.removeClass("e-scrollcss");
                    if (model.showTotalSummary && proxy._$totalSummaryRowContainer && proxy._$totalSummaryRowContainer.height() <= model.totalSummaryHeight) {
                        proxy._$totalSummaryRowContainer.removeClass("e-scrollcss");
                        proxy._$totalSummaryRowContainer.width(proxy._$gridContent.width());
                    }
                }
                if (proxy.getScrollElement().hasClass("e-scroller"))
                this._updateHeaderScrollLeft(this.getScrollElement().ejScroller("model.scrollLeft"));
                //update frozenContainer height
                if (this._frozenColumnsLength) {
                    var movableContainerHeight = model.sizeSettings.height == "auto" ? $("#" + proxy._id + "movablee-table").height() : this.getContent().find("#e-movablecontent" + proxy._id).height();
                    if (movableContainerHeight != this.getContent().find("#e-frozencontainer" + proxy._id).height()) {
                        this.getContent().find("#e-frozencontainer" + proxy._id).height(movableContainerHeight);
                    }
                }
                // Tree grid header not properly aligned in safari browser.
                // Because dynamically add or remove any css class it not affected immediately in safari browser
                // So just refresh the tree grid header
                if (isSafari) {
                    proxy._$gridHeaderContent[0].style.width=proxy._gridWidth+"px";
                    proxy._$gridHeaderContent.addClass("e-gridheader");
                }
                    proxy._resizeFilteringElements();
            }
        },

        _updatePagerIndex: function(){
            var proxy = this,
                model = proxy.model,
                expandedRecords = [], endIndex = -1, startIndex = -1,
                updatedRecords = model.updatedRecords;
            if (model.allowPaging) {
                var pageSize = model.pageSettings.pageSize;

                if (model.pageSettings.pageSizeMode === ej.TreeGrid.PageSizeMode.Root) {
                    var rootParentRecords = proxy._isHierarchyFiltering ? proxy._filterParentRecords : model.parentRecords.slice(),
                        index = -1, rootEndIndex = -1, rootStartIndex = -1;
                    proxy._gridRecordsCount = rootParentRecords.length;
                    var totalAvailablePages = proxy._isLoadonDemand == false ? (proxy._gridRecordsCount < 1) ? 0 : (pageSize == 0) ? 0 : Math.ceil(proxy._gridRecordsCount / pageSize) : Math.ceil(proxy._pageRecordCountOnDemand / pageSize);
                    if (proxy._currentPage() > totalAvailablePages)
                        proxy._currentPage(totalAvailablePages);
                    rootEndIndex = ((proxy._currentPage() * pageSize) > proxy._gridRecordsCount)? (proxy._gridRecordsCount) : (proxy._currentPage() * pageSize);
                    rootStartIndex = (proxy._gridRecordsCount <= pageSize) ? 0 : ((proxy._currentPage() * pageSize) - pageSize);                                     
                    endIndex = updatedRecords.indexOf(rootParentRecords[rootEndIndex]);
                    if (rootStartIndex > rootEndIndex)
                        rootStartIndex = rootEndIndex - pageSize;
                    else if (proxy._gridRecordsCount == rootEndIndex)
                        endIndex = updatedRecords.length;
                    startIndex = updatedRecords.indexOf(rootParentRecords[rootStartIndex]);
                    proxy._updatedPageData = (proxy._isHierarchyFiltering && proxy.model.enableVirtualization) ? proxy._updateFilterParent.slice(startIndex, endIndex) : this.getExpandedRecords(updatedRecords.slice(startIndex, endIndex));
                }
                else {
                    expandedRecords = this.getExpandedRecords(updatedRecords);
                    if (model.enableVirtualization) {
                        proxy._gridRecordsCount = proxy._isHierarchyFiltering ? proxy._updateFilterParent.length : updatedRecords.length;
                    }
                    else {                        
                        proxy._gridRecordsCount = expandedRecords.length;                        
                    }
                   
                    var currentTotalPages = (proxy._gridRecordsCount < 1) ? 0 : (pageSize == 0)? 0 : Math.ceil(proxy._gridRecordsCount / pageSize) ;
                    if (proxy._currentPage() > currentTotalPages)
                        proxy._currentPage(currentTotalPages);
                    endIndex = ((proxy._currentPage() * pageSize) > proxy._gridRecordsCount) ? (proxy._gridRecordsCount) : (proxy._currentPage() * pageSize),
                    startIndex = (proxy._gridRecordsCount < pageSize) ? 0 : ((proxy._currentPage() * pageSize) - pageSize);
                    if (startIndex > endIndex)
                        startIndex = endIndex - pageSize;
                    proxy._updatedPageData = (proxy._isHierarchyFiltering && proxy.model.enableVirtualization) ? proxy._updateFilterParent.slice(startIndex, endIndex) : expandedRecords.slice(startIndex, endIndex);
                }
                proxy.updateHeight();
            }
        },

        //UPDATE CURRENT VIEWPORT DATA FROM FLATRECORDS/UPDATED RECORDS
        _updateCurrentViewData: function (requestType) {

            var proxy = this,
                model = proxy.model,
                args = {},
                tMargin,
                isRangeModified = false;

            //CHECK VIRTUALIZATION IS ENABLED OR NOT 
            if (model.enableVirtualization) {

                if (model.allowPaging) {
                    if (requestType != ej.TreeGrid.Actions.Scroll)
                        proxy._updatePagerIndex();
                    proxy.getVisibleRange(requestType);
                    if (model.sizeSettings.height)
                        model.currentViewData = proxy._updatedPageData.slice(proxy._visibleRange.top,
                        proxy._visibleRange.bottom);
                    else
                        model.currentViewData = (proxy._isHierarchyFiltering && proxy.model.enableVirtualization) ? proxy._updateFilterParent : proxy._updatedPageData;
                }

                else {
                    proxy.getVisibleRange(requestType);
                    var record = (proxy._isHierarchyFiltering && proxy.model.enableVirtualization) ? proxy._updateFilterParent : proxy.model.updatedRecords;
                    model.currentViewData = record.slice(proxy._visibleRange.top,
                    proxy._visibleRange.bottom);
                }

                if (model.allowDragAndDrop && proxy._dragMouseDown && proxy._isTouchEvent) {
                    var updatedRecords = proxy.getUpdatedRecords(),
                       draggedItemIndex = updatedRecords.indexOf(proxy._draggedRecord);
                    if (proxy._visibleRange.top > (draggedItemIndex + 1) || proxy._visibleRange.bottom < draggedItemIndex) {
                        if (!proxy._dummyElement) {
                            var cloneElement = proxy._draggedElement.clone();
                            proxy._draggedElement.before(cloneElement)
                            proxy._draggedElement.hide();
                            $(proxy.element).append(proxy._draggedElement);
                            proxy._dummyElement = true;
                        }
                    }
                }

                if (proxy._vScrollDist !== 0) {

                    tMargin = proxy._scrollTop - proxy._offset;
                    if (requestType && requestType == ej.TreeGrid.Actions.Scroll) args.requestType = ej.TreeGrid.Actions.Scroll;
                    else if (requestType && requestType == ej.TreeGrid.Actions.ExpandCollapse) args.requestType = ej.TreeGrid.Actions.ExpandCollapse;

                    else args.requestType = ej.TreeGrid.Actions.Refresh;

                    if (requestType != "expandcollapse")
                        proxy.sendDataRenderingRequest(args);
                if (model.allowTextWrap && model.enableVirtualization && requestType == "scroll"){
                    proxy._updateWrapRowHeights("total,header");
                    proxy._textWrapRowsDatas = proxy._getAvgTextWrapRowHeight("getAvgHeight");
                }
                    isRangeModified = true;

                    if (this._frozenColumnsLength > 0) {
                        proxy._$frozenTableContent.css({ "top": tMargin });

                        proxy._$movableTableContent.css({ "top": tMargin });
                    } else 
                        proxy._$tableContent.css({ "top": tMargin });
                    if (model.allowTextWrap && model.enableVirtualization) {
                        if (proxy._textWrapRowsDatas.isEnd && !proxy._textWrapNoCalculate) {
                            proxy.updateHeight();
                            proxy._textWrapNoCalculate = true;
                        } else if (parseInt(proxy._scrollTop) >= parseInt(proxy.getMaxScrollHeight() - 10) && !proxy._textWrapRowsDatas.isEnd && !proxy._textWrapNoCalculate) {
                            proxy.updateHeight('extendHeight');
                            proxy._textWrapNoCalculate = false;
                        }
                        if (proxy._scrollTop >= (proxy.getMaxScrollHeight() - 10) && requestType != "expandcollapse") {
                            var offsetHeight, maxScrollHeight;
                            maxScrollHeight = proxy.getMaxScrollHeight();
                            if (this._frozenColumnsLength > 0) {
                                offsetHeight = (maxScrollHeight + $("#" + proxy._id + "frozene-table").height()) - $("#" + proxy._id + "e-gridcontainer").height();
                            } else {
                                offsetHeight = (maxScrollHeight + $("#" + proxy._id + "e-table").height()) - $("#" + proxy._id + "e-gridcontainer").height();
                            }
                            tMargin = maxScrollHeight - offsetHeight;
                            if (this._frozenColumnsLength > 0) {
                                proxy._$frozenTableContent.css({ "top": tMargin });
                                proxy._$movableTableContent.css({ "top": tMargin });
                            } else
                                proxy._$tableContent.css({ "top": tMargin });
                        }
                        if (proxy._scrollTop < 30) {
                            proxy._textWrapNoCalculate = false;
                        }
                    }
                }
                proxy._prevRTop = proxy._visibleRange.top;
                proxy._prevRBottom = proxy._visibleRange.bottom;

            } else {
                if (model.allowPaging) {
                    proxy._updatePagerIndex();
                    model.currentViewData = proxy._updatedPageData;
                }
                else
                    model.currentViewData = proxy.model.updatedRecords.slice();
            }


            if (proxy._isEnterKeyPressed) {

                proxy._isEnterKeyPressed = false;
                if (model.allowSelection && model.editSettings.allowEditing) {

                    if (!isRangeModified) proxy._cellEditingDetails.rowIndex += 1;

                    if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), proxy._cellEditingDetails.rowIndex)) {

                        proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                        proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);

                    }

                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, proxy.model.columns[proxy._cellEditingDetails.columnIndex].field);

                }
            }
        },
       
        //finding all parent records in hierarchy filtered data
        _findFilteredParentRecords: function (record) {
            var proxy = this,
                length = record.length, parentRecord = [];
            for (var count = 0; count < length; count++) {
                if (record[count].hasChildRecords)
                    parentRecord.push(record[count])
            }
            return parentRecord
        },
        //GET THE TABLE ROW POSITION
        _getRowPosition: function (y) {
            return Math.ceil((y) / (this.model.rowHeight + this._detailsRowHeight));
        },


        //GET THE COLUMNGROUP FOR HEADER TABLE
        _getMetaColGroup: function () {

            var $colgroup = this.getHeaderTable().find('colgroup').clone();
            return $colgroup;

        },


        //CREATE EMPTY ROW WHILE DATASOURCE OBJECT AS NULL
        _getEmptyTbody: function () {

            var proxy = this,
                $emptyTd = ej.buildTag('td', proxy.model.emptyRecordText, {}, { colSpan: (proxy.model.columns.length - this._hiddenColumns.length) });
            var element = document.createElement("tr");
            element.className = "e-emptyText";
            return $(element).height(30).append($emptyTd);

        },


        //ADD SORTICON TO THE COLUMN HEADER
        _addSortElementToColumn: function (field, direction) {

            var proxy = this, model = this.model,
                column,index;
            if (model.isFromGantt) {
                column = ej.TreeGrid.getColumnByMappingName(proxy.model.columns, field);
                if (ej.isNullOrUndefined(column))
                    column = proxy.getColumnByField(field);
            }
            else
                column = ej.TreeGrid.getColumnByField(proxy.model.columns, field);
            index = $.inArray(column, proxy.model.columns);
            var $headerCellDiv = this.getHeaderTable().find("thead tr:not('.e-stackedHeaderRow')").
                find(".e-headercell").not(".e-detailheadercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            $headerCellDiv.append(proxy._createSortElement().addClass("e-" + direction).addClass("e-icon"));
            $headerCellDiv.parent().attr("aria-sort", direction);
        },


        //CREATE SORT ELEMENT        
        _createSortElement: function () {
            return ej.buildTag('span', "&nbsp;");
        },


        //RENDERED EDITED CELL
        _renderCellEditObject: function (cellEditArgs, $td) {

            var proxy = this,
                $form = ej.buildTag("form.e-treegridform#" + proxy._id + "EditForm", "", {}, {}),
                $cellEditTemplate = proxy._cellEditTemplate,
                $element,
                htmlString,
                cellData = {},
                cellValue,
                columnObject = cellEditArgs.columnObject,
                columnField = columnObject.field.split('.').join(""),
                columnIndex = -1,
            cellValue = cellEditArgs.value;
            if (columnObject.editType == "dropdownedit" && cellValue == false)
                cellValue = cellValue.toString();
            cellData[columnField] = cellValue,
            columnIndex = proxy.getColumnIndexByField(columnObject.field);

            $td.empty();

            $td.addClass("e-validError");

            htmlString = $cellEditTemplate.find("#" + columnField + "_CellEdit").html();
            if (columnObject.field === "predecessor" || columnObject.field === "duration")
                $element = $($.templates(htmlString).render(cellEditArgs.data));
            else
                $element = $($.templates(htmlString).render(cellData));

            if ($element.get(0).tagName == "SELECT") {

                $element.val(cellData[columnField]);
                $element.val() == null && $element.val($element.find("option").first().val());

            }

            if ($element.get(0).tagName == "INPUT" && proxy.model.isFromGantt) {
                if (columnObject.field != "predecessor" && columnObject.field != "duration")
                    $element.val(cellData[columnField]);
            } else if ($element.get(0).tagName == "INPUT") {
                $element.val(cellData[columnField]);
            }

            $form.append($element);

           $td.append($form);

           var args = {};
           args.columnIndex = cellEditArgs.columnIndex;
           args.data = cellEditArgs.data.item;
           args.requestType = "beginEdit";
           proxy._refreshEditForm(args);

            if ($.isFunction($.validator) && !$.isEmptyObject(cellEditArgs.validationRules)) {

                proxy._initValidator();
                proxy.setValidation();

            }
            proxy.model.isEdit = true;

        },

        //get Index of resourceInfo
        getIndexofresourceInfo: function (dataSource) {

            var proxy = this, model = proxy.model,
                resourceIdMapping = model.resourceIdMapping,
                data = proxy._cellEditingDetails.data,
                resourceInfo = data.resourceInfo,
                count = 0,
                length = resourceInfo && resourceInfo.length,
                resourceIndex = [];

            for (count; count < length; count++) {                
                for (var index = 0; index < dataSource.length; index++) {
                    if (dataSource[index][resourceIdMapping] == resourceInfo[count][resourceIdMapping])
                        resourceIndex.push(index)
                }
            }

            return resourceIndex;

        },

        //REFRESH THE EDITED FORM
        _refreshEditForm: function (args) {

            var proxy = this, model = proxy.model, $form, rowHeight, isDialogEdit;
            if (proxy.model.editSettings.allowEditing && (proxy.model.editSettings.editMode == "dialogEditing" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog"))) {
                $form = $("#" + proxy._id + "EditFormDialog");
                rowHeight = "";
                isDialogEdit = true;
            } else {
                $form = $("#" + proxy._id + "EditForm");
                rowHeight = proxy.model.rowHeight - 5;
            }
            var $frozenForm = $("#" + proxy._id + "EditFrozenForm"),
                elementFocused = false,
                $formElement = $frozenForm.add($form).find("input,select,textarea"),
                percent = 86,
                i = 0,
                length = $formElement.length,
                $element, inputWidth,
                model = proxy.model,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat,
                formatVal,
                cellEditType;

            for (i; i < length; i++) {

                $element = $formElement.eq(i);
                cellEditType = $element.attr("data-edittype");
                column = ej.TreeGrid.getColumnByField(model.columns, $element.prop("name"));
                if (ej.isNullOrUndefined(column) && (proxy.model.editSettings.editMode == "dialogEditing" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog"))) {
                    if (($element).hasClass("e-btn") || (proxy.model.editSettings.editMode == "dialogEditing" && (!ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) && model.editSettings.dialogEditorTemplateID != ""))) {
                        return;
                    } else
                        $element.closest("tr").addClass("e-hide");
                } else {
              
                    if ((proxy.model.editSettings.editMode == "dialogEditing" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog")) && (!ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) && model.editSettings.dialogEditorTemplateID != ""))
                        cellEditType = column.editType;
              
                        inputWidth = "100%";

                    switch (cellEditType) {
              
                        case ej.TreeGrid.EditingType.Numeric:
                            params = {};
                            width = inputWidth;
                            value = $element.val();
                            params.width = width;
                            params.locale = model.locale;
                            params.height = rowHeight;
                            params.showSpinButton = true;
                            params.cssClass = model.cssClass;
                  
                        if (value.length)
                           {
                                  params.value = parseFloat(value);
                           }

                        if (model.isFromGantt && column.field === "status") {
                            params.minValue = 0;
                            params.maxValue = 100;
                            params.decimalPlaces = 0;
                        }

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                        $element.ejNumericTextbox(params);
                        $element.prop("name", $element.prop("name").replace(proxy._id, ""));
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:
                        params = {};
                        params.width = inputWidth;
                        params.height = rowHeight;
                        params.locale = model.locale;
                        params.cssClass = model.cssClass;
                        params.dateFormat = model.dateFormat;
                        params.enableStrictMode = true;
                        if (model.isFromGantt)
                            params.startDay = model.weekStartDay;
                        if ($element.val().length) params.value = proxy.getDateFromFormat($element.val());

                        if (column["format"] !== undefined && column.format.length > 0) {

                            toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                            formatVal = toformat.exec(column.format);
                            params.dateFormat = formatVal[2];
                            if ($element.val().length) {
                                params.value = ej.format(new Date($element.val()), params.dateFormat, params.locale);
                                $element.val(params.value);
                            }
                        }

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                        if (model.dateFormat.toLowerCase().indexOf("hh") == -1)
                            $element.ejDatePicker(params);
                        else {
                            $element.ejDateTimePicker(params);
                        }
                        break;
                    case ej.TreeGrid.EditingType.DateTimePicker:
                        params = {};
                        params.width = inputWidth;
                        params.height = rowHeight;
                        params.locale = model.locale;
                        params.cssClass = model.cssClass;
                        params.displayDefaultDate = true;
                        params.dateTimeFormat = model.dateFormat;
                        params.enableStrictMode = true;
                        if (model.isFromGantt)
                            params.startDay = model.weekStartDay;
                        if ($element.val().length) params.value = proxy.getDateFromFormat($element.val());

                        if (column["format"] !== undefined && column.format.length > 0) {

                            toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                            formatVal = toformat.exec(column.format);
                            params.dateTimeFormat = formatVal[2];
                            if ($element.val().length) {
                                params.value = ej.format(new Date($element.val()), params.dateTimeFormat, params.locale);
                                $element.val(params.value);
                            }
                        }

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                            $element.ejDateTimePicker(params);
                        break;
                    case ej.TreeGrid.EditingType.Dropdown:

                        var dataSource, fieldName;

                        if (proxy._isRowEdit)
                        {
                            dataSource = column.dropdownData;
                            fieldName = column.field;
                        }
                        else{
                            dataSource = column.dropdownData;
                            fieldName = column.field;
                        }

                        if (model.isFromGantt && fieldName === "resourceInfo") {
                            params = {};
                            params.width = inputWidth;
                            params.height = rowHeight;
                            params.showCheckbox = true;
                            params.dataSource = dataSource;
                            params.fields = {
                                id: model.resourceIdMapping,
                                text: model.resourceNameMapping,
                                value: model.resourceNameMapping
                            };
                            params.selectedItems = proxy.getIndexofresourceInfo(dataSource);
                            if (!ej.isNullOrUndefined(column["editParams"]))
                                $.extend(params, column["editParams"]);
                            $element.ejDropDownList(params);
                        } else {
                            var controlArgs = {};
                            controlArgs.cssClass = model.cssClass;
                            controlArgs.width = inputWidth;
                            controlArgs.height = rowHeight;
                            controlArgs.dataSource = dataSource;
                            if (!ej.isNullOrUndefined(column["editParams"]))
                                $.extend(controlArgs, column["editParams"]);

                            $element.ejDropDownList(controlArgs);

                            var obj = $element.ejDropDownList("instance");
                            obj._setValue($element.attr("data-cellvalue"));
                           
                        }
                        break;

                    case ej.TreeGrid.EditingType.String:
                        if (proxy.model.editSettings.allowEditing && proxy.model.editSettings.editMode == "dialogEditing") {
                            $element.outerWidth("100%").height(27);
                        } else {
                            $element.outerWidth(inputWidth).height(model.rowHeight - 9);
                        }
                        $element.css({ 'margin': '0px', 'padding': '1px' });
                        break;
                    case ej.TreeGrid.EditingType.Boolean:
                        if (proxy._checkboxSelection && column["nonColumn"]) {                            
                            $element[0].style.display = "none";
                        }
                        else {
                            var controlArgs = {};
                            controlArgs.cssClass = model.cssClass;
                            controlArgs.size = "small";                            

                            if (!ej.isNullOrUndefined(column["editParams"]))
                                $.extend(controlArgs, column["editParams"]);

                            $element.ejCheckBox(controlArgs);
                            ////center align of check box control
                            var parentElement = $element.parent(".e-chkbox-wrap"),
                                formElement = parentElement.parent("form");

                            if (!isDialogEdit) {
                                if (formElement.length > 0)
                                    formElement.css("margin-left", "45%");
                                else
                                    parentElement.css("margin-left", "45%");
                            }
                        }                       
                        break;
                   
                    case ej.TreeGrid.EditingType.Maskedit:
                        var controlArgs = {};
                        controlArgs.locale = model.locale;
                        controlArgs.cssClass = model.cssClass;
                        controlArgs.width = inputWidth;
                        controlArgs.height = rowHeight;
                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(controlArgs, column["editParams"]);

                        $element.ejMaskEdit(controlArgs);
                        break;
                    case "edittemplate":
                        var temp = { rowdata: args.data, "column": column, element: $element, requestType: args.requestType};
                        var temp1 = column.editTemplate.write;
                        if (!ej.isNullOrUndefined(args) && args.requestType == "add") temp.rowdata = {};
                        if (typeof temp1 == "string")
                            temp1 = ej.util.getObject(temp1, window);
                        temp1(temp);
                        break;
                }

                if (!$element.is(":disabled") && !elementFocused &&
                   (!$element.is(":hidden") || typeof ($element.data("ejDropDownList") || $element.data("ejNumericTextbox") ||
                   $element.data("ejDatePicker") || $element.data("ejCheckBox")) == "object")) {

                    if ((!proxy._isEnterKeyPressed && column == model.columns[args.columnIndex]) || ((!args.columnIndex || args.columnIndex == -1) && !proxy._isEnterKeyPressed)) {

                        proxy._focusElements($element.closest('td'));
                        elementFocused = true;

                    }
                }
                }
        }
            },

        _getRecordIndex: function () {
            var proxy = this,
                model = proxy.model,
                currentViewData, index,
                records = model.allowPaging ? proxy._updatedPageData : proxy.getExpandedRecords(model.updatedRecords);
            proxy.getVisibleRange();
            currentViewData =  records.slice(proxy._visibleRange.top, proxy._visibleRange.bottom); 
            index = currentViewData.indexOf(model.selectedItem);
            return index;
        },
         //INITIALIZE THE VALIDATOR
        _initValidator: function () {
            var proxy = this, elements = this._frozenColumnsLength > 0 ? this.element.find(".e-treegridform") : $("#" + this._id + "EditForm").length > 0 ? $("#" + this._id + "EditForm") : $("#" + this._id + "EditFormDialog");
            for (var i = 0; i < elements.length ; i++) {
                elements.eq(i).validate({
                    ignore: ".e-hide",
                    errorClass: 'e-field-validation-error',
                    errorElement: 'div',
                    wrapper: "div",
                    errorPlacement: function (error, element) {
                        proxy._renderValidator(error, element);
                    },
                });
            }
        },

        _renderValidator: function (error, element) {
            if (element.is(":hidden"))
                element = element.siblings("input:visible");
            if (!element.length)
                return;
            var proxy=this,
                model = proxy.model,
                recordIndex,
                $td = element.closest(".e-rowcell"),
                $container = $(error).addClass("e-error"),
                $tail = ej.buildTag("div.e-errortail");
            $td = $td.length == 0 ? element.closest("td") : $td;
            $td.removeClass(".e-error");
            if (model.editSettings.editMode != "dialogEditing") {
                recordIndex = proxy._getRecordIndex();                
                var recordPosition = (recordIndex + 1) * model.rowHeight;
                if (!model.enableVirtualization) {
                    var expandRecords = proxy.getExpandedRecords(model.updatedRecords);
                    recordIndex = model.allowPaging ? proxy._updatedPageData.indexOf(model.selectedItem) : expandRecords.indexOf(model.selectedItem);
                }
                if (recordIndex == -1) {
                    if (model.editSettings.rowPosition == "bottom")
                        recordIndex = model.updatedRecords.length - 1;
                    else if (model.editSettings.rowPosition == "top")
                        recordIndex = 0;
                    recordPosition = (recordIndex + 1) * model.rowHeight;
                }
                // here 45 is the error container height
                if (proxy._viewPortHeight < recordPosition + 45) {
                    $tail.addClass("e-bottomarrow");
                    var top = recordIndex * model.rowHeight - 45;
                    var operation = "append";
                }
                else {                   
                    $tail.addClass("e-toparrow");
                    var top = (recordIndex + 1) * model.rowHeight;
                    var operation = "prepend";
                }
            }
            else {
                var top = $td.offset().top - $("#" + this._id + "EditFormDialog").offset().top + $td.parent()[0].offsetHeight;
                $tail.addClass("e-toparrow");
                var operation = "prepend";
            }
            $($container).css({ "top": top });
            if (element.parent().hasClass("e-in-wrap"))
                $container.insertAfter(element.closest(".e-widget"));
            else
                $container.insertAfter(element);
            $container[operation]($tail);            
            $container.show("slow");
        },

        setValidation: function () {
            var proxy=this,
                model=proxy.model;
            
            for (var i = 0; i < this.model.columns.length; i++) {
                if (!ej.isNullOrUndefined(model.columns[i]["validationRules"]) && model.columns[i].visible == true) {
                    if (model.editSettings.editMode == ej.TreeGrid.EditMode.RowEditing
                         || model.editSettings.editMode == ej.TreeGrid.EditMode.DialogEditing || proxy.element.find(".e-rowedit").length > 0
						 || proxy.element.find(".e-addedrow").length > 0 || (model.editSettings.editMode == ej.TreeGrid.EditMode.BatchEditing &&
                        (model.editSettings.batchEditSettings.editMode == "dialog" || model.editSettings.batchEditSettings.editMode == "row")))
                        this.setValidationToField(model.columns[i].field, model.columns[i].validationRules);
                    else if ((model.editSettings.editMode == ej.TreeGrid.EditMode.CellEditing ||
                        (model.editSettings.editMode == ej.TreeGrid.EditMode.BatchEditing && model.editSettings.batchEditSettings.editMode == "cell"))
                        && proxy._cellEditingDetails.columnIndex == i) {
                        this.setValidationToField(model.columns[i].field, model.columns[i].validationRules);
                        return;
                    }
                }                
            }
        },

        setValidationToField: function (name, rules) {
            var fName = name, ele, form,proxy=this;
            if (!ej.isNullOrUndefined(name))
                fName = fName.replace(/[^a-z0-9\s_]/gi, ''); 
            var form = $("#" + this._id + "EditForm").length > 0 ? $("#" + this._id + "EditForm") : $("#" + this._id + "EditFormDialog"),
                column = proxy.getColumnByField(name);
            if (!ej.isNullOrUndefined(column) && column.isFrozen)
                var form = $("#" + this._id + "EditFrozenForm").length > 0 ? $("#" + this._id + "EditFrozenForm") : $("#" + this._id + "EditFormDialog").length > 0 ? $("#" + this._id + "EditFormDialog") : $("#" + this._id + "EditForm");

            ele = form.find("#" + this._id + fName).length > 0 ? form.find("#" + this._id + fName) : form.find("#" + fName);
            if (rules["regex"]) {
                rules[name + "regex"] = rules["regex"]; delete rules["regex"];
                $.validator.addMethod(fName + "regex", function (value, element, options) {
                    var ptn = options instanceof RegExp ? options : new RegExp(options);
                    return ptn.test(value);
                }, ej.getObject("messages.regex", rules) || this.getColumnByField(name).headerText + " should match the given pattern");
            }
            !ele.attr("name") && ele.attr("name", name);
            ele.rules("add", rules);
            var validator = $("#" + this._id + "EditForm").length > 0 ? $("#" + this._id + "EditForm").validate() : $("#" + this._id + "EditFormDialog").validate();
            validator.settings.messages[name] = validator.settings.messages[name] || {};
            if (!ej.isNullOrUndefined(rules["required"])) {
                if (!ej.isNullOrUndefined(rules["messages"] && rules["messages"]["required"]))
                    var message = rules["messages"]["required"];
                else
                    var message = $.validator.messages.required;
                if (message.indexOf("This field") == 0)
                    message = message.replace("This field", this.getColumnByField(name).headerText);
                validator.settings.messages[name]["required"] = message;
                if (ele.hasClass("e-datepicker"))
                    ele.ejDatePicker({ watermarkText: "" });
            }
        },

        //SET FOCUS TO EDITED ELEMENT
        _focusElements: function ($currentCell) {
            var proxy = this;

            if ($currentCell.length) {

                $currentCell.focus();
                var $childElem = $currentCell.children();

                if ($childElem[0]) {

                    if ($childElem[0].tagName.toLowerCase() == "select" || 
                        $childElem[0].tagName.toLowerCase() == "input") {

                        $childElem.focus().select();
                        $childElem[0].focus();

                    } else if ($childElem.find(".e-field.e-dropdownlist").length) {

                        $childElem.find(".e-ddl").focus();

                    } else if ($childElem.find(".e-chkbox-wrap.e-widget").length) {

                        $childElem.find(".e-chkbox-wrap.e-widget").focus();

                    }
                    else {
                        if ($childElem.find(".e-datepicker").length <= 0)
                            $childElem.find('input,select').select().focus();
                        else
                            $childElem.find('input,select').select();
                    }
                }
            }

            var contentScrollLeft = proxy.getScrollElement().children('.e-content').scrollLeft(),
                scrollLeft = proxy.getScrollElement().ejScroller("option", "scrollLeft");
            if (proxy.getScrollElement().hasClass("e-scroller") && contentScrollLeft != scrollLeft) {
                if (!ej.isNullOrUndefined(contentScrollLeft))
                proxy.getScrollElement().ejScroller("option", "scrollLeft", contentScrollLeft);
            }
            proxy._updateHeaderScrollLeft(proxy.getScrollElement().children('.e-content').scrollLeft());
        },
        //Set the focus for edit row elements
        _focusElementsForRowEdit: function ($currentCell) {
            if ($currentCell.length) {
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length)
                    $childElem[0].focus();
                else if ($childElem.hasClass("e-chkbox-wrap")) {
                    $childElem[0].focus();
                } else {
                    if ($childElem.find(".e-datepicker").length <= 0)
                        $childElem.find('input:visible,select').first().select().focus();
                    else
                        $childElem.find('input,select').select();
                }
            }
        },
        /*Get Total height based on resource child records*/
        _getHeightforResourceView: function (records) {
            var height = 0;
            for (var i = 0; i < records.length; i++) {
                height += this._getTemplateRowHeight(records[i]);
            }
            return height;
        },
        //Get records height based on resource view and normal view
        getRecordsHeight: function () {
            var model = this.model,
                height = 0;
            if (model.viewType == "resourceView") {
                if (model.enableVirtualization)
                    height = this._getHeightforResourceView(model.updatedRecords);
                else
                    height = this._getHeightforResourceView(this.getExpandedRecords(model.updatedRecords));
            } else if (model.enableVirtualization) {

                height = (this._isHierarchyFiltering&&this.model.enableVirtualization) ? this._updateFilterParent.length * model.rowHeight : this.getUpdatedRecords().length * model.rowHeight;
            } else {
                height = model.rowHeight * (model.updatedRecords.length - this.getCollapsedRecordCount());
            }
            return height;
        },
        //UPDATE THE GRID CONTAINER HEIGHT
        //By updatedRecords and CollapsedRecord count length
        updateHeight: function (type) {

            var proxy = this,
                model = this.model,
                height = 0,
                detailsRowHeight = 0;
            if (proxy._isInExpandCollapseAll)
                return true;
            if (proxy._$gridContainer && proxy._$gridContent) {
                if (model.allowPaging && !(proxy.model.allowTextWrap && model.enableVirtualization)) {
                    height =model.sizeSettings.height=="auto"?proxy._getViewPortHeight(): model.rowHeight * proxy._updatedPageData.length;
                }
                else if (model.enableVirtualization) {
                    if (model.sizeSettings.height == "auto") {
                        height = proxy._getViewPortHeight();
                    } else {
                        if (model.allowTextWrap) {
                            if (model.allowPaging && !proxy._textWrapRowsDatas.isEnd) {
                                proxy._textWrapRowsDatas = proxy._getAvgTextWrapRowHeight("getAvgHeight");
                                height = proxy._textWrapRowsDatas.avgHeight * proxy._updatedPageData.length;
                            } else if (proxy._textWrapRowsDatas.isEnd) {
                                height = proxy._getAvgTextWrapRowHeight('updateHeight').height;
                            } else if (type == "extendHeight") {
                                height = $("#" + proxy._id + "e-gridcontainer").height() + ((proxy.model.updatedRecords.length - proxy._visibleRange.bottom) * proxy.model.rowHeight);
                            } else {
                                proxy._textWrapRowsDatas = proxy._getAvgTextWrapRowHeight("getAvgHeight");
                                height = model.sizeSettings.height == "auto" ? proxy._getViewPortHeight() : model.updatedRecords.length * proxy._textWrapRowsDatas.avgHeight;
                            }
                        } else {
                            height = this.getRecordsHeight();
                        }
                    }
                }

                else {
                    height = model.sizeSettings.height == "auto" ? proxy._getViewPortHeight() : this.getRecordsHeight();
                }

                if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn&&model.sizeSettings.height!="auto") {
                    detailsRowHeight = proxy._getExpandedDetailsRowHeight();
                }

                height += detailsRowHeight;

                if (height == 0 && (model.currentViewData.length == 0 || model.flatRecords.length == 0))
                    height = 30;
                //To skip Vertical scrollbar
                if (model.allowPaging && !model.sizeSettings.height) {
                    var viewPortHeight;
                    if (model.allowTextWrap && !model.enableVirtualization) {
                        if (proxy._$frozenTableContent.length > 0) {
                            var frozenHeight = viewPortHeight = proxy._$gridContainer.find("#e-frozencontentdiv" + proxy._id).outerHeight(),
                                movableHeight = proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id).outerHeight();
                            if (movableHeight > frozenHeight)
                                viewPortHeight = proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id).outerHeight();
                        } else {
                            viewPortHeight = proxy._$gridContainer.find('.e-table').outerHeight();
                        }
                    }
                    else
                        viewPortHeight = height;
                    if (this._frozenColumnsLength > 0) {
                        proxy._$gridContent.css("height", viewPortHeight);
                        if (model.allowPaging && !model.sizeSettings.height) {
                            var scroller = proxy.getScrollElement().ejScroller("instance");
                            scroller.option("height", viewPortHeight);
                        }
                    }
                    else if (proxy.getScrollElement().hasClass("e-scroller")) {
                        if (proxy.getScrollElement().ejScroller("isHScroll")) {
                            viewPortHeight = height + proxy._scrollBarHeight;
                        }
                        proxy.getScrollElement().ejScroller({
                            height: viewPortHeight,//for border-width of gridcontent
                        });
                        
                    }
                    proxy._viewPortHeight = viewPortHeight;
                    proxy._gridHeight = height + proxy._getNonContentHeight();
                    if (proxy._summaryRowsCount > 0 && model.showTotalSummary)
                        proxy._gridHeight += proxy._$totalSummaryRowContainer.outerHeight();
                    proxy.element.height(proxy._gridHeight);                    
                }
                if (proxy.model.allowTextWrap && !proxy.model.enableVirtualization) {
                    proxy._$gridContainer.css({
                        "height": "auto",
                        "width": "auto"
                    });
                    if (this._frozenColumnsLength) {
                        proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id + ",#e-frozencontentdiv" + proxy._id).css("height", "auto");
                    }
                } else {
                    proxy._$gridContainer.css({
                        "height": height + "px",
                        "width": "auto"
                    });
                    if (this._frozenColumnsLength) {
                        proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id + ",#e-frozencontentdiv" + proxy._id).css("height", height + "px");
                    }
                }
                if (model.sizeSettings.height == "auto") {
                    if (proxy.getScrollElement().hasClass("e-scroller")) {
                        var scroller = proxy.getScrollElement().ejScroller("instance");
                        scroller.option("height", 0);
                    }
                    proxy._updateContentHeight();
                }
                if (proxy.getScrollElement().hasClass("e-scroller")) {
                    proxy.getScrollElement().ejScroller("refresh");
                    if (proxy._textWrapRowsDatas.isEnd) {
                        proxy._textWrapRowsDatas.isEnd = false;
                    }
                    proxy._updateScrollCss();
                }
            }
        },
        //Method to update last cell border as per records length and column resize mode
        _updateFixedTableBorder: function (action) {
            //Remove fixed column border when no record text displayed
            if (action == "sorting" || action == "dragAndDrop")
                return;
            var contentTable = this._frozenColumnsLength > 0 ? $(this.getContentTable()[1]) : $(this.getContentTable()),
                model = this.model;
            if (!model.isFromGantt && contentTable && contentTable.hasClass("e-fixed-border")) {
                if (model.currentViewData.length > 0 && !contentTable.hasClass("e-tableLastCell"))
                    contentTable.addClass("e-tableLastCell");
                else if (model.currentViewData.length == 0)
                    contentTable.removeClass("e-tableLastCell");
            }
        },
        //check scroller position in virtualization
        _setScrollTop: function (requestType) {

            var proxy = this,
            model = this.model,
            currentPosition = 0,
            top = proxy._scrollTop,
            rowHeight = model.rowHeight,
            length = model.currentViewData.length, id = model.isFromGantt ? proxy._id.replace("ejTreeGrid", "") : proxy._id,
            ganttbody = $("#" + id).find(".e-ganttviewerbodyContianer");

            /* check detail template and its row height */
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn) {
                rowHeight += proxy._detailsRowHeight;
            }
            if (model.allowPaging && proxy._isNextPage) {
                top = 0;
                proxy._isNextPage = false;
                proxy.getScrollElement().ejScroller("scrollY", top, true);
            }
            if (top !== 0) {
                if(model.enableVirtualization){
                    if ((length * rowHeight) < proxy._viewPortHeight) {
                        if (requestType == "expandcollapse" && model.allowTextWrap && model.enableVirtualization) {
                            top = 0;
                        }else if (length > 0) {
                            currentPosition = top - (proxy._viewPortHeight - ((length - 1) * rowHeight));
                            if (currentPosition < 0)
                                currentPosition = 0;
                        }
                        if (model.isFromGantt)
                            ganttbody.ejScroller("scrollY", currentPosition, true);
                        else
                        proxy.getScrollElement().ejScroller("scrollY", currentPosition, true);
                    }
                }
                else {
                    if (model.isFromGantt)
                        ganttbody.ejScroller("scrollY", top, true);
                    else {
                        if (requestType == "delete") {
                            var updatedRecords = proxy.getUpdatedRecords();
                            currentPosition = (updatedRecords.length * rowHeight) - proxy._viewPortHeight;
                            if (currentPosition < 0)
                                top = 0;
                        }
                        proxy.getScrollElement().ejScroller("scrollY", top, true);
                    }
                }
            }
        },

        // Height Refresh method that used in grid while perform column resize.
        rowHeightRefresh: function () {
            var proxy = this, model = this.model;
            proxy._updateWrapRowHeights(null, "virtualRefresh", null, "resize");
            if (proxy._$frozenTableContent.length > 0) {
                proxy.getScrollElement().ejScroller("refresh");
                var movableScrollTop = proxy.getScrollElement().ejScroller("model.scrollTop");
                proxy._getFrozenScrollElement().scrollTop(movableScrollTop);
            }
        },

        //Update the viewport,total summary container height for every action that affects columns when allowTextWrap is in enable state.
        _updateGridContentHeight: function () {
            var proxy = this, model = this.model;
            if (proxy._$totalSummaryRowContainer) {
                proxy._$totalSummaryRowContainer.ejScroller({ 'height': 'auto' });
                proxy._$footerContainer.css({ 'overflow': 'hidden' });
                if (proxy._$totalSummaryRowContainer.find(".e-footercontainer").height() < model.totalSummaryHeight) {
                    if (proxy.isVScroll())
                        proxy._$totalSummaryRowContainer.addClass("e-scrollcss");
                } else {
                    proxy._$totalSummaryRowContainer.removeClass("e-scrollcss");
                    proxy._$totalSummaryRowContainer.ejScroller({ 'height': model.sizeSettings.height == "auto" ? 0 : model.totalSummaryHeight });
                    proxy._$totalSummaryRowContainer.width(proxy._$gridContent.width());
                    if (model.sizeSettings.height != "auto")
                    proxy._$totalSummaryRowContainer.width('auto');
                }
            }
            if (model.allowPaging && !model.sizeSettings.height) {
                if (proxy._$frozenTableContent.length > 0) {
                    var frozenHeight = proxy._viewPortHeight = proxy._$gridContainer.find("#e-frozencontentdiv" + proxy._id).outerHeight(),
                        movableHeight = proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id).outerHeight();
                    if (movableHeight > frozenHeight)
                        proxy._viewPortHeight = proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id).outerHeight();
                } else {
                    proxy._viewPortHeight = proxy._$gridContainer.find('.e-table').outerHeight();
                }
            }
            else {
                proxy._viewPortHeight = proxy._getViewPortHeight();
                if (proxy._$totalSummaryRowContainer)
                    proxy._viewPortHeight = model.sizeSettings.height == "auto" ? proxy._viewPortHeight : proxy._viewPortHeight - proxy._$totalSummaryRowContainer.outerHeight();
            }
            proxy._$gridContent.css({ "height": proxy._viewPortHeight + "px" });
                if (proxy.getScrollElement().hasClass("e-scroller")) {
                    var scroller = proxy.getScrollElement().ejScroller("instance");
                    scroller.option("height", model.sizeSettings.height == "auto" ? 0 : proxy._viewPortHeight);
                }
                if (proxy._frozenColumnsLength) {
                    if (model.sizeSettings.height == "auto")
                        proxy._updateContentHeight();
      
                var movableContainerHeight = this.getContent().find("#e-movablecontent" + proxy._id).height();
                if (movableContainerHeight != this.getContent().find("#e-frozencontainer" + proxy._id).height()) {
                    this.getContent().find("#e-frozencontainer" + proxy._id).height(movableContainerHeight);
                    proxy.getScrollElement().ejScroller("refresh");
                }
            }
            proxy._updateScrollCss();
        },

         
        _updateContentHeight: function () {
            var proxy = this, height;
            height = proxy._getViewPortHeight();
            if ($("#" + proxy._id + "e-gridcontent").find(".e-hscrollbar").length == 1) {
                $("#e-movablecontainer" + proxy._id).css("height", height);
                $("#e-movablecontent" + proxy._id).css("height", height - 18);
                $("#e-movablecontentdiv" + proxy._id).css("height", height - 18)
            }
            else {
                $("#e-movablecontainer" + proxy._id).css("height", height);
                $("#e-movablecontent" + proxy._id).css("height", height)
            }
            $("#" + proxy._id + "e-gridcontent").css("height", height)
            $("#" + proxy._id + "e-gridcontainer").css("height", height)
            $("#" + proxy._id + "e-table").css("top", 0);
            proxy.getScrollElement().ejScroller("refresh");
            proxy._updateScrollCss();
        },
        getChildCount:function(record,count)
        {
            var currentRecord, proxy = this;
            if (!record.hasChildRecords)
                return 0;
            for (var i = 0; i < record.childRecords.length;i++)
            {
                currentRecord= record.childRecords[i];
                count++;
                if(currentRecord.hasChildRecords)
                {
                    count=proxy.getChildCount(currentRecord, count);
                }
            }
            return count;
        },
        //UPDATE THE FILTERED RECORDS FOR ADDING PARENTRECORDS
        _updateFilteredRecords: function (data, isSearching) {

            var proxy = this,
                count = 0,
                length = data.length,
                record,virtualization=proxy.model.enableVirtualization;
            proxy._filteredData = data;
            proxy._flatFilteredRecords = [];
            proxy._filteredRecords = [];
            proxy._filteredGanttRecords = [];
            proxy._filterParentRecords = [];
            proxy._updateFilterParent = [];
            for (count; count < length; count++) {

                record = data[count];
                if ((!isSearching && (proxy.model.filterSettings.filterHierarchyMode == "child" || proxy.model.filterSettings.filterHierarchyMode == "none"))
                    || (isSearching && (proxy.model.searchSettings.searchHierarchyMode == "child" || proxy.model.searchSettings.searchHierarchyMode == "none")))
                    proxy._isHierarchyFiltering = true;
                proxy._addParentRecord(record,isSearching);

                if (!this.model.isFromGantt) {
                    if (record.hasChildRecords && record.childRecords.length > 0) {
                        if (!this._checkChildExsist(record, data, isSearching)) {
                            record.hasFilteredChildRecords = false;
                        }else {
                            record.hasFilteredChildRecords = true;
                        }
                    }
                    if (record.parentItem) {
                        record.parentItem.hasFilteredChildRecords = true;
                    }
                }
            }
            if (data.length != 0 && proxy._isHierarchyFiltering)
            proxy._updateFilterLevel();
         
            if (virtualization)
                proxy._filteredRecords = proxy._isHierarchyFiltering ? proxy._flatFilteredRecords : proxy.getExpandedRecords(proxy._flatFilteredRecords);
            else
                proxy._filteredRecords = proxy._flatFilteredRecords.slice();
            if (data.length != 0 && proxy._isHierarchyFiltering)
                proxy._updateFilterExpandState(proxy._filteredRecords);
        },


        //check whether the filtered records are collapsed or expanded
        _updateFilterExpandState: function (record) {
            var proxy = this,
                length = record.length;
            for (var count = 0; count < length; count++) {
                if ($.inArray(record[count].parentItem, proxy._flatFilteredRecords) != -1) {
                    if (record[count].parentItem.expanded && ($.inArray(record[count].parentItem, proxy._updateFilterParent) != -1)) {
                        proxy._updateFilterParent.push(record[count])
                        record[count].isDisplay = true; // property for render the collapsed state record while hierarchyfiltering
                    }
                    else {
                        if (record[count].isDisplay)
                            record[count].isDisplay = false
                    }
                }
                else {
                    proxy._updateFilterParent.push(record[count])
                    record[count].isDisplay = true;
                }
            }
        },

        //set the filterLevel for filtered records and find zeroth level parentrecords.
        _updateFilterLevel: function () {
            var proxy = this, level = 0,
            record = proxy._flatFilteredRecords,
            length = proxy._flatFilteredRecords.length;
            for (var count = 0; count < length; count++) {
                if ($.inArray(record[count].parentItem, proxy._flatFilteredRecords) != -1)
                    record[count].filterLevel = record[count].parentItem.filterLevel + 1;
                else {
                    record[count].filterLevel = 0;
                    proxy._filterParentRecords.push(record[count])
                }
            }
        },

        /* Check child records of filtered parent records exists or not in updated records*/
        _checkChildExsist: function (record, collection, isSearching) {
            var childRecords = record.childRecords, proxy = this,
                length = childRecords.length, isExsist = false;
            for (var count = 0; count < length; count++) {
                if ((!isSearching && (proxy.model.filterSettings.filterHierarchyMode == "child" || proxy.model.filterSettings.filterHierarchyMode == "both"))
                    || (isSearching && (proxy.model.searchSettings.searchHierarchyMode == "child" || proxy.model.searchSettings.searchHierarchyMode == "both"))) {
                    if ($.inArray(childRecords[count], proxy._flatFilteredRecords) != -1) {
                        isExsist = true;
                        break;
                    }
                    if (!childRecords[count].isSummaryRow)
                        proxy._flatFilteredRecords.push(childRecords[count]);
                    isExsist = true;
                    if (childRecords[count].hasChildRecords)
                        isExsist = this._checkChildExsist(childRecords[count], collection, isSearching);
                }
                else {
                    if ($.inArray(childRecords[count], collection) != -1) {
                        isExsist = true;
                        break;
                    } else if (childRecords[count].hasChildRecords) {
                        isExsist = this._checkChildExsist(childRecords[count], collection, isSearching);
                    }
                }
            }
            return isExsist;
        },

       /* update hasFilteredChildRecords status and clear the filterLevel of all records when clear the filter */
        _clearFilterLevel: function (collection) {
            var count = 0, length = collection.length, currentRecord, proxy = this;
            for (count; count < length; count++) {
                currentRecord = collection[count];
                if (currentRecord.filterLevel || currentRecord.filterLevel == 0) {
                    if (currentRecord.hasChildRecords && !currentRecord.hasFilteredChildRecords)
                        currentRecord.hasFilteredChildRecords = true;
                    delete currentRecord.filterLevel

                }
                if (currentRecord.isDisplay)
                    delete currentRecord.isDisplay
                proxy._isHierarchyFiltering = false;
            }
        },

        /* While clear the filter, update hasFilteredChildRecords status of all records*/
        _updateHasFilteredChildRecordsStatus: function (collection) {
            var count = 0, length = collection.length, currentRecord, proxy = this;
            for (count; count < length; count++) {
                currentRecord = collection[count];
                if (currentRecord.hasChildRecords && !currentRecord.hasFilteredChildRecords)
                    currentRecord.hasFilteredChildRecords = true;
                if (currentRecord.filterLevel > 0) {
                    delete currentRecord.filterLevel
                }
            }
        },
        //ADD PARENTRECORD FOR FILTERED OTH LEVEL ITEMS
        _addParentRecord: function (record,isSearching) {

            var proxy = this;
            if ((proxy.model.filterSettings.filterHierarchyMode == "none" && !isSearching)
                || (proxy.model.searchSettings.searchHierarchyMode == "none" && isSearching)) {
                if (record.parentItem == null) {
                    if ($.inArray(record, proxy._filteredData) != -1) {
                        if ($.inArray(record, proxy._flatFilteredRecords) == -1) {
                            proxy._flatFilteredRecords.push(record);
                            record.hasFilteredChildRecords = true;
                        }
                        return;
                    }
                }
                else {
                    proxy._addParentRecord(record.parentItem, isSearching);
                    if ($.inArray(record.parentItem, proxy._filteredData) != -1 || $.inArray(record.parentItem, proxy._flatFilteredRecords) != -1) {
                        if ($.inArray(record, proxy._flatFilteredRecords) == -1)
                            proxy._flatFilteredRecords.push(record);
                    }
                    else {
                        if ($.inArray(record, proxy._filteredData) != -1)
                            if ($.inArray(record, proxy._flatFilteredRecords) == -1)
                                proxy._flatFilteredRecords.push(record);
                    }
                }
            }
            else {
                if (record.parentItem)
                    if ((proxy.model.filterSettings.filterHierarchyMode == "child" && !isSearching)
                        || (proxy.model.searchSettings.searchHierarchyMode == "child" && isSearching)) {
                        if ($.inArray(record.parentItem, proxy._filteredData) != -1)
                            proxy._addParentRecord(record.parentItem, isSearching);
                    }
                    else
                        proxy._addParentRecord(record.parentItem, isSearching);
                if ($.inArray(record, proxy._flatFilteredRecords) === -1) {
                    proxy._flatFilteredRecords.push(record);
                }
            }

        },


        //UPDATEEXPANDCOLLAPSE FOR SORTED RECORDS
        _updateExpandCollapseForSortedRecords: function (Record, expanded) {

            var proxy = this,
                count = 0,
                length = proxy._sortedRecords.length,
                record,
                index = proxy._sortedRecords.indexOf(Record);

            if (expanded) {

                proxy._addNestedSortRecords(Record, index);

            } else {

                var removedCount = 0;

                for (count = index + 1; count < length; count++) {

                    record = proxy._sortedRecords[count];

                    if (record.level === Record.level) {
                        break;
                    } else if (!proxy.getExpandStatus(record)) {
                        removedCount++;
                    }

                }

                proxy._sortedRecords.splice(index + 1, removedCount);

            }
        },


        //ADD NESTEDSORTRECORDS
        _addNestedSortRecords: function (record, index) {

            var proxy = this,
                childRecords,
                sortColumns = proxy.model.sortSettings.sortedColumns,
                length;

            if (sortColumns.length) {

                length = sortColumns.length - 1;
                proxy._queryManagar.queries = [];

                for (var i = length; i >= 0; i--) {

                    proxy._queryManagar.sortBy(sortColumns[i].field, sortColumns[i].direction);

                }

            }
            if (record.hasChildRecords) {

                var dataManager = ej.DataManager(record.childRecords),
                    count,
                    childrecord;

                childRecords = dataManager.executeLocal(proxy._queryManagar).result;
                length = childRecords.length;

                for (count = 0; count < length; count++) {

                    childrecord = childRecords[count];

                    if (proxy._filteredRecords.length) {

                        if ($.inArray(childrecord, proxy._filteredRecords) !== -1) {
                            proxy._sortedRecords.splice(++index, 0, childrecord);
                        }

                    } else {
                        proxy._sortedRecords.splice(++index, 0, childrecord);
                    }

                    if (childrecord.hasChildRecords && childrecord.expanded) {
                        proxy._addNestedSortRecords(childrecord, index);
                    }
                }
            }
        },

        //UPDATE Edit Settings
        updateEditSettings: function (edit) {

            var proxy = this;

            if (edit) {
                if (edit.editMode) {
                    if (proxy.model.editSettings.editMode === "normal" && edit.editMode === "cellEditing") {
                        proxy.model.editSettings.editMode = edit.editMode;
                        proxy._processEditing();
                    } else {
                        proxy.model.editSettings.editMode = edit.editMode;
                    }
                }
                if (!ej.isNullOrUndefined(edit.allowAdding))
                    proxy.model.editSettings.allowAdding = edit.allowAdding;
                if (!ej.isNullOrUndefined(edit.allowEditing))
                    proxy.model.editSettings.allowEditing = edit.allowEditing;
                if (!ej.isNullOrUndefined(edit.allowDeleting))
                    proxy.model.editSettings.allowDeleting = edit.allowDeleting;
                if (!ej.isNullOrUndefined(edit.beginEditAction))
                    proxy.model.editSettings.beginEditAction = edit.beginEditAction;
                proxy._enableEditingEvents();
            }
        },

        //UPDATE ALTROW
        updateAltRow: function () {

            var proxy = this,
                count = 0,
                currentViewData = proxy.model.updatedRecords,
                length = currentViewData.length,
                record, isAltRow = true;

            for (count = 0; count < length; count++) {
                record = currentViewData[count];
                if (record.isSummaryRow || !proxy.getExpandStatus(record))
                    continue;
                // record.isAltRow = count % 2 == 0 ? false : true;
                record.isAltRow = !isAltRow;
                isAltRow = record.isAltRow;
            }

        },


        
        //COUNT THE NUMBER OF CHILD RECORDS PRESENT IN THE GANTTRECORD
        _getFilteredChildRecordsCount: function (Record) {

            var proxy = this,
                model=proxy.model,
                length = Record&&Record.childRecords && Record.childRecords.length,
                i = 0,
                record;

            for (i = 0; i < length; i++) {

                record = Record.childRecords[i];
                if (model.enableVirtualization)
                {
                    if(proxy.getExpandStatus(record))
                    {
                        proxy._removedCount++;
                    }
                }
                else
                {
                    proxy._removedCount++;
                }
                   
                if (record.childRecords) {
                    proxy._getFilteredChildRecordsCount(record);
                }
            }

            return proxy._removedCount;
        },


        //SORTING THE RECORDS
        _sortingRecords: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns;

            proxy._queryManagar.queries = [];

            if (sortcolumns.length) {

                var length = sortcolumns.length - 1;

                for (var i = length; i >= 0; i--) {
                    var fieldName;
                    if (model.isFromGantt) {
                        var currentColumn = ej.TreeGrid.getColumnByMappingName(proxy.model.columns, sortcolumns[i].field);
                        if (ej.isNullOrUndefined(currentColumn))
                            currentColumn = this.getColumnByField(sortcolumns[i].field);
                        fieldName = currentColumn.field;
                    } else {
                        fieldName = sortcolumns[i].field;
                    }
                    if (model.isFromGantt) {
                            if (fieldName === "resourceInfo")
                                fieldName = "resourceNames";
                            else if (fieldName === "predecessor")
                                fieldName = "predecessorsName";
                    }

                    proxy._queryManagar.sortBy(fieldName, sortcolumns[i].direction);
                }
            }

            var dataManager = new ej.DataManager(model.parentRecords),
                records = dataManager.executeLocal(proxy._queryManagar).result;

            proxy._sortedRecords = [];
            proxy._storedIndex = -1;
            proxy._rowIndex = -1;

            proxy._createSortedRecords(records);
            proxy._tempsortedrecords = proxy._sortedRecords;
            proxy.model.updatedRecords = proxy._sortedRecords;
            if (proxy._isHierarchyFiltering)
                proxy._updateFilterParent = proxy._sortedRecords;
            //Update zeroLevelParentRecords
            if (model.allowPaging && model.pageSettings.pageSizeMode === ej.TreeGrid.PageSizeMode.Root)
                model.parentRecords = proxy._getParentRecords(proxy.model.updatedRecords, proxy, true);

            return true;
        },


        /**
        * Create sorted collection argumented data
        * @param {data} collection of records to sorting
        */
        _createSortedRecords: function (data) {

            var proxy = this, model = proxy.model,
                sortedRecords = proxy._sortedRecords,
                dataManager,
                enableAltRow = proxy.model.enableAltRow,model=this.model;
            data = proxy._spliceSummaryRows(data);
            $.each(data, function (index, record) {
                //Skip the summary record from sorting.
                if (record.isSummaryRow) {
                    var recordIndex = record.parentItem.childRecords.indexOf(record);
                    record.parentItem.childRecords.splice(recordIndex, 1);
                    return;
                }
                if ((proxy._filterString.length > 0 || proxy._searchString.length > 0) || (model.filterSettings.filteredColumns.length > 0)) {

                    if (proxy._filteredRecords.length > 0 && proxy._filteredRecords.indexOf(record) !== -1) {

                        proxy._storedIndex++;
                        proxy._rowIndex++;

                        if (enableAltRow) {

                            record.isAltRow = proxy._rowIndex % 2 == 0 ? false : true;

                        }

                        sortedRecords[proxy._storedIndex] = record;

                    }
                } else {

                    proxy._storedIndex++;
                    proxy._rowIndex++;

                    if (enableAltRow) {

                        record.isAltRow = proxy._rowIndex % 2 == 0 ? false : true;

                    }

                    sortedRecords[proxy._storedIndex] = record;

                }

                if (record.hasChildRecords && record.expanded) {
                    record.childRecords = proxy._spliceSummaryRows(record.childRecords);
                    dataManager = ej.DataManager(record.childRecords);
                    var childRecords = dataManager.executeLocal(proxy._queryManagar).result;
                    proxy._createSortedRecords(childRecords);

                }
                else if (record.hasChildRecords && !record.expanded && !model.enableVirtualization) {
                    record.childRecords = proxy._spliceSummaryRows(record.childRecords);
                    proxy._setChildRecords(record, sortedRecords);
                }
            });
        },

        _setChildRecords:function(record,sortedRecords)
        {
            var proxy = this,
                model = proxy.model,
                enableAltRow = proxy.model.enableAltRow,
                length = record.childRecords.length,
                dataManager;
            dataManager = ej.DataManager(record.childRecords);
            var childRecords = dataManager.executeLocal(proxy._queryManagar).result;
            for(var count=0;count<length;count++)
            {
                if ((proxy._filterString.length > 0 || proxy._searchString.length > 0) || (model.filterSettings.filteredColumns.length > 0)) {

                    if (proxy._filteredRecords.length > 0 && proxy._filteredRecords.indexOf(childRecords[count]) !== -1) {
                        proxy._storedIndex++;
                        if (enableAltRow) {
                            childRecords[count].isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        }
                        sortedRecords[proxy._storedIndex] = childRecords[count];
                        if (childRecords[count].hasChildRecords) {
                            this._setChildRecords(childRecords[count], sortedRecords);
                        }
                    }
                } else
                {
                    proxy._storedIndex++;
                    if (enableAltRow) {
                        childRecords[count].isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                    }
                    sortedRecords[proxy._storedIndex] = childRecords[count];
                    if (childRecords[count].hasChildRecords) {
                        this._setChildRecords(childRecords[count], sortedRecords);
                    }
                }
            }
        },
        //UPDAT THE SORTED RECORDS
        _updateSortedRecords: function (data) {

            var proxy = this,
                count = 0,
                length = data.length,
                record, model = this.model;

            for (count; count < length; count++) {

                record = data[count];

                if (!model.enableVirtualization) {

                    if (proxy._filteredRecords.length > 0) {

                        if ($.inArray(record, proxy._filteredRecords) !== -1 && $.inArray(record, proxy._sortedRecords) === -1) {
                            proxy._sortedRecords.push(record);
                        }

                    } else {

                        if ($.inArray(record, proxy._sortedRecords) === -1) {
                            proxy._sortedRecords.push(record);
                        }

                    }

                    if (record.hasChildRecords && record.childRecords.length > 0) {

                        var dataManager = ej.DataManager(record.childRecords),
                            sortedChildRecords = dataManager.executeLocal(proxy._queryManagar).result;
                        proxy._updateSortedRecords(sortedChildRecords);

                    }
                }
            }
        },


        //REMOVE SORT ICON FORM COLUMN HEADER
        _removeSortElementFromColumn: function (field) {

            var proxy = this,
                column = ej.TreeGrid.getColumnByField(proxy.model.columns, field),
                index = $.inArray(column, proxy.model.columns),
                $headerCellDiv = proxy.getHeaderTable().find("thead").find(".e-headercell").eq(index).find(".e-headercelldiv");

            $headerCellDiv.find(".ascending,.descending").remove();
            $headerCellDiv.parent().removeAttr("aria-sort");
        },


        //ADD EMPTYCOLUMN TO THE TREEGRID
        _addEmptyColumntoGrid: function () {

            var proxy = this;
            $(proxy.element).find('.e-gridheadercontainer table colgroup').append(ej.buildTag('col', "", {
                "width": "auto"
            },
            {}));
            proxy._$gridContainer.find('table colgroup').append(ej.buildTag('col', "", {
                "width": "auto"
            },
            {}));
            $(proxy.element).find('.e-gridheadercontainer table thead tr').append(ej.buildTag('th.e-headercell'));
        },


        //UPDATE THE SCROLLTOP OF THE GRIDCONTAINER
        updateScrollTop: function (y, isFromChart) {

            var args = {},
                proxy = this;

            args.requestType = ej.TreeGrid.Actions.Scroll;
            args.delta = y;

            var isHorizontalScroll = proxy.getScrollElement().ejScroller("isHScroll");

            if (!isHorizontalScroll && this.model.isFromGantt) {
                proxy._$gridContent.scrollTop(y);//grid container
            }
            else
            {
                proxy._$gridContent.scrollTop(0);
            }
            proxy.getScrollElement().ejScroller("scrollY", y, true);
            if (proxy.model.isFromGantt && !isFromChart) {
                proxy._completeAction(args);
               proxy._focusTreeGridElement();
            }
            else proxy._isFromChart = false;
            
        },

        _selectNextCell: function (direction, action) {
            var proxy = this, model = proxy.model,
                columns = model.columns,
                columnLength = columns.length,
                rowElement, cellElement,
                selectingRowIndex,
                currentSelectingRecord,
                $form = $("#" + proxy._id + "EditForm"),
                updatedRecords = proxy.getUpdatedRecords(),
                expandedRecords = model.allowPaging ? updatedRecords : proxy.getExpandedRecords(model.updatedRecords);
            proxy._isShiftKeyNavigation = false;

            switch (direction) {
                case "right":                   
                    if ((proxy._focusingRowIndex >= updatedRecords.length - 1 && proxy._cellIndex >= columnLength - 1)
                        || (proxy._focusingRowIndex == -1 || proxy._cellIndex == -1)) {
                        proxy.element[0].focus();
                        return;
                    }
                    proxy._cellIndex += 1;
                    if (proxy._cellIndex == columnLength) {
                        proxy._cellIndex = 0;
                        var selectedItem = updatedRecords[proxy._rowIndexOfLastSelectedCell];
                        selectingRowIndex = expandedRecords.indexOf(selectedItem);
                        currentSelectingRecord = expandedRecords[selectingRowIndex + 1];
                        //To skip the key Navigation from summary row.
                        if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                            currentSelectingRecord = proxy._getNextRecord(selectingRowIndex + 1, expandedRecords);

                        if (!currentSelectingRecord)
                            return;
                        proxy._focusingRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                    }
                    if (proxy._focusingRowIndex >= updatedRecords.length)
                        return;
                    if ((!columns[proxy._cellIndex].visible || !columns[proxy._cellIndex].allowCellSelection) && $form.length <= 0)
                        proxy._cellIndex = proxy.getUpNextVisibleColumnIndex(proxy._cellIndex, action);
                    if ($form.length > 0 && columns[proxy._cellEditingDetails.columnIndex].allowCellSelection)
                        proxy._cellIndex = proxy._cellEditingDetails.columnIndex;
                    var cellInfo = {
                        rowIndex: proxy._focusingRowIndex,
                        cellIndex: proxy._cellIndex
                    };
                    proxy.selectCells([cellInfo]);
                    proxy.updateScrollBar();
                    break;
                case "left":                    
                    proxy._cellIndex -= 1;
                    columnLength = columns.length;

                    if (proxy._cellIndex < 0) {
                        proxy._cellIndex = columnLength - 1;
                        var selectedItem = updatedRecords[proxy._rowIndexOfLastSelectedCell];
                        selectingRowIndex = expandedRecords.indexOf(selectedItem);
                        currentSelectingRecord = expandedRecords[selectingRowIndex - 1];
                        //To skip the key Navigation from summary row.
                        if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                            currentSelectingRecord = proxy._getNextRecord(selectingRowIndex - 1, expandedRecords, "upArrow");

                        if (!currentSelectingRecord)
                            return;
                        proxy._focusingRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                    }
                    if (proxy._focusingRowIndex < 0)
                        return;
                    if (proxy._focusingRowIndex >= updatedRecords.length)
                        proxy._focusingRowIndex = updatedRecords.length - 1;
                    if ((!columns[proxy._cellIndex].visible || !columns[proxy._cellIndex].allowCellSelection) && $form.length <= 0)
                        proxy._cellIndex = proxy.getPreviousVisibleColumnIndex(proxy._cellIndex, action);
                    if ($form.length > 0 && columns[proxy._cellEditingDetails.columnIndex].allowCellSelection)
                        proxy._cellIndex = proxy._cellEditingDetails.columnIndex;

                    var cellInfo = {
                        rowIndex: proxy._focusingRowIndex,
                        cellIndex: proxy._cellIndex
                    };
                    proxy.selectCells([cellInfo]);
                    proxy.updateScrollBar();
                    break;
            }
        },
        //MOVE CURRENT CELL FOR KEYBOARD INTERACTION
        _moveCurrentCell: function (direction) {

            var proxy = this,
                rowIndex = proxy._cellEditingDetails.rowIndex,
                columnIndex = proxy._cellEditingDetails.columnIndex,
                model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords();

            if (rowIndex == -1 && columnIndex == -1) return false;

            if (proxy.model.isEdit) {

                proxy.saveCell();
                proxy.model.isEdit = false;

            }
            if (model.enableVirtualization) {
                var record = model.currentViewData[rowIndex];
                proxy._cellEditingDetails.rowIndex = rowIndex = updatedRecords.indexOf(record);
            }
            switch (direction) {

                case "right":
                    var columns = model.columns,
                        columnLength = columns.length,
                        hiddenColumn = proxy.getHiddenColumns(),
                        visibleColumn = $.extend([], columns),
                        nextColumnIndex, newColumnIndex, visibleColumnLength;
                    if(hiddenColumn)
                    hiddenColumn.forEach(function (item) {
                        visibleColumn.splice(visibleColumn.indexOf(item), 1);
                    });
                    newColumnIndex = visibleColumn.indexOf(columns[columnIndex]);
                    visibleColumnLength = visibleColumn.length;
                    if (rowIndex == updatedRecords.length - 1 && newColumnIndex == visibleColumnLength - 1) {
                        return true;
                    }
                    if (newColumnIndex == visibleColumnLength - 1) {
                        nextColumnIndex = 0;
                        while (!columns[nextColumnIndex].visible) {
                            nextColumnIndex++;
                        }
                        proxy._cellEditingDetails.columnIndex = nextColumnIndex;
                        if(proxy._getNextRow(direction, rowIndex, proxy))
                            return true;
                        
                    }
                    else {
                        nextColumnIndex = columnIndex + 1;
                        while (!columns[nextColumnIndex].visible) {
                            nextColumnIndex++;
                        }
                        proxy._cellEditingDetails.columnIndex = nextColumnIndex;
                    }
                    while (model.columns[nextColumnIndex].allowEditing == false || !columns[nextColumnIndex].visible) {
                        if ((nextColumnIndex + 1) == model.columns.length) {
                            proxy._cellEditingDetails.columnIndex = nextColumnIndex = 0;
                            if (proxy._getNextRow(direction, rowIndex, proxy))
                                return true;
                            continue;
                        }
                        proxy._cellEditingDetails.columnIndex = ++nextColumnIndex;
                    }
                    proxy.updateScrollBar();
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex,
                        model.columns[proxy._cellEditingDetails.columnIndex].field);
                    proxy._editAddTools();
                    break;

                case "left":
                    var columns = model.columns,
                       columnLength = columns.length,
                       hiddenColumn = proxy.getHiddenColumns(),
                       visibleColumn = $.extend([], columns),
                       prevColumnIndex, newColumnIndex, visibleColumnLength;
                    if (hiddenColumn)
                        hiddenColumn.forEach(function (item) {
                            visibleColumn.splice(visibleColumn.indexOf(item), 1);
                        });
                    newColumnIndex = visibleColumn.indexOf(columns[columnIndex]);
                    visibleColumnLength = visibleColumn.length;
                    if ((rowIndex == 0 && newColumnIndex == 0)) return true;
                    if (newColumnIndex == 0) {
                        prevColumnIndex = model.columns.length - 1;
                        while (!columns[prevColumnIndex].visible) {
                            prevColumnIndex--;
                        }
                        proxy._cellEditingDetails.columnIndex = prevColumnIndex;
                        if(proxy._getNextRow(direction, rowIndex, proxy))
                            return true;
                    } else {
                        prevColumnIndex = columnIndex - 1;
                        while (!columns[prevColumnIndex].visible) {
                            prevColumnIndex--;
                        }
                        proxy._cellEditingDetails.columnIndex = prevColumnIndex;
                    }

                    while (model.columns[prevColumnIndex].allowEditing == false || !columns[prevColumnIndex].visible) {

                        if ((prevColumnIndex - 1) == -1) {
                            proxy._cellEditingDetails.columnIndex = prevColumnIndex = columnLength - 1;
                            if (proxy._getNextRow(direction, rowIndex, proxy))
                                return true;
                            continue;
                        }
                        proxy._cellEditingDetails.columnIndex = --prevColumnIndex;
                    }
                    proxy.updateScrollBar();
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                    proxy._editAddTools();
                    break;

                case "up":
                    if (rowIndex == 0) return true;
                    proxy._cellEditingDetails.rowIndex = rowIndex - 1;
                    proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                    proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                    break;

                case "down":
                    if (rowIndex + 1 == updatedRecords.length) {
                        proxy._focusTreeGridElement();
                        proxy._isEnterKeyPressed = true;
                    }
                    if (model.showSummaryRow) {
                        if (rowIndex + 1 == updatedRecords.length - model.summaryRows.length)
                            proxy._isEnterKeyPressed = true;
                    }
                    if (!proxy._isEnterKeyPressed) {
                        if (updatedRecords.length > (rowIndex + 1))
                            if (!model.enableVirtualization) {
                                var currentRecord = updatedRecords[rowIndex],
                                    expandedRecords = model.allowPaging? updatedRecords : proxy.getExpandedRecords(model.updatedRecords),
                                    nextRecord = expandedRecords[expandedRecords.indexOf(currentRecord) + 1];
                                //Skip the summary row from keyboard navigation.
                                if (nextRecord && nextRecord.isSummaryRow)
                                    nextRecord = proxy._getNextRecord(rowIndex + 1, expandedRecords);
                                var index = updatedRecords.indexOf(nextRecord);
                                if ((rowIndex == expandedRecords.length - 1 && columnIndex == model.columns.length - 1) || index == -1) {
                                    return true;
                                }
                                proxy._cellEditingDetails.rowIndex = index;
                            }
                            else {
                                var nextRecord = updatedRecords[rowIndex + 1];
                                //Skip the summary row from keyboard navigation.
                                if (nextRecord && nextRecord.isSummaryRow)
                                    nextRecord = proxy._getNextRecord(rowIndex + 1, updatedRecords);
                                index = updatedRecords.indexOf(nextRecord);
                                if (index == -1)
                                    return true;
                                proxy._cellEditingDetails.rowIndex = index;
                            }
                        if (!proxy._rowSelectingEventTrigger(proxy.selectedRowIndex(), proxy._cellEditingDetails.rowIndex)) {
                            proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                            proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
                        }
                        proxy.updateScrollBar();
                        var args = {};
                        args.requestType = ej.TreeGrid.Actions.Scroll;
                        args.delta = proxy._$gridContent.children('.e-content').scrollTop();
                        proxy._trigger("actionComplete", args);
                        proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                        proxy._editAddTools();
                    }
                    else
                        proxy._isEnterKeyPressed = false;
                    break;
            }

            return false;

        },
        //To skip the summary record. 
        _getNextRecord: function (index, expandedRecords, action) {
            var nextRecord;
            do {
                nextRecord = expandedRecords[index];
                if (action == "upArrow" || action == "lastRowSelection" || action == "bottomRowSelection" || action=="child" || action == "above")
                    index--;
                else
                    index++;
            } while (nextRecord && nextRecord.isSummaryRow);
            return nextRecord;
        },

        _getNextRow: function (direction, rowIndex, proxy) {
            var cellIndex,
            model = proxy.model,
            index,
            updatedRecords = proxy.getUpdatedRecords();
            if (direction == "right")
                cellIndex = 1;
            else
                cellIndex = -1;

            if (!model.enableVirtualization) {
                var currentRecord = updatedRecords[rowIndex],
                  expandedRecords = model.allowPaging ? updatedRecords : proxy.getExpandedRecords(model.updatedRecords),
                  nextRecord = expandedRecords[expandedRecords.indexOf(currentRecord) + cellIndex];
                //Skip the summary row from keyboard navigation.
                if (nextRecord && nextRecord.isSummaryRow) {
                    var action = cellIndex > 0 ? "" : "lastRowSelection";
                    nextRecord = proxy._getNextRecord(rowIndex + cellIndex, expandedRecords, action);
                }
                if (model.allowPaging && nextRecord) {
                    index = updatedRecords.indexOf(nextRecord);
                    if (index < 0 && cellIndex == 1)
                        proxy.gotoPage(proxy._currentPage() + 1);
                    else if (index < 0 && cellIndex == -1)
                        proxy.gotoPage(proxy._currentPage() - 1);
                }
                index = updatedRecords.indexOf(nextRecord);
                if(index == -1) {
                    return true;
                }
                proxy._cellEditingDetails.rowIndex = index;
            }
            else {
                var nextRecord = updatedRecords[rowIndex + cellIndex];
                //Skip the summary row from keyboard navigation.
                if (nextRecord && nextRecord.isSummaryRow) {
                    var action = cellIndex > 0 ? "" : "lastRowSelection";
                    nextRecord = proxy._getNextRecord(rowIndex + cellIndex, updatedRecords, action);
                }
                if (model.allowPaging && nextRecord) {
                    index = updatedRecords.indexOf(nextRecord);
                    if (index < 0 && cellIndex == 1)
                        proxy.gotoPage(proxy._currentPage() + 1);
                    else if(index < 0 && cellIndex == -1)
                        proxy.gotoPage(proxy._currentPage() - 1);
                }
                index = updatedRecords.indexOf(nextRecord);
                if (index == -1)
                    return true;
                proxy._cellEditingDetails.rowIndex = index;
            }
            if (model.allowSelection && model.selectionSettings.selectionMode == "cell") {
                model.selectedItem = updatedRecords[proxy._cellEditingDetails.rowIndex];
                proxy._rowIndexOfLastSelectedCell = proxy._cellEditingDetails.rowIndex;
            }
            else if (!proxy._rowSelectingEventTrigger(proxy.selectedRowIndex(), proxy._cellEditingDetails.rowIndex)) {
                proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
            }

        },

        //FIND THE COLUMNWIDTH OF THE TREEGRID COLUMN
        _findColumnsWidth: function (extra) {

            var proxy = this,
                length = proxy.model.columns.length,
                j = proxy.getHeaderTable().find("colgroup").find("col");
            
            for (var i = 0; i < length; i++)
                proxy.columnsWidthCollection[i] = j.eq(i).width();

        },

        //CALCULATE THE TOTAL WIDTH OF THE COLUMNS
        _calculateWidth: function () {

            var j = this.getHeaderTable().find("colgroup").find("col"),
                width = 0;

            for (var i = 0; i < j.length; i++)
                width += j.eq(i).width();

            return width;
        },
        getScrollObject: function () {
            return this.getScrollElement().data("ejScroller");

        },
        //update element height when viewport height is <0
        _updateElementHeight:function()
        {
            var proxy = this;
            proxy._calculateDimensions();
            proxy.updateViewPortHeight();
        },
        /* Get scroller rendered div in treegrid*/
        getScrollElement: function () {
            if (this._frozenColumnsLength > 0)
                return this.element.find("#e-movablecontainer" + this._id);
            else
                return this._$gridContent;
        },

        /* Get scroller rendered frozen div in treegrid*/
        _getFrozenScrollElement: function () {
            if (this._frozenColumnsLength > 0)
                return this.element.find("#e-frozencontainer" + this._id);
            else
                return null;
        },
        /* Get width of all columns before the index or get frozen column width*/
        _getFrozenColumnWidth: function (index) {
            var width = 0,
                index = ej.isNullOrUndefined(index) ? this._frozenColumnsLength : index,
                columns = ej.isNullOrUndefined(index) ? this._frozenColumns : this.model.columns;
            if (index > 0)
            {
                for (var count = 0; count < index; count++) {
                    if (columns[count].visible == true) {
                        width += columns[count] ? this.columnsWidthCollection[count] : 0;
                    }
                }
            }
            return width;
        },
        _renderScroller: function (isFromFreezeColumn) {

            var proxy = this,
            model = proxy.model;
            var containerHeight = model.flatRecords.length * model.rowHeight;
            var contentHeight = proxy._viewPortHeight, autoHide = false;            
            if (model.allowPaging && !model.sizeSettings.height) {
                if (model.allowTextWrap && !model.enableVirtualization) {
                    if (proxy._$frozenTableContent.length > 0) {
                        var frozenHeight = contentHeight = proxy._$gridContainer.find("#e-frozencontentdiv" + proxy._id).outerHeight(),
                            movableHeight = proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id).outerHeight();
                        if (movableHeight > frozenHeight)
                            contentHeight = proxy._$gridContainer.find("#e-movablecontentdiv" + proxy._id).outerHeight();
                    } else {
                        contentHeight = proxy._$gridContainer.find('.e-table').outerHeight();
                    }
                }
            }
            if (!proxy._$gridContent.hasClass("e-scroller") || (model.allowPaging && !model.sizeSettings.height)) {
                var height = proxy._gridHeight ? contentHeight : "auto", enableTouchScroll = false;
                var width = this._gridWidth, targetPane = null;
                if (this._frozenColumnsLength > 0) {
                    proxy._$gridContent.css("width", width - proxy._totalBorderWidth);
                    width = width - this._getFrozenColumnWidth() - 1;//frozen div border width
                }
                if (!model.isFromGantt) enableTouchScroll = true;
                if (this.mobileDevice() && !model.isFromGantt) {
                    autoHide = true;
                    proxy._scrollBarHeight = 10;
                }

                proxy.getScrollElement().ejScroller({
                    enableTouchScroll: false,
                    autoHide: autoHide,
                    scrollerSize: proxy._scrollBarHeight,
                    height: proxy.model.isFromGantt ? 0 : model.sizeSettings.height == "auto" ? 0 : height,
                    width: proxy.model.isFromGantt ? width : width - proxy._totalBorderWidth,
                    scroll: $.proxy(proxy._onScroll, proxy)
                });
            }
            else {
                proxy.getScrollElement().ejScroller("refresh");
            }
            proxy.getScrollElement().ejScroller("model.keyConfigs", { up: "", down: "", left: "", right: "" });
            if (ej.isNullOrUndefined(isFromFreezeColumn))
                proxy._updateScrollCss();
            var isHorizontalScroll = proxy.getScrollElement().ejScroller("isHScroll");
            if (model.allowPaging && !model.sizeSettings.height && isHorizontalScroll) {
                var scrollHeight = height + proxy._scrollBarHeight;
                proxy.getScrollElement().ejScroller({
                    height: scrollHeight,
                });
                proxy.getScrollElement().ejScroller("refresh");
            }
            if (!this.mobileDevice() && this.getBrowserDetails().browser== "safari" && this._frozenColumnsLength > 0) {
                this.getHeaderContent().find("#e-movableheader" + proxy._id).add(this.getContent().find("#e-movablecontainer" + proxy._id)).css("margin-left", "auto");
                if(this._$totalSummaryRowContainer)
                    this._$totalSummaryRowContainer.find("#e-movablefooter" + proxy._id).css("margin-left", "auto");
            }
            if (this._frozenColumnsLength > 0) {
                proxy._$gridContent.height(height);
                proxy.getContent().find("#e-frozencontainer" + proxy._id).height(proxy.getContent().find("#e-movablecontent" + proxy._id).height());
            }
        },
        

        
        //ALL EVENTS BOUND USING THIS._ON WILL BE UNBIND AUTOMATICALLY
        _destroy: function () {
            var proxy = this,
                model = this.model,
                columns = model.columns;
            proxy.element.off();
            
            proxy._off(proxy.element, "click", proxy._onClick);
            proxy._off(proxy.element, "keydown", proxy._keyDown);
            proxy._off(proxy.element, "click", "#" + proxy._id + "e-gridheader", proxy._onHeaderClick);
            proxy._off(proxy.element, "mousedown touchstart", "#" + proxy._id + "e-gridheader", proxy._headerMouseDown);
            proxy._off($(document), "mousedown touchstart", proxy._mouseDownHandler);
            proxy._off(proxy.element, "mousedown touchstart", ".e-gridcontent", proxy._contentMouseDown);
            proxy._off(proxy.element, "click", proxy._onRecordClick);// ".e-gridcontent",
            proxy._off(proxy.element, "dblclick", ".e-gridcontent > div", proxy._onRecordDoubleClick);

            //Method binding for filtering data.
            if (model.filterSettings.filterBarMode == "immediate") {
                proxy._off(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox")._off(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
            }

            if (model.isResponsive)
                proxy._off(this.element, "swipeleft swiperight", ".e-gridcontent div > .e-table", $.proxy(this._swipeAction, this));

            proxy._off(proxy.element, "mousemove", proxy._mouseMove);
            proxy._off(proxy.element, "touchmove", proxy._mouseMove);

            proxy._off(proxy.element, "focus click", ".e-filterbarcell input", proxy._filterBarClose);
            proxy._off(proxy.element, "click", ".e-filterbarcell .e-cancel", proxy._filterBarClose);
            if (proxy.model.showGridCellTooltip && !proxy.model.showGridExpandCellTooltip) {

                if (ej.browserInfo().name == "msie") {
                    proxy._off(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
                }
                else {
                    proxy._off(proxy.element, "mousemove", ".e-rowcell", proxy._mouseHover);
                }
            }
            else {
                proxy._off(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
            }
            if (proxy.model.showGridCellTooltip) {

                if (ej.browserInfo().name == "msie") {
                    proxy._off(proxy.element, "mouseenter", ".e-headercelldiv", proxy._headermouseHover);
                }
                else {
                    proxy._off(proxy.element, "mousemove", ".e-headercelldiv", proxy._headermouseHover);
                }
            }
            proxy._off(proxy.element, "mouseleave", ".e-rowcell", proxy._cellMouseLeave);
            proxy._off(proxy.element, "mouseleave", proxy._cellMouseLeave);
            proxy._off(proxy.element, "mouseleave", ".e-headercelldiv", proxy._headerMouseLeave);
            proxy._off(proxy.element, "mouseleave", proxy._headerMouseLeave);
            

            //MOUSE WHEEL EVENT FOR GANTT CONTROL
            if (model.isFromGantt || this._frozenColumnsLength) {
                proxy._off(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);
            }

            if (model.enableResize && model.isResponsive && !model.isFromGantt) {
                proxy._off($(window), "resize", proxy._windowResize);
            }
            if (ej.isTouchDevice())
                proxy._off(proxy.element, "doubletap ", ".e-gridcontent", proxy._editdblClickHandler);
            else
                proxy._off(proxy.element, "dblclick ", ".e-gridcontent", proxy._editdblClickHandler);
            //Mouse right click event for tree grid


            if (model.contextMenuSettings.showContextMenu) {
                proxy._off(proxy.element, "keyup contextmenu taphold", this._preventContextMenu);
            }
            proxy.getScrollElement().ejScroller("content").stop(true);
            this._clearColumnMenu();
            this._clearContextMenu();
            this._removeDetailsRow();
            $("#" + this._id + "_toolbarItems_Main").remove();
            if (proxy._columnRenameDialog) {
                $("#" + proxy._columnRenameDialog.attr("id") + "_wrapper").clearQueue();
                $("#" + proxy._columnRenameDialog.attr("id") + "_wrapper").stop();
                proxy._columnRenameDialog.data("ejDialog") && proxy._columnRenameDialog.data("ejDialog").destroy();
                $("#" + proxy._columnRenameDialog.attr("id") + "_wrapper").remove();
                $("#" + proxy._id + 'ColumnRenameDialog').remove();
            }
            if (this.model.showColumnChooser) {
                $("#" + proxy._id + "ccDiv_wrapper").clearQueue();
                $("#" + proxy._id + "ccDiv_wrapper").stop();
                $("#" + this._id + "ccDiv").data("ejDialog") && $("#" + this._id + "ccDiv").data("ejDialog").destroy();
                $("#" + proxy._id + "ccDiv_wrapper").remove();
                $("#" + this._id + "ccDiv").remove();
            }
            if (proxy._updateConfirmDialog) {
                $("#" + proxy._updateConfirmDialog.attr("id") + "_wrapper").clearQueue();
                $("#" + proxy._updateConfirmDialog.attr("id") + "_wrapper").stop();
                proxy._updateConfirmDialog.data("ejDialog") && proxy._updateConfirmDialog.data("ejDialog").destroy();
                $("#" + proxy._updateConfirmDialog.attr("id") + "_wrapper").remove();
                $("#" + proxy._id + 'ConfirmDialog').remove();
            }

            //destroying delete confirmation dialog
            if (proxy._confirmDialog) {
                $("#" + proxy._confirmDialog.attr("id") + "_wrapper").clearQueue();
                $("#" + proxy._confirmDialog.attr("id") + "_wrapper").stop();
                proxy._confirmDialog.data("ejDialog") && proxy._confirmDialog.data("ejDialog").destroy();
                $("#" + proxy._confirmDialog.attr("id") + "_wrapper").remove();
                $("#" + proxy._id + '_ConfirmDialog').remove();
            }
            if ($("#" + proxy._id + "_dialogColumnAdd_wrapper").length > 0) {
                $("#" + proxy._id + "_dialogColumnAdd_wrapper").clearQueue();
                $("#" + proxy._id + "_dialogColumnAdd_wrapper").stop();
                $("#" + proxy._id + "_dialogColumnAdd").find('.e-datepicker').ejDatePicker("destroy");
                $("#" + proxy._id + "_dialogColumnAdd").find('.e-datetimepicker').ejDateTimePicker("destroy");
                $("#" + proxy._id + "_dialogColumnAdd").find('.e-numerictextbox').ejNumericTextbox("destroy");
                $("#" + proxy._id + "_dialogColumnAdd").find('.e-dropdownlist').ejDropDownList("destroy");
                $("#" + proxy._id + "_dialogColumnAdd").find('.e-treegrid').ejTreeGrid("destroy");
                $("#" + proxy._id + "_dialogColumnAdd").data("ejDialog") && $("#" + proxy._id + "_dialogColumnAdd").data("ejDialog").destroy();
                $("#" + proxy._id + "_dialogColumnAdd_wrapper").remove();
                $("#" + proxy._id + "_dialogColumnAdd").remove();
            }

            //destroy all widgets in filtterbar
            if (model.allowFiltering && !model.isFromGantt) {
                for (var count = 0 ; count < columns.length; count++) {
                    var currentElement = $("#" + this._id + "_" + columns[count].field.split('.').join("") + "_filterbarcell");
                    currentElement.data("ejWidgets") && $(currentElement).data($(currentElement).data("ejWidgets")[0]).destroy();
                }
            }

            if (proxy.model.allowFiltering && proxy.model.filterSettings.filterType == "menu") {
                var menuFilter = proxy._renderedFilterMenuDialog, filtertype;
                for (var i = 0; i < menuFilter.length; i++) {
                    var str;
                    if (menuFilter[i]) {
                        var index = menuFilter[i].indexOf("edit");
                        filtertype = menuFilter[i];
                        if (index != -1) {
                            var editType = filtertype.slice(0, index);
                            str = editType.charAt(0).toUpperCase() + editType.slice(1);
                        } else {
                            str = filtertype.charAt(0).toUpperCase() + filtertype.slice(1);
                        }
                    }
                    if (str == "String")
                        $("#" + this._id + "filter" + str + "MenuDiv").find('.e-autocomplete').ejAutocomplete("destroy");
                    else if (str == "Datepicker")
                        $("#" + this._id + "filter" + str + "MenuDiv").find('.e-datepicker').ejDatePicker("destroy");
                    else if (str == "Datetimepicker")
                        $("#" + this._id + "filter" + str + "MenuDiv").find('.e-datetimepicker').ejDateTimePicker("destroy");
                    else if (str == "Numeric")
                        $("#" + this._id + "filter" + str + "MenuDiv").find('.e-numerictextbox').ejNumericTextbox("destroy");

                    $("#" + this._id + "filter" + str + "MenuDiv_wrapper").clearQueue();
                    $("#" + this._id + "filter" + str + "MenuDiv_wrapper").stop();
                    $("#" + this._id + "filter" + str + "MenuDiv").find('.e-dropdownlist').ejDropDownList("destroy");
                    $("#" + this._id + "filter" + str + "MenuDiv").data("ejDialog") && $("#" + this._id + "filter" + str + "MenuDiv").data("ejDialog").destroy();
                    $("#" + this._id + "filter" + str + "MenuDiv_wrapper").remove();
                    $("#" + this._id + "filter" + str + "MenuDiv").remove();
                }
            }
            if (proxy._commandColumnLength > 0) {
                for (var i = 0; i < columns.length; i++) {
                    if (columns[i].commands && columns[i].commands.length > 0) {
                        $("#" + this._id + i + "_UnboundTemplate").remove();
                    }
                }
            }
            //Clear excel filtering dialog
            var excelColType = ["number", "string", "date", "datetime","boolean"],
                colTypeLength = excelColType.length;
            for (var i = 0; i < colTypeLength; i++) {
                $("#"+proxy._id+excelColType[i]+"_CustomDrop1_popup_wrapper").clearQueue();
                $("#" + proxy._id + excelColType[i] + "_CustomDrop1_popup_wrapper").stop();
                $("#" + proxy._id + excelColType[i] + "_CustomDrop1").data("ejDropDownList") && $("#" + proxy._id + excelColType[i] + "_CustomDrop1").data("ejDropDownList").destroy();
                $("#" + proxy._id + excelColType[i] + "_CustomDrop2_popup_wrapper").clearQueue();
                $("#" + proxy._id + excelColType[i] + "_CustomDrop2_popup_wrapper").stop();
                $("#" + proxy._id + excelColType[i] + "_CustomDrop2").data("ejDropDownList") && $("#" + proxy._id + excelColType[i] + "_CustomDrop2").data("ejDropDownList").destroy();
                $("#" + proxy._id + excelColType[i] + "_CheckBoxList_WaitingPopup").remove();
                
                switch (excelColType[i]) {
                    case "string":
                        $("#" + proxy._id + excelColType[i] + "_CustomValue1_suggestion").remove();
                        $("#" + proxy._id + excelColType[i] + "_CustomValue2_suggestion").remove();
                        break;
                    case "date":
                        $("#e-" + proxy._id + excelColType[i] + "_CustomValue1").remove();
                        $("#e-" + proxy._id + excelColType[i] + "_CustomValue2").remove();
                        break;
                    case "datetime":
                        $("#" + proxy._id + excelColType[i] + "_CustomValue1_popup").remove();
                        $("#" + proxy._id + excelColType[i] + "_CustomValue2_popup").remove();
                        break;
                
            }
                
            }
            proxy.element.empty().removeClass("e-treegrid-core e-treegrid " + proxy.model.cssClass);  
			proxy.element.data("ejWaitingPopup") && proxy.element.ejWaitingPopup("destroy");            
            for (var i = 0; i < columns.length; i++) {
                if (columns[i].isTemplateColumn) {
                    var headerText = columns[i]['headerText'];
                    if (!ej.isNullOrUndefined(headerText) && !ej.isNullOrUndefined(headerText.match(/(\\|[^0-9A-z-_])/g)))
                        headerText = headerText.replace(/(\\|[^0-9A-z-_])/g, "");
                    $("#" + proxy._id + headerText + $.inArray(columns[i], proxy.model.columns) + "_Template").length > 0 &&
                        $("#" + proxy._id + headerText + $.inArray(columns[i], proxy.model.columns) + "_Template").remove();
                }
                if (columns[i].headerTemplateID) {
                    var idText = this._id + (columns[i].field ? columns[i].field : "") + $.inArray(columns[i], this.model.columns) + "_HeaderTemplate";
                    $("#" + idText).remove();
                }
            }     
            // remove jsrender helpers
            var helpers = $.views.helpers;
            for (var key in helpers) {
                if (key.indexOf(this._id) != -1) {
                    delete helpers[key];
                }
            }
            proxy._columnChooserList.remove();
        },

        

        collapseAll: function () {
            var proxy = this,
                model = proxy.model,                
                args = {},
                enableVirtualization = model.enableVirtualization,
                $gridRows = proxy.getRows(),
                $rowElement = null;                        
            if (!proxy.editFormValidate()) return;
            proxy._expandCollapseSettings.state = "collapseAll";
            proxy._expandCollapseSettings.atLevel = "";
            proxy._expandCollapseSettings.level = 0;
            proxy._expandCollapsePages = [];
            //To cancel the cell edit or row edit mode when collapse the all record.
            if (proxy._isRowEdit || proxy.model.isEdit) {
                proxy._isRowEdit ? proxy.cancelRowEditCell() : proxy.cancelEditCell();
                proxy.selectRows(-1);                          
                proxy._cancelSaveTools();
            }
            args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
            //loop excecute for iterate the GanttRecords preseent in the Gantt Control
            proxy._isInExpandCollapseAll = true;                                  
            proxy._collapseAllData();

            if (model.flatRecords.length > 0)
                proxy._refreshTreeGridOnExpandCollapseAll(args);

            proxy._isInExpandCollapseAll = false;
            proxy.updateHeight();            
        },
        _triggerExpandingCollapsingEvent: function (args) {
            var proxy = this, isExpandCollapseEnabeled;
            if (args.expanded) {
                isExpandCollapseEnabeled = proxy._trigger("expanding", args);
            } else {
                isExpandCollapseEnabeled = proxy._trigger("collapsing", args);
            }
            return isExpandCollapseEnabeled
        },
        _collapseAllData: function () {
            var proxy = this, model = proxy.model,
                count, args = {};
            args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
            if (model.allowPaging && model.pageSettings.pageSizeMode == ej.TreeGrid.PageSizeMode.Root && model.enableVirtualization) {
                var pageSize = model.pageSettings.pageSize,
                    pageIndex = proxy._currentPage(),
                    countLength = pageSize * pageIndex,
                    initCount = pageSize * (pageIndex - 1);
                proxy._expandCollapsePages.push(pageIndex);
                for (count = initCount; count < countLength; count++) {
                    var treegridRecord = proxy._isHierarchyFiltering ? proxy._filterParentRecords[count] : proxy.model.parentRecords[count];
                    if (treegridRecord) {
                        args.data = treegridRecord;
                        args.recordIndex = model.updatedRecords.indexOf(treegridRecord);
                        args.expanded = false;
                        if (treegridRecord.hasChildRecords) {
                            if (!proxy._triggerExpandingCollapsingEvent(args)) {
                                proxy._expandCollapseInnerLevelRecord(treegridRecord, args.expanded);
                                ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                            }
                        }
                    }
                }
            }            
            else {
                var parentRecord = proxy._isHierarchyFiltering ? proxy._filterParentRecords : proxy.model.parentRecords;
                for (count = 0; count < parentRecord.length; count++) {
                    var treegridRecord = parentRecord[count];
                    args.data = treegridRecord;
                    args.recordIndex = model.updatedRecords.indexOf(treegridRecord);
                    args.expanded = false;
                    if (treegridRecord.hasChildRecords) {
                        if (!proxy._triggerExpandingCollapsingEvent(args)) {
                            proxy._expandCollapseInnerLevelRecord(treegridRecord, args.expanded);
                            ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        }
                    }
                }
            }            
        },        
        //populate the context menu with items
        _getContextMenuItems: function () {
            var contextMenuItems = [],
            proxy = this,
            contextMenuLabel = proxy._contextMenuTexts;
            contextMenuItems = [ {
                headerText: contextMenuLabel["addRowText"],
                eventHandler: null,
                isDefault: true,
                disable: false,
                menuId: "Add",
                parentMenuId: null
            },
             {
                 iconClass: "e-edit",
                 headerText: contextMenuLabel["editText"],
                 eventHandler: null,
                 isDefault: true,
                 disable: false,
                 menuId: "Edit",
                 parentMenuId: null
             },
            {
                headerText: contextMenuLabel["deleteText"],
                eventHandler: null,
                isDefault: true,
                disable: false,
                menuId: "Delete",
                parentMenuId: null
            },
            {
                iconClass: "e-save",
                headerText: contextMenuLabel["saveText"],
                eventHandler: null,
                isDefault: true,
                isEdit: true,
                disable: false,
                menuId: "Save",
                parentMenuId: null
            },
             {
                 iconClass: "e-cancel",
                 headerText: contextMenuLabel["cancelText"],
                 eventHandler: null,
                 isDefault: true,
                 isEdit: true,
                 disable: false,
                 menuId: "Cancel",
                 parentMenuId: null
             },
             {
                 iconPath: null,
                 headerText: contextMenuLabel["aboveText"],
                 eventHandler: null,
                 menuId: "Above",
                 parentMenuId: "Add"
             }, {
                 iconPath: null,
                 headerText: contextMenuLabel["belowText"],
                 eventHandler: null,
                 menuId: "Below",
                 parentMenuId: "Add"
             }
            ];
            return contextMenuItems;
        },
         //populate the header context menu with items
        _getHeaderContextMenuItems: function (e) {
            var enableFreeze,
                enableFreezeBefore,
                headerContextMenuItems = [],
                allowSorting,
                columnChooserListIndex,
                proxy = this,
                model = proxy.model,
                $target = $(e.target),
                contextMenuLabel = proxy._columnMenuTexts,
                columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname"),
                column = proxy.getColumnByField(columnFieldName);
            if (column == null) {
                return false
            }
             allowSorting = column.allowSorting == undefined ? true : column.allowSorting;
             if (model.allowSorting && allowSorting) {
                      headerContextMenuItems.push({
                          iconClass: "e-columnmenu-ascending",
                          headerText: contextMenuLabel["sortAscendingText"],
                          eventHandler: null,
                          isDefault: false,
                          disable: false,
                          menuId: "contextMenu_SortAscending",
                          parentMenuId: null,

                      }),
                       headerContextMenuItems.push({
                           iconClass: "e-columnmenu-descending",
                           headerText: contextMenuLabel["sortDescendingText"],
                           eventHandler: null,
                           isDefault: false,
                           disable: false,
                           menuId: "contextMenu_SortDescending",
                           parentMenuId: null
                       })

                  }
                  if (model.showColumnChooser || model.contextMenuSettings.showContextMenu) {
                      columnChooserListIndex = headerContextMenuItems.length;
                      headerContextMenuItems.push({
                          iconClass: "e-columnchooser-icon",
                          headerText: contextMenuLabel["columnsText"],
                          eventHandler: null,
                          isDefault: false,
                          disable: false,
                          menuId: "contextMenu_Columns",
                          parentMenuId: null,
                          isExpanderIcon: true
                      })
                  }
                  if (!model.isFromGantt) {
                      if (column.allowFreezing || (proxy._allowFreezingDefault && ej.isNullOrUndefined(column.allowFreezing))) {
                          /*Check all other columns are in frozen*/
                          if (proxy._frozenColumnsLength == model.columns.length - 1 && !column.isFrozen)
                              enableFreeze = true;
                          /*Check frozen column width and container width */
                          var frozenWidth = this._getFrozenColumnWidth(),
                              columnIndex = model.columns.indexOf(column);
                          frozenWidth = frozenWidth + this.columnsWidthCollection[columnIndex] + 18;
                          if (frozenWidth > this._gridWidth)
                              enableFreeze = true;
                          headerContextMenuItems.push({
                              iconClass: "e-freezecolumn-icon",
                              headerText: contextMenuLabel["freezeText"],
                              eventHandler: null,
                              isDefault: false,
                              disable: !(enableFreeze ? false : !column.isFrozen),
                              menuId: "contextMenu_Freeze",
                              parentMenuId: null,

                          }),
                          headerContextMenuItems.push({
                              iconClass: "e-unfreezecolumn-icon",
                              headerText: contextMenuLabel["unfreezeText"],
                              eventHandler: null,
                              isDefault: false,
                              disable: !(column.isFrozen == true),
                              menuId: "contextMenu_Unfreeze",
                              parentMenuId: null,

                          });
                          if (proxy._checkboxSelection && model.columns[0].nonColumn) {
                              if (model.columns.indexOf(column) != 1 && model.columns.indexOf(column) > this._frozenColumnsLength)
                                  enableFreezeBefore = true;
                          }
                          else if (model.columns.indexOf(column) != 0 && model.columns.indexOf(column) > this._frozenColumnsLength)
                              enableFreezeBefore = true;
                          /*Check frozen column width and container width */
                          frozenWidth = this._getFrozenColumnWidth(columnIndex);
                          frozenWidth = frozenWidth + 18;
                          if (frozenWidth > this._gridWidth)
                              enableFreezeBefore = false;
                          headerContextMenuItems.push({
                              iconClass: "e-freezecolumnbefore-icon",
                              headerText: contextMenuLabel["freezePrecedingColumnsText"],
                              eventHandler: null,
                              menuId: "contextMenu_FreezePreceding",
                              isDefault: false,
                              disable: !enableFreezeBefore,
                              parentMenuId: null,

                          })
                      }
                  }
                  return headerContextMenuItems
        },

        //populate sub context menu items
        _getsubContextMenuItems: function () {
            var contextMenuItems = [],
            proxy = this,
            contextMenuLabel = proxy._contextMenuTexts;
            contextMenuItems = [{
                iconPath: null,
                headerText: contextMenuLabel["aboveText"],
                eventHandler: null,
                menuId: "Above",
                parentMenuId: "Add"
            }, {
                iconPath: null,
                headerText: contextMenuLabel["belowText"],
                eventHandler: null,
                menuId: "Below",
                parentMenuId: "Add"
            }
            ];
            return contextMenuItems;
        },
        _clearContextMenu: function () {
            //remove context menu items and header context menu items
            var proxy = this;
            $('.e-tgcontextmenu').remove();
            $('.e-tginnerContextmenu').remove();
            $('.e-tgheaderContextmenu').remove();
            $("#" + proxy._id + "ccDiv").ejDialog("close");
        },
        // create and append a context menu and also bind the events for context menu option
        _renderContextMenu: function (e, index, item, checkEditMenu) {

            var proxy = this,
            model = proxy.model,
                eventArgs = {},
                args = {},
                posx,
				subposX,
                posy,
                subposY,
                contextMenu,
                contextMenuUList,
                subContextMenu,
                subContextMenuUList,
                coord = proxy._getCoordinate(e);
            proxy._contextMenuSelectedIndex = index;
            proxy._contextMenuSelectedItem = item;
            proxy._contextMenuEvent = e;


            if (!checkEditMenu && !this.editFormValidate())
                return;
            if (!e) e = window.event;
            subposX = posx = coord.pageX;
            subposY = posy = coord.pageY;
            
            //Edge detection for context menu
            var treeGridOffset = this.getOffsetRect(proxy.element[0]);
            treeGridOffset.bottom = treeGridOffset.top + this.element[0].offsetHeight;
            treeGridOffset.right = treeGridOffset.left + $(this.element).width();
           
            args.targetElement = e;
            args.item = item;
            args.index = index;
            args.columnIndex = proxy.getCellIndex(e);
            args.requestType = "ContextMenuOpen";
            args.contextMenuItems = proxy._contextMenuItems;
            if (proxy._trigger("contextMenuOpen", args)) return false;
            proxy._activeMenuItemId = null;
            proxy._contextMenuItems = args.contextMenuItems;
            contextMenu = ej.buildTag("div.e-tgcontextmenu e-js", "", {
                'display': 'table',
                'position': 'absolute',
                'z-index': proxy.getMaxZIndex() + 1,
            }, { "id": proxy._id + "_ContextMenu" });
            model.cssClass && (contextMenu.addClass(model.cssClass));
            //Get zero th level menu items
            var parentMenuItems = args.contextMenuItems.filter(function (value) {
                if (value.parentMenuId == null || value.parentMenuId == undefined)
                    return true;
            });

            this._isContextMenuItems = true;
           
            contextMenuUList = ej.buildTag("ul.e-treegrid-contextmenu", $.render[proxy._id + "contextMenuTemplate"](parentMenuItems), {
                'margin': '0px',
                'padding-left': '0px',
                'list-style-type': 'none'
            }, {
                'data-icon': false,
                'data-role': 'list-divider'
            });
            
            
            contextMenu.append(contextMenuUList);
            $(document.body).append(contextMenu);

            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && model.isResponsive) 
                proxy._off($(window), "resize", proxy.windowResize);
            var contextMenuHeight = $(contextMenu).outerHeight(),
                contextMenuWidth = $(contextMenu).outerWidth(),
                contextMenuItems = contextMenu.find(".e-menuitem"),
                contextMenuSelectItem = contextMenuItems.eq(0);
            contextMenuItems.outerWidth(contextMenuWidth + 2);
            if (treeGridOffset.left > posx || (treeGridOffset.right < (posx + contextMenuWidth))) {
                if (treeGridOffset.right < (posx + contextMenuWidth)) {
                    posx = posx - contextMenuWidth;
                }
                if (posx < 0) {
                    posx = treeGridOffset.left + 10;
                }
            }
            if (treeGridOffset.bottom < (posy + contextMenuHeight)) {
                var tempPosY = posy - (contextMenuHeight),
                    exceededHeight = (posy + contextMenuHeight) - treeGridOffset.bottom,
                    scrollTop = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop;
                if (tempPosY > treeGridOffset.top) {
                    posy = tempPosY;
                } else if ((posy + contextMenuHeight) > (scrollTop + window.innerHeight)) {
                    var extraHeight = (posy + contextMenuHeight) - (scrollTop + window.innerHeight),
                        cTop = posy - extraHeight;
                    if (cTop > 0)
                        posy = parseInt(cTop);
                }
            }         
            contextMenu.css({ 'left': posx + 'px', 'top': posy + 'px', });
            proxy._focusTreeGridElement();
            $(contextMenu).css({ 'height': 'auto' });
            proxy._on(contextMenu, "contextmenu", function (argsE) {
                argsE.preventDefault();
            });
            //click event handlers for contextmenu items and subcontextmenu items
            $(contextMenuItems).click(function () {
                proxy._contextMenuClickHandler(this);
            });

            //Mouseenter and mouse leave handler for contextmenu items and Subcontextmenuitems
            $(contextMenuItems).mouseenter(function () {
                if (!$(this).hasClass("e-disable")) {
                    contextMenu.find(".e-tgcontextmenu-mouseover").removeClass("e-tgcontextmenu-mouseover");
                    $(this).addClass("e-tgcontextmenu-mouseover");
                    proxy._activeMenuItemId = $(this).attr("id");
                    proxy._showSubContextMenu(this, args.contextMenuItems);
                }
            });

            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                proxy._clearMultiSelectPopup();
            }

            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && model.isResponsive) 
                proxy._on($(window), "resize", proxy.windowResize);
            e.preventDefault();
        },
        //Creating headerContextMenu
        _renderHeaderContextMenu: function (e) {
            var proxy = this,
                model = proxy.model,
                posx,
                subposX,
                subposY,
                posy,
                args = {},
                headerContextMenu,
                headerContextMenuUList,
                columnFieldName, columnListPosX,
                column,
                $target = $(e.target),
                headerContextMenuItems;

            columnFieldName = $target.closest(".e-headercell").find(".e-headercelldiv").attr("data-ej-mappingname"),
            column = proxy.getColumnByField(columnFieldName);
            proxy._contextMenuColumnDetails = column;
            if ((model.isEdit || this._isRowEdit)) {
                if (this.editFormValidate()) {
                    proxy._saveEditedElements();
                }
                else
                    return true;
            }
            var coord = proxy._getCoordinate(e);
            if (!e) e = window.event;
            posx = coord.pageX;
            posy = coord.pageY;
            var treeGridOffset = proxy.getOffsetRect(proxy.element[0]);
            treeGridOffset.bottom = treeGridOffset.top + proxy.element[0].offsetHeight;
            treeGridOffset.right = treeGridOffset.left + $(this.element).width();
            args.targetElement = e;
            args.headerContextMenuItems = proxy._headerContextMenuItems;
            args.columnIndex = proxy.getColumnIndexByField(columnFieldName);
            args.requestType = "ContextMenuOpen";
            if (proxy._trigger("contextMenuOpen", args)) return false;
            args.headerContextMenuItems = proxy._headerContextMenuItems;
            proxy._headerContextMenuItems = args.headerContextMenuItems;
            headerContextMenu = ej.buildTag("div.e-tgheaderContextmenu e-js", "", {
                'display': 'table',
                'position': 'absolute',
                'z-index': proxy.getMaxZIndex() + 1,
            }, { "id": proxy._id + "_headerContextMenu" });
            model.cssClass && (headerContextMenu.addClass(model.cssClass));
            var parentMenuItems = args.headerContextMenuItems.filter(function (value) {
                if (value.parentMenuId == null || value.parentMenuId == undefined)
                    return true;
            });
            this._isContextMenuItems = false;
            headerContextMenuUList = ej.buildTag("ul.e-treegrid-headercontextmenu", $.render[proxy._id + "contextMenuTemplate"](parentMenuItems), {
                'margin': '0px',
                'padding-left': '0px',
                'list-style-type': 'none'
            }, {
                'data-icon': false,
                'data-role': 'list-divider'
            });

            headerContextMenu.append(headerContextMenuUList);
            $(document.body).append(headerContextMenu);
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && model.isResponsive)
                proxy._off($(window), "resize", proxy.windowResize);
            var headerContextMenuHeight = $(headerContextMenu).outerHeight(),
                  headerContextMenuWidth = ($(".e-tgheaderContextmenu").outerWidth() + 0.5),
                  headerContextMenuItems = headerContextMenu.find(".e-menuitem"),
                  headerContextMenuSelectItem = headerContextMenuItems.eq(0);
            proxy._activeMenuItemId = $(headerContextMenuSelectItem).attr("id");
            headerContextMenuItems.outerWidth(headerContextMenuWidth);

            if (treeGridOffset.left > posx || (treeGridOffset.right < (posx + headerContextMenuWidth))) {
                if (treeGridOffset.right < (posx + headerContextMenuWidth)) {
                    posx = posx - headerContextMenuWidth;
                }
                if (posx < 0) {
                    posx = treeGridOffset.left + 10;
                }
            }
            if (treeGridOffset.bottom < posy + (headerContextMenuHeight)) {
                var tempPosY = posy - (headerContextMenuHeight);
                if (tempPosY > 0) {
                    posy = tempPosY;
                }
            }

            headerContextMenu.css({ 'left': posx + 'px', 'top': posy + 'px', });
            proxy._focusTreeGridElement();
            $(headerContextMenu).css({ 'height': 'auto' });
            proxy._on(headerContextMenu, "contextmenu", function (argsE) {
                argsE.preventDefault();
            });

            columnListPosX = posx + headerContextMenuWidth;
            proxy._columnListPosX = columnListPosX;
            proxy._headerContextMenuWidth = headerContextMenuWidth;
            proxy._posx = posx;
            proxy._posy = posy;
            proxy._columnMenuPopupY = posy;

            $("#contextMenu_SortAscending").on("click", function () {
                if (model.allowMultiSorting) proxy._multiSortRequest = true;
                if ($(this).find(".e-expander").length == 0) {
                    if (model.isFromGantt)
                        proxy.sortColumn(column.mappingName ? column.mappingName : column.field, ej.sortOrder.Ascending);
                    else
                        proxy.sortColumn(column.field, ej.sortOrder.Ascending);
                    proxy._multiSortRequest = false;
                }
            });

            $("#contextMenu_SortDescending").on("click", function () {
                if (model.allowMultiSorting) proxy._multiSortRequest = true;
                if ($(this).find(".e-expander").length == 0) {
                    if (model.isFromGantt)
                        proxy.sortColumn(column.mappingName ? column.mappingName : column.field, ej.sortOrder.Descending);
                    else
                        proxy.sortColumn(column.field, ej.sortOrder.Descending);
                    proxy._multiSortRequest = false;
                }
            });
            $("#contextMenu_Freeze").click(function () {
                if ($(this).find(".e-expander").length == 0) {
                    if (!$(this).hasClass("e-disable")) {
                        if (proxy._checkboxSelection) {
                            if (model.columns[0].nonColumn && !model.columns[0].isFrozen)
                                model.columns[0]['isFrozen'] = true;
                        }
                        proxy.freezeColumn(column.field, true);
                    }
                }
            });
            $("#contextMenu_Unfreeze").click(function () {
                if ($(this).find(".e-expander").length == 0) {
                    if (!$(this).hasClass("e-disable")) {
                        if (proxy._checkboxSelection && proxy._frozenColumnsLength == 2) {
                            if (model.columns[0].nonColumn && model.columns[0].isFrozen)
                                model.columns[0]['isFrozen'] = false;
                        }
                        proxy.freezeColumn(column.field, false);
                    }
                }
            });
            $("#contextMenu_FreezePreceding").click(function () {
                if ($(this).find(".e-expander").length == 0) {
                    if (!$(this).hasClass("e-disable"))

                        proxy.freezePrecedingColumns(column.field);
                }
            });
            if (!model.isFromGantt) {
                if (proxy.model.isResponsive && $(window).width() <= proxy._responsiveMinWidth)
                    $("#" + proxy._id + "_headerContextMenu").find(".e-expander").removeClass("e-treegridadaptivesubicon");
            }

            $("#contextMenu_Columns").mouseenter("click", function (e) {
                proxy._showColumnChooser(e, treeGridOffset.right, columnListPosX, headerContextMenuWidth, posy, posx, $("#contextMenu_Columns"), true);

            });
            headerContextMenuSelectItem.addClass("e-columnmenuselection");
            //Mouseenter and mouse leave handler for header contextmenu items and Subheadercontextmenuitems
            $(headerContextMenuItems).mouseenter(function () {
                if (!$(this).hasClass("e-disable")) {
                    headerContextMenuItems.removeClass("e-columnmenuselection");
                    $(this).addClass("e-columnmenuselection");
                    proxy._activeMenuItemId = $(this).attr("id");
                    proxy._showSubContextMenu(this, args.headerContextMenuItems);
                }
            });

            $(".e-tgheaderContextmenu .e-menuitem").on("mouseenter click", function (e) {
                var $target = $(e.target)
                var targetid = $target.closest(".e-menuitem").attr("id");
                if (targetid == "contextMenu_Columns") {
                    $(".e-tginnerContextmenu").remove();
                }
                else if (targetid == "contextMenu_SortAscending" || targetid == "contextMenu_SortDescending" || targetid == "contextMenu_Freeze" || targetid == "contextMenu_Unfreeze" || targetid == "contextMenu_FreezePreceding") {
                    $("#" + proxy._id + "ccDiv").ejDialog("close");
                }
                else {
                    $("#" + proxy._id + "ccDiv").ejDialog("close");

                }
            })
            //click event handlers for header contextmenu items and subheadercontextmenu items
            $(headerContextMenuItems).click(function () {
                proxy._contextMenuClickHandler(this);

            })
            if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                proxy._clearMultiSelectPopup();
            }
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && model.isResponsive)
                proxy._on($(window), "resize", proxy.windowResize);

        },

        //Show SubContext menu for selected Menu items
        _showSubContextMenu: function (element, menuItems) {
            var model = this.model,
                proxy = this,
                currentMenuItem = [],
                menuId = $(element).attr("id"),
                subContextMenu, subContextMenuUList, subMenuItems = [], subContextMenuItems;

            subMenuItems = menuItems.filter(function (value) {
                if (value.menuId == menuId)
                    currentMenuItem = value;
                if (value.parentMenuId == menuId)
                    return true;
            });

            if (!currentMenuItem.parentMenuId)
                $(".e-tginnerContextmenu").remove();

            if (currentMenuItem.parentMenuId) {
                this._removeContextMenu(currentMenuItem, menuItems);
            }

            if (subMenuItems.length > 0) {
                subContextMenu = ej.buildTag("div.e-tginnerContextmenu e-js", "", {
                    'display': 'table',
                    'position': 'absolute',
                    'z-index': proxy.getMaxZIndex() + 1,
                }, { "id": this._id + "_SubContextMenu" + menuId });
                model.cssClass && (subContextMenu.addClass(model.cssClass));
                subContextMenuUList = ej.buildTag("ul", $.render[proxy._id + "contextMenuTemplate"](subMenuItems), {
                    'margin': '0px',
                    'padding-left': '0px'
                }, {
                    'data-icon': false,
                    'type': 'none',
                    'data-role': 'list-divider'
                });
                subContextMenu.append(subContextMenuUList);
                $(document.body).append(subContextMenu);
                subContextMenuItems = subContextMenu.find(".e-menuitem");
                var menuWidth = $(subContextMenu).outerWidth();
                subContextMenuItems.outerWidth(menuWidth > 100 ? menuWidth : 100);
                var subMenuOffset = proxy._getSubContextMenuPosition(element, subContextMenu);
                $(subContextMenu).css({ "top": subMenuOffset.top, "left": subMenuOffset.left });
                $(subContextMenuItems).mouseenter(function () {
                    if (!$(this).hasClass("e-disable")) {
                        proxy._showSubContextMenu(this, menuItems);
                        $(this).closest(".e-tginnerContextmenu").find(".e-tgcontextmenu-mouseover").removeClass("e-tgcontextmenu-mouseover");
                        $(this).addClass("e-tgcontextmenu-mouseover");
                        proxy._activeMenuItemId = $(this).attr("id");
                    }
                });
                proxy._on(subContextMenu, "contextmenu", function (argsE) {
                    argsE.preventDefault();
                });
                $(subContextMenuItems).click(function () {
                    proxy._contextMenuClickHandler(this);
                });
                if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && model.isResponsive)
                    proxy._on($(window), "resize", proxy.windowResize);
                
            }
        },
        _removeContextMenu: function (menuItem, menuItems) {

            var currentMeuItemContainer = $("#" + this._id + "_SubContextMenu" + menuItem.parentMenuId),
                expandedMenu = $(currentMeuItemContainer).find(".e-tgcontextmenu-mouseover");
            if ($(expandedMenu).length > 0 ) {
                var innerMenu = menuItems.filter(function (value) {
                    if (value.menuId == $(expandedMenu).attr("id"))
                        return true;
                });
                if (innerMenu.length > 0) {
                    var nextLevelItem = menuItems.filter(function (value) {
                        if (value.parentMenuId == innerMenu[0].menuId)
                            return true;
                    });
                    if (nextLevelItem.length > 0)
                        this._removeContextMenu(nextLevelItem[0], menuItems);
                }
                $("#" + this._id + "_SubContextMenu" + $(expandedMenu).attr("id")).remove();
            }
        },
        _getSubContextMenuPosition: function (element, subMenu) {
            var proxy = this, model = this.model, parentElement,
                subContextMenuWidth,
                subContextMenuHeight, contextMenuWidth,posx,posy,
                elementOffset, subMenuOffset = { top: "", left: "" }
            parentElement = this._isContextMenuItems ? $(element).closest(".e-tgcontextmenu") : $(element).closest(".e-tgheaderContextmenu")
            if (parentElement.length == 0)
                parentElement = $(element).closest(".e-tginnerContextmenu");

            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && model.isResponsive)
                proxy._off($(window), "resize", proxy.windowResize);

            subContextMenuWidth = $(subMenu).outerWidth(),
            subContextMenuHeight = $(subMenu).height(),
            elementOffset = this.getOffsetRect(element),

            contextMenuWidth = $(parentElement).outerWidth();
            subMenuOffset.top = elementOffset.top - 1;
            subMenuOffset.bottom = elementOffset.top + $(element).outerHeight();
            subMenuOffset.left = elementOffset.left + $(parentElement).width() + 1;
            posx = subMenuOffset.left;

            //Edge detection for context menu
            var treeGridOffset = this.getOffsetRect(this.element[0]);
            treeGridOffset.bottom = treeGridOffset.top + this.element[0].offsetHeight;
            treeGridOffset.right = treeGridOffset.left + $(this.element).width();
            if (treeGridOffset.left > posx || (treeGridOffset.right < (posx + subContextMenuWidth))) {
                if (treeGridOffset.right < (posx + subContextMenuWidth)) {
                    posx = posx - contextMenuWidth - subContextMenuWidth;
                }
                if (posx > 0) {
                    subMenuOffset.left = posx;
                }
            }
            if (treeGridOffset.bottom < subMenuOffset.top + subContextMenuHeight) {
                var tempPosY = subMenuOffset.bottom - subContextMenuHeight,
                    exceededHeight = (subMenuOffset.top + subContextMenuHeight) - treeGridOffset.bottom,
                    scrollTop = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop;
                if (tempPosY > treeGridOffset.top) {
                    subMenuOffset.top = tempPosY;
                } else if ((subMenuOffset.top + subContextMenuHeight) > (scrollTop + window.innerHeight)) {
                    var extraHeight = (subMenuOffset.top + subContextMenuHeight) - (scrollTop + window.innerHeight),
                        cTop = subMenuOffset.top - extraHeight;
                    if (cTop > 0)
                        subMenuOffset.top = parseInt(cTop) - 2; //border
                }
            }
            return subMenuOffset;
        },
        _contextMenuClickHandler: function (target)
        {
            var choice = $(target).attr('id'),
                proxy = this;
            if (!$(target).hasClass("e-disable") && $(target).find(".e-expander").length == 0) {
                switch (choice) {
                    case "Delete":
                        proxy.contextMenuOperations("Delete");
                        break;
                    case "Above":
                        proxy.contextMenuOperations("Above");
                        break;
                    case "Below":
                        proxy.contextMenuOperations("Below");
                        break;
                    case "Add":
                        proxy.contextMenuOperations("Add");
                        break;
                    case "Edit":
                        proxy.contextMenuOperations("Edit");
                        break;
                    case "Save":
                        proxy.contextMenuOperations("Save");
                        break;
                    case "Cancel":
                        proxy.contextMenuOperations("Cancel");
                        break;
                    default:
                        proxy.contextMenuOperations(choice);
                        break;
                }
            }
        },


        //Call custom context menu event handlers and custom header context menu event handlers.
        _triggerMenuEventHandler: function (menuId, eventArgs,isFromContextMenu) {
            var proxy = this,
                currentMenuItemCollection, menuItem,
                contextMenuItems = isFromContextMenu ? proxy._contextMenuItems : proxy._headerContextMenuItems;
            if (!ej.isNullOrUndefined(contextMenuItems)) {
                currentMenuItemCollection = contextMenuItems.filter(function (value) {
                    //convert menu id to string if it is given as number
                    if (value.menuId.toString() === menuId)
                        return true;
                });

                if (currentMenuItemCollection.length > 0) {
                    menuItem = currentMenuItemCollection[0];
                    var fn = menuItem.eventHandler;
                    if (fn) {
                        if (typeof fn === "string") {
                            fn = ej.util.getObject(fn, window);
                        }
                        if ($.isFunction(fn)) {
                            var args = ej.event("customContextMenuHandler", this.model, eventArgs);
                            fn.call(this, args);
                        }
                    }

                }
            }
        },

        _createContextMenuTemplate: function () {
            var proxy = this;
            var helpers = {};
            helpers["_" + proxy._id + "getHeaderName"] = $.proxy(proxy._getHeaderName, proxy);

            $.views.helpers(helpers);

            var menuItemList = "<li style='list-style-type:none;margin:0px;'>";
            var listChild = "<div class='e-menuitem {{if disable}}e-disable{{/if}}{{if ~_" + proxy._id + "getHeaderName(#data)}} e-parent-menuitem{{/if}}' id={{:menuId}}  style='display:table;cursor:{{if disable}}default{{else}}pointer{{/if}};min-width:100px;'>" +
                            "{{if iconPath}}" +
                           "<div class='e-icon e-tgcontextmenu-image'" +
                            " style='background-image:{{:iconPath}};background-repeat:no-repeat;'/> " + 
                            "{{else}}" +
                            "<div class='e-icon {{:iconClass}} e-tgcontextmenu-icon'/>" +
                            "{{/if}}" +
                            "<div class='e-tgcontextmenu-label'>" +
                            "<span>{{:headerText}}</span></div>" +
                            "{{if ~_" + proxy._id + "getHeaderName(#data)}}<div class='e-icon e-expander'/> {{/if}}";

            menuItemList += listChild;
            menuItemList += "</div></li>";
            var templates = {};
            templates[proxy._id + "contextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },
        _getHeaderName: function (data) {
            var templatecontextmenuitem = this._isContextMenuItems ? this._contextMenuItems : this._headerContextMenuItems,
                childMenuItems = templatecontextmenuitem.filter(function (value) {
                if (data.menuId != null && data.menuId == value.parentMenuId || data.isExpanderIcon)
                    return true;
            });
            if (childMenuItems.length > 0) {
                return true;
            }
            return false;
        },

        // Remove the speace from column menu name for chaning name to id.

        _getColumnId: function (data) {
            return data.name.replace(/\s/g, "");
        },

        //Change the context menu based on context menu item list.
        _updateContextmenuOption: function (recordIndex) {
            //removing empty string and undefined from contextmenu items.
            Array.prototype.clean = function (deleteValue) {
                for (var i = 0; i < this.length; i++) {
                    if (this[i] == deleteValue) {
                        this.splice(i, 1);
                        i--;
                    }
                }
                return this;
            };
            var proxy = this;
            
            if(proxy._isRowEdit || proxy.model.isEdit)
            {
                //Remove add, delete and edit context menu when row in edit mode.
                proxy._contextMenuItems.splice(0, 3);
            }
            else
            {
                proxy._contextMenuItems.splice(3, 2); //Remove save and cancel when row not in edit mode
                var editSetting = proxy.model.editSettings;
                var contextMenuItems = $.extend(true, [], proxy._contextMenuItems);
                var contextMenuList = proxy.model.contextMenuSettings.contextMenuItems.clean("").clean(undefined);
                var contextMenuItemList = [];
                //removing duplicate conetxt menu items
                $.each(contextMenuList, function (i, el) {
                    if ($.inArray(el.toLowerCase(), contextMenuItemList) === -1) contextMenuItemList.push(el.toLowerCase());
                });
                proxy._contextMenuItems = [];
                if (recordIndex == -1) {
                    if (contextMenuItemList.indexOf("add") != -1 || contextMenuItemList.length==0)
                        proxy._contextMenuItems[0] = contextMenuItems[0];
                    if (!editSetting.allowAdding)
                        proxy._contextMenuItems[0].disable = true;
                    proxy._contextAddMenuIndex = 0;
                }
                //Adding default context menu when user not specify the context menu items.
                else if (contextMenuItemList.length == 0)
                {
                    proxy._contextMenuItems[0] = contextMenuItems[0];
                    if (!editSetting.allowAdding)
                        proxy._contextMenuItems[0].disable = true;
                    proxy._contextAddMenuIndex = 0;
                    proxy._contextMenuItems[1] = contextMenuItems[2];
                    if (!editSetting.allowDeleting)
                        proxy._contextMenuItems[1].disable = true;
                    proxy._contextMenuItems.push.apply(proxy._contextMenuItems, proxy._subContextMenuItems);
                }
                //Change the context menu order based on context menu items order
                else {
                for(var i=0;i<contextMenuItemList.length;i++)
                {
                    var item=contextMenuItemList[i];
                    switch(item.toLowerCase())
                    {
                        case "add":
                            proxy._contextMenuItems[i] = contextMenuItems[0];
                            proxy._contextAddMenuIndex = i;
                            if (!editSetting.allowAdding)
                                proxy._contextMenuItems[i].disable = true;
                            break;
                        case "delete":
                            proxy._contextMenuItems[i] = contextMenuItems[2];
                            if (!editSetting.allowDeleting)
                                proxy._contextMenuItems[i].disable = true;
                            break;
                        case "edit":
                            proxy._contextMenuItems[i] = contextMenuItems[1];
                            if (!editSetting.allowEditing)
                                proxy._contextMenuItems[i].disable = true;
                            break;
                    }
                }
                proxy._contextMenuItems.push.apply(proxy._contextMenuItems, proxy._subContextMenuItems);
                }
            }
        },
        //Assign default value to columns based on column edit type
        _setInitialData:function()
        {
            var eventArgs = {}, proxy = this,
               model = proxy.model, data = [], item, initialData = {};
            for (var i = 0; i < model.columns.length; i++)
            {
                var column = model.columns[i];
                switch(column.editType)
                {
                    case "booleanedit":
                        initialData[column.field] = false;
                        break;
                    case "stringedit":
                        initialData[column.field] = null;
                        break;
                    case "datepicker":
                        initialData[column.field] = ej.format(new Date(), this.model.dateFormat, this.model.locale);
                        break;
                    case "datetimepicker":
                        initialData[column.field] = ej.format(new Date(), "MM/dd/yyyy hh:mm tt", this.model.locale);
                        break;
                    case "numericedit":
                        initialData[column.field] = 0;
                        break;
                    default:
                        initialData[column.field] = null;

            }
            }
            data.push(initialData);
            if (proxy.dataSource() == null || proxy.dataSource().length == 0 || model.flatRecords.length == 0) {
                
                item = proxy._createRecord(data[0], 0, null);
                item.index = 0;
                proxy._addRecord(item, ej.TreeGrid.RowPosition.Top);
                proxy._clearContextMenu();
                this._removeDetailsRow();
            }
        },
        
        // Perform add new row operation
           
        _subContextMenuAction: function (item, index, position, addrow)
        {
            var proxy = this, model = proxy.model,
            eventArgs = {},
            newItem, selectionIndex;

            eventArgs.requestType = "save";
            newItem = jQuery.extend(true, {}, item.item);
            newItem[proxy.model.childMapping] = null;
            if (model.idMapping && model.parentIdMapping && newItem[model.parentIdMapping])
                delete newItem[model.parentIdMapping];

            if (position == "Above") {
                proxy._addRecord(newItem, ej.TreeGrid.RowPosition.Above, null, "contextMenu");
            } else if (position == "Below") {
                proxy._addRecord(newItem, ej.TreeGrid.RowPosition.Below, null, "contextMenu");
            }
            proxy._clearContextMenu();
        },

        _appendEditTemplateRow: function (columns, $targetTr, index, isFrozenTr, addrow) {

            var model = this.model,
                proxy = this,
                rowHeight = proxy.model.rowHeight, $mainTd,
                $form, $frozenForm, $table, $detailCell, visibleColumns = [],
                updatedRecords = proxy.getUpdatedRecords();

            this._editedRowIndex = index;
            $table = ej.buildTag('table.e-table e-zerospace', "", {});
            if (this._frozenColumnsLength > 0) {
                if (isFrozenTr) {
                    $form = ej.buildTag("form.e-treegridform#" + proxy._id + "EditFrozenForm", "", { "margin": "-2px 0px" }, {});
                    $table.append(proxy._$gridHeaderContainer.find("#e-frozenheaderdiv" + proxy._id).find('colgroup').clone());
                } else {
                    $form = ej.buildTag("form.e-treegridform#" + proxy._id + "EditForm", "", { "margin": "-2px 0px" }, {});
                    $table.append(proxy._$gridHeaderContainer.find("#e-movableheaderdiv" + proxy._id).find('colgroup').clone());
                }
            } else {
                $form = ej.buildTag("form.e-treegridform#" + proxy._id + "EditForm", "", { "margin": "-2px 0px" }, {});
                $table.append(proxy.getHeaderTable().find('colgroup').clone());
            }


            if (!isFrozenTr && model.showDetailsRow && model.showDetailsRowInfoColumn && model.detailsTemplate) {
                $detailCell = $targetTr.find("td.e-detailsrowcell");
            }

            $targetTr.empty();
            if (addrow) {
                $targetTr.addClass('e-addedrow');
                if ($targetTr.next('tr').hasClass('e-detailsrow'))
                    $targetTr.next('tr').remove();
            }
            else
                $targetTr.addClass('e-rowedit');

            visibleColumns = columns.filter(function (value) {
                return !(value.visible === false);
            });
            $mainTd = ej.buildTag("td", "", { "padding": "1px" }, { "colspan": $detailCell ? visibleColumns.length + 1 : visibleColumns.length });
            if (model.allowSelection)
                var $tr = ej.buildTag('tr.e-treegridrows e-selectionbackground e-active', { "height": rowHeight }, {});
            else
                var $tr = ej.buildTag('tr.e-treegridrows', { "height": rowHeight }, {});
            var $tbody = ej.buildTag('tbody'),
            $cellEditTemplate = proxy._cellEditTemplate;
            var columnLength = columns.length;
            for (var column = 0; column < columnLength; column++) {
                if (!ej.isNullOrUndefined(column)) {
                    if (columns[column].visible) {
                        var data = {}, columnField = proxy._getFieldValue(columns[column].field, model.columns.indexOf(columns[column])),
                        //Create a html string for column field with specific edit type using cellEditTemplate
                        htmlString = $cellEditTemplate.find("#" + columnField + "_CellEdit").html();
                        var value = proxy._getCustomCellValue(updatedRecords[index], columns[column]["field"]);
                        data[columnField] = value;
                        if (columns[column]["editType"] == "dropdownedit" && data[columns[column]["field"]] == false)
                            data[columnField] = data[columnField].toString();
                        var $element = $($.templates(htmlString).render(data));
                        if (proxy._isCommandColumn(columns[column])) {
                            var $td = ej.buildTag('td.e-rowcell e-unboundcell e-unboundeditedcell', "", { "height": rowHeight, "vertical-align": "middle;", "overflow": "hidden" }, {});
                        }
                        else {
                            var $td = ej.buildTag('td.e-rowcell e-editedcell', "", { "height": rowHeight, "vertical-align": "middle;", "overflow": "hidden" }, {});
                            $td.addClass('e-validError');
                        }
                        if (!proxy._isCommandColumn(columns[column]) && columns[column].allowEditing == false)
                            $element.attr('disabled', true).addClass('e-disable');

                        if ($element.get(0).tagName == "SELECT") {
                            $element.val(data[columns[column]["field"]]);
                            $element.val() == null && $element.val($element.find("option").first().val());
                        }
                        if ($element.get(0).tagName == "INPUT") {
                            $element.val(data[columnField]);
                        }
                        $tr.append($td.append($element));
                        $tbody.append($tr);
                    }
                }
            }
            if ($detailCell) {
                $tr.append($detailCell);
            }
            $table.append($tbody);
            $form.append($table);
            $mainTd.append($form);
            $targetTr.append($mainTd);

        },
        //Create editable row with specific edit type.

        _editRow: function (index, addrow, columnIndex) {
            var proxy = this,
                model = proxy.model,
            rowHeight = model.rowHeight,
            $form, $frozenForm, $table,
            updatedRecords = proxy.getUpdatedRecords(),
            isAddRow = false;

            proxy._editedRowIndex = index;

            var $targetTr = ej.TreeGrid.getRowByIndex(proxy, index);
            if ($targetTr.hasClass("e-summaryrow")) {
                proxy._editAddTools();
                return;
            }
            if (ej.isNullOrUndefined(addrow)) {
                var args = {};
                args.data = updatedRecords[index];
                args.rowElement = $targetTr;
                args.rowIndex = index;
                args.columnIndex = columnIndex;
                isAddRow = false;
                if (proxy._trigger("beginEdit", args)) return false;
            }
            else
                isAddRow = true;
            if (this._frozenColumnsLength > 0) {
                proxy._appendEditTemplateRow(proxy._frozenColumns, $($targetTr[0]), index, true, addrow);
                proxy._appendEditTemplateRow(proxy._unFrozenColumns, $($targetTr[1]), index, false, addrow);
            } else {
                proxy._appendEditTemplateRow(model.columns, $($targetTr), index, false, addrow);
            }
            proxy._isRowEdit = true;
            proxy._editAddTools();
            var args = {};
            if (isAddRow)
            {
                args.data = {};
                args.requestType = "add";
            }
            else
            {
                args.columnIndex = columnIndex;
                args.data = updatedRecords[index];
                args.requestType = "beginEdit";
            }
           
           
            proxy._refreshEditForm(args);
            proxy._updateWrapRowHeights("total,header", null, index, "rowEdit");
            if ($.isFunction($.validator)) {
                proxy._initValidator();
                proxy.setValidation();
            }
            model.selectedItem = updatedRecords[index];
            for(var i=0;i<model.columns.length;i++){
                if (proxy._isCommandColumn(model.columns[i]) && model.columns[i]["isFrozen"]==true) {
                    this._refreshUnboundTemplate($targetTr.find("#" + this._id + "EditFrozenForm"))
                }
                else if (proxy._isCommandColumn(model.columns[i])) {
                    this._refreshUnboundTemplate($targetTr.find("#" + this._id + "EditForm"));
                }
            }            
        },

        //Creating column chooser menu

        _renderColumnChooser: function () {
            var proxy = this,
                $outerDiv = ej.buildTag("div.e-columnSelector", '',"", { id: this._id + "ccDiv" });
            $("#" + proxy._id + "ccDiv_wrapper").remove();
            proxy._renderColumnChooserList(false);
            $outerDiv.append(proxy._columnChooserList);
            $outerDiv.insertAfter(proxy.element);
            $outerDiv.data("columnMenuDialog", "TreeGrid");
            $outerDiv.ejDialog({ showOnInit: false, allowKeyboardNavigation: false, enableResize: false, "enableRTL": false, showHeader: false, width: "auto",allowDraggable: false,
                                 position: { X: 1, Y: 1 }, enableAnimation: false, minWidth:"0px", minHeight:"0px" });
            $("#" + proxy._id + "ccDiv_wrapper").removeClass("e-dialog").removeClass("e-shadow").find("div.e-dialog-scroller").css("height", "100%").removeClass("e-widget-content").addClass("e-treegrid-columnchooser");
        },

        //render filter menu dialog
        _renderFilterMenu: function (filterType) {
            var proxy = this, numeric = 0, srting = 0, datepicker = 0, DropDown = 0, datetimepicker = 0, boolean = 0, column = this.model.columns, dropdownValue;
            if (!filterType) {
                for (var x = 0; x < column.length; x++) {
                    if (column[x].filterEditType == "numericedit" || column[x].editType == "numericedit")
                        numeric++;
                    if (column[x].filterEditType == "stringedit" || column[x].editType == "stringedit")
                        srting++;
                    if (column[x].filterEditType == "datepicker" || column[x].editType == "datepicker")
                        datepicker++;
                    if (column[x].filterEditType == "datetimepicker" || column[x].editType == "datetimepicker")
                        datetimepicker++;
                    if (column[x].filterEditType == "booleanedit" || column[x].editType == "booleanedit")
                        boolean++;
                    if (column[x].filterEditType == "dropdownedit" || column[x].editType == "dropdownedit") {
                        DropDown++;
                        dropdownValue = column[x];
                    }
                }
            }
            if (srting > 0 || filterType == "stringedit") {
                proxy._renderEdittypeFilterMenu("String");
                proxy._renderedFilterMenuDialog.push("stringedit");
            }
            if (numeric > 0 || filterType == "numericedit") {
                proxy._renderEdittypeFilterMenu("Numeric");
                proxy._renderedFilterMenuDialog.push("numericedit");
            }
            if (datepicker > 0 || filterType == "datepicker") {
                proxy._renderEdittypeFilterMenu("Datepicker");
                proxy._renderedFilterMenuDialog.push("datepicker");
            }
            if (datetimepicker > 0 || filterType == "datetimepicker") {
                proxy._renderEdittypeFilterMenu("Datetimepicker");
                proxy._renderedFilterMenuDialog.push("datetimepicker");
            }
            if (DropDown > 0 || filterType == "dropdownedit") {
                proxy._renderEdittypeFilterMenu("Dropdown", dropdownValue);
                proxy._renderedFilterMenuDialog.push("dropdownedit");
            }
            if (boolean > 0 || filterType == "booleanedit") {
                proxy._renderEdittypeFilterMenu("Boolean");
                proxy._renderedFilterMenuDialog.push("booleanedit");
            }
        },

        //render filter menu dialog box depends on its filterEditType
        _renderEdittypeFilterMenu: function (idvalue, dropdownValue) {
            var proxy = this, filterTable,
                $outerDiv = ej.buildTag("div.e-" + proxy._id + "filter" + idvalue + "Selector e-filterMenuDiv e-treegridmenufilterwraper", "", {}, { id: this._id + "filter" + idvalue + "MenuDiv" }),
                localeFilterTexts = proxy._filterMenuTexts;
            if (idvalue == "Dropdown") {
                filterTable = "<div class='e-filter-container'><div class='e-dropdown-filter-container'><input id='" + this._id + "filter" + idvalue + "DropDown'/></div><div class='e-filter-button-container'><div class='e-filter-btn-filter'><button id='" + this._id + "filter" + idvalue + "MenuButton' class='e-filterMenuDiv'>" + localeFilterTexts.filterButton + "</button></div><div class='e-filter-btn-clear'><button id='" + this._id + "filter" + idvalue + "ClearButton' class='e-filterMenuDiv'>" + localeFilterTexts.clearButton + "</button></div></div></div>";
            } else if (idvalue == "Boolean") {
                filterTable = "<div class='e-filter-container'><div class='e-checkbox-filter-container'><span class='e-checkbox-filter-text'>" + localeFilterTexts.filterValue + "</span><div class='e-checkbox-filter-control' style='padding-bottom:14px;padding-left: 48%;'><input id='" + this._id + "filter" + idvalue + "InputBox'/></div><div class='e-filter-button-container'><div class='e-filter-btn-filter'><button id='" + this._id + "filter" + idvalue + "MenuButton' class='e-filterMenuDiv'>" + localeFilterTexts.filterButton + "</button></div><div class='e-filter-btn-clear'><button id='" + this._id + "filter" + idvalue + "ClearButton' class='e-filterMenuDiv'>" + localeFilterTexts.clearButton + "</button></div></div></div>";
            }
            else {
                filterTable = "<div class='e-filter-container'><div class='e-filter-operator'><input id='" + this._id + "filter" + idvalue + "DropDown'/></div><span class='e-filter-text'>" + localeFilterTexts.filterValue + "</span><div class='e-filter-value'><input id='" + this._id + "filter" + idvalue + "InputBox'/></div><div class='e-filter-button-container'><div class='e-filter-btn-filter'><button id='" + this._id + "filter" + idvalue + "MenuButton' class='e-filterMenuDiv'>" + localeFilterTexts.filterButton + "</button></div><div class='e-filter-btn-clear'><button id='" + this._id + "filter" + idvalue + "ClearButton' class='e-filterMenuDiv'>" + localeFilterTexts.clearButton + "</button></div></div></div>";
            }
            $outerDiv.append(filterTable);
            $outerDiv.insertAfter(proxy.element);
            proxy._renderFilterMenuDialog(idvalue + "MenuDiv", idvalue, dropdownValue, localeFilterTexts);
            $outerDiv.data("filter" + idvalue + "MenuDialog", "TreeGrid");
            $outerDiv.ejDialog({
                showOnInit: false, allowKeyboardNavigation: false, enableResize: false, "enableRTL": false, showHeader: false, width: "auto", allowDraggable : false,
                position: { X: 1, Y: 1 }, enableAnimation: false, cssClass: proxy.model.cssClass
            });
            $("#" + proxy._id + "filter" + idvalue + "MenuDiv_wrapper").removeClass("e-dialog").removeClass("e-shadow").find("div.e-dialog-scroller").css({ 'height': 'auto','width':'auto' }).removeClass("e-widget-content");
            $("#" + proxy._id + "filter" + idvalue + "MenuDiv_wrapper").css("outline", "none").addClass("e-treegrid-columnchooser");
        },

        //render filter menu inner components for each dialog box depends on its filterEditType
        _renderFilterMenuDialog: function (idValue, inputIdValue, dropdownValue, localeFilterTexts) {

            var proxy = this, localization, groups, initialtextvalue, field1;
            if (inputIdValue == "String") {
                groups = localeFilterTexts.stringMenuOptions;
                initialtextvalue = "endswith";
                field1 = { value: "value", text: "text" };
            }
            if (inputIdValue == "Numeric" || inputIdValue == "Datepicker" || inputIdValue == "Datetimepicker") {
                groups = localeFilterTexts.numberMenuOptions;
                initialtextvalue = "GreaterThan";
                field1 = { value: "value", text: "text" };
            }
            if (inputIdValue == "Dropdown") {
                groups = dropdownValue.dropdownData;
                if (dropdownValue.editParams)
                    field1 = { value: dropdownValue.editParams.fields.value, text: dropdownValue.editParams.fields.text };
                else
                    field1 = { value: "value", text: "text" };
            }
            if (inputIdValue == "String") {
                $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'InputBox').ejAutocomplete({
                    width: "100%",
                    watermarkText: localeFilterTexts.enterValueText,
                    filterType: initialtextvalue,
                    enableResize:false,
                    enableDistinct: true,
                    locale: proxy.model.locale,
                    cssClass: proxy.model.cssClass
                });
            }
            if (inputIdValue == "Numeric") {
                $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'InputBox').ejNumericTextbox({
                    decimalPlaces: 2,
                    width: "100%",
                    locale: proxy.model.locale,
                });
            }
            if (inputIdValue == "Boolean") {
                $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'InputBox').ejCheckBox({
                    size: "small"
                });
            }
            if (inputIdValue == "Datetimepicker") {
                $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'InputBox').ejDateTimePicker({
                    width: "100%",
                    locale: proxy.model.locale,
                    cssClass: proxy.model.cssClass
                });
            }
            if (inputIdValue == "Datepicker") {
                $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'InputBox').ejDatePicker({
                    width: "100%",
                    locale: proxy.model.locale,
                    cssClass: proxy.model.cssClass
                });
            }
            if (inputIdValue != "Boolean") {
                $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'DropDown').ejDropDownList({
                    dataSource: groups,
                    selectedIndex: 1,
                    fields: field1,
                    select: function (args) {
                        if (inputIdValue == "String") {
                            var suggesionBox = $("#" + proxy._id + "filter" + inputIdValue + "InputBox").data("ejAutocomplete");
                            suggesionBox.option("filterType", args.value);
                        }
                    },
                    cssClass: proxy.model.cssClass,
                    width: "100%"
                });
            }

            $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'MenuButton').ejButton({
                size: "normal",
                width: "100%",
                cssClass: proxy.model.cssClass,
                click: function (args) {
                    var columnFieldName = $.trim(proxy._columnMenuTarget.prev("div.e-headercelldiv").attr("data-ej-mappingname")),
                    column = proxy.getColumnByField(columnFieldName), suggesionBox, dropDown;
                    if (args.e.target.id == proxy._id + "filterStringMenuButton") {
                        suggesionBox = $("#" + proxy._id + "filterStringInputBox").data("ejAutocomplete");
                        dropDown = $("#" + proxy._id + "filterStringMenuDiv").find("#" + proxy._id + "filterStringDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, suggesionBox.value(), dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterNumericMenuButton") {
                        suggesionBox = $("#" + proxy._id + "filterNumericInputBox").data("ejNumericTextbox");
                        dropDown = $("#" + proxy._id + "filterNumericMenuDiv").find("#" + proxy._id + "filterNumericDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, suggesionBox.model.value, dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterDropdownMenuButton") {
                        dropDown = $("#" + proxy._id + "filterDropdownMenuDiv").find("#" + proxy._id + "filterDropdownDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, dropDown.model.value, "equal");
                    }
                    if (args.e.target.id == proxy._id + "filterDatepickerMenuButton") {
                        suggesionBox = $("#" + proxy._id + "filterDatepickerInputBox").data("ejDatePicker");
                        dropDown = $("#" + proxy._id + "filterDatepickerMenuDiv").find("#" + proxy._id + "filterDatepickerDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, suggesionBox.model.value, dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterDatetimepickerMenuButton") {
                        suggesionBox = $("#" + proxy._id + "filterDatetimepickerInputBox").data("ejDateTimePicker");
                        dropDown = $("#" + proxy._id + "filterDatetimepickerMenuDiv").find("#" + proxy._id + "filterDatetimepickerDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, suggesionBox.model.value, dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterBooleanMenuButton") {
                        suggesionBox = $("#" + proxy._id + "filterBooleanInputBox").data("ejCheckBox");
                        if (suggesionBox.model.checkState == "uncheck")
                            proxy._performMenuFilter(column, false, "equal");
                        else if (suggesionBox.model.checkState == "check")
                            proxy._performMenuFilter(column, true, "equal");
                    }
                }
            });

            $("#" + proxy._id + "filter" + idValue).find('#' + proxy._id + "filter" + inputIdValue + 'ClearButton').ejButton({
                size: "normal",
                width: "100%",
                cssClass: proxy.model.cssClass,
                click: function (args) {
                    var columnFieldName = $.trim(proxy._columnMenuTarget.prev("div.e-headercelldiv").attr("data-ej-mappingname")),
                    column = proxy.getColumnByField(columnFieldName), dropDown;
                    if (proxy._isExcelFilter)
                        delete proxy._excelFilter._predicates[0][column.field];
                    if (args.e.target.id == proxy._id + "filterStringClearButton") {
                        dropDown = $("#" + proxy._id + "filterStringMenuDiv").find("#" + proxy._id + "filterStringDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, "", dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterNumericClearButton") {
                        dropDown = $("#" + proxy._id + "filterNumericMenuDiv").find("#" + proxy._id + "filterNumericDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, "", dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterDropdownClearButton") {
                        dropDown = $("#" + proxy._id + "filterDropdownMenuDiv").find("#" + proxy._id + "filterDropdownDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, "", "equal");
                    }
                    if (args.e.target.id == proxy._id + "filterDatepickerClearButton") {
                        dropDown = $("#" + proxy._id + "filterDatepickerMenuDiv").find("#" + proxy._id + "filterDatepickerDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, "", dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterDatetimepickerClearButton") {
                        dropDown = $("#" + proxy._id + "filterDatetimepickerMenuDiv").find("#" + proxy._id + "filterDatetimepickerDropDown").data("ejDropDownList");
                        proxy._performMenuFilter(column, "", dropDown.model.value);
                    }
                    if (args.e.target.id == proxy._id + "filterBooleanClearButton") {
                        proxy._performMenuFilter(column, "", "equal");
                    }
                }
            });
        },

        //Render excel type filter menu
        _renderExcelFilter:function()
        {
            var proxy = this,
                model = proxy.model,
                filterSettings = model.filterSettings,
                filterCol = filterSettings.filteredColumns.length != 0 ? filterSettings.filteredColumns[0].field : null,
                filterModel = {
                instance: this,
                showSortOptions: model.allowSorting ? (model.showColumnChooser ? false:true):false,
                allowCaseSensitive: filterSettings.enableCaseSensitivity,
                maxFilterLimit: filterSettings.maxFilterChoices,
                interDeterminateState:true,
                enableComplexBlankFilter: filterSettings.enableComplexBlankFilter,
                blankValue: filterSettings.blankValue,
                filteredColumns:filterSettings.filteredColumns,
                filterHandler: $.proxy(proxy._filterHandler, proxy),
                cancelHandler: $.proxy(proxy._cancelHandler, proxy),
                initFilterCol: filterCol,
                actionBegin: "actionBegin",
                actionComplete: "actionComplete"
            };
            proxy._excelFilter = new ej.excelFilter(filterModel);
          //  $.extend(proxy._excelFilter, model.filterSettings);
            proxy._isExcelFilter = true;
        },

        //Cancel client side event for excel filter
        _cancelHandler:function(args)
        {
            var proxy = this;
            $("#" + proxy._id).find(".e-filtericon").data("isClicked", false);
            proxy._mediaStatus = false;
            if (proxy.model.showColumnChooser && !proxy._isCloseFilterMenu)
                proxy._clearColumnMenu("cancelHandler");
            proxy._isCloseFilterMenu = false;
        },

        //filter client side event for excel fitler
        _filterHandler:function(args)
        {
            var proxy = this,
                model = proxy.model,
                arg = {}, queryManager;
            arg.requestType = args.action == "sorting" ? args.action : "filtering";
            var filterColumn = model.filterSettings.filteredColumns;
            switch(args.action)
            {
                case "sorting":
                    var column = proxy.getColumnByField(args.sortDetails.field);
                    if (column.allowSorting != false) {
                        if (model.allowMultiSorting) proxy._multiSortRequest = true;
                        proxy.sortColumn(args.sortDetails.field, args.sortDetails.direction);
                        proxy._multiSortRequest = false;
                    }
                    break;
                case "filtering":
                    proxy.clearAllSelection();
                    queryManager = ej.DataManager(model.filterSettings.filteredColumns);
                    var query = new ej.Query().where("field", ej.FilterOperators.equal, args.fieldName),
                        object = queryManager.executeLocal(query);
                    for (var i = 0; i < object.length; i++) {
                        var objectIndex = $.inArray(object[i], model.filterSettings.filteredColumns)
                        if (objectIndex != -1)
                            model.filterSettings.filteredColumns.splice(objectIndex, 1);
                    }
                    ej.merge(model.filterSettings.filteredColumns, args.filterCollection);
                    args.currentFilterCollection = args.filterCollection;
                    arg.currentFilteringColumn = args.fieldName;
                    arg.predicated = args.ejpredicate;
                    var returnValue = proxy.processBindings(arg);
                    if (returnValue)
                        model.filterSettings.filteredColumns = filterColumn;
                    if(proxy._filterIconTarget)
                        proxy._filterIconTarget.target.className = "e-column-filtered-icon e-icon e-filtericon";
                    proxy._filteredColumnValueID.push(args.fieldName);
                    break;

                case "clearfiltering":
                    proxy.clearAllSelection();
                    var filterObj = args.filterDetails;
                    delete proxy._excelFilter._predicates[0][args.fieldName];
                    proxy.filterColumn(filterObj.field, filterObj.operator, filterObj.value, filterObj.predicate);
                    proxy._storePreviousFilteredValue(filterObj.field, filterObj.value, filterObj.operator);
                    if (proxy._filterIconTarget)
                        proxy._filterIconTarget.target.className = "e-column-filter-icon e-icon e-filtericon";
                    if (proxy._filteredColumnValueID.indexOf(args.fieldName) != -1) {
                        var filteredColumnIdIndex = proxy._filteredColumnValueID.indexOf(args.fieldName);
                        proxy._filteredColumnValueID.splice(filteredColumnIdIndex, 1);
                    }
                    break;
            }
            $("#" + proxy._id).find(".e-filtericon").data("isClicked", false);
            proxy._mediaStatus = false;
            if (model.showColumnChooser)
                proxy._clearColumnMenu();
        },
        /*  Creating Column Chooser menu list with column header name
		    argument refresh in boolean data type
		    If it is false, rendering the column chooser list
            If is is true, rerendering the column chooser list
        */

        _renderColumnChooserList: function (refresh, headerText) {
            var proxy = this,
                length = proxy.model.columns.length,
                columns = proxy.model.columns,
                $listBox = ej.buildTag("div", '', { 'margin-left': '0px', "width": "auto" });
            for (var index = 0; index < length; index++) {
                if (proxy.model.treeColumnIndex != index && columns[index].showInColumnChooser ==true ) {
                    var column = columns[index];
                    if (column.nonColumn) continue;
                    if (!ej.isNullOrUndefined(column)) {
                        var $innerDiv = ej.buildTag('div', '',"", { 'class': 'e-columnMenuListDiv' }),
                            styleAttr = {},
                            colValue = ej.isNullOrUndefined(column.headerText) ? column.field : column.headerText,
                            $input = ej.buildTag('input', '', styleAttr, { 'id': proxy._id + "_columnMenu_" + column.field.split('.').join(""), 'value': colValue, 'type': 'checkbox' }),
                            label = ej.buildTag('label', colValue, {"width": "auto" }, { 'for': proxy._id + "_columnMenu_" + column.field.split('.').join("") });
                        $innerDiv.append($input);
                        $innerDiv.append(label);
                        $innerDiv.data("column", column.headerText);
                        $listBox.append($innerDiv);
                        if (column.headerText == headerText)
                            $innerDiv.addClass("e-columnmenuselection");
                        $input.ejCheckBox({
                            checked: !ej.isNullOrUndefined(column.visible) ? column.visible : true,
                            cssClass: proxy.model.cssClass,
                            change: function(args)
                            {
                                var columnName = "";
                                if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                                    columnName = this.element.closest(".e-columnMenuListDiv").data().column;
                                } else {
                                    columnName = args.model.value;
                                }
                                if (!args.isChecked)
                                    proxy.hideColumn(columnName);
                                else
                                    proxy.showColumn(columnName);
                                proxy._updateWrapRowHeights(null,"virtualRefresh");
                            }
                        });
                    }
                }
            }
            if (!refresh) 
                proxy._columnChooserList = $listBox;
            else {
                proxy._columnChooserList.empty().append($listBox.children());
                proxy._focusTreeGridElement();
            }
            var columnChooserListItem = proxy._columnChooserList.find("div.e-columnMenuListDiv");
            $(columnChooserListItem).mouseenter(function () {
                columnChooserListItem.removeClass("e-columnmenuselection");
                $(this).addClass("e-columnmenuselection");
            });
        },

        //Template for column menu

        _createColumnMenuTemplate:function()
        {
            var proxy = this;
            var columnMenuList = "<li style='list-style-type:none;'>" +
                                 "<div class='e-columnmenuitem{{if enabled==false}} e-disable{{/if}}{{if (menuId =='Columns' || menuId =='Filter')}} e-parent-menuitem{{/if}}' style='display:table;cursor:pointer'" +
                                    " id='" + proxy._id + "_{{:menuId}}Chooser'>" +
                                "<div class='e-icon {{:icon}} e-treegridicon' style='display:inline-block;'></div>" +
                                "<div class='e-menuitem-label' style='display:inline-block;width:auto'>" +
                                "<span>{{:name}}</span></div>" +
                                "{{if (menuId =='Columns' || menuId =='Filter')}}<div class='e-icon e-expander e-treegridicon' style='display:inline-block;float:right;'/> {{/if}}" +
                                "</div>" +
                                "{{if menuId == 'RenameColumn' || menuId == 'SortDescending'}}<hr class='e-menuitem-hr'>{{/if}}" +
                                "</li>";            
            var templates = {};
            templates[proxy._id + "columnMenuTemplate"] = columnMenuList;
            $.templates(templates);
        },

        //render filter menu dialog in a respective position while clicking filter menu icon
        _updateFilterMenuList: function (e) {
            var proxy = this,
               model = proxy.model,
               columnFilterPosX, posY,
               posX,
               containerOffset,
               $target = $(e.target),
               targetWidth = targetHeight = 0,
               targetWidth = $target[0].offsetWidth,
               targetHeight = $target[0].offsetHeight,
               columnFieldName = $.trim($target.prev("div.e-headercelldiv").attr("data-ej-mappingname")), position,
               element, editType, str, index, filtertype,
               column = proxy.getColumnByField(columnFieldName);
            posX = columnFilterPosX = posY = 0;
            proxy._clearFilterMenu();
            proxy._clearColumnMenu("columnmenu");
            proxy._columnMenuTarget = $target;
            proxy._filterIconTarget = e;

            proxy._saveEditedElements();

            position = proxy.getOffsetRect($target[0]);
            position.top = position.top + targetHeight;
            if (!proxy.model.isFromGantt)
                posY = position.top + targetHeight;
            else
                posY = position.top + targetHeight;
            element = !model.isFromGantt ? $(this.element) : $(this.element).closest(".e-gantt"),
            containerOffset = this.getOffsetRect(element[0]);
            containerOffset.bottom = containerOffset.top + element[0].offsetHeight;
            containerOffset.right = containerOffset.left + $(element).width();
            if (column.filterEditType) {
                index = column.filterEditType.indexOf("edit");
                filtertype = column.filterEditType;
            }
            else {
                index = column.editType.indexOf("edit");
                filtertype = column.editType;
            }
            if (index != -1) {
                editType = filtertype.slice(0, index);
                str = editType.charAt(0).toUpperCase() + editType.slice(1);
                proxy._showFilterMenu("filtermenu", "filter" + str, position, targetWidth, containerOffset.right, posX, null, null, column, posY);
            } else {
                str = filtertype.charAt(0).toUpperCase() + filtertype.slice(1);
                proxy._showFilterMenu("filtermenu", "filter" + str, position, targetWidth, containerOffset.right, posX, null, null, column, posY);
            }
            proxy._updatePreviousFilteredValue(column);
        },
        //Method for showing column chooser 
        _showColumnChooser: function (e, container, columnListPosX, columnMenuWidth, posY, posX, columnMenu, isFromColumnmenu) {
            var proxy = this,
                 model = proxy.model,
                 width = $(window).width();
            if (((e.type == "mouseenter" && width > proxy._responsiveMinWidth || isFromColumnmenu) || !proxy.model.isResponsive) || ((e.type == "click" && width <= proxy._responsiveMinWidth && proxy.model.isResponsive)) || model.isFromGantt) {
                if ((width > proxy._responsiveMinWidth || isFromColumnmenu || !proxy.model.isResponsive) || model.isFromGantt) {
                    var evntArgs = {};
                    evntArgs.width = "auto";
                    evntArgs.height = "auto";
                    evntArgs.minWidth = "0px";
                    evntArgs.minHeight = "0px";
                    evntArgs.showHeader = false;
                    evntArgs.position = { X: posX, Y: posY };
                    evntArgs.enableModal = false;
                    evntArgs.isResponsive = false;
                    evntArgs.enableResize = false;
                    evntArgs.cssClass = proxy.model.cssClass;
                    $("#" + proxy._id + "ccDiv_wrapper").addClass("e-treegridadaptivedialog")
                    $("#" + proxy._id + "ccDiv_wrapper").removeClass("e-dialog").removeClass("e-shadow").find("div.e-dialog-scroller").css("height", "100%").removeClass("e-widget-content");
                    $("#" + proxy._id + "ccDiv").ejDialog(evntArgs);
                    $("#" + proxy._id + "ccDiv").css({ 'height': 'auto' });

                    proxy._updateColumnMenuVisibility();
                    var columnchooser = $("#" + proxy._id + "ccDiv").ejDialog("instance"),
                        columnChooserList = $("#" + proxy._id + "ccDiv_wrapper"),
                        columnChooserMenuList = columnChooserList.find("div.e-columnMenuListDiv");
                    columnchooser.open();
                    if (container < columnListPosX + columnChooserList.width()) {
                        columnListPosX = columnListPosX - columnChooserList.width() - columnMenuWidth;
                        
                    }
                    var position = {
                        X: columnListPosX,
                        Y: posY + columnMenu.position().top
                    };
                    columnchooser.option({ "position": position });
                    columnChooserList.css("z-index", proxy.getMaxZIndex() + 1);
                    columnChooserMenuList.removeClass("e-columnmenuselection");
                    columnChooserMenuList.eq(0).addClass("e-columnmenuselection");
                    proxy._focusTreeGridElement();
                    proxy._renderedSubMenuType = "ColumnDesktop";
                } else {
                    proxy._adaptiveColumnChooser();

                }
                if (model.allowFiltering && (model.filterSettings.filterType == "menu" || model.filterSettings.filterType == "excel")) {
                    proxy._isCloseFilterMenu = true;
                    proxy._clearFilterMenu();
                }
            }

        },
        //Creating Column Menu

        _renderColumnMenu: function (e) {
            var proxy = this,
                model = proxy.model,
                posX, columnListPosX, columnFilterPosX, posY,
                $target = $(e.target),
                targetWidth = targetHeight = 0,
                columnMenuItems = [],
                columnChooserListIndex = 0,
                targetWidth = $target[0].offsetWidth,
                targetHeight = $target[0].offsetHeight,
                columnMenuText = proxy._columnMenuTexts,
                columnFieldName = $.trim($target.prev("div.e-headercelldiv").attr("data-ej-mappingname")),
                column = proxy.getColumnByField(columnFieldName),
                filtermenuindex,
                allowSorting = column.allowSorting == undefined ? true : column.allowSorting;
            
            if (proxy._batchEditStarts && !(proxy._confirmDialog).is(":visible")) {
                proxy._batchSaveLostWarning("ColumnMenu");
                proxy._batchTempDetails.menuTarget = e;
                return true;
            }
            posX = columnListPosX = columnFilterPosX = posY = 0;
            proxy._clearColumnMenu("columnmenu");
            proxy._columnMenuTarget = $target;
            /* save edited value before rendering contextmenu on non edited cell*/
            if ((model.isEdit || this._isRowEdit)) {
                if (this.editFormValidate()) {
                    proxy._saveEditedElements();
                }
                else
                    return true;
            }
            

            if (model.showColumnChooser && model.showColumnOptions) {
                var isInsert = proxy._enableDisableInsertLabel();
                columnMenuItems.push({ "name": columnMenuText["insertColumnLeft"], "icon": "e-column-insertleft-icon", "menuId": "ColumnLeft", enabled: isInsert });
                columnMenuItems.push({ "name": columnMenuText["insertColumnRight"], "icon": "e-column-insertright-icon", "menuId": "ColumnRight", enabled: isInsert });
                columnMenuItems.push({ "name": columnMenuText["deleteColumn"], "icon": "e-column-delete-icon", "menuId": "DeleteColumn", enabled: proxy._enableDisableDEL(column) });
                columnMenuItems.push({ "name": columnMenuText["renameColumn"], "icon": "e-column-rename-icon", "menuId": "RenameColumn" });
            }
            if (model.allowFiltering && (ej.isNullOrUndefined(column.allowFiltering) || column.allowFiltering) && (model.filterSettings.filterType == "menu" || model.filterSettings.filterType == "excel")) {
                if (proxy._filteredColumnValueID.indexOf(column.field) != -1) {
                    columnMenuItems.push({ "name": columnMenuText["menuFilter"], "icon": "e-column-filtered-icon", "menuId": "Filter" });
                    filtermenuindex = columnMenuItems.length;
                } else {
                    columnMenuItems.push({ "name": columnMenuText["menuFilter"], "icon": "e-column-filter-icon", "menuId": "Filter" });
                    filtermenuindex = columnMenuItems.length;
                }
            }
            if (model.allowSorting && allowSorting)
            {
                columnMenuItems.push({ "name": columnMenuText["sortAscendingText"], "icon": "e-columnmenu-ascending", "menuId": "SortAscending" });
                columnMenuItems.push({ "name": columnMenuText["sortDescendingText"], "icon": "e-columnmenu-descending", "menuId": "SortDescending" });
            }
            if (model.showColumnChooser) {
                columnChooserListIndex = columnMenuItems.length;
                columnMenuItems.push({ "name": columnMenuText["columnsText"], "icon": "e-columnchooser-icon", "menuId": "Columns" });
            }

            if (!model.isFromGantt) {
                if (column.allowFreezing || (proxy._allowFreezingDefault && ej.isNullOrUndefined(column.allowFreezing))) {
                    var enableFreeze, enableFreezeBefore = false;
                    /*Check all other columns are in frozen*/
                    if (proxy._frozenColumnsLength == model.columns.length - 1 && !column.isFrozen)
                        enableFreeze = true;

                    /*Check frozen column width and container width */
                    var frozenWidth = this._getFrozenColumnWidth(),
                        columnIndex = model.columns.indexOf(column);
                    frozenWidth = frozenWidth + this.columnsWidthCollection[columnIndex] + 18;
                    if (frozenWidth > this._gridWidth)
                        enableFreeze = true;

                    columnMenuItems.push({ "name": columnMenuText["freezeText"], "icon": "e-freezecolumn-icon", "menuId": "FreezeColumns", enabled: enableFreeze ? false : !column.isFrozen });
                    columnMenuItems.push({ "name": columnMenuText["unfreezeText"], "icon": "e-unfreezecolumn-icon", "menuId": "UnfreezeColumns", enabled: (column.isFrozen == true) });
                    if (proxy._checkboxSelection && model.columns[0].nonColumn) {
                        if (model.columns.indexOf(column) != 1 && model.columns.indexOf(column) > this._frozenColumnsLength)
                            enableFreezeBefore = true;
                    }
                    else if (model.columns.indexOf(column) != 0 && model.columns.indexOf(column) > this._frozenColumnsLength)
                        enableFreezeBefore = true;

                    /*Check frozen column width and container width */
                    frozenWidth = this._getFrozenColumnWidth(columnIndex);
                    frozenWidth = frozenWidth + 18;
                    if (frozenWidth > this._gridWidth)
                        enableFreezeBefore = false;
                    columnMenuItems.push({ "name": columnMenuText["freezePrecedingColumnsText"], "icon": "e-freezecolumnbefore-icon", "menuId": "FreezePrecedingColumns", enabled: enableFreezeBefore });
                }
            }
            var position = proxy.getOffsetRect($target[0]);            
            var elementPosition = proxy.getOffsetRect(proxy.element[0]);           
            if (!proxy.model.isFromGantt)
                posY = position.top + targetHeight;
            else
                posY = position.top + targetHeight;            
            proxy._columnChooserListIndex = columnChooserListIndex;
            var $outerDiv = ej.buildTag("div.e-columnmenu e-js", "", {
                "display": "table",
                "position": "absolute", "top": posY, "z-index": 10000//proxy.getMaxZIndex() + 1
            }, { id: proxy._id + "_ColumnMenu" }),
                $outerUl = ej.buildTag("ul.e-treegrid-columnmenu", "", {}, {}),
                $columnMenuList = $.render[proxy._id + "columnMenuTemplate"](columnMenuItems);
            model.cssClass && ($outerDiv.addClass(model.cssClass));
            $outerUl.append($columnMenuList);
            $outerDiv.append($outerUl);
            $(document.body).append($outerDiv);
            var columnMenuHeight = $outerDiv.height(),
                columnMenuWidth = ($outerDiv.outerWidth() + 1);
            posX = position.left + targetWidth - columnMenuWidth;
            if (posX <= elementPosition.left)
                posX = elementPosition.left;
            columnListPosX = posX + columnMenuWidth;
            columnFilterPosX = posX + columnMenuWidth;
            proxy._columnListPosX = columnListPosX;
            proxy._columnFilterPosX = columnFilterPosX;
            proxy._columnMenuPopupY = posY;
            $outerDiv.css({ 'left': posX });
            $outerDiv.find(".e-columnmenuitem").outerWidth(columnMenuWidth);

            //Edge detection for column menu
            var element = !model.isFromGantt ? $(this.element) : $(this.element).closest(".e-gantt"),
                containerOffset = this.getOffsetRect(element[0]);
            containerOffset.bottom = containerOffset.top + element[0].offsetHeight;
            containerOffset.right = containerOffset.left + $(element).width();
            if (!model.isFromGantt) {
                if (proxy.model.isResponsive && $(window).width() <= proxy._responsiveMinWidth)
                    $("#" + proxy._id + "_ColumnMenu").find(".e-expander").addClass("e-treegridadaptivesubicon");
                else
                    $("#" + proxy._id + "_ColumnMenu").find(".e-expander").removeClass("e-treegridadaptivesubicon");
            }

            $("#" + proxy._id + "_ColumnsChooser").on("mouseenter click", function (e) {
                proxy._showColumnChooser(e, containerOffset.right, columnListPosX, columnMenuWidth, posY, posX, $("#" + proxy._id + "_ColumnsChooser"));
                });

            $("#" + proxy._id + "_FilterChooser").on("mouseenter click", function (e) {
                if (((e.type == "mouseenter" && $(window).width() > proxy._responsiveMinWidth) || !proxy.model.isResponsive) || ((e.type == "click" && $(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive))) {

                    var index, editType, str, filtertype;
                    proxy._updateColumnMenuVisibility();
                    proxy._checkFilterType(column);
                    if ((model.filterSettings.filterType == "menu" && column.filterType != "excel") || (model.filterSettings.filterType == "excel" && column.filterType == "menu")) {
                        if (column.filterEditType) {
                            index = column.filterEditType.indexOf("edit");
                            filtertype = column.filterEditType;
                        }
                        else {
                            column.editType = (column.editType == "edittemplate") ? "stringedit" : column.editType;
                            index = column.editType.indexOf("edit");
                            filtertype = column.editType;
                        }
                        if (index != -1) {
                            editType = filtertype.slice(0, index);
                            str = editType.charAt(0).toUpperCase() + editType.slice(1);
                            proxy._showFilterMenu("columnchooser", "filter" + str, columnFilterPosX, columnMenuWidth, containerOffset.right, posX, columnChooserListIndex, filtermenuindex, column, posY);
                        } else {
                            str = filtertype.charAt(0).toUpperCase() + filtertype.slice(1);
                            proxy._showFilterMenu("columnchooser", "filter" + str, columnFilterPosX, columnMenuWidth, containerOffset.right, posX, columnChooserListIndex, filtermenuindex, column, posY);
                        }
                        proxy._updatePreviousFilteredValue(column);
                        $("#" + proxy._id + "ccDiv").ejDialog("close");
                    }
                    else {
                        proxy._openExcelFilter(column, columnFilterPosX, posY,containerOffset,columnMenuWidth);
                    }
                }
            });
            $("#" + proxy._id + "_SortAscendingChooser," + "#" + proxy._id + "_SortDescendingChooser," +
              "#" + proxy._id + "_FreezeColumnsChooser," + "#" + proxy._id + "_UnfreezeColumnsChooser," +
              "#" + proxy._id + "_FreezePrecedingColumnsChooser," + "#" + proxy._id + "_ColumnLeftChooser," +
              "#" + proxy._id + "_ColumnRightChooser," + "#" + proxy._id + "_DeleteColumnChooser," +
              "#" + proxy._id + "_RenameColumnChooser").mouseenter(function () {
                $("#" + proxy._id + "ccDiv").ejDialog("close");
                if (model.allowFiltering && (model.filterSettings.filterType == "menu" || model.filterSettings.filterType == "excel")) {
                    proxy._isCloseFilterMenu = true;
                      proxy._clearFilterMenu();
                  }
              });
            $("#" + proxy._id + "_SortAscendingChooser").click(function () {
                if (model.allowMultiSorting)  proxy._multiSortRequest = true;
                if (model.isFromGantt)
                    proxy.sortColumn(column.mappingName ? column.mappingName : column.field, ej.sortOrder.Ascending);
                else
                    proxy.sortColumn(column.field, ej.sortOrder.Ascending);
                proxy._multiSortRequest = false;
            });
            $("#" + proxy._id + "_SortDescendingChooser").click(function () {
                if(model.allowMultiSorting) proxy._multiSortRequest = true;
                if (model.isFromGantt)
                    proxy.sortColumn(column.mappingName ? column.mappingName : column.field, ej.sortOrder.Descending);
                else
                    proxy.sortColumn(column.field, ej.sortOrder.Descending);
                proxy._multiSortRequest = false;
            });
            $("#" + proxy._id + "_FreezeColumnsChooser").click(function () {
                if (!$(this).hasClass("e-disable")) {
                    if (proxy._checkboxSelection) {
                        if (model.columns[0].nonColumn && !model.columns[0].isFrozen)
                            model.columns[0]['isFrozen'] = true;                        
                    }
                    proxy.freezeColumn(column.field, true);
                }
            });
            $("#" + proxy._id + "_UnfreezeColumnsChooser").click(function () {
                if (!$(this).hasClass("e-disable")) {                    
                    if (proxy._checkboxSelection && proxy._frozenColumnsLength == 2) {
                        if (model.columns[0].nonColumn && model.columns[0].isFrozen)
                            model.columns[0]['isFrozen'] = false;                            
                    }
                    proxy.freezeColumn(column.field, false);
                }
            });
            $("#" + proxy._id + "_FreezePrecedingColumnsChooser").click(function () {
                if (!$(this).hasClass("e-disable"))
                    proxy.freezePrecedingColumns(column.field);
            });

            $("#" + proxy._id + "_ColumnLeftChooser").click(function () {
                if (!$(this).hasClass("e-disable"))
                    proxy.insertColumnChooser(column, "left");
                else
                    return false;
            });
            $("#" + proxy._id + "_ColumnRightChooser").click(function () {
                if (!$(this).hasClass("e-disable"))
                    proxy.insertColumnChooser(column, "right");
                else
                    return false;
            });

            $("#" + proxy._id + "_DeleteColumnChooser").click(function () {
                if (!$(this).hasClass("e-disable")) {
                    proxy._targetColumnIndex = model.columns.indexOf(column);
                    proxy._updateConfirmDialog.ejDialog("open");
                    proxy._clearColumnMenu();
                }
                else
                    return false;
            });

            $("#" + proxy._id + "_RenameColumnChooser").click(function () {
                proxy._targetColumnIndex = model.columns.indexOf(column);
                proxy._renderColumnRenameDialog(column);
                proxy._columnRenameDialog.ejDialog("open");
                proxy._clearColumnMenu();
            });

            var columnMenu = $("#" + proxy._id + "_ColumnMenu"),
                columnMenuItem = columnMenu.find("div.e-columnmenuitem");
            columnMenuItem.eq(0).addClass("e-columnmenuselection");
            $(columnMenuItem).mouseenter(function () {
                if (!$(this).hasClass("e-disable")) {
                    columnMenuItem.removeClass("e-columnmenuselection");
                    $(this).addClass("e-columnmenuselection");
                }
            });
        },

        _openExcelFilter: function (column, posX, posY, containerOffset, columnMenuWidth)
        {
            //Calcualte x and y position for display the excel filter
            var proxy = this,
                model = proxy.model,
                windowWidth = $(document).width(), position,
                           colType = column.filterEditType ? column.filterEditType : column.editType,
                           positionX = posX;
            colType = colType.toLowerCase();
            if ((containerOffset.right < positionX + 266) || (windowWidth < positionX + 266)) {
                if ((positionX - columnMenuWidth - 266) > 0)
                    positionX = positionX - columnMenuWidth - 266 - 2;
            }
            else
                positionX = positionX + 2;
            position = {
                X: positionX,
                Y: posY + $("#" + proxy._id + "_FilterChooser").position().top
            };
            switch (colType) {
                case "numericedit":
                    colType = "number";
                    break
                case "stringedit":
                case "maskedit":
                case "dropdownedit":
                    colType = "string";
                    break;
                case "booleanedit":
                    colType = "boolean";
                    break;
                case "datepicker":
                case "datetimepicker":
                    colType = colType.replace("picker", "");
                    break

            }
            if (windowWidth <= proxy._responsiveMinWidth && model.isResponsive && model.enableResize) {
                proxy._renderResponsiveExcelFilter(column, colType);
                proxy._mediaStatus = true;
                proxy._clearColumnMenu();
            }
            else {

                // proxy._clearFilterMenu();
                if ($.inArray(colType, proxy._excelColTypes) == -1) {
                    $("#" + proxy._id + colType + "_excelDlg").remove();
                    $("#" + proxy._id + colType + "_CustomFDlg_wrapper").remove();
                    $("#" + proxy._id + colType + "_CustomFDlg").remove();
                    proxy._excelFilter._openedFltr = null;
                    proxy._excelColTypes.push(colType);
                    proxy._excelFilter.renderDialog(colType);
                }
                var localXFLabel = { True: proxy.localizedLabels.trueText, False: proxy.localizedLabels.falseText },
                    records = [];
                $.grep(proxy.model.flatRecords, function (record) {
                    if (!record.isSummaryRow)
                        records.push(record.item);
                });
                proxy._isCloseFilterMenu = true;
                proxy._excelFilter.openXFDialog({ field: column.field, enableResponsiveRow: true, displayName: column.headerText, dataSource: records, query: "", position: { X: position.X, Y: position.Y }, dimension: { height: $(window).height(), width: $(window).width() }, cssClass: "resFilter", type: colType, format: column.format, localizedStrings: localXFLabel });
                proxy._isCloseFilterMenu = false;
                $("#" + proxy._id + colType + "_CheckBoxList").css("display", "inline-block");
                proxy._customFilter = null;
            }
            $("#" + proxy._id + "ccDiv").ejDialog("close");
        },
        // Method to render adaptive column chooser

        _adaptiveColumnChooser: function () {
            var proxy = this, evntArgs = {}, controlPosition  = proxy.getOffsetRect($("#" + proxy._id)[0]),columnMenuText = proxy._columnMenuTexts;
            evntArgs.cssClass = proxy.model.cssClass,
            evntArgs.enableModal = false,
            evntArgs.enableResize = false;
            evntArgs.enableResize = false,
            evntArgs.contentSelector = "#" + proxy._id,
            evntArgs.rtl = proxy.model.rtl,
            evntArgs.allowDraggable = false;
            evntArgs.showHeader = true;
            evntArgs.minHeight = "50px";
            evntArgs.minWidth = "50px";
                evntArgs.width = $("#" + proxy._id).width() + "px";
                evntArgs.height = $("#" + proxy._id).height() + "px";
                evntArgs.position = { X: controlPosition .left + "px", Y: controlPosition .top + "px" };
                evntArgs.isResponsive = false;
            evntArgs.allowKeyboardNavigation = false;
            evntArgs.title = columnMenuText["columnsText"];
            $("#" + proxy._id + "ccDiv_wrapper").removeClass("e-treegridadaptivedialog");
            $("#" + proxy._id + "ccDiv_wrapper").addClass("e-dialog").addClass("e-shadow").find("div.e-dialog-scroller").css("height", "100%");
            $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").addClass("e-treegridadaptivedialog");
            $("#" + proxy._id + "ccDiv").ejDialog(evntArgs);
            $("#" + proxy._id + "ccDiv").ejDialog("open");

            $("#" + proxy._id + "ccDiv").css({ 'width': '100%', 'height': '100%' });
            if (proxy.model.isResponsive) {
                var dialogInnerDivHeight = ($("#" + proxy._id).height() - 50) + "px"; // (48px - Dialog box header height) + (2px - excess border of parent div).
                $("#" + proxy._id + "ccDiv_wrapper").css({ 'width': $("#" + proxy._id).width(), 'height': $("#" + proxy._id).height() });
                $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").removeClass("e-treegridadaptivedialog");
                $("#" + proxy._id + "ccDiv").css({ 'width': '100%', 'height': 'auto' });
                if ($("#" + proxy._id + "ccDiv").height() > $("#" + proxy._id + "ccDiv_wrapper").height() - 48) { // 48px - Dialog box header height
                    $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").css({ 'height': ($("#" + proxy._id).height() - 48) + "px", 'width': 'auto', 'padding': '0px' }); // 48px - Dialog box header height
                    $("#" + proxy._id + "ccDiv").removeClass("e-columnSelector");
                    $("#" + proxy._id + "ccDiv").ejDialog("refresh");
                } else {
                    $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").css({ 'height': dialogInnerDivHeight, 'width': 'auto' });
                    $("#" + proxy._id + "ccDiv").css({ 'padding': '8px 16px' });
                }
                $("#" + proxy._id + "ccDiv_wrapper").removeClass("e-shadow");
                $("#" + proxy._id + "ccDiv").removeClass("e-shadow");
            }

            proxy._renderedSubMenuType = "AdaptiveColChooser";
            $("#" + proxy._id + "_ColumnMenu").remove();

        },

        //update previous filtered value for current rendering dialog box depends on its filter edit type
        _updatePreviousFilteredValue: function (column) {
            var proxy = this, suggesionBox, dropDown, dataManager, query, result, fieldIndex, groups = [], field1, filtertype, emptydrop;
            if (column.filterEditType)
                filtertype = column.filterEditType;
            else
                filtertype = column.editType;
            if (filtertype == "stringedit") {
                suggesionBox = $("#" + proxy._id + "filterStringInputBox").data("ejAutocomplete");
                dropDown = $("#" + proxy._id + "filterStringMenuDiv").find("#" + proxy._id + "filterStringDropDown").data("ejDropDownList");
                query = ej.Query().select(column.field);
                dataManager = new ej.DataManager(proxy.model.flatRecords.slice());
                result = dataManager.executeLocal(query);
                suggesionBox.option("dataSource", result);
                if (proxy._filteredColumnValueID.indexOf(column.field) != -1) {
                    fieldIndex = proxy._filteredColumnValueID.indexOf(column.field);
                    if (proxy._filteredColumnValue[fieldIndex].value == "")
                        dropDown.option("value", "startswith");
                    else
                        dropDown.option("value", proxy._filteredColumnValue[fieldIndex].dropdown);
                    suggesionBox.option("value", proxy._filteredColumnValue[fieldIndex].value);
                    $("#" + proxy._id + "filterStringInputBox").val(proxy._filteredColumnValue[fieldIndex].value);
                } else {
                    dropDown.option("value", "startswith");
                    suggesionBox.option("value", null);
                }
            }
            if (filtertype == "numericedit") {
                suggesionBox = $("#" + proxy._id + "filterNumericInputBox").data("ejNumericTextbox");
                dropDown = $("#" + proxy._id + "filterNumericMenuDiv").find("#" + proxy._id + "filterNumericDropDown").data("ejDropDownList");
                if (proxy._filteredColumnValueID.indexOf(column.field) != -1) {
                    fieldIndex = proxy._filteredColumnValueID.indexOf(column.field);
                    if (proxy._filteredColumnValue[fieldIndex].value == "")
                        dropDown.option("value", "lessthan");
                    else
                        dropDown.option("value", proxy._filteredColumnValue[fieldIndex].dropdown);
                    suggesionBox.option("value", proxy._filteredColumnValue[fieldIndex].value);
                    $("#" + proxy._id + "filterNumericInputBox").val(proxy._filteredColumnValue[fieldIndex].value);
                } else {
                    suggesionBox.option("value", null);
                    dropDown.option("value", "lessthan");
                }
            }
            if (filtertype == "datepicker") {
                suggesionBox = $("#" + proxy._id + "filterDatepickerInputBox").data("ejDatePicker");
                dropDown = $("#" + proxy._id + "filterDatepickerMenuDiv").find("#" + proxy._id + "filterDatepickerDropDown").data("ejDropDownList");
                if (proxy._filteredColumnValueID.indexOf(column.field) != -1) {
                    fieldIndex = proxy._filteredColumnValueID.indexOf(column.field);
                    if (proxy._filteredColumnValue[fieldIndex].value == "")
                        dropDown.option("value", "lessthan");
                    else
                        dropDown.option("value", proxy._filteredColumnValue[fieldIndex].dropdown);
                    suggesionBox.option("value", proxy._filteredColumnValue[fieldIndex].value);
                } else {
                    suggesionBox.option("value", null);
                    dropDown.option("value", "lessthan");
                }
                if (column.format) {
                    suggesionBox.option("dateFormat", proxy._getDateFormat(column.format));
                }
            }
            if (filtertype == "datetimepicker") {
                suggesionBox = $("#" + proxy._id + "filterDatetimepickerInputBox").data("ejDateTimePicker");
                dropDown = $("#" + proxy._id + "filterDatetimepickerMenuDiv").find("#" + proxy._id + "filterDatetimepickerDropDown").data("ejDropDownList");
                if (proxy._filteredColumnValueID.indexOf(column.field) != -1) {
                    fieldIndex = proxy._filteredColumnValueID.indexOf(column.field);
                    if (proxy._filteredColumnValue[fieldIndex].value == "")
                        dropDown.option("value", "lessthan");
                    else
                        dropDown.option("value", proxy._filteredColumnValue[fieldIndex].dropdown);
                    suggesionBox.option("value", proxy._filteredColumnValue[fieldIndex].value);
                } else {
                    suggesionBox.option("value", null);
                    dropDown.option("value", "lessthan");
                }
                if (column.format) {
                    suggesionBox.option("dateFormat", proxy._getDateFormat(column.format));
                }
            }
            if (filtertype == "dropdownedit") {
                $.extend(groups, column.dropdownData);
                if (column.editParams) {
                    field1 = { value: column.editParams.fields.value, text: column.editParams.fields.text };
                        emptydrop = {};
                        emptydrop[column.editParams.fields.text] = proxy._dropDownListBlanksText;
                        emptydrop[column.editParams.fields.value] = "(Blanks)";
                } else {
                    field1 = { value: "value", text: "text" };
                    emptydrop = { text: proxy._dropDownListBlanksText, value: "(Blanks)" };
                }
                dropDown = $("#" + proxy._id + "filterDropdownDropDown").data("ejDropDownList");
                if (!ej.isNullOrUndefined(groups) && groups.length != 0) {

                    if (column.allowFilteringBlankContent || ej.isNullOrUndefined(column.allowFilteringBlankContent))
                        groups.unshift(emptydrop);

                    dropDown.option("dataSource", groups);
                }
                else {
                    groups = proxy._dropDownArrayCollection(column, column.field);
                    if (column.allowFilteringBlankContent || ej.isNullOrUndefined(column.allowFilteringBlankContent))
                        groups.unshift(emptydrop);
                    dropDown.option("dataSource", groups);
                }
                dropDown.option("fields", field1);

                if (proxy._filteredColumnValueID.indexOf(column.field) != -1) {
                    fieldIndex = proxy._filteredColumnValueID.indexOf(column.field);
                    dropDown.option("value", proxy._filteredColumnValue[fieldIndex].value);

                } else {
                        dropDown.option("value", null);
                }
            }
            if (filtertype == "booleanedit") {
                var suggesionBox = $("#" + proxy._id + "filterBooleanInputBox").data("ejCheckBox");
                if (proxy._filteredColumnValueID.indexOf(column.field) != -1) {
                    fieldIndex = proxy._filteredColumnValueID.indexOf(column.field);
                    if (proxy._filteredColumnValue[fieldIndex].value == "") {
                        suggesionBox.option("checkState", "uncheck");
                    }
                    else
                        suggesionBox.option("value", proxy._filteredColumnValue[fieldIndex].value);
                } else {
                    suggesionBox.option("selectedIndex", "");
                }
            }
        },

        //perform filter operation
        _performMenuFilter: function (column, suggesionBoxValue, dropDownValue) {
            var proxy = this, filteredColumnIdIndex, filtertype;
            proxy.clearAllSelection();
            if (column.filterEditType)
                filtertype = column.filterEditType;
            else
                filtertype = column.editType;
            if (ej.isNullOrUndefined(suggesionBoxValue)) {
                suggesionBoxValue = "";
            }
            proxy._filterString = suggesionBoxValue;
            if (proxy._filteredColumnValueID.indexOf(column.field) == -1 && suggesionBoxValue.toString() == "") {
                proxy.clearColumnMenu();
                proxy._clearFilterMenu();
            } else {
                if (filtertype == "numericedit" || filtertype == "datepicker" || filtertype == "datetimepicker") {
                    if ((filtertype == "datepicker" || filtertype == "datetimepicker") && (dropDownValue == "equal" || dropDownValue == "notequal"))
                        proxy.filterColumn(column.field, dropDownValue, suggesionBoxValue, "and");
                    else
                        proxy.filterColumn(column.field, dropDownValue, suggesionBoxValue, "and", true);
                }
                else {
                    proxy.filterColumn(column.field, dropDownValue, suggesionBoxValue, "and");
                }
            proxy.clearColumnMenu();
            proxy._clearFilterMenu();
            proxy._cancelSaveTools();
        }

        },

        // Resore the previous filtered values.
        _storePreviousFilteredValue: function (field, suggesionBoxValue, dropDownValue) {
            var proxy = this, filteredColumnIdIndex;
            if (proxy._filteredColumnValueID.indexOf(field) == -1 && (suggesionBoxValue.toString() != "" || suggesionBoxValue.toString() == "false")) {
                proxy._filteredColumnValue.push({ column: field, value: suggesionBoxValue, dropdown: dropDownValue });
                proxy._filteredColumnValueID.push(field);
            } else {
                if (suggesionBoxValue.toString() != "" || suggesionBoxValue.toString() == "false") {
                    filteredColumnIdIndex = proxy._filteredColumnValueID.indexOf(field);
                    proxy._filteredColumnValue[filteredColumnIdIndex].value = suggesionBoxValue;
                    proxy._filteredColumnValue[filteredColumnIdIndex].dropdown = dropDownValue;
                } else {
                    if (proxy._filteredColumnValueID.indexOf(field) != -1) {
                        filteredColumnIdIndex = proxy._filteredColumnValueID.indexOf(field);
                        proxy._filteredColumnValue.splice(filteredColumnIdIndex, 1);
                        proxy._filteredColumnValueID.splice(filteredColumnIdIndex, 1);
                    }
                }
            }
        },

        //render filter menu dialog in a respective position
        _showFilterMenu: function (from, idValue, columnFilterPosX, columnMenuWidth, containerRightOffset, posX, columnChooserListIndex, filtermenuindex, column, posY) {
            if (!this.editFormValidate()) return;
            if (($(window).width() <= this._responsiveMinWidth && this.model.isResponsive) && !this.model.isFromGantt) {
                var proxy = this,
                evntArgs = {},
                controlPosition  = proxy.getOffsetRect($("#" + proxy._id)[0]);
                evntArgs.cssClass = this.model.cssClass;
                evntArgs.enableModal = false;
                evntArgs.isResponsive = false;
                evntArgs.enableRTL = this.model.enableRTL;
                evntArgs.enableResize = false;
                evntArgs.allowDraggable = false;
                evntArgs.minWidth = "50px";
                evntArgs.minHeight = "50px";
                evntArgs.width = $("#" + proxy._id).width() + "px";
                evntArgs.height = $("#" + proxy._id).height() + "px";
                evntArgs.position = { X: controlPosition .left + "px", Y: controlPosition .top + "px" };
                evntArgs.contentSelector = "#" + proxy._id;
                evntArgs.rtl = proxy.model.rtl;
                evntArgs.allowKeyboardNavigation = false;
                evntArgs.showHeader = true;
                evntArgs.title = column.headerText;
                $("#" + proxy._id + idValue + "MenuDiv").removeClass("e-treegridadaptivedialog");
                $("#" + proxy._id + idValue + "MenuDiv").ejDialog(evntArgs);
                $("#" + proxy._id + idValue + "MenuDiv_wrapper").addClass("e-dialog").addClass("e-shadow").find("div.e-dialog-scroller").addClass("e-treegridadaptivewidth");
                $("#" + proxy._id + idValue + "MenuDiv").css({ 'width': '100%','height':'auto' });
                var dialogInnerDivHeight = $("#" + proxy._id).height() - 50 + "px"; // (48px - Dialog box header height) + (2px - excess border of parent div).
                $("#" + proxy._id + idValue + "MenuDiv").ejDialog('open');
                    $("#" + proxy._id + idValue + "MenuDiv_wrapper").find("div.e-dialog-scroller").css({ 'height': dialogInnerDivHeight });
                    $("#" + proxy._id + idValue + "MenuDiv_wrapper").removeClass("e-shadow");
                    $("#" + proxy._id + idValue + "MenuDiv").removeClass("e-shadow");

                    if ($("#" + proxy._id + idValue + "MenuDiv").height() > $("#" + proxy._id + idValue + "MenuDiv_wrapper").height() - 48) { // 48px - Dialog box header height
                        $("#" + proxy._id + idValue + "MenuDiv").ejDialog("refresh");
                    }
                proxy._renderedSubMenuType = "#" + proxy._id + idValue + "MenuDiv";
                $("#" + proxy._id + "_ColumnMenu").remove();
            } else {
                var proxy = this;
                var evntArgs = {};
                evntArgs.width = "auto",
                evntArgs.height = "auto",
                evntArgs.minWidth = "0px",
                evntArgs.minHeight = "0px",
                evntArgs.showHeader = false,
                evntArgs.position = { X: 1, Y: 1 },
                evntArgs.enableModal = false,
                evntArgs.isResponsive = false,
                evntArgs.enableResize = false,
                evntArgs.cssClass = this.model.cssClass;
                $("#" + proxy._id + idValue + "MenuDiv_wrapper").removeClass("e-dialog").removeClass("e-shadow").find("div.e-dialog-scroller").css("height", "100%").removeClass("e-widget-content");
                $("#" + proxy._id + idValue + "MenuDiv").ejDialog(evntArgs);
                var filterMenuDiv, filterMenuDivWrapper, filterMenuPosition;
                filterMenuDiv = $("#" + proxy._id + idValue + "MenuDiv").ejDialog("instance");
                filterMenuDivWrapper = $("#" + proxy._id + idValue + "MenuDiv_wrapper");
                if (from == "columnchooser") {
                    var windowWidth = $(document).width();
                    if ((containerRightOffset < columnFilterPosX + filterMenuDivWrapper.width()) || (windowWidth < columnFilterPosX + filterMenuDivWrapper.width())) {
                        if ((columnFilterPosX - columnMenuWidth - filterMenuDivWrapper.width()) > 0)
                            columnFilterPosX = columnFilterPosX - columnMenuWidth - filterMenuDivWrapper.width();
                    }
                    filterMenuPosition = {
                        X: columnFilterPosX,
                        Y: posY + $("#" + proxy._id + "_FilterChooser").position().top
                    };
                    filterMenuDiv.option({ "position": filterMenuPosition });
                    filterMenuDiv.open();
                } else if (from = "filtermenu") {
                    var position = columnFilterPosX,
                    targetWidth = columnMenuWidth;
                    posX = position.left + targetWidth - filterMenuDivWrapper.width();
                    position.left = posX + filterMenuDivWrapper.width() + 4;
                    if (containerRightOffset < position.left + filterMenuDivWrapper.width()) {
                        position.left = position.left - filterMenuDivWrapper.width() - targetWidth;
                    }
                    filterMenuPosition = {
                        X: position.left,
                        Y: position.top
                    };
                    filterMenuDiv.option({ "position": filterMenuPosition });
                    filterMenuDiv.open();
                }
                proxy._renderedSubMenuType = "#" + proxy._id + idValue + "MenuDivDesktop";
            }
        },

        _clearResponsiveColumnMenu:function(){
            var proxy = this;
            $("div[id$='ccDiv']").each(function () {
                if ($(this).data("columnMenuDialog") == "TreeGrid")
                    $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").css({ 'padding': '0px' });
            $("#" + proxy._id + "ccDiv").addClass("e-columnSelector");
            $("div[id$='filterStringMenuDiv']").each(function () {
                if ($(this).data("filterStringMenuDialog") == "TreeGrid")
                    $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("div[id$='filterNumericMenuDiv']").each(function () {
                if ($(this).data("filterNumericMenuDialog") == "TreeGrid")
                    $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("div[id$='filterBooleanMenuDiv']").each(function () {
                if ($(this).data("filterBooleanMenuDialog") == "TreeGrid")
                    $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("div[id$='filterDatepickerMenuDiv']").each(function () {
                if ($(this).data("filterDatepickerMenuDialog") == "TreeGrid")
                    $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("div[id$='filterDatetimepickerMenuDiv']").each(function () {
                if ($(this).data("filterDatetimepickerMenuDialog") == "TreeGrid")
                    $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("div[id$='filterDropdownMenuDiv']").each(function () {
                if ($(this).data("filterDropdownMenuDialog") == "TreeGrid")
                    $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("div[id$='ColumnRenameDialog']").each(function () {
                $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $("div[id$='dialogColumnAdd']").each(function () {
                $(this).data("ejDialog") && $(this).ejDialog("close");
            });

            $("#" + proxy._id).find(".e-filtericon").data("isClicked", false);
        },

        //Clear the column menu and menu list

        _clearColumnMenu:function(args)
        {
            var proxy = this,
                model = proxy.model;
            if (args === "columnmenu" && $(window).width() < proxy._responsiveMinWidth) {
                proxy._clearResponsiveColumnMenu();
            }
            if (model.isResponsive && $(window).width() < proxy._responsiveMinWidth) {
                $("#" + proxy._id + "_ColumnMenu").remove();
                if (proxy._renderedSubMenuType && proxy._renderedSubMenuType.indexOf("AdaptiveColChooser") == -1) {
                    $("div[id$='" + proxy._id + "ccDiv']").each(function () {
                        if ($(this).data("columnMenuDialog") == "TreeGrid")
                            $(this).data("ejDialog") && $(this).ejDialog("close");
                    });
                    $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").css({'padding': '0px' });
                    $("#" + proxy._id + "ccDiv").addClass("e-columnSelector");
                }
            }
        else{
            $("#" + proxy._id + "_ColumnMenu").remove();
            if (args != "resize") {
                $("div[id$='" + proxy._id + "ccDiv']").each(function () {
                    if ($(this).data("columnMenuDialog") == "TreeGrid")
                        $(this).data("ejDialog") && $(this).ejDialog("close");
                });
                $("#" + proxy._id + "ccDiv_wrapper").find("div.e-dialog-scroller").css({ 'padding': '0px' });
                $("#" + proxy._id + "ccDiv").addClass("e-columnSelector");
                if (args != "resize" && args!="cancelHandler") {
                    if (model.allowFiltering && (model.filterSettings.filterType == "menu" || model.filterSettings.filterType == "excel")) {
                        proxy._clearFilterMenu();
                    }
                }
            }
    }

            $("#" + proxy._id).find(".e-columnicon").data("isClicked", false);
            $("#" + proxy._id).find(".e-filtericon").data("isClicked", false);
        },

        // clear the filter menu dialog and its inner components
        _clearFilterMenu: function () {
            var proxy = this;
            if (proxy.model.allowFiltering) {
                $("div[id$='" + proxy._id + "filterStringMenuDiv']").each(function () {
                    if ($(this).data("filterStringMenuDialog") == "TreeGrid")
                        $(this).data("ejDialog") && $(this).ejDialog("close");
                });
                $("div[id$='" + proxy._id + "filterNumericMenuDiv']").each(function () {
                    if ($(this).data("filterNumericMenuDialog") == "TreeGrid")
                        $(this).data("ejDialog") && $(this).ejDialog("close");
                });
                $("div[id$='" + proxy._id + "filterBooleanMenuDiv']").each(function () {
                    if ($(this).data("filterBooleanMenuDialog") == "TreeGrid")
                        $(this).data("ejDialog") && $(this).ejDialog("close");
                });
                $("div[id$='" + proxy._id + "filterDatepickerMenuDiv']").each(function () {
                    if ($(this).data("filterDatepickerMenuDialog") == "TreeGrid")
                        $(this).data("ejDialog") && $(this).ejDialog("close");
                });
                $("div[id$='" + proxy._id + "filterDatetimepickerMenuDiv']").each(function () {
                    if ($(this).data("filterDatetimepickerMenuDialog") == "TreeGrid")
                        $(this).data("ejDialog") && $(this).ejDialog("close");
                });
                $("div[id$='" + proxy._id + "filterDropdownMenuDiv']").each(function () {
                    if ($(this).data("filterDropdownMenuDialog") == "TreeGrid")
                        $(this).data("ejDialog") && $(this).ejDialog("close");
                });
                $("#" + proxy._id + "filterStringDropDown_popup_list_wrapper").hide();
                $("#" + proxy._id + "filterStringInputBox_suggestion").hide();
                $("#" + proxy._id + "filterNumericDropDown_popup_list_wrapper").hide();
                $("#" + proxy._id + "filterNumericInputBox_suggestion").hide();
                $("#" + proxy._id + "filterDatepickerDropDown_popup_list_wrapper").hide();
                $("#" + proxy._id + "filterDatepickerInputBox_suggestion").hide();
                $("#" + proxy._id + "filterDatetimepickerDropDown_popup_list_wrapper").hide();
                $("#" + proxy._id + "filterDatetimepickerInputBox_suggestion").hide();
                $("#" + proxy._id + "filterDropdownDropDown_popup_list_wrapper").hide();
                $("#" + proxy._id).find(".e-filtericon").data("isClicked", false);
                var datetimepicobj = $("#" + proxy._id + "filterDatetimepickerInputBox").data("ejDateTimePicker");
                if (datetimepicobj)
                    datetimepicobj.hide();
                var datepicobj = $("#" + proxy._id + "filterDatepickerInputBox").data("ejDatePicker");
                if (datepicobj)
                    datepicobj.hide();
                if (proxy._isExcelFilter) {
                    proxy._excelFilter.closeXFDialog();
                    proxy._mediaStatus = false;
                }
            }
        },

        _getDateFormat: function (format) {
            var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm"),
                formatVal = toformat.exec(format),
                dateFormat = formatVal[2];
            return dateFormat;
        },

        //Enabling or disabling delete button of columnchooser
        _enableDisableDEL: function (colItem) {
            var proxy = this, model = proxy.model,
                fieldName = colItem.field,
                columnIndex = model.columns.indexOf(colItem);
            if (model.isFromGantt) {
                if (fieldName == "taskId" || fieldName == "serialNumber" || fieldName == "taskName" || fieldName == "startDate" || fieldName == "endDate" || (fieldName == "duration" && !model.endDateMapping) || columnIndex == model.treeColumnIndex)
                    return false;
                else
                    return true;
            }
            else {
                if (columnIndex == model.treeColumnIndex)
                    return false;
                else
                    return true;
            }
        },

        //Enabling or disabling Insert column label columnchooser
        _enableDisableInsertLabel: function () {
            var proxy = this, model = proxy.model;
            //Block column add operation while frozen columns Width is near to grid width
            if (!model.isFromGantt && proxy._frozenColumnsLength > 0) {
                var frozenWidth = this._getFrozenColumnWidth();
                frozenWidth = frozenWidth + 250;
                if (frozenWidth > this._gridWidth)
                    return false;
                else
                    return true;
            }
            else
                return true;
        },

        //Insert column through column chooser
        insertColumnChooser: function (column, position) {
            var proxy = this, model = proxy.model;
            if ($("#" + proxy._id + "_dialogColumnAdd").length === 0) {
                var $dialogCol = ej.buildTag("div.e-dialog e-dialog-content e-widget-content", "", { display: "none" }, { id: proxy._id + "_dialogColumnAdd" });
                proxy.element.append($dialogCol);
                proxy._on($("#" + proxy._id + "_dialogColumnAdd"),
               "click keypress", "#ColumnAddDialog_" + proxy._id + "_Ok ,#ColumnAddDialog_" + proxy._id + "_Cancel", proxy._buttonClick);
            }
            proxy.renderColumnAddDialog();
            proxy._targetColumnIndex = model.columns.indexOf(column);
            proxy._insertPosition = position;
        },

        // To change the headerText of column
        renameColumn: function (colIndex, name) {
            var proxy = this, model = proxy.model,
                columns = model.columns, args = {};
            args.requestType = "renameColumn";
            args.columnObject = columns[colIndex];
            args.previousName = columns[colIndex].headerText;
            args.currentName = name;          
            if (!proxy._trigger("actionComplete", args)) {
                columns[colIndex].headerText = name;
                proxy._refreshFrozenColumns();
            }
            proxy._updateWrapRowHeights();
            if (model.allowTextWrap || model.headerTextOverflow == "wrap")
                proxy._updateGridContentHeight();
        },

        //Method to render dialog box for renaming column
        _renderColumnRenameDialog: function (columnItem) {
            var proxy = this, columnMenuText = proxy._columnMenuTexts,
                $contentDiv = ej.buildTag('div.e-treegridrenamedialog', "", {}, { 'data-unselectable': 'on' }), controlWidth = $("#" + proxy._id).width(), controlHeight = $("#" + proxy._id).height(),
                $form = ej.buildTag('form', "", { 'height': '100%', 'width': '100%', 'font-size': '14px' }, { id: proxy._id + "ColumnRenameForm", onsubmit: "return false" }),
            $buttons = ej.buildTag('span.e-buttons', "<input class='e-treegrid-rename-dialog-btn-ok' type='button' id=" + proxy._id + 'RenameDialogOK' + " value=" + proxy._okButtonText + " /> "
            + "<input class='e-treegrid-rename-dialog-btn-cancel' type='button' id=" + proxy._id + 'RenameDialogCancel' + " value='" + proxy._cancelButtonText + "' />"),
            btnDiv = ej.buildTag('div', "", {}, { 'class': "e-treegrid-rename-dialog-btn" }),

            $innerTable = ej.buildTag('table', "", { 'outline': 'none', 'width': '100%', 'margin-top': '0px' }, { 'data-unselectable': "on" }),

            $inTr1 = ej.buildTag('tr'),

            $inTd = ej.buildTag('td.e-editLabel', "", { "outline": "none" }, { 'data-unselectable': "on" });
            $inTd.append("<label style='font-weight:normal;'>" + proxy._columnDialogTitle["renameColumn"] + "</label>");

            $inTr1.append($inTd);

            $innerTable.append($inTr1);

            var $inTr2 = ej.buildTag('tr'),
            $inTd2 = ej.buildTag('td.e-editValue'),

            $input = ej.buildTag('input.e-field e-ejinputtext',
                                            "",
                                            {},
                                            {
                                                value: columnItem.headerText,
                                                id: proxy._id + "RenameHeaderText",
                                                name: columnItem.field,
                                                "data-dialog": "RenameHeaderText"
                                            });
            $input.css({ 'margin': '0px', 'padding': '1px 4px 1px 0px' });
            $inTd2.html($input);
            $inTr2.append($inTd2);
            $innerTable.append($inTr2);
            $form.append($innerTable);
            $contentDiv.append($form);

            proxy._columnRenameDialog = ej.buildTag('div', '', '', { 'title': columnMenuText["renameColumn"], 'id': proxy._id + 'ColumnRenameDialog' });
            btnDiv.append($buttons);
            proxy._columnRenameDialog.append($contentDiv).append(btnDiv);
            proxy.element.append(proxy._columnRenameDialog);
            $buttons.find("input").ejButton({
                cssClass: proxy.model.cssClass,
                showRoundedCorner: true,
                size: "normal",
                click: $.proxy(proxy._triggerRenameColumn, proxy)
            });
            if (($(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive) && !proxy.model.isFromGantt) {
                if (proxy.model.isResponsive) {
                    var controlPosition = proxy.getOffsetRect($("#" + proxy._id)[0]);
                    $("#" + proxy._id + "ColumnRenameDialog").removeClass("e-treegridadaptivedialog");
                    proxy._columnRenameDialog.ejDialog({ width: controlWidth + "px", height: controlHeight + "px", position: { X: controlPosition.left + "px", Y: controlPosition.top + "px" }, minHeight: "50px", minWidth: "50px", showOnInit: false, allowDraggable: false, enableResize: false, enableModal: false, isResponsive: false, cssClass: this.model.cssClass });
                    $("#" + proxy._id + "ColumnRenameDialog_wrapper").removeClass("e-shadow").addClass("e-treegrid-dialog");
                    $("#" + proxy._id + "ColumnRenameDialog").removeClass("e-shadow");
                }
            } else {
                proxy._columnRenameDialog.ejDialog({ width: "auto", height: "auto", minHeight: 0, minWidth: 240, showOnInit: false, enableResize: false, allowDraggable: true, enableModal: true, isResponsive: true, cssClass: this.model.cssClass });
                $("#" + proxy._id + "ColumnRenameDialog_wrapper").addClass("e-treegrid-dialog");
            }
            $input.focus().select();
        },

        _triggerRenameColumn: function (args) {
            var proxy = this, $elem, text;
            if (args !== undefined && args.model.text == proxy._okButtonText) {
                $elem = proxy._columnRenameDialog.find("#" + proxy._id + "RenameHeaderText");
                text = $elem.val();
                proxy.renameColumn(proxy._targetColumnIndex, text);
                if (this.model.editSettings.allowEditing && this.model.editSettings.editMode == "dialogEditing" && (ej.isNullOrUndefined(proxy.model.editSettings.dialogEditorTemplateID) || proxy.model.editSettings.dialogEditorTemplateID == "")) {
                    proxy._treeGridAddEditDialog();
                } else if (this.model.editSettings.allowEditing && this.model.editSettings.editMode == "dialogEditing" && (!ej.isNullOrUndefined(proxy.model.editSettings.dialogEditorTemplateID) && proxy.model.editSettings.dialogEditorTemplateID != "")) {
                    proxy._treeGridAddEditDialogTemplate();
                }
            }
            proxy._columnRenameDialog.ejDialog("close");
        },

        //To render dialog for adding custom column
        renderColumnAddDialog: function () {

            var proxy = this, model = proxy.model, colDialogFields = model.columnDialogFields,
                args = {}, controlPosition  = proxy.getOffsetRect($("#" + proxy._id)[0]);;

            args.requestType = "openColumnAddDialog";
            if (colDialogFields.length) {
                args.data = {};
                for (var i = 0; i < colDialogFields.length; i++) {
                    var val = colDialogFields[i];
                    args.data[val] = "";
                }
            }
            else args.data = { "field": "", "headerText": "", "editType": "" };

            if (!proxy._trigger("actionBegin", args)) {
                args.requestType = "columnAdd";
                var temp = document.createElement('div');
                $(temp).addClass("e-addedColumn");
                temp.innerHTML = $.render[proxy._id + "_ColumnAddTemplate"](args.data);
                $("#" + proxy._id + "_dialogColumnAdd").html($(temp));
                var evntArgs = {};
                evntArgs.cssClass = this.model.cssClass,
                evntArgs.enableResize = false,
                evntArgs.width = "auto",
                evntArgs.height = "auto",
                evntArgs.contentSelector = "#" + proxy._id,
                evntArgs.rtl = proxy.model.rtl,
                evntArgs.allowKeyboardNavigation = false;
                var title = proxy._columnDialogTitle["insertColumn"];
                evntArgs.title = title;
                if (($(window).width() > proxy._responsiveMinWidth || !proxy.model.isResponsive) || proxy.model.isFromGantt) {
                    evntArgs.enableModal = true,
                    evntArgs.allowDraggable = true,
                    evntArgs.isResponsive = true;
                    $("#" + proxy._id + "_dialogColumnAdd").ejDialog(evntArgs);
                    $("#" + proxy._id + "_dialogColumnAdd").addClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").addClass("e-treegridadaptivedialog e-treegrid-dialog");
                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").find("div.e-dialog-scroller").addClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "_dialogColumnAdd").ejDialog("open");
                    $("#" + proxy._id + "_dialogColumnAdd").find(".e-dropdowntreegrid").css('width', '438px'); // fixed width of dialog dropdown treegrid div.
                }
                if (($(window).width() <= proxy._responsiveMinWidth && proxy.model.isResponsive) && !proxy.model.isFromGantt) {

                    evntArgs.isResponsive = false;
                    evntArgs.enableResize = false;
                    evntArgs.enableModal = false,
                    evntArgs.allowDraggable = false;
                    evntArgs.width = $("#" + proxy._id).width() + "px";
                    evntArgs.height = $("#" + proxy._id).height() + "px";
                    evntArgs.minHeight= "150px";
                    evntArgs.minWidth = "150px";
                    $("#" + proxy._id + "_dialogColumnAdd").removeClass("e-treegridadaptivedialog");
                    $("#" + proxy._id + "_dialogColumnAdd").find(".e-dropdowntreegrid").css({ 'width': ($("#" + proxy._id).width() - 80) + 'px', 'height': '150px' }); // 80px ((28px of $("#" + proxy._id + "_dialogColumnAdd"))+(18px of Hscrollbar)+(30px of $("#" + proxy._id + "ColumnAddForm div"))) get minused in width to place the dropdown dialog treegrid div in absolute position.
                        $("#" + proxy._id + "_dialogColumnAdd").ejDialog(evntArgs);
                        $("#" + proxy._id + "_dialogColumnAdd").ejDialog("open");
                        $("#" + proxy._id + "_dialogColumnAdd_wrapper").removeClass("e-treegridadaptivedialog").addClass("e-treegrid-dialog");
   
                $("#" + proxy._id + "_dialogColumnAdd_wrapper").find("div.e-dialog-scroller").removeClass("e-treegridadaptivedialog");

                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").css({ 'left': controlPosition .left + "px", 'top': controlPosition .top + "px" });
                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").removeClass("e-shadow");
                    $("#" + proxy._id + "_dialogColumnAdd").removeClass("e-shadow");
                    $("#" + proxy._id + "_dialogColumnAdd_wrapper").find("div.e-dialog-scroller").css({ 'height': ($("#" + proxy._id).height() - 50) + "px" }); // (48px - Dialog box header height) + (1px - excess border of parent div).

            }
                proxy._clearColumnMenu();
                proxy._refreshColumnAddForm();
                $("#" + proxy._id + "_dialogColumnAdd").ejDialog("refresh");
            }
        },

        setAddColumnFields: function () {
            var proxy = this,
                model = proxy.model;
            if (model.columnDialogFields.length) {
                proxy._addColumnFields = model.columnDialogFields;
                var tempFields = ["field", "headerText", "editType"];
                //Ensure the above 3 default fields with "model.columnDialogFields"
                for (var i = 0; i < tempFields.length; i++) {
                    if ($.inArray(tempFields[i], proxy._addColumnFields) == -1) {
                        switch (tempFields[i]) {
                            case "field":
                                proxy._addColumnFields.splice(0, 0, tempFields[i]);
                                break;
                            case "headerText":
                                proxy._addColumnFields.splice(1, 0, tempFields[i]);
                                break;
                            case "editType":
                                proxy._addColumnFields.splice(2, 0, tempFields[i]);
                                break;
                        }
                    }
                }
                //Check the "model.columnDialogFields" with allowSorting, allowFiltering and selectionType APIs
                if (model.isFromGantt) {
                    if (!model.allowSorting && proxy._addColumnFields.indexOf("allowSorting") != -1) {
                        proxy._addColumnFields.splice(proxy._addColumnFields.indexOf("allowSorting"), 1);
                    }
                    if (model.selectionSettings.selectionMode == "row" && proxy._addColumnFields.indexOf("allowCellSelection") != -1) {
                        proxy._addColumnFields.splice(proxy._addColumnFields.indexOf("allowCellSelection"), 1);
                    }
                }
                else {
                    if (!model.allowSorting && proxy._addColumnFields.indexOf("allowSorting") != -1) {
                        proxy._addColumnFields.splice(proxy._addColumnFields.indexOf("allowSorting"), 1);
                    }
                    if (!model.allowFiltering && proxy._addColumnFields.indexOf("allowFiltering") != -1) {
                        proxy._addColumnFields.splice(proxy._addColumnFields.indexOf("allowFiltering"), 1);
                    }
                    if (model.selectionSettings.selectionMode == "row" && proxy._addColumnFields.indexOf("allowCellSelection") != -1) {
                        proxy._addColumnFields.splice(proxy._addColumnFields.indexOf("allowCellSelection"), 1);
                    }
                }
                //Move displayAsCheckbox to end o hide this field in column add dialog, if edittype choosed ad boolean then it will be shown
                if (proxy._addColumnFields.indexOf("displayAsCheckbox") != -1) {
                    proxy._addColumnFields.push(proxy._addColumnFields.splice(proxy._addColumnFields.indexOf("displayAsCheckbox"), 1)[0]);
                }

                //Assign appropriate header text to fields
                proxy._addColumnFieldsTxt = [];
                for (var i = 0; i < proxy._addColumnFields.length; i++) {
                    switch (proxy._addColumnFields[i]) {
                        case "field":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["field"]);
                            break;
                        case "headerText":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["headerText"]);
                            break;
                        case "editType":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["editType"]);
                            break;
                        case "filterEditType":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["filterEditType"]);
                            break;
                        case "allowFiltering":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["allowFiltering"]);
                            break;
                        case "allowFilteringBlankContent":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["allowFilteringBlankContent"]);
                            break;
                        case "allowSorting":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["allowSorting"]);
                            break;
                        case "visible":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["visible"]);
                            break;
                        case "width":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["width"]);
                            break;
                        case "textAlign":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["textAlign"]);
                            break;
                        case "headerTextAlign":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["headerTextAlign"]);
                            break;
                        case "isFrozen":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["isFrozen"]);
                            break;
                        case "allowFreezing":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["allowFreezing"]);
                            break;
                        case "allowCellSelection":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["allowCellSelection"]);
                            break;
                        case "showInColumnChooser":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["showInColumnChooser"]);
                            break;
                        case "clipMode":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["clipMode"]);
                            break;
                        case "tooltip":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["tooltip"]);
                            break;
                        case "headerTooltip":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["headerTooltip"]);
                            break;
                        case "displayAsCheckbox":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["displayAsCheckbox"]);
                            break;
                        case "priority":
                            proxy._addColumnFieldsTxt.push(proxy._columnDialogTexts["priority"]);
                            break;
                    }                  
                }
            }
            else {
                proxy._addColumnFields = ["field", "headerText", "editType"];
                proxy._addColumnFieldsTxt = [proxy._columnDialogTexts["field"], proxy._columnDialogTexts["headerText"], proxy._columnDialogTexts["editType"]];
            }
        },

        //Method for creating cell edit type
        setColumnCelEditType: function (columnAttr) {
            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                $input;
            if (columnAttr == "field" || columnAttr == "filterEditType" || columnAttr == "editType" || columnAttr == "textAlign" || columnAttr == "headerTextAlign" || columnAttr == "clipMode") {
                $input = ej.buildTag('input.e-field e-dropdownlist',
                                         "",
                                         {},
                                         {
                                             value: "{{:#data['" + columnAttr + "']}}",
                                             id: proxy._id + columnAttr + "ColumnAdd",
                                             name: columnAttr,
                                            "data-dialog": "ColumnAdd"
                                         });
            }
            else if (columnAttr == "headerText" || columnAttr == "tooltip" || columnAttr == "headerTooltip") {
                $input = ej.buildTag('input.e-field e-ejinputtext',
                                         "",
                                         {},
                                         {
                                             value: "{{:#data['" + columnAttr + "']}}",
                                             id: proxy._id + columnAttr + "ColumnAdd",
                                             name: columnAttr,
                                            "data-dialog": "ColumnAdd"
                                         });
                $input.css({ 'margin': '0px', 'padding': '1px 4px 1px 0px' });
            }
            else if (columnAttr == "allowFiltering" || columnAttr == "allowSorting" || columnAttr == "isFrozen" || columnAttr == "allowFreezing") {
                $input = ' <input' +
                         ' type ="checkbox"' +
                         ' id="' + proxy._id + columnAttr + 'ColumnAdd"' +
                         ' data-dialog ="ColumnAdd"' +
                         ' name="' + columnAttr + '" data-edittype="booleanedit">';
            }
            else if (columnAttr == "allowFilteringBlankContent" || columnAttr == "visible") {
                $input = ' <input' +
                         ' type ="checkbox"' +
                         ' id="' + proxy._id + columnAttr + 'ColumnAdd"' +
                         ' data-dialog = "ColumnAdd"' +
                         ' checked = "checked"' +
                         ' name="' + columnAttr + '" data-edittype="booleanedit">';
            }
            else if (columnAttr == "allowCellSelection") {
                $input = ' <input' +
                         ' type ="checkbox"' +
                         ' id="' + proxy._id + columnAttr + 'ColumnAdd"' +
                         ' data-dialog = "ColumnAdd"' +
                         ' checked = "checked"' +
                         ' name="' + columnAttr + '" data-edittype="booleanedit">';
            }
            else if (columnAttr == "showInColumnChooser" || columnAttr == "displayAsCheckbox") {
                $input = ' <input' +
                         ' type ="checkbox"' +
                         ' id="' + proxy._id + columnAttr + 'ColumnAdd"' +
                         ' data-dialog = "ColumnAdd"' +
                         ' checked = "checked"' +
                         ' name="' + columnAttr + '" data-edittype="booleanedit">';
            }
            else if (columnAttr == "width" || columnAttr == "priority") {
                $input = ej.buildTag('input.e-numerictextbox e-field',
                                         "",
                                         {},
                                         {
                                             type: "text",
                                             value: "{{:#data['" + columnAttr + "']}}",
                                             id: proxy._id + columnAttr + "ColumnAdd",
                                             name: columnAttr,
                                             "data-dialog": "ColumnAdd"
                                         });
                $input.attr("data-edittype", "numericedit")
                $input.css({ 'margin': '0px', 'padding': '1px 4px 1px 0px' });
            }
            return $input;
        },

        //Template for adding custom column through dialog
        columnAddDialogTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                length = columns.length,
                $inputElem;

            if (length == 0)
                return;

            proxy.setAddColumnFields();

            var $tbody = ej.buildTag('div', "", {}, { 'data-unselectable': 'on' }),
            $form = ej.buildTag('form', "", { 'height': '100%', 'width': '100%', 'font-size': '14px' }, { id: proxy._id + "ColumnAddForm" }),
            $table = ej.buildTag('table', "", { width: "100%", "margin-top": "0px" }, { 'data-unselectable': "on" }),
            $tr = ej.buildTag('tr', "", { width: "100%" }),
            $td, $innerTable, $inTr, $inTd, $inTr2, $inTd2;
            for (var i = 0; i < proxy._addColumnFields.length; i++) {
                if ($tr.children("td").length < 2) {
                    $td = ej.buildTag('td.e-add-property-container', "", {}, { 'data-unselectable': "on" });
                    $innerTable = ej.buildTag('table', "", { "width": "100%", "outline": "none", "margin-top": "0px" }, { 'data-unselectable': "on" });

                    $inTr = ej.buildTag('tr');

                    $inTd = ej.buildTag('td.e-editLabel', "", { "outline": "none" }, { 'data-unselectable': "on" });
                    $inTd.append("<label style='font-weight:normal;'>" + proxy._addColumnFieldsTxt[i] + "</label>");

                    $inTr.append($inTd);

                    $innerTable.append($inTr);

                    $inTr2 = ej.buildTag('tr');
                    $inTd2 = ej.buildTag('td.e-editValue');

                    $inputElem = proxy.setColumnCelEditType(proxy._addColumnFields[i]);


                    $inTd2.html($inputElem);

                    $inTr2.append($inTd2);
                    $innerTable.append($inTr2);
                    if (proxy._addColumnFields[i] == "displayAsCheckbox") {
                        $innerTable.addClass("e-display-checkbox-conatiner").css("display","none");
                    }
                    $td.append($innerTable);
                    $tr.append($td);

                }
                else {
                    $table.append($tr);
                    $tr = ej.buildTag('tr');
                    i--;
                }

            }
            $table.append($tr);

          //  Table of Dropdown data for drop down edit type
            var $tr2 = ej.buildTag("tr"),
                $inTr3, $inTd3;

            if (model.showColumnChooser) {

                $td = ej.buildTag('td.e-add-property-container', "", {}, { colspan: 2, 'data-unselectable': "on" });

                $innerTable = ej.buildTag('table#' + proxy._id + 'dropdownDataTable', "", { "width": "100%", "outline": "none" }, { 'data-unselectable': "on" });

                $inTr3 = ej.buildTag('tr');
                $inTd3 = ej.buildTag('td.e-editLabel', "", { "outline": "none" }, { 'data-unselectable': "on" });
                $inTd3.append("<label for='dropdownData' style='font-weight:normal;'>" + proxy._columnDialogTexts["columnsDropdownData"] + "</label>");

                $inTr3.append($inTd3);
                $innerTable.append($inTr3);

                $inTr = ej.buildTag('tr');
                $inTd = ej.buildTag('td.e-editLabel', "", { "outline": "none" }, { 'data-unselectable': "on" });

                $inTd.append("<span class='e-addpre e-icon e-enable e-add-dialog' style='cursor:pointer; width: auto;'>" + proxy._columnDialogTexts["addData"] + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class='e-deletepre e-icon e-disable e-add-dialog' style='cursor:pointer; width: auto;'>&nbsp;&nbsp;" + proxy._columnDialogTexts["deleteData"] + "</span>");

                $inTr.append($inTd);
                $innerTable.append($inTr);

                $inTr2 = ej.buildTag('tr');
                $inTd2 = ej.buildTag('td.e-editValue');
                if ($("#" + proxy._id).width() <= proxy._responsiveMinWidth) {
                    var dropdownTreegridWidth = ($("#" + proxy._id).width() - 80) + 'px';
                    $inTd3 = ej.buildTag('div.e-dropdowntreegrid', "", { 'width': dropdownTreegridWidth, 'overflow':'hidden' });
                } else {
                    $inTd3 = ej.buildTag('div.e-dropdowntreegrid', "", { 'width': '438px', 'overflow': 'hidden' });
                }
                var $preDiv = ej.buildTag("div#treegrid" + proxy._id + "dropdownData", "", { "width": "100%", "height": "150px" }, {});
                $inTd3.append($preDiv);
                $inTd2.append($inTd3);

                $inTr2.append($inTd2);
                $innerTable.append($inTr2);

                $td.append($innerTable);
                $tr2.append($td);
                $table.append($tr2);
            }


            $form.append($table);

            $tbody = proxy.renderColumnAddDialogButton($form, $tbody);
            $.templates(proxy._id + "_ColumnAddTemplate", $tbody.html());
        },

        //button templates for column add dialog
        renderColumnAddDialogButton: function (form, tbody) {

            var btnId = "ColumnAddDialog_",
                proxy = this,
                model = proxy.model,
                $btnokDiv = ej.buildTag("div.e-add-dialog-btn-ok", "", {}, {}),
                $innerButokDiv = ej.buildTag("div", "", {}, {});
            var okbtn = ej.buildTag('input', "",
                 {
                     'border-radius': '3px'
                 },
                 { type: "button", id: btnId + proxy._id + "_Ok" }),
             okText = proxy._okButtonText;

            okbtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: okText,
                    width: "100%"
                });

            var $btncanDiv = ej.buildTag("div.e-add-dialog-btn-cancel", "", {}, {}),
            $innerButcanDiv = ej.buildTag("div", "", {}, {});
            var cancelText = proxy._cancelButtonText,
             cancelbtn = ej.buildTag('input', "",
                 {
                     'border-radius': '3px'
                 },
                 {
                     type: "button",
                     id: btnId + proxy._id + "_Cancel"
                 });

            cancelbtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: cancelText,
                    width: "100%"
                });

                var btnDiv = ej.buildTag('div', "", {}, { 'class': "e-treegrid-add-btn" });

            $innerButokDiv.append(okbtn)
            $btnokDiv.append($innerButokDiv);
            btnDiv.append($btnokDiv);
            $innerButcanDiv.append(cancelbtn)
            $btncanDiv.append($innerButcanDiv);
            btnDiv.append($btncanDiv);
            form.appendTo(tbody);
            btnDiv.appendTo(tbody);

            return tbody;
        },

        //button click event column add dialog
        _buttonClick: function (e) {

            if (e.keyCode != 0 && e.keyCode !== undefined && e.keyCode != 13)
                return true;

            var proxy = this,
                model = proxy.model, treeIndex = model.treeColumnIndex, setFrozen = false,
                dialogId = "ColumnAdd", $element, columnVals = {}, colFields = proxy._addColumnFields, columns = model.columns,
                insertIndex, args = {}, isAlreadyRender,filtertype;

            if (e.target.id == "ColumnAddDialog_" + proxy._id + "_Ok") {
                var formelement = document.getElementById(proxy._id + "ColumnAddForm"), ddl;
                for (var count = 0; count < colFields.length; count++) {
                    var colField = colFields[count];
                    $element = $(formelement).find("#" + proxy._id + colField + dialogId);
                    if (colField == "editType" || colField == "filterEditType" || colField == "textAlign" || colField == "headerTextAlign" || colField == "clipMode") {
                        ddl = $element.data("ejDropDownList");
                        columnVals[colField] = ddl._selectedValue;
                        //To get column's dropdown data if edit type is dropdownedit
                        if (colField == "editType" && columnVals[colField] == "dropdownedit") {
                            var tableTreeGrid = $(formelement).find("#treegrid" + proxy._id + "dropdownData"),
                                tableTreeGridObj = tableTreeGrid.data("ejTreeGrid"),
                                treeData = tableTreeGridObj.model.dataSource;
                            if (treeData.length > 0)
                                columnVals["dropdownData"] = treeData;
                        }
                    }
                    else if (colField == "showInColumnChooser" || colField == "allowCellSelection" || colField == "allowSorting" || colField == "allowFiltering" || colField == "allowFilteringBlankContent" || colField == "visible" || colField == "isFrozen" || colField == "allowFreezing" || colField == "displayAsCheckbox") {
                        var ch = $element.data("ejCheckBox");
                        columnVals[colField] = ch._isChecked;
                    }
                    else if ($element.val() == "" && (colField == "headerText" || colField == "field")) {
                        if (colField == "field")
                            columnVals[colField] = "Column" + (model.columns.length + 1);
                        else if (colField == "headerText")
                            columnVals[colField] = "Column " + (model.columns.length + 1);
                    }
                    else if (colField == "priority") {
                        columnVals[colField] = $element.ejNumericTextbox("getValue");
                    }
                    else
                        columnVals[colField] = $element.val();
                }
                if (proxy._insertPosition == "left")
                    insertIndex = proxy._targetColumnIndex;
                else if (proxy._insertPosition == "right")
                    insertIndex = proxy._targetColumnIndex + 1;
                //To shift and maintain treecolumn index
                if (insertIndex <= treeIndex)
                    model.treeColumnIndex = treeIndex + 1;
                //Inserting new columns between frozen columns
                if (!model.isFromGantt) {
                    if (insertIndex <= proxy._frozenColumnsLength - 1)
                        setFrozen = true;
                }
                //Inserting columns -  left or right
                var newColumnObject = {};
                if (model.isFromGantt) {
                    newColumnObject = {
                        field: columnVals.field,
                        headerText: columnVals.headerText,
                        editType: columnVals.editType,
                        allowSorting: ej.isNullOrUndefined(columnVals.allowSorting) ? null : columnVals.allowSorting,
                        visible: columnVals.visible == false ? false : true,
                        width: columnVals.width ? columnVals.width : 150,
                        textAlign: columnVals.textAlign ? columnVals.textAlign : "left",
                        headerTextAlign: columnVals.headerTextAlign ? columnVals.headerTextAlign : "left",
                        dropdownData: columnVals.dropdownData ? columnVals.dropdownData : null,
                        allowCellSelection: ej.isNullOrUndefined(columnVals.allowCellSelection) ? null : columnVals.allowCellSelection,
                        mappingName: columnVals.field,
                        displayAsCheckbox: columnVals.displayAsCheckbox == false ? false : true,
                        clipMode: columnVals.clipMode ? columnVals.clipMode : "clip",
                        tooltip: columnVals.tooltip,
                        headerTooltip: columnVals.headerTooltip
                    };
                }
                else {
                    newColumnObject = {
                        field: columnVals.field,
                        headerText: columnVals.headerText,
                        editType: columnVals.editType,
                        filterEditType: columnVals.filterEditType ? columnVals.filterEditType : "stringedit",
                        allowFiltering: ej.isNullOrUndefined(columnVals.allowFiltering) ? null : columnVals.allowFiltering,
                        allowFilteringBlankContent: ej.isNullOrUndefined(columnVals.allowFilteringBlankContent) ? null : columnVals.allowFilteringBlankContent,
                        allowSorting: ej.isNullOrUndefined(columnVals.allowSorting) ? null : columnVals.allowSorting,
                        visible: columnVals.visible == false ? false : true,
                        width: columnVals.width ? columnVals.width : 150,
                        textAlign: columnVals.textAlign ? columnVals.textAlign : "left",
                        headerTextAlign: columnVals.headerTextAlign ? columnVals.headerTextAlign : "left",
                        clipMode: columnVals.clipMode ? columnVals.clipMode : "clip",
                        tooltip: columnVals.tooltip,
                        headerTooltip: columnVals.headerTooltip,
                        isFrozen: (columnVals.isFrozen == true || setFrozen == true) ? true : false,
                        allowFreezing: ej.isNullOrUndefined(columnVals.allowFreezing) ? null : columnVals.allowFreezing,
                        dropdownData: columnVals.dropdownData ? columnVals.dropdownData : null,
                        allowCellSelection: ej.isNullOrUndefined(columnVals.allowCellSelection) ? null : columnVals.allowCellSelection,
                        showInColumnChooser: columnVals.showInColumnChooser == false ? false : true,
                        displayAsCheckbox: columnVals.displayAsCheckbox == false ? false : true,
                        priority: columnVals.priority ? columnVals.priority : "",
                    };
                }
                args.requestType = "insertColumn";
                args.columnObject = newColumnObject;
                args.insertIndex = insertIndex;
                if (!proxy._trigger("actionComplete", args)) {
                    columns.splice(args.insertIndex, 0, args.columnObject);
                    proxy._refreshFrozenColumns();
                    proxy._updateWrapRowHeights(null,"virtualRefresh");
                    if (model.isFromGantt)
                        proxy._updateAddEditDialogTemplate();
                }
                $("#" + proxy._id + "_dialogColumnAdd").ejDialog("close");
                if (model.allowFiltering && model.filterSettings.filterType == "menu") {
                    if (args.columnObject.filterEditType)
                        filtertype = args.columnObject.filterEditType;
                    else
                        filtertype = args.columnObject.editType;
                    var isAlreadyRender = proxy._renderedFilterMenuDialog.indexOf(filtertype);
                    if (isAlreadyRender == -1) {
                        proxy._renderFilterMenu(filtertype);
                    }
                }
                if (this.model.editSettings.allowEditing && this.model.editSettings.editMode == "dialogEditing" && (ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) || model.editSettings.dialogEditorTemplateID == "")) {
                    proxy._treeGridAddEditDialog();
                } else if (this.model.editSettings.allowEditing && this.model.editSettings.editMode == "dialogEditing" && (!ej.isNullOrUndefined(model.editSettings.dialogEditorTemplateID) && model.editSettings.dialogEditorTemplateID != "")) {
                    proxy._treeGridAddEditDialogTemplate();
                }
            }
            else if (e.target.id == "ColumnAddDialog_" + proxy._id + "_Cancel") {
                $("#" + proxy._id + "_dialogColumnAdd").ejDialog("close");
            }

            if (this.model.editSettings.editMode == "dialogEditing" || (model.editSettings.editMode.toLowerCase() == "batchediting" && model.editSettings.batchEditSettings.editMode == "dialog")) {
                if (e.target.id == "EditFormDialog_" + this._id + "_Save") {
                    if (!proxy.editFormValidate())
                        return true;
                    if (this.element.find(".e-dialogaddrow .e-treegridform").length > 0) {
                        if (proxy.model.allowSelection && proxy.model.selectionSettings.selectionType == "multiple" && this._multiSelectPopup && this._multiSelectPopup.is(":visible")) {
                            proxy._clearMultiSelectPopup();
                        }
                        if (proxy._treegridDialogAddRow()) {
                            proxy._isTreeAddEditDialogSave = true;
                            $("#" + this._id + "_dialogEdit").ejDialog("close");
                        }
                    } else if (this.element.find(".e-dialogeditedrow .e-treegridform").length > 0) {
                        if (!proxy.endEdit()) {
                            proxy._isTreeAddEditDialogSave = true;
                            $("#" + this._id + "_dialogEdit").ejDialog("close");
                        }
                    }
                }
                else if (e.target.id == "EditFormDialog_" + this._id + "_Cancel") {
                    $("#" + this._id + "_dialogEdit").ejDialog("close");
                }
            }

            return false;
        },

        // Method to save the newly added record via Add dialog or Add template dialog
        _treegridDialogAddRow: function () {
            var proxy = this, columns = proxy.model.columns, newItem = {}, value;
            for (var i = 0; i < columns.length; i++) {
                var column = columns[i], fieldName = column["field"];
                if (!ej.isNullOrUndefined(column) && fieldName != "") {
                    if (!proxy._isCommandColumn(column)) {
                        if (proxy._checkboxSelection && column["nonColumn"]) continue;
                        value = proxy.getCurrentEditCellDataForRowEdit(fieldName, column["editType"], i);
                        value = (ej.isNullOrUndefined(value) || (column.editType == ej.TreeGrid.EditingType.Numeric && isNaN(value))) ? null : value;
                        if (fieldName.indexOf(".") != -1)
                            ej.createObject(fieldName, value, newItem);
                        else
                            newItem[fieldName] = value;
                    }
                }
            }
            if (proxy._addRecord(newItem, proxy.model.editSettings.rowPosition, null, "dialog"))
                return true;
            else return false;
        },

        //REFRESH THE EDITED FORM
        _refreshColumnAddForm: function () {
            var proxy = this,
               $form = $("#" + proxy._id + "ColumnAddForm"),
               $formElement = $form.find("input,select"),
               i = 0,
               elementFocused = false,
               length = $formElement.length,
               $element, inputWidth,
               model = proxy.model,
               width,
               params = {},
               value,
               columnPpty,
               customParams,
               toformat,
               formatVal,
               tableTreeGridId,
               dropdownDataTable,
               allColFields = [],
               rowHeight = proxy.model.rowHeight,
               editTypeLocaleText = proxy._editTypeText,
               textAlignLocaleText = proxy._textAlignTypeText,
               clipModeLocaleText = proxy._clipModeText,
               cellEditType;


            for (i; i < length; i++) {

                $element = $formElement.eq(i);
                inputWidth = '100%';

                columnPpty = $element.prop("name");

                if ($element.hasClass("e-dropdownlist")) {
                    if (columnPpty == "field") {
                        var fieldData = [], reqFields = [];

                        if (model.columns.length) {
                            allColFields = $.map(model.columns, function (col) {
                                if (model.isFromGantt)
                                    return col.mappingName ? col.mappingName : col.field;
                                else
                                    return col.field;
                            });
                        }

                        //getting all the fields of datasource
                        if (!proxy._dataSourcefields.length) {
                            proxy._dataSourcefields = $.extend(true, [], allColFields);
                            proxy.getAllDataSourceFields();
                        }
                        //comaparing and taking the non column fields alone for add column dialog - fields
                        for (var k = 0; k < proxy._dataSourcefields.length; k++) {
                            if ($.inArray(proxy._dataSourcefields[k], allColFields) == -1 && proxy._dataSourcefields[k] != model.notesMapping)
                                if ((model.isFromGantt && proxy._dataSourcefields[k] != "serialNumber") || !model.isFromGantt) {
                                    reqFields.push(proxy._dataSourcefields[k]);
                                }
                             }

                        for (var fd = 0; fd < reqFields.length; fd++) {
                            fieldData.push({ "text": reqFields[fd] });
                        }
                        $element.ejDropDownList({
                            width: inputWidth,
                            cssClass: model.cssClass,
                            dataSource: fieldData,
                            fields: {
                                text: "text",
                                value: "text"
                            },
                        });
                    }
                    else if (columnPpty == "editType" || columnPpty == "filterEditType") {
                        var fieldData = [
                            {  "text": editTypeLocaleText['string'], "value": "stringedit" }, {  "text": editTypeLocaleText['numeric'], "value": "numericedit" },
                            {  "text": editTypeLocaleText['datePicker'], "value": "datepicker" }, {  "text": editTypeLocaleText['dateTimePicker'], "value": "datetimepicker" },
                            {  "text": editTypeLocaleText['dropdown'], "value": "dropdownedit" }, {  "text": editTypeLocaleText['boolean'], "value": "booleanedit" }
                        ];
                        $element.ejDropDownList({
                            width: inputWidth,
                            dataSource: fieldData,
                            cssClass: model.cssClass,
                            fields: {
                                text: "text",
                                value: "value"
                            },
                            selectedIndex: 0,
                            change: function (args) {
                                var isRefreshDialog = false;
                                if ($(dropdownDataTable).is(":hidden") == true && args.value == "dropdownedit") {
                                    $(dropdownDataTable).attr("hidden", false);
                                    isRefreshDialog = true;
                                }
                                else if ($(dropdownDataTable).is(":hidden") == false && args.value != "dropdownedit") {
                                    $(dropdownDataTable).attr("hidden", true);
                                    isRefreshDialog = true;
                                }

                                var checkBoxContainer = $("#" + proxy._id + "_dialogColumnAdd").find(".e-display-checkbox-conatiner");
                                if (args.value == "booleanedit" && checkBoxContainer.length > 0 && checkBoxContainer.css("display") == "none") {
                                    checkBoxContainer.css("display", "block");
                                    isRefreshDialog = true;
                                } else if (args.value != "booleanedit" && checkBoxContainer.length > 0 && checkBoxContainer.css("display") == "block") {
                                    checkBoxContainer.css("display", "none");
                                    isRefreshDialog = true;
                                }
                                if (isRefreshDialog) {
                                    $("#" + proxy._id + "_dialogColumnAdd").ejDialog("refresh");
                                }
                            }
                        });
                    }
                    else if (columnPpty == "textAlign" || columnPpty == "headerTextAlign") {
                        var fieldData1 = [
                            {  "text": textAlignLocaleText["left"], "value": "left" }, { "text": textAlignLocaleText["right"], "value": "right" },
                            {  "text": textAlignLocaleText["center"], "value": "center" }];

                        $element.ejDropDownList({
                            width: inputWidth,
                            dataSource: fieldData1,
                            cssClass: model.cssClass,
                            fields: {
                                text: "text",
                                value: "value"
                            },
                            selectedIndex: 0
                        });
                    } else if (columnPpty == "clipMode") {
                        var clipModeData = [
                                                    {  "text": clipModeLocaleText["clip"], "value": "clip" }, {  "text": clipModeLocaleText["ellipsis"], "value": "ellipsis" }];

                        $element.ejDropDownList({
                            width: inputWidth,
                            dataSource: clipModeData,
                            cssClass: model.cssClass,
                            fields: {
                                text: "text",
                                value: "value"
                            },
                            selectedIndex: 0
                        });
                    }
                }
                else if ($element.hasClass("e-numerictextbox")) {
                    value = $element.val();
                    params.width = inputWidth;
                    params.showSpinButton = true;
                    params.cssClass = model.cssClass;
                    if (columnPpty == "width") {
                        params.maxValue = 250;
                        params.minValue = 30;
                        if (value.length)
                            params.value = parseFloat(value);
                        else
                            params.value = 150;
                    }
                    else if (columnPpty == "priority")
                    {
                        params.maxValue = 6;
                        params.minValue = 1;
                        if (value.length)
                            params.value = parseFloat(value);
                        else
                            params.value = null;
                    }
                    $element.ejNumericTextbox(params);
                }
                else if (columnPpty == "showInColumnChooser" || columnPpty == "allowCellSelection" || columnPpty == "allowFiltering" || columnPpty == "allowFilteringBlankContent" || columnPpty == "allowSorting" || columnPpty == "visible" || columnPpty == "isFrozen" || columnPpty == "allowFreezing" || columnPpty == "displayAsCheckbox") {

                    var controlArgs = {};
                    controlArgs.cssClass = model.cssClass;
                    controlArgs.size = "small";
                    $element.ejCheckBox(controlArgs);
                }               

                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                    if (!proxy._isEnterKeyPressed) {
                        proxy._focusElements($element.closest('td'));
                        elementFocused = true;
                    }
                }
            }

            tableTreeGridId = "#treegrid" + proxy._id + "dropdownData";
            //Render dropdown data table for column add dialog 
            if ($(tableTreeGridId).length) {
                var ds = [];

                $(tableTreeGridId).ejTreeGrid({
                    dataSource: ds,
                    allowSorting: false,
                    allowAdding: true,
                    columns: [{ headerText: proxy._columnDialogTexts["dropdownTableText"], field: "text", editType: ej.TreeGrid.EditingType.String, width: "150px" },
                              { headerText: proxy._columnDialogTexts["dropdownTableValue"], field: "value", editType: ej.TreeGrid.EditingType.String, width: "150px" }],
                    enableAltRow: true,
                    allowColumnResize: true,
                    cssClass: model.cssClass,
                    enableResize:true,
                    editSettings: {
                        allowAdding: true,
                        allowDeleting: true,
                        allowEditing: true,
                        editMode: "cellEditing",
                    },
                    locale: model.locale,
                    treeColumnIndex: 5,
                    emptyRecordText: "Add values for dropdown",
                    rowSelected: function (args) {
                        if (args.data) {
                            $('.e-deletepre').addClass('e-enable').removeClass('e-disable');
                            $(".e-deletepre").bind("click", $.proxy(proxy._deleteDropDownRow, proxy, $(tableTreeGridId)));
                        }
                    },

                });
                //Hide the dropdown data table while initializing
                dropdownDataTable = $form.find("#" + proxy._id + "dropdownDataTable").parent()[0];
                $(dropdownDataTable).attr("hidden", true);
                //Enable the click event for Add icon
                $(".e-addpre").bind("click", $.proxy(proxy._addDropDownRow, proxy, $(tableTreeGridId)));
            }

        },

        // Add new row for the drop down table
        _addDropDownRow: function (tableTreeGrid, e) {
            var proxy = this, args = {},
                treeGridObj = tableTreeGrid.data("ejTreeGrid"),
                treeData = treeGridObj.model.dataSource,
                dlength = treeData.length,
                item = { text: "Text " + (dlength + 1), value: "Value" + (dlength + 1) };
            treeGridObj.addRow(item, "bottom");
        },
        // delete row from the drop down table
        _deleteDropDownRow: function (tableTreeGrid, e) {
            var proxy = this, args = {},
                treeGridObj = tableTreeGrid.data("ejTreeGrid"),
                treeData = treeGridObj.model.updatedRecords,
                dlength = treeData.length,
                selectedItem = treeGridObj.model.selectedItem;
            if (dlength != 0 && selectedItem) {
                treeGridObj.deleteRow();
                $('.e-deletepre').addClass('e-disable').removeClass('e-enable');
                $(".e-deletepre").unbind("click", $.proxy(proxy._deleteDropDownRow, proxy, tableTreeGrid));
            }

        },

        //get all the fields of the dataSource
        getAllDataSourceFields: function () {
            var proxy = this, model = proxy.model,
                dataSourceCol = model.dataSource;
            for (var d = 0; d < dataSourceCol.length; d++) {
                var jsonData = dataSourceCol[d];
                for (var dsKey in jsonData) {
                    if (dsKey == model.childMapping) {
                        proxy.getChildKeys(jsonData[dsKey]);
                    }
                    else if ($.inArray(dsKey, proxy._dataSourcefields) == -1)
                        if (model.isFromGantt) {
                            if (dsKey != "undefined" && dsKey != model.notesMapping)
                                proxy._dataSourcefields.push(dsKey);
                        }
                        else {
                            if (typeof jsonData[dsKey] != "object" && dsKey != "ejvalue")
                            proxy._dataSourcefields.push(dsKey);
                        }
                }
            }
        },

        //get keys of child from datasource.
        getChildKeys: function (childData) {
            var proxy = this, model = proxy.model;
            if (childData) {
                for (var c = 0; c < childData.length; c++) {
                    var chData = childData[c];
                    for (var childKey in chData)
                        if (childKey == model.childMapping) {
                            proxy.getChildKeys(chData[childKey]);
                        }
                        else if ($.inArray(childKey, proxy._dataSourcefields) == -1) {
                            if (typeof chData[childKey] != "object" && childKey != "ejvalue")
                            proxy._dataSourcefields.push(childKey);
                        }
                }
            }
        },

        //Method to render confirm dialog box while saving and cancelling and deleting
        _renderUpdateConfirmDialog: function () {
            var proxy = this,
                confDelTxt = proxy._confirmDeleteText,
                $contentDiv = ej.buildTag('div.e-content', proxy._deleteColumnText),
                $buttons = ej.buildTag('span.e-buttons', "<input class='e-confirm-dialog-btn-ok' type='button' id=" + proxy._id + 'ConfirmDialogOK' + " value=" + proxy._okButtonText + " /> "
                + "<input class='e-confirm-dialog-btn-cancel' type='button' id=" + proxy._id + 'ConfirmDialogCancel' + " value='" + proxy._cancelButtonText + "' />"),
                btnDiv = ej.buildTag('div', "", {}, { 'class': "e-confirm-dialog-btn" });
            btnDiv.append($buttons);
            proxy._updateConfirmDialog = ej.buildTag('div', '', '', { 'title': confDelTxt, 'id': proxy._id + 'ConfirmDialog' });
            proxy._updateConfirmDialog.append($contentDiv).append(btnDiv);
            proxy.element.append(proxy._updateConfirmDialog);
            $buttons.find("input").ejButton({
                cssClass: proxy.model.cssClass,
                showRoundedCorner: true,
                size: "normal",
                click: $.proxy(proxy._triggerUpdateConfirm, proxy)
            });
            proxy._updateConfirmDialog.ejDialog({ width: "auto", minHeight: 0, minWidth: 0, showOnInit: false, isResponsive: true, enableResize: false, enableModal: true, cssClass: proxy.model.cssClass });
            $("#" + proxy._id + "ConfirmDialog_wrapper").addClass("e-treegrid-dialog");
        },

        _triggerUpdateConfirm: function (args) {
            var proxy = this;
            if (args !== undefined && args.model.text == proxy._okButtonText) {
                if (proxy._updateConfirmDialog.find(".e-content").text() == proxy._deleteColumnText)
                    proxy.deleteColumn(proxy._targetColumnIndex);
            }
            proxy._updateConfirmDialog.ejDialog("close");
        },

        //Method to delete column
        deleteColumn: function (columnIndex) {
            var proxy = this, model = proxy.model, columns = model.columns, deletedColumn,
                sortedColumns = model.sortSettings && model.sortSettings.sortedColumns.length ? model.sortSettings.sortedColumns : null,
                column = columns[columnIndex], args = {};
            if (columnIndex == model.treeColumnIndex || columns[columnIndex].nonColumn) return;
            args.requestType="deleteColumn";
            args.columnObject = column;
            args.deleteIndex = columnIndex;
            if (!proxy._trigger("actionComplete", args)) {
                //Clear filter for deleted column
                if (!model.isFromGantt && model.allowSorting && model.filterSettings.filteredColumns.length > 0)
                    proxy.clearFilter(column.field);
                deletedColumn = columns.splice(columnIndex, 1);
                if (sortedColumns) {
                    var resultCol = $.grep(sortedColumns, function (sCol) {
                        if (model.isFromGantt)
                            return sCol.field == deletedColumn[0].mappingName;
                        else
                            return sCol.field == deletedColumn[0].field;
                    });
                    if (resultCol.length) {
                        sortedColumns.splice(sortedColumns.indexOf(resultCol[0]), 1);
                        proxy.processBindings();
                        if (model.isFromGantt) {
                            args.requestType = "sorting";
                            proxy._trigger("actionComplete", args)
                        }
                    }
                }
                if (columnIndex <= model.treeColumnIndex)
                    model.treeColumnIndex = model.treeColumnIndex - 1;
                if (model.showTotalSummary && model.collapsibleTotalSummary && columnIndex == proxy._summaryColumnIndex) {
                    proxy._summaryColumnIndex = 0;                    
                }
                proxy._refreshFrozenColumns();
                proxy._updateWrapRowHeights(null,"virtualRefresh");
                if (model.isFromGantt)
                    proxy._updateAddEditDialogTemplate();
            }
        },

        //Get the element Left and Top position

        getOffsetRect:function(elem) {
            var box = elem.getBoundingClientRect(),
                body = document.body,
                docElem = document.documentElement,
                scrollTop = window.pageYOffset || docElem.scrollTop || body.scrollTop,
                scrollLeft = window.pageXOffset || docElem.scrollLeft || body.scrollLeft,
                clientTop = docElem.clientTop || body.clientTop || 0 ,
                clientLeft = docElem.clientLeft || body.clientLeft || 0 ,
                top  = box.top +  scrollTop - clientTop ,
                left = box.left + scrollLeft- clientLeft;
            return { top: Math.round(top), left: Math.round(left) };
        },
        _moveToNextMenuItem: function (move) {

            var proxy,
              currentMenuItem,
              contextMenu, model = this.model, menuItem, parentMenuId, nextSubContextMenu, subContextMenu,
              nextMenuItem = {};
            if (this._isContextMenuItems || model.isFromGantt) {
                if (model.isFromGantt) {
                    proxy = $("#" + this._id.replace("ejTreeGrid", "")).data("ejGantt");
                    contextMenu = $("#" + this._id.replace("ejTreeGrid", "") + "_ContextMenu");
                }
                else {
                    proxy = this;
                    contextMenu = $("#" + this._id + "_ContextMenu");
                }
                menuItem = proxy._contextMenuItems
            }
            else {
                proxy = this;
                contextMenu = $("#" + this._id + "_headerContextMenu");
                menuItem = proxy._headerContextMenuItems
            }
            currentMenuItem = $("#" + proxy._activeMenuItemId);

            switch (move) {
                case "next":
                    if (currentMenuItem.length == 0) {
                        nextMenuItem = contextMenu.find("div.e-menuitem:not(.e-disable)").first();
                    } else {
                        nextMenuItem = currentMenuItem.closest("li").next("li").find("div.e-menuitem");
                        if (nextMenuItem.length > 0) {
                            if (nextMenuItem.hasClass("e-disable")) {
                                nextMenuItem = this._findContextMenuItem(nextMenuItem, "next");
                            }
                        }
                    }
                    nextSubContextMenu = $("#" + proxy._id + "_SubContextMenu" + proxy._activeMenuItemId);
                    nextSubContextMenu.remove();
                    break;
                case "prev":
                    nextMenuItem = this._findContextMenuItem(currentMenuItem, "prev");
                    nextSubContextMenu = $("#" + proxy._id + "_SubContextMenu" + proxy._activeMenuItemId);
                    nextSubContextMenu.remove();
                    break;
                case "expand":
                    if (currentMenuItem.length > 0) {
                        proxy._showSubContextMenu(currentMenuItem[0], menuItem);
                        subContextMenu = $("#" + proxy._id + "_SubContextMenu" + proxy._activeMenuItemId);
                        if (subContextMenu.length > 0) {
                            nextMenuItem = subContextMenu.find("div.e-menuitem:not(.e-disable)").first();
                        }
                    }
                    break;
                case "collapse":
                    subContextMenu = $(currentMenuItem).closest(".e-tginnerContextmenu");
                    if (subContextMenu.length > 0) {
                        parentMenuId = subContextMenu.attr("id").replace(proxy._id + "_SubContextMenu", "");
                        var nextMenuItem = $("#" + parentMenuId);
                        nextSubContextMenu = $("#" + proxy._id + "_SubContextMenu" + proxy._activeMenuItemId);
                        nextSubContextMenu.remove();
                        subContextMenu.remove();
                    }
                    break;
                case "save":
                    if (currentMenuItem.length > 0) {
                        if ($("#" + proxy._activeMenuItemId).find(".e-expander").length == 0) {
                            if (!model.isFromGantt)
                                proxy.contextMenuOperations(proxy._activeMenuItemId);
                            else
                                proxy._contextMenuClickHandler($("#" + proxy._activeMenuItemId));
                            proxy._clearContextMenu();
                        } else {
                            proxy._showSubContextMenu(currentMenuItem[0],menuItem);
                            subContextMenu = $("#" + proxy._id + "_SubContextMenu" + proxy._activeMenuItemId);
                            if (subContextMenu.length > 0) {
                                nextMenuItem = subContextMenu.find("div.e-menuitem:not(.e-disable)").first();
                            }
                        }
                    }
            }

            if (nextMenuItem.length > 0) {
                var currentClassName = this._isContextMenuItems || model.isFromGantt ? "e-tgcontextmenu-mouseover" : "e-columnmenuselection";
                if ((move == "next" || move == "prev")) 
                    currentMenuItem.removeClass(currentClassName);
                    nextMenuItem.addClass(currentClassName);
                    proxy._activeMenuItemId = $(nextMenuItem).attr("id");

                    
                }
               
        },
        /*Get Next active columnMenu item*/
        _findNextColumnMenuItem: function (item, move,isFromColumnMenu) {
            var proxy = this, nextItem, prevItem;
                nextItem = isFromColumnMenu ? item.closest("li").next("li").find("div.e-columnmenuitem") : item.closest("li").next("li").find("div.e-menuitem ");
            prevItem = isFromColumnMenu ? item.closest("li").prev("li").find("div.e-columnmenuitem") : item.closest("li").prev("li").find("div.e-menuitem");
           
            if (move === "next") {
               
                if (nextItem.length > 0) {
                    if (!nextItem.hasClass("e-disable")) {
                        return nextItem;
                    }
                    else
                        if (isFromColumnMenu) {
                            return proxy._findNextColumnMenuItem(nextItem, "next", true);
                        }
                        else {
                            return proxy._findNextColumnMenuItem(nextItem, "next",false)
                        }
                }
                else
                    return nextItem;
            }
            else {
              
                if (prevItem.length > 0) {
                    if (!prevItem.hasClass("e-disable")) {
                        return prevItem;
                    }
                    else
                        if (isFromColumnMenu) {
                            return proxy._findNextColumnMenuItem(prevItem, "prev", true);
                        }
                        else {
                            return proxy._findNextColumnMenuItem(prevItem, "prev", false);
                        }
                }
                else
                    return prevItem;
            }
        },
        // get concade field value
        _getFieldValue: function (field, columnCount) {

            if (field != "") {
                return field.split('.').join("");
            } else {
                return "TGcolumn" + columnCount;
            }

        },

        // Find the next context menu item to select
        _findContextMenuItem: function (item, move) {
            var proxy = this, nextItem, prevItem;
            if (move === "next") {
                nextItem = item.closest("li").next("li").find("div.e-menuitem");
                if (nextItem.length > 0) {
                    if (!nextItem.hasClass("e-disable")) {
                        return nextItem;
                    }
                    else
                        return proxy._findContextMenuItem(nextItem, "next");
                }
                else
                    return nextItem;
            }
            else {
                prevItem = item.closest("li").prev("li").find("div.e-menuitem");
                if (prevItem.length > 0) {
                    if (!prevItem.hasClass("e-disable")) {
                        return prevItem;
                    }
                    else
                        return proxy._findContextMenuItem(prevItem, "prev");
                }
                else
                    return prevItem;
            }
        },
        updateRecordByIndex: function (index, data) {
            var proxy = this, model = proxy.model,
                updatedRecords = model.updatedRecords, args = {},
                recordIndex = index,
                record = recordIndex > -1 ? updatedRecords.length > 0 ? updatedRecords[recordIndex] : null : null;

            if (proxy._isRowEdit)
                proxy.cancelRowEditCell();
            else if (proxy.model.isEdit)
                proxy.cancelEditCell();

            if (!ej.isNullOrUndefined(record) && !record.isSummaryRow) {
                var item = record.item,
                previousValue = $.extend(true, {}, item),
                columns = $.extend(true, {}, model.columns);
                $.each(data, function (key, value) {
                    var fieldName = key, fieldValue = value;
                    if (!ej.isNullOrUndefined(fieldName)) {
                        var isValidColumn = false;
                        $.each(columns, function (index, column) {
                            if (column.field == fieldName) {
                                isValidColumn = true;
                                delete columns[parseInt(index)];
                                return false;
                            }
                        });

                        if (isValidColumn) {
                            if (model.editSettings.editMode == "batchEditing") {
                                var column = {},
                                    currentValue = ej.copyObject({}, record.item),
                                columnObj = model.columns.filter(function (key) {
                                    if (key.field == fieldName) {
                                        column = key;
                                    }
                                });
                                if (column.editType == ej.TreeGrid.EditingType.DatePicker || column.editType == ej.TreeGrid.EditingType.DateTimePicker) {
                                    fieldValue = ej.parseDate(fieldValue, model.dateFormat, model.locale);
                                }
                                if (proxy._checkModfiedCellValue(ej.getObject(fieldName, currentValue), fieldValue, column)) {
                                    proxy._createBatchEditedFields(record, fieldName, fieldValue, column);
                                    proxy._batchEditStarts = true;
                                    proxy._updateBatchChanges(record.item);
                                }
                            }
                            record = proxy._restoreCustomData(record, fieldValue, fieldName);
                            record.item = proxy._restoreCustomData(record.item, fieldValue, fieldName);
                        }
                    }
                });

            }
            else
                return false;

            var rowIndex = model.currentViewData.indexOf(record);

            if (model.enableVirtualization) {
                if (rowIndex > -1)
                    ej.TreeGrid.refreshRow(proxy, rowIndex);
            } else {
                ej.TreeGrid.refreshRow(proxy, rowIndex);
            }
            if (model.idMapping && model.parentIdMapping) {
                proxy._updateLastInsertedId(record[model.idMapping]);
            }
            args.editType = "rowedit";
            args.previousValue = previousValue;
            args.currentValue = record.item;
            args.rowIndex = recordIndex;

            if (model.showSummaryRow)
                proxy._updateSummaryRow(args);
            if (model.showTotalSummary)
                proxy._updateTotalSummaryRow(args);
            if (model.editSettings.editMode == "batchEditing") {
                proxy._cancelSaveTools();
            }
        },
        _saveEditedElements: function () {
            var proxy = this,
                model = proxy.model;
            if (model.isEdit)
                proxy.saveCell();
            else if (proxy._isRowEdit)
                proxy.saveRow();
        }
    });

    //ENUM FOR EACH ACTION IN TREEGRID
    ej.TreeGrid.Actions = {
        Sorting: "sorting",
        BeginEdit: "beginedit",
        Save: "save",
        Add: "add",
        Delete: "delete",
        Cancel: "cancel",
        Refresh: "refresh",
        Searching: "searching",
        ExpandCollapse: "expandcollapse",
        Selection: "selection",
        rowHover: "rowHover",
        Scroll: "scroll",
        ContextMenuAdd: "contextMenuAdd",
        RefreshDataSource: "refreshDataSource",
        DragAndDrop: "dragAndDrop",
        Paging: "paging",
        Filter: "filtering",
        RowIndexUpdate: "rowIndexUpdate"
    };

    //ENUM FOR SELECTION MODE IN TREEGRID
    ej.TreeGrid.SelectionType = {
        Single: "single",
        Multiple: "multiple",
        Checkbox: "checkbox"
    };

    //ENUM FOR FILTER TYPE IN TREEGRID
    ej.TreeGrid.FilterType = {
        FilterBar: "filterbar",
        Menu: "menu",
        Excel:"excel"
    };


    //ENUM FOR FILTER HIERARCHY MODE IN TREEGRID
    ej.TreeGrid.FilterHierarchyMode = {
        Parent: "parent",
        Child: "child",
        Both: "both",
        None: "none"
    };

    //ENUM FOR SEARCH HIERARCHY MODE IN TREEGRID
    ej.TreeGrid.SearchHierarchyMode = {
        Parent: "parent",
        Child: "child",
        Both: "both",
        None: "none"
    };

    //ENUM FOR HEADERTEXTOVERFLOW IN TREEGRID
    ej.TreeGrid.HeaderTextOverflow = {
        None: "none",
        Wrap:"wrap"
    };

    //ENUM FOR DIFFERENT EDIT MODE IN TREEGRID
    ej.TreeGrid.EditMode = {
        CellEditing: "cellEditing",
        RowEditing: "rowEditing",
        DialogEditing: "dialogEditing",
        BatchEditing: "batchEditing",
    };

    //ENUM FOR DIFFERENT BATCH EDIT MODE IN TREEGRID
    ej.TreeGrid.BatchEditMode = {
        Cell: "cell",
        Row: "row",
        Dialog: "dialog"
    };

    //ENUM FOR DIFFERENT WRAP MODE IN TREEGRID
    ej.TreeGrid.ClipMode = {
        Clip: "clip",
        Ellipsis: "ellipsis"
    };

    //ENUM FOR DIFFERENT PAGE MODE IN TREEGRID
    ej.TreeGrid.PageSizeMode = {
        All: "all",
        Root: "root",
    };

    ej.TreeGrid.PrintMode = {
        AllPages: "allPages",
        CurrentPage: "currentPage",
    };

    //Enum for position of new added row.
    ej.TreeGrid.RowPosition = {
        Top: "top",
        Bottom: "bottom",
        Above: "aboveSelectedRow",
        Below: "belowSelectedRow",
        Child : "child"
    }

    //Enum for Edit Action of TreeGrid.
    ej.TreeGrid.BeginEditAction = {
        DblClick: "dblClick",
        Click: "click"
    };
    /* enum for six different columnEditType for treegrid */
    ej.TreeGrid.EditingType = {
        String: "stringedit",
        Boolean: "booleanedit",
        Numeric: "numericedit",
        Dropdown: "dropdownedit",
        DatePicker: "datepicker",
        DateTimePicker: "datetimepicker",
        Maskedit : "maskedit"
    };
    /* enum for different context menu in treegrid */
    ej.TreeGrid.ContextMenuItems = {
        Add: "add",
        Edit: "edit",
        Delete: "delete",
    };
    //enum for column width mode
    ej.TreeGrid.ColumnResizeMode = {
        Normal: "normal",
        FixedColumns: "fixedcolumns",
        NextColumn: "nextcolumn"
    }
    //commands
    ej.TreeGrid.UnboundType = {
        /** Used to specify unbound type as Edit   */
        Edit: "edit",
        /** Used to specify unbound type as Save   */
        Save: "save",
        /** Used to specify unbound type as Delete   */
        Delete: "delete",
        /** Used to specify unbound type as Cancel   */
        Cancel: "cancel"
    };
    ej.TreeGrid = ej.TreeGrid || {};


    //INITIALIZE THE CELLEDIT ELEMENT FOR DIFFERENT TYPES OF EDITORS
    ej.TreeGrid._initCellEditType = function (instance, $element, id, columnCount, dialogId, isResourceView, ganttColumns) {

        var proxy = instance,
            model = proxy.model,
            columns = isResourceView ? ganttColumns : model.columns,
            column = columns[columnCount],
            dialogId = dialogId ? dialogId : "", splits = (columns[columnCount].field || "").split("."), columnName;

        if (columns[columnCount]["editTemplate"])
            columns[columnCount]["editType"] = "edittemplate";
        else if (ej.isNullOrUndefined(columns[columnCount]["editType"])) {
            column["editType"] = "stringedit";
        }

        if (proxy.model.editSettings.editMode == "dialogEditing") {
            columnName = columns[columnCount].field;
        } else {
            columnName = proxy._getFieldValue(columns[columnCount].field, columnCount);
        }

        switch (column["editType"]) {

            case "stringedit":

                if (model.isFromGantt) {

                    var helpers = {};

                    if (column.field === "predecessor") {
                        helpers["_" + id + "predecessorCell"] = $.proxy(proxy._getPredecessorsValue, proxy);
                    }

                    $.views.helpers(helpers);


                    if (column.field === "predecessor") { //get the predecessor value from item of GanttRecord

                        var $input = ej.buildTag('input.e-field e-ejinputtext',
                                       "", {},
                                       {
                                           value: "{{:~_" + id + "predecessorCell(#data)}}",
                                           id: id + column.field + dialogId,
                                           name: column.field,
                                           "data-dialog": dialogId
                                       });

                    } else if (column.field === "duration") { //get the predecessor value from item of GanttRecord
                        var helpers = {};
                        helpers["_" + id + "getDurationStringValue"] = $.proxy(ej.Gantt._getDurationStringValue, proxy);
                        $.views.helpers(helpers);
                        var $input = ej.buildTag('input.e-field e-ejinputtext',
                                       "", {},
                                       {
                                           value: "{{:~_" + id + "getDurationStringValue(#data)}}",
                                           id: id + column.field + dialogId,
                                           name: column.field,
                                           "data-dialog": dialogId
                                       });

                    }
                    else { /*get the cellValue if the column does not exist in the GanttRecord object and
                           present in the item of GanttRecord*/
                        $input = ej.buildTag('input.e-field e-ejinputtext',
                                            "",
                                            {},
                                            {
                                                value: "{{:~_" + proxy._id + "cellValue(#data,'" + columnName + "')}}",
                                                id: id + columns[columnCount].field + dialogId,
                                                name: columns[columnCount].field,
                                                "data-dialog": dialogId
                                            });

                    }

                } else { /*Get the cellvalue of the normal gridcell*/
                    $input = ej.buildTag('input.e-field e-ejinputtext',
                                          "",
                                          {},
                                          {
                                              value: "{{:~_" + proxy._id + "cellValue(#data,'" + columnName + "')}}",
                                              id: id + proxy._getFieldValue(column.field,columnCount) + dialogId,
                                              name: column.field,
                                              autocomplete: "off",
                                              "data-dialog": dialogId
                                          });
                }
                $input.attr("data-edittype", columns[columnCount].editType);
                $element.html($input);
            
                break;

            case "maskedit":
                $input = ej.buildTag('input.e-field e-maskedit', "", {},
                                         {
                                             value: "{{:~_" + proxy._id + "cellValue(#data,'" + columnName + "')}}",
                                             id: id + proxy._getFieldValue(column.field,columnCount) + dialogId,
                                             name: column.field,
                                             "data-dialog": dialogId
                                         });
                $input.attr("data-edittype", columns[columnCount].editType);
                $element.html($input);
                break;

            case "booleanedit":
                $element.html('{{if true===~_' + proxy._id + 'cellValue(#data,"' + columnName + '")}}' +
                         '<input type ="checkbox" ' +
                         'id="' + id + proxy._getFieldValue(columns[columnCount].field,columnCount) + dialogId +
                         '" data-dialog ="' + dialogId +
                         '" name="' + columns[columnCount].field +
                         '" checked="checked" data-edittype="' + columns[columnCount].editType + '"></input>' +
                         ' {{else}}' +
                         '<input' +
                         ' type ="checkbox"' +
                         ' id="' + id + proxy._getFieldValue(columns[columnCount].field,columnCount) + dialogId +
                         '" data-dialog ="' + dialogId +
                         '" name="' + columns[columnCount].field + '" data-edittype="' + columns[columnCount].editType + '">' +
                         '{{/if}}');

                break;

            case "numericedit":

                var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, {
                    type: "text",
                    value: "{{:~_" + proxy._id + "cellValue(#data,'" + columnName + "')}}",
                    id: id + proxy._getFieldValue(columns[columnCount].field,columnCount) + dialogId,
                    name: columns[columnCount].field,
                    "data-dialog": dialogId
                });
                $numericText.attr("data-edittype", columns[columnCount].editType);
                $element.append($numericText);

                break;

            case "datepicker":

                var $datePicker = ej.buildTag('input.e-datepicker e-field', "",
                                             {},
                                             {
                                                 type: "text",
                                                 value: "{{:~_" + proxy._id + "cellValue(#data,'" + columnName + "')}}",
                                                 id: id + proxy._getFieldValue(columns[columnCount].field,columnCount) + dialogId,
                                                 name: columns[columnCount].field,
                                                 "data-dialog": dialogId
                                             });
                $datePicker.attr("data-edittype", columns[columnCount].editType);
                $element.append($datePicker);

                break;
            case "datetimepicker":

                var $datePicker = ej.buildTag('input.e-datetimepicker e-field', "",
                                             {},
                                             {
                                                 type: "text",
                                                 value: "{{:~_" + proxy._id + "cellValue(#data,'" + columnName + "')}}",
                                                 id: id + proxy._getFieldValue(columns[columnCount].field,columnCount) + dialogId,
                                                 name: columns[columnCount].field,
                                                 "data-dialog": dialogId
                                             });
                $datePicker.attr("data-edittype", columns[columnCount].editType);
                $element.append($datePicker);

                break;
            case "dropdownedit":

                var $dropDownList = ej.buildTag('input.e-field e-dropdownlist',
                    "",
                    {},
                    {
                        type: "text",
                        id: id + proxy._getFieldValue(columns[columnCount].field,columnCount) + dialogId,
                        name: columns[columnCount].field,
                        "data-dialog": dialogId
                    });
                $dropDownList.attr("data-edittype", columns[columnCount].editType)
                    .attr("data-cellvalue", "{{:~_" + proxy._id + "cellValue(#data,'" + columnName + "')}}");
                $element.append($dropDownList);

                break;
            case "edittemplate":
                var temp = columns[columnCount].editTemplate.create;
                if (typeof temp == "string") {
                    var temp1 = ej.util.getObject(temp, window);
                    if (!$.isFunction(temp1)) {
                        if ($(temp).length == 1 && $(temp).get(0).tagName == "SCRIPT")
                            var $edittemplate = $($(temp).html()).attr({ "id": id + proxy._getFieldValue(column.field,columnCount), "name": column.field });
                        else
                            var $edittemplate = $(temp).attr({ "id": id + proxy._getFieldValue(column.field,columnCount), "name": column.field });
                    }
                    else
                        var $edittemplate = $(temp1()).attr({ "id": id + proxy._getFieldValue(column.field,columnCount), "name": column.field });
                }
                else
                    var $edittemplate = $(temp()).attr({ "id": id + proxy._getFieldValue(column.field,columnCount), "name": column.field });
                $edittemplate.attr("data-edittype", columns[columnCount].editType);
                $element.append($edittemplate);
                break;
        }
    };
    
    //Initialize tree grid toolbar item.
    ej.TreeGrid.ToolbarItems = {
        Add: "add",
        Edit: "edit",
        Delete: "delete",
        Update: "update",
        Cancel: "cancel",
        ExpandAll: "expandAll",
        CollapseAll: "collapseAll",
        PdfExport: "pdfExport",
        ExcelExport: "excelExport",
        Print: "print",
        Search: "search"
    };

    //Enum Values of summaryType
    ej.TreeGrid.SummaryType = {
        Sum: "sum",
        Average: "average",
        Maximum: "maximum",
        Minimum: "minimum",
        Count: "count",
        MinimumDate: "minimumDate",
        MaximumDate: "maximumDate",
        TrueCount: "trueCount",
        FalseCount: "falseCount",
        Custom: "custom"
    };

    //Enum Value of selectionMode.
    ej.TreeGrid.SelectionMode = {
        Row: "row",
        Cell: "cell"
    };

    ej.TreeGrid.Locale = ej.TreeGrid.Locale || {};

    ej.TreeGrid.Locale["default"] = ej.TreeGrid.Locale["en-US"] = {
        
        //string to be displayed in Toolbox's tooltip 
        toolboxTooltipTexts: {
            addTool: "Add",
            editTool: "Edit",
            updateTool: "Update",
            deleteTool: "Delete",
            cancelTool: "Cancel",
            expandAllTool: "Expand All",
            collapseAllTool: "Collapse All",
            pdfExportTool: "PDF Export",
            excelExportTool: "Excel Export",
            printTool: "Print",
            searchTool: "Search"
        },
        //string to be displayed in context menu 
        contextMenuTexts: {
            addRowText: "Add Row",
            editText: "Edit",
            deleteText: "Delete",
            saveText: "Save",
            cancelText : "Cancel",
            aboveText: "Above",
            belowText: "Below"
        },
        filterMenuTexts:{
            stringMenuOptions: [{ text: "Starts With", value: "startswith" },
                    { text: "Ends With", value: "endswith" },
                    { text: "Contains", value: "contains" },
                    { text: "Equals", value: "equal" },
                    { text: "Does Not Equal", value: "notequal" }],
            numberMenuOptions: [{ text: "Less Than", value: "lessthan" },
                        { text: "Greater Than", value: "greaterthan" },
                        { text: "Less Than Or Equal To", value: "lessthanorequal" },
                        { text: "Greater Than Or Equal To", value: "greaterthanorequal" },
                        { text: "Equals", value: "equal" },
                        { text: "Does Not Equal", value: "notequal" }],
            filterValue: "Filter Value",
            filterButton: "Filter",
            clearButton: "Clear",
            enterValueText: "enter value",
        },
        //string to be displayed in column menu 
        columnMenuTexts: {
            sortAscendingText: "Sort Ascending",
            sortDescendingText: "Sort Descending",
            columnsText: "Columns",
            freezeText: "Freeze",
            unfreezeText: "Unfreeze",
            freezePrecedingColumnsText: "Freeze Preceding Columns",
            insertColumnLeft: "Insert Column Left",
            insertColumnRight: "Insert Column Right",
            deleteColumn: "Delete Column",
            renameColumn: "Rename Column",
            menuFilter: "Filter"
        },

        //string to display in column add dialog 
        columnDialogTexts: {
            field: "Field",
            headerText: "Header Text",
            editType: "Edit Type",
            filterEditType: "Filter Edit Type",
            allowFiltering: "Allow Filtering",
            allowFilteringBlankContent: "Allow Filtering Blank Content",
            allowSorting: "Allow Sorting",
            visible: "Visible",
            width: "Width",
            textAlign: "Text Alignment",
            headerTextAlign: "Header Text Alignment",
            isFrozen: "Is Frozen",
            allowFreezing: "Allow Freezing",
            columnsDropdownData: "Column Dropdown Data",
            dropdownTableText: "Text",
            dropdownTableValue: "Value",
            addData: "Add",
            deleteData: "Remove",
            allowCellSelection: "Allow Cell Selection",
            showInColumnChooser: "Show In Column Chooser",
            clipMode: "Clip Mode",
            tooltip: "Tooltip",
            headerTooltip: "Header Tooltip",
            displayAsCheckbox: "Display As Checkbox",
            priority:"Priority"
        },

        //string to be displayed in column add dialog title 
        columnDialogTitle: {
            insertColumn: "Insert Column",
            deleteColumn: "Delete Column",
            renameColumn: "Rename Column"
        },

        //String to be display for edittype and filter edit type.
        editTypeTexts: {
            string: "String",
            numeric: "Numeric",
            datePicker: "Date Picker",
            dateTimePicker: "Date Time Picker",
            dropdown: "Dropdown",
            boolean: "Boolean"

        },

        // String to be display for text align type.
        textAlignTypes:{
            right: "Right",
            left: "Left",
            center: "Center"
        },

        // String to be display for clip mode type.
        clipModeTexts:{
            clip: "Clip",
            ellipsis: "Ellipsis"
        },

        //Locale Text for delete confirm dialog
        deleteColumnText: "Are you sure you want to delete this column?",
        deleteRecordText: "Are you sure you want to delete record?",
        okButtonText: "OK",
        cancelButtonText: "Cancel",
        confirmDeleteText: "Confirm Delete",

        //Locale Text for batch edit confirm dialog
        batchSaveConfirmText: "Are you sure you want to save changes?",
        batchSaveLostChangesText: "Unsaved changes will be lost. Are you sure you want to continue?",
        cancelEditText: "Are you sure you want to cancel the changes?",

        //String to be displayed in drop down list for filtering blank items
        dropDownListBlanksText: "(Blanks)",
        //String to be displayed in drop down list to clear filtered items
        dropDownListClearText: "(Clear Filter)",
        //Text to be displayed in drop down list for FALSE value in boolean edit type column
        trueText: "True",
        //Text to be displayed in drop down list for True value in boolean edit type column
        falseText: "False",
        emptyRecord: "No records to display",
    };

    //GET ROW CLASS NAME FOR EXPAND COLLAPSE ACTION
    ej.TreeGrid._getrowClassName = function () {

        var rowClass = "gridrowIndex",
            proxy = this;

        if (proxy.data.parentItem) {

            rowClass += proxy.data.parentItem.index.toString();

        }

        rowClass += "level";
        rowClass += proxy.data.level.toString();
        if (proxy.data.isSummaryRow) {
            rowClass += " ";
            rowClass += "e-summaryrow";
        }
        if (proxy.data.footerSummaryRowRecord) {
            rowClass += " ";
            rowClass += "e-footersummaryrow";
        }
        if (proxy.data.isFirstFooterSummary) {
            rowClass += " ";
            rowClass += "e-firstfootersummaryrow";
        }

        return rowClass;
    };
    ej.TreeGrid._getSummaryRowtdClassName = function () {
        var proxy = this;
        var tdClass = "";
        if (proxy.data.isSummaryRow) {
            tdClass += " ";
            tdClass += "e-summaryrowcell";
        }
        if (proxy.data.footerSummaryRowRecord) {
            tdClass += " ";
            tdClass += "e-footersummaryrowcell";
        }
        return tdClass;
    };

    //GET COLUMN BY FIELD NAME
    ej.TreeGrid.getColumnByField = function (columns, field) {

        var column = 0;

        for (column; column < columns.length; column++) {

            if (columns[column]["field"].split('.').join("") == field.split('.').join("")) break;

        }

        return column == columns.length ? null : columns[column];
    };


    ej.TreeGrid.getColumnByMappingName = function (columns, field) {
        var column = 0;
        for (column; column < columns.length; column++) {
            if (columns[column]["mappingName"] == field) break;
        }
        return column == columns.length ? null : columns[column];
    };

    //EXPAND COLLAPSE THE PARTICULAR RECORD OBJECT
    ej.TreeGrid.sendExpandCollapseRequest = function (element, args) {

        var proxy = element,
             model = proxy.model,
             toolbarItems = model.toolbarSettings.toolbarItems,
             updatedRecords;

        args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
        //To cancel the cell edit or row edit mode when expand the all record.
        if (proxy._isRowEdit) 
            proxy.cancelRowEditCell();
        else if (proxy.model.isEdit)
            proxy.cancelEditCell();

        if (model.enableVirtualization) {

            if (proxy._isFromGantt) {
                if (proxy._isLoadonDemand && !args.data.isChildRetrieved && args.expanded == true) {
                    ej.TreeGrid.expandRowsonDemand(proxy, args.data, args);
                    return;
                }
                else
                    proxy._$treegridHelper.ejTreeGrid("processBindings", args);
            }
            else if (proxy._isLoadonDemand && !args.data.isChildRetrieved && args.expanded == true) {
                ej.TreeGrid.expandRowsonDemand(proxy, args.data, args);
                return;
            }
            else {
                proxy.processBindings(args);
            }
            if (proxy._isFromGantt) {
                if (proxy._isInExpandCollapseAll === false) {
                    if (model.enableAltRow) {
                        proxy._$treegridHelper.ejTreeGrid("updateAltRow");
                    }
                    proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", args);
                    proxy._$treegridHelper.ejTreeGrid("updateHeight");
                    proxy.model.updatedGanttRecords = proxy.getUpdatedRecords();
                    proxy.model.currentViewData = proxy.getCurrentViewData();
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedGanttRecords);
                  
                }
            }
            else {
                if (proxy._isInExpandCollapseAll === false) {
                    if (model.enableAltRow) {
                        proxy.updateAltRow();
                    }
                    proxy.sendDataRenderingRequest(args);
                    proxy._updateWrapRowHeights("total,header");
                    proxy._setScrollTop();
                    proxy.updateHeight();
                }
            }
            
        } else {
            var record = args.data,
                index,
                $row,
                height = 0,
                isModified = record.expanded !== args.expanded;
                index = proxy.model.currentViewData.indexOf(record);
            
                if (proxy._frozenColumnsLength && proxy.getRows()) {
                    $row = $(proxy.getRows()[0][index]).add(proxy.getRows()[1][index]);
                }
                else if (proxy.getRows()) {
                    $row = proxy.getRows() && $(proxy.getRows()[index]);
                }

                if (args.expanded) {
                    record.expanded = args.expanded;
                    if (proxy._isLoadonDemand && !record.isChildRetrieved) {
                        ej.TreeGrid.expandRowsonDemand(proxy, record, args);
                        return;
                    }
                    else
                        $row && ej.TreeGrid.expandRecord(proxy, record, args.expanded, args);
                    if (record.hasFilteredChildRecords) {
                        $row && $row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
                        $row && $row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');
                    }
                    if (model.enableAltRow && proxy._isInExpandCollapseAll === false && !model.allowPaging) {
                        ej.TreeGrid.updateAltRow(proxy, record, index, 1);
                    }
                }
                else {
                    proxy._collapsedRecordCount = 0;//For update alt row
                    $row && ej.TreeGrid.collapseRecord(proxy, record, args.expanded);
                    record.expanded = args.expanded;
                    if (record.hasFilteredChildRecords) {
                        $row && $row.find(".e-treegridexpand").removeClass('e-treegridexpand').addClass('e-treegridcollapse');
                        $row && $row.removeClass('e-treegridrowexpand').addClass('e-treegridrowcollapse');
                    }
                    if (model.enableAltRow && proxy._isInExpandCollapseAll === false && proxy._collapsedRecordCount != 0 && !model.allowPaging)
                        ej.TreeGrid.updateAltRow(proxy, record, index, proxy._collapsedRecordCount);
                }

            var collapsedRecordCount= 0;
            if (proxy._isFromGantt) {
                proxy._$treegridHelper.ejTreeGrid("updateExpandStatus", record, args.expanded);
                proxy._$treegridHelper.ejTreeGrid("updateCollapsedRecordCount");
                collapsedRecordCount =  proxy._$treegridHelper.ejTreeGrid("getCollapsedRecordCount");
            } else {
                proxy.updateExpandStatus(record, args.expanded);
                proxy.updateCollapsedRecordCount();
                collapsedRecordCount = proxy.getCollapsedRecordCount();
            }

            if (model.allowPaging && (args.type == "ejTreeGridcollapsing" || args.type =="ejTreeGridexpanding")) {
                if (!ej.isNullOrUndefined(proxy.getPager())) {                    
                    proxy.gotoPage();
                }
                if (model.enableAltRow && proxy._isInExpandCollapseAll === false)
                    ej.TreeGrid.updateAltRow(proxy, record, index, 1);               
            }
            
            if (proxy._isFromGantt)
            {
                if (proxy._isInExpandCollapseAll === false) {
                    proxy._totalCollapseRecordCount = collapsedRecordCount;
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._$treegridHelper.ejTreeGrid("updateHeight");
                    height = proxy._$treegridHelper.ejTreeGrid("getRecordsHeight");
                    proxy._$ganttchartHelper.ejGanttChart("updateHeight", height);
                    proxy._$ganttchartHelper.ejGanttChart("setCollapsedRecordCount", collapsedRecordCount);
                    proxy._$ganttchartHelper.ejGanttChart("refreshGridLinesTable", proxy.model.updatedRecords.length - collapsedRecordCount);
                }
                
            } else {
                if (proxy._isInExpandCollapseAll === false) {
                    if (model.allowTextWrap && proxy._$frozenTableContent.length > 0) {
                        proxy._updateWrapRowHeights("total,header");
                        proxy._updateScrollCss();
                    } else {
                        proxy.updateHeight();
                    }
                }
            }
        }

        
        if (proxy._isFromGantt && proxy.model.predecessorMapping && proxy._isInExpandCollapseAll === false) {
            proxy._refreshConnectorLines(false, false, true);
        }

        if (proxy._isFromGantt) {
            var treeGrid = proxy._$treegridHelper.ejTreeGrid("instance");
            var chart = proxy._$ganttchartHelper.ejGanttChart("instance");
            var top = treeGrid._$gridContent.ejScroller("option", "scrollTop");
            chart._$bodyContainer.ejScroller("option", "scrollTop", top);
            var isHorizontalScroll = treeGrid.getScrollElement().ejScroller("isHScroll");
            if (!isHorizontalScroll)
                treeGrid.getScrollElement().scrollTop(top);
        }
       
        var argstemp = {};
        argstemp.recordIndex = args.recordIndex;
        argstemp.data = args.data;
        argstemp.requestType = args.requestType;
        argstemp.expanded = args.expanded;
        if (!ej.isNullOrUndefined(argstemp.data.item[model.expandStateMapping])) {
            argstemp.data[model.expandStateMapping] = args.expanded;
            argstemp.data.item[model.expandStateMapping] = args.expanded;
        }
        if (args.expanded) {
            proxy._trigger("expanded", argstemp);
        } else {
            proxy._trigger("collapsed", argstemp);
        }

        var selectItem = proxy.selectedItem();
        if (!proxy._isFromGantt && selectItem && (model.selectedItems.length == 1 || proxy._checkboxSelection)) {
            var expanded = proxy.getExpandStatus(selectItem);
            if (!expanded && !args.expanded) {
                proxy.selectRows(-1);
                proxy.selectedRowIndex(-1);
                selectItem.isSelected = false;
                model.selectedItem = null;
                selectItem = null;
            }
            if (model.allowPaging)
                updatedRecords = proxy._updatedPageData;
            else
                updatedRecords = proxy.model.updatedRecords;
            if (model.enableVirtualization && selectItem && updatedRecords.indexOf(selectItem) !== proxy.selectedRowIndex())
                proxy.selectRows(updatedRecords.indexOf(selectItem));
            proxy._cancelSaveTools();
        }
    };

    ej.TreeGrid.expandRowsonDemand = function (controlObject, parent, expcolargs) {
        var proxy = controlObject, model = proxy.model,         
            queryChildRecords = ej.Query().select(proxy._fieldNamesToQuery),
            flatRecords = model.flatRecords, updatedRecords = model.updatedRecords, args = {};
        if (proxy._isFromGantt) {
            var condition = ej.Predicate(model.parentTaskIdMapping, ej.FilterOperators.equal, parent.item[model.taskIdMapping]),
                idMapping = model.taskIdMapping, parentMapping = model.parentTaskIdMapping;
        }
        else {
            var condition = ej.Predicate(model.parentIdMapping, ej.FilterOperators.equal, parent.item[model.idMapping]),
                idMapping = model.idMapping, parentMapping = model.parentIdMapping;
        }

        if (parent.isChildRetrieved) {
            parent.expanded = true;
            ej.TreeGrid.sendExpandCollapseRequest(proxy, expcolargs);
            return true;
        }
        if (proxy._trigger("actionBegin", expcolargs))
            return true;

        queryChildRecords.where(condition);
        proxy._waitingPopupObj.show();
        queryChildRecords.requiresCount();
        var queryPromise = proxy.dataSource().executeQuery(queryChildRecords);
        queryPromise.done(ej.proxy(function (e) {
            var queriedChildItems = e.result, length = queriedChildItems.length, recordCollection = [], idCollection = [], jsonData = [],
                flatRecordIndex = flatRecords.indexOf(parent), childLength = parent.childRecords && parent.childRecords.length;
            //if (model.isFromGantt)
            //    var ganttObject = $("#" + proxy._id.replace("ejTreeGrid", "")).data("ejGantt");
            for (var index = 0; index < length; index++) {
                if (proxy._isFromGantt) {
                    var expandedChildRecord = proxy._createGanttRecord(queriedChildItems[index], parent.level + 1, parent);
                    if (model.enableWBS) {
                        var parentWBS = parent.WBS;
                        expandedChildRecord["WBS"] = parentWBS + "." + (index + 1);
                    }
                }
                else
                    var expandedChildRecord = proxy._createRecord(queriedChildItems[index], parent.level + 1, parent);
                expandedChildRecord.index = flatRecords.length + recordCollection.length;
                parent.childRecords.push(expandedChildRecord);

                //Update Child Record Info of Queried Child Items.
                if (expandedChildRecord.item[model.hasChildMapping] || proxy._assignedParendIDs.indexOf(expandedChildRecord.item[idMapping]) != -1) {
                    expandedChildRecord.hasChildRecords = true;
                    expandedChildRecord.childRecords = [];
                    expandedChildRecord.isChildRetrieved = false;
                    proxy._allParentItems.push(expandedChildRecord);
                    if (proxy._isFromGantt) {
                        expandedChildRecord.manualStartDate = expandedChildRecord.startDate;
                        expandedChildRecord.manualEndDate = expandedChildRecord.endDate;
                        expandedChildRecord.manualDuration = expandedChildRecord.duration;
                        if(!expandedChildRecord.isAutoSchedule)
                            proxy._updateManualParentItem(expandedChildRecord, null, true);
                    }
                }

                if (expandedChildRecord.item[idMapping])
                    idCollection.push(expandedChildRecord.item[idMapping]);

                recordCollection.push(expandedChildRecord);

                jsonData.push(queriedChildItems[index]);
            }

            //update Task Id Collection.
            proxy._taskIds.splice.apply(proxy._taskIds, [flatRecordIndex + 1, 0].concat(idCollection));
            //update gantt model Id Collection
            if (proxy._isFromGantt)
                model.ids.splice.apply(model.ids, [flatRecordIndex + 1, 0].concat(idCollection.toString().split(",")));
            //Update Record Collections
            flatRecords.splice.apply(flatRecords, [flatRecordIndex + 1, 0].concat(recordCollection));
            //updateJson Data
            if (proxy._isDataManagerUpdate)
                proxy._jsonData.splice.apply(proxy._jsonData, [flatRecordIndex + 1, 0].concat(jsonData));

            parent.isChildRetrieved = true;
            parent.expanded = true;
            var exargs = {
                recordIndex: flatRecordIndex,
                data: parent,
                requestType: "expandcollapse",
                expanded: true
            };
            if (expcolargs && (expcolargs.isExpandingParticularLevelParent || proxy._isInExpandCollapseAll)) {
                ej.TreeGrid.expandMultipleRecordsonDemand(proxy, parent, expcolargs, exargs);
            }
            else {
                proxy._renderExpandedChildRecords(expcolargs);
                proxy._trigger("expanded", exargs);
            }

        })).fail(ej.proxy(function (e) { proxy._triggerFailureEvent(e, proxy); }));          
    };

    ej.TreeGrid.expandMultipleRecordsonDemand = function (controlObject, parent, expcolargs, exargs) {
        var proxy = controlObject;
        if (expcolargs.isExpandingParticularLevelParent) {
            var collapsedParentItems = proxy._allParentItems.filter(function (record) {
                if (record.level == expcolargs.recordLevel && !record.expanded && record.hasChildRecords)
                    return record;
            });

            //Expand, if higher level parent is in collapsed state.
            var targetParentItem = parent.parentItem;
            for (var level = 0; level < expcolargs.recordLevel; level++) {
                if (targetParentItem && targetParentItem.isChildRetrieved)
                    targetParentItem.expanded = true;
                targetParentItem = targetParentItem && targetParentItem.parentItem;
            }
        }
        if (proxy._isInExpandCollapseAll) {
            var collapsedParentItems = proxy._allParentItems.filter(function (record) {
                if (!record.expanded && record.hasChildRecords)
                    return record;
            });
        }
        for (var targetItem = 0; targetItem < collapsedParentItems.length; targetItem++) {
            var exponDemand = {};
            exponDemand.data = collapsedParentItems[targetItem];
            exponDemand.recordIndex = proxy.model.updatedRecords.indexOf(collapsedParentItems[targetItem]);
            exponDemand.expanded = true;
            if (expcolargs.isExpandingParticularLevelParent) {
                exponDemand.recordLevel = expcolargs.recordLevel;
                exponDemand.isExpandingParticularLevelParent = true;
            }
            exponDemand.requestType = ej.TreeGrid.Actions.ExpandCollapse;
            var isExpanded = ej.TreeGrid.expandRowsonDemand(proxy, collapsedParentItems[targetItem], exponDemand);
            if (isExpanded) {
                proxy._allParentItems.splice(proxy._allParentItems.indexOf(collapsedParentItems[targetItem]), 1);                   
                collapsedParentItems.splice(collapsedParentItems.indexOf(collapsedParentItems[targetItem]), 1);
                if (collapsedParentItems.length > 0)
                    targetItem = -1;
            }
            else
                break;
        }
        if (collapsedParentItems.length == 0) {
            proxy._isInExpandCollapseAll = false;
            proxy._renderExpandedChildRecords(expcolargs);
            proxy._trigger("expanded", exargs);
        }
    },    

    //EXPAND THE RECORD
    ej.TreeGrid.expandRecord = function (controlObject, Record, expanded, args) {
  
        var proxy = controlObject,
            rowClassName = ".gridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            detailsRowClassName = ".detailsrowgridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            $rows = $(proxy.element).find(rowClassName),
            count = 0,
            length = $rows.length,
            row,
            index,
            record,
            model = proxy.model,
            $detailRows = $(proxy.element).find(detailsRowClassName),
            $expandedRows = $(proxy.element).find(rowClassName),
            index,
            $gridRows = proxy.getRows(),
            $row = $($gridRows[index]);

            index = model.currentViewData.indexOf(Record);

        if (Record.hasChildRecords && Record.childRecords.length > 0) {
            $row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
            $row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');
        }
        proxy._expandedRecordsCount += (proxy._isFromGantt || proxy._frozenColumnsLength > 0) ? $expandedRows.length / 2 : $expandedRows.length;
        $expandedRows.css({ 'display': 'table-row' });
        $detailRows.filter("tr.e-detailsrowexpanded").css('display', 'table-row');
        if (proxy._isFromGantt || proxy._frozenColumnsLength > 0) {
            length = length / 2;
        }

        for (count; count < length; count++) {
            row = $rows[count];
            index = -1;
            if (proxy._frozenColumnsLength > 0)
                index = $(proxy.getRows()[0]).index(row);
            else
                index = proxy.getRows().index(row);

            if (index !== -1) {
                record = proxy._isFromGantt ? proxy.model.updatedRecords[index] : proxy.model.updatedRecords[index];
                record.isExpanded = true;
                if (record.expanded)
                    ej.TreeGrid.expandRecord(proxy,record, expanded);
            }
        }
    },
    
    //COLLAPSE THE RECORD
    ej.TreeGrid.collapseRecord = function (controlObject, Record, expanded) {

        var proxy = controlObject,
            model = proxy.model,
            rowClassName = ".gridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            detailsRowClassName = ".detailsrowgridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            $rows = $(proxy.element).find(rowClassName),
            count = 0,
            length = $rows.length,
            row,
            index,
            record,
            $detailRows = $(proxy.element).find(detailsRowClassName),
            $collapsedRows = $(proxy.element).find(rowClassName),
        $gridRows = proxy.getRows();

        proxy._collapsedRecordCount += (proxy._isFromGantt || proxy._frozenColumnsLength > 0) ? $collapsedRows.length / 2 : $collapsedRows.length;
        $collapsedRows.css({ 'display': 'none' });
        $detailRows.css({ 'display': 'none' });

        if (proxy._isFromGantt || proxy._frozenColumnsLength > 0) {
            length = length / 2;
        }

        for (count; count < length; count++) {

            row = $rows[count];
            index = -1;
            if (proxy._frozenColumnsLength > 0)
                index = $(proxy.getRows()[0]).index(row);
            else
                index = proxy.getRows().index(row);


            if (index !== -1) {
                record = proxy.getUpdatedRecords()[index];

                if (record.expanded)
                    ej.TreeGrid.collapseRecord(proxy,record, expanded);
            }
        }
    },

    //UPDATE THE ALT ROW FUNCTION
    ej.TreeGrid.updateAltRow = function (controlObject, Record, recordIndex, offset) {

        var proxy = controlObject,
            count = 0,
            model = proxy.model,
            currentViewData = model.updatedRecords,          
            length = currentViewData.length,
            isAltRow = Record.isAltRow,
            record,
            $gridRows,
            $gridAllRows = proxy.getRows();

        if (proxy._frozenColumnsLength)
            $gridRows = proxy.getRows()[0];
        else
            $gridRows = proxy.getRows();

        /* for rowTemplate no need set alt row class*/
        if (model.rowTemplateID || model.altRowTemplateID)
            return;

        for (count = recordIndex + offset; count < length; count++) {

            if ($($gridRows[count]).hasClass("e-summaryrow"))
                continue;

            if ($($gridRows[count]).css('display') === "none")
                continue;

            record = currentViewData[count];
            if(count!=0)
            record.isAltRow = !isAltRow;
            isAltRow = record.isAltRow;
            var currentRow = {};
            if (proxy._frozenColumnsLength) {
                currentRow = $($gridAllRows[0][count]).add($gridAllRows[1][count]);
            }
            else
                currentRow = $($gridAllRows[count])
            if (isAltRow) {
                $(currentRow).addClass('e-alt-row');
            }
            else {
                $(currentRow).removeClass('e-alt-row');
            }
        }
    };

    //RERENDERED THE EDITED RECORD
    ej.TreeGrid.refreshRow = function (controlObject, index) {
        var proxy = controlObject,
            model = proxy.model,
            data = proxy.model.currentViewData[index], eventArgs = {},
            currentRefreshingRow, currentFrozenRow;
        if (data) {
            if (proxy._frozenColumnsLength > 0) {
                currentFrozenRow = $(proxy.getRows()[0][index]);
                currentRefreshingRow = $(proxy.getRows()[1][index]);
                if (model.showDetailsRow && model.detailsTemplate && data.isDetailsExpanded) {
                    var detailsRow = currentFrozenRow.next("tr");
                    if (detailsRow.hasClass("e-detailsrow")) {
                        detailsRow.remove();
                        currentRefreshingRow.next("tr").remove();
                    }
                }
                currentFrozenRow.replaceWith($($.render[proxy._id + "_JSONFrozenTemplate"](data)));
                currentRefreshingRow.replaceWith($($.render[proxy._id + "_Template"](data)));
            } else {
                currentRefreshingRow = $(proxy.getRows()[index]);
                if (model.showDetailsRow && model.detailsTemplate && data.isDetailsExpanded) {
                    var detailsRow = currentRefreshingRow.next("tr");
                    if (detailsRow.hasClass("e-detailsrow"))
                        detailsRow.remove();
            }
            if (!model.allowSelection)
                data.isSelected = false;
            currentRefreshingRow.replaceWith($($.render[proxy._id + "_Template"](data)));
            }
            proxy.setGridRows($(proxy.getContentTable().get(0).rows));

            proxy._gridRows = proxy.getContentTable().get(0).rows;
            if (proxy._frozenColumnsLength > 0)
                proxy._gridRows = [proxy._gridRows, proxy.getContentTable().get(1).rows];

            var rowElement = ej.TreeGrid.getRowByIndex(proxy, proxy.getUpdatedRecords().indexOf(model.currentViewData[index]));
            proxy._trigger("refresh");
            proxy._rowEventTrigger(rowElement, data);
        }
    };

    //Return corressponding treegrid rows jquery object
    ej.TreeGrid.getRowByIndex = function (controlObject, from, to) {

        try {
            var proxy;
            if (controlObject.pluginName == "ejTreeGrid")
                proxy = controlObject;
            else
                proxy = controlObject._$treegridHelper.data("ejTreeGrid");

            var $gridRows = proxy.getRows(),
                model = proxy.model,
                updatedRecords = proxy.getUpdatedRecords(),
               // gridRows=proxy._gridRows,
                $row = $();

            if (proxy.model.enableVirtualization) {
                var recordstart = updatedRecords[from];
                from = proxy.model.currentViewData.indexOf(recordstart);
            }

            if (ej.isNullOrUndefined(to)) {
                if (proxy._frozenColumnsLength > 0) {
                    if ($gridRows[0][from]) {
                        $row.push($gridRows[0][from]);
                        $row.push($gridRows[1][from]);
                    }
                    return $row;
                }
                return $($gridRows[from]);
            } else {
                if (proxy.model.enableVirtualization) {
                    var recordend = updatedRecords[to];
                    to = proxy.model.currentViewData.indexOf(recordend);
                }
                if (proxy._frozenColumnsLength > 0) {
                    $row.push($($gridRows[0]).slice(from, to));
                    $row.push($($gridRows[1]).slice(from, to));
                    return $row;
                }
                return $($gridRows.slice(from, to));
            }
        } catch (e) {
            return $();
        }
    }

})(jQuery, Syncfusion);;;